package roboforge;

import com.sun.j3d.utils.behaviors.mouse.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
//import com.sun.j3d.utils.behaviors.picking.*;
import javax.vecmath.AxisAngle4f;

public class BCameraMouseRotate extends MouseBehavior
{


    double x_angle, y_angle;
    double x_factor = .0025;
    double y_factor = .0025;
    double totalYrot = 0.0;

  //private BCameraMouseRotate callback = null;


  /**
   * Creates a rotate behavior given the transform group.
   * @param transformGroup The transformGroup to operate on.
   */
  public BCameraMouseRotate(TransformGroup transformGroup) {
    super(transformGroup);
  }

  /**
   * Creates a default mouse rotate behavior.
   **/
  public BCameraMouseRotate() {
      super(0);
  }

  /**
   * Creates a rotate behavior.
   * Note that this behavior still needs a transform
   * group to work on (use setTransformGroup(tg)) and
   * the transform group must add this behavior.
   * @param flags interesting flags (wakeup conditions).
   */
  public BCameraMouseRotate(int flags) {
      super(flags);
   }

  public void initialize() {
    super.initialize();
    x_angle = 0;
    y_angle = 0;
    if ((flags & INVERT_INPUT) == INVERT_INPUT) {
       invert = true;
       // x_factor *= -1;
       y_factor *= -1;
    }
  }

  /**
   * Return the x-axis movement multipler.
   **/

  public double getXFactor() {
    return x_factor;
  }

  /**
   * Return the y-axis movement multipler.
   **/

  public double getYFactor() {
    return y_factor;
  }

  /**
   * Set the x-axis amd y-axis movement multipler with factor.
   **/

  public void setFactor( double factor) {
    x_factor = y_factor = factor;

  }

  /**
   * Set the x-axis amd y-axis movement multipler with xFactor and yFactor
   * respectively.
   **/

  public void setFactor( double xFactor, double yFactor) {
    x_factor = xFactor;
    y_factor = yFactor;
  }

  public void setTotalY(double newTotalY)
  {
    totalYrot = newTotalY;

  }


  // Local Variables to stop memory chewing
  // DO NOT USE ANYWHERE EXCEPT in processStimulus
  private Matrix4d processStimulas_mat = new Matrix4d();
  private Vector3d zeroTranslation = new Vector3d(0.0,0.0,0.0);
  private Vector3f xAxis = new Vector3f(1.0f,0,0);
  private AxisAngle4f xRot = new AxisAngle4f();
  private Vector3d processStimulusTranslation = new Vector3d();
  private Transform3D totalYTran = new Transform3D();


  public void processStimulus (Enumeration criteria) {
      WakeupCriterion wakeup;
      AWTEvent[] event;
      int id;
      int dx, dy;

      while (criteria.hasMoreElements()) {
         wakeup = (WakeupCriterion) criteria.nextElement();
         if (wakeup instanceof WakeupOnAWTEvent) {
            event = ((WakeupOnAWTEvent)wakeup).getAWTEvent();
            for (int i=0; i<event.length; i++) {
	      processMouseEvent((MouseEvent) event[i]);

	      if (((buttonPress)&&((flags & MANUAL_WAKEUP) == 0)) ||
		  ((wakeUp)&&((flags & MANUAL_WAKEUP) != 0))){

		id = event[i].getID();
		if ((id == MouseEvent.MOUSE_DRAGGED) &&
		    !((MouseEvent)event[i]).isMetaDown() &&
		    !((MouseEvent)event[i]).isAltDown()){

                  x = ((MouseEvent)event[i]).getX();
                  y = ((MouseEvent)event[i]).getY();

                  dx = x - x_last;
                  dy = y - y_last;

		  if (!reset)
      {
		    x_angle = dy * y_factor;
		    y_angle = dx * x_factor;

        if (!BGlobal.INVERT_PLAYER_Y)
        x_angle = -x_angle;

		    transformX.rotX(x_angle);
		    transformY.rotY(-y_angle);
        totalYrot = totalYrot - y_angle;

		    transformGroup.getTransform(currXform);


		    // Remember old matrix
		    currXform.get(processStimulas_mat);

		    // Translate to origin
		    currXform.setTranslation(zeroTranslation);

        xAxis.set(1.0f,0,0);
        totalYTran.setIdentity();
        totalYTran.rotY(totalYrot);
        totalYTran.transform(xAxis);

        xRot.set(xAxis.x, xAxis.y, xAxis.z, (float)x_angle);
        transformX.set(xRot);



        if (invert)
        {
          currXform.mul(currXform, transformY);
          currXform.mul(currXform, transformX);

        }
        else
        {
          currXform.mul(transformY, currXform);
          currXform.mul(transformX, currXform);
          }

                    // Set old translation back
          processStimulusTranslation.set(processStimulas_mat.m03, processStimulas_mat.m13, processStimulas_mat.m23);
          currXform.setTranslation(processStimulusTranslation);

          // Update xform
          transformGroup.setTransform(currXform);

          transformChanged( currXform );




		  }
		  else
                  {
		    reset = false;
		  }

                  x_last = x;
                  y_last = y;
        }
               else if (id == MouseEvent.MOUSE_PRESSED) {
                  x_last = ((MouseEvent)event[i]).getX();
                  y_last = ((MouseEvent)event[i]).getY();
               }
	      }
	    }
         }
      }

      wakeupOn (mouseCriterion);

   }

  /**
    * Users can overload this method  which is called every time
    * the Behavior updates the transform
    *
    * Default implementation does nothing
    */
  public void transformChanged( Transform3D transform ) {
  }


}
