package roboforge;

//: BChassisPicker.java

import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import java.awt.*;

/*

* Description:
  Enables the user to pick a chassis when creating a new bot.

* Example:  BChassis myChassis = BChassisPicker.getChassis(this);
* Modification History ( Name, Date, Notes )
  DJG, 20 April 1999


*/

public class BChassisPicker extends JDialog
{
  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
static int FRAMESIZE = 2;
  BChassis selectedChassis;
  TreePath lastFPath;
  BFileSelectionTree lastFTree;

  public BChassisPicker(JFrame frame)
  {

    super(frame,"Select a Chassis",true);

    this.getContentPane().setLayout(null);
    this.setSize(BGlobal.FRAMEWIDTH/3+6,BGlobal.FRAMEHEIGHT - 75);
    this.setLocation(BGlobal.FRAMEWIDTH/3*2-10,25);

    try
    {
      // FILE LOAD: Set Up File Tree and add Listeners
      UIManager.put("Tree.leafIcon",new IconUIResource(new ImageIcon(BGlobal.IMAGEPATH+"component.gif")));
      UIManager.put("Tree.openIcon",new IconUIResource(new ImageIcon(BGlobal.IMAGEPATH+"tboxopen.gif")));
      UIManager.put("Tree.closedIcon",new IconUIResource(new ImageIcon(BGlobal.IMAGEPATH+"tboxclosed.gif")));
      BFileSelectionTree fTree = new BFileSelectionTree(BGlobal.CHASSISPATH,BGlobal.CHASSISSUFFIX, "Chassis Library",true);
      fTree.putClientProperty("JTree.lineStyle","Angled");
      fTree.addTreeSelectionListener(new FTreeSelectionHandler());
      // Set Tree Cell Renderer Parameters
      ComponentUI ftui = fTree.getUI();
      if (ftui instanceof BasicTreeUI)
      {
        fTree.setRowHeight(20);
        ((BasicTreeUI)ftui).setRightChildIndent(21);
        ((BasicTreeUI)ftui).setCollapsedIcon(new ImageIcon(BGlobal.IMAGEPATH+"collapsed.gif"));
        ((BasicTreeUI)ftui).setExpandedIcon(new ImageIcon(BGlobal.IMAGEPATH+"expanded.gif"));
      }
      fTree.setVisible(true);
      fTree.addMouseListener(new MouseAdapter()
      {
        public void mouseClicked(MouseEvent evt)
        {
          if (evt.getClickCount()==2)
          {
            loadFilePath();
            BChassisPicker.this.setVisible(false);
            BChassisPicker.this.dispose();
          }
        }
      });

      JScrollPane scrollPane = new JScrollPane(fTree);
      scrollPane.setSize(BGlobal.FRAMEWIDTH/3 - FRAMESIZE*2,BGlobal.FRAMEHEIGHT - FRAMESIZE*4 - 130);
      scrollPane.setLocation(FRAMESIZE,FRAMESIZE);
      this.getContentPane().add(scrollPane);
    }
    catch (Throwable t)
    {
       t.printStackTrace();
    }

    JButton button = new JButton("Select Chassis");
    button.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent evt)
      {
        loadFilePath();
        BChassisPicker.this.setVisible(false);
        BChassisPicker.this.dispose();
      }
    });

    button.setSize(BGlobal.FRAMEWIDTH/3 - FRAMESIZE*2,30);
    button.setLocation(FRAMESIZE,BGlobal.FRAMEHEIGHT - FRAMESIZE*2 - 130);
    this.getContentPane().add(button);

    this.setVisible(true);

  }

  static public BChassis getChassis(JFrame f)
  {
    BChassisPicker chassisPicker = new BChassisPicker(f);
    return chassisPicker.getValue();
  }

  private BChassis getValue()
  {
    //selectedChassis.setChassisLock(true);
    return selectedChassis;
  }

  // Load File Path
  private void loadFilePath()
  {
    try
    {
      String fname = lastFTree.getPathName(lastFPath);
      File f = new File(fname);
      if (f.isFile())
      {
        // Instantiate BChassis
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        selectedChassis = (BChassis) BComponent.loadFromFile(fname);
      }
    }
    catch (Throwable t)
    {
      // Do Nothing
    }
  }

  // LISTENER HANDLERS

  // File Tree
  private class FTreeSelectionHandler implements TreeSelectionListener
  {
    public void valueChanged(TreeSelectionEvent evt)
    {
      try
      {
        lastFPath = evt.getPath();
        lastFTree = (BFileSelectionTree)evt.getSource();
      } catch (Throwable t){
        // Do Nothing
      }
    }
  }

}///:~