package roboforge;

import java.awt.*;
//import java.io.*;
import javax.swing.*;
import java.awt.event.*;
//import javax.swing.border.*;
import java.awt.image.MemoryImageSource;

class BSBPicker extends JPanel
{
  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
private BColorPicker holder;
  private Image     display     = null;
  private double    hue         = 0.0;
  private boolean   initialized = false;

  //private int selected_x = 0;
  //private int selected_y = 0;

  protected int[] pixels = null;

  protected double s = 0.0;
  protected double b = 0.0;

  // paint constants
  private static final float BRIGHTTHRESH = 0.8f;
  private static final float SATTHRESH = 0.2f;
  private static final int H_RAD = 5;
  private static int H_BOARDER = 2;


  public BSBPicker(BColorPicker holder)
  {

    this.holder = holder;
    setForeground(Color.black);

    this.addMouseListener(new MyMouseListener());
    this.addMouseMotionListener(
        new MouseMotionAdapter()
        {
                public void mouseDragged(MouseEvent e)
                {

                        processMouse (e);
                }

        }

        );

  }

    class MyMouseListener extends MouseAdapter
  {
    public void mousePressed(MouseEvent e)
    {
        processMouse (e);
    }

  }


  public void processMouse (MouseEvent e)
  {
      int x = e.getX();
      int y = e.getY();

      Dimension dimensions = getSize();
      int width = dimensions.width, height = dimensions.height;


      if (x < 0)
        x = 0;
      else if (x >= width)
        x = width - 1;

      // cap at 1 instead of 0, because if y = 0,
      // the color stored is set with a Saturation and brightness of 0   (black)
      // this is the right colour, but is inconsistant with the xy display


      if (y < 1)
        y = 1;
      else if (y >= height)
        y = height - 1;

      s = (double)x / (double)(width-1);
      b = (double)y / (double)(height-1);

      holder.setHSB(hue, s, b);
      repaint();
  }

  public double getHue()
  {
    return hue;
  }

  public void setHue(double hue) {

    this.hue = hue;
    generateImage();

    holder.setHSB(hue, s, b);

    repaint();
  }

  public void setHSB(double h, double s, double b)
  {
    this.hue = h;
    this.s = s;
    this.b = b;

    generateImage();
    repaint();
  }

  public void generateImage() {

    Dimension d = getSize();

    int w = d.width;
    int h = d.height;
    int size = (w*h);

    // SJS [24/02/2000] - Only create pixel array when necessary.
    if (pixels == null);
      pixels = new int[size];

    if (size != pixels.length)
      pixels = new int[size];

    int pos = 0;
    for (int y = 0; y < h; y++)
      for (int x = 0; x < w; x++)
      {
        pixels[pos++] = Color.HSBtoRGB(
          (float) hue,
          (float) ((double)x / (double)w),
          (float) ((double)y / (double)h));
      }

    display = createImage(new MemoryImageSource(w, h, pixels, 0, w));
  }

  public void update(Graphics g) {
     paint(g);
  }

  public void paint(Graphics g)
  {
    /*
    Color selectedColor = holder.getSelectedColor();
    float[] hSB = Color.RGBtoHSB(selectedColor.getRed(),selectedColor.getGreen(),selectedColor.getBlue(),null);

    //setHue((new Float(hSB[0])).doubleValue());
    float s =  hSB[1];
    float b =  hSB[2];
    */

    int x = (int) (s * getWidth());
    int y = (int) (b * getHeight());

    if (!initialized)
    {
      generateImage();
      initialized = true;
    }
    if (display != null)
      g.drawImage(display, 0, 0, this);

    Graphics2D g2 = (Graphics2D)g;

    if (holder.getHue() == hue || (!BGlobal.DEPRECATED))
    {
        Color paintColor;
        //BDebug.print ("b " + b);

        float satprop    = (float) s / SATTHRESH;
        float highBright = (1f - (float) b)/ (1f - BRIGHTTHRESH);

        highBright =  java.lang.Math.min( highBright +  satprop, 1f);
        //BDebug.print ("satprop " + satprop);

        if ( (s < SATTHRESH) && (b > BRIGHTTHRESH)  )
          paintColor = Color.getHSBColor(0.5f, 0, java.lang.Math.max(highBright, 0f)  );
        else
          paintColor = Color.white;

        g2.setColor(paintColor);
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
        g2.drawOval(x - H_RAD, y - H_RAD, H_RAD * 2, H_RAD*2 );
        g2.drawOval(x - H_RAD -H_BOARDER, y - H_RAD -H_BOARDER, (H_RAD * 2) + (H_BOARDER * 2), (H_RAD*2) + (H_BOARDER * 2) );
    }


  }
}

