package roboforge;

import java.awt.Image;
import java.util.*;
import java.io.*;
import javax.vecmath.*;

import javax.swing.tree.*;
import roboforge.util.graphics.BSerializableImage;
import java.awt.image.*;


public class BSnapshot
  implements Serializable, BNameable
{
 /*

  * Description:
    A Collection
    which holds BComponentTransforms.

  * Example:
  * Modification History ( Name, Date, Notes ):
    HSC, 4-April-1999

  */

  static final long serialVersionUID = -1L;

  private Vector componentTransformCollection;
  private String snapshotName;
  private BMove chassisMove;

  private float metersToMove;
  private Integer percentPower;
  private double chassisX;
  private double chassisY;

  private boolean delayJoints = true;

  private BSerializableImage thumbnail;
    // A 'preview' image of this snapshot.

  
  // SJS [14/08/2000]: Constants for snapshot power...
  public static final Integer POWER_NONE   = new Integer(0);
  public static final Integer POWER_LOW    = new Integer(30);
  public static final Integer POWER_MED    = new Integer(90);
  public static final Integer POWER_HIGH   = new Integer(100);

  public static final String  POWER_LOW_DESC  = "Low";
  public static final String  POWER_MED_DESC  = "Medium";
  public static final String  POWER_HIGH_DESC = "High";

  public static final float  POWER_LOW_MAXVELOCITY  = 0.3f;
  public static final float  POWER_MED_MAXVELOCITY  = 1f;
  public static final float  POWER_HIGH_MAXVELOCITY = 1.5f;

  private transient float percentPowerRatio = 0f;

  public BSnapshot(String nameIn)
  {
    /*

    * Description:
    Constructor for BSnapshot

    * Example:
    * Modification History ( Name, Date, Notes ):
      HSC, 4-April-1999

    */

    snapshotName = nameIn;
    componentTransformCollection = new Vector();
    //BDebug.print ("BSnapshot.constructor called ");
    percentPower = new Integer(POWER_HIGH.intValue());
  }

  public void cleanse()
  {
    componentTransformCollection = null;
    chassisMove = null;
    thumbnail = null;
  }

  public void cleanseJointsNotInBot(BBot theBot)
  // removes all transforms not in this bot
  {
   // BDebug.print (" BSnapshot .cleanseJointsNotInBot Checking " );

    DefaultMutableTreeNode botRoot = theBot.getChassis().getTreeNode();

    BComponentTransform tempT;

    for (int i = componentTransformCollection.size() -1; i>= 0 ; i--)
    {
      tempT = ( BComponentTransform ) componentTransformCollection.get(i);
     // BDebug.print (" BSnapshot .cleanseJointsNotInBot checking "  + tempT.getJoint());

      if(   (tempT.getJoint().getTreeNode() == null) ||
            ( tempT.getJoint().getTreeNode().getRoot() != botRoot )   )
      {
        componentTransformCollection.remove(i);
      }


    }
 // BDebug.print (" BSnapshot .cleanseJointsNotInBot FINISHED " );


  }



  public void add (BComponentTransform componentTransform)
  {
    //if (componentTransform.getJoint()
    componentTransformCollection.add(componentTransform);
  }

  public boolean contains(BJoint j)
  {
    boolean c = false;

    for (int i = componentTransformCollection.size() - 1; i >=0 && !c; i-- )
    {
      BComponentTransform ct =  (BComponentTransform)componentTransformCollection.get(i);
      if (ct.getJoint() == j)
        c = true;
    }

    return c;



  }


  public String toString()
  {
    return snapshotName;
  }

  public void setChassisMove (BMove moveIn)
  {
    chassisMove = moveIn;
  }

  public BMove getChassisMove ()
  {
     return chassisMove;
  }

  public Vector getComponentTransforms ()
  {
    return (Vector)componentTransformCollection;
  }

  public void setComponentTransforms ( Vector transformsIn )
  {
    componentTransformCollection = transformsIn;
  }

  public void setPowerLevel(int level)
  {
    switch (level)
    {
      case 0:
        setPercentPower(POWER_LOW); break;
      case 1:
        setPercentPower(POWER_HIGH); break;
      default:
        setPercentPower(POWER_HIGH); break;
    }
  }
  
  public int getPowerLevel()
  {
    if (percentPower.equals(POWER_LOW))
      return 0;
    else
      return 1;
  }
  
  public void setPercentPower(Integer power)
  {
    if (power.equals(POWER_NONE)  )
      power = POWER_HIGH;

    percentPower = power;
  }

  public Integer getPercentPower()
  {
    if (percentPower.equals(POWER_NONE) )
      percentPower = POWER_HIGH;

    return percentPower;
  }

  public String getPowerDescription()
  {
    if (percentPower.equals(POWER_LOW))
      return POWER_LOW_DESC;
    else if (percentPower.equals(POWER_MED))
      return POWER_HIGH_DESC;
    else //if (percentPower.equals(POWER_HIGH));
      return POWER_HIGH_DESC;
    //else
    //  return percentPower.toString()+ "%";
  }

  public float getPowerVelocityMultiplier()
  {
     if (percentPower.equals(POWER_LOW))
      return POWER_LOW_MAXVELOCITY;
    else if (percentPower.equals(POWER_MED))
      return POWER_MED_MAXVELOCITY;
    else if (percentPower.equals(POWER_HIGH))
      return POWER_HIGH_MAXVELOCITY;
    else
      return 1f;
  }

  public void setName (String nameIn)
    { snapshotName = nameIn; }

  public String getName()
    { return snapshotName; }


  public void setThumbnail(BufferedImage thumb)
  {

    // if it is null then cleart the list
    if (thumb == null)
      thumbnail = null;
    else
    {
      if (thumbnail == null)
        thumbnail = new BSerializableImage(thumb);
      else
        thumbnail.setImage(thumb);
    }

  }

  public Image getThumbnail()
  {
    if (thumbnail == null)
      return null;
    else
      return thumbnail.getImage();

  }


  public float getPercentPowerRatio()
  {
    if (percentPowerRatio == 0f)
      percentPowerRatio = getPercentPower().floatValue() / 100.0f;

    return  percentPowerRatio;

  }


  public void setMetersToMove (float meters)
    { metersToMove = meters; }

  public float getMetersToMove()
    { return metersToMove; }

  public void setChassisOrientation (double x, double y )
  {
    chassisX = x;
    chassisY = y;
  }

  public void applyToBot()
  {

    BComponentTransform bTransform;
    BJoint bJoint;

    for (int i = 0 ; i <getComponentTransforms().size() ; i++ )
    {
          bTransform = (BComponentTransform) getComponentTransforms().get(i);
          bJoint = bTransform.getJoint();
          bJoint.setJointPosition((BJointPosition) bTransform.getJointPosition().clone());
          bJoint.applyTransform();
    }


  }


  public double getChassisX()
  {
   return  chassisX;
  }

    public double getChassisY()
  {
   return  chassisY;
  }

  /*
  public int getChassisMoveIndex_ForUI()
  {
    if (chassisMove == null)
      return -1;

    // Retrieve the list of available chassis moves...
    BMoveList chassisMoves = chassisMove.getChassis().getChassisMoveList();

    return chassisMoves.indexOf_ForUI(chassisMove);
  }     */

  public String getChassisMoveUnits()
    { return getChassisMoveUnits(false); }
    
  public String getChassisMoveUnits(boolean abbreviated)
  {
    String result = "";

    if (chassisMove == null)
      return result;

    // Retrieve the list of available chassis moves...
    BMoveList chassisMoves =
      chassisMove.getChassis().getChassisMoveList();


    return chassisMoves.getChassisMoveUnits(chassisMove, abbreviated);
  }

  public Vector3d getTranslation(BChassis chassis, BMove move)
  {
    // Returns the chassis translation that takes place in this snapshot.

    Vector3d result = new Vector3d();

    // If there's no chassis move, there's no translation...
    if ((chassisMove == null) && !BGlobal.CONSTRUCTING_CHASSISBOT)
      return null;

    // Retrieve the list of available chassis moves...
    BMoveList chassisMoves =
      BGlobal.CONSTRUCTING_CHASSISBOT ?
      chassis.getBot().getMoveList() :
      chassis.getChassisMoveList();

    int moveType;

    if (BGlobal.CONSTRUCTING_CHASSISBOT)
      moveType = chassisMoves.getChassisMoveType(move);
    else
      moveType = chassisMoves.getChassisMoveType(chassisMove);

    switch (moveType)
    {
      case BMoveList.CHASSIS_MOVE_FORWARD:
        result.z = +metersToMove;
        break;
      case BMoveList.CHASSIS_MOVE_BACKWARDS:
        result.z = -metersToMove;
        break;
      case BMoveList.CHASSIS_MOVE_LEFT:
        result.x = +metersToMove;
        break;
      case BMoveList.CHASSIS_MOVE_RIGHT:
        result.x = -metersToMove;
        break;
      default:
        result = null;
        break;
    }

    return result;
  }

  public float getRotation(BChassis chassis, BMove move)
  {
    // Returns the chassis rotation (degrees) that takes place in this snapshot.

    float result;

    // If there's no chassis move, there's no rotation...
    if ((chassisMove == null) && !BGlobal.CONSTRUCTING_CHASSISBOT)
      return 0.0f;

    // Retrieve the list of available chassis moves...
    BMoveList chassisMoves =
      BGlobal.CONSTRUCTING_CHASSISBOT ?
      chassis.getBot().getMoveList() :
      chassis.getChassisMoveList();

    int moveType;

    if (BGlobal.CONSTRUCTING_CHASSISBOT)
      moveType = chassisMoves.getChassisMoveType(move);
    else
      moveType = chassisMoves.getChassisMoveType(chassisMove);

    switch (moveType)
    {
      case BMoveList.CHASSIS_ROTATE_LEFT:
        result = +metersToMove;
        break;
      case BMoveList.CHASSIS_ROTATE_RIGHT:
        result = -metersToMove;
        break;
      default:
        result = 0.0f;
        break;
    }

    return result * BPhysics.RAD_CONVERT;
  }

  public void replaceBot (BBot bot)
  {
    BComponentTransform tempT;

    for (int i = componentTransformCollection.size() - 1; i >= 0; i-- )
    {
      tempT = ( BComponentTransform ) componentTransformCollection.get(i);
      BJoint j = (BJoint)bot.getComponent(tempT.getJoint().getName()) ;

      if ( j != null)
        tempT.setJoint(j);
    }

  }

  public boolean getDelayJoints()
    { return delayJoints ; }

  public void setDelayJoints(boolean delay)
    { delayJoints = delay; }
    

}///:~