package roboforge;

import javax.vecmath.*;
import java.util.*;
import javax.media.j3d.*;

//import com.sun.j3d.loaders.ParsingErrorException;
//import com.sun.j3d.loaders.IncorrectFormatException;
import com.sun.j3d.loaders.Scene;


public class BTelescopicJoint extends BJoint
{

/*

* Description:          Joints that Slide along a vector  (Curtain Rails, Telescope)
* Modification History ( Name, Date, Notes ):
                        YLY, 11 May 99

*/



static final long serialVersionUID = -1L;
private Vector3f directionOfMovement;
private static double STEPSCALE = 0.02;

private float maxVelocity; // Max velocity ( metres Sec)
//private float acceleration;    // (metres / sec squared per torque)
private float maxAcceleration;  // radians per second squares


 public BTelescopicJoint (String nameIn, String filename)
  {
    super(nameIn, filename);


  }

 public  void setMaxAcceleration(float newMaxAcceleration)
 {  maxAcceleration = newMaxAcceleration; }

 public  float getMaxAcceleration()
 { return maxAcceleration; }


  protected void parseLights(Scene s)
  {
    super.parseLights(s);
    BConnectPoint  current;

     // only the first connectPoint found can be connected
    for (int i = 0 ; i < connectPoints.size()-1 ; i++ )
    {

      current = (BConnectPoint)connectPoints.get(i);
      current.active = false;

    }

  }


  // once the joint has been added to the scene then all the points becomne active
  public void setConnectPointBase( BConnectPoint newPoint)
 {
    super.setConnectPointBase( newPoint);
    BConnectPoint  current;

    for (int i = 0 ; i < connectPoints.size()-1 ; i++ )
    {

      current = (BConnectPoint)connectPoints.get(i);
      current.active = true;

    }

 }


  public void setDirectionOfMovement( Vector3f newD)
  {
    directionOfMovement = newD;

  }

    public Vector3f getDirectionOfMovement()
  {

   return directionOfMovement;
  }


 public void parseDirectionalLight(DirectionalLight tempL)
  {
   directionOfMovement = new Vector3f();
   tempL.getDirection(directionOfMovement);
   directionOfMovement.normalize();
   BDebug.print("directionOfMovement.parseDirectionalLight.direction of Movement : " +  directionOfMovement.toString() );

  }




  private static Vector3f getDeltaTransform_translateV = new Vector3f();

  public Transform3D getDeltaTransform (int x, int y, int x_last, int y_last, BJointPosition lastPosition)
  {

       double ystep = ( y_last - y) * STEPSCALE;

       lastPosition.setWorldTranslate(         java.lang.Math.max(maxFreedomNegative ,
                                                                  java.lang.Math.min(maxFreedomPositive,lastPosition.getWorldTranslate() + (float)ystep)
                                                )
                                        );

       getDeltaTransform_translateV.set(directionOfMovement);

       //BDebug.print("BTelescopicJoint.getDeltatransform " +  lastPosition.getWorldTranslate() );

       getDeltaTransform_translateV.scale(lastPosition.getTranslate());
       Transform3D tempT = new Transform3D();
       tempT.set(getDeltaTransform_translateV);
       return tempT;
  }


  public Transform3D getDeltaTransform(float delta, BJointPosition last)
  {
    float dist = last.getWorldTranslate() + delta;

    if (dist < maxFreedomNegative)
      dist = maxFreedomNegative;

    if (dist > maxFreedomPositive)
      dist = maxFreedomPositive;

    last.setWorldTranslate(dist);

    getDeltaTransform_translateV.set(directionOfMovement);
    getDeltaTransform_translateV.scale(last.getTranslate());

    Transform3D t = new Transform3D();
    t.set(getDeltaTransform_translateV);
    return t;
  }


   // local temp variables,  to reduce garbage collection of player
 // DO NOT USE ANYWHERE BUT the Method Below
 // or else multiple threads will fuck you up BIG TIME

 transient private static Transform3D getTempTransformTempT = new Transform3D();
 transient private static Vector3f getTempTransformTempV = new Vector3f();


  public Transform3D getTempTransform(boolean useDefault)
  // The Transform returned  by this method is a static variable filled with the data.
  // It MUST Be Cloned if you are going to store it.

 {


    getTempTransformTempV.x = directionOfMovement.x;
    getTempTransformTempV.y = directionOfMovement.y;
    getTempTransformTempV.z = directionOfMovement.z;


    if (useDefault || (currentPosition == null)  )
        getTempTransformTempV.scale(defaultJointPosition.getTranslate() );
    else
        getTempTransformTempV.scale(currentPosition.getTranslate() );


    //Transform3D tempT = new Transform3D();
    getTempTransformTempT.set(getTempTransformTempV);
    return getTempTransformTempT;

 }



  /*
  public void setAcceleration(float newAcceleration)
  {
    acceleration = newAcceleration;
  }

  public float getAcceleration()
  {
    return acceleration;
  }
  */



  //private static  Vector3f move_torqueAcceleration = new Vector3f();

  public  boolean move(BJointPosition endPosition,  float timeSlice, float power,float maxVelocityMutiplier, boolean deccelerate )

  // All translates in this function are calculated in REAL WORLD CO_ORDINATES, NOT CANVAS CO_ORDINATES

  {
    if (delay > 0 && !( getChassisLock() || BGlobal.CONSTRUCTING_CHASSISBOT )  )
    {
      delay = delay - timeSlice;
      return true;

    }

    if ( standDown(pauseTime) )
    {
      return true;
    }


    // Cap the End Position in case james decreases the range

    if (endPosition.getWorldTranslate() > maxFreedomPositive)
      endPosition.setWorldTranslate(maxFreedomPositive);
    else if (endPosition.getWorldTranslate() < maxFreedomNegative)
      endPosition.setWorldTranslate(maxFreedomNegative);


   // BDebug.print("BTelescopicJoint.move called ");

    int direction = 0;
    float jointCapability = ((float)damageMax-(float)damageCurrent)/(float)damageMax;

    if (maxAcceleration ==0)
      maxAcceleration = 2;

    // Get the direction of movement, or return false

    if (  currentPosition.worldTranslateEquals(endPosition)   )
    {
        // Joint Transform Completed
        return false;
    }
    else if (endPosition.getWorldTranslate() > currentPosition.getWorldTranslate() )
      direction = 1;
    else if (endPosition.getWorldTranslate() < currentPosition.getWorldTranslate() )
      direction =  -1;

    float distanceInThisSlice = 0f;
    float oldPosition = currentPosition.getWorldTranslate();
    float newPosition = 0f;
    float oldVelocity = 0;

    if (  ( getChassisLock() || BGlobal.CONSTRUCTING_CHASSISBOT )  && !deccelerate )
    {

     // BDebug.print(  "BTelescopicJoint.move getBot().getChassis().distanceThisSnapshot" + getBot().getChassis().distanceThisSnapshot );
     // BDebug.print(  "BTelescopicJoint.move getBot().getChassis().distanceMovedThisSnapshot" + getBot().getChassis().distanceMovedThisSnapshot );


      if ( getBot().getChassis().distanceThisSnapshot <= getBot().getChassis().distanceMovedThisSnapshot )
      {
        //BDebug.print("BTelescopicJoint.move: Forcing Chassis Joint to End Snapshot Position");

        currentVelocity = 0;

        /* DJG June 2001 - Causing Passthru
        setPreviousJointPosition(currentPosition);
        currentPosition.setWorldTranslate( endPosition.getWorldTranslate() );
        applyTransform();
        */

        return false;
      }

          // Part of Chassis. Distance moved is dependant on distance travelled by Chassis
      float chassisMoveDist = java.lang.Math.max( getBot().getChassis().distanceThisSnapshot
                                   - getBot().getChassis().distanceMovedThisSnapshot , 0.0f );

     // BDebug.print(  "BTelescopicJoint.move chassisMoveDist" + chassisMoveDist);

      float distThisSlice = getBot().getChassis().distThisSlice;

     // BDebug.print(  "BTelescopicJoint.move distThisSlice" + distThisSlice );

      chassisMoveDist = distThisSlice / chassisMoveDist; // ratio of distThisSlice with DistToGo

     // BDebug.print(  "BTelescopicJoint.move chassisMoveDist" + chassisMoveDist );

     //distanceInThisSlice = endPosition.getTranslate() - currentPosition.getTranslate();
      distanceInThisSlice = endPosition.getWorldTranslate()  - currentPosition.getWorldTranslate();

     // BDebug.print(  "BTelescopicJoint.move total dist left t move" + distanceInThisSlice );

      distanceInThisSlice = distanceInThisSlice * chassisMoveDist;

      // BDebug.print(  "BTelescopicJoint.move distanceInThisSlice" + distanceInThisSlice );

      //BDebug.print(  "BTelescopicJoint.move chassisMoveDist ratio" + chassisMoveDist );
     // BDebug.print(  "BTelescopicJoint.move distanceInThisSlice " + distanceInThisSlice );



      if ( distanceInThisSlice == 0.0f || distanceInThisSlice == Float.NaN )
      {
        currentVelocity = 0;
        //setPreviousJointPosition(currentPosition);
        currentPosition.setWorldTranslate( endPosition.getWorldTranslate() );
        applyTransform();
        return false;
      }


    }
    else
    {

      float netLimbLoad = this.getNetLimbLoad(this.getLimbWeight(), power);
      float acceleration = jointPowerOutput * BPhysics.POWER_ACCELERATION_RATIO;

      // Joints velocity
      oldVelocity = currentVelocity;
      float deltaVelocity =  (         direction *  (    (acceleration * jointCapability * timeSlice * power) / netLimbLoad  )          );


      if (!deccelerate)
      {
        deltaVelocity = java.lang.Math.min( deltaVelocity , maxAcceleration * maxVelocityMutiplier * timeSlice );

        if (deltaVelocity > 0)
          deltaVelocity = java.lang.Math.min( deltaVelocity , maxAcceleration * maxVelocityMutiplier * timeSlice );
        else
          deltaVelocity = java.lang.Math.max( deltaVelocity ,-maxAcceleration * maxVelocityMutiplier * timeSlice );


      }




      //setPreviousJointPosition(currentPosition);
      oldPosition =  currentPosition.getWorldTranslate();
      newPosition = 0;

      // delta x as a derivative of ut + 1/2at^2
      // x = ut + 1/2vt
      distanceInThisSlice = oldVelocity * timeSlice + ( ( deltaVelocity / 2.0f ) * timeSlice  )  ;

      /////////

      if (!deccelerate)
          {
                      //---DJG---
            if ( maxVelocityExceededCtr > 0 )
            {
              deltaVelocity = 0.0f; // No Acceleration
              if ( maxVelocityExceededCtr > 1 ) // Decelerate
              {
                float decelerationRate = (timeSlice/BGlobal.CALC_TIMESLICE) * ((maxVelocityForLimb-BPhysics.MAX_VELOCITY_CAP) / maxVelocityForLimb);
                deltaVelocity = ( direction * java.lang.Math.abs(currentVelocity) * decelerationRate) * -1;
              }

              currentVelocity =  java.lang.Math.min( java.lang.Math.abs(currentVelocity + deltaVelocity),  getMaxVelocity() * maxVelocityMutiplier);
              currentVelocity = direction * java.lang.Math.abs(currentVelocity);
              //---DJG---

            }
            else
            {

              currentVelocity =  java.lang.Math.abs(currentVelocity);
              currentVelocity =  java.lang.Math.min( currentVelocity + java.lang.Math.abs(deltaVelocity),  getMaxVelocity() * maxVelocityMutiplier);
              currentVelocity = direction * currentVelocity;
              // currentVelocity = direction * java.lang.Math.min(java.lang.Math.abs(currentVelocity +  deltaVelocity), getMaxVelocity());
            }
          }
          else
          {
            //BDebug.print("Old currentVelocity" + currentVelocity);
            //getBot().setArenaComments("BTelescopic Joint decelerate currentVelocity" +   currentVelocity  );

            currentVelocity =  java.lang.Math.abs(currentVelocity);
            currentVelocity = java.lang.Math.max( currentVelocity -  java.lang.Math.abs(deltaVelocity )  * ( BPhysics.DECELERATE_COEFFICIENT / 2f )   ,0f);
            currentVelocity = java.lang.Math.min( currentVelocity, getMaxVelocity() * maxVelocityMutiplier );
            //currentVelocity = direction * currentVelocity;


            if (currentVelocity <= 0f)
            {
              currentVelocity = 0;
              endPosition.setWorldTranslate( currentPosition.getWorldTranslate());
              return false;
            }
            else
            {
              currentVelocity = direction * java.lang.Math.min(java.lang.Math.abs(currentVelocity -  java.lang.Math.abs(deltaVelocity)), getMaxVelocity() * maxVelocityMutiplier);
            }


           // BDebug.print ("TelescopicJoint.move currentVelocity after thingy" + currentVelocity);

            //getBot().setArenaComments("BTelescopic Joint decelerate currentVelocity" +   currentVelocity  );

            //BDebug.print("New " + currentVelocity);
          }

          // delta x as a derivative of ut + 1/2at^2
          // x = ut + 1/2vt


          // cap delta velocity
          if (currentVelocity == oldVelocity)
            deltaVelocity = 0f;

          if (!deccelerate)
            distanceInThisSlice = oldVelocity * timeSlice + ( ( deltaVelocity / 2.0f ) * timeSlice  )  ;
          else
            distanceInThisSlice = currentVelocity * timeSlice ;

      


          /*
          BDebug.print("TelescopicJoint.move " + this);
          
          BDebug.print("TelescopicJoint.netLimbLoad " + netLimbLoad);
          BDebug.print("currentVelocity" + currentVelocity);
          BDebug.print("deltaVelocity" + deltaVelocity);
          BDebug.print("distanceInThisSlice" + distanceInThisSlice);

          BDebug.print("direction" + direction);
          BDebug.print("acceleration" + acceleration);
          BDebug.print("jointCapability" + jointCapability);
          BDebug.print("timeSlice" + timeSlice);
          BDebug.print("power" + power);
          BDebug.print ("maxAcceleration * maxVelocityMutiplier * timeSlice " + (maxAcceleration * maxVelocityMutiplier * timeSlice));
          */



    }




    if ( direction < 0 )
      newPosition = java.lang.Math.max ( oldPosition + distanceInThisSlice,  endPosition.getWorldTranslate());
    else
      newPosition = java.lang.Math.min ( oldPosition + distanceInThisSlice,  endPosition.getWorldTranslate());

    //BDebug.print("oldPosition" + oldPosition);

    // cap new position
    newPosition  = java.lang.Math.max(maxFreedomNegative ,
                                      java.lang.Math.min(maxFreedomPositive,newPosition)
                                                );

    //BDebug.print(  "BTelescopicJoint.move newPosition" + newPosition );

    //Debug.print("oldPosition" + oldPosition);
    //BDebug.print("newPosition" + newPosition);


    currentPosition.setWorldTranslate(newPosition);
    applyTransform();


    //if ( !getChassisLock() )
    //  getBot().totalEnergy -= ((jointEnergyUsage * java.lang.Math.abs(currentVelocity) * limbLoad) * timeSlice * (maxVelocityMutiplier * maxVelocityMutiplier)) / 1000f;

    // if you have reached the end, return true
      return true;


  }


  public void setMaxVelocity(float newVelocity)
  {
    maxVelocity = newVelocity;
  }

  public float getMaxVelocity()
  {
   if (  maxVelocity <= 0.0f )
      maxVelocity = 9999999.0f;
   return maxVelocity;
  }

 public void setDelay (float totalMoveTime)
  {
    delay = ( totalMoveTime - timeToNextSnap ) * 0.4f;

  }

  public float workOutProcessingTime(BJointPosition endPosition, float power)
  {
    // Work out time the time that it would take for the joint to get to this position
    // This is only used while transforming to the first snapshot position
    // All subsequent snapshots should have pre-calculated timing

    if ( endPosition.getWorldTranslate() == currentPosition.getWorldTranslate() )
    {
      timeToNextSnap = 0.0f;
    }
    else
    {

      /*
      Work out the time taken with formula s=ut + 1/2at^2

      1- re-arange to be a quadratic
        = (1/2)at^2 + ut + s = 0

      2- Combinine with general quadratic to get:
          t =  (   u (-+) sqrt( u^2 - 4(1/2)as )    )
                       /  2(1/2)a
         We are not carrying velocity through,  so the fomula can be simplified to

         t = Sqrt( 2s/a)
      */

     // Average the torque between the current position to the target snapshot position
     float avgLimbLoad = this.getLimbWeight();

     
     //BDebug.print(this.toString()+"  Avg Limb Load="+avgLimbLoad);
     float acceleration = jointPowerOutput * BPhysics.POWER_ACCELERATION_RATIO;
    // if (acceleration == 0.0f )
   //   BDebug.print ("WARNING WARNING " + toString() + " has no Acceleration, this will stop the entire move ");

     timeToNextSnap = (float)java.lang.Math.sqrt(
                                java.lang.Math.abs(
                                                    (2.0 * ( endPosition.getWorldTranslate() - currentPosition.getWorldTranslate() )  ) / (    (acceleration * power) / avgLimbLoad  )
                                                  )
                                                );

    }
    return timeToNextSnap;


  }

  // DJG June 2001 - Used in getResultantPosition
  private transient static Transform3D t = new Transform3D();
  private transient static Vector3f jointPos = new Vector3f();
  private transient static Matrix3f rotOfJoint = new Matrix3f();
  private transient static Transform3D cT = new Transform3D();
  private transient static Vector3f collidedPos = new Vector3f();
  private transient static Vector3f collidedRelative = new Vector3f();

  public  float getResultantPosition ( Vector3f momentum,BComponent collidedComponent,   BJointPosition newMove, BComponent otherComponent )
  // given a momentum vector, this method will return
  // momentumLeftOver,   - once this joint has moved to it's limit, how much momentum is used up.
  // newMove,  the Ending joint position of the new Move
  {


        //BDebug.print (getName() + "get resultantPosition called");
        if (this.getDestroyed() )
                return 0.0f;

        //BDebug.print( getName() + " axisOfRotation " + axisOfRotation);
       // getBot().setArenaComments( getName() + " axisOfRotation " + axisOfRotation);

        // get the transform of the Joint
        //Transform3D t = new Transform3D();
        if ( !getLiveNode().isLive())
           BDebug.print("BRotateJoint.getResultantPosition :: This Joint is Not part of a live Graph!" + getName());

        getLiveNode().getLocalToVworld(t);

        //Vector3f jointPos = new Vector3f();
        t.get(jointPos);

        // INVERT , WE WILL USE THIS INVERTED MATRIX TO CONVERT OTHER Nodes into
        // a virtual space where the joint is the centre of the universe
        t.invert();
        // get only the rotational component  of the Joint's Transform
        //Matrix3f rotOfJoint = new Matrix3f();
        t.get(rotOfJoint);

        float powerBefore =  momentum.length();
        //BDebug.print( "power before transform " + momentum.length() );
        //BDebug.print( "Speed before " + momentum.length()  / this.weight);
        //BDebug.print( "   Momentum before transform " + momentum);

        //BDebug.print( "jointPos" + jointPos);
        //getBot().setArenaComments( "jointPos " + jointPos);

        // get the position of the collidedComponent
        //Transform3D cT = new Transform3D();
        if ( !collidedComponent.getLiveNode().isLive())
           BDebug.print("BRotateJoint.getResultantPosition :: collidedComponent is Not part of a live Graph!" + collidedComponent.getName());

        collidedComponent.getLiveNode().getLocalToVworld(cT);
        //Vector3f collidedPos = new Vector3f();
        cT.get(collidedPos);

        //BDebug.print( "collidedPos" + collidedPos);
        //getBot().setArenaComments( "collidedPos " + collidedPos);

        // get the vector from the joint to the collided component
        //Vector3f collidedRelative = new Vector3f();
        collidedRelative.sub(collidedPos, jointPos);
        // convert that vector into Joint Space (Where the Joint is the centre of the universe
        rotOfJoint.transform(collidedRelative);

        //BDebug.print( "collidedRelative" + collidedRelative);
        //getBot().setArenaComments( "collidedRelative " + collidedRelative);


        //Transform the Momentum into Joint Space (Where the joint is the centre of the universe )
        rotOfJoint.transform(momentum);

        //BDebug.print( "Transformed Momentum " + momentum);
        //getBot().setArenaComments( "Transformed Momentum " + momentum);

        //project momentum  and collitionRelative onto the joints 1n line of movement;
        //float direction ;
        float momentum1d;

        if (directionOfMovement.x != 1)
        {
                momentum1d =  momentum.x;

                momentum.y = 0;
                collidedRelative.y = 0;

                momentum.z = 0;
                collidedRelative.z = 0;

        }
        else if (directionOfMovement.y != 1)
        {

                momentum1d =  momentum.y;

                momentum.x = 0;
                collidedRelative.x = 0;

                 momentum.z = 0;
                collidedRelative.z = 0;
        }
        else
        {
                momentum1d =  momentum.z;

                momentum.x = 0;
                collidedRelative.x = 0;

                momentum.y = 0;
                collidedRelative.y = 0;




        }



        float projectedMomentum = momentum1d;
        //BDebug.print( "projectedMomentum " + projectedMomentum);

        /// XXXXXX THIS WILL CHANGE WHEN INERTIA IS PLACE IN
        // WILL CHANGE TO LIMB LOAD
        float v0 =  projectedMomentum  / ( collidedComponent.weight  - otherComponent.weight );   // initial Velocity in m/s

        if (this.getChassisLock())
          v0 = v0 * BPhysics.CHASSISLOCKRESISTANCE ;

        float limbWeight = ((BComponent)this).getLimbWeight();
        //BDebug.print ("limbWeight   " + limbWeight );


        float weightRatio;

        if (otherComponent == BArena.arenaComponent)
          weightRatio = BPhysics.GROUND_RATIO;
        else
          weightRatio = java.lang.Math.min( otherComponent.weight / limbWeight, BPhysics.WEIGHT_RATIO_CAP);

        //BDebug.print ("weightRatio   " + weightRatio );
        if (  otherComponent.getDestroyed()  )
        //if (weightRatio < BPhysics.SUPERIOR_MOMENTUM_COEFFICENT ||  otherComponent.getDestroyed()  )
        {
          currentVelocity = currentVelocity * BPhysics.SUPERIOR_MOMENTUM_DELETERATE_COEFF;
          return 1f;
        }


        //getBot().setArenaComments( "v0  " + v0 );

        // r0 == v0 for non-rotation joint  (Code copied from rpotattionalJoint )
        float r0 = v0;

        float powerResist = (RESIST_POWER_COEFFICIENT / jointPowerOutput );


        if ( BGlobal.DEADLOCKDETECTED )
          this.currentVelocity = 0;
        else
        {

          // SET THE CURRENT VELOCITY OF THE JOINT
          this.currentVelocity =  r0  * BPhysics.RESULTANT_BOUNCE_SCALE_PERCENT ;
          this.currentVelocity = java.lang.Math.min(this.currentVelocity, getMaxVelocity()  );
          this.currentVelocity = this.currentVelocity * java.lang.Math.min( powerResist,1);
        }




        r0 = r0 * BPhysics.RESULTANT_BOUNCE_SCALE_PERCENT;
        //getBot().setArenaComments( "r0  " + r0 );


        r0 = java.lang.Math.min(r0, getMaxVelocity());
       // getBot().setArenaComments( "getMaxVelocity()  " + getMaxVelocity() );

        //float rawAccreleration =    (       getAcceleration() /   ( limbLoad / BPhysics.JOINTLIMBLOAD_COEFFICENT)  );

        /*
        BDebug.print ("Raw Bounce   " + r0 );
        getBot().setArenaComments( this.getName() + " resultant Raw Movement  " + r0 );
        BDebug.print ("rawAccreleration  " + rawAccreleration );
        BDebug.print ("MAX_RAWACCELERATION  " + BPhysics.MAX_RAWACCELERATION );
        rawAccreleration = java.lang.Math.min(rawAccreleration, BPhysics.MAX_RAWACCELERATION);

        // Inverse it soo that greater power of the joint means less bounce
        rawAccreleration = BPhysics.MAX_RAWACCELERATION - rawAccreleration;
        rawAccreleration = rawAccreleration / BPhysics.JOINTRESISTANCE_COEFFICENT;
        rawAccreleration = java.lang.Math.max(rawAccreleration, MINBOUNCE_PERCENT);

        BDebug.print ("processed Acc  " + rawAccreleration );
        */

        float bounce = r0 * weightRatio ;

        if (this.getChassisLock())
          bounce = BPhysics.CHASSISLOCKRESISTANCE * bounce;
          
        //BDebug.print ("Bounce   " + bounce );
        //getBot().setArenaComments( this.getName() + " resultant delta Movement after joint resisted " + bounce );


        float momentumAbsorbed = projectedMomentum / powerBefore;
        bounce = bounce * powerResist;

       // BDebug.print ("Bounce  after resist " + bounce );

        float resultantPos = currentPosition.getWorldTranslate() + bounce;

       // BDebug.print ("resultantPos  after resist " + resultantPos );
       //BDebug.print ("maxFreedomPositive" + maxFreedomPositive );
        //BDebug.print ("maxFreedomNegative" + maxFreedomNegative );

        if (resultantPos > maxFreedomPositive)
        {
                resultantPos = maxFreedomPositive;
           //     BDebug.print ( "+momentumAbsorbed was " + momentumAbsorbed);
                // reduce momentumAbsorbed by the actualAmount moved
                momentumAbsorbed =  momentumAbsorbed * ( ( maxFreedomPositive -  currentPosition.getWorldTranslate()  )  /  bounce  );

             //    BDebug.print ( "+momentumAbsorbed is now " + momentumAbsorbed);

        }

        else if (resultantPos < maxFreedomNegative)
        {
                resultantPos = maxFreedomNegative;

       //         BDebug.print ( "-momentumAbsorbed is was " + momentumAbsorbed);
                // reduce momentumAbsorbed by the actualAmount moved
                momentumAbsorbed =  momentumAbsorbed * (
                                                                java.lang.Math.abs(
                                                                        (  maxFreedomNegative -   currentPosition.getWorldTranslate()  )
                                                                                )
                                                                /  bounce
                                                        );
            //    BDebug.print ( "-momentumAbsorbed is now " + momentumAbsorbed);

        }


       // BDebug.print ("resultantPos  after capping " + resultantPos );
       //getBot().setArenaComments("BRotateJoint.getResultantMove()resultantPos" + resultantPos );
      // getBot().setArenaComments("BTelescopic.getResultantMove()currentPosition" + currentPosition.getWorldTranslate() );
       newMove.setWorldTranslate(resultantPos);
      // getBot().setArenaComments("BTelescopic.getResultantMove()resultantPos" + newMove.getWorldTranslate() );

       if (this.getChassisLock())
          momentumAbsorbed = momentumAbsorbed / BPhysics.CHASSISLOCKRESISTANCE;

       return momentumAbsorbed;
  }

  public float getFreedomRange()
    { return (maxFreedomPositive - maxFreedomNegative); }

  public String getFreedomUnits()
    { return "m"; }



  public void upgrade()
 {
   BTelescopicJoint upgradeTwin =   (BTelescopicJoint)BComponent.loadFromFile( BGlobal.COMPONENTPATH+getComponentFilePath(), false );

   super.upgrade(upgradeTwin);
   setMaxVelocity ( upgradeTwin.getMaxVelocity());
   setDirectionOfMovement ( upgradeTwin.getDirectionOfMovement());

   // HACK TO THROW EXCEPTION
   if (!this.connectPointBase.equals(connectPoints.get(connectPoints.size()-1 )        )  )
   {
    BConnectPoint hack = null;
    hack.active = false;
   }
 }

 public void enforceBaseConnectPoint()
 {
    for (int i = 0 ; i < connectPoints.size()-1 ; i++ )
      ((BConnectPoint)connectPoints.get(i)).active = false;


 }
 
  
   public Vector getAttributeStrings()
 // returns a Vector of tuples  (heading, value )
 // 2-d Vector
 {

  Vector theList = super.getAttributeStrings();


  Vector tempV;

  tempV = new Vector(2);
  tempV.add("max velocity");
  tempV.add(getMaxVelocity() +" m/sec");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("max acceleration");
  tempV.add(getMaxAcceleration() +" m/sec^2");
  tempV.trimToSize();
  theList.add(tempV);


  return theList;
  }

  /*
  // DJG June 2001
  public void setCappedJointPosition(BJointPosition jointPos, float posFloat)
  {
    jointPos.setWorldTranslate( java.lang.Math.max( currentMoveMinPosition.getAngle(), java.lang.Math.min(currentMoveMaxPosition.getAngle(),posFloat) )
                                       * BGlobal.CANVAS_TO_WORLD_SCALE
    													);
  }
  */



}