package roboforge;

import java.io.*;
import java.util.*;
//import javax.swing.tree.*;
//import javax.swing.event.*;

//import javax.vecmath.*;
//import javax.media.j3d.*;
//import javax.swing.text.*;



public class BWizardLimbOptions implements Serializable
{

  static final long serialVersionUID = -1L;

  private Vector limbOptions;
  private String name;

  private transient BWizardTemplateBot bot;
  private transient BWizardLimb selectedLimb;
  private boolean allowNone = false;

  public BWizardLimbOptions(Vector limbOptions)
  {
    this.limbOptions = limbOptions;
    name =   limbOptions.size() + " limbOptions ";
  }

  public void addLimb (BWizardLimb lo )
  {
    limbOptions.add(lo);
    lo.detachButRetainLinks();
  }

  public void removeLimb (BWizardLimb lo )
  {
    limbOptions.remove(lo);
  }



  public void setAllowNoSelection(boolean allowNone)
  {
    this.allowNone = allowNone;
  }

  public boolean getAllowNoSelection()
  {
    return allowNone;
  }


  public void setParentBot(BWizardTemplateBot bot)
    { this.bot = bot; }

  public BWizardTemplateBot getParentBot()
    { return this.bot; }

  public void addSelectedLimbToBot()
  {
    if (selectedLimb != null)
      selectedLimb.reattachToBot();
  }

  public void reattachAllLimbs()
  {
     for (int i = 0; i < limbOptions.size(); i++ )
         ((BWizardLimb) limbOptions.get(i)).reattachToBot();

  }


  public void cleanNonSelectedLimbOptions()
  {
    for (int i = 0; i < limbOptions.size(); i++ )
      if ( selectedLimb !=  limbOptions.get(i) )
         ((BWizardLimb) limbOptions.get(i)).cleanse();

    limbOptions = null;
  }

  public Vector getLimbs()
    { return limbOptions; }

  public BWizardLimb getLimb(int i)
    { return (BWizardLimb) limbOptions.get(i); }

  public int size()
    { return limbOptions.size(); }
    
  public void setSelected(int i)
  {
    selectedLimb = (BWizardLimb) limbOptions.get(i);
    updateBotInfo();
  }

  public void setSelected(BWizardLimb s)
  {
    if (limbOptions.indexOf(s) == -1)
      selectedLimb = null;
    else
      selectedLimb = s;

    updateBotInfo();
  }

  protected void updateBotInfo()
  {
    if (bot != null)
      bot.recalculateBotInfo();
  }


  public BWizardLimb getSelectedLimb()
  {
    return selectedLimb;
  }

  public void init()
  {
    for (int i = 0; i < limbOptions.size(); i++ )
         ((BWizardLimb) limbOptions.get(i)).init();

  }



  public void detachAllLimbsFromBot()
  {
     for (int i = 0; i < limbOptions.size() ; i++ )
     {
          BWizardLimb current = (BWizardLimb) limbOptions.get(i);
          current.detachButRetainLinks();
     }

  }

  public String toString()
  {

    return name;

  }

  public void setName (String name)
  {
    this.name = name;
  }

  public String getName()
  {
    return name;
  }


  public float getPrice()
  {
    if (selectedLimb != null)
      return selectedLimb.getPrice();
    else
      return 0.0f;
  }

  public float getWeight()
  {
    if (selectedLimb != null)
      return selectedLimb.getWeight();
    else
      return 0.0f;
  }

  public int getDamageMax()
  {
    if (selectedLimb != null)
      return selectedLimb.getDamageMax();
    else
      return 0;
  }

  public int getComponentCount()
  {
    if (selectedLimb != null)
      return selectedLimb.getComponentCount();
    else
      return 0;
  }
 


}