package roboforge;

import java.util.*;
import roboforge.ai.BDecisionTreeAI;

public class BWizardTemplateBot extends BBot
{

  static final long serialVersionUID = -5385212414078252581L;
  private Vector limbOptions = new Vector();    // Vector of Vectos
                                        // innner Vector are an array of limb options

  private Vector blockOptions = new Vector();

//  private Vector memoryOptions = new Vector();



  public BWizardTemplateBot(BChassis c)
    { super(c); }

  public BWizardTemplateBot(BChassis c, boolean init)
    { super(c, init); }

/*
  public void addMemoryOption( BWizardMemoryOptions newOption )
  {
    if (memoryOptions == null)
      memoryOptions = new Vector();

   if (!memoryOptions.contains(newOption))
    memoryOptions.add(newOption);


  }

  public void removeMemoryOption( BWizardMemoryOptions option )
  {
    if (memoryOptions != null)
    { memoryOptions.remove(option); }

  }

  public int getMemoryOptionsCount()
  { return  memoryOptions.size() ;}

  public BWizardMemoryOptions getMemoryOptionAt(int index)
  { return (BWizardMemoryOptions) memoryOptions.get(index); }

  public Vector getMemoryOptions()
  { return memoryOptions;}

*/
  // dont disable the parent connect point
  public void addComponent
  (
    BComponent child,  BConnectPoint childPt,
    BComponent parent, BConnectPoint parentPt
  )
  {
    super.addComponent  ( child,   childPt,
                           parent,  parentPt
                          );


    parentPt.setActive(true);

  }


  public Vector getLimbOptions()
  {
    return limbOptions;
  }

  public BWizardLimbOptions getLimbOptions(int i)
  {
    return (BWizardLimbOptions) limbOptions.get(i);
  }


  public void movePlaceInListTo(BWizardLimbOptions s, int newIndex)
  {
      if (limbOptions.indexOf(s) == -1)
        BDebug.print ("BWizardTemplateBot.movePlaceInListTo ERROR, selected Not in options List ");
      else if ( (newIndex >= 0) && (newIndex < limbOptions.size() ) )
      {
        limbOptions.remove(s);
        limbOptions.insertElementAt(s, newIndex);
      }
  }

  public Vector getBlockOptions()
  { return blockOptions; }

  public void setBlockOptions(Vector blockOptions)
  {
    this.blockOptions = blockOptions;

  }


  public BWizardBlockOptions getBlockOption(int i)
  { return (BWizardBlockOptions) blockOptions.get(i); }

  public void moveBlockPlaceInListTo(BWizardBlockOptions s, int newIndex)
  {
      if (blockOptions.indexOf(s) == -1)
        BDebug.print ("BWizardTemplateBot.moveBlockPlaceInListTo ERROR, selected Not in options List ");
      else if ( (newIndex >= 0) && (newIndex < blockOptions.size() ) )
      {
        blockOptions.remove(s);
        blockOptions.insertElementAt(s, newIndex);
      }
  }

  public void purge()
  {
    BComponent[] components = getBreathComponents();

    // start at 1 so we miss the chassis
    for (int i = 1; i < components.length; i++)
    {
      BDebug.print ("BWizardTemplateBot.purge " + components[i] + " parent  " + components[i].getParent()  );
      if (components[i].getParent() == null)
        components[i].getTreeNode().removeFromParent();
    }
  }


  public void reCreateOriginalWizardBot()
  {
    BWizardLimbOptions current;


    purge();
    // connect selected areas to bot
   for (int i = 0; i < limbOptions.size(); i++)
   {
      current =  (BWizardLimbOptions)limbOptions.get(i);
      current.reattachAllLimbs();

   }




  }

   public void saveWizardToFile(String filename)
  {

     if (  ! BGlobal.WIZSUFFIX.equalsIgnoreCase(     filename.substring(filename.length() - 4 )  )  )
                filename = filename +  BGlobal.WIZSUFFIX;

     try
     {
      save(filename);
     } catch (Exception e ) { BDebug.print ("BWizardTemplateBot.SaveWizardToFile Exception " + e.getMessage() ); }
  }



 public void convertToBaseBot()
 {
   BWizardLimbOptions current;
   BWizardBlockOptions current2;

    // connect selected areas to bot
   for (int i = 0; i < limbOptions.size(); i++)
   {
      current =  (BWizardLimbOptions)limbOptions.get(i);
      current.addSelectedLimbToBot();

   }


    // cleanse moves
   this.getMoveList().cleanseJointsNotInBot(this);

    // cleanse limbOptions
   for (int i = 0; i < limbOptions.size(); i++)
   {

      current =  (BWizardLimbOptions)limbOptions.get(i);
      current.cleanNonSelectedLimbOptions();

   }
   limbOptions = null;

   breadthFirstComponents = null;
   // update all connectPoints
   BComponent[] myComponents = getBreathComponents();

   // start at 1, so we set the connect point on the chassis
   for (int i = 1; i < myComponents.length; i++)
   {
     myComponents[i].getParent().setConnectPointActive(myComponents[i].getConnectPointParent(), false);
      //myComponents[i].getConnectPointParent().active = false;
   }

    // connect selected blocks to bot
   for (int i = 0; i < blockOptions.size(); i++)
   {
      current2 = (BWizardBlockOptions)blockOptions.get(i);
      current2.addSelectedBlockToBot();
   }

   // remove moves and zones not used by the selected AI
   BDecisionTreeAI theAI = (BDecisionTreeAI)getAI();
   Vector refMoves = theAI.getReferencedMoves();
   Vector refZones = theAI.getReferencedZoneNames();

   Vector allMoves = getMoveList().getMoves();
   Vector allZones = getZones().getZones();

   boolean referenced = false;

   for(int i = allMoves.size() - 1; i >= 0; i--)
   {
     for(int j = 0; j < refMoves.size(); j++)
     {
       if(refMoves.elementAt(j).equals( allMoves.elementAt(i) ))
        referenced = true;
     }
     if(!referenced)
       getMoveList().remove(allMoves.elementAt(i));
     referenced = false;
   }

   referenced = false;
   for(int i = allZones.size() - 1; i >= 0; i--)
   {
     for(int j = 0; j < refZones.size(); j++)
     {
       if( ((String)refZones.elementAt(j)).equals( ((BZone)allZones.elementAt(i)).getName() ))
        referenced = true;
     }
     if(!referenced)
       getZones().remove(allZones.elementAt(i));
     referenced = false;
   }
 }





  public void convertToWizardTemplate()
  {

    if (limbOptions.size() == 0  )
    {

      BComponent[] myComponents = getBreathComponents(true);

      // clear resultant moves flag,
      // using this flag to indicate if the component has been processed,
      // yes, I know this is bad, but didnt want to create another uneeded variable

      for (int o = 1; o < myComponents.length ;o++ )
      {
        myComponents[o].inResultantMove = false;
      }

      // start at 1 because chasssis should not be included
      for (int o = 1; o < myComponents.length ;o++ )
      {
        BComponent outerTest =  myComponents[o];
        Vector limbs = new Vector();

        for (int i = 1; i < myComponents.length ; i++ )
        {
          BComponent innerTest =   myComponents[i];

          if (    (innerTest.getTreeNode().getParent() == outerTest.getTreeNode().getParent()  )
              &&  innerTest.getConnectPointParent().equals(outerTest.getConnectPointParent())

            )
          {

            //using resultant flag to indicate that the component is in a list
            if (!innerTest.inResultantMove)
            {
             limbs.add( new BWizardLimb( innerTest) );
              innerTest.inResultantMove = true;
            }
          }
        }

        // always at lest 1 because the testing component will be added
        if (limbs.size() > 1 )
        {

          BWizardLimbOptions lo = new  BWizardLimbOptions(limbs);
          limbOptions.add(lo);
          lo.detachAllLimbsFromBot();
        }


      }
     }
     else // limbOption.size () != 0
     {
        for (int i = 0; i < limbOptions.size(); i++)
        {
           ((BWizardLimbOptions)limbOptions.get(i)).detachAllLimbsFromBot();

        }
     }


     // remove thumbnails
     this.removeThumbnails();


  }


  public void init(boolean forceUpgrade, Vector upgradeErrors, Vector uniqueItemsOwned, boolean optimise)
  {
    super.init(forceUpgrade, upgradeErrors, uniqueItemsOwned, false);

    if (limbOptions != null)
      for (int i = 0; i < limbOptions.size(); i++)
      {
        BWizardLimbOptions w =
          (BWizardLimbOptions) limbOptions.get(i);

        w.init();
        w.setParentBot(this);
      }

    if (blockOptions != null)
      for (int i = 0; i < blockOptions.size(); i++)
      {
        BWizardBlockOptions w =
          (BWizardBlockOptions) blockOptions.get(i);

        //w.init();
        w.setParentBot(this);
      }

  }


  public static BWizardTemplateBot loadWizFromFile(String file)
    { return loadWizFromFile(file, true); }

  public static BWizardTemplateBot loadWizFromFile(String file, boolean init)
    { return (BWizardTemplateBot) loadFromFile(file, init); }
  

  public static String BOT_TYPE = "WizardTemplateBot";

  public String getType()
    { return BOT_TYPE; }   

  protected void recalculateBotInfo(boolean fireChangeEvent)
  {

   // BDebug.print ("BWiz.recalInfo entered");

    float oldPrice     = botTotalPrice;
    float oldWeight    = botTotalWeight;
    int oldComponents  = botTotalComponents;
    int oldHealth      = botTotalHealth;

    // Recalculate bot info (sans limb-options).
    super.recalculateBotInfo(false);

    float tbotTotalPrice      = botTotalPrice;
    float tbotTotalWeight     = botTotalWeight;
    int   tbotTotalComponents = botTotalComponents;
    int   tbotTotalHealth     = botTotalHealth;

   // BDebug.print ("BWiz.recalInfo aftersuper");

    if (limbOptions != null)
    {
      // Calculate all limb-options info.
      Enumeration e = limbOptions.elements();

      while (e.hasMoreElements())
      {
        BWizardLimbOptions w =
          (BWizardLimbOptions) e.nextElement();

        tbotTotalPrice      += w.getPrice();
        tbotTotalWeight     += w.getWeight();
        tbotTotalHealth     += w.getDamageMax();
        tbotTotalComponents += w.getComponentCount();
      }
    }

    botTotalPrice      = tbotTotalPrice;
    botTotalWeight     = tbotTotalWeight;
    botTotalHealth     = tbotTotalHealth;
    botTotalComponents = tbotTotalComponents;

    // BDebug.print ("BWiz.recalInfo aftersuper");

    // Fire off property-change events.
    if (fireChangeEvent)
      fireBotInfoChange(oldPrice, oldWeight, oldComponents, oldHealth);
  }

}

