package roboforge;

public class SVerifyCC
{
 /*

  * Description:
    Checks that a credit card number is valid
  * Example:
  * Modification History ( Name, Date, Notes ):
    Created HSC, 23-November-1999

  */

  public SVerifyCC()
  {
  }

  public static void verifyCCNumber (String cardNumber) throws CCException
  {
    int numberLength;
    String cardName;
    int shouldLength;
    int missing;
    int left4Digits;


    /*
     1) Do the first four digits fit within proper ranges?
        If so who is the card issuer and how long should the number be.
    */

    numberLength = cardNumber.length();
    left4Digits = (Integer.valueOf(cardNumber.substring(0,4))).intValue();


    /*if ((left4Digits >= 3000) && (left4Digits <= 3000) || ((left4Digits >= 3600) && (left4Digits <= 3699)) || ((left4Digits >= 3800) && (left4Digits <= 3889)))
    {
      cardName = "Diners Club";
      shouldLength = 14;
    }
    else if ((left4Digits >= 3400) && (left4Digits <= 3499) || ((left4Digits >= 3700) && (left4Digits <= 3799)))
    {
      cardName = "American Express";
      shouldLength = 15;
    }
    else if ((left4Digits >= 3528) && (left4Digits <= 3589))
    {
      cardName = "JCB";
      shouldLength = 16;
    }
    else if ((left4Digits >= 3890) && (left4Digits <= 3899))
    {
      cardName = "Carte Blache";
      shouldLength = 14;
    }*/
    if ((left4Digits >= 4000) && (left4Digits <= 4999))
    {
      cardName = "Visa";
      if (numberLength > 14)
        shouldLength = 16;
      else if (numberLength < 14)
        shouldLength = 13;
      else
      {
        throw  new CCException ("The Visa number entered only has 14 numbers.  Visa cards have either 13 or 16 numbers.  Please check and re-enter the number.");
      }
    }
    else if ((left4Digits >= 5100) && (left4Digits <= 5599))
    {
      cardName = "MasterCard";
      shouldLength = 16;
    }
    /*else if (left4Digits == 5610)
    {
      cardName = "Australian BankCard";
      shouldLength = 16;
    }
    else if (left4Digits == 6011)
    {
      cardName = "Discover/Novus";
      shouldLength = 16;
    }*/
    else
    {
      throw  new CCException ("Unknown Credit Card.  Please check that the first four numbers are correct");
    }

    // 2) Is the number the right length?
    if (numberLength != shouldLength)
    {
      missing = numberLength - shouldLength;
      if (missing < 0)
      {
        throw  new CCException ("Not enough card numbers entered.  " + cardName + " cards need " + shouldLength + " numbers and only " + numberLength + " were entered.  Please check and re-enter the number.");
      }
      else
      {
        throw  new CCException ("Too many card numbers entered.  " + cardName + " cards only need " + shouldLength + " numbers and " + numberLength + " were entered.  Please check and re-enter the number.");
      }
    }

    // 3) Does the number pass the Mod 10 Algorithm Checksum?
    if (!mod10Solution (cardNumber))
    {
      throw  new CCException ("Invalid card number, please check the number and try again.");
    }

  }
 public static boolean mod10Solution (String n)
 {
    try
    {
      /*
      ** known as the LUHN Formula (mod10)
      */
      int j = n.length();
      String [] s1 = new String[j];
      for (int i=0; i < n.length(); i++) s1[i] = "" + n.charAt(i);

      int checksum = 0;

      for (int i=s1.length-1; i >= 0; i-= 2)
      {
        int k = 0;
        if (i > 0)
        {
          k = Integer.valueOf(s1[i-1]).intValue() * 2;
          if (k>9)
          {
            String s = "" + k;
            k = Integer.valueOf(s.substring(0,1)).intValue() + Integer.valueOf(s.substring(1)).intValue();
          }
          checksum += Integer.valueOf(s1[i]).intValue() + k;
        }
        else
          checksum += Integer.valueOf(s1[0]).intValue();
      }
      return ((checksum % 10) == 0);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      return false;
    }
 }
}



class CCException extends Exception
{ 
  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
public CCException()
  {
  }
  public CCException(String msg)
  {
    super(msg);
  }
}