package roboforge.ai.actions;

import roboforge.*;
import java.util.*;

public class BActionList
{
  // vector of actions
  protected Vector aList;
  protected BAction[] continualActions;
  //protected Vector removedActions;    // this has been removed because itwas not used

  public BActionList()
  {
    aList = new Vector();
    //removedActions = new Vector();
    continualActions = new BAction[0];
  }


  public boolean canAddAction(BAction a)
  // can I add this action to the list
  {
    boolean canDo = true;

    BAction current;

    for (int i = 0; i < aList.size() && canDo ; i++ )
    {
      current =  (BAction) aList.get(i);

      if (a.conflictsWith( current ) )
        if ( ! a.overrides( current ) )
          canDo = false;

    }

    return canDo;

  }


  protected BAction removeAction( int i )
  {
    BAction toRemove = (BAction)aList.remove(i);
    toRemove.stop();

    return toRemove;
   // BDebug.print ("Removing Action " + toRemove.getName() );

    //if (!removedActions.contains(toRemove) )
    //  removedActions.add(toRemove);


  }

  protected Vector getConflicting( BAction a)
  {
    Vector newVector = new Vector(2);

    for (int i = aList.size() -1 ; i >=0  ; i-- )
    {

      if (a.conflictsWith( (BAction) aList.get(i) ) )
        newVector.add( aList.get(i)  );
    }

     return newVector;

  }

  private void removeConflicting( BAction a)
  {
    // go backwards to remove
    for (int i = aList.size() -1 ; i >=0  ; i-- )
    {

      if (a.conflictsWith( (BAction) aList.get(i) ) )
        removeAction(i);

    }

  }



  public void addAction (BAction a)
  // PRE this method should only be called after canDoAction has been checked
  {
    //BDebug.print ("Adding Action " + a.getName() );
    removeConflicting(a);
    a.start();
    aList.add(a);

  }

  public void addContinualAction(BAction a)
  {
    // can only have the action in the list once
    for (int i = continualActions.length - 1; i >=0; i--)
      if (continualActions[i].equals(a) )
      {
        continualActions[i] = a;
        return;
      }

    BAction[] newCon = new BAction[ continualActions.length +1 ];

    for (int i = 0; i <  continualActions.length ; i ++ )
      newCon[i] = continualActions[i];

    newCon[continualActions.length ] = a;

    continualActions = newCon;
  }



  public void clearActions()
  {
    for (int i = aList.size() -1 ; i >=0  ; i-- )
        removeAction(i);

    for (int i = continualActions.length -1 ; i >=0  ; i-- )
        continualActions[i].stop();


  }


  public Vector getActionNames()
  {
    Vector actionNames = new Vector(aList.size());

    for (int i = 0 ; i < aList.size() ; i++ )
        actionNames.add(  ((BAction)aList.get(i)).getName()    );

    return actionNames;
  }


  public void executeActions( float secs)
  // execute the actions and removes them if they have finished
  {

    BAction current = null;

    for (int i = aList.size() -1 ; i >=0  ; i-- )
    {
        current = (BAction) aList.get(i);

        if (!current.execute(secs) )
          removeAction(i);
    }

     for (int i = continualActions.length -1 ; i >=0  ; i-- )
        continualActions[i].execute(secs);


  }

  public String toString()
  {
    return "BActionList " + hashCode() + " " + aList.toString();
  }





}