package roboforge.ai.actions;

import roboforge.*;
import java.util.*;


public class BBotActionList extends BActionList
{

  Vector chainedMoves = new Vector(5);

  public BBotActionList()
  {

  }


  protected class ChainedActionPair
  {
    BMove resultantMove;
    BMove chainedMove;

    public ChainedActionPair(BMove rMove, BMove cMove )
    {
      resultantMove = rMove;
      chainedMove = cMove;
    }

  }


  public void  getHighestLimbsInResultantMove (Vector vToFill)
  {
    for (int i = 0; i < aList.size(); i++ )
      if (aList.get(i) instanceof BMove)
      {
        BMove tempM =  (BMove) aList.get(i);

        if (tempM.getIsResultantMove() )
        {
          BJoint[] movedJoints = tempM.getMovedJoints(true);

          for (int x = movedJoints.length -1; x >=0; x-- )
            vToFill.add( movedJoints[x] );

        }
      }

  }

  /*
  // DJG June 2001 - Stop Any Chassis Moves
  public void stopChassisMoves()
  {
  	for (int i = 0; i < aList.size(); i++ )
      if (aList.get(i) instanceof BMove)
      {
        BMove tempM =  (BMove) aList.get(i);

        if ( tempM.getContainsChassisMove() )
          tempM.stop();

      }
  }
  */

  public boolean  containsResultantMoves ()
  {

    for (int i = 0; i < aList.size(); i++ )
      if (aList.get(i) instanceof BMove)
      {
        BMove tempM =  (BMove) aList.get(i);

        if (tempM.getIsResultantMove() )
          return true;

      }

    return false;

  }


  private void replaceResultantChain(BMove oldResult,BMove rMove)
  {
    for (int i = chainedMoves.size()-1; i >=0; i-- )
    {
      ChainedActionPair cP = (ChainedActionPair) chainedMoves.get(i);

      if ( rMove.conflictsWith( cP.chainedMove ) )
        cP.resultantMove = rMove;
      else if ( cP.resultantMove == oldResult)
        reInsertChainedMove(cP);

    /*
      ChainedActionPair cP = (ChainedActionPair) chainedMoves.get(i);
      if (cP.resultantMove == m)
        cP.resultantMove = rMove;     */
    }
  }


  private void reInsertChainedMove(ChainedActionPair cP)
  {
     BMove m = cP.chainedMove;
     cP.chainedMove.getNextSnapshot(false);

     Vector conflicting = super.getConflicting(cP.chainedMove );
     if (conflicting.isEmpty() )
      aList.add(cP.chainedMove );
     else
     {
        //BDebug.print ("Re-Insertion of " + cP.chainedMove + " blocked by"  + conflicting );
        cP.chainedMove.stop();

      }

      //BDebug.print ("BBotActionList.removeAction re-adding " + cP.chainedMove);
      chainedMoves.remove(cP);
  }


    protected BAction removeAction( int index )
  {

    BAction removed = super.removeAction(index);

    if ( removed instanceof BMove)
    {
      BMove m = (BMove)removed;
      if (m.getIsResultantMove() )
        for (int i = chainedMoves.size() - 1; i >=0; i-- )
        {
          ChainedActionPair cP = (ChainedActionPair) chainedMoves.get(i);

          if (cP.resultantMove == m)
            reInsertChainedMove( cP);


        }

    }

    return removed;


  }


   public void addResultantMove (BMove rMove, boolean isDeadlockMove)
  // PRE this method should only be called after canDoAction has been checked
  {
    //BDebug.print ("BBotActionList.addResultantMove rMove " + rMove);



    if (!isDeadlockMove)
    {
      Vector conflicting = super.getConflicting(rMove );

     // BDebug.print ("BBotActionList.addResultantMove conflicting Moves " + conflicting);

      for (int i = conflicting.size() - 1; i >=0; i-- )
        if (conflicting.get(i) instanceof BMove )
        {
          BMove m = (BMove) conflicting.get(i);

          // if the move is not a resultant move, remove and chain the move without notifying it
          if ( m.getIsResultantMove() )
          {
            aList.remove(m);
            replaceResultantChain(m, rMove);
          }
          else
          {

            // remove the action without notifying it
            aList.remove(m);

            // chain the move
            m.clearChassisWait();
            chainedMoves.add( new ChainedActionPair(rMove, m) );

          }

      }
    }
    else
    {
        clearChainedMoves();



    }

      super.addAction(rMove);


  }

  private void clearChainedMoves()
  {
    // isf this is a deadlock move, clear the list
       for (int i = chainedMoves.size() - 1; i >=0; i-- )
       {
          ChainedActionPair cP = (ChainedActionPair) chainedMoves.get(i);
          cP.chainedMove.stop();
       }

       chainedMoves.clear();

  }

  public void clearActions()
  {
    super.clearActions();
    clearChainedMoves();


  }


  public boolean canAddAction(BAction a)
  // can I add this action to the list
  {
    boolean canDo = super.canAddAction(a);

    for (int i = chainedMoves.size() - 1; i >=0; i-- )
    {
          ChainedActionPair cP = (ChainedActionPair) chainedMoves.get(i);
          canDo = canDo && (!a.conflictsWith(cP.chainedMove));
          //BDebug.print("BBotActLst.canAddAction: "+canDo+" - "+a.toString());
    }

    //BDebug.print("BBotActLst.canAddAction: "+canDo+" - "+a.toString());

    return canDo;



  }

  public boolean isEmpty()
  {
    return aList.isEmpty();
  }


  // DJG June 2001 - Pause any Moves with Joints as parents of argument
  public void pauseMoves(BComponent collisionComponent)
  {

    //BDebug.print("BBotActionList.pauseMoves() called.");

  	for (int i = 0; i < aList.size(); i++ )
      if (aList.get(i) instanceof BMove)
      {
        BMove tempM =  (BMove) aList.get(i);

        boolean found = false;

        BJoint[] movedJoints = tempM.getMovedJoints(true); // Highest Only

        for(int j = movedJoints.length-1; j >=0; j--)
        {
          if ( movedJoints[j].getTreeNode().isNodeDescendant(collisionComponent.getTreeNode()) )
          {
            //BDebug.print("Pausing move "+tempM+" as "+movedJoints[j].toString()+" is parent of "+collisionComponent.toString());
            movedJoints[j].pause();
            found = true;
          }
        }

        if ( found )
          tempM.pause();
          
      }
  }

  // DJG June 2001 - Stop current snapshot of any Moves with Joints as parents of argument
  public void stopMoves(BComponent collisionComponent)
  {

    //BDebug.print("BBotActionList.pauseMoves() called.");

  	for (int i = 0; i < aList.size(); i++ )
      if (aList.get(i) instanceof BMove)
      {
        BMove tempM =  (BMove) aList.get(i);

        boolean found = false;

        BJoint[] movedJoints = tempM.getMovedJoints(true); // Highest Only

        for(int j = movedJoints.length-1; j >=0 && !found; j--)
        {
          if ( movedJoints[j].getTreeNode().isNodeDescendant(collisionComponent.getTreeNode()) )
          {
            //BDebug.print("Stopping move "+tempM+" as "+movedJoints[j].toString()+" is parent of "+collisionComponent.toString());
            tempM.getNextSnapshot(false);
            found = true;
          }
        }

      }
  }



} 