package roboforge.ai.decisions;

import javax.swing.tree.*;
import roboforge.ui.ai.*;

public class BDecisionTreeModel extends DefaultTreeModel
{

  BDecisionTree tree;

  public BDecisionTreeModel()
    { super(new BRootNode()); }

  public BDecisionTreeModel(BRootNode root, BDecisionTree tree)
  {
    super(root);
    this.tree = tree;
  }

  public void valueForPathChanged(TreePath path, Object value)
  {
	  BDecisionTreeNode node =
      (BDecisionTreeNode) path.getLastPathComponent();

    node.setName(value.toString());

    nodeChanged(node);
  }


  // Child maintenance methods...
  // -------------------------------------------------------------


  public void removeNodeFromParent(MutableTreeNode node)
  {
    MutableTreeNode parent =
      (MutableTreeNode) node.getParent();

    if (parent == null)
      throw new IllegalArgumentException("Node does not have a parent.");

    int      n            = parent.getChildCount();
    int[]    childIndex   = new int[1];

    childIndex[0] = parent.getIndex(node);
    parent.remove(childIndex[0]);

    // HACK: Check if a node actually has been deleted.
    //       This is done because BDecisionNode inserts
    //       a new child during the remove() op.

    if (parent.getChildCount() < n)
    {
      Object[] removedArray = new Object[1];
      removedArray[0] = node;
      nodesWereRemoved(parent, childIndex, removedArray);
    }
    else
      nodeStructureChanged(parent);

  }

  public void notifyTree()
  {
    //tree.expandChildren((BRootNode)this.getRoot());

    //reload();

    //this.nodeStructureChanged((BRootNode)this.getRoot());
    tree.updateExpansion();
  }


  public void insertNodeInto
  (
    MutableTreeNode child,
    MutableTreeNode parent,
    int index
  )
  {
    int n = parent.getChildCount();

    MutableTreeNode grandpa =
      (MutableTreeNode) parent.getParent();

    int[] childIndex = new int[1];
    parent.insert(child, index);

    // HACK: Check if a node actually has been inserted.
    //       This is done because BNOPNode replaces
    //       itself with its child during the insert() op.

    if (parent.getChildCount() > n) // parent.getParent() == grandpa)
    {
      childIndex[0] = index;
      nodesWereInserted(parent, childIndex);
    }
    else
    {
      if (grandpa != null)
        nodeStructureChanged(grandpa);

      if (parent.getParent() != null)
        nodeStructureChanged(parent);
    }
  }


}