package roboforge.ai.decisions;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BSimulation;
import roboforge.BBot;

import roboforge.ai.expressions.*;
import roboforge.ai.values.*;
import roboforge.ai.worldstate.*;
import roboforge.ui.components.HSBImageIcon;
import roboforge.ui.components.BBorderedIcon;

import java.util.Enumeration;

import javax.swing.Icon;
import javax.swing.tree.*;
import java.awt.Color;
import java.io.*;


public class BSimpleDecisionNode extends BDecisionNode
{

  BBorderedIcon smallIcon;

  private Color ENEMY_COLOR = BGlobal.colors.getAIDecisionColor_Enemy();
  private Color ME_COLOR = BGlobal.colors.getAIDecisionColor_Me();

  String worldItemRef;
  String operatorRef;
  BValue valueRef;
  String iconName;

  public BSimpleDecisionNode()
  {
    super();
  }

  public BSimpleDecisionNode(String worldItemRef, String operatorRef, BValue valueRef, String iconName, String defaultName)
  {
    super();
    this.worldItemRef = worldItemRef;
    this.operatorRef = operatorRef;
    this.valueRef = valueRef;
    this.iconName = iconName;

    setName(defaultName);

  }

  public BSimpleDecisionNode(BSimpleDecisionNode node)
  {
    super(node);
    this.worldItemRef = node.worldItemRef;
    this.operatorRef = node.operatorRef;
    this.valueRef = node.valueRef;
    this.smallIcon = node.smallIcon;
    this.iconName = node.iconName;

  }


  public Object clone()
  {
   // BDebug.print("BSimpleDecisionNode.clone()" );

    BSimpleDecisionNode node =
      new BSimpleDecisionNode(this);

    // Add in dummy nodes as needed...

    node.add(new BNOPNode());

    return node;
  }

  public void setWorldItemRef(String worldItemRef)
    { this.worldItemRef = worldItemRef;}

  public String getWorldItemRef()
    { return worldItemRef;}

  public void setOperatorRef(String operatorRef)
    { this.operatorRef = operatorRef;}

  public String getOperatorRef()
    { return operatorRef;}

  public void setValueRef( BValue valueRef)
    { this.valueRef = valueRef;}

  public BValue getValueRef()
    { return valueRef;}

  public int getMaxChildren()
    { return 1; }


  public boolean canAddChildren()
    { return getChildCount() < getMaxChildren(); }

  public boolean isRemoveable()
  {
    return !( getParent() instanceof BZoneGroupNode) ;
  }


 /* public void insert(MutableTreeNode child, int index)
  {
    super.insert(child, index);

    // If size limit exceeded, remove displaced sibling.
    //if (getChildCount() > 1)
      super.remove(1);
  }  */


  public Icon getIcon()
  // smallIcon
  {

    BBorderedIcon icon = smallIcon;

    // create the icon if none exists
    if(icon == null)
    {
      icon = createTreeIcon(iconName, BGlobal.colors.getAIDecisionColor_Me()); //new BBorderedIcon(new Color(100, 100, 100), createTreeIcon(iconName), 6);
      smallIcon = icon;
    }

    String name = worldItemRef;

    if (this.getVisited())
      icon.setBorderColor(BGlobal.colors.getLightestShade());
    else if (this.getParent() instanceof BZoneGroupNode )
      icon.setBorderColor(BGlobal.colors.getAIGroupColor());
    else
      icon.setBorderColor(BGlobal.colors.getAIDecisionColor_Me());

    return icon;
  }

  public Icon getSelectedIcon()
    { return getIcon(); }


    public void setState(Object o)
  {
    // check instance for old decisions;

    if (o instanceof DecisionNodeState )
      super.setState(o);

    BSimpleDecisionState state =
      (BSimpleDecisionState) o;

    this.worldItemRef = state.worldItemRef;
    this.operatorRef =state.operatorRef;
    this.valueRef = state.valueRef;
    this.iconName = state.iconName;
  }


  public Object getState()
    { return new BSimpleDecisionState(this); }

  protected static class BSimpleDecisionState extends DecisionNodeState
  {
    static final long serialVersionUID = -1L;
    protected String worldItemRef;
    protected String operatorRef;
    protected BValue valueRef;
    protected String iconName;


    public BSimpleDecisionState(BSimpleDecisionNode node)
    {
      super(node);
      this.worldItemRef = node.worldItemRef;
      this.operatorRef =node.operatorRef;
      this.valueRef = node.valueRef;
      this.iconName = node.iconName;

    }
  }








} 