package roboforge.ai.expressions;

import roboforge.ai.*;
import roboforge.ai.worldstate.*;
import roboforge.ai.values.*;


public class BWorldItemRef extends BTerminal
{

  // Serialization ID.
  // -----------------------------------------------------------
  static final long serialVersionUID = -1L;


  protected String key;
    // Key which identifies the referenced world-item.

  protected transient BWorldItem item;
    // Referenced world item; obtained by key lookup.

  protected BDecisionTreeAI ai;
    // Reference to the bot's AI - used for worldState lookup.


  public BWorldItemRef(BDecisionTreeAI ai, String key)
  {
    super();
    this.key = key;
    this.ai  = ai;
  }

  public BWorldItemRef(BWorldItemRef o)
  {
    super();
    this.key = new String(o.key);
    this.ai  = o.ai;
  }

  public Object clone()
    { return new BWorldItemRef(this); }


  public BValue evaluate()
  {
    // Look up appropriate item...
    if (item == null)
      lookupItem();

    if (item != null)
      return item.getValue();
    else
      return BValue.UNDEFINED;
  }

  public int getType()
  {
    if (item == null)
      lookupItem();

    if (item != null)
      return item.getType();
    else
      return BType.UNDEFINED;
  }

  protected void lookupItem()
  {
    BWorldState world = ai.getWorldState();

    if (world != null)
      item = world.get(key);
  }


  public String toString()
    { return "["+ key+ "]"; }

}