package roboforge.ai.memory;

import roboforge.ui.BNamedIconic;
import roboforge.ai.values.*;

import java.io.Serializable;

import javax.swing.Icon;


public class BMemoryItem
  implements BNamedIconic, BType, Serializable
{

  // Serialization ID.
  // -----------------------------------------------------------
  static final long serialVersionUID = -1L;


  // Internal variables...
  // -------------------------------------------------------------

  protected String name;
    // Name of this memory item.

  protected int type = BType.UNDEFINED;
    // Type of value that this item holds.

  protected BValue initial;
    // Initial value of this item.

  protected transient BValue value;
    // Current value of this item.


  // Constructor.
  // -------------------------------------------------------------

  public BMemoryItem()
    { }

  public BMemoryItem(String name)
    { this.name = name; }

  public BMemoryItem(BMemoryItem copy)
  {
    if (copy != null)
    {
      this.name = copy.name;
      this.type = copy.type;
      this.initial = (BValue)copy.initial.clone();
      this.value = (BValue)copy.initial.clone();
    }
  }

  public BMemoryItem(String name, int type, BValue initial)
  {
    this.name    = name;
    this.type    = type;
    this.initial = initial;
    this.value   = initial;
  }

  private void createDefaultInitial()
  {
    if (this.type == BValue.BOOLEAN)
    {
      this.initial = new BBoolean(false);
      this.value = new BBoolean(false);

    }
    else if (this.type == BValue.NUMBER)
    {
      this.initial = new BNumber(0);
      this.value = new BNumber(0);
    }
    else
    {
      this.initial = new BString("");
      this.value = new BString("");

    }

  }

  public void validateAndFixInitialValue()
  {
    if (this.initial == null)
      createDefaultInitial();
    else if (this.value == null)
      createDefaultInitial();
    else if (this.initial.getType() != this.type)
      createDefaultInitial();
    else if (this.value.getType() != this.type)
      createDefaultInitial();

  }

  public void init()
    { this.value = initial; }

  // Accessor methods...
  // -------------------------------------------------------------

  public void setName(String name)
    { this.name = name; }

  public String getName()
    { return name; }

  public String toString()
    { return name; }


  public int getType()
    { return type; }

  public void setType(int type)
    { this.type = type; }


  public void setInitialValue(BValue value)
  {
    if (value == null)
      initial = BValue.UNDEFINED;
    else if (type == value.getType())
      initial = this.value = value;
    else
      initial = BValue.UNDEFINED;

    setValue(initial);
  }

  public BValue getInitialValue()
  {
    if (initial == null)
      return BValue.UNDEFINED;
    else
      return initial;
  }

  public void setValue(BValue value)
  {
    if (type == value.getType())
    {
      this.value = (BValue)value.clone();

    }
    else
      this.value = BValue.UNDEFINED;
  }

  public BValue getValue()
  {
    if (value == null)
      return BValue.UNDEFINED;
    else
      return value;
  }


  public Icon getIcon()
    { return BTypeIcons.getIconForType(type); }

  public Icon getSelectedIcon()
    { return BTypeIcons.getSelectedIconForType(type); }
  
  public void setIcon(Icon i)
    { /* Ignore */ }
}
