package roboforge.ai.orient;

import roboforge.*;
import roboforge.ai.worldstate.*;
import roboforge.ai.values.*;

import java.util.Vector;


public class BOrientDistanceOnly extends BOrient
{
  static final long serialVersionUID = -1L;

  private float distance;
  private float errorMargin;

  // working variables
  private transient BNumber oppAng;
  private transient BNumber oppDst;

  private static float STOPANDTURN_THRESHOLD   = 20f;
  private static float ANGLE_EQUALS_THRESHOLD   = 2f;
  private static float MAX_FORWARD_BEND = 10f;


  public BOrientDistanceOnly(float distance, float errorMargin)
  {
    this.distance = distance;
    this.errorMargin = errorMargin;
  }

  public Object clone()
    { return new BOrientDistanceOnly(this.distance, this.errorMargin); }

  public boolean equals(Object o)
    { return (o instanceof BOrientDistanceOnly); }

  public void setDistance(float distance)
    { this.distance = distance; }

  public float getDistance()
    { return distance; }


   public boolean executeOrient(BBot bot)
  {

    BWorldState state = bot.getWorldState();

    // find opponent Angle
    BWorldItem angle = state.get(BSimulation.WSTATE_OPPANGLE);
    oppAng = (BNumber)angle.getValue();

    BWorldItem dist = state.get(BSimulation.WSTATE_OPPDISTANCE);
    oppDst = (BNumber)dist.getValue();


    float angleToTurn = (float) oppAng.getValue() ;
    float distToGo = (float) oppDst.getValue() - distance;
    float smallestAngle;

    if (angleToTurn < 0 )
      smallestAngle = -angleToTurn;
    else if (angleToTurn < 180 )
        smallestAngle = angleToTurn;
    else
        smallestAngle = 360 - angleToTurn ;

    if (  java.lang.Math.abs(distToGo) >  errorMargin )
    {
        // get to the position
        //BDebug.print("BOrientDistanceOnly smallestAngle" +  smallestAngle + " error Margin ");

        if (smallestAngle > STOPANDTURN_THRESHOLD )
        {
          int rotateDir;

          if ( angleToTurn < 180 )
            rotateDir =  BMoveList.CHASSIS_ROTATE_RIGHT;
          else
            rotateDir =  BMoveList.CHASSIS_ROTATE_LEFT;

          bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(rotateDir), smallestAngle, BChassis.ADD_MODE_OVERRIDE, null );


         }
        else
        {


          smallestAngle = java.lang.Math.min(MAX_FORWARD_BEND, smallestAngle);

          if ( angleToTurn < 180 )
            smallestAngle = -smallestAngle;
          else
            smallestAngle = smallestAngle;

          if (distToGo > 0)
          {
            if ( bot.getChassis().getCurrentMoveType() ==  BMoveList.CHASSIS_MOVE_BACKWARDS )
              bot.getChassis().terminateChassisMove(false);

            bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(BMoveList.CHASSIS_MOVE_FORWARD), distToGo,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE , smallestAngle,  null);
          }
          else
          {

             if ( bot.getChassis().getCurrentMoveType() ==  BMoveList.CHASSIS_MOVE_FORWARD )
              bot.getChassis().terminateChassisMove(false);

             bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(BMoveList.CHASSIS_MOVE_BACKWARDS), - distToGo,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE , null);

          }
        }
    }
    else
      return true;  // orient complete



     return false;


  }


  /*
  public String getName()
    { return "Orienting" + BPhysics.roundToString(distance,1) + " meters from opponent."; }
  */

  public String getName()
    { return "Orient distance only"; }



}



