package roboforge.animation;

import java.util.*;
import roboforge.BDebug;

public abstract class BAbstractAnimatorAdapter implements BAnimator, BAnimation
{

  protected boolean active = false;
  protected boolean autoActive = false;

  // Manual Implement a Vector, because we need to be as fast as possible for animations
  private static int CHUNK = 5;
  BAnimation animations[] = new BAnimation[CHUNK];
  private int animationSize = 0;

  public BAbstractAnimatorAdapter()
  { this(false); }

  public BAbstractAnimatorAdapter(boolean autoActive)
  {
    setAutoActive(autoActive);
  }


  public void stop()
  {
   // BDebug.print (this + " stopped" );
    active = false;
  }

  public void start()
  {
    active = true;
    //BDebug.print (this + " started" );
  }

  public boolean getActive()
  { return active;}

  public int getSize()
  { return animationSize; }

  public void setAutoActive(boolean on)
  { autoActive = on; }

  public boolean animate()
  {
  	// TODO: Required?
  	if (!active) return false;

    for (int i = animationSize - 1 ; i >=0 ; i--)
    {
      if (!animations[i].animate() )
        remove(animations[i]);
    }

    return true;
  }

  private void extendAnimationsArray()
  {
    BAnimation[] tempA = new BAnimation[animations.length + CHUNK]   ;
    for (int i = animationSize - 1 ; i >=0 ; i--)
      tempA[i] = animations[i];

    animations = tempA;

  }

  private void removeFromArray (BAnimation a)
  {
    boolean foundElement = false;

    for (int i = 0; i <animationSize; i++ )
    {
      if (!foundElement)
      {
        if (animations[i] == a)
          foundElement = true;
      }
      else
        animations[i-1] = animations[i];
    }


    if (foundElement)
      animationSize --;

  }

  public boolean contains (BAnimation a)
  {
    if (a == null)
      return false;
    else if (animationSize == 0 )
      return false;
    else
    {
      for (int i = 0; i <animations.length; i++ )
        if (animations[i] == a)
          return true;

    }

    return false;

  }


   public void add(BAnimation a)
  {
    if (a != null)
    {
      animations[animationSize] = a;
      animationSize ++;

      if (animationSize == animations.length)
        extendAnimationsArray();

      a.start();

      if (autoActive)
        if (!this.getActive() )
          start();
    }


    //BDebug.print ("add caled for " + a);
  }

  public void remove(BAnimation a)
  {
    //BDebug.print ("remove caled for " + a);
    if (a != null)
    {
      removeFromArray(a);
      a.stop();

      if (autoActive && (getSize() == 0))
          stop();
    }
  }

  public void clear()
  {
     for (int i = animationSize - 1; i >=0 ; i--)
          remove(animations[i]);

     if (autoActive)
        if (!this.getActive() )
          stop();


  }

  public void cleanse()
  { animations = null; }




}