package roboforge.animation;

import java.util.*;



public abstract class BAnimationCollection extends BAbstractAnimation
{

  // BAnimationCollection keeps track of a set of animations,
  // and issues the correct messages to each animation.

  protected ArrayList slots;


  public BAnimationCollection()
  {
    super();
    slots = new ArrayList();
  }

  public BAnimationCollection(int n)
  {
    super();
    slots = new ArrayList(n);
  }


  public void add(BAnimation a)
    { slots.add(new BAnimationSlot(a, false)); }

  public void remove(BAnimation a)
    { slots.remove(a); }



  public void start()
  {
    if (isAnimating()) return;
    super.start();

    int n = slots.size();
    for (int i=0; i<n; i++)
      startSlot(getSlot(i), i);
  }

  public void stop()
  {
    if (!isAnimating()) return;
    super.stop();

    int n = slots.size();
    for (int i=0; i<n; i++)
      stopSlot(getSlot(i), i);
  }

  public boolean animate()
  {
    boolean isAnyActive = false;

    int n = slots.size();

    for (int i=0; i<n; i++)
      if (animateSlot(getSlot(i), i))
        isAnyActive = true;

    return isAnyActive;
  }


  protected abstract void    startSlot(BAnimationSlot slot, int i);
  protected abstract boolean animateSlot(BAnimationSlot slot, int i);
  protected abstract void    stopSlot(BAnimationSlot slot, int i);




  protected BAnimationSlot getSlot(int i)
    { return (BAnimationSlot) slots.get(i); }

  protected static class BAnimationSlot
  {
    protected BAnimation animation;
    protected boolean    isActive;

    public BAnimationSlot(BAnimation animation, boolean isActive)
    {
      this.animation = animation;
      this.isActive  = isActive;
    }

    public boolean equals(Object o)
    {
      if (o instanceof BAnimation)
        return this.animation == ((BAnimation) o);
      else
        return super.equals(o);
    }

  }


}
