package roboforge.arenaSystems;

import roboforge.*;
import roboforge.ui.console.*;
import roboforge.util.graphics.*;
import roboforge.util.text.*;
import roboforge.sound.*;
import roboforge.math.*;

import roboforge.animation.*;
import roboforge.animation.animation.*;
import roboforge.animation.animation.b2d.*;
import roboforge.animation.animation.b3d.*;

import javax.media.j3d.*;

import java.awt.*;
import java.awt.image.*;
import java.util.*;

import javax.vecmath.*;
import javax.sound.sampled.*;
import java.awt.font.FontRenderContext;



public class BConsolePopupRef extends BVirtualRefModel
{
  private BConsoleCanvas canvas;

  protected String refPath;
  protected boolean isRefActive;

  // console components
  protected BConsoleContainer refbox;
  protected BConsoleRectangle backdrop;
  protected BConsoleImage refPicture;
  protected BConsoleImage messageArea;

  protected BTextureAnimation refAnimation;

  protected BufferedImage messageBuffer;
  protected Graphics2D messageGraphics;
  protected WritableRaster clearingRaster;

  protected Font titleFont      = new Font(BGlobal.FONT_NAME, Font.BOLD,  18);
  protected Font messageFont    = new Font(BGlobal.FONT_NAME, Font.BOLD, 12);
  protected RefNotBusyListener  busyListener = new RefNotBusyListener();
  protected SwapRefOnStop changeRef = new SwapRefOnStop();

  protected BSound winSound;
  protected BSound deadlockSound;
  protected BSound introASound;
  protected BSound introBSound;

  // formating constants
  private static int BORDER = 25;
  private static int GAP = 10;
  private static int BACKDROPH = 150;
  private static int REF_IMAGESIZE = 128;

  public BConsolePopupRef(BArena arena, BConsoleCanvas canvas, BBattleRecorder recorder)
  {
    super(recorder);

    this.refPath = arena.getRefereeFullPath();

    this.canvas = canvas;
    this.recorder = recorder;

    int x = BORDER;
    int y = canvas.getHeight() - BACKDROPH - BORDER;
    int w = canvas.getWidth() - (BORDER * 2);
    int h = BACKDROPH;

    refbox = new BConsoleContainer();
    refbox.setBounds(x, y, w, h);
    refbox.setRemoveable(true);

    backdrop = new BConsoleRectangle();
    backdrop.setBounds(0, 0, w, h);
    backdrop.setColor(BGlobal.BLACK_3F);
    backdrop.setTransparency(0.5f);

    Texture reftex = BTextures.getTexture(refPath+ "active0000.gif");

    refPicture = new BConsoleImage(reftex);
    refPicture.setLocation(GAP, GAP);
    refPicture.setSize(REF_IMAGESIZE, REF_IMAGESIZE);

    refAnimation = new BTextureAnimation(refPicture.getAppearance(), refPath+ "active", 4, ".gif");

    int messageBufferSize = 256;

    //if (BGlobal.VIDEOMODE.equals(BGlobal.VIDEOMODE_DIRECTX) )
    //  messageBufferSize = 256;
    messageBuffer = new BufferedImage(messageBufferSize,128, BufferedImage.TYPE_INT_ARGB);
    clearingRaster = messageBuffer.copyData(null);

    messageArea = new BConsoleImage(messageBuffer);
    messageArea.setLocation(REF_IMAGESIZE + GAP * 2, GAP);
    messageArea.setTransparency(0);

    messageGraphics =  (Graphics2D) messageBuffer.getGraphics();

    //messageGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);


    // Add elements to the ref. box...
    refbox.add(backdrop);
    refbox.add(messageArea);
    refbox.add(refPicture);


    //sound
    if (recorder.getVictoriousBot() != null )
      winSound = BSound.getSound(refPath+ "win.wav"); //  BGlobal.SOUNDPATH + "/arena/ref/win.wav");
    else
      winSound = BSound.getSound(refPath+ "draw.wav"); // BGlobal.SOUNDPATH + "/arena/ref/draw.wav");

    winSound.setPriority (BSound.PRIORITY_MAX );
    winSound.addLineListener(changeRef) ;

    if (recorder.getDeadlockCount() > 0 )
    {
      deadlockSound = BSound.getSound(refPath+ "deadlock.wav");  // BGlobal.SOUNDPATH + "/arena/ref/deadlock.wav");
      deadlockSound.addLineListener(changeRef) ;
      deadlockSound.setPriority (BSound.PRIORITY_MAX );
    }

    introASound = BSound.getSound(refPath+ "bota.wav");  // BGlobal.SOUNDPATH + "/arena/ref/bota.wav");
    introASound.addLineListener(changeRef) ;
    introASound.setPriority (BSound.PRIORITY_MAX );

    introBSound = BSound.getSound(refPath+ "botb.wav");  // BGlobal.SOUNDPATH + "/arena/ref/botb.wav");
    introBSound.addLineListener(changeRef) ;
    introBSound.setPriority (BSound.PRIORITY_MAX );

  }

  public void cleanse()
  {
      clearRef();

      if (winSound != null)
      {
        winSound.removeLineListener(changeRef );
        winSound.onClose();
      }
      if (deadlockSound != null)
      {
        deadlockSound.removeLineListener(changeRef );
        deadlockSound.onClose();
      }
      if (introASound != null)
      {
        introASound.removeLineListener(changeRef );
        introASound.onClose();
      }
      if (introBSound != null)
      {
        introBSound.removeLineListener(changeRef );
        introBSound.onClose();
      }
      winSound = null;
      deadlockSound = null;
      introASound = null;
      introBSound = null;


      canvas = null;

      refbox = null;
      backdrop = null;
      refPicture = null;
      // ref_inactive = null;

      messageArea = null;
      messageBuffer = null;
      messageGraphics = null;
      clearingRaster = null;

      titleFont = null;
      messageFont = null;
      busyListener = null;
      changeRef = null;




  }


  public void executeActions()
  {
    super.executeActions();

    // BDebug.print("ref.executeActions()");
    if (isRefActive)
      refAnimation.animate();
  }


  public void processEvent(BArenaEvent event)
  {
    super.processEvent(event);

    // use .equals for objects that have been serialized and == for objects that have not
    if (event.getArenaEventType().equals(BArenaEvent.DEADLOCK))
      displayDeadlock();
    else if (event.getArenaEventType() == BArenaEvent.END)
      displayEnd();
    else if (event.getArenaEventType() == BArenaEvent.START)
      displayStart();
    else if (event.getArenaEventType().equals(BArenaEvent.BOTA_STARTMOVE) )
      displayBotA();
    else if (event.getArenaEventType().equals(BArenaEvent.BOTB_STARTMOVE) )
      displayBotB();
    else if (event.getArenaEventType().equals(BArenaEvent.VICTORY_MOVES_START) )
      displayEnd();
    else if (event.getArenaEventType().equals(BArenaEvent.STARTMOVES_FIN) )
      clearRef();
    else if (event.getArenaEventType() == BArenaEvent.CLEAR_EVENTS)
      clearRef();
  }

  private void clearMessageArea()
  {
    if (messageArea.isLive() )
      canvas.remove(messageArea);

    messageBuffer.setData(clearingRaster);
    //messageGraphics.setColor(clearColor);
    //messageGraphics.fillRect(0,0, messageBuffer.getWidth(), messageBuffer.getHeight());


  }

  class RefNotBusyListener implements LineListener
  {

    public RefNotBusyListener()
    {
    }

    public void update(LineEvent event)
    {
      if (event.getType().equals(LineEvent.Type.STOP) )
        setBusy(false);
    }

  }

  class SwapRefOnStop implements LineListener
  {

    public SwapRefOnStop()
    {
    }

    public void update(LineEvent event)
    {
      if (event.getType().equals(LineEvent.Type.STOP) )
      {
        setBusy(false);
        setRefActive(false);
      }
    }

  }

  private void displayBotA()
  {
    displayBot( "", recorder.getBotA() );
    introASound.play();
  }

  private void displayBotB()
  {
    displayBot( "" , recorder.getBotB() );
    introBSound.play();
  }

  private void  displayBot (String chalString, BBot theBot)
  {
    clearRef();

    setBusy(true);
    setRefActive(true);

    clearMessageArea();

    messageGraphics.setFont(titleFont);

    messageGraphics.setColor(BGlobal.colors.getLightestShade());
    messageGraphics.drawString(chalString +  BDictionaryTree.getSharedInstance().filter(theBot.getName()) + "", 0, GAP * 2);
    messageGraphics.setFont(messageFont);
    String creator  = theBot.getCreatorName();
    if (!creator.equals("") )
    {
      creator = BDictionaryTree.getSharedInstance().filter(creator);
      messageGraphics.drawString("Creator:  " + creator, 0, GAP *4);
      messageGraphics.drawString("Cost:  " + BMath.format(theBot.getTotalPrice(),0)  +" cr.", 0, GAP *6);
      messageGraphics.drawString("Weight:  " + BMath.format(theBot.getTotalWeight(),0) +" kg.", 0, GAP *8);
    }
    else
    {
      messageGraphics.drawString("Cost:  " + BMath.format(theBot.getTotalPrice(),0) + " cr.", 0, GAP *4);
      messageGraphics.drawString("Weight:  " + BMath.format(theBot.getTotalWeight(),0) + " kg.", 0, GAP *6);
    }

    messageArea.setImage(messageBuffer);

    canvas.add(refbox);
    setBusy(false);
  }

  private void displayStart()
  {
    setBusy(true);
    setRefActive(false);

    clearMessageArea();

    messageGraphics.setFont(titleFont);
    messageGraphics.setColor(BGlobal.colors.getLightestShade());

    String botAText =  BDictionaryTree.getSharedInstance().filter(recorder.getBotA().getName());
    String botBText =  BDictionaryTree.getSharedInstance().filter(recorder.getBotB().getName());

    FontRenderContext fontR = messageGraphics.getFontRenderContext();

    double aWidth = titleFont.getStringBounds(botAText, fontR).getWidth();
    double bWidth = titleFont.getStringBounds(botBText, fontR).getWidth();
    double vsWidth = titleFont.getStringBounds("vs", fontR).getWidth();
    double maxWidth = Math.max( aWidth, bWidth);

    messageGraphics.drawString( botAText, 0, GAP * 2);
    messageGraphics.drawString( "vs", 0, GAP * 4);
    messageGraphics.drawString( botBText , 0, GAP * 6);
    //messageGraphics.drawString( BDictionaryTree.getSharedInstance().filter(recorder.getBotVsBot()), 0, GAP * 2);


    messageGraphics.setFont(messageFont);
    messageGraphics.drawString(recorder.getTournamentName(), 0, GAP *9);
    messageGraphics.drawString(recorder.getRoundName(), 0, GAP *11);
    messageArea.setImage(messageBuffer);

    canvas.add(refbox);
    setBusy(false);
  }

  private void displayDeadlock()
  {
    clearRef();

    clearMessageArea();
    messageGraphics.setFont(titleFont);
    messageGraphics.setColor(BGlobal.colors.getLightestShade());
    messageGraphics.drawString("Deadlocked.", 0, GAP * 2);
    messageGraphics.setFont(messageFont);
    messageGraphics.drawString("Returning bots to starting positions.", 0, GAP *4);

    messageArea.setImage(messageBuffer);
    canvas.add(refbox);

    setRefActive(true);
    deadlockSound.play();
  }

  boolean endIsUp = false;


  private void displayEnd()
  {
    // BDebug.print("Ref.displayEnd(): endIsUp: "+ endIsUp);

    if (!endIsUp)
    {
      clearRef();

      endIsUp = true;
      setBusy(true);   // to be unset by the stopListener the winSound

      clearMessageArea();
      messageGraphics.setFont(titleFont);
      messageGraphics.setColor(BGlobal.colors.getLightestShade());
      messageGraphics.drawString(recorder.getVictorName(), 0, GAP * 2);
      messageGraphics.setFont(messageFont);

      if (recorder.getVictoriousBot() != null)
      {
        boolean creatorfound = false;

        messageGraphics.drawString(recorder.getWinString(), 0, GAP *4);

        if (recorder.getVictoriousBot().getCreatorName() != null)
          if( !recorder.getVictoriousBot().getCreatorName().equals("") )
          {
            creatorfound = true;
            messageGraphics.drawString("Creator:  " + recorder.getVictoriousBot().getCreatorName(), 0, GAP *8);
            // messageGraphics.drawString("Losing bot destroyed.", 0, GAP *10);
          }

        // if (!creatorfound)
        //  messageGraphics.drawString("Losing bot destroyed.", 0, GAP *8);

      }


      messageGraphics.drawString("Fight duration:  " + recorder.getFightDurationString() + " seconds", 0, GAP *6);



      messageArea.setImage(messageBuffer);

      canvas.add(refbox);
      //if (recorder.getVictoriousBot() == null)
      //{
      //  canvas.add(ref_inactive);
      //}
     // else
      //{

      setRefActive(true);
      // canvas.add(refPicture);
        winSound.play();
     // }
    }

  }

  private void clearRef()
  {
    if (refbox == null)
      return;

    //BDebug.print ("BConsolePopupRef clearRef called");
    endIsUp = false;

    if (refbox.isLive())
      canvas.remove(refbox);

    isRefActive = false;

    // sounds
    if (winSound.isRunning() )
      winSound.stop();
    if (deadlockSound != null)
      if (deadlockSound.isRunning() )
        deadlockSound.stop();
    if (introASound.isRunning() )
      introASound.stop();

  }



  protected void setRefActive(boolean active)
  {
    isRefActive = active;

    // Ensure ref picture is visible...
    /*
      if (backdrop.isLive())
      if (!refPicture.isLive())
        canvas.add(refPicture);
    */

    if (isRefActive)
      refAnimation.start();
    else
      refAnimation.stop();
  }

}