package roboforge.attributes;

import roboforge.BDebug;
import roboforge.math.*;
import roboforge.ui.components.*;
import roboforge.ui.components.table.*;

import java.awt.Component;
import javax.swing.JTable;
import javax.swing.border.*;
import javax.swing.table.*;



public class BRelativeValue extends BAttributeValue
{

  protected static final TableCellRenderer renderer
    = new RelativeCellRenderer();
  
  protected double min, max, value;
  
  
  public BRelativeValue(double value)
    { this(0.0, 1.0, value); }
  
  public BRelativeValue(double min, double max, double value)
  {

    this.min   = min;
    this.max   = max;
    this.value = value;


  }
  
  
  
  public TableCellRenderer getRenderer()
    { return renderer; }

  public double getRelativeValue()
    { return (value - min) / (max - min); }
  
  
  public String toString()
  { 
    long p = Math.round(getRelativeValue() * 100);
    return BMath.format(p)+ "%";
  }
  
  
  
  protected static class RelativeCellRenderer extends BBlockBar
    implements TableCellRenderer
  {

    // Constructor...
    // -------------------------------------------------------------
    
    public RelativeCellRenderer()
    {
      super();
      setMinimum(0);
      setMaximum(100);
      setBorder(new EmptyBorder(4,4,4,4));
    }
    
    // Accessor methods...
    // -------------------------------------------------------------

    public Component getTableCellRendererComponent
    (
      JTable table, Object value,
      boolean isSelected, boolean hasFocus,
	  	int row, int col
    )
    {
      BRelativeValue v = (BRelativeValue) value;
      double p = v.getRelativeValue();

      //BDebug.print ("BRelativeValue Renderer p " + p);
      int    n = (int) Math.round(p * 100);

      setValue(n);
      
      return this;
	  }
  }
  
  
}

