package roboforge.internal;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
//import javax.swing.event.*;
//import javax.swing.border.*;
//import javax.swing.UIManager;
import java.util.*;
import javax.swing.tree.*;
//import javax.swing.border.*;
//import javax.swing.plaf.metal.*;

import roboforge.ui.components.*;

import roboforge.btree.*;
import roboforge.BGlobal;
import roboforge.BDebug;
import roboforge.BBot;
//import roboforge.BWizardLimb;
//import roboforge.BShape3D;
import roboforge.BComponent;
import roboforge.BFileFilter;
import roboforge.BTextures;

//import roboforge.BTheme;

import roboforge.BComponentEditorInfo;
import roboforge.BComponentMaterial;
import roboforge.BComponentCanvas;
import roboforge.BChassis;

import roboforge.BRotateJoint;
import roboforge.BMoveList;
import roboforge.BAutoRotateJoint;
import roboforge.BTelescopicJoint;
import roboforge.BCpu;
import roboforge.BSensor;
import roboforge.BEnergyGenerator;
import roboforge.BWeapon;
import roboforge.BShielding;
import roboforge.BAppearanceDialog;
import roboforge.B3dShapeManager;
import roboforge.BPlaceHolder;
import roboforge.BArenaOrnament;
import roboforge.BHoverChassis;
//import roboforge.BMove;
import roboforge.BWheel;
import roboforge.BPhysics;

import roboforge.BMobileArenaOrnament;
import roboforge.BAnimatedTextureOrnament;
import roboforge.BRotatingArenaOrnament;


/*
* Description:
  Allows the modification, creation of Components.

* Example:
* Modification History ( Name, Date, Notes )
  HSC, 8 April 1999
  HSC, 19 April changed paths to be relative.
  AWH, 01 Feb 2001, Changed components to new and made some minor improvements
*/
public class BCompEditor extends JFrame implements ActionListener
{
	static final long serialVersionUID = -1L;
  protected BUIResources res = new BUIResources( new Object[]
  {

    "defAppButton.text",                "Default appearance",
    "defAppButton.width",               new Integer(200),

    "vrmlButton.text",                  "Change VRML",
    "vrmlButton.width",                 new Integer(200),

    "guessButton.text",                 "Guess weight & dmg",
    "guessButton.width",                new Integer(200),

    "updateCButton.text",               "Update chassis moves",
    "updateCButton.width",              new Integer(200),

    "newButton.text",                   "New",
    "newButton.width",                  new Integer(66),

    "loadButton.text",                  "Load",
    "loadAppButton.width",              new Integer(66),

    "saveButton.text",                  "Save",
    "saveButton.width",                 new Integer(66),

    "refreshRButton.text",              "Refresh ratios",
    "refreshButton.width",              new Integer(100),

    "setPriceButton.text",              "Set price",
    "setPriceButton.width",             new Integer(100),

    "volLabel.text",                    "Vol (m3)",
    "volLabel.width",                   new Integer(50),

    "volField.width",                   new Integer(150),

    "matCombo.width",                   new Integer(200),

    "matLabel.text",                    "Materials",
    "matLabel.width",                   new Integer(200),

    "leftBox.width",                    new Integer(200),

    "rightBox.width",                   new Integer(590),

    "topLeftBox.width",                 new Integer(200),
    "topLeftBox.height",                new Integer(200),

    "confirmLabel.text",               "",
    "confirmLabel.width",              new Integer(200),
    "confirmLabel.fontname",           "Arial",
    "confirmLabel.fontsize",           new Integer(14),
    "confirmLabel.fontstyle",          new Integer(1),
    "confirmLabel.height",             new Integer(28),
    "confirmLabel.antialiased",        Boolean.TRUE,

  });


  private BButton defAppButton = new BButton(res, "defAppButton");
  private BButton vrmlButton = new BButton(res, "vrmlButton");
  private BButton guessButton = new BButton(res, "guessButton");
  private BButton updateCButton = new BButton(res, "updateCButton");
  private BButton newButton = new BButton(res, "newButton");
  private BButton loadButton = new BButton(res, "loadButton");
  private BButton saveButton = new BButton(res, "saveButton");
  private BButton refreshRButton = new BButton(res, "refreshRButton");
  private BButton setPriceButton = new BButton(res, "setPriceButton");

  private BComboBox matCombo = new BComboBox(res, "matCombo");
  private BLabel matLabel = new BLabel(res, "matLabel");
  private BLabel confirmLabel = new BLabel(res, "confirmLabel");

  private BLabel volLabel = new BLabel(res, "volLabel");
  private BTextField volField = new BTextField(res, "volField");

  private JScrollPane infoScroll;

  // Other
  private BComponent component;
  private BComponent stdComponent;
  private String filePath;
  private BComponentEditorInfo info;

  private String lastFileName;

  // 3D Stuff
  //private JPanel v3dPane;
  private BComponentCanvas bot3dCanvas;
  //private Border paneBorder;

  // File Stuff
  private BFileFilter filter;
  private File startFilePath;
  private JFileChooser chooser;
  private String lastFilePath;

  private String HOVERCHASSIS = "Hover Chassis";


  // Main method
  static public void main(String[] args)
  {
    new BCompEditor();
  }

  // Construct the frame.
  public BCompEditor()
  {
    BTextures.setListener(this);
    BGlobal.init();

    enableEvents(AWTEvent.WINDOW_EVENT_MASK);

    this.setSize(new Dimension(800, 600));
    this.setResizable(false);
    this.setTitle("Component Editor");

    bot3dCanvas = new BComponentCanvas(null);
    bot3dCanvas.setSize(200,200);

    info = new BComponentEditorInfo(true);

    infoScroll = new JScrollPane(info);
    infoScroll.setSize(565, 520);

    matCombo.setModel(new DefaultComboBoxModel(BComponentMaterial.getAllMaterials()));

    layoutCompEditorChildren();
    configCompEditorChildren();

    this.validate();
    this.setVisible(true);
  }

  // layout all the components
  protected void layoutCompEditorChildren()
  {
    BBoxPanel mainBox = new BBoxPanel(BoxLayout.X_AXIS);
    BBoxPanel leftBox = new BBoxPanel(res, "leftBox", BoxLayout.Y_AXIS);
    BBoxPanel rightBox = new BBoxPanel(res, "rightBox", BoxLayout.Y_AXIS);
    BBoxPanel topLeftBox = new BBoxPanel(res, "topLeftBox", BoxLayout.X_AXIS);
    BBoxPanel volBox = new BBoxPanel(BoxLayout.X_AXIS);
    BBoxPanel optBox = new BBoxPanel(BoxLayout.X_AXIS);
    BBoxPanel valBox = new BBoxPanel(BoxLayout.X_AXIS);

    mainBox.add(leftBox);
    mainBox.addSmallMargin();
    mainBox.add(rightBox);

    leftBox.add(topLeftBox);
    topLeftBox.add(bot3dCanvas);
    leftBox.addGlue();
    leftBox.add(confirmLabel);
    leftBox.addBigMargin();
    leftBox.add(defAppButton);
    leftBox.addSmallMargin();
    leftBox.add(vrmlButton);
    leftBox.addSmallMargin();
    leftBox.add(matLabel);
    leftBox.add(matCombo);
    leftBox.addSmallMargin();
    leftBox.add(volBox);
    leftBox.addSmallMargin();
    leftBox.add(guessButton);
    leftBox.addSmallMargin();
    leftBox.add(updateCButton);
    leftBox.addMargin();
    leftBox.add(optBox);

    volBox.add(volLabel);
    volBox.addSmallMargin();
    volBox.add(volField);

    optBox.add(newButton);
    optBox.add(loadButton);
    optBox.add(saveButton);

    rightBox.add(infoScroll);
    rightBox.addSmallMargin();
    rightBox.add(valBox);

    valBox.add(refreshRButton);
    valBox.add(setPriceButton);

    this.getContentPane().add(mainBox);
  }

  // configure the components
  protected void configCompEditorChildren()
  {
    loadButton.addActionListener(this);
    newButton.addActionListener(this);
    saveButton.addActionListener(this);
    vrmlButton.addActionListener(this);
    guessButton.addActionListener(this);
    updateCButton.addActionListener(this);
    defAppButton.addActionListener(this);
    refreshRButton.addActionListener(this);
    updateCButton.addActionListener(this);
    setPriceButton.addActionListener(this);
  }


  // called when an action has been performed
  public void actionPerformed(ActionEvent e)
  {
    confirmLabel.setText("");
    if(e.getSource() == loadButton)
    {
      if(buttonLoad_actionPerformed(e))
        confirmLabel.setText("Load Complete");
    }
    if(e.getSource() == newButton)
      buttonNewComponent_actionPerformed(e);
    if(e.getSource() == saveButton)
    {
      if(buttonSave_actionPerformed(e))
        confirmLabel.setText("Save Complete");
    }
    if(e.getSource() == vrmlButton)
      buttonLoadVRML_actionPerformed(e);
    if(e.getSource() == defAppButton)
      buttonApplyColor_actionPerformed(e);
    if(e.getSource() == guessButton)
      buttonGuessWeight_actionPerformed(e);
    if(e.getSource() == updateCButton)
    {
      if(updateChassisMoves())
        confirmLabel.setText("Chassis Updated");
    }

    if(e.getSource() == refreshRButton)
      buttonShowRatios_actionPerformed(e);
    if(e.getSource() == setPriceButton)
      buttonSetPrice_actionPerformed(e);
  }


  // updates chassis moves - returns true if the update is a success
  protected boolean updateChassisMoves()
  {
     boolean success = true;

     String  botPath = BGlobal.BOTPATH + "/bot";

     // Create the File Chooser
     String[] suffix ;

     suffix = new String[] {BGlobal.BOTSUFFIX.substring(1,BGlobal.BOTSUFFIX.length())};

     BFileFilter filter = new BFileFilter(suffix,botPath,true);

     File startFilePath = new File(botPath);
     JFileChooser chooser = new JFileChooser(startFilePath);
     chooser.setFileFilter(filter);
     chooser.setSelectedFile(startFilePath);
     chooser.setDialogTitle("Update chassis moves");

     int option = chooser.showSaveDialog(this);

     if (option == JFileChooser.APPROVE_OPTION)
     {
       botPath = chooser.getSelectedFile().getPath();

       BBot b = BBot.loadFromFile(botPath) ;

       if (b != null)
       {
         BChassis newChas = (BChassis)((DefaultMutableTreeNode)b.getComponentTree().getRoot()).getUserObject();

         // Set the moves on the Chassis to be equal to the moves on the bot.
         newChas.setChassisMoveList(b.getMoveList());
         newChas.setChassisComponentTree(b.getComponentTree());
         newChas.upgrade((BChassis)component, false);

         component = newChas;

         BDebug.print ("Moves upgrade complete");
       }
       else {
         BDebug.print ("Moves upgrade NOT complete");
         success = false;
       }
     }
     else
       success = false;

     return success;


     /*
     BBot b = BBotPicker.getBot(BCompEditor.this);
     if (b != null)
     {
       BChassis newChas = (BChassis)((DefaultMutableTreeNode)b.getComponentTree().getRoot()).getUserObject();

       // Set the moves on the Chassis to be equal to the moves on the bot.
       newChas.setChassisMoveList(b.getMoveList());
       newChas.setChassisComponentTree(b.getComponentTree());
       newChas.upgrade((BChassis)component, false);

       component = newChas;

       BDebug.print ("moves Upgrade complete ");
     }
      */

  }

  public void setUpChooser (String suffix, String desc, String path)
  {
    // Remove the dot from the suffix.
    suffix = suffix.substring(1,suffix.length());

    // A bit of a hack to allow the editor always see Chassis (chs) types.
    filter = new BFileFilter(new String[] {suffix,"chs"},desc,true);

    if ( path != BGlobal.VRMLPATH )
      lastFilePath = path;

    startFilePath = new File(path);
    chooser = new JFileChooser(startFilePath);
    if (!startFilePath.isDirectory() )
      chooser.setSelectedFile(startFilePath);
    chooser.setFileFilter(filter);

  }

  //Overriden so we can exit on System Close
  protected void processWindowEvent(WindowEvent e)
  {
    super.processWindowEvent(e);
    if(e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      System.exit(0);
    }
  }


  protected boolean buttonSave_actionPerformed(ActionEvent e)
  {

    boolean success = true;

    String suffix;
    //String path;

    // Recalculate Ratios
    info.setRatios();

    // Determine the type of component being saved.
    if (component instanceof BChassis)
    {
      // Set up the file paths.
      suffix = BGlobal.CHASSISSUFFIX;
      //path = BGlobal.CHASSISPATH;

      if (lastFileName == null)
        lastFileName =  BGlobal.CHASSISPATH;
    }
    else
    {
      suffix = BGlobal.COMPONENTSUFFIX;

      if (lastFileName == null)
        lastFileName =  BGlobal.COMPONENTPATH;
    }


    // Create the File Chooser
    setUpChooser (suffix,"Component",lastFileName);

    // Open the Chooser
    if (component != null)
      {
      int option = chooser.showSaveDialog(this);

      if (option == JFileChooser.APPROVE_OPTION)
      {
        // Save the Component
        filePath = chooser.getSelectedFile().getPath();

        // Create the offset path.
        // filePath =  filePath.substring(path.length());

        try{
                info.updateComponent();

                //BComponentMaterial material = (BComponentMaterial)materialList.getSelectedValue();
                BComponentMaterial material = (BComponentMaterial)matCombo.getSelectedItem();
                if ( material != null )
                  component.setMaterialType(material);

                // Volume
                float newVol = Float.parseFloat(volField.getText());
                if ( newVol > 0f )
                  component.setVolume(newVol);
                else
                  component.guessVolume();

                // Update the Move Sideways FLag
                if (component instanceof BChassis)
                {
                  BMoveList ml = ((BChassis)component).getChassisMoveList();
                  if ( ml != null )
                  {
                    if ( ((BChassis)component).sidewaysMaxVelocity == 0f )
                    {
                      ml.setCanMoveSideWays( false );
                      BDebug.print("Chassis.canMoveSideways set to FALSE");
                    } else {
                      ml.setCanMoveSideWays( true );
                      BDebug.print("Chassis.canMoveSideways set to TRUE");
                    }
                  }
                }
                component.saveToFile(filePath);
            } catch (Exception ex)
            {
              BDebug.print("BCompEditorInfo.buttonSave_actionPerformed ERROR :" + ex.getMessage());
              ex.printStackTrace();
              success = false;
            }
        }
        else
          success = false;
     }
     else
       success = false;

     return success;
  }

  protected boolean buttonLoad_actionPerformed(ActionEvent e)
  {
    boolean success = true;

    // Create the File Chooser
    if ( lastFilePath == null )
    {
      String componentType = (String)JOptionPane.showInputDialog(this,"Please select the component type.","Type Picker",JOptionPane.QUESTION_MESSAGE,null,new Object[] {"Component","Chassis"},"Component");
      if (componentType == HOVERCHASSIS)
        lastFilePath = BGlobal.CHASSISPATH;
      else if ( componentType == "Chassis" )
        lastFilePath = BGlobal.CHASSISPATH;
      else
        lastFilePath = BGlobal.COMPONENTPATH;
    }
    setUpChooser (BGlobal.COMPONENTSUFFIX,"Component",lastFilePath);

    // Open the Chooser
    int option = chooser.showOpenDialog(this);

    if (option == JFileChooser.APPROVE_OPTION)
    {

      // Create the Component
      filePath = chooser.getSelectedFile().getPath();
      lastFilePath = chooser.getCurrentDirectory().toString();


      // Create the offset path.
      //filePath = filePath.substring(BGlobal.COMPONENTPATH.length());

      lastFileName = filePath;
      component = BComponent.loadFromFile(filePath);
      stdComponent = null;

      // If the component is of Type chassis then initialise the Tree
      // Need to loop through the BComponentTree of BBot calling the init method for all
      // components.
      if (component instanceof BChassis)
      {
        DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)((BChassis)component).getChassisComponentTree().getRoot();
        Enumeration enumerator = rootNode.breadthFirstEnumeration();
        BDebug.print("Initialising Chassis Components");
        while (enumerator.hasMoreElements())
        {
          DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
          ((BComponent)node.getUserObject()).init(false);
        }
        stdComponent = BComponent.loadFromFile(BGlobal.CHASSISPATH+"internal/defaults/chassis.chs");
      } else if (component instanceof BAutoRotateJoint) {
        stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/autojoint.cmp");
      } else if (component instanceof BRotateJoint) {
        stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/rotatejoint.cmp");
      } else if (component instanceof BTelescopicJoint) {
        stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/telescopicjoint.cmp");
      } else if (component instanceof BCpu) {
        stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/controller.cmp");
      } else if (component instanceof BSensor) {
        stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/sensor.cmp");
      } else if (component instanceof BEnergyGenerator) {
        stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/energy.cmp");
      } else if (component instanceof BWeapon) {
        stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/weapon.cmp");
      } else if (component instanceof BShielding) {
        stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/shielding.cmp");
      } else {
        stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/component.cmp");
      }


      bot3dCanvas.loadComponent(component);
      info.loadComponent(component,stdComponent);
      BComponentMaterial thisMaterial = component.getMaterial();
      matCombo.setSelectedItem(thisMaterial);
      if ( component.getVolume() < 0.0001 )
        component.setVolume(component.guessVolume());
      volField.setText(""+component.getVolume());

    }
    else
      success = false;

    return success;
  }

  void buttonLoadVRML_actionPerformed(ActionEvent e)
  {
    //B3dShapeManager.clear();
    // Create the File Chooser
    setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH);

    // Open the Chooser
    int option = chooser.showOpenDialog(this);

    if (option == JFileChooser.APPROVE_OPTION)
    {
      //Get the VRML file and path.
      filePath = chooser.getSelectedFile().getPath();

      // delete j3dFile.
      String j3dFileName =  chooser.getSelectedFile().getPath();
      j3dFileName = j3dFileName.substring( 0,j3dFileName.length() - 4);
      j3dFileName = j3dFileName + BGlobal.J3DSUFFIX;

      File j3dFile = new File(j3dFileName);

      if (j3dFile.exists() )
        j3dFile.delete();

      B3dShapeManager.clear();

      BDebug.print("Deleting " + j3dFileName);


      // Create the offset path.
      filePath =  filePath.substring(BGlobal.VRMLPATH.length());

      bot3dCanvas.removeComponent();

      // Change the VRML
      component.replaceStructureANDConnectPoints(filePath);

      // Reload the VRML graphic
      bot3dCanvas.loadComponent(component);

      volField.setText(""+component.guessVolume());

    }

  }

  void buttonApplyColor_actionPerformed(ActionEvent e)
  {
    new BAppearanceDialog(this, component, null);
    //component.setColor(JColorChooser.showDialog(this,"Colour Picker",component.getColor()),50);
  }


  class BComponentTypeDialog extends JDialog
  {static final long serialVersionUID = -1L;
    JComboBox  comptypes    = new JComboBox(new Object[] {"Component","RotateJoint", "AutoRotateJoint", "TelescopicJoint","Controller","Sensor","Weapon","Shielding", HOVERCHASSIS, "Chassis","Energy","Placeholder","Ornament", "MobileOrnament", "AnimatedTextureOrnament", "RotatingOrnament", "Wheel"});
    JLabel     textureLabel = new JLabel("Texture Path: ");
    JTextField texturePath  = new JTextField();
    JLabel     delayLabel   = new JLabel("Delay (ms): ");
    BIntField  delayTime    = new BIntField();

    JLabel     periodLabel  = new JLabel("Period (ms): ");
    BIntField  periodTime   = new BIntField();


    JDialog me;

    BComponentTypeDialog(JFrame frame)
    {
      super(frame,"Please Choose the component type",true);

      me = this;

      this.getContentPane().setLayout(null);
      this.setSize(400, 275);

      comptypes.setSize(300, 30);
      comptypes.setLocation(50, 25);

      textureLabel.setSize(100, 30);
      textureLabel.setLocation(50, 75);

      texturePath.setSize(200, 30);
      texturePath.setLocation(150, 75);

      delayLabel.setSize(100, 30);
      delayLabel.setLocation(50, 125);

      delayTime.setSize(200, 30);
      delayTime.setLocation(150, 125);

      periodLabel.setSize(100, 30);
      periodLabel.setLocation(50, 125);

      periodTime.setSize(200, 30);
      periodTime.setLocation(150, 125);

      setAnimatedVisible(false);
      setRotatingVisible(false);

      JButton ok = new JButton("OK");
      ok.setSize(300, 50);
      ok.setLocation(50, 175);

      ok.addActionListener( new ActionListener()
      {
          public void actionPerformed(ActionEvent e)
          {
            me.setVisible(false);
          }
      }
      );

      comptypes.addActionListener( new ActionListener()
      {
          public void actionPerformed(ActionEvent e)
          {
            boolean animated = comptypes.getSelectedItem().equals("AnimatedTextureOrnament");
            setAnimatedVisible(animated);

            boolean rotating = comptypes.getSelectedItem().equals("RotatingOrnament");
            setRotatingVisible(rotating);
          }
      }
      );

      this.getContentPane().add(comptypes);
      this.getContentPane().add(textureLabel);
      this.getContentPane().add(texturePath);
      this.getContentPane().add(delayLabel);
      this.getContentPane().add(delayTime);
      this.getContentPane().add(periodLabel);
      this.getContentPane().add(periodTime);
      this.getContentPane().add(ok);
    }

    protected void setAnimatedVisible(boolean v)
    {
      textureLabel.setVisible(v);
      texturePath.setVisible(v);
      delayLabel.setVisible(v);
      delayTime.setVisible(v);
    }

    protected void setRotatingVisible(boolean v)
    {
      periodLabel.setVisible(v);
      periodTime.setVisible(v);
    }


    public String getType()
    {
      return  (String)comptypes.getSelectedItem();
    }

    public String getTexturePath()
      { return texturePath.getText(); }

    public float getPeriod()
      { return (float) periodTime.getValue() / 1000f; }

    public long getDelay()
      { return (long) delayTime.getValue(); }
  }

  BComponentTypeDialog typechooser = new  BComponentTypeDialog(this);


  void  buttonNewComponent_actionPerformed(ActionEvent e)
  {
    B3dShapeManager.clear();
    String defaultComponentName;
    stdComponent = null;

     // Determine the type of BComponent to create.
    //String componentType = (String)JOptionPane.showInputDialog(this,"Please select the component type.","Type Picker",JOptionPane.QUESTION_MESSAGE,null,new Object[] {"Component","RotateJoint", "AutoRotateJoint", "TelescopicJoint","Controller","Sensor","Weapon","Shielding","Chassis","Energy","Placeholder","Ornament"},"Component");
    typechooser.setVisible(true);
    String componentType = typechooser.getType();

      // Create new component depending upon type.


        if (componentType == "Component")
        {
          // Create the File Chooser
          setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH + "component");

          // Open the Chooser
          int option = chooser.showOpenDialog(this);

          if (option == JFileChooser.APPROVE_OPTION)
          {
            //Get the VRML file and path.
            filePath = chooser.getSelectedFile().getPath();

            //Set the default Name and remove the suffix
            defaultComponentName = chooser.getSelectedFile().getName();
            defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

            // Create the offset path.
            filePath = filePath.substring(BGlobal.VRMLPATH.length());

            // Get default settings from Default Component
            stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/component.cmp");
            component = (BComponent)stdComponent.clone();
            component.setName(defaultComponentName);
            component.replaceStructureANDConnectPoints(filePath);
            lastFilePath = BGlobal.COMPONENTPATH + "component";

            //component = new BComponent(defaultComponentName,filePath);
            bot3dCanvas.loadComponent(component);

            info.loadComponent(component,stdComponent);
          }
        }
        else if (componentType == "Placeholder")
        {
          String placeholderType = (String)JOptionPane.showInputDialog(this,"Please select the placeholder type.","Type Picker",JOptionPane.QUESTION_MESSAGE,null,new Object[] {"BOT","ORNAMENT"},"Placeholder Represents");

          // Create the File Chooser
          setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH + "internal/placeholder/" + placeholderType);

          // Open the Chooser
          int option = chooser.showOpenDialog(this);

          if (option == JFileChooser.APPROVE_OPTION)
          {
            //Get the VRML file and path.
            filePath = chooser.getSelectedFile().getPath();

            //Set the default Name and remove the suffix
            defaultComponentName = chooser.getSelectedFile().getName();
            defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

            // Create the offset path.
            filePath = filePath.substring(BGlobal.VRMLPATH.length());
            lastFilePath = BGlobal.COMPONENTPATH + "internal/placeholder";


            if (placeholderType.equals(BPlaceHolder.TYPE_BOT))
              component = new BPlaceHolder(defaultComponentName,filePath,BPlaceHolder.TYPE_BOT);
            else
              component = new BPlaceHolder(defaultComponentName,filePath,BPlaceHolder.TYPE_ORNAMENT);

            bot3dCanvas.loadComponent(component);
            info.loadComponent(component,stdComponent);
          }

        }
        else if (componentType.endsWith("Ornament"))
        {
          // Create the File Chooser
          setUpChooser (BGlobal.VRMLSUFFIX,"VRML File", BGlobal.VRMLPATH + "internal/ornament");

          // Open the Chooser
          int option = chooser.showOpenDialog(this);

          if (option == JFileChooser.APPROVE_OPTION)
          {
            //Get the VRML file and path.
            filePath = chooser.getSelectedFile().getPath();

            //Set the default Name and remove the suffix
            defaultComponentName = chooser.getSelectedFile().getName();
            defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

            // Create the offset path.
            filePath = filePath.substring(BGlobal.VRMLPATH.length());
            lastFilePath = BGlobal.COMPONENTPATH + "internal/ornament";

            if (componentType.equals("Ornament"))
              component = new BArenaOrnament(defaultComponentName,filePath);
            else if (componentType.equals("MobileOrnament"))
              component = new BMobileArenaOrnament(defaultComponentName,filePath);
            else if (componentType.equals("AnimatedTextureOrnament"))
            {
              // Snimated texture ornament...
              BAnimatedTextureOrnament ornament;
              ornament  = new BAnimatedTextureOrnament(defaultComponentName, filePath);
              component = ornament;

              ornament.setTextureRoot(typechooser.getTexturePath());

              long delay = typechooser.getDelay();
              if (delay > 0) ornament.setDelay(delay);
            }
            else if (componentType.equals("RotatingOrnament"))
            {
              BRotatingArenaOrnament ornament;
              ornament  = new BRotatingArenaOrnament(defaultComponentName, filePath);
              component = ornament;

              ornament.setPeriod(typechooser.getPeriod());
            }


            bot3dCanvas.loadComponent(component);
            info.loadComponent(component,stdComponent);
         }
        }
        else if (componentType.equals("Wheel") )
        {

          setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH );

          int option = chooser.showOpenDialog(this);
          if (option == JFileChooser.APPROVE_OPTION)
          {
            //Get the VRML file and path.
            filePath = chooser.getSelectedFile().getPath();

            //Set the default Name and remove the suffix
            defaultComponentName = chooser.getSelectedFile().getName();
            defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

            // Create the offset path.
            filePath = filePath.substring(BGlobal.VRMLPATH.length());

            // Get default settings from Default Component
            BDebug.print ( "filePath " + filePath );
            component = new BWheel(defaultComponentName, filePath);
            lastFilePath = BGlobal.COMPONENTPATH + "internal";

            bot3dCanvas.loadComponent(component);
            info.loadComponent(component,stdComponent);
          }
        }

        else if (componentType == "Controller")
        {
          // Create the File Chooser
          setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH + "controller");

          // Open the Chooser
          int option = chooser.showOpenDialog(this);

          if (option == JFileChooser.APPROVE_OPTION)
          {
            //Get the VRML file and path.
            filePath = chooser.getSelectedFile().getPath();

            //Set the default Name and remove the suffix
            defaultComponentName = chooser.getSelectedFile().getName();
            defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

            // Create the offset path.
            filePath = filePath.substring(BGlobal.VRMLPATH.length());

            // Get default settings from Default Component
            stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/controller.cmp");
            component = (BCpu)stdComponent.clone();
            component.setName(defaultComponentName);
            component.replaceStructureANDConnectPoints(filePath);
            lastFilePath = BGlobal.COMPONENTPATH + "controller";

            //component = new BCpu(defaultComponentName,filePath);
            bot3dCanvas.loadComponent(component);
            info.loadComponent(component,stdComponent);
          }
        }
        else if (   (componentType == "RotateJoint")  || (componentType == "TelescopicJoint" )  || ( componentType == "UniversalJoint") || ( componentType == "AutoRotateJoint"))
        {
          // Create the File Chooser
          if ( componentType == "AutoRotateJoint")
            setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH + "joint");  // No Auto Specific VRMLS
          else
            setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH + "joint");

          // Open the Chooser
          int option = chooser.showOpenDialog(this);

          if (option == JFileChooser.APPROVE_OPTION)
          {
            //Get the VRML file and path.
            filePath = chooser.getSelectedFile().getPath();

            //Set the default Name and remove the suffix
            defaultComponentName = chooser.getSelectedFile().getName();
            defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

            // Create the offset path.
            filePath = filePath.substring(BGlobal.VRMLPATH.length());
            if ( componentType == "AutoRotateJoint")
              lastFilePath = BGlobal.COMPONENTPATH + "rotator";
            else
              lastFilePath = BGlobal.COMPONENTPATH + "joint";

            if ( componentType == "AutoRotateJoint")
            {
                //component = new BAutoRotateJoint(defaultComponentName,filePath);
                // Get default settings from Default Component
                stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/autojoint.cmp");
                component = (BAutoRotateJoint)stdComponent.clone();
                component.setName(defaultComponentName);
                component.replaceStructureANDConnectPoints(filePath);
            } else if (componentType == "RotateJoint") {
                //component = new BRotateJoint(defaultComponentName,filePath);
                // Get default settings from Default Joint
                stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/rotatejoint.cmp");
                component = (BRotateJoint)stdComponent.clone();
                component.setName(defaultComponentName);
                component.replaceStructureANDConnectPoints(filePath);
            } else if (componentType == "TelescopicJoint" ) {
                //component = new BTelescopicJoint(defaultComponentName,filePath);
                // Get default settings from Default Component
                stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/telescopicjoint.cmp");
                component = (BTelescopicJoint)stdComponent.clone();
                component.setName(defaultComponentName);
                component.replaceStructureANDConnectPoints(filePath);
            //else if (componentType == "UniversalJoint" )
            //    component = new BUniversalJoint(defaultComponentName,filePath);
            }

            bot3dCanvas.loadComponent(component);
            info.loadComponent(component,stdComponent);
          }
        }
        else if (componentType == "Sensor")
        {
          // Create the File Chooser
          setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH + "sensor");

          // Open the Chooser
          int option = chooser.showOpenDialog(this);

          if (option == JFileChooser.APPROVE_OPTION)
          {
            //Get the VRML file and path.
            filePath = chooser.getSelectedFile().getPath();

            //Set the default Name and remove the suffix
            defaultComponentName = chooser.getSelectedFile().getName();
            defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

            // Create the offset path.
            filePath = filePath.substring(BGlobal.VRMLPATH.length());

            // Get default settings from Default Component
            stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/sensor.cmp");
            component = (BSensor)stdComponent.clone();
            component.setName(defaultComponentName);
            component.replaceStructureANDConnectPoints(filePath);
            lastFilePath = BGlobal.COMPONENTPATH + "sensor";

            //component = new BSensor(defaultComponentName,filePath);
            bot3dCanvas.loadComponent(component);
            info.loadComponent(component,stdComponent);
          }
        }
        else if (componentType == "Weapon")
        {

          // Create the File Chooser
          setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH + "weapon");

          // Open the Chooser
          int option = chooser.showOpenDialog(this);

          if (option == JFileChooser.APPROVE_OPTION)
          {
            //Get the VRML file and path.
            filePath = chooser.getSelectedFile().getPath();

            //Set the default Name and remove the suffix
            defaultComponentName = chooser.getSelectedFile().getName();
            defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

            // Create the offset path.
            filePath = filePath.substring(BGlobal.VRMLPATH.length());

            // Get default settings from Default Component
            stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/weapon.cmp");
            component = (BWeapon)stdComponent.clone();
            component.setName(defaultComponentName);
            component.replaceStructureANDConnectPoints(filePath);
            lastFilePath = BGlobal.COMPONENTPATH + "weapon";

            //component = new BWeapon(defaultComponentName,filePath);
            bot3dCanvas.loadComponent(component);
            info.loadComponent(component,stdComponent);
          }
        }

        else if (componentType == "Shielding")
        {

          // Create the File Chooser
          setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH + "shielding");

          // Open the Chooser
          int option = chooser.showOpenDialog(this);

          if (option == JFileChooser.APPROVE_OPTION)
          {
            //Get the VRML file and path.
            filePath = chooser.getSelectedFile().getPath();

            //Set the default Name and remove the suffix
            defaultComponentName = chooser.getSelectedFile().getName();
            defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

            // Create the offset path.
            filePath = filePath.substring(BGlobal.VRMLPATH.length());

            // Get default settings from Default Component
            stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/shielding.cmp");
            component = (BShielding)stdComponent.clone();
            component.setName(defaultComponentName);
            component.replaceStructureANDConnectPoints(filePath);
            lastFilePath = BGlobal.COMPONENTPATH + "shielding";

            //component = new BShielding(defaultComponentName,filePath);
            bot3dCanvas.loadComponent(component);
            info.loadComponent(component,stdComponent);
          }
        }

        else if (componentType == "Energy")
        {
          // Create the File Chooser
          setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH + "energy");

          // Open the Chooser
          int option = chooser.showOpenDialog(this);

          if (option == JFileChooser.APPROVE_OPTION)
          {
            //Get the VRML file and path.
            filePath = chooser.getSelectedFile().getPath();

            //Set the default Name and remove the suffix
            defaultComponentName = chooser.getSelectedFile().getName();
            defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

            // Create the offset path.
            filePath = filePath.substring(BGlobal.VRMLPATH.length());

            // Get default settings from Default Component
            stdComponent = BComponent.loadFromFile(BGlobal.COMPONENTPATH+"internal/defaults/energy.cmp");
            component = (BEnergyGenerator)stdComponent.clone();
            component.setName(defaultComponentName);
            component.replaceStructureANDConnectPoints(filePath);
            lastFilePath = BGlobal.COMPONENTPATH + "energy";

            //component = new BEnergyGenerator(defaultComponentName,filePath);
            bot3dCanvas.loadComponent(component);
            info.loadComponent(component,stdComponent);
          }
        }

        else if ( (componentType == "Chassis") || (componentType == HOVERCHASSIS))
        {
          // Determine if the chassis is going to be created from a
          // chassis VRML file or from a Bot (stored in the ChassisBot
          // directory.

          int selection = JOptionPane.showConfirmDialog(this,"Create Chassis from Bot?","Chassis Creation",JOptionPane.YES_NO_CANCEL_OPTION);
          if (selection != JOptionPane.CANCEL_OPTION)
          {
            if (selection == JOptionPane.NO_OPTION)
            {
              // Create the Chassis from a VRML file

              // Create the File Chooser
              setUpChooser (BGlobal.VRMLSUFFIX,"VRML File",BGlobal.VRMLPATH + "chassis");

              // Open the Chooser
              int option = chooser.showOpenDialog(this);

              if (option == JFileChooser.APPROVE_OPTION)
              {
                //Get the VRML file and path.
                filePath = chooser.getSelectedFile().getPath();

                //Set the default Name and remove the suffix
                defaultComponentName = chooser.getSelectedFile().getName();
                defaultComponentName = defaultComponentName.substring(0,defaultComponentName.length()-BGlobal.VRMLSUFFIX.length());

                // Create the offset path.
                filePath = filePath.substring(BGlobal.VRMLPATH.length());

                BDebug.print("XX BCompEditor opening " + BGlobal.CHASSISPATH+"internal/defaults/chassis.chs");
                // Get default settings from Default Component

                File tester;
                String fileString;

                if (componentType == HOVERCHASSIS)
                  fileString = BGlobal.CHASSISPATH+"internal/defaults/hoverchassis.chs";
                else
                  fileString = BGlobal.CHASSISPATH+"internal/defaults/chassis.chs";


                tester = new File(fileString);

                if (tester.exists())
                  stdComponent = BComponent.loadFromFile(fileString);
                else
                {
                  if (componentType == HOVERCHASSIS)
                    stdComponent = new BHoverChassis("Default",filePath);
                  else
                    stdComponent = new BChassis("Default",filePath);
                }

                component = (BChassis)stdComponent.clone();

                ((BChassis)component).setChassisComponentTree(new BComponentTree((BChassis)component));
                component.setName(defaultComponentName);
                component.replaceStructureANDConnectPoints(filePath);
                lastFilePath = BGlobal.CHASSISPATH;

                //component = new BChassis(defaultComponentName,filePath);
                bot3dCanvas.loadComponent(component);
                info.loadComponent(component,stdComponent);
              }
            }
            else
            {
              String  botPath = BGlobal.BOTPATH + "/bot";

              // Create the File Chooser
              String[] suffix ;

              suffix = new String[] {BGlobal.BOTSUFFIX.substring(1,BGlobal.BOTSUFFIX.length())};

              BFileFilter filter = new BFileFilter(suffix,botPath,true);

              File startFilePath = new File(botPath);
              JFileChooser chooser = new JFileChooser(startFilePath);
              chooser.setFileFilter(filter);
              chooser.setSelectedFile(startFilePath);
              chooser.setDialogTitle("Update chassis moves");

              int option = chooser.showSaveDialog(this);

              if (option == JFileChooser.APPROVE_OPTION)
              {
                botPath = chooser.getSelectedFile().getPath();

                BBot b = BBot.loadFromFile(botPath) ;
                b.removeThumbnails();

                if (b != null)
                {
                  component = (BChassis)((DefaultMutableTreeNode)b.getComponentTree().getRoot()).getUserObject();

                  // Set the moves on the Chassis to be equal to the moves on the bot.
                  if ( b.getMoveList() == null )
                    b.setMoveList(new BMoveList());
                  ((BChassis)component).setChassisMoveList(b.getMoveList());

                  // DG 29/4/2000: Someone screwed this. The chassis base does not have any of its own components
                  // Which means the code throws a null exception error.

                  ((BChassis)component).setChassisComponentTree(b.getComponentTree());

                  // A little clean up
                  b = null;

                  // Need to loop through the componentTree setting all members to
                  // Chassis Locked.
                  BChassis chassis = (BChassis)component;
                  DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)chassis.getChassisComponentTree().getRoot();
                  rootNode.setUserObject(chassis);

                  Enumeration enumerator = rootNode.breadthFirstEnumeration();
                  while (enumerator.hasMoreElements())
                  {
                    DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
                    ((BComponent)node.getUserObject()).setChassisLock(true);
                  }

                  // Need to set Limb Lock for all chassis moves
                  Vector moves = chassis.getChassisMoveList().getMoves();
                  for ( int i=0; i < moves.size(); i++)
                  {
                    ;//BMove thisMove = (BMove)moves.get(i);
                  }

                  if (componentType == HOVERCHASSIS)
                    stdComponent = BComponent.loadFromFile(BGlobal.CHASSISPATH+"internal/defaults/hoverchassis.chs");
                  else
                    stdComponent = BComponent.loadFromFile(BGlobal.CHASSISPATH+"internal/defaults/chassis.chs");

                  bot3dCanvas.loadComponent(component);
                  info.loadComponent(component,stdComponent);
                }
              }
            }
          }
        }

        // Ensure correct Material is Selected
        //Vector materialVector = BComponentMaterial.getAllMaterials();
        //String materialDesc = component.getMaterialDesc();
        //BComponentMaterial thisMaterial = BComponentMaterial.getMaterial(materialDesc);
        if ( component.getMaterial() != null )
        {
          String materialCode = component.getMaterial().getCode();
          BComponentMaterial thisMaterial = BComponentMaterial.getMaterial(materialCode);
          matCombo.setSelectedItem(thisMaterial);
        } else {
          matCombo.setSelectedIndex(0);
        }
        volField.setText(""+component.guessVolume());


  }

  // ********************************  Make changes here to add a base damage to the total damage *********
  void buttonGuessWeight_actionPerformed(ActionEvent e)
  {
    try
    {
      //BComponentMaterial material = (BComponentMaterial)materialList.getSelectedValue();
      BComponentMaterial material = (BComponentMaterial)matCombo.getSelectedItem();
      float newVol = Float.parseFloat(volField.getText());
      if ( newVol == 0f )
      {
        newVol = component.guessVolume();
        volField.setText(""+newVol);
      }
      float weight = ( newVol * material.density * BPhysics.CMS_IN_M_3 * BPhysics.FILL_PERCENT)  / 1000;   /// div 100 to make Kg's
      float damage = weight * material.damageToWeightRatio + material.baseDamage;

      info.weight.setText("" + (int)weight );
      info.damageMax.setText("" + (int)damage );
      if ( component instanceof BWeapon )
        info.setWeaponWeight(weight);
      info.setRatios();

      if ( component instanceof BWheel )
      {
        info.setWheelRadius(((BWheel)component).getRadius() + "" );
      }
    } catch (Exception ex) {BDebug.print(" A Material and Component must be selected to guess weight :" + ex.getMessage()); }
  }

  void buttonShowRatios_actionPerformed(ActionEvent e)
  {
    try
    {
      info.setRatios();
    } catch (Exception ex) {BDebug.print("Error Setting ratios :" + ex.getMessage()); }
  }

  void buttonSetPrice_actionPerformed(ActionEvent e)
  {
    try
    {
      info.setPrice();
    } catch (Exception ex) {BDebug.print("Error Setting ratios :" + ex.getMessage()); }
  }
}
