package roboforge.net;

import roboforge.*;
import roboforge.net.*;
import roboforge.net.services.*;
import roboforge.net.data.*;
import roboforge.util.io.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.util.zip.*;
import roboforge.net.client.*;

public class BAutoPatcher implements BProgressMonitorable
{

  //private float numberOfPatches = 0;
  private float currentPatch = 0;
  private float progressThroughCurrentPatch = 1;
  private float entriesInCurrentPatch = 0;
  private boolean busy = false;
  private boolean restart = false;
  private boolean error = false;
  private String errorDesc = null;
  private String patchDesc = null;
  private boolean isHandshaking = true;
  private long timeStamp = 0;

  private BClientStack  server;

  private BTransmission transmission = new BTransmission();
    // SJS [12/06/2001] Transmission object.
    // Replaces the monitor service.

  private PatchThread patchThread = null;

  public BAutoPatcher(BClientStack stack)
  {
    server = stack;
  }

  public synchronized boolean isBusy()
  {
    return busy;
  }

  public synchronized boolean isRestartRequired()
  {
    return restart;
  }

  public synchronized boolean isError()
  {
    return error;
  }

  public synchronized String getErrorDesc()
  {
    return errorDesc;
  }

  /*public synchronized int getPatchCount()
  {
    return (int)numberOfPatches;
  }  */

  public synchronized int getCurrentPatch()
  {
    return (int)currentPatch;
  }

  public synchronized String getPatchDesc()
  {
    return patchDesc;
  }

  public synchronized void stop ()
  {
    //busy = false;
    transmission.stop();

    //throw new BAutoPatcherException("Patching Process Stopped by user");
    //TODO: Address other concerns of the stop functionality
    //Rollback etc
  }

  private class PatchThread extends Thread
  {

    PatchThread()
    {
      this.setPriority(Thread.MIN_PRIORITY);
      this.start();
    }

    public void run()
    {
      try
      {
        isHandshaking = true;
        // TODO: Put this SOFTURL redirection
        // request in a more appropriate place.

        transmission.setMessage(
          new BRequest(server, "SURL"));

        // Get the response back from the server...
        BMessage response = server.request(transmission);

        if (response.isAcknowledgement())
        {
          BURLData urlData;

          if (response.getData() != null)
          {
            urlData = (BURLData)response.getData();

            if (urlData.URL != null)
            {
              BGlobal.URLSGATEWAY = urlData.URL;

              // Need to set the URL to reflect the change in URL
              URL url = new URL(BGlobal.URLSGATEWAY);
              server.getServerTarget().setURL(url);

              BGlobal.save();
            }
            else
              BDebug.print("Null URL Returned");

            if (urlData.URL != null)
            {
              BGlobal.URLSCHALLENGE_ROOM = urlData.challengeRoomsURL;
              BGlobal.save();
            }
            else
              BDebug.print("Null URL Returned");

            if (urlData.URL != null)
            {
              BGlobal.URLSFARMFEEDER = urlData.farmFeederURL;
              BGlobal.save();
            }
            else
              BDebug.print("Null URL Returned");
          }
          else
          {
            // No redirection so nothing to do
            BDebug.print("Response Data Null - No Redirect Required");
          }
        }
        else
        {
          BDebug.print("Got NAK");
          errorDesc = "ERROR: "+ response;
        }
      }
      catch (Exception ne)
      {
        BDebug.print("EXCEPTION ERROR: " + ne.getMessage());
        error = true;
        busy  = false;
        errorDesc = ne.getMessage();
      }

      while (busy)
      {
        BMessage response = null;

        try
        {

          //isHandshaking = true;

          BRequest r = new BRequest(server, "AUT");
          transmission.setMessage(r);

          // Prepare the request Data...
          BPatchRequestData patchRequestData = new BPatchRequestData(0l);
          patchRequestData.versionID = BGlobal.APP_VERSION;

          r.setData(patchRequestData);

          timeStamp = System.currentTimeMillis();

          // Get response back from the server...
          response = server.request(transmission);

          isHandshaking = false;
          if (response != null)
          {
            if (response.isAcknowledgement())
            {
              BPatchData patchData;

              if (response.getData() != null)
              {
                // A patch is applied so we need to restart
                restart = true;
                patchData = (BPatchData)response.getData();
                patchDesc = patchData.patchBlurb;
                performPatch(patchData);
              }
              else
              {
                // No more patches to perform
                BDebug.print("Response Data Null");

                busy = false;
              }
            }
            else
            {
              BDebug.print("Got NAK: "+ response);
              error = true;
              errorDesc = "ERROR: "+ response;
              busy = false;
            }
          }
          else
          {
            BDebug.print("ERROR: Null Response");
            error = true;
            errorDesc = "ERROR: Null Response";
            busy = false;
          }
        }
        catch (Exception ex)
        {
          BDebug.print("EXCEPTION ERROR: " + ex.getMessage());
          error = true;
          errorDesc = ex.getMessage();
          busy = false;
        }

      }

    }
  }

  public void start() throws BAutoPatcherException
  {

    if (busy)
      throw new BAutoPatcherException("Another patch is in progress.");

    if (restart)
      throw new BAutoPatcherException("Restart required before patch can be applied.");

    busy = true;

    // Create the thread to perform the patch (if it isn't already created)
    if (patchThread == null)
      patchThread = new PatchThread();

  }


  public String getProgressMessage()
  {
    if (isHandshaking)
    {
      if (timeStamp != 0)
        return "Checking for updates...";
      else if (transmission != null)
        return transmission.getProgressMessage();
      else
        return "";
    }
    else
    {
      //return "P% :" + this.getProgressPercent();

      //return "A:" + currentPatch + " B:" + numberOfPatches + " C:"
      //+ progressThroughCurrentPatch + " D:" + entriesInCurrentPatch + " E%:" + this.getProgressPercent()
      //+ " F:" + (int)(((progressThroughCurrentPatch/entriesInCurrentPatch)/10) + currentPatch/10) * 100;

      return "Applying patch..."; //+ (int)currentPatch + " of " + (int)numberOfPatches;
    }

  }

  public int getProgressPercent()
  {

    if (isHandshaking)
    {
      if (transmission != null)
        return transmission.getProgressPercent();
      else
        return 0;
    }
    else
    {
      if (entriesInCurrentPatch != 0)
      {
        return (int)(((progressThroughCurrentPatch/entriesInCurrentPatch)*100));
        //return (int)((((progressThroughCurrentPatch/entriesInCurrentPatch)/10) + currentPatch/10) * 100);
      }
      else
      {
        return 0;
      }
    }
  }

  private void performPatch(BPatchData patchData) throws Exception
  {

      ByteArrayInputStream bAIS = new ByteArrayInputStream(patchData.patch);

      /*if (numberOfPatches == 0)
      {
        // This must be the first patch so set the number of patches
        numberOfPatches = patchData.countOfPatches;
      }   */

      currentPatch++;
      entriesInCurrentPatch = patchData.itemsInPatch;

      // Extract the files
      CheckedInputStream csumi = new CheckedInputStream(bAIS, new Adler32());
      ZipInputStream in2 = new ZipInputStream(new BufferedInputStream(csumi));
      ZipEntry ze;
      System.out.println("Checksum: " +csumi.getChecksum().getValue());


      int deleteCount = 0;
      String remainder;
      String fileAndPath;

      // Delete all the removed files
      try
      {
        while (deleteCount < patchData.filesToDelete.length)
        {
          // Replace the Start Directory with the BGlobal start directory for the individuals machine

          // Get the remainder of the string past the start directory
          remainder = patchData.filesToDelete[deleteCount].substring(patchData.deleteDirectoryStripLength + 1);

          //%% Possibly Need this code to convert all \'s to /'s in the String remainder
          remainder = remainder.replace('\\','/');

          BDebug.print("Delete Remainder: " + remainder);


          fileAndPath = BGlobal.SYSTEMPATH  + remainder;
          BDebug.print("Delete Path And File: " + fileAndPath);

          //Delete File
          File deleteFile = new File(fileAndPath);
          if (deleteFile.exists())
          {
            // Try - Catch here so while loop will
            // contiune attempting to delete files even if an error is
            // raised during delete
            try
            {
              // Delete the file
              if (deleteFile.delete())
                BDebug.print("Deleted File: " + fileAndPath);
              else
                BDebug.print("Couldn't delete File: " + fileAndPath);
            }
            catch (Exception e1)
            {
              // Throw it away...
              BDebug.print("Deleting File Error: " + e1.getMessage());
            }
          }
          else
            BDebug.print("Couldn't find File to delete: " + fileAndPath);

          deleteCount++;
        }
      }
      catch (Exception e)
      {
        // Throw it away...
        BDebug.print("Error in Deleting File While Loop: " + e.getMessage());
      }

      // Start adding new files and Directories

      progressThroughCurrentPatch = 0;

      // Keep patching the files until finished or until the user cancels the process
      while((ze = in2.getNextEntry()) != null)
      {
        int x;
        // Replace the Start Directory with the BGlobal start directory for the individuals machine

        // Get the remainder of the string past the start directory
        remainder = ze.getName().substring(patchData.directoryStripLength + 1);

        //%% Possibly Need this code to convert all \'s to /'s in the String remainder
        remainder = remainder.replace('\\','/');

        //BDebug.print("Remainder: " + remainder);

        fileAndPath = BGlobal.SYSTEMPATH  + remainder;

        BDebug.print("Path And File: " + fileAndPath);

        // Need to make the directory if it doesn't exist
        // Strip off the file to get the directory path

        int finalDirLoc = 0;
        // Loop backwards through the string until the first '/' is encountered
        for (int i = fileAndPath.length()-1;i>0;i--)
        {
          char c = fileAndPath.charAt(i);
          //BDebug.print(c);
          if (c=='/')
          {
            finalDirLoc = i;
            break;
          }
        }

        String dir = fileAndPath.substring(0,finalDirLoc);
        //BDebug.print("Directory to create: " + dir);

        //Check to see if the Directory exists
        File dirFile = new File(dir);
        if (!dirFile.exists())
        {
          // Make the directory(ies)
          dirFile.mkdirs();
        }

        // So now the directory(ies) should always exist so we can now just write the file

        FileOutputStream fos = new FileOutputStream(fileAndPath);
        BufferedOutputStream bos = new BufferedOutputStream(fos);

        // Read from the input stream, and write to the output stream
        byte[] l_buffer = new byte[512]; // buffer holding bytes to be transferred
        int l_nbytes = 0;  // Number of bytes read
        while ((l_nbytes = in2.read(l_buffer)) != -1) // Read from InputStream stream
          bos.write(l_buffer,0,l_nbytes); // Write to output stream

        bos.flush();
        bos.close();

        progressThroughCurrentPatch++;
      }
      in2.close();

      // Change the version number and save it away
      BDebug.print("Old Version No: " + BGlobal.APP_VERSION);
      BGlobal.APP_VERSION = patchData.versionID;
      BGlobal tempG = new BGlobal();
      tempG.saveToFile();
      BDebug.print("New Version No: " + BGlobal.APP_VERSION);

      //if (patchData.isJ3DUpdated)
        //BGlobal.swapInVideoMode();
      
    }

  
}


