package roboforge.net;

import roboforge.net.data.BErrorData;
import roboforge.util.data.*;

import java.io.Serializable;


public class BMessage implements Serializable
{

  // Serialization ID.
  // -----------------------------------------------------------

  static final long serialVersionUID = -1L;



  // Member variables...
  // -------------------------------------------------------------

  protected BMessageHeader header;
    // Header for this message. Contains descriptive information
    // about message data content.

  protected BData data;
    // Data to transmit in this message.
    // May be null, for simple messages.



  // Constructor...
  // -----------------------------------------------------------

  public BMessage(BData data)
    { this.data = data; }

  public BMessage(BMessageHeader header, BData data)
  {
    this.header = header;
    this.data   = data;
  }


  // Static factory methods...
  // -----------------------------------------------------------
  
  public static BMessage ACK(String type)
  {
    return new BMessage(
      new BMessageHeader(BMessageHeader.ACK, type),
      null);
  }

  public static BMessage ACK(String type, BData data)
  {
    return new BMessage(
      new BMessageHeader(BMessageHeader.ACK, type), data);
  }
  
  public static BMessage NAK(String type)
  {
    return new BMessage(
      new BMessageHeader(BMessageHeader.NAK, type),
      null);
  }

  public static BMessage error(String type, String message)
  {
    return new BMessage(
      new BMessageHeader(BMessageHeader.NAK, type),
      new BErrorData(message));
  }
  
  
  // Accessor methods...
  // -----------------------------------------------------------

  public void setHeader(BMessageHeader header)
    { this.header = header; }

  public BMessageHeader getHeader()
    { return header; }

  public void setData(BData data)
    { this.data = data; }

  public BData getData()
    { return data; }
  
  public int getCode()
    { return (header != null) ? header.getCode() : -1; }

  public String getType()
    { return (header != null) ? header.getType() : null; }
  
  public boolean isOfType(String type)
    { return (type != null) ? type.equals(getType()) : false; }

  public String toString()
    { return "["+ header+ "]"; }


  // Convenience methods...
  // -----------------------------------------------------------

  public boolean isAcknowledgement()
    { return header.code == BMessageHeader.ACK; }

}