package roboforge.net;

import roboforge.net.services.*;

import java.io.Serializable;
import java.security.Key;



public class BPresentationHeader implements Serializable
{

  // Serialization ID.
  // -----------------------------------------------------------

  static final long serialVersionUID = -1L;



  // Special data sizes...
  // -------------------------------------------------------------

  public static final int NONE = 0;
    // Indicates no data follows this header.

  public static final int UNKNOWN = -1;
    // Indicates an unknown quantity of data follows.



  // Member variables...
  // -------------------------------------------------------------

  protected long size;
    // Size of accompanying message, in bytes.
    //  - A value of  0 indicates there is no message.
    //  - A value of -1 indicates that message size is not known.

  protected BMessageHeader messageHeader;
    // The message header. For security reasons, the message header
    // is transported here, not within the message itself.

  protected BServiceSet services;
    // The set of data services that have been applied to
    // the message accompanying this header.
    // (For example - zipping, encryption, etc.)


  
  // Constructor...
  // -----------------------------------------------------------

  public BPresentationHeader()
  { 
    this.size     = NONE; 
    this.services = new BServiceSet();
  }


  
  // Accessor methods...
  // -----------------------------------------------------------

  public String toString()
  {
    StringBuffer result = 
      new StringBuffer("BPresentationHeader[");
    
    result.append(size);
    result.append(" bytes, ");
    result.append(services);
    result.append(", BMessageHeader[");
    result.append(messageHeader);
    result.append("]]");
    
    return result.toString();
  }
  
  public void setSize(long size)
    { this.size = size; }

  public long getSize()
    { return size; }

  
  public void setMessageHeader(BMessageHeader messageHeader)
    { this.messageHeader = messageHeader; }

  public BMessageHeader getMessageHeader()
    { return messageHeader; }
  
  
  public void setServices(BServiceSet services)
  { 
    if (services == null)
      throw new IllegalArgumentException("BServiceSet can't be null.");
    
    this.services = services; 
  }
  
  public BServiceSet getServices()
    { return services; }
  
  
  public void addService(BDataService service)
    // Convenience method for adding a data service.
    { services.add(service); }
  
}