package roboforge.net.sockets;

import roboforge.net.*;
import roboforge.util.io.*;

import java.io.*;
import java.security.*;
import javax.crypto.*;

import iaik.security.provider.IAIK;



public class BRSASocket extends BCipherSocket
{

  // RSA Keys...
  // -----------------------------------------------------------

  protected PublicKey  encryptionKey;
  protected PrivateKey decryptionKey;


  
  // Constructor...
  // -----------------------------------------------------------

  public BRSASocket(BSocket socket)
    { super(socket); }

  
  // Accessor methods...
  // -----------------------------------------------------------

  public void setEncryptionKey(PublicKey key)
    { this.encryptionKey = key; }

  public void setDecryptionKey(PrivateKey key)
    { this.decryptionKey = key; }

  
  
  // Abstract methods...
  // -----------------------------------------------------------

  protected Cipher getInputCipher() throws IOException
  {
    // PRE: decrypt key has been set.
    if (decryptionKey == null)
      throw new BNetException(
        "Could not perform RSA decryption; no private key found.");

    try
    {
      Cipher c =
        Cipher.getInstance("RSA");  // "RSA/ECB/PKCS1Padding"
    
      c.init(Cipher.DECRYPT_MODE, decryptionKey);
      return c;
    }
    catch (Exception e)
      { throw new BNetException("Could not initialize RSA decryption cipher."); }
  }
  
  
  protected Cipher getOutputCipher() throws IOException
  {
    // PRE: decrypt key has been set.
    if (encryptionKey == null)
      throw new BNetException(
        "Could not perform RSA encryption; no public key found.");

    try
    {
      Cipher c =
        Cipher.getInstance("RSA"); // "RSA/ECB/PKCS1Padding"
    
      c.init(Cipher.ENCRYPT_MODE, encryptionKey);
      return c;
    }
    catch (Exception e)
      { throw new BNetException("Could not initialize RSA output cipher."); }
  }
  

  
  // Interface methods...
  // -----------------------------------------------------------

  public InputStream filter(InputStream in) throws IOException
  {
    Cipher cipher = getInputCipher();
    return new BBlockInputStream(in, new BCipherTransform(cipher));
  }

  public OutputStream filter(OutputStream out) throws IOException
  {
    Cipher cipher = getOutputCipher();
    return new BBlockOutputStream(out, new BCipherTransform(cipher));
  }
  

  // Block Cipher stream classes...
  // -----------------------------------------------------------
  
  protected static class BCipherTransform implements BBlockTransform
  {
    protected Cipher cipher;
    
    // protected int blocks;
    
    public BCipherTransform(Cipher c)
      { this.cipher = c; }
    
    public byte[] transform(byte[] b, int off, int len) throws IOException
    { 
      try
      { 
        // System.out.print("transforming block: "+ blocks+ " - size before: "+ len);
        // blocks++;
        
        byte[] result = cipher.doFinal(b, off, len); 
        
        // System.out.println(" - size after: "+ result.length);
        return result;
        
      }
      catch (Exception e)
        { throw new BNetException("Could not apply RSA cipher: "+ e); }
    }
    
    public int getBlockSize()
      { return 100; }
  }
  
}
