package roboforge.net.sockets;

import roboforge.BDebug;
import roboforge.net.handlers.*;
import roboforge.BGlobal;

import java.io.*;
import java.net.*;



public class BURLSocket implements BSocket
{

  protected URLConnection connection;
    // The URL connection - provides input/output streams.

  protected InputStream  in;
    // URL InputStream.

  protected OutputStream out;
    // URL OutputStream.




  public BURLSocket(URL url, long timeout) throws IOException
  {
    if (url == null)
      throw new IllegalArgumentException("No URL specified.");

    try
    {
      // SJS [13/06/2001]
      // Replaced default handler with one that has timeouts.
      // Nabbed from URL http://www.logicamente.com/sockets.html
      url = new URL(url, "", new HttpTimeoutHandler((int) timeout));

    }
    catch (MalformedURLException e)
      { throw new IOException("Could not convert URL."); }

    // Configure the URL connection...
    connection = url.openConnection();

    // If we are using a Proxy and a password has been set
    if (BGlobal.USE_PROXY && BGlobal.PROXY_HOST.length() > 0 && BGlobal.PROXY_PORT.length() > 0 && BGlobal.PROXY_PASSWORD.length() > 0)
    {
      String auth = "Basic " + new sun.misc.BASE64Encoder().encode(BGlobal.PROXY_PASSWORD.getBytes());
      connection.setRequestProperty("Proxy-Authorization", auth);
    }

    connection.setRequestProperty("CONTENT_TYPE", "application/octet-stream");
    connection.setUseCaches(false);
    connection.setDoInput(true);
    connection.setDoOutput(true);
  }


  public InputStream getInputStream() throws IOException
  { 
    if (in == null)
      in = connection.getInputStream(); 
    
    return in;
  }

  public OutputStream getOutputStream() throws IOException
  { 
    if (out == null)
      out = connection.getOutputStream();
    
    return out;
  }
  
  public void flush() throws IOException
    { if (out != null) out.flush(); }

  public void close() throws IOException
  {
    if (in  != null) in.close(); 
    if (out != null) out.close(); 
  }
  
}
