package roboforge.particles.systems;

import roboforge.BBot;
import roboforge.BComponent;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BTextures;
import roboforge.BRenderer;

import roboforge.particles.*;
import roboforge.particles.emitters.*;
import roboforge.particles.forces.*;
import roboforge.particles.forms.*;
import roboforge.particles.looks.*;
import roboforge.particles.physics.*;

import roboforge.scalability.*;

import java.util.*;
import javax.vecmath.*;
import javax.media.j3d.*;



public class BBotSmokeSystem extends BComponentSystem
{

  protected static Texture smoke
    = BTextures.getTexture(BGlobal.TEXTUREPATH +
      "internal/particles/particle_smoke.gif");
      // "internal/particles/particle_test.gif");


  protected static float SCALE
    = BGlobal.WORLD_TO_CANVAS_SCALE;


  protected static float MIN_DAMAGE = 20f;
  protected static float MAX_DAMAGE = 100f;
  protected static float LIFESPAN   = 3f;

  protected BBot bot;
    // The bot that's smoking...


  public BBotSmokeSystem(BranchGroup parent, BScalabilityMetric metric)
  {
    super(metric, 1, 10, 20);

    emitter.setEmissionPattern(new BBotSmokeEmission());
    emitter.setSharedEmitter(new BBotSmokeEmitter(emitter));

    BSystemPhysicsForces physics = new BSystemPhysicsForces();
    physics.setForces(new BBotSmokeForces());

    BSystemForm form;

    if (BGlobal.isParticleQualityHigh())
      form = new BSystemFormCard(parent, BSystemFormCard.SUBTRACTIVE);
    else
      form = new BSystemFormCluster(parent, BSystemFormCluster.SUBTRACTIVE);


    // SJS [07/04/2001] - If immediate mode wasn't so fucking annoying,
    // we might be able to use the following form, which is more efficient
    // and allows per-particle alpha transparency...

    // BSystemForm form =
    //   new BSystemFormQuad(renderer, BSystemFormQuad.SUBTRACTIVE, 20);


    BSystemLookShared look = new BSystemLookShared();
    look.setSharedLook(new BBotSmokeParticleLook());

    addAspect(physics);
    addAspect(form);
    addAspect(look);

    // Preload particles into system...
    preloadParticles();
  }


  public void setBot(BBot bot)
  {
    this.bot = bot;

    if (bot != null)
      setComponent(bot.getChassis());
  }

  public String getName()
  	{ return "BBotSmokeSystem"; }


  protected class BBotSmokeEmission extends BEmissionPatternPeriodic
  {
    public BBotSmokeEmission()
    {
      super();
      setDuration(BEmissionPatternPeriodic.FOREVER);
    }

    public int getEmissionCount(float t, float dt)
    {
      // Check if explosion should stop...
      float r = getDamageRatio();
      setN((r > 0f) && (r < 1f) ? 1 : 0);

      float p = Math.max(0.05f, 1f - (r * 2f));
      setPeriod(p);

      return super.getEmissionCount(t, dt);
    }
  }


  protected class BBotSmokeEmitter extends BParticleEmitter3D
  {
    public BBotSmokeEmitter(BSystemEmitter3D parent)
      { super(parent); }

    public void start(BParticle p)
    {
      Transform3D t = getEmitterTransform();

      p.p.set(0f, 0f, 0f);
      t.transform(p.p);
      p.oldp.set(p.p);

      float v = SCALE;

      p.v.set(rnd(v), v + rnd(v), rnd(v));
      t.transform(p.v);

      p.setMass(1f);
      p.setLifeSpan(LIFESPAN);
    }

    protected float rnd(float range)
      { return ((float) Math.random() - 0.5f) * range; }
  }



  protected class BBotSmokeParticleLook
    extends    BParticleAspectAdapter
    implements BParticleLook
  {

    protected float maxsize;

    protected Color4f color = new Color4f();

    public void start(BParticle p)
    {
      maxsize = (1f + getDamageRatio()) * SCALE;

      p.form.setSize(0f);
      p.form.setTexture(smoke);

      float b = 0.5f + (float) Math.random() * 0.5f;
      color.set(b, b, b, 1f);
      p.form.setColor(color);
    }


    public void update(BParticle p, float t, float dt)
    {
      if (p.alpha < 0.5f)
        p.form.setSize(maxsize * (0.5f + p.alpha));
      else
        p.form.setSize(maxsize * (2f - p.alpha * 2f));
    }
  }


  protected class BBotSmokeForces extends BRandomForce
  {
    public BBotSmokeForces()
    {
      super();

      float f = SCALE / 2f;
      setBase(new Vector3f(0, f/2f,0));
      setVariance(new Vector3f(f,f,f));
    }
  }

  protected float getDamageRatio()
  {
    float d = bot.getPercentageDamageLevel();
    d = (d - MIN_DAMAGE) / (MAX_DAMAGE - MIN_DAMAGE);
    return Math.max(0f, Math.min(1f, d));
  }


  public boolean isAlive()
  {
    float d = bot.getPercentageDamageLevel();

    if ((d >= MAX_DAMAGE) || (d < MIN_DAMAGE))
      return super.isAlive();
    else
      return true;
  }

}