package roboforge.particles.systems;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BTextures;

import roboforge.particles.*;
import roboforge.particles.emitters.*;
import roboforge.particles.forces.*;
import roboforge.particles.forms.*;
import roboforge.particles.looks.*;
import roboforge.particles.physics.*;

import roboforge.math.*;
import roboforge.scalability.*;

import javax.vecmath.*;
import javax.media.j3d.*;



public class BFireworkSystem extends BParticleSystemScalable
{

  protected static Texture spark
    = BTextures.getTexture(BGlobal.TEXTUREPATH +
      "internal/particles/particle_spark.gif");

  protected static float SCALE
    // = 0.5f;
    = BGlobal.WORLD_TO_CANVAS_SCALE;

  protected Point3f origin = new Point3f();

  protected float intensity = 1f;
  protected float duration  = 1f;


  // Fireworks color scheme...
  protected Color4f base = new Color4f();
  protected Color4f vary = new Color4f();


  protected BEmissionPatternPeriodic pattern;


  public BFireworkSystem
    (BranchGroup parent, BScalabilityMetric metric)
  {
    super(metric, 1, 50, 100);

    BSystemEmitterShared emitter =
      new BSystemEmitterShared(this);

    pattern = new BEmissionPatternPeriodic(100, 1f, 0.1f);
    emitter.setEmissionPattern(pattern);
    emitter.setSharedEmitter(new BFireworkEmitter());

    BSystemPhysicsForces physics = new BSystemPhysicsForces();
    physics.setForces(new BFireworkForces());

    BSystemForm form = new BSystemFormCluster(parent);

    BSystemLookShared look = new BSystemLookShared();
    look.setSharedLook(new BFireworkLook());

    addAspect(physics);
    addAspect(form);
    addAspect(look);
    addAspect(emitter);

    // Preload particles into system...
    preloadParticles();
  }

  public String getName()
  	{ return "BFireworkSystem"; }


  public void setOrigin(Point3f o)
    { origin.set(o); }

  public void setIntensity(float intensity)
  {
    this.intensity = intensity;

    // Determine particle lifespan...
    duration = 2.0f + intensity * 2;
  }

  public void setColors(Color4f base, Color4f vary)
  {
  	this.base.set(base);
    this.vary.set(vary);
  }



  protected class BFireworkEmitter
    extends    BParticleAspectAdapter
    implements BParticleEmitter
  {
    protected float BASE_SPEED = 1.5f;

    protected Point3f spherep = new Point3f();

    public void start(BParticle p)
    {
      p.p.set(origin);
      p.oldp.set(p.p);

      // Calculate initial velocity.
      // Influenced by intensity.

      float v = (2f + intensity * BASE_SPEED) * SCALE;

      BMath3D.randomPtOnSphere(v, spherep);
      p.v.set(spherep);

      p.setMass(1f);
      p.setLifeSpan(duration);
    }

    protected float rnd(float range)
      { return ((float) Math.random() - 0.5f) * range; }
  }



  protected class BFireworkLook
    extends    BParticleAspectAdapter
    implements BParticleLook
  {
    protected float size;

    protected Color4f color
    	= new Color4f();

    public void start(BParticle p)
    {
      size = (0.25f + intensity * 0.25f) * SCALE;
      p.form.setSize(size);
      p.form.setTexture(spark);

      BMath3D.randomize(base, vary, color);
      p.form.setColor(color);
    }

    public void update(BParticle p, float t, float dt)
      { p.form.setSize(size * (1f - p.alpha)); }
  }


  protected class BFireworkForces extends BNetForce
  {
    public BFireworkForces()
    {
      super();
      add(new BGravityForce(new Vector3f(0, -2f * SCALE, 0)));
      add(new BDragForce(1.5f));
    }
  }

  // public boolean isAlive()
  //  { return (time < duration); }


}
