package roboforge.scalability;

import javax.media.j3d.View;



public class BFPSMetric
  extends    BFPSTracker
  implements BScalabilityMetric
{

  protected static final float MIN_FPS = 15;
    // Minimum framerate allowable.
    // Any lower fps will receive a metric of 0.0.

  protected static final float MAX_FPS = 75;
    // Maximum practical fps.
    // Any higher fps will receive a metric of 1.0.

  protected static final float DEFAULT_TARGET_FPS = 40;
    // Default target framerate.


  protected float target;
    // Targeted FPS rating.

  protected float metric;
    // The performance metric, in the range [0.0 .. 1.0].


  public BFPSMetric(View view)
    { this(view, DEFAULT_TARGET_FPS); }
    
  public BFPSMetric(View view, float targetFPS)
  {
    super(view);
    setTargetFPS(targetFPS);
    metric = 0f;
  }


  public void setTargetFPS(float fps)
    { this.target = Math.max(Math.min(fps, MAX_FPS), MIN_FPS); }

  public float getTargetFPS()
    { return target; }


  public void update()
  {
    super.update();

    float fps = getFPS();

    if (fps < target)
      metric = (fps - MIN_FPS) / (target - MIN_FPS) * 0.5f;
    else
      metric = 0.5f + (fps - target) / (MAX_FPS - target) * 0.5f;

    metric = Math.max(0f, Math.min(1f, metric));
  }

  public float getMetric()
    { return metric; }


}