package roboforge.scalability;

import javax.media.j3d.*;



public class BFPSTracker
{

  protected static final int BUF_SIZE = 100;


  protected View view;
    // Canvas3D view; source of frame timings.

  protected long[] frames = new long[BUF_SIZE];
    // Circular buffer of frame timestamps.


  protected int now;
    // Index of current frame.

  protected int old;
    // Index of frame at least one second old.


  protected long nowtime;
    // System time, in ms.

  protected long oldtime;
    // Time one second ago.


  protected float estimate;
    // Current estimate of framerate.



  public BFPSTracker(View view)
  {
    this.view = view;
    reset();
  }


  protected void reset()
  {
    this.now = 0;
    this.old = 0;
    estimate = 0f;
  }


  public void update()
  {
    // Must be called after every rendered frame for
    // the tracker to function.

    // Determine current time.
    nowtime = System.currentTimeMillis();
    oldtime = nowtime - 1000;

    // Store current frame timing.
    frames[now] = view.getCurrentFrameStartTime();

    // Early out if view's timing is not correct.
    if (frames[now] == 0) return;

    // Increment frame.
    now = (now + 1) % BUF_SIZE;

    // Search for frames that are <= 1 second old.
    while ((frames[old] < oldtime) && (old != now))
      old = (old + 1) % BUF_SIZE;

    // Catch any buffer overruns...
    if (old == now) { reset(); return; }

    // Compute number of frames between old and now.
    if (old > now)
      estimate = now + (BUF_SIZE - old);
    else
      estimate = now - old;
  }


  public float getFPS()
    { return estimate; }
}