package roboforge.sound;

import javax.media.j3d.*;
import javax.vecmath.*;
import roboforge.*;

public class B3DSoundManager
{
  // array management variables
  private BPointSound3D[] sounds;
  private int noOfSounds = 0;
  private static int ARRAY_CHUNK = 10;

  ViewPlatform viewPlatform;

  public B3DSoundManager( View view    )
  {
    viewPlatform = view.getViewPlatform();

    sounds = new BPointSound3D[ARRAY_CHUNK];
    noOfSounds = 0;
  }

  private void extendSoundArray()
  {
    BPointSound3D[] tempA = new BPointSound3D[sounds.length + ARRAY_CHUNK]   ;
    for (int i = noOfSounds - 1 ; i >=0 ; i--)
      tempA[i] = sounds[i];

    sounds = tempA;

  }


  public void add(BPointSound3D newS)
  {
    BDebug.print ("B3DSoundManager adding Sound " + newS + " to position " + noOfSounds );

    if (noOfSounds == sounds.length)
      extendSoundArray();

    sounds[noOfSounds] = newS;
    update(newS);
    //newS.play(); 
    noOfSounds++;


  }

  public void clear()
  {
    for (int i = noOfSounds - 1; i >=0; i--)
    {
      sounds[i].stop();
      sounds[i] = null;
    }

    noOfSounds = 0;

  }


  public void remove(BPointSound3D s)
  {
    removeFromArray(s);
    s.stop();
  }

  private void removeFromArray (BPointSound3D s)
  {
    boolean foundElement = false;

    for (int i = 0; i <noOfSounds; i++ )
    {
      if (!foundElement)
      {
        if (sounds[i] == s)
          foundElement = true;
      }
      else
        sounds[i-1] = sounds[i];
    }


    if (foundElement)
      noOfSounds--;

  }




  private transient Transform3D updateViewTran = new Transform3D();
  private transient Point3f updateViewPoint = new Point3f();
  private transient Vector3f updateViewVec = new Vector3f();

  public void update()
  {
    if (noOfSounds > 0)
    {
      viewPlatform.getLocalToVworld(updateViewTran);
      updateViewTran.get(updateViewVec);
      updateViewPoint.set(updateViewVec);

      for (int i = noOfSounds - 1; i >= 0; i-- )
       sounds[i].update(updateViewPoint);
    }
  }

  private void update(BPointSound3D s)
  {
    viewPlatform.getLocalToVworld(updateViewTran);
    updateViewTran.get(updateViewVec);
    updateViewPoint.set(updateViewVec);

    s.update(updateViewPoint);

  }





}