package roboforge.sound;

//import java.net.URL;
import javax.sound.sampled.*;
import java.io.*;
import java.util.*;

import roboforge.*;
import roboforge.util.text.BStrings;

public class BSound {

//Mixer
// Static Sound Manager Members
  protected static Mixer staticMixer = null;
  protected static Line.Info lineType;
  protected static Hashtable dataCache = new Hashtable();


  public static int LOOP_CONTINUOUSLY = Clip.LOOP_CONTINUOUSLY;
  // sound class

  public static int SOUND_TYPE_FOREGROUND = -69;
  public static int SOUND_TYPE_BACKGROUND = -9;
  public static int SOUND_TYPE_INTERFACE  = 666;

  public static float BACKGROUND_SOUND_MAX = -5;      // DB
  public static float BACKGROUND_SOUND_MIN= -20;
  public static float FOREGROUND_SOUND_MAX = 4;
  public static float FOREGROUND_SOUND_MIN = -20;
  public static float INTERFACE_SOUND_MAX = FOREGROUND_SOUND_MAX;
  public static float INTERFACE_SOUND_MIN = FOREGROUND_SOUND_MIN;

  // sound priority
  private static int activeLines = 0;
  private static final int PRIORITY_LINES = 10;
  private static int maxLines;    // this is set in init()
  private static int max_cap;     // this is set in init()
  private static int min_cap = 10;

  public static final int PRIORITY_MIN = 0;
  public static final int PRIORITY_MED = 1;
  public static final int PRIORITY_MAX = 2;


// BSound Members

  protected Clip mySound = null;
  protected String myName = null;
  protected int loopTimes = 0;
  protected int soundType;
  protected transient float gain;
  protected int priority = PRIORITY_MED;

  byte[] soundData;
  AudioFormat dataFormat;

  private boolean autoUpdateVolume;

  // DJG July 2001
  private double startTime = 0d; // Used for sound "standoff"

  private static LineListener activeDecrementer  = new   LineListener()
  {
    public void update(LineEvent event)
    {
      if (event.getType().equals(LineEvent.Type.STOP) )
      {
        activeLines--;
        //BDebug.print ("removing sound activeLines " + activeLines );

      }
    }
  } ;

  private static LineListener activeIncrementer  = new   LineListener()
  {
    public void update(LineEvent event)
    {
      if (event.getType().equals(LineEvent.Type.START) )
      {
        activeLines++;
        //BDebug.print ("playing sound: activeLines " + activeLines );

      }
    }
  } ;



  public BSound(String filename,  int type)
  // the Contructor
  // Dont use this, use getSound()
  {
      myName = filename;
      soundType = type;


      DataFormatPair pair = getData(filename);
      soundData = pair.data;
      dataFormat = pair.format;

      setVolume(getTypeVolume());

      if (soundType == SOUND_TYPE_INTERFACE)
        setAutoUpdateVolume(true);

  }

  public void setAutoUpdateVolume(boolean b)
  {
    autoUpdateVolume = b;
  }


  protected float getTypeVolume()
  {
      if (soundType == SOUND_TYPE_BACKGROUND )
        return  BGlobal.BACKGROUND_SOUND_VOLUME;
      else if (soundType == SOUND_TYPE_INTERFACE)
        return BGlobal.INTERFACE_SOUND_VOLUME;
      else
        return BGlobal.FOREGROUND_SOUND_VOLUME;
  }

  public BSound()
  {}
  
   public static BSound getSound(String fileName)
   {
    return  BSound.getSound(fileName, 0, SOUND_TYPE_FOREGROUND);
   }


   public static BSound getSound(String fileName, int loopTimesnew, int soundTypeNew)
  // The static getSound method.
  // Use this to create sounds
  {


   if ( (staticMixer == null) || !BGlobal.USE_SOUND)
   {

      //BDebug.print("BSound.getSound ERROR BSound.init has NOT BEEN CALLED!!!!!");
      return new BSound();
   }


   try
    {

      // check Cache

      
      BSound tempS = new BSound(fileName, soundTypeNew);
      tempS.loopTimes = loopTimesnew;
      return tempS;




     }
     catch (Exception e)
     {
        BDebug.print("BSound.getCachedSound ERROR "  + e.getMessage()
                                         + " in file :: " +  fileName);


    }


     return new BSound();

  }



  public static DataFormatPair getData(String fileName)
  {
    fileName = BStrings.convertToFilePath(fileName);
    DataFormatPair pair = (DataFormatPair)dataCache.get(fileName);

    if (pair != null)
      return pair;

    byte[] data = null;
    AudioFormat format = null;

    // else fill the cache
    try
      {
        File locator = new File(fileName);
        AudioInputStream in = AudioSystem.getAudioInputStream(locator);
        format = in.getFormat();
        data = new byte[format.getFrameSize()  * (int)in.getFrameLength() ];

        in.read(data);
        in.close();
      }
      catch (Exception e)
      {
        BDebug.print ("BSound.constructor ERROR " + fileName + " " + e.getMessage() );

      }

      BDebug.assert(format.getFrameSize() == 1,  "Sound Data must be 8 bit, " + fileName + " is " + (format.getFrameSize() * 8) + " bits, " );
      pair = new DataFormatPair(data, format);
      dataCache.put(fileName, pair);
      return pair;


  }

  public static void clearCache()
  // Only use this when you have no Active sounds
  {
    dataCache.clear();
  }

  public boolean equals (BSound other)
  {
    if (! BGlobal.USE_SOUND )
      return false;

    return (myName.equals(other.myName) );

  }


  private void setGain(float newGain)
  {
        gain = newGain;
        FloatControl gainControl =  (FloatControl) mySound.getControl(FloatControl.Type.MASTER_GAIN);
        gainControl.setValue(newGain);

  }


  protected float getCurrentGain()
  {
    float currentGain = 0;

    if ( mySound != null)
    {
      FloatControl gainControl =  (FloatControl) mySound.getControl(FloatControl.Type.MASTER_GAIN);
      currentGain = gainControl.getValue();

    }


    return currentGain;

  }

  public float getVolumeRatio()
  {
       // BDebug.print ("BSound.setVolumeRatio " + ratio );
        float max;
        float min;
        float ratio = 0f;

        if (soundType == SOUND_TYPE_BACKGROUND )
        {
          max =  BGlobal.BACKGROUND_SOUND_VOLUME ;
          min = BACKGROUND_SOUND_MIN;
        }
         if (soundType == SOUND_TYPE_INTERFACE )
        {
          max =  BGlobal.INTERFACE_SOUND_VOLUME ;
          min = INTERFACE_SOUND_MIN;
        }
        else
        {
          max = BGlobal.FOREGROUND_SOUND_VOLUME ;
          min = FOREGROUND_SOUND_MIN;
        }

        float volumeRange = (max-min);
        ratio = (gain - min) / volumeRange;

        return ratio;

  }



  public void setVolume(float newGain)
  {
    gain = newGain;
  }

  public void setVolumeRatio(float ratio)
  {
       // BDebug.print ("BSound.setVolumeRatio " + ratio );
        float max;
        float min;
        //float userVol;

        if (soundType == SOUND_TYPE_BACKGROUND )
        {
          max =  BGlobal.BACKGROUND_SOUND_VOLUME ;
          min = BACKGROUND_SOUND_MIN;
        }
        if (soundType == SOUND_TYPE_INTERFACE )
        {
          max =  BGlobal.INTERFACE_SOUND_VOLUME ;
          min = INTERFACE_SOUND_MIN;
        }
        else
        {
          max = BGlobal.FOREGROUND_SOUND_VOLUME ;
          min = FOREGROUND_SOUND_MIN;
        }

        float volumeRange = (max-min);
        gain =  min + (ratio * volumeRange);

        if (mySound != null)
          setGain(gain);
       // BDebug.print ("BSound.setVolumeRatio gain" + gain );

  }

  public void setPriority(int p)
    { priority = p; }

  public void play()
  {
   if (BGlobal.USE_SOUND)
   {
      if (staticMixer != null)
      {
        if ( (activeLines >= max_cap) && ( priority != PRIORITY_MAX) )
        {
          //BDebug.print ("BSound.play rejected, inadequate no of lines ");
          return;
        }
        else if ( (activeLines >= min_cap) && ( priority == PRIORITY_MIN) )
        {
         // BDebug.print ("BSound.play MIN PRIORITY REJECTED");
          return;
        }

        reset();
        if (mySound != null)
        {

          if (autoUpdateVolume)
            setGain( getTypeVolume() );

          if (loopTimes != 0)
            { mySound.loop(loopTimes); }
          else
            mySound.start();

          //if (listenerToAdd != null)
           // listenerToAdd.bufferChainedSound();

          startTime = System.currentTimeMillis();

        }

      }
    }
  }




  class SoundChainer implements LineListener
  {
    BSound chainedSound;
    public SoundChainer(BSound chainedSoundNew)
    {
      chainedSound = chainedSoundNew;
    }

    public void update(LineEvent event)
    {
      if (event.getType().equals(LineEvent.Type.STOP) )
      {
        chainedSound.play();

      }
    }

    public void bufferChainedSound()
    {
      chainedSound.reset();
    }



  }


  public void addLineListener(LineListener l)
  {
    if (listeners == null)
      listeners = new Vector();

    listeners.add(l);

    if (mySound != null)
      mySound.addLineListener(l);

  }


  private transient Vector listeners;

  public void chainSound(BSound chainedSound)
  {                                               
      SoundChainer listenerToAdd = new SoundChainer(chainedSound);

      if (mySound != null)
        mySound.addLineListener(listenerToAdd);
      else
      {
        if (listeners == null)
          listeners = new Vector();
        listeners.add(listenerToAdd);
      }

  }

  public String toString()
  { return "BSound " + myName;  }

  private void createSound()
  {

      try
      {

        //stream = AudioSystem.getAudioInputStream(tempF);


        Clip clip = (Clip) staticMixer.getLine(lineType);
        //clip.open(stream);
        clip.open(dataFormat, soundData, 0, soundData.length );
        mySound = clip;
        mySound.setFramePosition(0);

        if (listeners != null)
        {
          for (int i = 0; i < listeners.size(); i++ )
            mySound.addLineListener( ( LineListener ) listeners.get(i) );
          listeners = null;
        }
        setGain(gain);

        mySound.addLineListener(activeDecrementer);
        mySound.addLineListener(activeIncrementer);


      }
      catch(Exception e)
      {
       BDebug.print( "BSound.createSound ERROR in " + myName + " ::" + e.getMessage() );
      }
  }



  public void reset()
  // Resets the sound to be played
  {
    if (BGlobal.USE_SOUND)
    {
      if (mySound == null)
        createSound();
      else
      {
        if (loopTimes == LOOP_CONTINUOUSLY)
        {
          mySound.setFramePosition(0);
        }
        else if (isRunning())
          createSound();
        else
        {
          mySound.setFramePosition(0);
        }
      }



     }



  }

  public void stop()
  {
    //if (mySound.is
    if (BGlobal.USE_SOUND)
      if (mySound != null)
      {
        if (loopTimes > 0 )
          mySound.loop(0);

        mySound.stop();

      }

  }

  public void removeLineListener ( LineListener line)
  {
    if (listeners != null)
      listeners.remove(line);
    if ( mySound != null)
      mySound.removeLineListener(line);
  }

  public void onClose()
  {
    if (mySound != null)
    {
      if (listeners != null)
        for (int i = listeners.size() - 1; i >=0; i--)
          mySound.removeLineListener((LineListener) listeners.get(i) );

      mySound.close();

    }
    mySound = null;
    soundData = null;
    dataFormat = null;


  }


  public static void close()
  {
    if (staticMixer != null)
       staticMixer.close();

  }

  public boolean isRunning()
  {
   if (mySound == null)
    return false;
   else
    return mySound.isRunning();
  }


  public int getFrameLength()
  {
    if (mySound != null)
      return mySound.getFrameLength();
    else
      return 0;
  }



  public int getFramePosPercent()
  // get the percent of the clip, that the playing is at
  {
    if (isRunning())
        return  (int) ( ( (float)mySound.getFramePosition() / (float)mySound.getFrameLength()  ) * 100f);
    else
      return 0;

  }



 

  public static void init()
  // MUST BE CALLED BEFORE ANYTHING ELSE IS USED
  {

    if (BGlobal.USE_SOUND)
    {
      try
      {
        Mixer.Info[] mInfo = AudioSystem.getMixerInfo() ;
        staticMixer = AudioSystem.getMixer(mInfo[0]);
        //Line.Info[] lineInfos = staticMixer.getSourceLineInfo();
        lineType = new Line.Info(Clip.class);

        maxLines = staticMixer.getMaxLines(lineType) ;
        max_cap = maxLines - PRIORITY_LINES;

        BDebug.print( "BSound.init: max lines: " + maxLines  );


      }
      catch(Exception e)
      {
       BDebug.print( "BSound.init ERROR " + e.getMessage() );
       BGlobal.USE_SOUND = false;
      }

    }
   else
    BDebug.print( "BSound disabled " );
  }



  // class for caching pairs
  static class DataFormatPair
  {
    public byte[] data;
    public AudioFormat format;

    DataFormatPair(byte[] data, AudioFormat format)
    {
      this.format = format;
      this.data = data;
    }


  }

  public double getStartTime()
  {
    return startTime;
  }


}