package roboforge.ui;

import roboforge.BChassis;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BBot;

import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.modes.*;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.util.Enumeration;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.vecmath.*;
import javax.media.j3d.*;
import java.awt.event.*;

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.*;
import com.sun.image.codec.jpeg.*;

import roboforge.ui.action.*;
import roboforge.util.io.*;


import roboforge.animation.*;
import roboforge.animation.animation.b3d.*;

public class B3DEditingPanel extends BTooledPanel
  implements PropertyChangeListener, SwingConstants
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BWorkshopScreen workshop;
  protected BBot bot;
  protected B3DBotCanvas canvas;
  

  // UI Resources ...
  // -------------------------------------------------------------

  protected final static Dimension TOOL_BUTTON_SIZE = new Dimension(30, 30);

  protected BUIResources res = new BUIResources( new Object[]
  {
    "select.icon",             BIconCache.getHSB("select_icon.gif"),
    "select.tooltip",          "Select Tool [S]",
    "select.keystroke",        KeyStroke.getKeyStroke('s'),

    "rotate.icon",             BIconCache.getHSB("rotb_icon.gif"),
    "rotate.tooltip",          "Rotate View Tool [R]",
    "rotate.keystroke",        KeyStroke.getKeyStroke('r'),

    "pan.icon",                BIconCache.getHSB("pan_icon.gif"),
    "pan.tooltip",             "Pan View Tool [P]",
    "pan.keystroke",           KeyStroke.getKeyStroke('p'),

    "zoom.icon",               BIconCache.getHSB("zoom_icon.gif"),
    "zoom.tooltip",            "Zoom View Tool [Z]",
    "zoom.keystroke",          KeyStroke.getKeyStroke('z'),

    "rotc.icon",               BIconCache.getHSB("rotc_icon.gif"),
    "rotc.tooltip",            "Component Rotate Tool [C]",
    "rotc.keystroke",          KeyStroke.getKeyStroke('c'),

    "rotj.icon",               BIconCache.getHSB("rotj_icon.gif"),
    "rotj.tooltip",            "Joint Rotate Tool [J]",
    "rotj.keystroke",          KeyStroke.getKeyStroke('j'),

    "3Dborder.foreground",     colors.getDarkerShade(),

    "snapPicture.icon",        BIconCache.getHSB("snapshot_icon_small.gif"),
    "snapPicture.height",      new Integer(19),
    "snapPicture.width",       new Integer(19),

    "snapAction.icon",         BIconCache.getHSB("snapshot_icon_small.gif"),
    "snapAction.tooltip",      "Save picture of bot.",

  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BModeButton select            = new BModeButton(res, "select");
  protected BModeButton rotate            = new BModeButton(res, "rotate");
  protected BModeButton translate         = new BModeButton(res, "pan");
  protected BModeButton zoom              = new BModeButton(res, "zoom");
  protected BModeButton rotc              = new BModeButton(res, "rotc");
  protected BModeButton rotj              = new BModeButton(res, "rotj");

  protected BTitlebarLabel botName        = new BTitlebarLabel();
  protected BRect canvasBorder            = new BRect(res, "3Dborder");

  protected BActionButton pictureButton   = new BActionButton(res, "snapPicture");

  // Constructor.
  // -------------------------------------------------------------

  public B3DEditingPanel
  (
    BWorkshopScreen parent,
    String name,
    B3DBotCanvas canvas,
    BBot bot
  )
  {
    super(parent, name);

    this.workshop = parent;
    this.canvas   = canvas;
    this.bot      = bot;

    layoutEditingPanelChildren();
    configEditingPanelChildren();

    bot.addPropertyChangeListener(this);

    res.cleanse();
    res = null;
  }


  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutEditingPanelChildren()
  {
    layoutToolbar();
    layoutCanvas();
  }

  protected void layoutToolbar()
  {
    toolbar.add(select);
    toolbar.addSmallMargin();
    toolbar.add(rotate);
    toolbar.addSmallMargin();
    toolbar.add(translate);
    toolbar.addSmallMargin();
    toolbar.add(zoom);

    toolbar.addBigMargin();
    toolbar.add(rotc);
    toolbar.addSmallMargin();
    toolbar.add(rotj);
  }



  protected void layoutCanvas()
  {

    BBoxPanel titleBar = new BBoxPanel(BBoxPanel.X_AXIS);
      titleBar.addHGlued(botName);
      titleBar.addTinyMargin();
      titleBar.add(pictureButton);

    contentArea.addHGlued(titleBar);

 //   contentArea.addHGlued(botName);


   /* if (BGlobal.DEVELOPER_MODE)
    {
      botName.addMouseListener( new MouseAdapter()
      {
        public void mouseClicked(MouseEvent e)
        {
          try
          {
            BDebug.print (BGlobal.SYSTEMPATH + "reader/reader/AcroRd32.exe " + BGlobal.HELPPATH + "help.pdf");
            Runtime.getRuntime().exec(BGlobal.SYSTEMPATH + "reader/reader/AcroRd32.exe " + BGlobal.HELPPATH + "help.pdf");

          }
          catch (Exception ex)
          {
                BDebug.print(ex.getMessage() );

          }

        }

      }
      );
    }    */

    contentArea.addVStrut(9);
    contentArea.addHVGlued(canvasBorder);
  }

  protected void configEditingPanelChildren()
  {
    botName.setText(bot.getName());

    setupModeButton(select,    workshop.modeSelect);
    setupModeButton(rotate,    workshop.modeViewRotate);
    setupModeButton(translate, workshop.modeViewPan);
    setupModeButton(zoom,      workshop.modeViewZoom);
    setupModeButton(rotc,      workshop.modeRotateComponent);

    pictureButton.setAction( new BSnapshotAction(res, "snapAction") );
  }


  // Initialization helper methods...
  // -------------------------------------------------------------

  protected void setupModeButton(BModeButton b, BModeMouse mode)
    { b.setMode(mode, canvas); }


  // UI Listeners.
  // -------------------------------------------------------------

  public void propertyChange(PropertyChangeEvent e)
  {
    if (e.getPropertyName().equals("name"))
      botName.setText((String) e.getNewValue());
  }


  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();

    setDefaultMode();

    canvas.requestActivate();
    canvas.setSelectedComponent(null);

    // reset the orientation, just in case the bot has tipped during a play
    bot.setOrientation();
  }


  public void onDeactivate()
  {
    canvas.requestDeactivate();
    super.onDeactivate();
  }

  public void onClose()
  {
    bot = null;
    canvas = null;
    workshop = null;

    select              = null;
    rotate              = null;
    translate           = null;
    zoom                = null;
    rotc                = null;
    rotj                = null;

    botName             = null;
    canvasBorder        = null;

    super.onClose();
  }


  public void setVisible(boolean visible)
  {
    super.setVisible(visible);
    canvas.setVisible(visible);
  }

  protected void setDefaultMode()
    { canvas.setMode(workshop.modeSelect); }


  // Accessor methods...
  // -------------------------------------------------------------

  public B3DBotCanvas getCanvas()
    { return canvas; }

  public BBot getBot()
    { return bot; }

  // snap class----------------------------------------------------

  /*public class SnapActionListerner implements ActionListener
  {

    public void actionPerformed(ActionEvent ae)
    {
      BufferedImage thumb = canvas.getImage();

    }

  } */
  // Dave Dialog-------------------------------------------------

   protected class BSnapshotAction extends BThreadedAction
  {

    protected BFileDialog saveDlg =
      BFileDialog.createSaveDialog("save picture",
        "Save picture of screenshot as:", BBot.MAX_NAME_LENGTH );

    protected BPromptDialog replaceDlg =
      BPromptDialog.createYesNoDialog("replace File?", "");


    public BSnapshotAction(BUIResources res, String key)
    {
      super(res, key);

      saveDlg.setOKDefault();
      saveDlg.setFilter(new BBasicFilter(BGlobal.JPEGSUFFIX));
    }

    public void swingPrepAction(ActionEvent e)
    {
      try
      {
        String path;

        path = BGlobal.IMAGEPATH;

        saveDlg.setPath(path);
        BDebug.print ("B3DEditingPanel Screen saveasAction test" );
        saveDlg.setSelectedFile("test.jpg");
      }
      catch (Exception ex)
        { BApplication.notifyUser("Could not save file."); }
    }

    public void threadedAction(ActionEvent e)
    {
      BApplication.showDialog(saveDlg);

      String filename = saveDlg.getSelectedFile();
      String name     = saveDlg.getSelectedName();

      // Check for user cancel...
      if (filename == null)
        return;

      boolean nameChanged =
        !name.equalsIgnoreCase(bot.getName());

      if (filename != null)
      {
        File f = new File(filename);
        if (f.exists() && nameChanged)
        {
          replaceDlg.setMessage(
            "the file named "+ filename+ " already exists. \n\n"+
            "Do you want to replace it with the current bot?");

          BApplication.showDialog(replaceDlg);

          if (!replaceDlg.getResult())
            filename = null;
        }

        if (filename != null)
        {
          takesnapshot(filename);

        }
      }
    }



    private void takesnapshot(String savePictureName)
    {
   	  try
      {
        BufferedImage botImage = canvas.getImage();



     	  if (!BGlobal.JPEGSUFFIX.equalsIgnoreCase(savePictureName.substring(savePictureName.length() - 4 )  )  )
      	  savePictureName = savePictureName +  BGlobal.JPEGSUFFIX;

        //BDebug.print("Writing snapshot to file: " + savePictureName );

        FileOutputStream fos = new FileOutputStream(savePictureName);
        BufferedOutputStream bos = new BufferedOutputStream(fos);

        JPEGImageEncoder jie = JPEGCodec.createJPEGEncoder(bos);
        JPEGEncodeParam param = jie.getDefaultJPEGEncodeParam(botImage);
        param.setQuality(1.0f,true);
        jie.setJPEGEncodeParam(param);
        jie.encode(botImage);

        bos.flush();
        fos.close();

        setStatus("Bot picture saved.");


    }
    catch (Exception e)
    {
      BDebug.print("BArenaCanvas.takeSnapshot() ERROR: " + e.getMessage());
      setStatus("Error creating " + savePictureName);
    }
   }


  }




}
