package roboforge.ui;

import roboforge.BBot;
import roboforge.BChassis;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BProgressMonitorable;
import roboforge.BWizardTemplateBot;

import roboforge.attributes.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.components.border.*;
//import roboforge.ui.events.*;
import roboforge.ui.layers.*;
//import roboforge.ui.net.*;

import roboforge.util.graphics.*;
import roboforge.util.io.*;
//import roboforge.util.processors.*;
import roboforge.util.text.*;

import java.awt.*;
import java.awt.event.*;
//import java.awt.Font;
import java.io.*;

//import java.util.Vector;
import javax.media.j3d.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;



public class BGarageScreen extends BSystemScreen
{

  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleIcon.icon",             BIconCache.getHSB("workshop_icon.gif"),
    "titleLabel.icon",            BIconCache.getHSB("workshop_title.gif"),

    "background.icon",            BGraphics.getHSBIcon("background_garage.jpg"),
    "background.foreground",      colors.getMidShade(),
    "background.bounds",          new Rectangle(0, 1, 800, 450),
    
    "wizardBot.bounds",           new Rectangle(50, 50, 350, 70),
    "wizardBot.icon",             BGraphics.getUIImage("wizard_icon.gif"),
    "wizardBot.title",            BGraphics.getUIImage("wizard_small_title.gif"),
    "wizardBot.subtitle",         "Build a new bot using the "+ BGlobal.BOT+ " wizard...",

    "newBot.bounds",              new Rectangle(50, 145, 350, 70),
    "newBot.icon",                BGraphics.getUIImage("new_bot_icon.gif"),
    "newBot.title",               BGraphics.getUIImage("new_bot_small_title.gif"),
    "newBot.subtitle",            "Build a new "+ BGlobal.BOT+ ", starting with a base chassis...",

    "openBot.bounds",             new Rectangle(50, 240, 350, 70),
    "openBot.icon",               BGraphics.getUIImage("open_bot_icon.gif"),
    "openBot.title",              BGraphics.getUIImage("open_bot_small_title.gif"),
    "openBot.subtitle",           "Open an existing "+ BGlobal.BOT+ "...",

    "panels.bounds",              new Rectangle(420, 0, 370, 450),
    "panels.opaque",              Boolean.FALSE,
  });


  // Visual elements ...
  // -------------------------------------------------------------
  
  protected JLabel background           = new BLabel(res, "background");
  
  protected ButtonGroup    buttonGroup  = new ButtonGroup();
  protected BSectionButton wizardBot    = new BSectionButton(res, "wizardBot");
  protected BSectionButton newBot       = new BSectionButton(res, "newBot");
  protected BSectionButton openBot      = new BSectionButton(res, "openBot");
  
  protected BPanel panels               = new BPanel(res, "panels");


  // Member variables...
  // -------------------------------------------------------------

  protected BLayerStack stack         = new BSingleLayerStack(panels);
  
  protected BLayer wizardLayer        = new WizardLayer();
  protected BLayer newLayer           = new NewLayer();
  protected BLayer openLayer          = new OpenLayer();
  

  
  // Actions...
  // -------------------------------------------------------------
  
  protected Action wizardBotAction    = new OpenLayerAction(wizardLayer);
  protected Action newBotAction       = new OpenLayerAction(newLayer);
  protected Action openBotAction      = new OpenLayerAction(openLayer);
  


  // Constructor.
  // -------------------------------------------------------------

  public BGarageScreen()
  {
    super(true);
    //BDebug.seedTimer();

    VirtualUniverse.setJ3DThreadPriority(Thread.MIN_PRIORITY );

    layoutGarageScreenChildren();
    configGarageScreenChildren();

    res.cleanse();
    res = null;

    //background.setIcon( BGraphics.getHSBIcon("background_garage.jpg") );

    //BDebug.printTime("Garage Screen init() " );
  }
  

  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutGarageScreenChildren()
  {
    titleIcon.setIcon(res.getIcon("titleIcon.icon"));
    titleLabel.setIcon(res.getIcon("titleLabel.icon"));
    
    body.setLayout(null);
    body.addOnTop(background);

    body.addOnTop(wizardBot);
    body.addOnTop(newBot);
    body.addOnTop(openBot);
    
    panels.setLayout(new BorderLayout());
    body.addOnTop(panels);
    
  }

  protected void configGarageScreenChildren()
  {
    stack.push(openLayer);
    stack.push(newLayer);
    stack.push(wizardLayer);

    wizardBot.setAction(wizardBotAction);
    newBot.setAction(newBotAction);
    openBot.setAction(openBotAction);

    buttonGroup.add(wizardBot);
    buttonGroup.add(newBot);
    buttonGroup.add(openBot);
    
    wizardBot.setSelected(true);
  }
  


  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();
    stack.requestActivate();
    
  }

  public void onDeactivate()
  {
    stack.requestDeactivate();
    super.onDeactivate();
  }

  public void onClose()
  {
    stack.requestClose();
    super.onClose();

    
    background  = null;

    buttonGroup  = null;
    wizardBot    = null;
    newBot       = null;
    openBot      = null;
  
    panels       = null;

    stack         = null;
  
    wizardLayer        = null;
    newLayer           = null;
    openLayer          = null;

    wizardBotAction    = null;
    newBotAction       = null;
    openBotAction      = null;


    BDebug.print ("BGarageScreen onClose Called " );
  }
  
  
  // UI Actions...
  // -------------------------------------------------------------
  
  protected class OpenLayerAction extends BAction
  {
    protected BLayer layer;
    
    public OpenLayerAction(BLayer l)
    {
      super();
      this.layer = l;
    }
      
    public void actionPerformed(ActionEvent e)
      { stack.moveToTop(layer); }
  }

  
  
  // Option Layer class...
  // -------------------------------------------------------------
  
  protected abstract class OptionLayer extends BBoxLayer
  {
    
    // UI Resources...
    // -----------------------------------------------------------

    protected BUIResources res = new BUIResources(new Object[]
    {
      "title.foreground",           colors.getMidShade(),
      "preview.background",         colors.getVeryDarkShade(),
      "preview.height",             new Integer(102),
      "preview.border",             new BLineBorder(),
      "previewScroller.height",     new Integer(102),
      "buttons.height",             new Integer(30),
    });
    
    
    // UI Controls...
    // -----------------------------------------------------------

    protected BLabel      title           = new BLabel(res, "title");
    protected BLabel      prompt          = new BLabel();
    protected BFileList   items           = new BFileList();
    protected BScrollPane itemScroller    = new BScrollPane(items);

    protected BAttributeTableModel model  = new BAttributeTableModel();
    protected BAttributeTable attribs     = new BAttributeTable(model);

    protected BScrollPane previewScroller  = new BScrollPane(res, "previewScroller", attribs);
    
    protected BBoxPanel preview           = new BBoxPanel(res, "preview", BBoxPanel.X_AXIS);
    protected BBoxPanel buttons           = new BBoxPanel(res, "buttons", BBoxPanel.X_AXIS);
    
    protected BButton open                = new BButton();
   // protected BButton cancel              = new BButton();


    
    // UI Actions...
    // -----------------------------------------------------------
    
    protected OpenAction openAction       = new OpenAction();
   // protected CancelAction cancelAction   = new CancelAction();

    
    
    // Contructor...
    // -----------------------------------------------------------

    public OptionLayer()
    { 
      super(BBoxPanel.Y_AXIS); 
      layoutOptionChildren();
      configOptionChildren();
    }
  
    // Children initialization methods...
    // -----------------------------------------------------------

    protected void layoutOptionChildren()
    {
      setOpaque(false);
      
      setBorder(new EmptyBorder(30, 20, 20, 20));
      
      addHGlued(title);
      addSmallMargin();
      addHGlued(prompt);
      addTinyMargin();
      
      add(itemScroller);
      addStrut(20);
      add(preview);
     
      previewScroller.setBorder(null);
      preview.add(previewScroller);
      
      addBigMargin();
      add(buttons);
      
      buttons.addGlue();
      //buttons.add(cancel);
      //buttons.addSmallMargin();
      buttons.add(open);
    }

    protected void configOptionChildren()
    {
      open.setAction(openAction);
      //cancel.setAction(cancelAction);
      
      openAction.setEnabled(false);
      
      items.addListSelectionListener(
        new ItemSelectionListener());
      
      attribs.setTableHeader(null);
    }
  
    
    // Activation methods...
    // -----------------------------------------------------------
    
    public void onActivate()
    {
      super.onActivate();
      
      String itemPath = getItemPath();
      String oldPath  = items.getSourcePath();
      
      if (!itemPath.equals(oldPath))
        try
        {
          items.setFilter(getItemFilter());
          items.setSourcePath(itemPath);
        }
        catch (FileNotFoundException e)
          { } // ???
    }

    

    // Layer methods...
    // -----------------------------------------------------------

    public void addTo(JComponent holder)
    {
      holder.add(this);
      setVisible(true);
      requestFocus();
    }

    public void removeFrom(JComponent holder)
    {
      setVisible(false);
      holder.remove(this);
    }
    

    // UI Actions...
    // -----------------------------------------------------------
    
    protected class OpenAction extends BThreadedAction
      implements BProgressMonitorable
    {
      protected BBot bot;

      protected boolean initializingBot;
      protected String  progressMessage;
      protected int     progressPercent;
      

      public OpenAction()
        { super("Open"); }

      public void threadedAction(ActionEvent e)
      { 
        initializingBot = false;

        if (!BGlobal.DEVELOPER_MODE)
          showLoadingScreen();
        
        String item = items.getSelectedFile();
        bot = getBot(item); 

        if (bot == null)
          return;

        if (BGlobal.DEVELOPER_MODE)
          showLoadingScreen();
        
        update("Initializing...", 0);

        initializingBot = true;
        bot.init(BGlobal.DEVELOPER_MODE);
        initializingBot = false;

        update("Preparing bot design...", 100);
        BLoadingScreen.forceUpdate();

        try
          { Thread.sleep(100); }
        catch (InterruptedException ex)
          { }
      }
      
      protected void showLoadingScreen()
      {
        update("Loading bot design...", 0);

        BLoadingScreen.setSource(this);
        BLoadingScreen.setBackdrop(BGraphics.getHSBIcon("background_workshop.jpg"));
        BLoadingScreen.open();



      }

      public void swingCleanupAction(ActionEvent e)
      {
        if (bot != null)
        {
          BScreen next =
            getNextScreen(bot);

          BApplication.openScreen(next);
          BLoadingScreen.close();
          BLoadingScreen.setBackdrop(null);

          BApplication.closeScreen(BGarageScreen.this);
          BGarageScreen.this.onClose();
        }
      }
      
      // Progress update methods...
      // ......................................................
    
      protected void update(String msg, int percent)
      { 
        progressMessage = msg;  
        progressPercent = percent; 
      }

      public String getProgressMessage()
      { 
        return initializingBot ? 
          bot.getProgressMessage() : progressMessage; 
      }

      public int getProgressPercent()
      { 
        return initializingBot ? 
          bot.getProgressPercent() : progressPercent; 
      }
    }
    

    /*
    protected class CancelAction extends BAction
    {
      public CancelAction()
      { 
        super("Cancel"); 
        setShortDescription("Back to main screen.");
      }
      
      public void actionPerformed(ActionEvent e)
        { BApplication.closeScreen(BGarageScreen.this); }
    } */

    
    // UI Listeners...
    // -----------------------------------------------------------
    
    protected class ItemSelectionListener implements ListSelectionListener
    {
      public void valueChanged(ListSelectionEvent e)
      { 
        if (!e.getValueIsAdjusting())
        {
          String item = items.getSelectedFile();
          openAction.setEnabled(item != null);
          
          setStatus(items.getSelectedValue()+ " selected.");
          setBusyWorking(true);
          selectItem(item); 
          setBusyWorking(false);
        }
      }
    }
        
    
    // Abstract methods...
    // -----------------------------------------------------------
    
    protected abstract String getItemPath();
    protected abstract BFilenameFilter getItemFilter();
    protected abstract void selectItem(String item);
    protected abstract BBot getBot(String item);
    protected abstract BScreen getNextScreen(BBot bot);

  }
  
  

  // Bot Preview Image Layer class...
  // -------------------------------------------------------------

  protected abstract class PreviewLayer extends OptionLayer
  {
    protected BLabel previewImage = new BLabel();
    protected BButton create      = new BButton();
    protected BButton edit        = new BButton();
    
    protected boolean isCreating  = false;
    protected boolean isEditing   = false;

    
    public PreviewLayer()
    {
      super();

      Dimension d = new Dimension(100, 100);
      previewImage.setMinimumSize(d);
      previewImage.setPreferredSize(d);
      previewImage.setMaximumSize(d);
      previewImage.setOpaque(false);
      
      preview.addBefore(previewImage, previewScroller);
      preview.addAfter(getHStrut(5), previewImage);
      
      if (BGlobal.DEVELOPER_MODE)
      {
        create.setAction(new CreateAction());
        edit.setAction(new EditAction());

        buttons.addSmallMargin();
        buttons.add(create);
        buttons.addSmallMargin();
        buttons.add(edit);
      }
    }

    protected void selectItem(String item)
    { 
      isCreating = false;
      isEditing  = false;
      
      String image = items.getSelectedFileWithPathNoExt()+ ".jpg";
      previewImage.setIcon(new ImageIcon(image));
    }
    
    protected class CreateAction extends BAction
    {
      public CreateAction()
      { 
        super("Create"); 
        setShortDescription("Create new item (DEVMODE.)");
      }
      
      public void actionPerformed(ActionEvent e)
      {
        isCreating = true;
        openAction.perform();
      }
    }
    
    protected class EditAction extends BAction
    {
      public EditAction()
      { 
        super("Edit"); 
        setShortDescription("Edit item (DEVMODE.)");
      }
      
      public void actionPerformed(ActionEvent e)
      {
        isEditing = true;
        openAction.perform();
      }
    }
    
  }
  
  
  // Wizard Bot Layer class...
  // -------------------------------------------------------------

  protected class WizardLayer extends PreviewLayer
  {

    public WizardLayer()
    {
      super();
      title.setIcon(BIconCache.getHSB("wizard_small_title.gif"));
      prompt.setText("Please select a wizard bot design...");
    }
    

    // Implementation methods...
    // -----------------------------------------------------------
    
    protected String getItemPath()
      { return BGlobal.WIZARDPATH; }
    
    protected BFilenameFilter getItemFilter()
      { return new BRoboforgeFilter(BGlobal.WIZSUFFIX); }

    protected void selectItem(String item)
    { 
      super.selectItem(item);

      BBot bot = getBot(item);
      model.setAttributes(bot.getAttributes());
    }
    
    protected BBot getBot(String item)
    {
      if (isCreating)
        return getWizardBotFromUser(BGlobal.CHASSISPATH);

      if (isEditing)
        return BWizardTemplateBot.loadWizFromFile
          (items.getSelectedFileWithPathNoExt()+ ".bot", false);

      return BWizardTemplateBot.loadWizFromFile
        (BGlobal.WIZARDPATH+ item, false);
    }

    protected BScreen getNextScreen(BBot bot)
    { 
      BWizardTemplateBot wiz =
        (BWizardTemplateBot) bot;

      if (isCreating || isEditing)
        return new BWorkshopScreen(wiz);
      
      return new BWizardScreen(wiz); 
    }
  }
  
  

  // New Bot Layer class...
  // -------------------------------------------------------------
  
  protected class NewLayer extends PreviewLayer
  {
    protected BLabel  previewImage     = new BLabel();
    
    public NewLayer()
    {
      super();
      title.setIcon(BIconCache.getHSB("new_bot_small_title.gif"));
      prompt.setText("Please select the chassis for your new bot...");
    }

  
    // Implementation methods...
    // -----------------------------------------------------------
    
    protected String getItemPath()
      { return BGlobal.CHASSISPATH; }
    
    protected BFilenameFilter getItemFilter()
      { return new BRoboforgeFilter(BGlobal.CHASSISSUFFIX); }

    protected void selectItem(String item)
    { 
      super.selectItem(item);
      
      BChassis c = getChassis(item);
      model.setAttributes(c.getAttributes());
    }

    protected BBot getBot(String item)
    {
      if (isCreating)
        return getChassisBotFromUser(BGlobal.CHASSISPATH+ "Internal/");

      if (isEditing)
        return BBot.loadFromFile(
          BGlobal.CHASSISBOTPATH+ BStrings.stripExtension(item)+ ".bot", false);

      return new BBot(getChassis(item));
    }

    String lastItem;
    BChassis lastChassis;

    protected BChassis getChassis(String item)
    {
      if (item.equals(lastItem) )
      {
        return lastChassis;
      }
      else
      {
        BChassis c =  (BChassis) BChassis.loadFromFile(BGlobal.CHASSISPATH+ item);
        c.upgradeComponents();
        lastItem = item;
        lastChassis = c;
        return c;
      }
      
    }

    protected BScreen getNextScreen(BBot bot)
      { return new BWorkshopScreen(bot); }
  }
  
  
  // Open Bot Layer class...
  // -------------------------------------------------------------
  
  protected class OpenLayer extends OptionLayer
  {
    
    protected BButton delete          = new BButton();
    protected Action  deleteAction    = new DeleteAction();

    
    public OpenLayer()
    {
      super();
      title.setIcon(BIconCache.getHSB("open_bot_small_title.gif"));
      prompt.setText("Please select the bot you wish to work on...");

      //buttons.addBefore(delete, cancel);
      buttons.addAfter(delete, open);
      buttons.addBefore(getHStrut(10), delete);

      delete.setAction(deleteAction);
      deleteAction.setEnabled(false);

     // does anything here wok

    }
    

    // Implementation methods...
    // -----------------------------------------------------------
    
    protected String getItemPath()
      { return BGlobal.BOTPATH; }
    
    protected BFilenameFilter getItemFilter()
      { return new BRoboforgeFilter(BGlobal.BOTSUFFIX); }
    
    protected void selectItem(String item)
    { 
      BBot bot = getBot(item);
      model.setAttributes(bot.getAttributes());
      deleteAction.setEnabled(bot != null);
    }

    protected String lastItem;
    protected BBot lastBot;

    protected BBot getBot(String item)
    {
      if (item.equals(lastItem))
        return lastBot;
      else
      {
        BBot b = BBot.loadFromFile(BGlobal.BOTPATH+ item, false);
        lastItem = item;
        lastBot = b;
        return b;
      }

    }

    protected BScreen getNextScreen(BBot bot)
      { return new BWorkshopScreen(bot); }


    // UI Actions...
    // -----------------------------------------------------------

    protected class DeleteAction extends BThreadedAction
    {
      protected boolean    isRemoving;
      protected String     toRemove;
      protected String     botName;

      protected BPromptDialog removeDialog =
        BPromptDialog.createYesNoDialog("Delete "+ BGlobal.BOT+ "?");

      public DeleteAction()
      {
        super("Delete");
        setShortDescription("Delete a "+ BGlobal.BOT+ ".");
      }

      public void swingPrepAction(ActionEvent e)
      {
        setWorking(true);

        botName  = items.getSelectedValue().toString();
        toRemove = items.getSelectedFile();
        setStatus("About to delete "+ botName+ "...");

        removeDialog.setMessage(
          "Do you wish to delete '"+ botName+ "'?");
      }

      public void threadedAction(ActionEvent e)
      {
        BApplication.showDialog(removeDialog);
        isRemoving = removeDialog.getResult();

        if (isRemoving)
          try
          {
            File f = new File(BGlobal.BOTPATH+ toRemove);
            f.delete();

            items.setSourcePath(getItemPath());
          }
          catch (Exception ex)
          {
            isRemoving = false;
            BApplication.notifyUser("Could not delete "+ botName+ ".");
          }
      }

      public void swingCleanupAction(ActionEvent e)
      {
        setWorking(false);

        setStatus(isRemoving ?
          (botName+ " deleted.") :
          "Deletion cancelled.");
      }
    }

  }



  // Developer-mode stuff...
  // -----------------------------------------------------------

  protected BBot getWizardBotFromUser(String path)
  {
    BChassis c = getChassis(path);
    return (c != null) ? new BWizardTemplateBot(c) : null;
  }

  protected BBot getChassisBotFromUser(String path)
  {
    BChassis c = getChassis(path);
    return (c != null) ? new BBot(c) : null;
  }


  protected BChassis getChassis(String path)
  {
    try
    {
      BFileDialog openDlg = BFileDialog.
        createOpenDialog(
          "Select chassis",
          "Please select a base chassis for your new bot:",
          "Select");

      openDlg.setFilter(new BRoboforgeFilter(BGlobal.CHASSISSUFFIX));
      openDlg.setPath(path);

      BApplication.showDialog(openDlg);

      String fileName = openDlg.getSelectedFile();

      if (fileName != null)
      {
        BChassis c = (BChassis)
          BChassis.loadFromFile(fileName);

        return c;
      }
      else
        return null;
    }
    catch (Exception e)
      { return null; }
  }


    protected void finalize()
    { BDebug.print( this.getClass().toString() + ".finalize()"); }


}
