package roboforge.ui.ai;

import roboforge.BBot;
import roboforge.BGlobal;
import roboforge.BDebug;

import roboforge.ai.*;
import roboforge.ai.decisions.*;

import roboforge.ui.BToolPanel;
import roboforge.ui.events.*;
import roboforge.ui.components.*;
import roboforge.ui.dnd.*;
import roboforge.ui.*;
import roboforge.util.text.*;
import roboforge.util.graphics.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.io.*;

import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.*;

import roboforge.util.io.*;
import roboforge.util.processors.*;

public class BBlockPanel extends BToolPanel implements ActionListener
{
  BBot bot;
  BAIBlock selectedBlock;
  String selectedPath;
  BDecisionTree tree;

  protected BPromptDialog blockDialog = BPromptDialog.createNotifyDialog("Block Import", "");



  boolean myDrop = false;
  boolean blockListReject = false;
  BAIBlock clonedBlock;

  protected static final DataFlavor OBJECT =
    ObjectSelection.createObjectFlavor();

  // UI

  protected BUIResources res = new BUIResources(new Object[]
  {
    "blockList.columns",      new Integer(4),
    "blockList.v_align",      new Integer(SwingConstants.BOTTOM),
    "blockList.tooltip",      "Block library. Drag blocks to the AI tree to load. Drop tree nodes here to save.",
    "blockScroller.height",    new Integer(187),
    "blockScroller.border",    BUIResources.getLineBorder(colors.getDarkerShade()),

    "addBlockButton.icon",    BIconCache.getHSB("ai/add_block_icon_20.gif"),
    "addBlockButton.width",   new Integer(31),
    "addBlockButton.height",  new Integer(31),
    "addBlockButton.tooltip", "Add branch to block library.",

    "remBlockButton.icon",    BIconCache.getHSB("ai/remove_block_icon_20.gif"),
    "remBlockButton.width",   new Integer(31),
    "remBlockButton.height",  new Integer(31),
    "remBlockButton.tooltip", "Delete selected block from library.",

  } );


  BBlockList blockList = new BBlockList(res, "blockList");
  BScrollPane blockScroller   = new BScrollPane(res, "blockScroller", blockList);
  BBlockViewer blockViewer = new BBlockViewer(res, "blockViewer");

  BButton addBlockButton = new BButton(res, "addBlockButton");
  BButton remBlockButton = new BButton(res, "remBlockButton");
  ListSelectionListener listener;

  protected Cursor linkCursor = BGraphics.getCursor("link_cursor.gif", 0, 0);

  public BBlockPanel( BBot bot, BDecisionTree tree )
  {

    super("blocks");

    this.bot = bot;
    this.tree = tree;

    layoutBlockChildren();
    configBlockChildren();

  }

  protected void layoutBlockChildren()
  {
    BBoxPanel blockBox = new BBoxPanel(BoxLayout.X_AXIS);

    blockBox.addLinear(addBlockButton, getBigMargin(), remBlockButton, getVGlue());
    blockBox.addSmallMargin();
    blockBox.add(blockScroller);

    add(blockBox);

    addMargin();
    addHGlued( new BLine() );
    addSmallMargin();
    add(blockViewer);


    blockDialog.setScrollable(true);
  }

  protected void configBlockChildren()
  {
    listener = new BlockListListener();
    blockList.addListSelectionListener(listener);
    blockList.setCursor(BGraphics.getCursor("drag_cursor.gif", 0, 0));
    addBlockButton.addActionListener(this);
    remBlockButton.addActionListener(this);
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource() == addBlockButton)
    {
      try{
        BDecisionTreeNode node = (BDecisionTreeNode)tree.getSelectedNode();
        if(node != null && !node.isRoot())
        {
          BAIBlock saveBlock = new BAIBlock(node, bot);

          int blockNumber = 0;
          String fileName = "new_block_" + blockNumber + BGlobal.BLOCKSUFFIX;
          File newFile = new File(BGlobal.BLOCKPATH +  fileName);

          while( newFile.exists() )
          {
            blockNumber ++;
            fileName = "new_block_" + blockNumber + BGlobal.BLOCKSUFFIX;
            newFile = new File( BGlobal.BLOCKPATH +  fileName);
          }

          saveBlock.setName("new block " + blockNumber );

          saveBlock.saveToFile(BGlobal.BLOCKPATH +  fileName);
          blockList.load();

          blockList.setSelectedFile(fileName);
        }
      }
      catch(Exception ex) {}
    }

    if(e.getSource() == remBlockButton)
    {
      if(blockList.getSelectedIndex() > -1)
      {
        BProcessor p = new BProcessor()
        {
          public void process()
            { deleteBlockCheck(); }
        };
        p.start();
      }
    }
  }

  protected void deleteBlockCheck()
  {
    BPromptDialog deleteDialog =
    BPromptDialog.createYesNoDialog("Delete block?",
         "Do you wish to delete the selected block?");

    BApplication.showDialog(deleteDialog);

    boolean deleteBlock = deleteDialog.getResult();

    if(deleteBlock)
    {
      blockViewer.setBlock(null, "");
      String fileName = blockList.getSelectedFileWithPath();
      File file = new File(fileName);
      file.delete();
      blockList.load();
      selectedBlock = null;
    }
  }


  protected class BlockListListener implements ListSelectionListener
  {
    public void valueChanged(ListSelectionEvent e)
    {
      String path      = BGlobal.BLOCKPATH;
      String block    = blockList.getSelectedFile();

      if (block != null)
        loadBlock(path+ block);
    }
  }


  protected void loadBlock(String aiPath)
  {
    if (!aiPath.equals(selectedPath))
    {
      selectedBlock = BAIBlock.loadFromFile(aiPath);
      blockViewer.setBlock(selectedBlock, aiPath);
    }

  }



  // INTERNAL CLASSES
  //----------------------------------------------------------------------------

  class BBlockList extends BFileList implements DragGestureListener, DragSourceListener, DropTargetListener
  {
    DragSource dragSource = DragSource.getDefaultDragSource();


    public BBlockList(BUIResources res, String key)
    {
        super();

        dragSource.createDefaultDragGestureRecognizer
        (
          this, // component where drag originates
          DnDConstants.ACTION_COPY_OR_MOVE, // actions
          this
        );

        new DropTarget(this,DnDConstants.ACTION_COPY_OR_MOVE, this);


        layoutBlockListChildren();
        configureBlockChildren();

    }



    protected void configureBlockChildren()
    {
      String[] extensions = new String[1];
      extensions[0] = BGlobal.BLOCKSUFFIX;
      this.setFilter(new BRoboforgeFilter(extensions));

      setCellRenderer(new BlockListRenderer());

      load();


    }

    protected void layoutBlockListChildren()
      {}

    protected void load()
    {
      if (listener != null)
        this.removeListSelectionListener(listener);

      try {this.setSourcePath(BGlobal.BLOCKPATH); }
      catch(Exception e)
        { e.printStackTrace(); }

      if (listener != null)
        this.addListSelectionListener(listener);



    }

    // drag methods
    //------------------------------------------------------------------

    public void dragGestureRecognized(DragGestureEvent e)
    {
         // drag anything ...
         if (selectedBlock != null)
         {
            clonedBlock = (BAIBlock) selectedBlock.clone();
            //clonedBlock.setBot(bot);

            e.startDrag(  linkCursor, // cursor
                          new ObjectSelection( clonedBlock.getRoot() ), // transferable
                          BBlockList.this); // drag source listener
            myDrop = true;
            blockListReject = false;
         }
    }

    public void dragDropEnd(DragSourceDropEvent e)
    {
     // if (!myDrop)
        if ( e.getDropSuccess())
          dropSuccess();

        myDrop = false;
    }

    public void dragEnter(DragSourceDragEvent e) {
      DragSourceContext context =
      e.getDragSourceContext();

      context.setCursor(null);
      context.setCursor(linkCursor);
    }

    public void dragExit(DragSourceEvent e) {}

    public void dragOver(DragSourceDragEvent e) {
      DragSourceContext context =
      e.getDragSourceContext();

      context.setCursor(null);
      context.setCursor(linkCursor);
    }

    public void dropActionChanged(DragSourceDragEvent e) {}


    // drop events
    //-------------------------------------------------------

    public void dragEnter(DropTargetDragEvent dtde){};
    public void dragExit(DropTargetEvent dte){};
    public void dragOver(DropTargetDragEvent dtde){};
    public void dropActionChanged(DropTargetDragEvent dtde){};
    public void drop(DropTargetDropEvent dtde)
    {
      //BDebug.print ( "BBlockList.source " + dtde.getSource());
      BDebug.print("BBlockPanel.blockList.drop " + myDrop);

      if (myDrop)
      {
        blockListReject = true;
        //BDebug.print ("Drag rejected.");
      }
      else
      {
        try
        {
          Transferable t = dtde.getTransferable();

          BDecisionTreeNode node   = (BDecisionTreeNode)t.getTransferData(OBJECT);
          BAIBlock saveBlock = new BAIBlock(node, bot);

          int blockNumber = 0;
          String fileName = "new_block_" + blockNumber + BGlobal.BLOCKSUFFIX;
          File newFile = new File(  BGlobal.BLOCKPATH +  fileName);
         // BDebug.print("BBlockPanel checking new file " + fileName);

          while(  newFile.exists() )
          {
            blockNumber ++;
            fileName = "new_block_" + blockNumber + BGlobal.BLOCKSUFFIX;
          //  BDebug.print("BBlockPanel checking new file " + fileName);
            newFile = new File( BGlobal.BLOCKPATH +  fileName);
          }

          saveBlock.setName("new block " + blockNumber );
          //String fileName = "new_block_" + (BBlockList.this.getModel().getSize() + 1) + BGlobal.BLOCKSUFFIX;


          saveBlock.saveToFile(BGlobal.BLOCKPATH +  fileName);
          load();


          setSelectedFile(fileName);
        }catch(Exception e){ e.printStackTrace(); }


      }

      dtde.rejectDrop();
      dtde.dropComplete(true);

    }



    // inner class
    class BlockListRenderer extends JLabel implements ListCellRenderer
    {
      Icon brickIcon = BIconCache.getHSB("ai/block_18.gif");

      public BlockListRenderer()
      {
        setOpaque(true);
      }
      
      public Component getListCellRendererComponent(
       JList list,
       Object value,            // value to display
       int index,               // cell index
       boolean isSelected,      // is the cell selected
       boolean cellHasFocus)    // the list and the cell have the focus
      {
          String s = value.toString();
          setText(s);
          setIcon(brickIcon);
   	      if (isSelected)
          {
            setBackground(list.getSelectionBackground());
	          setForeground(list.getSelectionForeground());
	        }
          else
          {
	          setBackground(list.getBackground());
	          setForeground(list.getForeground());
	        }

	        setEnabled(list.isEnabled());
	        setFont(list.getFont());
         return this;
     }
    }






  }

  public void dropSuccess()
  {

      //BDebug.print("BBlockPanel cloneBlock dopped into bot= " + clonedBlock.isAttachedToBot()  );
       if ( clonedBlock.isAttachedToBot()  )
      {

        String msg = clonedBlock.getNewBotMessage(bot);
        clonedBlock.setBot(bot);

        blockDialog.setMessage(msg);
        BApplication.showDialog(blockDialog);
      }

      else
      {
        blockViewer.setBlock(null, "");
        String fileName = blockList.getSelectedFileWithPath();
        File file = new File(fileName);
        file.delete();
        blockList.load();
        selectedBlock = null;
        BDebug.print("BBlockPanel.dropSuccess deleted" + fileName);

      }

      myDrop = false;


  }

  //------------------------------------------------------------------------------


  class BBlockDragButton extends BButton // implements DragGestureListener, DragSourceListener
  {
   // DragSource dragSource = DragSource.getDefaultDragSource();
    
    BBlockDragButton(BUIResources res, String key)
    {
      super(res, key);

      /*dragSource.createDefaultDragGestureRecognizer
      (
          this, // component where drag originates
          DnDConstants.ACTION_COPY_OR_MOVE, // actions
          this
      );

    this.setCursor(BGraphics.getCursor("drag_cursor.gif", 0, 0));    */
    }

    protected void processMouseEvent(MouseEvent e)
      {}



    /*
    public void dragGestureRecognized(DragGestureEvent e)
    {
         // drag anything ...
         if (selectedBlock != null)
         {
            BAIBlock clonedBlock = (BAIBlock) selectedBlock.clone();
            ///clonedBlock.setBot(bot);

            e.startDrag(  linkCursor, // cursor
                          new ObjectSelection( clonedBlock.getRoot() ), // transferable
                          this); // drag source listener
            myDrop = true;
            blockListReject = false;
         }
    }

    public void dragDropEnd(DragSourceDropEvent e)
      {
      //  if (!myDrop)
          dropSuccess();

          myDrop = false;
      }

    public void dragEnter(DragSourceDragEvent e) {
      DragSourceContext context =
      e.getDragSourceContext();

      context.setCursor(null);
      context.setCursor(linkCursor);
    }

    public void dragExit(DragSourceEvent e) {}

    public void dragOver(DragSourceDragEvent e) {
      DragSourceContext context =
      e.getDragSourceContext();

      context.setCursor(null);
      context.setCursor(linkCursor);
    }
    public void dropActionChanged(DragSourceDragEvent e) {}   */

  }

  class BBlockViewer extends BBoxPanel
  {

    protected BUIResources res = new BUIResources(new Object[]
    {

      "iconButton.width",       new Integer(45),
      "iconButton.height",      new Integer(45),
      "iconButton.icon",        BIconCache.getHSB("ai/block_30.gif")

    });


   // BBlockDragButton iconButton = new BBlockDragButton(res, "iconButton");
    BTextField nameField = new BTextField(res, "nameField");
    protected BTextArea description = new BTextArea(this.res, "text");
    protected BScrollPane descriptionScoller = new BScrollPane(res, "descriptionScoller", description);


    BAIBlock selectedBlock;
    String oldName;


    BBlockViewer(BUIResources res, String key)
    {
      super(res, key, BBoxPanel.X_AXIS);
      layoutViewerChildren();
      configureViewerChildren();

    }

    protected void layoutViewerChildren()
    {
         // addLinear(iconButton, getVGlue() );
        //  addTinyMargin();

          BBoxPanel info = new BBoxPanel(Y_AXIS);
            info.addLinear(new BLabel("name:"), getHGlue() );
            info.add(nameField);
            info.addTinyMargin();
            info.addLinear(new BLabel("description:"), getHGlue() );
            info.addVGlued(descriptionScoller);

          add(info);
    }

    protected void configureViewerChildren()
    {
      nameField.addFocusListener( new FocusListener()
      {
            public void focusGained(FocusEvent e)
              {}

            public void focusLost(FocusEvent e)
            {
              if (selectedBlock != null)
              {
                updateSelectedBlock();
                oldName =  BGlobal.BLOCKPATH +   BStrings.convertToFilename(nameField.getText()) + ".aib";
              }
            }
      });


    }

    protected void setBlock(BAIBlock selectedBlock, String fileName)
    {
      //BDebug.print ("BBlockPanel.setBlock called this.selectedBlock" + this.selectedBlock);

      if (selectedBlock == null)
      {
        this.selectedBlock = null;
        description.setText("");
        nameField.setText("");
        oldName = "";

        return;
      }

      if (this.selectedBlock != null)
      //  if (!fileName.equals(oldName) )
          updateSelectedBlock();

      this.selectedBlock = selectedBlock;
    //  iconButton.setIcon(selectedBlock.getIcon() );
      description.setText(selectedBlock.getDescription() );
      nameField.setText(selectedBlock.getName());
      oldName = fileName;

    }

    protected void updateSelectedBlock()
    {
      String newFileName = BGlobal.BLOCKPATH +   BStrings.convertToFilename(nameField.getText()) + ".aib";
      boolean fileChanged = false;
     // BDebug.print ("blockViewer name is " + newFileName  );


      if (oldName.equals(newFileName) )
      {
      }
      else
      {
        File old = new File(oldName);
        old.delete();

        File file = new File(newFileName);
        if (file.exists() )
        {
          nameField.setText(nameField.getText() + " NEW");
          newFileName = BGlobal.BLOCKPATH +   BStrings.convertToFilename(nameField.getText()) + ".aib";

        }

        fileChanged =true;


      }

      selectedBlock.setName(nameField.getText() );
      selectedBlock.setDescription(description.getText() );

      try
      {
        selectedBlock.saveToFile(newFileName);
      }
      catch (Exception e)
      {
        //setStatus("ERROR Saving Block :: " + e.getMessage());
      }

      if (fileChanged)
      {
        if (listener != null)
          blockList.removeListSelectionListener(listener);

        blockList.updateFile(oldName.substring(BGlobal.BLOCKPATH.length()), newFileName.substring(BGlobal.BLOCKPATH.length()));

        if (listener != null)
          blockList.addListSelectionListener(listener);


      }
    }

    protected void onDeactivate()
    {
      if (this.selectedBlock != null)
        updateSelectedBlock();
    }


  }





}