package roboforge.ui.ai;

import roboforge.BGlobal;
import roboforge.ai.*;
import roboforge.ai.expressions.*;
import roboforge.ai.expressions.parser.*;
import roboforge.ai.memory.*;
import roboforge.ai.values.*;
import roboforge.ai.worldstate.*;

import roboforge.ui.BNamedIconic;
import roboforge.ui.components.*;

import roboforge.util.graphics.*;
import roboforge.util.parser.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;



public class BExpressionBuilder extends BBoxPanel
{

  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources(new Object[]
  {
    "worldPanel.tooltip",     "Double-click on items to paste them below.",

    "typeLabel.size",   new Dimension(20, 20),

    "pad.height",       new Integer(80),
    "pad.tooltip",      "Construct your expression here.",

    "eq.op",            "=",
    "eq.tooltip",          "Equal to",
    "eq.icon",          BIconCache.getHSB("op_icon_eq.gif", BGlobal.colors.getDarkShade()),

    "neq.op",           "!=",
    "neq.tooltip",         "Not equal to",
    "neq.icon",         BIconCache.getHSB("op_icon_neq.gif", BGlobal.colors.getDarkShade()),

    "plus.op",          "+",
    "plus.tooltip",        "Plus",
    "plus.icon",        BIconCache.getHSB("op_icon_plus.gif", BGlobal.colors.getDarkShade()),

    "minus.op",         "-",
    "minus.tooltip",       "Minus",
    "minus.icon",       BIconCache.getHSB("op_icon_minus.gif", BGlobal.colors.getDarkShade()),

    "times.op",         "*",
    "times.tooltip",       "Times",
    "times.icon",       BIconCache.getHSB("op_icon_times.gif", BGlobal.colors.getDarkShade()),

    "divide.op",        "/",
    "divide.tooltip",      "Divide",
    "divide.icon",      BIconCache.getHSB("op_icon_divide.gif", BGlobal.colors.getDarkShade()),

    "and.op",           "and",
    "and.tooltip",         "And",
    "and.icon",         BIconCache.getHSB("op_icon_and.gif", BGlobal.colors.getDarkShade()),

    "or.op",            "or",
    "or.tooltip",          "Or",
    "or.icon",          BIconCache.getHSB("op_icon_or.gif", BGlobal.colors.getDarkShade()),

    "not.op",           "not",
    "not.tooltip",         "Not",
    "not.icon",         BIconCache.getHSB("op_icon_not.gif", BGlobal.colors.getDarkShade()),

    "gtr.op",           ">",
    "gtr.desc",         "Greater than",
    "gtr.icon",         BIconCache.getHSB("op_icon_gtr.gif", BGlobal.colors.getDarkShade()),

    "less.op",          "<",
    "less.tooltip",        "Less than",
    "less.icon",         BIconCache.getHSB("op_icon_less.gif", BGlobal.colors.getDarkShade()),

    "geq.op",           ">=",
    "geq.tooltip",         "Greater than or equal to",
    "geq.icon",         BIconCache.getHSB("op_icon_geq.gif", BGlobal.colors.getDarkShade()),

    "leq.op",           "<=",
    "leq.tooltip",         "Less than or equal to",
    "leq.icon",         BIconCache.getHSB("op_icon_leq.gif", BGlobal.colors.getDarkShade()),

    "pad.tooltip",     "(Expression pad) Write expression here."

  });

  // Visual Elements ...
  // -------------------------------------------------------------

  protected WorldPanel    worldPanel      = new WorldPanel();
  protected OperatorPanel operatorPanel   = new OperatorPanel();

  protected BLabel typeLabel          = new BLabel(res, "typeLabel");

  protected BExpressionPad pad        = new BExpressionPad();

  protected BTextArea testPad = new BTextArea();
  protected BScrollPane padScroll     = new BScrollPane(res, "pad", pad);

  protected Icon OK     = res.getIcon("checkOK.icon");
  protected Icon WRONG  = res.getIcon("checkWRONG.icon");


  // Constructors...
  // -------------------------------------------------------------

  public BExpressionBuilder()
    { this(null, null); }

  public BExpressionBuilder(BUIResources res, String key)
  {
    super(res, key, BoxLayout.Y_AXIS);

    layoutBuilderChildren();
    configBuilderChildren();
  }

  protected void layoutBuilderChildren()
  {
    //tabs.add(worldTab, "world");
    //tabs.add(memoryTab, "memory");
    //tabs.add(operatorTab, "operators");

    add(worldPanel);
    addTinyMargin();

    add(operatorPanel);
    this.addMargin();
    //this.addGlued (new BLine() );
    //addSmallMargin();
    addLinear(new BLabel("Expression:"), getHGlue(), typeLabel);

    addTinyMargin();
    add(padScroll);
  }

  protected void configBuilderChildren()
  {
    pad.getDocument().addDocumentListener(new CheckTypeListener());

  }



  // Accessor methods...
  // -------------------------------------------------------------

  public void setAI(BDecisionTreeAI ai)
  {
    //memoryTab.setAI(ai);
    worldPanel.setAI(ai);
    //operatorTab.setAI(ai);
    pad.setAI(ai);
  }

  public void setExpression(BExpression e)
  {
    pad.setExpression(e);
    checkExpressionType(e);
  }

  public BExpression getExpression()
  {
    BExpression e = pad.getExpression();

    if (e != null)
      return e;
    else
    {
      String text = pad.getText().trim();

      if (!text.equals(""))
        return new BConstant(new BString(text));
      else
        return null;
    }
  }


  protected void checkExpressionType(BExpression e)
  {
    Icon typeIcon;

    if (e != null)
    {
      int type  = e.getType();
      typeIcon  = BTypeIcons.getIconForType(type);
    }
    else
      typeIcon  = BTypeIcons.getIconForType(BType.UNDEFINED);

    typeLabel.setIcon(typeIcon);
  }


  // UI Listeners...
  // -------------------------------------------------------------

  protected class CheckTypeListener implements DocumentListener
  {
    public void changedUpdate(DocumentEvent e)
      { check(); }

    public void insertUpdate(DocumentEvent e)
      { check(); }

    public void removeUpdate(DocumentEvent e)
      { check(); }

    protected void check()
    {
      BExpression e = pad.getExpression();
      checkExpressionType(e);
    }
  }


  // Memory Tab Class
  // -------------------------------------------------------------

 /* protected class MemoryTab extends BBoxPanel
  {

    protected BMemoryBank bank        = new BMemoryBank();

    // Visual elements...
    // -----------------------------------------------------------

    protected BList  items           = new BList();
    protected BScrollPane scroller   = new BScrollPane(items);


    // Constructor...
    // -----------------------------------------------------------

    public MemoryTab()
    {
      super(BoxLayout.Y_AXIS);

      scroller.setBorder(null);
      scroller.setViewportBorder(new EmptyBorder(5, 5, 5, 5));
      addHVGlued(scroller);

      // items.setBorder(empty);
      items.setCellRenderer(
        new BNamedIconicListCellRenderer());

      items.addMouseListener(new ItemClickListener());
    }

    public void setAI(BDecisionTreeAI ai)
    {
      bank = ai.getMemoryBank();
      items.setModel(bank);
    }

    protected class ItemClickListener extends MouseAdapter
    {
      public void mouseClicked(MouseEvent e)
      {
        if (e.getClickCount() == 2)
        {
          int index = items.locationToIndex(e.getPoint());

          if (index != -1)
            insertItem(index);
        }
      }
    }

    protected void insertItem(int i)
      {  pad.paste(bank.getAt(i).toString()); }
  }  */


  // World Tab Class
  // -------------------------------------------------------------

  protected class WorldPanel extends BBoxPanel
  {

    // Visual elements...
    // -----------------------------------------------------------

    protected BTree  items           = new BTree()
    {

      public String getToolTipText(MouseEvent ev)
      {
        TreePath path = items.getPathForLocation(ev.getX(),ev.getY());

        if (path != null)
        {
          BWorldStateNode node = (BWorldStateNode)path.getLastPathComponent();

          if (node.getNodeType() == BWorldStateNode.NODE_TYPE_ITEM )
          {
            BWorldItem item = node.get("");
            return item.getDescription();
          }
          else
            return path.getLastPathComponent().toString();
        }
        else
          return "Double click an world item  to place in the Expression pad. (below) ";

      }

    };


    protected BScrollPane scroller   = new BScrollPane(items);

    // Constructor...
    // -----------------------------------------------------------

    public WorldPanel()
    {
      super(BoxLayout.Y_AXIS);

      scroller.setBorder(null);
      scroller.setViewportBorder(new EmptyBorder(5, 5, 5, 5));
      addHVGlued(scroller);

      items.addMouseListener(new ItemClickListener());
      ToolTipManager.sharedInstance().registerComponent(items);

    }

    public void setAI(BDecisionTreeAI ai)
    {
      BWorldStateNode root =
        (BWorldStateNode) ai.getWorldState();

      // adding in memory items
      root.remove("me.memory");
      
      BMemoryBank bank = ai.getMemoryBank();
      if (bank.size() > 0 )
      {
        for (int i = bank.size() - 1; i >=0 ; i -- )
          root.put( bank.getItem(i) );
      }
      else
        root.put( new BWorldCategory("me.memory", "me.memory") );

      TreeModel model =
        new DefaultTreeModel(root);

      items.setModel(model);
    }


    protected class ItemClickListener extends MouseAdapter
    {
      public void mouseClicked(MouseEvent e)
      {
        if (e.getClickCount() == 2)
        {
          int x = e.getX();
          int y = e.getY();

          TreePath path = items.getPathForLocation(x, y);

          if (path != null)
            insertItem(path);
        }
      }
    }

    protected void insertItem(TreePath path)
    {
      BWorldStateNode node =
        (BWorldStateNode) path.getLastPathComponent();

      if (node.getNodeType() == BWorldStateNode.NODE_TYPE_ITEM)
      {
        BWorldItem item =
          (BWorldItem) node.getUserObject();

        pad.paste("["+ item.getKey()+ "]");
      }
      else if (node.getNodeType() == BWorldStateNode.NODE_TYPE_MEMORY)
      {
        BMemoryItem item = (BMemoryItem) node.getUserObject();

        pad.paste(item.getName());
      }
    }

    public void onClose()
    {
      items           = null;
      scroller        = null;

    }

  }



  // Operator tab class...
  // -------------------------------------------------------------

  protected class OperatorPanel extends BBoxPanel
  {

    // inner class
    class BOperatorButton extends BButton
    {
      private String op;


      public BOperatorButton (BUIResources res, String key )
      {
        super (res, key );
        this.op   = res.getString(key+ ".op");
        setBorderPainted(false);

        this.addActionListener
        ( new ActionListener()
          {
            public void actionPerformed (ActionEvent e)
            {
              pad.paste(op);
            }
          }
        );

      }

      public void processMouseMotionEvent()
      {
      }

    }


    // Constructor...
    // -----------------------------------------------------------

    public OperatorPanel()
    {
      super(BoxLayout.X_AXIS);

      addGlue();
      add(new BOperatorButton(res, "eq"));
      add(new BOperatorButton(res, "neq"));

      add(new BOperatorButton(res, "plus"));
      add(new BOperatorButton(res, "minus"));
      add(new BOperatorButton(res, "times"));
      add(new BOperatorButton(res, "divide"));

      add(new BOperatorButton(res, "and"));
      add(new BOperatorButton(res, "or"));
      add(new BOperatorButton(res, "not"));

      add(new BOperatorButton(res, "gtr"));
      add(new BOperatorButton(res, "less"));
      add(new BOperatorButton(res, "geq"));
      add(new BOperatorButton(res, "leq"));

      addGlue();


     /* scroller.setBorder(null);
      scroller.setViewportBorder(new EmptyBorder(5, 5, 5, 5));
      addHVGlued(scroller);

      items.setCellRenderer(
        new BNamedIconicListCellRenderer());

      items.addMouseListener(new ItemClickListener());

      model = new DefaultListModel();

      model.addElement(new Operator(res, "eq"));
      model.addElement(new Operator(res, "neq"));

      model.addElement(new Operator(res, "plus"));
      model.addElement(new Operator(res, "minus"));
      model.addElement(new Operator(res, "times"));
      model.addElement(new Operator(res, "divide"));

      model.addElement(new Operator(res, "and"));
      model.addElement(new Operator(res, "or"));
      model.addElement(new Operator(res, "not"));

      model.addElement(new Operator(res, "gtr"));
      model.addElement(new Operator(res, "less"));
      model.addElement(new Operator(res, "geq"));
      model.addElement(new Operator(res, "leq"));

      items.setModel(model);  */
    }
   /*
    public void setAI(BDecisionTreeAI ai)
    {
    }

    protected class ItemClickListener extends MouseAdapter
    {
      public void mouseClicked(MouseEvent e)
      {
        if (e.getClickCount() == 2)
        {
          int index = items.locationToIndex(e.getPoint());

          if (index != -1)
            insertItem(index);
        }
      }
    }

    protected void insertItem(int i)
    {
      Operator o = (Operator) model.getElementAt(i);
      pad.paste(o.getOpName());
    }    */

  }

  public void onClose()
  {

    res         = null;
    //tabs        = null;

    worldPanel.onClose();
    worldPanel    = null;
    //memoryTab   = null;
    operatorPanel = null;

    typeLabel   = null;

    pad         = null;
    padScroll   = null;

    OK          = null;
    WRONG       = null;

    super.onClose();

  }


  // Operator class
  // -------------------------------------------------------------

 /* protected static class Operator implements BNamedIconic
  {
    protected String op, desc;
    protected Icon   icon;

    public Operator(BUIResources res, String key)
    {
      this.op   = res.getString(key+ ".op");
      this.desc = res.getString(key+ ".desc");
      this.icon = res.getIcon(key+ ".icon");
    }

    public Icon getIcon()
      { return icon; }

    public Icon getSelectedIcon()
      { return icon; }
    
    public void setIcon(Icon i)
      { this.icon = icon; }

    public String getOpName()
      { return op; }

    public String getName()
      { return desc; }

    public void setName(String name)
      { desc = name; }

  }  */



  // Expression text-area class
  // -------------------------------------------------------------




}