package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BComponentAppearance;
import roboforge.util.graphics.*;

import java.awt.*;


public class BAppearanceSwatch extends BToggleButton
{

  // Internal variables...
  // -------------------------------------------------------------

  private static RenderingHints hints = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );


  protected float roundness;
    // Roundness of the interior swatch display.

  protected BImageCache cache = BImageCache.getSharedInstance();
    // Cache for loading textures...

  protected Image texture;
    // Current texture...

  protected AlphaComposite composite =
    AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);
    // Alpha compositor...


  // Member variables...
  // -------------------------------------------------------------

  protected BComponentAppearance appearance = null;

  // Constructor.
  // -------------------------------------------------------------

  public BAppearanceSwatch(BUIResources res, String resourceKey)
  {
    super(res, resourceKey);

    // Default appearance.
    appearance = new BComponentAppearance();
  }


  // Visual Initialization methods...
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {
    super.initializeUI(res, key);
    roundness = res.getFloat(key+ ".roundness");
    setOpaque(false);
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setAppearance(BComponentAppearance ca)
  {
    appearance = ca;

    if (ca.getTexture() != null)
      texture = cache.lookup(BGlobal.TEXTUREPATH+ ca.getTexture());

    repaint();
  }

  public BComponentAppearance getAppearance()
    { return appearance; }


  // Action methods...
  // -------------------------------------------------------------

  public void paint(Graphics g)
  {
    super.paint(g);

    Graphics2D g2D = (Graphics2D) g;
    g2D.addRenderingHints(hints);

    if (appearance != null)
      paintAppearance(g2D, appearance);

    // TODO: Default appearance?
  }

  protected Insets swatchInsets = new Insets(5, 5, 5, 5);

  protected void paintAppearance(Graphics2D g, BComponentAppearance ca)
  {
    Dimension d = getSize();

    int x = swatchInsets.left;
    int y = swatchInsets.top;
    int w = d.width  - swatchInsets.left - swatchInsets.right;
    int h = d.height - swatchInsets.top  - swatchInsets.bottom;

    int arcsize = Math.round(roundness * Math.min(w, h));

    // Setup color ramp...
    if (isEnabled())
    {
      // Paint Appearance Color ramp...
      Color c, hi, lo;
      c = ca.getColor();

      float[] hsb = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
      float metallicity = (float) ca.getColorDepth() / 200.0f;

      float hiB = hsb[2] + metallicity;
      if (hiB > 1.0f)
        hiB = 1.0f;

      float loB = hsb[2] - metallicity;
      if (loB < 0.0f)
        loB = 0.0f;

      hi = Color.getHSBColor(hsb[0], hsb[1], hiB);
      lo = Color.getHSBColor(hsb[0], hsb[1], loB);

      GradientPaint gp = new GradientPaint(
        0, 0, lo, d.width, d.height, hi);

      g.setPaint(gp);

      // Paint texture overlay...
      if (texture != null )
      {
        g.setComposite(composite);
        g.drawImage(texture, x, y, w, h, this);
      }
    }
    else
    {
      Color c = getBackground();
      float[] hsb = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);

      float hiB = hsb[2] + 0.1f;
      if (hiB > 1.0f)
        hiB = 1.0f;

      g.setColor(Color.getHSBColor(hsb[0], hsb[1], hiB));
    }

    // Paint color ramp...
    g.fillRoundRect(x, y, w, h, arcsize, arcsize);
  }
}
