package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.ui.BApplication;
import roboforge.ui.events.*;
import roboforge.util.processors.*;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.*;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.*;

public class BComboBox extends JComboBox
{

  // Constants...
  // -------------------------------------------------------------

  protected static final KeyStroke DELETE =
    KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0);

  protected static final KeyStroke ARROW_UP =
    KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0);

  protected static final KeyStroke ARROW_DOWN =
    KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0);

  protected static final KeyStroke ARROW_LEFT =
    KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0);

  protected static final KeyStroke ARROW_RIGHT =
    KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0);


  protected static final int  PREF_HEIGHT = 19;


  // Internal variables...
  // -------------------------------------------------------------

  protected int               editingClickCount   = 2;
  protected boolean           isEditable          = false;
  protected boolean           additionsAllowed    = false;
  protected EditingListener   editListener        = null;
  protected boolean           rowsDeleteable      = false;
  protected boolean           confirmDelete       = true;
  protected String            itemName            = "item";


  // Constructor.
  // -------------------------------------------------------------

  public BComboBox(BUIResources res, String key, ComboBoxModel model)
  {
    super(model);
    initializeUI(res, key);
  }

  public BComboBox(BUIResources res, String key, Object[] items)
  {
    super(items);
    initializeUI(res, key);
  }

  public BComboBox(BUIResources res, String key, Vector items)
  {
    super(items);
    initializeUI(res, key);
  }

  public BComboBox(BUIResources res, String key)
  {
    super();
    initializeUI(res, key);
  }

  public BComboBox()
  {
    super();
    initializeUI(null, null);
  }

  public BComboBox(Object[] items)
    { this(null, null, items); }



  // Visual Initialization methods...
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {
    setMinimumSize(new Dimension(getMinimumSize().width, PREF_HEIGHT));
    setPreferredSize(new Dimension(getPreferredSize().width, PREF_HEIGHT));
    setMaximumSize(new Dimension(getMaximumSize().width, PREF_HEIGHT));

    if (res != null)
    {
      res.initComponent(this, key);

      Boolean l = (Boolean) res.get(key+ ".lightweight");
      if (l != null)
        setLightWeightPopupEnabled(l.booleanValue());
    }

    // Set centered alignment, for correct BoxLayout display...
    setAlignmentX(Component.CENTER_ALIGNMENT);
    setAlignmentY(Component.CENTER_ALIGNMENT);

    // Initialize selection arrow keys...
    SelectPreviousListener prev = new SelectPreviousListener();
    SelectNextListener     next = new SelectNextListener();

    registerKeyboardAction(prev, ARROW_UP,    JComponent.WHEN_FOCUSED);
    registerKeyboardAction(prev, ARROW_LEFT,  JComponent.WHEN_FOCUSED);
    registerKeyboardAction(next, ARROW_DOWN,  JComponent.WHEN_FOCUSED);
    registerKeyboardAction(next, ARROW_RIGHT, JComponent.WHEN_FOCUSED);
  }

  /*
  public void actionPerformed(ActionEvent e)
  {
    super.actionPerformed(e);
    selectedItemChanged();
  }
  */

  protected class SelectPreviousListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      int i = getSelectedIndex();
      if (i > 0)
        setSelectedIndex(i-1);
    }
  }

  protected class SelectNextListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      int i = getSelectedIndex();
      if (i < (getModel().getSize()-1))
        setSelectedIndex(i+1);
    }
  }

  public boolean isEmpty()
    { return getModel().getSize() == 0; }



  public void setAdditionsAllowed(boolean a)
    { additionsAllowed = a; }

  public boolean areAdditionsAllowed()
    { return additionsAllowed; }


  public void setEditable(boolean e)
  {
    super.setEditable(e);

    if (e && !isEditable)
    {
      if (editListener == null)
        editListener = new EditingListener();

      if (editor != null)
        editor.addActionListener(editListener);
    }
    else if (!e)
    {
      if (editor != null)
        editor.removeActionListener(editListener);
    }

    isEditable = e;
  }


  protected class EditingListener
    implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      Object item = editor.getItem();

      if (additionsAllowed)
      {
        DefaultComboBoxModel model =
          (DefaultComboBoxModel) dataModel;

        int i = model.getIndexOf(item);
        if (i == -1)
          model.addElement(item);

        setSelectedItem(item);
      }
    }
  }

  public void setClicksToEdit(int i)
    { editingClickCount = i; }


  /*
  public void setEditable(boolean e)
  {
    if (e && !isEditable)
    {
      if (editListener == null)
        editListener = new EditingListener();

      addMouseListener(editListener);
    }
    else if (!e)
    {
      removeMouseListener(editListener);

      if (editor != null)
        editor.removeActionListener(editListener);

      setEditorUsed(false);
    }
  }

  protected void setEditorUsed(boolean e)
  {
    super.setEditable(e);

    if (e)
    {
      Component c = editor.getEditorComponent();
      c.requestFocus();
    }
  }

  protected class EditingListener
    extends    MouseAdapter
    implements ActionListener
  {
    public void mouseClicked(MouseEvent e)
    {
      if (e.getClickCount() >= editingClickCount)
      {
        setEditorUsed(true);
        editor.addActionListener(this);
      }
    }

    public void actionPerformed(ActionEvent e)
    {
      Object item = editor.getItem();

      if (additionsAllowed)
      {
        DefaultComboBoxModel model =
          (DefaultComboBoxModel) dataModel;

        int i = model.getIndexOf(item);
        if (i == -1)
        {
          BDebug.print("BComboBox.edit: inserting "+ item+" - class = "+ item.getClass());
          model.addElement(item);
        }
      }

      editor.removeActionListener(this);
      setEditorUsed(false);

      if (additionsAllowed)
        setSelectedItem(item);
    }
  }
  */

  /*
  public void setSelectedItem(Object o)
  {
    if (isEditable())
      editor.getEditorComponent().repaint();

    super.setSelectedItem(o);
  }
  */

  public Object getItem()
  {
    if (isEditable())
      return editor.getItem();
    else
      return getSelectedItem();
  }

  // Row Deletion methods...
  // -------------------------------------------------------------

  public void setRowsDeleteable(boolean d)
  {
    if (d)
    {
      if (!rowsDeleteable)
        registerKeyboardAction
          (new DeleteListener(), DELETE, WHEN_FOCUSED);
    }
    else
    {
      if (rowsDeleteable)
        unregisterKeyboardAction(DELETE);
    }

    this.rowsDeleteable = d;
  }

  public boolean areRowsDeleteable()
    { return this.rowsDeleteable; }

  public void setConfirmDelete(boolean c)
    { this.confirmDelete = c; }

  public boolean getConfirmDelete()
    { return this.confirmDelete; }

  public void setItemName(String name)
    { itemName = name; }

  protected BPromptDialog getDeleteConfirmDialog(Object o)
  {
    return
      BPromptDialog.createYesNoDialog
      (
        "confirm remove...",
        "You are about to remove the following "+ itemName+ ":\n"+
        "   "+ o.toString()+ "\n\n"+
        "Are you sure you wish to do this?"
      );
  }

  public void removeSelectedItem()
  {
    BProcessor handler = new BProcessor()
    {
      public void process()
      {
        int selected = getSelectedIndex();

        if (selected != -1)
        {
          BMutableListModel model =
            (BMutableListModel) getModel();

          if (model.isRemoveableAt(selected))
          {
            if (confirmDelete)
            {
              // Get user confirmation...
              Object o = getSelectedItem();

              BPromptDialog dlg = getDeleteConfirmDialog(o);
              BApplication.showDialog(dlg);

              if (dlg.getResult())
                model.removeAt(selected);

              // Get the focus back.
              requestFocus();
            }
            else
              // Delete without confirmation.
              model.removeAt(selected);
          }
        }
      }
    };

    handler.start();
  }

  protected class DeleteListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      { removeSelectedItem(); }
  }


}
