package roboforge.ui.components;

import roboforge.ui.BApplication;
import roboforge.util.io.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;

public class BFileDialog extends BOKCancelDialog
{
  protected String promptText;
  protected BFileClient client;


  // UI Resources...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
    { "this.size",        new Dimension(350, 300) });


  protected static int DEFAULT_NAME_MAX = 40;
  // Constructor...
  // -------------------------------------------------------------

  public BFileDialog
  (
    String  title,
    String  prompt,
    String  OKName,
    String  cancelName,
    boolean isFileNameEditable
  )
  {
    this(  title, prompt, OKName, cancelName, isFileNameEditable,DEFAULT_NAME_MAX );
  }

  public BFileDialog
  (
    String  title,
    String  prompt,
    String  OKName,
    String  cancelName,
    boolean isFileNameEditable,
    int maxNameSize
  )
  {
    super(title, true, true, OKName, cancelName);

    initializeUI(res, "this");

    this.promptText = prompt;

    client = new BFileClient(isFileNameEditable, maxNameSize);
    setClient(client);

    res.cleanse();
    res = null;
  }


  public static BFileDialog createOpenDialog(String title, String msg)
    { return  createOpenDialog( title,  msg, DEFAULT_NAME_MAX); }

  public static BFileDialog createOpenDialog(String title, String msg, String OKName)
    { return new BFileDialog (title, msg, OKName, "Cancel", false); }
  
  public static BFileDialog createOpenDialog(String title, String msg, int maxFilenameSize)
    { return new BFileDialog (title, msg, "Open", "Cancel", false, maxFilenameSize ); }

  public static BFileDialog createLoadDialog(String title, String msg)
    { return new BFileDialog (title, msg, "Load", "Cancel", false); }

  public static BFileDialog createSaveDialog(String title, String msg)
    { return createSaveDialog( title,  msg, DEFAULT_NAME_MAX);  }

  public static BFileDialog createSaveDialog(String title, String msg, int maxFilenameSize)
    { return new BFileDialog (title, msg, "Save As", "Cancel", true, maxFilenameSize); }


  public static BFileDialog createSaveAsDialog(String title, String msg)
    { return new BFileDialog (title, msg, "Save As", "Cancel", true); }


  public void setSelectedFile(String file)
    { client.setSelectedFile(file); }

  public String getSelectedFile()
    { return client.getSelectedFile(); }

  public String getSelectedName()
    { return client.getSelectedName(); }

  public void setPath(String p)
    throws FileNotFoundException
    { client.setSourcePath(p); }

  public String getPath()
    { return client.getSourcePath(); }

  public void setFilter(BFilenameFilter f)
    { client.setFilter(f); }


  protected class BFileClient extends OKCancelClient
  {
    protected String selectedFile;
    protected boolean isFileNameEditable = false;

    // Visual elements...
    // -------------------------------------------------------------

    protected BLabel prompt        = new BLabel(promptText);
    protected BFileList files      = new BFileList();
    protected BScrollPane scroller = new BScrollPane(files);
    protected BFilenameField name  = new BFilenameField();

    public BFileClient(boolean isFileNameEditable, int fileNameMax)
    {
      super();

      this.isFileNameEditable = isFileNameEditable;
      name.setFieldSize(fileNameMax);

      layoutFileChildren();
      configFileChildren();

      setCancelDefault();
    }

    protected void layoutFileChildren()
    {
      addHGlued(prompt);
      addTinyMargin();

      // setHVGlue(files);
      add(scroller);
      addSmallMargin();

      if (isFileNameEditable)
      {
        setHGlue(name);
        add(name);
        addMargin();
      }
    }

    protected void configFileChildren()
    {
      prompt.setText(promptText);

      if (!isFileNameEditable)
      {
        name.setEnabled(false);
        OK.setEnabled(false);
      }

      files.addListSelectionListener(new FileSelectListener());
    }


    protected class FileSelectListener
      implements ListSelectionListener
    {
      public void valueChanged(ListSelectionEvent e)
      {
        name.setText((String) files.getSelectedValue());
        OK.setEnabled(true);
      }
    }


    protected void setSourcePath(String p)
      throws FileNotFoundException
      { files.setSourcePath(p); }

    protected String getSourcePath()
      { return files.getSourcePath(); }

    protected void setFilter(BFilenameFilter f)
      { files.setFilter(f); }

    protected void setSelectedFile(String f)
    {
      files.setSelectedValue(f, true);
      name.setText(f);
    }

    protected String getSelectedFile()
    {
      String s = files.toFileName(getSelectedName());

      if (s != null)
        s = getPath()+ s;

      return s;
    }

    protected String getSelectedName()
    {
      String s = name.getText();
      return (s.equals("") ? null : s);
    }

    protected void onCancelPressed(ActionEvent e)
    {
      name.setText("");
      super.onCancelPressed(e);
    }

    protected void onClosePressed(ActionEvent e)
    {
      name.setText("");
      super.onClosePressed(e);
    }
  }
}