package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.ui.events.*;

import javax.swing.BoundedRangeModel;
import javax.swing.JProgressBar;
import java.awt.Component;
import java.awt.Font;

public class BProgressBar extends JProgressBar
  implements BProgressListener
{

  // Constructors...
  // -------------------------------------------------------------

  public BProgressBar(BUIResources res, String key)
  {
    super();
    initializeUI(res, key);
  }

  public BProgressBar(BUIResources res, String key, int min, int max)
  {
    super(min, max);
    initializeUI(null, null);
  }

  public BProgressBar()
    { this(null, null); }


  // Visual Initialization methods...
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {
    if (res != null)
      res.initComponent(this, key);

    /*
    // Set centered alignment, for correct BoxLayout display...
    setAlignmentX(Component.CENTER_ALIGNMENT);
    setAlignmentY(Component.CENTER_ALIGNMENT);
    */
  }


  // Progress monitoring methods...
  // -------------------------------------------------------------

  public void startListeningTo(BProgressive p)
  {
    setValue(model.getMinimum());
    p.addProgressListener(this);
  }

  public void stopListeningTo(BProgressive p)
    { p.removeProgressListener(this); }

  public void progressUpdated(BProgressEvent e)
  {
    BoundedRangeModel model = getModel();

    int min = model.getMinimum();
    int max = model.getMaximum();

    int percentProgress =
      (int) Math.round(min + (e.getProgress() * (max - min)));

    // BDebug.print("Progress updated: "+ percentProgress);

    setValue(percentProgress);
    repaint();
  }

}