package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.BGlobal;

import roboforge.ui.BApplication;
import roboforge.ui.events.*;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;


public class BSystemDialog extends BDialog
{

  // Class variables...
  // -------------------------------------------------------------

  protected static final int BUTTON_WIDTH = 75;
  

  // Internal variables...
  // -------------------------------------------------------------

  protected boolean closeable = false;
  protected String title = null;


  // UI Resources...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleLine.foreground",     BGlobal.colors.getDarkerShade(),

    "systemClose.icon",         BIconCache.getHSB("small_exit_icon.gif"),
    "systemClose.size",         new Dimension(20, 20),

    "titleDrag.cursor",         BGraphics.getCursor("pan_cursor.gif", 15, 15),
    
    "buttons.height",           new Integer(30),

  });



  // Visual elements...
  // -------------------------------------------------------------

  protected SystemDialogContent content;


  // Constructor...
  // -------------------------------------------------------------

  public BSystemDialog(String title, boolean closeable)
  {
    super();

    this.title = title;
    this.closeable = closeable;

    content = new SystemDialogContent();
    setContentPane(content);

    res.cleanse();
    res = null;
  }

  public void addButton(BButton b)
    { content.addButton(b); }

  public void setClient(SystemDialogClient c)
    { content.setClient(c); }

  public BPanel getClient()
    { return content.client; }

  public void setTitle(String title)
    { content.setTitle(title); }


  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();
    if (content != null)
      content.onActivate();
  }

  public void onDeactivate()
  {
    if (content != null)
      content.onDeactivate();
    super.onDeactivate();
  }

  // Visual Initialization methods...
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {
    if (res != null)
      res.initComponent(this, key);

    // Place dialog at the center of the screen...
    Dimension d = this.getSize();
    Dimension s = BGlobal.SCREEN_RESOLUTION;
    setLocation(
      (s.width  - d.width)  / 2,
      (s.height - d.height) / 2);
  }


  protected class SystemDialogContent extends BBoxPanel
  {

    // Visual elements...
    // -------------------------------------------------------------

    protected BLine titleLine      = new BLine(res, "titleLine");

    protected BTitlebarLabel titleLabel
      = new BTitlebarLabel();

    protected BButton close        = null;

    protected BBoxPanel buttons    = new BBoxPanel(res, "buttons", BBoxPanel.X_AXIS);
    protected BBoxPanel clientArea = new BBoxPanel(BBoxPanel.Y_AXIS);

    protected SystemDialogClient client  = null;


    // Constructor...
    // -------------------------------------------------------------

    public SystemDialogContent()
    {
      super(BoxLayout.Y_AXIS);

      if (closeable)
      {
        close = new BButton(res, "systemClose");
        close.setFocusTraversable(false);
      }

      layoutSystemDialogContentChildren();
      configSystemDialogContentChildren();
    }

    protected void layoutSystemDialogContentChildren()
    {
      // Setup border...
      setEmptyBorder(MARGIN);
      addBorderOutside(new LineBorder(BGlobal.colors.getDarkShade(), 1));

      setHGlue(titleLabel);

      if (closeable)
        addLinear(titleLabel, getHStrut(SMALL_MARGIN), close);
      else
        add(titleLabel);

      addMargin();
      setHGlue(titleLine);
      add(titleLine);

      addMargin();

      add(clientArea);

      addGlue();
      add(buttons);
      buttons.addHGlue();
    }

    protected void configSystemDialogContentChildren()
    {
      titleLabel.setText(title);

      TitleDragListener dragger = new TitleDragListener();
      titleLabel.addMouseListener(dragger);
      titleLabel.addMouseMotionListener(dragger);

      Cursor dragCursor = res.getCursor("titleDrag.cursor");
      titleLabel.setCursor(dragCursor);


      if (closeable)
        close.addActionListener( new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
            { onClosePressed(e); }
        });
    }

    protected void addButton(BButton b)
    {
      buttons.addSmallMargin();
      buttons.add(b);
    }

    public void setClient(SystemDialogClient c)
    {
      if (client != null)
        clientArea.remove(client);

      client = c;

      if (client != null)
        clientArea.add(client);
    }

    public void setTitle(String title)
      { titleLabel.setText(title); }

    public void onActivate()
      { if (client != null) client.onActivate(); }

    public void onDeactivate()
      { if (client != null) client.onDeactivate(); }


    protected void onClosePressed(ActionEvent e)
    {
      if (client != null)
        client.onClosePressed(e);
      BApplication.hideDialog(BSystemDialog.this);
    }


  protected class TitleDragListener extends MouseInputAdapter
  {
    protected Point loc, offset;
    protected MouseEvent m;

    public void mousePressed(MouseEvent e)
    {
      m = SwingUtilities.convertMouseEvent(
        titleLabel, e, BSystemDialog.this.getParent());

      loc     = BSystemDialog.this.getLocation();
      offset  = new Point(m.getX(), m.getY());

      offset.x -= loc.x;
      offset.y -= loc.y;

      // BDebug.print("down ["+x+","+y+"]");
    }

    public void mouseDragged(MouseEvent e)
    {
      m = SwingUtilities.convertMouseEvent(
        titleLabel, e, BSystemDialog.this.getParent());

      loc.x = m.getX() - offset.x;
      loc.y = m.getY() - offset.y;

      // BDebug.print("drag ["+x+","+y+"]");

      BSystemDialog.this.setLocation(loc);
      // BSystemDialog.this.repaint();
    }
  }

  }


  protected class SystemDialogClient extends BBoxPanel
  {
    protected JComponent focal  = null;

    public SystemDialogClient(int axis)
    {
      super(axis);
      setOpaque(true);
    }

    protected void setDefaultFocus(JComponent c)
    { 
      focal = c; 
      
      if (focal != null)
        focal.requestDefaultFocus();
    }

    public void onActivate()
    {
      if (focal != null)
        focal.requestFocus();
    }

    public void onDeactivate()
      { }

    protected void onClosePressed(ActionEvent e)
      { }

  }

}
