package roboforge.ui.components;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import roboforge.BDebug;

public class HSBImageIcon extends ImageIcon
  implements ColorableIcon
{

  // Class variables...
  // -------------------------------------------------------------

  protected static Toolkit toolkit =
    Toolkit.getDefaultToolkit();


  // Member variables...
  // -------------------------------------------------------------

  protected float[] oldHSB = {-1.0f, -1.0f, -1.0f};
  protected float[] newHSB = {-1.0f, -1.0f, -1.0f};

  protected Image brightnessMap;

  protected Color   color;
  protected Color   disabledColor;
  protected boolean useComponentColor;




  // Constructors...
  // -------------------------------------------------------------

  public HSBImageIcon(String fileName)
    { this(fileName, null); }

  public HSBImageIcon(Image image)
    { this(image, null); }

  public HSBImageIcon(ImageIcon icon)
    { this(icon.getImage(), null); }
  
  public HSBImageIcon(Image image, Color c)
    { super(image); init(c); }

  public HSBImageIcon(ImageIcon icon, Color c)
    { super(icon.getImage()); init(c); }
    
  public HSBImageIcon(String fileName, Color c)
    { super(fileName); init(c); }

  public HSBImageIcon()
    { super(); init(null); }



  // Initialization methods...
  // -------------------------------------------------------------

  public void init(Color c)
  {
    // Retain image intensity...
    brightnessMap = getImage();

    this.useComponentColor =
      (c == null);

    if (!useComponentColor)
      setColor(c);
  }



  // Paint method...
  // -------------------------------------------------------------

  public synchronized void paintIcon(Component c, Graphics g, int x, int y)
  {
    if (useComponentColor)
    {
      if (c.isEnabled())
        setColor(c.getForeground());

      else if (disabledColor != null)
        setColor(disabledColor);
    }

    super.paintIcon(c, g, x, y);
  }



  // Accessor methods...
  // -------------------------------------------------------------

  public void setUseComponentColor(boolean b)
    { this.useComponentColor = b; }

  public void setColor(Color c)
  {
    this.color = c;

    newHSB = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);

    // Determine if HSB color is changing...
    if ((newHSB[0] != oldHSB[0]) ||
        (newHSB[1] != oldHSB[1]) ||
        (newHSB[2] != oldHSB[2]))
    {
      // If so, generate a new (filtered) image.
      try
      {
        setImage(getHSBImage(brightnessMap, newHSB));
      }
      catch(Exception e)
      {
        BDebug.print ("HSBImageIcon.setColor Exception : " + e.getMessage() );

      }
    }

    // Remember (potentially) new values of HSB.
    oldHSB = newHSB;
  }

  public Color getColor()
    { return color; }

  public void setDisabledColor(Color disabled)
    { this.disabledColor = disabled; }

  public Color getDisabledColor()
    { return disabledColor; }





  // HSB Color filter class...
  // -------------------------------------------------------------

    static BImageObserver observer = new BImageObserver();
  protected static Image getHSBImage(Image image, float[] hsb)
  {
    //ImageFilter filter = new HSBFilter(hsb);
   // Image result = toolkit.createImage(
   //   new FilteredImageSource(image.getSource(), filter));

   try
   {

    if (image.getWidth(observer) <= 0 || image.getHeight(observer) <= 0)
      return new BufferedImage(1,1,  BufferedImage.TYPE_INT_ARGB);

    BufferedImage result = new BufferedImage(image.getWidth(observer), image.getHeight(observer), BufferedImage.TYPE_INT_ARGB);

    int wid = result.getWidth();
    int hei = result.getHeight();

  	int[] pixels = new int[wid * hei];

  	PixelGrabber pg = new PixelGrabber(image, 0, 0, wid, hei, pixels, 0, wid);

    try{
      pg.grabPixels();
    }catch(Exception e) {}

    int scan = 0;
    for (int y = 0; y < hei; y++) {
      for (int x = 0; x < wid; x++) {
        result.setRGB(x, y, filterRGB(x, y, pixels[scan], hsb[0], hsb[1], hsb[2]));
        scan ++;
      }
    }

    return result;

   } catch (Exception ei) {
    BDebug.print( "HSBImageIcon.getHSBImage: Exception "+ei.getMessage() );
    return new BufferedImage(1,1,  BufferedImage.TYPE_INT_ARGB);
   }


  }

    public static int filterRGB(int x, int y, int rgb, float h, float s, float b)
    {
      float scaleFactor = 1.0f;

      scaleFactor = b * (1.0f / 255.0f);

      b = ((float) (rgb & 0xFF)) * scaleFactor;

      int alpha = (rgb >> 24) & 0xff;
      int col = Color.HSBtoRGB(h, s, b);

      col = col & 0xffffff;
      int newCol = ((alpha << 24) & 0xff000000) + col;

		  return newCol;
  	}

  public static class BImageObserver implements ImageObserver
  {

    public BImageObserver()
    {}

    public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height)
    {return false;}
  }

  protected static class HSBFilter extends RGBImageFilter
  {
    protected float h = 0.0f;
    protected float s = 0.0f;
    protected float b = 0.0f;
    protected float scaleFactor = 1.0f;
	  public HSBFilter(float[] hsb)
    {

      this.h = hsb[0];
      this.s = hsb[1];
      this.b = hsb[2];
      this.scaleFactor = b * (1.0f / 255.0f);

      canFilterIndexColorModel = true;
    }

    public int filterRGB(int x, int y, int rgb)
    {
      b = ((float) (rgb & 0xFF)) * scaleFactor;
		  return Color.HSBtoRGB(h, s, b);
  	}
  }

}
