package roboforge.ui.console;

import roboforge.BBot;
import roboforge.BGlobal;

import java.awt.*;

import javax.swing.SwingConstants;
import javax.vecmath.*;




public class BBotConsole extends BConsoleContainer
{

  // A BBotConsole displays the vital statisics of a bot on
  // the canvas console.


  // Constants...
  // -------------------------------------------------------------

  protected static final int NAME_BORDER = 5;
    // Border around bot name, in pixels.


  // Data Members...
  // -------------------------------------------------------------

  protected BBot bot;


  // UI Components...
  // -------------------------------------------------------------

  protected BConsoleLabel       name;
  protected BConsoleRectangle   nameBackdrop;
  protected BConsoleProgressBar health;
  protected BConsoleProgressBar energy;


  // UI Members...
  // -------------------------------------------------------------

  protected Color3f black         = new Color3f(0, 0, 0);

  protected Color3f green         = new Color3f(0, 0.75f, 0);
  protected Color3f yellow        = new Color3f(0.8f, 0.8f, 0);
  protected Color3f red           = new Color3f(1f, 0, 0);
  protected Color3f darkred       = new Color3f(0.5f, 0, 0);

  protected Color3f cyan          = new Color3f(0, 0.5f, 1f);
  protected Color3f midcyan       = new Color3f(0, 0.25f, 1f);
  protected Color3f blue          = new Color3f(0, 0, 1f);
  protected Color3f darkblue      = new Color3f(0, 0, 0.5f);

  /*
  protected Color3f darkgreen     = new Color3f(0, 0.5f, 0);
  protected Color3f darkyellow    = new Color3f(0.5f, 0.5f, 0);
  protected Color3f darkred       = new Color3f(0.5f, 0, 0);
  protected Color3f darkblue      = new Color3f(0, 0, 0.5f);
  */


  protected Color3f healthColor = new Color3f();
    // Current color of the health bar.

  protected Color3f energyColor = new Color3f();
    // Current color of the energy bar.



  // Constructor...
  // -------------------------------------------------------------

  public BBotConsole(BBot bot)
  {
    this.bot = bot;

    configChildren();
  }


  protected void configChildren()
  {

    // Bot backdrop...
    nameBackdrop = new BConsoleRectangle();
    nameBackdrop.setColor(black);
    nameBackdrop.setTransparency(0.5f);

    // Bot name...
    Font namefont = new Font(BGlobal.FONT_NAME, Font.BOLD, 12);
    name = new BConsoleLabel(bot.getName(), namefont);

    // Bot health / energy bars..
    health = new BConsoleProgressBar(green, green, SwingConstants.LEFT);
    energy = new BConsoleProgressBar(cyan,  cyan,  SwingConstants.LEFT);

    add(nameBackdrop);
    add(name);
    add(health);
    add(energy);
  }



  public void setSize(int w, int h)
  {
    super.setSize(w, h);
    updateLayout();
  }


  protected void updateLayout()
  {
    int ny = name.getHeight() + NAME_BORDER * 2;
    name.setLocation(NAME_BORDER, NAME_BORDER + 2);

    nameBackdrop.setLocation(0, 0);
    nameBackdrop.setSize(this.width, ny);

    double remainder = this.height - ny;

    int hy = (int) Math.round(remainder * 0.6);
    health.setLocation(0, ny);
    health.setSize(this.width, hy);

    int ey = (int) Math.round(remainder * 0.4);
    energy.setLocation(0, ny + hy);
    energy.setSize(this.width, ey);
  }


  public void update()
  {
    int healthPercent
      = 100 - (int) bot.getPercentageDamageLevel();

    int energyPercent
      = (int) bot.getPercentageEnergyLevel();

  	health.setValue(healthPercent);
    energy.setValue(energyPercent);

    // Update health bar's color..
    interpolateColor(healthPercent, darkred, red, yellow, green, healthColor);
    health.setColor(healthColor);
    health.setBackgroundColor(healthColor);

    // Update energy bar's color..
    interpolateColor(energyPercent, darkblue, blue, midcyan, cyan, energyColor);
    energy.setColor(energyColor);
    energy.setBackgroundColor(energyColor);
  }


  public void interpolateColor
  (
    int percent,
    Color3f floor,
    Color3f lo,
    Color3f med,
    Color3f hi,
    Color3f result
  )
  {
    float t = (float) percent / 100.0f;

    if (t < 0.5)
      result.interpolate(floor, lo, t * 2f);
    else if (t < 0.75)
      result.interpolate(lo, med, (t - 0.50f) * 4f);
    else
      result.interpolate(med, hi, (t - 0.75f) * 4f);
  }


  public void cleanse()
  // cleanses everything apart from detaching from j3d scene
  {

    bot = null;

    name = null;
    nameBackdrop = null;
    health = null;
    energy = null;
    black  = null;

    green         = null;
    yellow        = null;
    red           = null;
    darkred       = null;

    cyan          = null;
    midcyan       = null;
    blue          = null;
    darkblue      = null;

    healthColor =null;
    // Current color of the health bar.

    energyColor = null;


  }


}

