package roboforge.ui.console;

import roboforge.BDebug;

import java.awt.*;
import java.awt.image.*;

public class BGlyphFont {

	protected final static int DEFAULT_SPACING = 1;

	protected BGlyph[] glyphs;
	protected int  numGlyphs;
	protected char firstGlyph;
	
	// FONT METRICS
	protected int maxHeight = 0;
	protected int maxWidth  = 0;
	protected int totWidth  = 0;
	protected int avgWidth  = 0;

	protected int spacing = DEFAULT_SPACING;

	protected Toolkit toolkit = Toolkit.getDefaultToolkit();

	public BGlyphFont(Image source, int numGlyphs, char firstGlyph)
  {

    BDebug.assert(source != null, "Null image supplied to BGlyphFont");

		this.glyphs		= new BGlyph[numGlyphs];
		this.numGlyphs	= numGlyphs;
		this.firstGlyph = firstGlyph;
		this.maxHeight	= source.getHeight(null) - 1;
		int[][] data	= getData(source);

		int head  = 0;
		int tail  = 0;
		int width = 0;

    int i = 0;

    try
    {
		for (i=0; i<numGlyphs; i++) {

			// Locate bounding box for glyph.
			while ( !isBlack(data[head][0]) )
				head++;
			tail = head + 1;
			while ( !isBlack(data[tail][0]) )
				tail++;

			// Grab image rect and initialise glyph i.
			width = tail - head + 1;
			glyphs[i] = new BGlyph(
				grabImageRect(data, head, 1, width, maxHeight),
				width, maxHeight );

			// Advance past current glyph in source image.
			head = tail + 1;
		}
    }
    catch (Exception e)
    {
      BDebug.print("ERROR: BGlyphFont.<init> - Ran out of image at index "+ i);

      for (int j=i; j<numGlyphs; j++)
        glyphs[j] = glyphs[i-1];
    }

		calculateMetrics();
	}

	protected void calculateMetrics() {
		maxWidth = 0;
		totWidth = 0;

		int cur = 0;
		for (int i=0; i<numGlyphs; i++) {
			cur = glyphs[i].getWidth();
			if (cur > maxWidth)
				maxWidth = cur;
			totWidth += cur;
		}

		avgWidth = totWidth / numGlyphs;
	}

	public int getMaxHeight() {
		return maxHeight;
	}

	public int getMaxWidth() {
		return maxWidth;
	}

	public void draw(Graphics g, String text, int x, int y) {
		int  index;
		for (int i=0; i<text.length(); i++) {
			index = getIndex( text.charAt(i) );
			if ( isValidIndex(index) ) {
				glyphs[index].draw(g, x, y);
				x += glyphs[index].getWidth();
			}
		}
	}


	public void draw(Graphics g, char c, int x, int y)
  {
    int index = getIndex(c);
    if (isValidIndex(index))
      glyphs[index].draw(g, x, y);
	}

	protected int getIndex(char key) {
		return key - firstGlyph;
	}

	protected boolean isValidIndex(int index) {
		return (index >= 0) && (index < numGlyphs);
	}

	public void setSpacing(int newSpacing) {
		spacing = newSpacing;
	}

	public int getSpacing() {
		return spacing;
	}

	protected boolean isBlack(int pixel) {
		return (pixel << 8) == 0;	// Ignore transparency.
	}
			
	protected Image grabImageRect(int[][] data, 
			int x, int y, int w, int h) {

		// Returns an image created from a rectangular region 
		// of the 2d int array, 'data'.

		int[] buffer	= new int[w*h];
		int bot = y + h;
		int rgt = x + w;
		int offset = 0;

		for (int i=y; i<bot; i++)
			for (int j=x; j<rgt; j++)
				buffer[offset++] = data[j][i];

		MemoryImageSource m = new MemoryImageSource(w, h, buffer, 0, w);

		return toolkit.createImage(m);
	}

	protected int[][] getData(Image img) {

		int	w			= img.getWidth(null);
		int	h			= img.getHeight(null);
        
		int[] buffer	= new int[w*h];
		int[][] data	= new int[w][h];
        
		PixelGrabber pg	= 
			new PixelGrabber(img, 0, 0, w, h, buffer, 0, w);

        try   { pg.grabPixels(); }
		catch ( InterruptedException e ) { return null; }

        if ((pg.status() & ImageObserver.ABORT) != 0)
			return null;

		int offset = 0;
		for (int i=0; i<h; i++)
			for (int j=0; j<w; j++)
				data[j][i] = buffer[offset++];

		return data;
	}

  public int getWidthOf(String text)
  {
		int index, w = 0;

		for (int i=0; i<text.length(); i++)
    {
			index = getIndex(text.charAt(i));
			if (isValidIndex(index))
				w += glyphs[index].getWidth();
		}

    return w;
  }

  public int getHeightOf(String text)
    { return maxHeight; }


}

