package roboforge.ui.console;

import roboforge.geometry.BRectangle3D;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import javax.media.j3d.*;
import javax.vecmath.*;


public abstract class BManagedTexture extends Texture2D
{

  // Constants...
  // -------------------------------------------------------------

  protected final static int DEFAULT_SIZE = 128;
  protected final static int DEFAULT_TYPE = Texture.RGBA;


  // Internal variables...
  // -------------------------------------------------------------

  protected BufferedImage image;


  // Constructors...
  // -------------------------------------------------------------

  public BManagedTexture()
    { this(DEFAULT_SIZE, DEFAULT_SIZE); }

  public BManagedTexture(int w, int h)
    { this(Texture.BASE_LEVEL, DEFAULT_TYPE, w, h); }

  public BManagedTexture(int format, int w, int h)
    { this(Texture.BASE_LEVEL, format, w, h); }

  public BManagedTexture(int mip, int format, int w, int h)
  {
    super(mip, format, w, h);

    image = new BufferedImage(w, h, getBufferedImageType(format));

    /*
    Graphics2D g = image.createGraphics();
    g.setColor(Color.black);
    g.fillRect(0, 0, 256, 256);
    */

    update();
  }


  protected int getBufferedImageType(int format)
  {
    int type;

    switch (format)
    {
      case Texture.RGBA:
        type = BufferedImage.TYPE_4BYTE_ABGR;
        break;

      case Texture.RGB:
        type = BufferedImage.TYPE_INT_BGR;
        break;

      case Texture.ALPHA:
        type = BufferedImage.TYPE_4BYTE_ABGR; // BufferedImage.TYPE_BYTE_GRAY;
        break;

      case Texture.LUMINANCE:
        type = BufferedImage.TYPE_BYTE_GRAY;
        break;

      default:
        type = BufferedImage.TYPE_4BYTE_ABGR;
        break;
    }

    return type;
  }

  // Accessor methods...
  // -------------------------------------------------------------

  public abstract BTexturePortion requestPortion(int w, int h);
    // Request a portion of this texture, of size (w, h).
    // Returns null if a suitable portion could not be found.

  public abstract void returnPortion(BTexturePortion p);
    // Indicates that the given portion is no longer needed,
    // and can be allocated to other portions.

  public BTexturePortion requestPortion(BufferedImage image)
  {
    int w = image.getWidth();
    int h = image.getHeight();

    BTexturePortion result = requestPortion(w, h);

    if (result != null)
      result.setImage(image);

    return result;
  }

  public Graphics2D createGraphics()
    { return image.createGraphics(); }

  public int getWidth()
    { return image.getWidth(); }

  public int getHeight()
    { return image.getHeight(); }

    

  // Internal methods...
  // -------------------------------------------------------------

  protected void update()
  {
    ImageComponent2D ic =
      //new ImageComponent2D(ImageComponent.FORMAT_RGBA, image, true, true);  // (this new line) does not work
      new ImageComponent2D(ImageComponent.FORMAT_RGBA, image);

    setImage(0, ic);
  }

}