package roboforge.ui.fusion;

import java.awt.*;
import javax.swing.*;

public class FusionArrowButton extends JButton
  implements SwingConstants
{
    private static RenderingHints hints = new RenderingHints (
      RenderingHints.KEY_ANTIALIASING,
      RenderingHints.VALUE_ANTIALIAS_ON );

    protected int orientation       = 0;
    protected Color bg, fg, highlightColor;

    // For drawing arrows...
    protected Polygon arrowPoly = null;

    public FusionArrowButton(int orientation, Color bg, Color fg, Color h)
    {
      super();
      this.orientation = orientation;
      this.bg = bg;
      this.fg = fg;
      this.highlightColor  = h;
    }

    public Dimension getPreferredSize()
      { return new Dimension(16, 16); }

    public Dimension getMinimumSize()
      { return new Dimension(5, 5); }

    public Dimension getMaximumSize()
      { return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE); }

    public boolean isFocusTraversable()
      { return false; }


	  public void paint(Graphics g)
    {
      int w = getSize().width;
      int h = getSize().height;

      if (model.isPressed())
        g.setColor(highlightColor);
      else
        g.setColor(bg);

      g.fillRect(0,0,w,h);

      Graphics2D g2d = (Graphics2D) g;
      g2d.setRenderingHints(hints);

      int s = Math.min((h - 3) / 3, (w - 3) / 3);
      s = Math.max(s, 2);

      int m = s / 2;
      int tx = (w - s) / 2;
      int ty = (h - s) / 2;


      g.translate(tx, ty);

      if (isEnabled())
        g.setColor(fg);
      else
        g.setColor(highlightColor);

      switch (orientation)
      {
        case NORTH:
          g.drawLine(0, s, m, 0);
          g.drawLine(m, 0, s, s);
          g.drawLine(s, s, 0, s);
          break;

        case SOUTH:
          g.drawLine(0, 0, m, s);
          g.drawLine(m, s, s, 0);
          g.drawLine(s, 0, 0, 0);
          break;

        case EAST:
          g.drawLine(0, 0, s, m);
          g.drawLine(s, m, 0, s);
          g.drawLine(0, s, 0, 0);
          break;

        case WEST:
          g.drawLine(s, 0, 0, m);
          g.drawLine(0, m, s, s);
          g.drawLine(s, s, s, 0);
          break;
      }

      g.translate(-tx, -ty);
    }
}
