package roboforge.ui.fusion;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.sound.*;

import roboforge.ui.components.BUIColorScheme;
import roboforge.ui.components.HSBImageIcon;
import roboforge.ui.components.LazyHSBImageIcon;
import roboforge.ui.components.border.*;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;

import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.border.*;
import javax.swing.text.JTextComponent;
import javax.swing.text.DefaultEditorKit;

import javax.swing.plaf.metal.*;


public class FusionLookAndFeel extends BasicLookAndFeel
  implements java.io.Serializable
{
  // MetalLookAndFeel

  public String getID()
    { return "Fusion"; }

  public String getName()
    { return "Fusion Look and Feel"; }

  public String getDescription()
    { return "Look and feel for the game."; }

  public boolean isNativeLookAndFeel()
    { return false; }

  public boolean isSupportedLookAndFeel()
    { return true; }


  public static boolean install()
  {
    try
    {
      // Install our custom look and feel...
      FusionLookAndFeel fusionLookAndFeel =
        new FusionLookAndFeel();

      UIManager.LookAndFeelInfo info =
        new UIManager.LookAndFeelInfo(
          fusionLookAndFeel.getName(),
          fusionLookAndFeel.getClass().getName()
        );

      UIManager.installLookAndFeel(info);
      UIManager.setLookAndFeel(fusionLookAndFeel);
    }
    catch (Exception e)
      { return false; }

    return true;
  }

  protected void initClassDefaults(UIDefaults table)
  {
    super.initClassDefaults(table);

    // Regular components...
    installComponentUI(table, "ButtonUI");
    installComponentUI(table, "CheckBoxUI");
    installComponentUI(table, "ComboBoxUI");
    installComponentUI(table, "LabelUI");
    installComponentUI(table, "ListUI");
    installComponentUI(table, "ScrollBarUI");
    installComponentUI(table, "SliderUI");
    installComponentUI(table, "TabbedPaneUI");
    installComponentUI(table, "TextFieldUI");
    installComponentUI(table, "TreeUI");

    // Custom components...
    installComponentUI(table, "ConnectorButtonUI");
    installComponentUI(table, "FilledLabelUI");
    installComponentUI(table, "SystemButtonUI");
    installComponentUI(table, "ToggleButtonUI");
  }

  protected void installComponentUI(UIDefaults table, String key)
  {
    try
    {
      String className = BGlobal.BASE_PACKAGE_NAME+ ".ui.fusion.Fusion"+ key;
      Class theClass = Class.forName(className);

      table.put(key, className);
      table.put(className, theClass);
    }
    catch (Exception ex) { ex.printStackTrace(); }
  }

  protected void initComponentDefaults(UIDefaults table)
  {
    super.initComponentDefaults(table);

    // UI Defaults for L&F
    // ------------------------------------------------------

    // Shared Insets...
    Insets defaultInsets = new Insets(5, 5, 5, 5);
    Insets horzPadInsets = new Insets(0, 30, 2, 10);
    Insets zeroInsets    = new Insets(0, 0, 0, 0);
    Insets unitInsets    = new Insets(1, 1, 1, 1);
    Insets leftPadInsets = new Insets(2, 5, 3, 5);

    
    // Shared roundness of component rectangles.
    float ROUNDNESS = 0.25f;
    Float defaultRoundness = new Float(ROUNDNESS);

    
    // Component borders...
    BorderUIResource systemBorder = new
      BorderUIResource( new BRoundedBorder(
        table.getColor("control"),
        table.getColor("controlHighlight"),
        table.getColor("controlShadow"),
        ROUNDNESS));

    BorderUIResource defaultBorder  = new
      BorderUIResource( new BRoundedBorder(
        table.getColor("controlShadow"),
        table.getColor("control"),
        table.getColor("controlDkShadow"),
        ROUNDNESS));

    BorderUIResource lineBorder  = new
      BorderUIResource( new BLineBorder(
        BLineBorder.SIDES_ALL,
        table.getColor("controlDkShadow"),
        table.getColor("controlShadow"),
        table.getColor("controlDkShadow")));

    BorderUIResource lightLineBorder  = new
      BorderUIResource( new BLineBorder(
        BLineBorder.SIDES_ALL,
        table.getColor("controlShadow"),
        table.getColor("control"),
        table.getColor("controlShadow")));

    BorderUIResource paddedLineBorder  = new
      BorderUIResource( BorderFactory.createCompoundBorder(
        lineBorder, new EmptyBorder(2,2,2,2)));

    BorderUIResource leftPadBorder = new
      BorderUIResource( new EmptyBorder(leftPadInsets));

    BorderUIResource focusBorder = new
      BorderUIResource( new EmptyBorder(2,2,2,2));

    BorderUIResource textBorder = new
      BorderUIResource( BorderFactory.createCompoundBorder(
        lineBorder, leftPadBorder));

    
    // Shared fonts...
    FontUIResource defaultFont =
      new FontUIResource(BGlobal.FONT_NAME, Font.PLAIN, 12);

    FontUIResource smallFont =
      new FontUIResource("Dialog", Font.PLAIN, 10);

    FontUIResource boldFont =
      new FontUIResource("Dialog", Font.BOLD, 11);
    
    
    // UI Color scheme.
    BUIColorScheme colors = BGlobal.colors;

    // Key Bindings...
    UIDefaults.LazyInputMap fieldInputMap = getFieldInputMap();
    UIDefaults.LazyInputMap multiInputMap = getMultiLineInputMap();

    
    // List cell renderer...
  	Object listCellRendererActiveValue = new UIDefaults.ActiveValue() 
    {
	    public Object createValue(UIDefaults table) 
        { return new FusionListCellRenderer.UIResource(); }
  	};
    

    Object[] defaults = {
      "Button.background",              table.getColor("desktop"),
      "Button.border",                  defaultBorder,
      "Button.clickSound",              uiSound("8-1.wav"),
      "Button.disabledColor",           table.getColor("controlDkShadow"),
	    "Button.font",                    defaultFont,
      "Button.foreground",              table.getColor("controlText"),
      "Button.insets",                  zeroInsets,
      "Button.pressedColor",            table.getColor("controlShadow"),
      "Button.roundness",               defaultRoundness,

      "CheckBox.background",            table.getColor("desktop"),
      "CheckBox.foreground",            table.getColor("controlShadow"),
      "CheckBox.disabledColor",         table.getColor("controlDkShadow"),
      "CheckBox.font",                  smallFont,
      "CheckBox.icon",                  checkBoxIcon("check_icon.gif", HSBCheckBoxIcon.STYLE_SQUARE),

      "ComboBox.background",            colors.getVeryDarkShade(),
      "ComboBox.foreground",            table.getColor("control"),
      "ComboBox.font",                  smallFont,
      "ComboBox.border",                lineBorder,
      "ComboBox.disabledBackground",    table.getColor("desktop"),
      "ComboBox.disabledForeground",    table.getColor("controlDkShadow"),
      "ComboBox.selectedBackground",    table.getColor("controlShadow"),
      "ComboBox.selectedForeground",    table.getColor("controlText"),
      "ComboBox.selectionBackground",   table.getColor("controlShadow"),
      "ComboBox.selectionForeground",   table.getColor("control"),
      "ComboBox.buttonBorder",          lineBorder,
      "ComboBox.buttonBackground",      table.getColor("controlDkShadow"),
      "ComboBox.buttonForeground",      table.getColor("controlHighlight"),
      "ComboBox.buttonHighlight",       table.getColor("controlShadow"),

      "ConnectorButton.circleSize",     new Dimension(20, 20),
      "ConnectorButton.clickSound",     uiSound("8-2.wav"),
      "ConnectorButton.disabledColor",  table.getColor("controlDkShadow"),
      "ConnectorButton.fillColor",      table.getColor("controlLtHighlight"),
      "ConnectorButton.fillInsets",     defaultInsets,
      "ConnectorButton.font",           new FontUIResource(BGlobal.FONT_NAME, Font.PLAIN, 18),
      "ConnectorButton.lineColor",      table.getColor("controlShadow"),
      "ConnectorButton.textColor",      table.getColor("controlText"),
      "ConnectorButton.textIconGap",    new Integer(10),

	    "EditorPane.focusInputMap",       multiInputMap,

      "FilledLabel.fillColor",          table.getColor("controlShadow"),
      "FilledLabel.disabledFillColor",  colors.getDarkestShade(),
      "FilledLabel.fillRoundness",      defaultRoundness,
      "FilledLabel.fillInsets",         defaultInsets,
      "FilledLabel.border",             defaultBorder,
      "FilledLabel.disabledColor",      table.getColor("controlDkShadow"),
      "FilledLabel.iconColor",          table.getColor("controlHighlight"),

      "Label.font",                     smallFont,
      "Label.disabledColor",            table.getColor("controlDkShadow"),
      "Label.background",               table.getColor("window"),
      "Label.foreground",               table.getColor("controlShadow"),

      "Line.foreground",                table.getColor("controlDkShadow"),

      "List.background",                colors.getVeryDarkShade(),
      "List.foreground",                table.getColor("controlText"),
      "List.border",                    null,
      "List.cellRenderer",              listCellRendererActiveValue,
      "List.focusCellHighlightBorder",  leftPadBorder,
      "List.font",                      smallFont,
      "List.selectionBackground",       table.getColor("controlShadow"),
      "List.selectionForeground",       table.getColor("controlText"),
      "List.selectSound",               uiSound("8-3.wav"),

      "MenuItem.background",            table.getColor("controlDkShadow"),
      "MenuItem.font",                  smallFont,
      "MenuItem.disabledForeground",    table.getColor("controlShadow"),
      "MenuItem.foreground",            table.getColor("controlText"),
      "MenuItem.selectionBackground",   table.getColor("controlShadow"),
      "MenuItem.selectionForeground",   table.getColor("controlText"),

      "OptionPane.background",          table.getColor("desktop"),

	    "Panel.font",                     smallFont,
	    "Panel.background",               table.getColor("desktop"),
	    "Panel.foreground",               table.getColor("control"),

      "PasswordField.background",           colors.getVeryDarkShade(),
      "PasswordField.foreground",           table.getColor("control"),
      "PasswordField.inactiveForeground",   table.getColor("controlShadow"),
      "PasswordField.font",                 smallFont,
      "PasswordField.caretForeground",      table.getColor("controlHighlight"),
      "PasswordField.caretBlinkRate",       new Integer(250),
	    "PasswordField.focusInputMap",        fieldInputMap,
      "PasswordField.margin",               zeroInsets,
      "PasswordField.selectionBackground",  table.getColor("controlShadow"),
      "PasswordField.selectionForeground",  table.getColor("control"),
      "PasswordField.border",               textBorder,

      "PopupMenu.background",           table.getColor("controlDkShadow"),
      "PopupMenu.foreground",           table.getColor("control"),
      "PopupMenu.border",               lineBorder,
      "PopupMenu.font",                 smallFont,

      "ProgressBar.background",         colors.getVeryDarkShade(),
      "ProgressBar.foreground",         table.getColor("controlShadow"),
      "ProgressBar.border",             lineBorder,
      "ProgressBar.cellLength",         new Integer(1),
      "ProgressBar.cellSpacing",        new Integer(1),

	    "RadioButton.background",         table.getColor("window"),
      "RadioButton.foreground",         table.getColor("controlShadow"),
      "RadioButton.disabledColor",      table.getColor("controlDkShadow"),
	    "RadioButton.font",               smallFont,
      "RadioButton.icon",               checkBoxIcon("radio_icon.gif", HSBCheckBoxIcon.STYLE_ROUND),

      "Rect.foreground",                table.getColor("controlDkShadow"),

      "ScrollBar.background",           table.getColor("controlDkShadow"),
      "ScrollBar.border",               null,
      "ScrollBar.buttonBorder",         null,
      "ScrollBar.buttonBackground",     table.getColor("controlDkShadow"),
      "ScrollBar.buttonForeground",     table.getColor("controlHighlight"),
      "ScrollBar.buttonHighlight",      table.getColor("controlShadow"),
      "ScrollBar.thumb",                table.getColor("controlDkShadow"),
      "ScrollBar.thumbHighlight",       table.getColor("controlShadow"),

      "ScrollPane.background",          colors.getVeryDarkShade(),
      "ScrollPane.border",              lineBorder,
      "ScrollPane.viewportBorder",      null,

	    "Slider.background",              table.getColor("desktop"),
	    "Slider.foreground",              table.getColor("controlShadow"),
	    "Slider.highlight",               table.getColor("controlShadow"),
      "Slider.shadow",                  table.getColor("controlDkShadow"),
      "Slider.focus",                   table.getColor("controlText"),
      "Slider.border",                  null,

      // BasicSliderUI


      "SystemButton.background",        table.getColor("window"),
      "SystemButton.foreground",        table.getColor("control"),
      "SystemButton.pressedColor",      table.getColor("controlShadow"),
      "SystemButton.disabledColor",     table.getColor("controlDkShadow"),
      "SystemButton.border",            systemBorder,

      "TabbedPane.background",           table.getColor("desktop"),
      "TabbedPane.foreground",           table.getColor("controlText"),
      "TabbedPane.highlight",            table.getColor("controlDkShadow"),
      "TabbedPane.lightHighlight",       table.getColor("controlDkShadow"),
      "TabbedPane.tabShadow",            table.getColor("controlDkShadow"),
      "TabbedPane.selectedTabColor",     table.getColor("controlDkShadow"),
      "TabbedPane.font",                 boldFont,
      "TabbedPane.tabInsets",            horzPadInsets,
      "TabbedPane.contentBorderInsets",  unitInsets,
      "TabbedPane.selectedTabPadInsets", zeroInsets,
      "TabbedPane.tabAreaInsets",        new InsetsUIResource(0, 2, 0, 2),
      "TabbedPane.contentBorderInsets",  new InsetsUIResource(1, 0, 0, 0),
      "TabbedPane.tabRunOverlay",        new Integer(15),


      "Table.font",                     smallFont,
      "Table.background",               colors.getVeryDarkShade(),
      "Table.foreground",               table.getColor("controlText"),
      "Table.focusCellHighlightBorder", focusBorder,
      "Table.focusCellBackground",      table.getColor("controlShadow"),
      "Table.focusCellForeground",      table.getColor("controlText"),
      "Table.selectionBackground",      table.getColor("controlShadow"),
      "Table.selectionForeground",      table.getColor("controlText"),
      "Table.selectedBackground",       table.getColor("controlShadow"),
      "Table.selectedForeground",       table.getColor("controlText"),

      "Table.scrollPaneBorder",         null,
      "Table.border",                   null,
      "Table.gridColor",                table.getColor("desktop"),

      "TableHeader.font",               smallFont,
      "TableHeader.foreground",         table.getColor("controlText"),
      "TableHeader.background",         table.getColor("controlDkShadow"),
      "TableHeader.border",             null,
      "TableHeader.cellBorder",         BorderFactory.createCompoundBorder(
                                          new BLineBorder(
                                            BLineBorder.SIDES_TOP_BOTTOM_RIGHT,
                                            colors.getDarkestShade()),
                                          new EmptyBorder(2,2,2,2)),

	    "TextArea.font",                  smallFont,
      "TextArea.border",                textBorder,
      "TextArea.background",            colors.getVeryDarkShade(),
	    "TextArea.focusInputMap",         multiInputMap,
      "TextArea.foreground",            table.getColor("controlText"),
      "TextArea.inactiveForeground",    table.getColor("controlShadow"),
      "TextArea.selectionBackground",   table.getColor("controlShadow"),
      "TextArea.selectionForeground",   table.getColor("control"),

      "TextField.background",           colors.getVeryDarkShade(),
      "TextField.foreground",           table.getColor("control"),
      "TextField.inactiveForeground",   table.getColor("controlShadow"),
      "TextField.font",                 smallFont,
      "TextField.caretForeground",      table.getColor("controlHighlight"),
      "TextField.caretBlinkRate",       new Integer(250),
	    "TextField.focusInputMap",        fieldInputMap,
      "TextField.margin",               zeroInsets,
      "TextField.selectionBackground",  table.getColor("controlShadow"),
      "TextField.selectionForeground",  table.getColor("control"),
      "TextField.border",               textBorder,

	    "TextPane.focusInputMap",         multiInputMap,
      "TextPane.selectionBackground",   table.getColor("controlShadow"),
      "TextPane.selectionForeground",   table.getColor("control"),

	    "ToggleButton.background",        table.getColor("desktop"),
      "ToggleButton.border",            defaultBorder,
      "ToggleButton.clickSound",        uiSound("8-2.wav"),
      "ToggleButton.disabledColor",     table.getColor("controlDkShadow"),
	    "ToggleButton.font",              smallFont,
      "ToggleButton.insets",            zeroInsets,
      "ToggleButton.pressedColor",      table.getColor("controlShadow"),
      "ToggleButton.roundness",         defaultRoundness,
      "ToggleButton.selectedColor",     table.getColor("controlDkShadow"),

      "ToolTip.font",                   smallFont,
      "ToolTip.background",             table.getColor("controlDkShadow"),
      "ToolTip.foreground",             table.getColor("control"),
      "ToolTip.border",                 lightLineBorder,

      "Tree.background",                colors.getVeryDarkShade(),
      "Tree.closedIcon",                hsbIcon("icon_square_small.gif", table.getColor("controlShadow")),
      "Tree.collapsedIcon",             hsbIcon("tree_expand_icon.gif"),
      "Tree.editorBorder",              null,
      "Tree.expandedIcon",              hsbIcon("tree_collapse_icon.gif"),
      "Tree.font",                      smallFont,
      "Tree.foreground",                table.getColor("controlShadow"),
      "Tree.hash",                      table.getColor("controlDkShadow"),
      "Tree.leafIcon",                  hsbIcon("icon_hole_small.gif", table.getColor("controlShadow")),
      "Tree.openIcon",                  hsbIcon("icon_square_small.gif", table.getColor("controlShadow")),
	    "Tree.rowHeight",                 new Integer(20),
      "Tree.scrollsOnExpand",           new Boolean(false),
      "Tree.selectSound",               uiSound("8-3.wav"),
      "Tree.selectionBackground",       table.getColor("controlShadow"),
      "Tree.selectionBorderColor",      table.getColor("controlDkShadow"),
      "Tree.selectionForeground",       table.getColor("controlHighlight"),
      "Tree.textBackground",            colors.getVeryDarkShade(),
      "Tree.textForeground",            table.getColor("controlShadow"),

      "Viewport.background",            colors.getVeryDarkShade(),
    };

    table.putDefaults( defaults );
  }

  protected void initSystemColorDefaults(UIDefaults table)
  {
    BUIColorScheme colors = BGlobal.colors;

    String[] defaultSystemColors =
    {
                    "desktop", colors.getDarkestStr(), /* Color of the desktop background */
	            "activeCaption", colors.getBlackStr(),   /* Color for captions (title bars) when they are active. */
          "activeCaptionText", colors.getBlackStr(),   /* Text color for text in captions (title bars). */
        "activeCaptionBorder", colors.getBlackStr(),   /* Border color for caption (title bar) window borders. */
            "inactiveCaption", colors.getBlackStr(),   /* Color for captions (title bars) when not active. */
        "inactiveCaptionText", colors.getBlackStr(),   /* Text color for text in inactive captions (title bars). */
      "inactiveCaptionBorder", colors.getBlackStr(),   /* Border color for inactive caption (title bar) window borders. */
          	         "window", colors.getBlackStr(),   /* Default color for the interior of windows */
          	   "windowBorder", colors.getBlackStr(),   /* ??? */
          	     "windowText", colors.getWhiteStr(),   /* ??? */
                		   "menu", colors.getBlackStr(),   /* Background color for menus */
          	       "menuText", colors.getBlackStr(),   /* Text color for menus  */
                		   "text", colors.getVeryDarkStr(),   /* Text background color */
          	       "textText", colors.getWhiteStr(),   /* Text foreground color */
          	  "textHighlight", colors.getMidStr(),     /* Text background color when selected */
          "textHighlightText", colors.getWhiteStr(),   /* Text color when selected */
           "textInactiveText", colors.getLightStr(),   /* Text color when disabled */
            "controlDkShadow", colors.getDarkerStr(),  /* Dark shadow color for controls */
          	  "controlShadow", colors.getDarkStr(),    /* Shadow color for controls */
	                  "control", colors.getMidStr(),     /* Default color for controls (buttons, sliders, etc) */
           "controlHighlight", colors.getLightStr(),   /* Specular highlight (opposite of the shadow) */
         "controlLtHighlight", colors.getLighterStr(), /* Highlight color for controls */
                "controlText", colors.getMidStr(),     /* Default color for text in controls */
          	      "scrollbar", colors.getDarkestStr(), /* Scrollbar background (usually the "track") */
                		   "info", colors.getDarkStr(),    /* ??? */
          	       "infoText", colors.getWhiteStr(),   /* ??? */
	  };

    loadSystemColors(table, defaultSystemColors, false);
  }

  
  public static LazyHSBImageIcon hsbIcon(String name)
    { return new LazyHSBImageIcon(name); }

  public static LazyHSBImageIcon hsbIcon(String name, Color c)
    { return new LazyHSBImageIcon(name, c); }

  public static LazyHSBCheckBoxIcon checkBoxIcon(String name, int style)
    { return new LazyHSBCheckBoxIcon(name, style); }

  public static BSound uiSound(String name)
    { return BSoundSystem.getUISound(name); }


  public UIDefaults.LazyInputMap getFieldInputMap()
  {
    return new UIDefaults.LazyInputMap(new Object[]
    {
			   "ctrl C", DefaultEditorKit.copyAction,
			   "ctrl V", DefaultEditorKit.pasteAction,
			   "ctrl X", DefaultEditorKit.cutAction,
			     "COPY", DefaultEditorKit.copyAction,
			    "PASTE", DefaultEditorKit.pasteAction,
			      "CUT", DefaultEditorKit.cutAction,
		       "shift LEFT", DefaultEditorKit.selectionBackwardAction,
                    "shift KP_LEFT", DefaultEditorKit.selectionBackwardAction,
		      "shift RIGHT", DefaultEditorKit.selectionForwardAction,
		   "shift KP_RIGHT", DefaultEditorKit.selectionForwardAction,
			"ctrl LEFT", DefaultEditorKit.previousWordAction,
		     "ctrl KP_LEFT", DefaultEditorKit.previousWordAction,
		       "ctrl RIGHT", DefaultEditorKit.nextWordAction,
		    "ctrl KP_RIGHT", DefaultEditorKit.nextWordAction,
		  "ctrl shift LEFT", DefaultEditorKit.selectionPreviousWordAction,
	       "ctrl shift KP_LEFT", DefaultEditorKit.selectionPreviousWordAction,
		 "ctrl shift RIGHT", DefaultEditorKit.selectionNextWordAction,
	      "ctrl shift KP_RIGHT", DefaultEditorKit.selectionNextWordAction,
			   "ctrl A", DefaultEditorKit.selectAllAction,
			     "HOME", DefaultEditorKit.beginLineAction,
			      "END", DefaultEditorKit.endLineAction,
		       "shift HOME", DefaultEditorKit.selectionBeginLineAction,
		        "shift END", DefaultEditorKit.selectionEndLineAction,
		       "typed \010", DefaultEditorKit.deletePrevCharAction,
                           "DELETE", DefaultEditorKit.deleteNextCharAction,
                            "RIGHT", DefaultEditorKit.forwardAction,
                             "LEFT", DefaultEditorKit.backwardAction,
                         "KP_RIGHT", DefaultEditorKit.forwardAction,
                          "KP_LEFT", DefaultEditorKit.backwardAction,
			    "ENTER", JTextField.notifyAction,
		  "ctrl BACK_SLASH", "unselect"/*DefaultEditorKit.unselectAction*/,
                   "control shift O", "toggle-componentOrientation"/*DefaultEditorKit.toggleComponentOrientation*/
	  });
  }

  public UIDefaults.LazyInputMap getMultiLineInputMap()
  {
	  return new UIDefaults.LazyInputMap(new Object[]
    {
			   "ctrl C", DefaultEditorKit.copyAction,
			   "ctrl V", DefaultEditorKit.pasteAction,
			   "ctrl X", DefaultEditorKit.cutAction,
			     "COPY", DefaultEditorKit.copyAction,
			    "PASTE", DefaultEditorKit.pasteAction,
			      "CUT", DefaultEditorKit.cutAction,
		       "shift LEFT", DefaultEditorKit.selectionBackwardAction,
                    "shift KP_LEFT", DefaultEditorKit.selectionBackwardAction,
		      "shift RIGHT", DefaultEditorKit.selectionForwardAction,
		   "shift KP_RIGHT", DefaultEditorKit.selectionForwardAction,
			"ctrl LEFT", DefaultEditorKit.previousWordAction,
		     "ctrl KP_LEFT", DefaultEditorKit.previousWordAction,
		       "ctrl RIGHT", DefaultEditorKit.nextWordAction,
		    "ctrl KP_RIGHT", DefaultEditorKit.nextWordAction,
		  "ctrl shift LEFT", DefaultEditorKit.selectionPreviousWordAction,
	       "ctrl shift KP_LEFT", DefaultEditorKit.selectionPreviousWordAction,
		 "ctrl shift RIGHT", DefaultEditorKit.selectionNextWordAction,
	      "ctrl shift KP_RIGHT", DefaultEditorKit.selectionNextWordAction,
			   "ctrl A", DefaultEditorKit.selectAllAction,
			     "HOME", DefaultEditorKit.beginLineAction,
			      "END", DefaultEditorKit.endLineAction,
		       "shift HOME", DefaultEditorKit.selectionBeginLineAction,
		        "shift END", DefaultEditorKit.selectionEndLineAction,

			       "UP", DefaultEditorKit.upAction,
			    "KP_UP", DefaultEditorKit.upAction,
			     "DOWN", DefaultEditorKit.downAction,
			  "KP_DOWN", DefaultEditorKit.downAction,
			  "PAGE_UP", DefaultEditorKit.pageUpAction,
			"PAGE_DOWN", DefaultEditorKit.pageDownAction,
		    "shift PAGE_UP", "selection-page-up",
 	          "shift PAGE_DOWN", "selection-page-down",
	       "ctrl shift PAGE_UP", "selection-page-left",
 	     "ctrl shift PAGE_DOWN", "selection-page-right",
			 "shift UP", DefaultEditorKit.selectionUpAction,
		      "shift KP_UP", DefaultEditorKit.selectionUpAction,
		       "shift DOWN", DefaultEditorKit.selectionDownAction,
		    "shift KP_DOWN", DefaultEditorKit.selectionDownAction,
			    "ENTER", DefaultEditorKit.insertBreakAction,
		       "typed \010", DefaultEditorKit.deletePrevCharAction,
                           "DELETE", DefaultEditorKit.deleteNextCharAction,
                            "RIGHT", DefaultEditorKit.forwardAction,
                             "LEFT", DefaultEditorKit.backwardAction,
                         "KP_RIGHT", DefaultEditorKit.forwardAction,
                          "KP_LEFT", DefaultEditorKit.backwardAction,
			      "TAB", DefaultEditorKit.insertTabAction,
		  "ctrl BACK_SLASH", "unselect"/*DefaultEditorKit.unselectAction*/,
			"ctrl HOME", DefaultEditorKit.beginAction,
			 "ctrl END", DefaultEditorKit.endAction,
		  "ctrl shift HOME", DefaultEditorKit.selectionBeginAction,
		   "ctrl shift END", DefaultEditorKit.selectionEndAction,
                           "ctrl T", "next-link-action",
                     "ctrl shift T", "previous-link-action",
                       "ctrl SPACE", "activate-link-action",
                   "control shift O", "toggle-componentOrientation"/*DefaultEditorKit.toggleComponentOrientation*/
	  });
  }

}

