package roboforge.ui.fusion;

import roboforge.BDebug;

import java.awt.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

import roboforge.ui.components.BTabbedPane;


public class FusionTabbedPaneUI extends BasicTabbedPaneUI
{

  protected static final RenderingHints smooth = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );

  protected static final RenderingHints jaggies = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_OFF );


  protected static int TAB_BORDER_INSET = 5;

  protected Color selectedTabColor = null;


  public static ComponentUI createUI(JComponent c)
    { return new FusionTabbedPaneUI(); }


  public void installDefaults()
  {
    super.installDefaults();

    String pp = "TabbedPane.";
    selectedTabColor = UIManager.getColor(pp+ "selectedTabColor");
  }


  protected void paintFocusIndicator(
    Graphics g, int tabPlacement,
    Rectangle[] rects, int tabIndex,
    Rectangle iconRect, Rectangle textRect,
    boolean isSelected)
  {
  }

  protected void paintTabBackground
  (
    Graphics g,
    int tabPlacement,
    int tabIndex,
    int x, int y, int w, int h,
    boolean isSelected
  )
  {
    // TODO: Assumes placement = top.

    if (isSelected)
    {
      int topL = x+ TAB_BORDER_INSET;
      int topR = x+ w- TAB_BORDER_INSET;
      int bot  = y+ h;

      Polygon p = new Polygon();

      p.addPoint(x, bot);
      p.addPoint(topL, y);
      p.addPoint(topR, y);
      p.addPoint(x+w-1, bot);

      g.setColor(selectedTabColor);
      g.fillPolygon(p);
    }
  }

  protected void paintTabBorder
  (
    Graphics g,
    int tabPlacement,
    int tabIndex,
    int x, int y, int w, int h,
    boolean isSelected
  )
  {
    Graphics2D g2D = (Graphics2D) g;
    g2D.addRenderingHints(smooth);

    // TODO: Assumes placement = top.
    g.setColor(lightHighlight);

    int topL = x+ TAB_BORDER_INSET;
    int topR = x+ w- TAB_BORDER_INSET;
    int bot  = y+ h-1;

    // Left side
    g.drawLine(x, bot, topL, y);

    // Top
    g.drawLine(topL, y, topR, y);

    // Right side
    g.drawLine(topR, y, x+w-1, bot);
    
    g2D.addRenderingHints(jaggies);
  }

  protected void paintContentBorder(Graphics g, int tabPlacement, int selectedIndex)
  {
    int width = tabPane.getWidth();
    int height = tabPane.getHeight();
    Insets insets = tabPane.getInsets();

    int x = insets.left;
    int y = insets.top;
    int w = width - insets.right - insets.left;
    int h = height - insets.top - insets.bottom;

    // TOP tab placement.
    y += calculateTabAreaHeight(tabPlacement, runCount, maxTabHeight);
    h -= (y - insets.top);

    paintContentBorderTopEdge(g, tabPlacement, selectedIndex, x, y, w, h);
    paintContentBorderLeftEdge(g, tabPlacement, selectedIndex, x, y, w, h);
    paintContentBorderBottomEdge(g, tabPlacement, selectedIndex, x, y, w, h);
    paintContentBorderRightEdge(g, tabPlacement, selectedIndex, x, y, w, h);
  }

  protected void paintContentBorderTopEdge(
    Graphics g, int tabPlacement,
    int selectedIndex,
    int x, int y, int w, int h)
  {
    g.setColor(lightHighlight);
    g.drawLine(x, y, x+w-2, y);

    /*
    Rectangle selRect = rects[selectedIndex];

    g.drawLine(x, y, selRect.x - 1, y);

    if (selRect.x + selRect.width < x+w-2)
      g.drawLine(selRect.x + selRect.width, y, x+w-2, y);
    else
      g.drawLine(x+w-2, y, x+w-2, y);
    */

    
  }

  protected void paintContentBorderLeftEdge(
    Graphics g, int tabPlacement,
    int selectedIndex,
    int x, int y, int w, int h)
  {
    // g.setColor(lightHighlight);
    // g.drawLine(x, y, x, y+h-2);
  }

  protected void paintContentBorderBottomEdge(
    Graphics g, int tabPlacement,
    int selectedIndex,
    int x, int y, int w, int h)
  {
    // g.setColor(lightHighlight);
    // g.drawLine(x, y+h-1, x+w-1, y+h-1);
  }

  protected void paintContentBorderRightEdge(
    Graphics g, int tabPlacement,
    int selectedIndex,
    int x, int y, int w, int h)
  {
    // g.setColor(lightHighlight);
    // g.drawLine(x+w-1, y, x+w-1, y+h-1);
  }


  protected int getTabLabelShiftY
  (
    int tabPlacement, 
    int tabIndex, 
    boolean isSelected
  ) 
  {
    Rectangle tabRect = rects[tabIndex];
    int nudge = 0;
      
    switch(tabPlacement) 
    {
      case BOTTOM:
        nudge = -1;
        break;
      case LEFT:
      case RIGHT:
        nudge = tabRect.height % 2;
        break;
      case TOP:
      default:
        nudge = 1;
    }
    return nudge;
  }
  
}
