package roboforge.ui.modes;

import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.lang.Math.*;

import roboforge.*;
import roboforge.ui.*;


public class BModeArenaCameraMove extends BModeArena
{

  private BConsoleCanvas canvas;
  private TransformGroup tg;
  private double arenaSize;
  private double minY, maxY;

  // working variables
  private Transform3D  workingTran = new Transform3D();
  private Transform3D tranXZ = new Transform3D();

  private Vector3d translation  = new Vector3d();
  private Vector3d oldTran  = new Vector3d();
  private Vector3d zeroTranslation  = new Vector3d(0,0,0);

  double x_factor = 1;
  double y_factor = 1;


  private Vector3d position;
  private Transform3D transform = new Transform3D();
  private Transform3D rotYTran = new Transform3D();
  private Transform3D rotXTran = new Transform3D();
  private Transform3D totalYTran = new Transform3D();
  private Transform3D oldTransform = new Transform3D();

  private Vector3d oldPos = new Vector3d();
  private Vector3d zeroVector = new Vector3d();
  private Vector3d yAngleTester = new Vector3d();
  private Vector3d yAnglePlane = new Vector3d(0,0,1);

  private static double MAXUPDOWN = BPhysics.HALF_PI - ( BPhysics.QUARTER_PI/  2.0);
  private static double rot_factor = .0035;

  private double totalX, totalY = 0;
  private AxisAngle4d xRot = new AxisAngle4d();

  private TransformGroup oldTransformGroup;

 // private int totalXAngle = 0;
 // private int totalYAngle = 0;

  private static double inset =0.03;



  public BModeArenaCameraMove(TransformGroup tg, BConsoleCanvas canvas, BArena arena, double minY, double maxY)
  {
    super();

    this.tg = tg;
    this.canvas = canvas;

    if (arena == null)
      this.arenaSize = 50;
    else
      this.arenaSize = arena.getSize() /2.0;

    this.arenaSize = this.arenaSize - (this.arenaSize * inset );


    this.minY = minY;
    this.maxY = maxY;
  }




  public void onActivate()
  {
    super.onActivate();
    //setFOV(defaultFOV);
    //tg.getTransform(workingTran);
    if(oldTransformGroup != null)
    {
      Transform3D temp = new Transform3D();
      oldTransformGroup.getTransform(temp);
      tg.setTransform(temp);
    }
    capAndApplyPosition (workingTran);
//    rotateCamera(totalXAngle, totalYAngle);
  }

  public void onDeactivate()
  {
    super.onDeactivate();
    Transform3D temp = new Transform3D();
    tg.getTransform(temp);
    oldTransformGroup = new TransformGroup(temp);
  }

  public void animationStep(double t, double dt)
  {
  }


      //private working members
  private Vector3d cross = new Vector3d();
  private Vector3d xAxis = new Vector3d(1.0,0,0);
  private Vector3d tester  = new Vector3d(1.0,0,0);

  private void rotateCamera(int dx, int dy)
  {
        if (BGlobal.INVERT_PLAYER_Y)
          dy = -dy;
          
        double x_angle =  -dy * rot_factor;
        double y_angle =  -dx * rot_factor;

        //totalXAngle += -dx;
        //totalYAngle += -dy;

        tg.getTransform(transform);
        transform.get(oldPos);

        // nullify translation
        transform.setTranslation(zeroVector);

        // Y Angle
        xAxis.set(1.0,0,0);
        tester.set(1.0,0,0);

        transform.transform(tester);
        tester.y = 0;
        double yAngle = tester.angle(xAxis);

        cross.cross(xAxis, tester);

        if (cross.y < 0)
          yAngle = -yAngle;

        totalYTran.rotY(yAngle);

        xRot.set(xAxis, x_angle);
        rotXTran.set(xRot);

        oldTransform.set(transform);
        transform.mul( rotXTran);

         // Check Max upDown
        yAngleTester.set(0,0,1);
        transform.transform(yAngleTester);
        yAnglePlane.set(yAngleTester);
        yAngleTester.y = 0;
        double updownAngle =  yAngleTester.angle(yAnglePlane) ;

        if (updownAngle > MAXUPDOWN)
          transform.set(oldTransform);


        // rot Y
        rotXTran.rotY(y_angle);
        transform.mul(rotXTran, transform);
        transform.setTranslation(oldPos);

        tg.setTransform(transform);
        moveCamera(0,0);
        //capAndApplyPosition (transform);
  }


  public void moveCamera(int dx, int dy)
  {
      tg.getTransform(workingTran);

      //set translation
      translation.x = dx * x_factor;
      translation.y = 0;
      translation.z = dy * y_factor;

      // calculating translation
      tranXZ.set(translation);
      workingTran.mul(workingTran, tranXZ);
       /*
      // setRotation
      workingTran.get(oldTran);

      workingTran.setTranslation(zeroTranslation);
      tranXZ.rotY(-dx * x_factor);
      workingTran.mul(tranXZ);
      workingTran.setTranslation(oldTran); */

      capAndApplyPosition (workingTran);
  }


  public void mouseDragged(MouseEvent e)
  {
    super.mouseDragged(e);

    if (isMouseActive)
    {
      if (e.isShiftDown())
        moveCamera(dx, dy);
      else
        rotateCamera(dx, dy);
    }
  }


  protected void incrementCameraKey(InputEvent e, int dx, int dy)
  {
    if (e.isShiftDown())
      rotateCamera(dx, dy);
    else
      moveCamera(dx, dy);
  }


  protected static final int KEY_DX = 10;
  protected static final int KEY_DY = 10;

  public void keyUpPressed(KeyEvent e)
    { incrementCameraKey(e, 0, -KEY_DY); }

  public void keyDownPressed(KeyEvent e)
    { incrementCameraKey(e, 0, +KEY_DY); }

  public void keyLeftPressed(KeyEvent e)
    { incrementCameraKey(e, -KEY_DX, 0); }

  public void keyRightPressed(KeyEvent e)
    { incrementCameraKey(e, +KEY_DX, 0); }





    private void capAndApplyPosition (Transform3D position)
    {
       // capping new position
      position.get(translation);

      translation.y = java.lang.Math.max(translation.y, minY);
      translation.y = java.lang.Math.min(translation.y, maxY);


      double distFromCentre = translation.length();
      if (distFromCentre * BGlobal.CANVAS_TO_WORLD_SCALE > arenaSize)
      {
        translation.normalize();
        translation.scale(arenaSize * BGlobal.WORLD_TO_CANVAS_SCALE);
      }

      position.setTranslation(translation);

      canvas.updateView(position);
      //tg.setTransform(position );

    }





}

/*

public class BModeArenaFixedCamera extends BModeFOVZoom
{
  private TransformGroup tg;


  private Vector3d position;
  private Transform3D transform = new Transform3D();
  private Transform3D rotYTran = new Transform3D();
  private Transform3D rotXTran = new Transform3D();
  private Transform3D totalYTran = new Transform3D();
  private Transform3D oldTransform = new Transform3D();

  private Vector3d oldPos = new Vector3d();
  private Vector3d zeroVector = new Vector3d();
  private Vector3d yAngleTester = new Vector3d();
  private Vector3d yAnglePlane = new Vector3d(0,0,1);

  private static double MAXUPDOWN = BPhysics.HALF_PI - ( BPhysics.QUARTER_PI/  2.0);
  private static double rot_factor = .0015;

  private double totalX, totalY = 0;
  private AxisAngle4d xRot = new AxisAngle4d();

  public BModeArenaFixedCamera(TransformGroup tg, Vector3d pos, BArenaCanvas canvas)
  {
    super(canvas);

    this.tg = tg;
    this.position = pos;

  }

  public void setPosition(double x,double y,double z)
  {
    if (position == null)
      position = new Vector3d();

    position.x = x;
    position.y = y;
    position.z = z;

    setPosition();

  }

  public void setPosition()
  {
    tg.getTransform(transform);
    transform.setTranslation(position);
    tg.setTransform(transform);

  }

  public void onActivate()
  {
    super.onActivate();
    setPosition();
    canvas.lookAt(true);
  }



  public void animate()
  // do a slow automatic arena animation
  {
    if (!active)
    {
      long time = System.currentTimeMillis();
      long deltaTime =  time -lastTime;
      int moveDelta = (int)java.lang.Math.min( deltaTime /10, 5);
      lastTime = time;
      moveCamera(moveDelta,0);


    }
  }


    //private working members
  private Vector3d cross = new Vector3d();
  private Vector3d xAxis = new Vector3d(1.0,0,0);
  private Vector3d tester  = new Vector3d(1.0,0,0);

  private void moveCamera(int dx, int dy)
  {
        double x_angle =  -dy * rot_factor;
        double y_angle =  -dx * rot_factor;

        tg.getTransform(transform);
        transform.get(oldPos);

        // nullify translation
        transform.setTranslation(zeroVector);

        // Y Angle
        xAxis.set(1.0,0,0);
        tester.set(1.0,0,0);

        transform.transform(tester);
        tester.y = 0;
        double yAngle = tester.angle(xAxis);

        cross.cross(xAxis, tester);

        if (cross.y < 0)
          yAngle = -yAngle;

        totalYTran.rotY(yAngle);

        xRot.set(xAxis, x_angle);
        rotXTran.set(xRot);

        oldTransform.set(transform);
        transform.mul( rotXTran);

         // Check Max upDown
        yAngleTester.set(0,0,1);
        transform.transform(yAngleTester);
        yAnglePlane.set(yAngleTester);
        yAngleTester.y = 0;
        double updownAngle =  yAngleTester.angle(yAnglePlane) ;

        if (updownAngle > MAXUPDOWN)
          transform.set(oldTransform);


        // rot Y
        rotXTran.rotY(y_angle);
        transform.mul(rotXTran, transform);
        transform.setTranslation(oldPos);

        tg.setTransform(transform);
  }





  public void mouseDragged(MouseEvent e)
  {
    super.mouseDragged(e);

    if (active)
    {
      if (e.isShiftDown() ) // for zoom
      {
        // handled in the super()
      }
      else
      {

        if (BGlobal.INVERT_PLAYER_Y)
          dy = -dy;

        moveCamera(dx, dy);
      }

    }
  }


}
*/



