package roboforge.ui.modes;

import roboforge.BGlobal;
import roboforge.ui.*;
import roboforge.ui.components.BUIResources;
import roboforge.util.graphics.*;

import java.awt.Cursor;
import java.awt.event.MouseEvent;

import javax.media.j3d.*;
import javax.vecmath.*;


public class BModeViewPan extends BModeCanvas
{

  // Class variables...
  // -------------------------------------------------------------

  private static final double baseWeightFactor   = 0.75;
  private static final double standardZDistance  = 300.0;
  private static final double logBias            = 1.0;

  private static final Cursor CURSOR =
    BGraphics.getCursor("pan_cursor.gif",  15, 15);


  // Internal variables.
  // -------------------------------------------------------------

  private double totalXPan  = 0.0;
  private double totalYPan  = 0.0;

  protected Vector3d pan       = null;
  protected Transform3D translation = new Transform3D();
  protected TransformGroup translationGroup;

  protected double weightFactor   = 0.0;


  // Constructor.
  // -------------------------------------------------------------

  public BModeViewPan(B3DCanvas canvas)
  {
    super(canvas);

    this.cursor = CURSOR;
    translationGroup = canvas.getViewTranslateTG();
  }


  // Activation methods.
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();

    // Remember the bot's position...
    pan = canvas.getViewTranslation();
  }

  public void onDeactivate()
  {
    // Tell the bot to remember this pan [i.e. translation]...
    canvas.setViewTranslation(pan);

    super.onDeactivate();
  }


  // Mouse-event methods.
  // -------------------------------------------------------------

  public void mouseDragged(MouseEvent e)
  {
    super.mouseDragged(e);

    weightFactor = getWeightFactor(pan.z);

		totalXPan = (dx * weightFactor);
		totalYPan = (dy * weightFactor);

    // Pan by the prescribed amount.
    pan.x -= totalXPan;
    pan.y += totalYPan;

    // Update transform group.
    translation.setTranslation(pan);
    translationGroup.setTransform(translation);

    // Tell the bot to remember this pan [i.e. translation]...
    canvas.setViewTranslation(pan);
  }

  protected double getWeightFactor(double z)
    { return baseWeightFactor * Math.log((z / standardZDistance) + logBias); }

}
