package roboforge.ui.modes;

import roboforge.BGlobal;
import roboforge.BPhysics;
import roboforge.ui.*;
import roboforge.ui.components.BUIResources;
import roboforge.util.graphics.*;

import java.awt.Cursor;
import java.awt.event.MouseEvent;

import javax.media.j3d.*;
import javax.vecmath.*;



public class BModeViewRot extends BModeCanvas
{

  // Class variables...
  // -------------------------------------------------------------

  private static final Cursor CURSOR =
    BGraphics.getCursor("rotb_cursor.gif",  15, 15);


  // Internal variables.
  // -------------------------------------------------------------

  private double factor = 0.01;

  private double totalXAngle  = 0.0;
  private double totalYAngle  = 0.0;

  protected Transform3D    transformX = new Transform3D();
  protected Transform3D    transformY = new Transform3D();
  protected TransformGroup rotAboutX;
  protected TransformGroup rotAboutY;

  Vector3d r = new Vector3d();

  protected double TWO_PI = BPhysics.PI * 2.0;
  

  // Constructor.
  // -------------------------------------------------------------

  public BModeViewRot(B3DCanvas canvas)
  {
    super(canvas);
    cursor = CURSOR;

    rotAboutY = canvas.getViewRotateYTG();
    rotAboutX = canvas.getViewRotateXTG();
  }

  public BModeViewRot(B3DCanvas canvas, double rotFactor)
  {
    this(canvas);
    this.factor = rotFactor;
  }

  // Activation methods.
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();

    // Remember the bot's orientation...
    Vector3d r = canvas.getViewRotation();

    totalXAngle = r.x;
    totalYAngle = r.y;
  }

  public void onDeactivate()
  {
    // Tell the bot to remember this angle...
    r.set(totalXAngle, totalYAngle, 0.0);

    if (canvas != null)
      canvas.setViewRotation(r);

    super.onDeactivate();
  }



  // Mouse-event methods.
  // -------------------------------------------------------------

  public void mouseDragged(MouseEvent e)
  {
    super.mouseDragged(e);

		totalXAngle -= (dy * factor);
		totalYAngle -= (dx * factor);

    // Clamp rotation about X to +90, -90 degrees.
    if (totalXAngle < -BPhysics.HALF_PI)
      totalXAngle = -BPhysics.HALF_PI;

    else if (totalXAngle > BPhysics.HALF_PI)
      totalXAngle = BPhysics.HALF_PI;

    // Keep Y rotation in range [0..2pi].
    if (totalYAngle < 0)
      totalYAngle += TWO_PI;

    else if (totalYAngle > TWO_PI)
      totalYAngle -= TWO_PI;

    // Rotate object.
    transformY.rotY(totalYAngle);
    transformX.rotX(totalXAngle);

    rotAboutX.setTransform(transformX);
    rotAboutY.setTransform(transformY);

    // inform the canvas  (This if for the moves panel)
    r.set(totalXAngle, totalYAngle, 0.0);

    if (canvas != null)
      canvas.setViewRotation(r);
  }

}
