package roboforge.ui.net;

import roboforge.ui.components.*;
import roboforge.ui.components.tree.*;

import roboforge.net.data.*;
import roboforge.util.data.*;

import java.util.*;
import javax.swing.tree.*;


public class BEntryTree extends BTree implements BDatabase
{

  protected BDataNode root = 
    new BDataNode("Open Tournaments");
  

  protected DefaultTreeModel  model;
  protected TournamentStore  tournamentStore;
  protected EntryStore       entryStore;
  
    
  public BEntryTree()
    { this(null, null); }
  

  public BEntryTree(BUIResources res, String key)
  {
    super(res, key);

    model = new DefaultTreeModel(root);
    setModel(model);

    tournamentStore = 
      new TournamentStore();
    
    entryStore =
      new EntryStore();
    
    BDelegatingCellRenderer renderer =
      new BDelegatingCellRenderer();

    setCellRenderer(renderer);
  }


  public void expand()
  {
    expandChildren((DefaultMutableTreeNode)model.getRoot() );
  }


  public void expandChildren(DefaultMutableTreeNode node)
  {

    Enumeration e = node.depthFirstEnumeration();
    TreePath rootPath = new TreePath( node );

    while (e.hasMoreElements() )
    {
      DefaultMutableTreeNode currentN = (DefaultMutableTreeNode) e.nextElement();

      if (currentN.isLeaf() )
      {
        TreeNode[] path = currentN.getPath();

        makeVisible( new TreePath(path ));

      }

   

    }

  }




   public void reload(BData d)
  {
    tournamentStore.clear();
    d.store(this);
    model.reload();
    expand();
  }
  
  
  public BDataStore get(int type)
  {
    switch (type)
    {
      case BDataTypes.TOURNAMENT:
        return tournamentStore;
        
      case BDataTypes.TOURNAMENT_ENTRY:
        return entryStore;
        
      default:
        return null;
    }
  }
    


  protected class TournamentStore implements BDataStore
  {
    protected HashMap tournaments
      = new HashMap();
    
    public void clear()
    { 
      tournaments.clear();
      root.removeAllChildren();
    }
    
    public void put(BDataAtom datum)
    {
      BTournamentData d = 
        (BTournamentData) datum;
      
      BDataNode node = 
        new BDataNode(d);
      
      tournaments.put(d.getIDLong(), node);
      
      // root.add(node);
      model.insertNodeInto(node, root, root.getChildCount());
      // expandNode(node);
    }
    
    public BDataNode get(Long id)
      { return (BDataNode) tournaments.get(id); }
      
  }
    

  protected class EntryStore implements BDataStore
  {
    protected HashMap entries
      = new HashMap();

    public void put(BDataAtom datum)
    {
      BTournamentEntryData d = 
        (BTournamentEntryData) datum;
      
      BDataNode entry = 
        new BDataNode(d);
      
      BDataNode tourn =
        tournamentStore.get(d.tournamentID);          

      if (tourn != null)
      {
        // tourn.add(entry);
        
        model.insertNodeInto(entry, tourn, tourn.getChildCount());
        expandNode(entry);
        
        entries.put(d.getIDLong(), entry);
      }
    }
    
    public void remove(Long ID)
    {
      BDataNode node =
        (BDataNode) entries.remove(ID);
      
      if (node != null)
        model.removeNodeFromParent(node);
    }
  }

  
  public void removeEntry(Long ID)
    { entryStore.remove(ID); }

}