package roboforge.ui.net;

import roboforge.*;

import roboforge.BProgressMonitorable;
import roboforge.ui.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.util.graphics.*;
import roboforge.util.io.*;
import roboforge.util.text.BStrings;
import roboforge.util.data.*;

import roboforge.net.*;
import roboforge.net.client.*;
import roboforge.net.data.*;
import roboforge.net.services.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;
import java.util.Calendar;
import java.util.Enumeration;
import java.text.DateFormat;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.table.*;

import roboforge.util.processors.*;

public class BHistoryPanel extends BOnlinePanel
{

  // Internal variables...
  // -------------------------------------------------------------

  protected boolean             isRefreshRequired;
  protected DefaultTreeModel    treeModel;
  protected TreeSelectionModel  selectionModel;
  protected SelectionListener   selectionListener;

  protected BFightData          fight;
  protected BFightDataNode      fightNode;
  protected TournamentDetails   details     = new TournamentDetails();
  protected RankingList         proRank;
  protected RankingList         amRank;

  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "treeTitle.text",     "tournament results view",

    "download.name",      "Download",
    "download.icon",      BIconCache.getHSB("play_icon.gif"),
    "download.tooltip",   "Download and play selected battle.",

    "refresh.name",       "Refresh",
    "refresh.icon",       BIconCache.getHSB("net_refresh_icon.gif"),
    "refresh.tooltip",    "Refresh Tournament view.",

    "play.name",          "Play",
    "play.icon",          BIconCache.getHSB("play_icon.gif"),
    "play.tooltip",       "Play selected battle.",

    "save.name",          "Save",
    "save.icon",          BIconCache.getHSB("save_icon.gif"),
    "save.tooltip",       "Save selected battle to archives.",

    "downSave.name",      "Download Save",
    "downSave.icon",      BIconCache.getHSB("save_icon.gif"),
    "downSave.tooltip",   "Download and save selected battle.",

    "cancel.name",        "Cancel",
    "cancel.icon",        BIconCache.getHSB("stop_icon.gif"),
    "cancel.tooltip",     "Cancel download",

    "stopRef.name",       "Cancel refresh",
    "stopRef.icon",       BIconCache.getHSB("stop_icon.gif"),
    "stopRef.tooltip",    "Cancel refresh",

    "nominate.name",      "Nominate",
    "nominate.icon",      BIconCache.getHSB("nominate_icon.gif"),
    "nominate.tooltip",   "Nominate featured fight.",

    "clearCache.name",    "Clear battle cache",
    "clearCache.icon",    BIconCache.getHSB("clear_cache_icon.gif"),
    "clearCache.tooltip", "Clear all downloaded battles.",
  });



  // Visual elements...
  // -------------------------------------------------------------

  protected BTitlebarLabel      treeTitle   = new BTitlebarLabel(res, "treeTitle");
  protected BHistoryTree        tree        = new BHistoryTree();
  protected BScrollPane         scroller    = new BScrollPane(tree);
  protected BActionButton       refresh     = new BActionButton();
  protected BActionButton       play        = new BActionButton();

  protected BActionButton       save        = new BActionButton();

  protected BActionButton       nominate    = new BActionButton();

  protected BActionButton       clearCache  = new BActionButton();

  // Actions...
  // -------------------------------------------------------------
  protected RefreshAction  refreshAction    = new RefreshAction(res, "refresh");
  protected DownloadAction downloadAction   = new DownloadAction(res, "download");
  protected PlayAction     playAction       = new PlayAction(res, "play");
  protected CancelAction   cancelAction     = new CancelAction(res, "cancel");
  protected StopRefreshAction stopRefAction = new StopRefreshAction(res, "stopRef");
  protected SaveAction     saveAction       = new SaveAction(res, "save");
  protected DownSaveAction downSaveAction   = new DownSaveAction(res, "downSave");
  protected NominateAction nominateAction   = new NominateAction(res, "nominate");

  protected ClearCacheAction clearCacheAction = new ClearCacheAction(res, "clearCache");


  protected BFightDataNode selectedFightNode;

  BPlayerRankingDialog rankingDialog;

  // Constructor.
  // -------------------------------------------------------------
  public BHistoryPanel(BTournamentScreen parent)
  {
    super(parent, "results");

    this.isRefreshRequired  = true;
    this.selectionModel     = tree.getSelectionModel();
    this.treeModel          = (DefaultTreeModel) tree.getModel();

    tree.setServer(server);

    layoutEnterChildren();
    configEnterChildren();
  }


  // Initialization helper methods...
  // -------------------------------------------------------------

  protected void layoutEnterChildren()
  {
    layoutToolbar();

    contentArea.addHGlued(treeTitle);
    contentArea.addStrut(8);
    contentArea.add(scroller);

    scroller.setViewportBorder(new EmptyBorder(10,10,10,10));

  }

  protected void layoutToolbar()
  {
    toolbar.add(play);
    toolbar.addSmallMargin();
    toolbar.add(save);
    toolbar.addSmallMargin();
    toolbar.add(nominate);
    toolbar.addSmallMargin();
    toolbar.add(refresh);
    toolbar.addGlue();
    toolbar.add(clearCache);
  }

  protected void configEnterChildren()
  {
    openToolPanel(details);
    selectToolPanel(details);
    
    selectionListener = new SelectionListener();
    selectionModel.addTreeSelectionListener(selectionListener);

    refresh.setAction(refreshAction);
    play.setAction(playAction);
    save.setAction(saveAction);
    nominate.setAction(nominateAction);
    clearCache.setAction(clearCacheAction);
    setActionStatus();
    clearCacheAction.setEnabled(false);
  }



  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();

    if (isRefreshRequired)
    {
      refreshAction.perform();
      proRank = new RankingList("pro rank");
      amRank = new RankingList("amateur rank");
      openToolPanel(proRank);
      openToolPanel(amRank);
      isRefreshRequired = false;
    }
  }

  public void onDeactivate()
  {


    super.onDeactivate();
  }

  // UI Listeners...
  // -------------------------------------------------------------
  protected class ClearCacheAction extends BAction
  {
    public ClearCacheAction(BUIResources res, String key)
    { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {
      BProcessor p = new BProcessor()
      {
        public void process()
        { checkClearCache(); }
      };
      p.start();
    }

    // pop up a dialog to make sure the user wants to clear the cache
    public void checkClearCache()
    {
      BPromptDialog clearCheck =
      BPromptDialog.createYesNoDialog("Clear battles?",
           "Are you sure you want to delete all downloaded battles?");

      BApplication.showDialog(clearCheck);

      boolean clearBattles = clearCheck.getResult();

      if(clearBattles)
      {
        parent.setBusyWorking(true);
        parent.setStatus("Clearing cache");
        BFileSystem.clearOldFiles(BGlobal.BATTLERECORDINGCACHEPATH, 0);

        // set all nodes to not downloaded
        Enumeration enum = ((DefaultMutableTreeNode)treeModel.getRoot()).breadthFirstEnumeration();
        while(enum.hasMoreElements())
        {
          BDataNode node = (BDataNode)enum.nextElement();
          if(node.isOfType(BDataTypes.FIGHT))
            ((BFightDataNode)node).setDownloaded(false);
        }

        parent.setStatus("Cache cleared");
        parent.setBusyWorking(false);
        setActionStatus();
      }
    }
  }

  protected class SaveAction extends BAction
  {
    public SaveAction(BUIResources res, String key)
    { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {

        BProcessor p = new BProcessor()
        {
          public void process()
          { downloadAction.archiveSave( null, false ); }
        };
        p.start();

    }
  }

  protected class DownSaveAction extends BAction
  {
    public DownSaveAction(BUIResources res, String key)
    { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {
      downloadAction.actionPerformed(e);
    }
  }

  protected class RefreshAction extends BThreadedAction
  {
    protected BMessage response;
    BTransmission transmission;
    protected boolean manualStop = false;

    public RefreshAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
    {
      parent.setStatus("Refreshing battle result listings...");

      downloadAction.setEnabled(false);
      playAction.setEnabled(false);
      downSaveAction.setEnabled(false);
      cancelAction.setEnabled(false);
      saveAction.setEnabled(false);
      nominateAction.setEnabled(false);
      clearCacheAction.setEnabled(false);
      tree.setEnabled(false);

      parent.setWorking(true);
      refresh.setAction(stopRefAction);
      stopRefAction.setEnabled(true);
    }

    public void threadedAction(ActionEvent e)
    {
      try
      {
        transmission = new BTransmission(
        new BRequest(server, "HIS"));

        parent.setMonitored(transmission);

        response = server.request(transmission);
      }
      catch (BNetException ex)
      {
        if(!manualStop)
          parent.setStatus("ERROR: "+ ex.getMessage());
        else
          parent.setStatus("Refresh terminated");
        response = null;
      }
      finally
      {
        parent.setMonitored(null);
      }
    }
    // terminate the refresh
    public void stop()
    {
      manualStop = true;          // the download has been interrupted by the user
      transmission.stop();        // stop the download thread
      tree.setEnabled(true);
      downloadAction.setEnabled(false);
      playAction.setEnabled(false);
      downSaveAction.setEnabled(false);
      cancelAction.setEnabled(false);
      saveAction.setEnabled(false);
      nominateAction.setEnabled(false);
      refresh.setAction(refreshAction);  // reset the play button action
    }

    // true if the refresh has stopped
    public boolean isStopped()
    {
      if (transmission != null)
        return transmission.isStopped();
      else
        return true;
    }

    public void swingCleanupAction(ActionEvent e)
    {
      proRank.refreshRankings();
      amRank.refreshRankings();

      if (response != null)
      {
        if (response.isAcknowledgement())
        {
          if (tree != null)
            tree.store(response.getData());
          if (parent != null)
            parent.setStatus("Battle results refreshed.");
        }
        else if (parent != null)
        {
          if(!manualStop)
            parent.setStatus("ERROR: "+ response);
          else
            parent.setStatus("Refresh terminated");
        }
      }

      if (parent != null)
        parent.setWorking(false);

      tree.setEnabled(true);
      refresh.setAction(refreshAction);  // reset the play button action
      setActionStatus();
      downloadAction.setEnabled(false);
      playAction.setEnabled(false);
      downSaveAction.setEnabled(false);
      cancelAction.setEnabled(false);
      saveAction.setEnabled(false);
      nominateAction.setEnabled(false);
    }
  }

  protected class DownloadAction extends BThreadedAction
  {
    protected BMessage response;
    protected BTransmission transmission;

    protected Object source;
    // was the download stopped by the user ?
    protected boolean manualStop = false;

    public DownloadAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
    {
      parent.setStatus("Downloading battle...");
      parent.setWorking(true);

      refresh.setEnabled(false);
      tree.setEnabled(false);

      manualStop = false;
      cancelAction.setEnabled(true);

      source = e.getSource();
    //  play.setAction(cancelAction);
      if(source == play)
        play.setAction(cancelAction);
      else
        save.setAction(cancelAction);

      playAction.setEnabled(false);
      downloadAction.setEnabled(false);
      downSaveAction.setEnabled(false);
      nominateAction.setEnabled(false);
      clearCacheAction.setEnabled(false);
    }

    public void threadedAction(ActionEvent e)
    {
      try{
      try
      {
        response = null;

        BRequest r = new BRequest(server, "GBA");
        r.setData(new BIdentData(fight.getID()));

        transmission = new BTransmission(r);
        parent.setMonitored(transmission);
        
        response = server.request(transmission);
      }
      catch (BNetException ex)
      {
        if(!manualStop)
          parent.setStatus("ERROR: " + ex.getMessage());
        else
          parent.setStatus("Download terminated.");
      }
      finally
        { parent.setMonitored(null); }

      if (response != null)
      {
        if (response.isAcknowledgement())
          saveBattle((BFightRecordData) response.getData());
        else
        {
          if(!manualStop)
            parent.setStatus("ERROR: " + response);
          else
            parent.setStatus("Download terminated.");

          // release UI

          parent.setBusyWorking(false);

          refresh.setEnabled(true);
          tree.setEnabled(true);

          setActionStatus();
          play.setAction(playAction);
          save.setAction(saveAction);


        }



      }
      }
      catch(Exception ex)
      {
        BDebug.print("BHistoryPanel " + ex.getMessage());
      }
    }

    public void swingCleanupAction(ActionEvent e)
    {
      if(parent != null)
      {
        parent.setBusyWorking(false);
        setActionStatus();
      }
    }

    protected void saveBattle(BFightRecordData data)
    {
      try
      {
        cancelAction.setEnabled(false);
        parent.setStatus("Saving battle to local cache...");

        parent.setBusyWorking(true);

        String filename =
          BGlobal.BATTLERECORDINGCACHEPATH+ data.getID();

        data.save(filename);
        System.gc();


        parent.setStatus("Battle download completed.");

        if(selectedFightNode != null)
        {
          Enumeration enum = ((DefaultMutableTreeNode)treeModel.getRoot()).breadthFirstEnumeration();

          while(enum.hasMoreElements())
          {
            BDataNode node = (BDataNode)enum.nextElement();
            if(node.isOfType(BDataTypes.FIGHT))
            {
              if(selectedFightNode.getID() == ((BFightDataNode)node).getID())
                ((BFightDataNode)node).setDownloaded(true);
            }
          }

          selectedFightNode.setDownloaded(true);
        }

        parent.setBusyWorking(false);
        //parent.setWorking(false);

        if(source == save)
          archiveSave(data, true);

        refresh.setEnabled(true);
        tree.setEnabled(true);

        setActionStatus();
        play.setAction(playAction);
        save.setAction(saveAction);

        if(source == play)
          play.doClick();
      }
      catch (Exception e)
      {
         if(!manualStop)
          parent.setStatus("ERROR: "+ e);
        else
          parent.setStatus("Download terminated.");
      }
    }

    // prompt the user to save the chosen battle to the recordings archive
    public void archiveSave(BFightRecordData data, boolean justDownloaded)
    {
      BFileDialog saveDlg;
      String filename;
      String shortFilename;

     // parent.setBusyWorking(false);

      saveDlg = BFileDialog.createSaveDialog("Save battle", "Save battle as:");

      parent.setStatus("Save battle to archives.");
      try
      {
        saveDlg.setOKDefault();
        saveDlg.setFilter(new BRoboforgeFilter(BGlobal.RECORDINGSUFFIX));
        saveDlg.setPath(BGlobal.BATTLERECORDINGSPATH);
        System.gc();
      }
      catch (Exception ex)
      {
        BApplication.notifyUser("Could not save battle.");
        parent.setStatus("Save failed.");
      }


      BApplication.showDialog(saveDlg);

      if(saveDlg.getResult())
      {
        filename = saveDlg.getSelectedFile();
        shortFilename = saveDlg.getSelectedName();
        // Check for user cancel...
        if (filename != null)
        {
          File f = new File(filename);
          if (f.exists())
          {
            BPromptDialog replaceDlg = BPromptDialog.createYesNoDialog(
            "replace battle?", "A battle named "+
            saveDlg.getSelectedName()+ " already exists. \n\n"+
            "Do you want to replace it with the current battle?");

            BApplication.showDialog(replaceDlg);
            if (!replaceDlg.getResult())
              filename = null;
          }
        }

        try
        {
          if (filename != null)
          {
            parent.setBusyWorking(true);
            //parent.setWorking(true);
            parent.setStatus("Saving battle to archives.");

            shortFilename = BStrings.convertToFilename(shortFilename);

            if(justDownloaded)
              data.save(BGlobal.BATTLERECORDINGSPATH + shortFilename);

            else
            {
              BFileSystem.copyFile(getBattleFile(), BGlobal.BATTLERECORDINGSPATH + shortFilename + ".rec");
              if(getBattleStatFile() != null)
                BFileSystem.copyFile(getBattleStatFile(), BGlobal.BATTLERECORDINGSPATH + shortFilename + ".sta");
            }

            parent.setStatus("Battle saved to archives.");
            parent.setBusyWorking(false);
            //parent.setWorking(false);
          }
        }
        catch(IOException ex)
        {
          parent.setStatus("Error saving battle to archives.");
          BDebug.print("BHistoryPanel - error saving file: " + ex);
          parent.setBusyWorking(false);
          //parent.setWorking(false);
        }
      }
    }

    public boolean isStopped()
    {
      if (transmission != null)
        return transmission.isStopped();
      else
        return true;
    }

    // Terminate the download
    public void stop()
    {
      manualStop = true;          // the download has been interrupted by the user
      transmission.stop();   // stop the download thread
      refresh.setEnabled(true);   // enable the previously disabled components
      tree.setEnabled(true);
      play.setAction(downloadAction);  // reset the play button action
    }
  }

  // action for cancelling a download that is currently in progress
  protected class CancelAction extends BThreadedAction
  {
    protected BPromptDialog confirmation
      = BPromptDialog.createYesNoDialog(
        "Cancel download?", "Do you want to cancel the current download?");

    protected boolean isStopping;

    public CancelAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
      { }

    public void threadedAction(ActionEvent e)
    {
      BApplication.showDialog(confirmation);
      isStopping = confirmation.getResult();
    }

    public void swingCleanupAction(ActionEvent e)
    {
      setEnabled(!isStopping);

      if (isStopping)
        downloadAction.stop();
    }
  }

  // action for stopping a refresh that is currently in progress
  protected class StopRefreshAction extends BThreadedAction
  {
    public StopRefreshAction(BUIResources res, String key)
    { super(res, key); }
    public void swingPrepAction(ActionEvent e)
    { }
    public void threadedAction(ActionEvent e)
    { }
    public void swingCleanupAction(ActionEvent e)
    {
      setEnabled(false);
      refreshAction.stop();
    }
  }

  protected class PlayAction extends BAction
  {
    public PlayAction(BUIResources res, String key)
    { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {
      String fileName = getBattleFile();
      if (fileName != null)
        BBattleScreen.load(fileName);
    }
  }


  // action for cancelling a download that is currently in progress
  protected class NominateAction extends BThreadedAction
  {
    protected BMessage response;

    public NominateAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
    {
      parent.setStatus("Nominating fight...");
      parent.setBusyWorking(true);
    }

    public void threadedAction(ActionEvent e)
    {
      try
      {
        try
        {
          BRequest r = new BRequest(server, "NFF");
          r.setData(new BIdentData(fight.getID()));

          BTransmission t = new BTransmission(r);
          parent.setMonitored(t);

          response = server.request(t);
        }
        catch (BNetException ex)
          { parent.setStatus("ERROR: " + ex.getMessage()); }
        finally
          { parent.setMonitored(null); }

        if (response != null)
        {
          if (response.isAcknowledgement())
          {
            parent.setStatus("Nomination complete.");
            if(selectedFightNode != null)
            {
              Enumeration enum = ((DefaultMutableTreeNode)treeModel.getRoot()).breadthFirstEnumeration();

              while(enum.hasMoreElements())
              {
              BDataNode node = (BDataNode)enum.nextElement();
                if(node.isOfType(BDataTypes.FIGHT))
                {
                  if(selectedFightNode.getID() == ((BFightDataNode)node).getID())
                  {
                    ((BFightDataNode)node).setNominated(true);
                    treeModel.nodeChanged((BFightDataNode)node);
                  }
                }
              }
              selectedFightNode.setNominated(true);
              treeModel.nodeChanged(selectedFightNode);
            }
          }
          else
            parent.setStatus("ERROR: " + response);
        }
        else
          parent.setStatus("ERROR: null response");
      }
      catch(Exception ex)
      { BDebug.print("BHistoryPanel " + ex.getMessage()); }
    }

    public void swingCleanupAction(ActionEvent e)
    {
      if(parent != null)
      {
        parent.setBusyWorking(false);
        setActionStatus();
      }
    }
  }


  // UI Listeners...
  // -------------------------------------------------------------

  protected class SelectionListener implements TreeSelectionListener
  {
    public void valueChanged(TreeSelectionEvent e)
    {
      TreePath path =
        selectionModel.getSelectionPath();

      boolean allTimeGreat = false;
      boolean fightNominated = false;

      if (path != null)
      {
        BDataNode node = (BDataNode)
          path.getLastPathComponent();

        if (node.isOfType(BDataTypes.TOURNAMENT))
        {
            details.setTournament((BTournamentData) node.getData() );
        }
        else
        {
          int level = node.getLevel();
          if(level == 2)
          {
            if(((BDataNode)node.getParent()).isOfType(BDataTypes.TOURNAMENT))
              details.setTournament( (BTournamentData) ((BDataNode)node.getParent()).getData() );
            else
              allTimeGreat = true;
          }
          if(level == 3)
            details.setTournament( (BTournamentData) ((BDataNode)node.getParent().getParent()).getData() );
          if(level == 4)
            details.setTournament( (BTournamentData) ((BDataNode)node.getParent().getParent().getParent()).getData() );
        }


        if (node.isOfType(BDataTypes.FIGHT))
        {
          fight = (BFightData) node.getData();
          fightNode = (BFightDataNode)node;
          selectedFightNode = (BFightDataNode) path.getLastPathComponent();
          // set up the fight info on the right panel
          details.setFight( fight, allTimeGreat );

          fightNominated = ((BFightDataNode)node).isNominated();
        }
        else
        {
          fight = null;
          fightNode = null;
        }
        setActionStatus();
      }
    }
  }

  protected void setActionStatus()
  {
    if (fight != null && fightNode != null)
    {
      File f = new File(getBattleFile());
      boolean fightIsDownloaded = f.exists();

      // check here to see if the fight is an all time great
      long features [] = fight.featuredFightTypes;
      boolean allow = true;

      if (features != null  || fightNode.isNominated())
      {
//        for(int i = 0; i < features.length; i++)
//        {
//          if(features[i] == BFeaturedFightTypeData.ALL_TIME_GREAT)
            allow = false;
//        }
      }


      // if it is then don't allow it to be nominated
      if(allow)
        nominateAction.setEnabled(true);
      else
        nominateAction.setEnabled(false);

      if(fightIsDownloaded)
      {
        play.setAction(playAction);
        save.setAction(saveAction);
        playAction.setEnabled(true);
        saveAction.setEnabled(true);
        downloadAction.setEnabled(false);
        downSaveAction.setEnabled(false);
      }
      else
      {
        play.setAction(downloadAction);
        save.setAction(downSaveAction);
        playAction.setEnabled(false);
        saveAction.setEnabled(false);
        downloadAction.setEnabled(true);
        downSaveAction.setEnabled(true);
      }
  //  downloadAction.setEnabled(!fightIsDownloaded);
  //    playAction.setEnabled(true);
    }
    else
    {
      downloadAction.setEnabled(false);
      downSaveAction.setEnabled(false);
      playAction.setEnabled(false);
      nominateAction.setEnabled(false);
      saveAction.setEnabled(false);
    }
    clearCacheAction.setEnabled(true);
  }

  protected String getBattleFile()
    { return BGlobal.BATTLERECORDINGCACHEPATH+ fight.getID()+ ".rec"; }

  protected String getBattleStatFile()
  {
    String returnVal = null;
    String fileName = BGlobal.BATTLERECORDINGCACHEPATH+ fight.getID()+ ".sta";
    File f = new File(fileName);
    boolean statsExist = f.exists();

    if(statsExist)
      returnVal = fileName;

    return returnVal;
  }

  public void onClose()
  {
    if( !downloadAction.isStopped() )
      downloadAction.stop();
    if( !refreshAction.isStopped() )
      refreshAction.stop();

    // busy wait until the download thread has stopped
    while ( !downloadAction.isStopped() )
    {
      try {
        wait((long)20);
      }catch (InterruptedException e) {}
    }

    // busy wait until the refresh thread has stopped
    while ( !refreshAction.isStopped() )
    {
      try {
        wait((long)20);
      }catch (InterruptedException e) {}
    }
    super.onClose();
  }

  protected class TournamentDetails extends BToolPanel
  {

    // Member variables.
    // -----------------------------------------------------------

    protected BTournamentData tournament;
    protected BFightData fight;

    // UI Resources ...
    // -----------------------------------------------------------

    protected BUIResources res = new BUIResources(new Object[]
    {
      "name.font",                new Font(BGlobal.FONT_NAME, Font.PLAIN, 14),
      "name.antialiased",         Boolean.TRUE,
      "name.height",              new Integer(20),

      "image.height",             new Integer(50),

      "statsButton1.text",        "Creator stats",
      "statsButton1.height",      new Integer(22),
      "statsButton2.text",        "Creator stats",
      "statsButton2.height",      new Integer(22),

      "creator1.text",            "Creator:",
      "creator1.width",           new Integer(150),
      "creator2.text",            "Creator:",
      "creator2.width",           new Integer(150),
      "bot1.text",                "Bot:",
      "bot1.width",               new Integer(150),
      "bot2.text",                "Bot:",
      "bot2.width",               new Integer(150),
    });


    // Visual elements.
    // -----------------------------------------------------------

    protected BLabel name       = new BLabel(res, "name");
    protected BMultiLabel desc  = new BMultiLabel(res, "desc");

    protected BLabel image      = new BLabel(res, "image");

    protected BLine line = new BLine();
    protected BLabel creatorLabel1 = new BLabel(res, "creator1");
    protected BLabel creatorLabel2 = new BLabel(res, "creator2");
    protected BLabel botLabel1 = new BLabel(res, "bot1");
    protected BLabel botLabel2 = new BLabel(res, "bot2");
    protected BButton statsButton1 = new BButton(res, "statsButton1");
    protected BButton statsButton2 = new BButton(res, "statsButton2");

    protected BBoxPanel statsPanel = new BBoxPanel(BoxLayout.Y_AXIS);
    // Constructor.
    // -----------------------------------------------------------

    public TournamentDetails()
    {
      super("results");

      layoutSelectedTournamentChildren();
      configSelectedTournamentChildren();
    }


    // Initialization helper methods...
    // -----------------------------------------------------------

    protected void layoutSelectedTournamentChildren()
    {
      BBoxPanel box = new BBoxPanel(BoxLayout.X_AXIS);
      BBoxPanel left = new BBoxPanel(BoxLayout.Y_AXIS);
      BBoxPanel box2 = new BBoxPanel(BoxLayout.X_AXIS);
      BBoxPanel left2 = new BBoxPanel(BoxLayout.Y_AXIS);

      addHGlued(name);
      addSmallMargin();

      addHGlued(image);

      addSmallMargin();
      add(desc);
      addSmallMargin();

      left.addHGlued(creatorLabel1);
      left.addHGlued(botLabel1);

      left2.addHGlued(creatorLabel2);
      left2.addHGlued(botLabel2);

      box.add(left);
      box.addSmallMargin();
      box.addHGlued(statsButton1);
      box.addSmallMargin();

      box2.add(left2);
      box2.addSmallMargin();
      box2.addHGlued(statsButton2);
      box2.addSmallMargin();

      statsPanel.addHGlued(line);
      statsPanel.addMargin();
      statsPanel.add(box);
      statsPanel.addSmallMargin();
      statsPanel.add(box2);

      add(statsPanel);
    }

    protected void configSelectedTournamentChildren()
    {
      statsPanel.setVisible(false);
      statsButton1.addActionListener(new StatsActionListener1());
      statsButton2.addActionListener(new StatsActionListener2());
    }


    // Accessor methods...
    // -----------------------------------------------------------

    public void setTournament(BTournamentData tournament)
    {
      statsPanel.setVisible(false);
      this.tournament = tournament;
      name.setText(tournament.name);

      String status = null;

      if ( tournament.description == null)
        tournament.description = "";
      if (tournament.commentary != null)
        desc.setText(tournament.commentary + "\n\n");
      else
        desc.setText("\n\n\n");

      if(tournament.statusCode == tournament.PENDING)
        status = "Pending";
      else if (tournament.statusCode == tournament.FIGHTING)
        status = "In progress";
      else if (tournament.statusCode == tournament.COMPLETE)
        status = "Complete";

      Calendar closingDate = (Calendar) tournament.closingDate.clone();

      String timeString = "";
      if ( closingDate != null)
      {
          DateFormat df =   DateFormat.getDateInstance(DateFormat.FULL );
          timeString = df.format(closingDate.getTime());
      }


      String arenaName = BStrings.convertFromFilename(tournament.arena);

      if(arenaName.substring(0, 10).equals("/" + BGlobal.HIDDENPATH))
        arenaName = arenaName.substring(10, arenaName.length());

      String firstLetter = arenaName.substring(0,1);
      firstLetter = firstLetter.toUpperCase();

      arenaName = firstLetter + arenaName.substring(1, arenaName.length());

      desc.append("Tournament Information\n");

      String typeString = "";

      if(tournament.tournamentType != null)
      {
        if(tournament.tournamentType.equals("PN"))
         typeString = "Professional (Nonranked)";
        if(tournament.tournamentType.equals("AN"))
         typeString = "Amateur (Nonranked)";
        if(tournament.tournamentType.equals("PR"))
          typeString = "Professional (Ranked)";
        if(tournament.tournamentType.equals("AR"))
          typeString = "Amateur (Ranked)";
      }
      desc.append("Type:\t\t" + typeString + "\n");
      desc.append("Arena:\t\t" + arenaName + "\n");
      desc.append("Bot cost limit:\t\t" + (int)tournament.botPriceCap+"\n");
      desc.append("Closed date:\t\t"+ timeString +"\n" );


      if (status != null)
        desc.append("Status:\t\t" + status + "\n\n");

      if (tournament.summary != null)
        desc.append(tournament.summary);


      Image img = tournament.getImage();
      if (img == null)
        image.setVisible(false);
      else
      {
        image.setIcon(new ImageIcon(img));
        image.setVisible(true);
      }

      /*desc.append("Bot cost limit:\t\t" + (int)tournament.botPriceCap+"\n");
      desc.append("Closing date:\t\t"+ tournament.closingDate+"\n");
      desc.append("Max. entries:\t\t"+ tournament.maximumEntries+"\n");
      desc.append("Entries to date:\t\t"+ tournament.currentEntries+"\n");
      desc.append("Length of fights:\t"+ tournament.fightDuration+" seconds\n");
      desc.append("Max submissions allowed:\t"+ tournament.allowedEntriesPerAccount + "\n");
      desc.append("Submission fee:\t$"+ tournament.entryFee+"\n");   */


    }

    public void setFight(BFightData fight, boolean allTimeGreat)
    {
      statsPanel.setVisible(true);
      creatorLabel1.setText("Creator: " + fight.designerName1);
      creatorLabel2.setText("Creator: " + fight.designerName2);
          botLabel1.setText("Bot:       " + fight.botName1);
          botLabel2.setText("Bot:       " + fight.botName2);

      if(allTimeGreat)
        name.setText("All Time Greats");

      String winString = "";
      String winnerString = "";

      this.fight = fight;

      desc.setText(fight.toString()+"\n\n");

      if (fight.comments != null)
        desc.setText(fight.comments + "\n\n");

      if (fight.winDescription == null || fight.winDescription.equals("null"))
        winString = "points victory";
      else
        winString = fight.winDescription;

      if(fight.winnerName == null)
      {
        winnerString = "Draw";
        winString = "Draw";
      }
      else
        winnerString = fight.winnerName;

      desc.append("Winner: " + winnerString + "\n\n");
      desc.append("Victory condition: " + winString);

      if(!allTimeGreat)
      {
        Image img = tournament.getImage();

        if (img != null)
          image.setIcon(new ImageIcon(img));
      }
      else
        image.setIcon(BIconCache.get("featured_fight.gif"));
    }

  }

  // a stats button has bee pressed
  public class StatsActionListener1 implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      BProcessor p = new BProcessor()
      {
        public void process()
        { showPlayerStats("creator1"); }
      };
      p.start();
    }
  }
  public class StatsActionListener2 implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      BProcessor p = new BProcessor()
      {
        public void process()
        { showPlayerStats("creator2"); }
      };
      p.start();
    }
  }

  public void showPlayerStats(String player)
  {
    long accountID;
    String creatorName;

    if(player.equals("creator1"))
    {
      creatorName = fight.designerName1;
      accountID = fight.designerAccountID1;
    }
    else
    {
      creatorName = fight.designerName2;
      accountID = fight.designerAccountID2;
    }
    rankingDialog = new BPlayerRankingDialog(accountID, creatorName, server);
    BApplication.showDialog(rankingDialog);
  }

  // displays a list of the highest ranked players
  protected class RankingList extends BToolPanel
  {

    // Member variables.
    // -----------------------------------------------------------
    Vector rankList;

    // UI Resources ...
    // -----------------------------------------------------------

    protected BUIResources res = new BUIResources(new Object[]
    {
    });


    // Visual elements.
    // -----------------------------------------------------------
    BTable rankTable = new BTable();
    BScrollPane listScroll = new BScrollPane();
    String level;
    // Constructor.
    // -----------------------------------------------------------

    public RankingList(String level)
    {
      super(level);

      this.level = level;

      //setupTable();

      layoutRankingListChildren();
      configRankingListChildren();
    }

  // fills the table with the ranking information returned from the server
  protected void setupTable()
  {
      if(level.equals("pro rank"))
        rankList = getRankingList(BRankingData.PROFESSIONAL);
      else
        rankList = getRankingList(BRankingData.AMATEUR);

      if(rankList != null)
      {
        if(rankList.size() > 0)
        {
          Object [][] data = new Object [rankList.size()][3];
          for(int i = 0; i < rankList.size(); i++)
          {
            data[i][0] = ""+((BRankingData)rankList.elementAt(i)).ranking;
            data[i][1] = ((BRankingData)rankList.elementAt(i)).designerName;
            data[i][2] = ""+(int)((BRankingData)rankList.elementAt(i)).points;
          }
          Object [] names = new Object [3];
          names[0] = new String("rank");
          names[1] = new String("creator");
          names[2] = new String("points");
          rankTable = new BTable(data, names);
          listScroll = new BScrollPane(rankTable);

          TableColumnModel columns =
          rankTable.getColumnModel();
          TableColumn rank = columns.getColumn(0);
          rank.setMaxWidth(100);
          rank = columns.getColumn(2);
          rank.setMaxWidth(100);
        }
      }
  }

  // refresh the rankings table
  public void refreshRankings()
  {
    remove(listScroll);

    setupTable();

    addGlued(listScroll);
  }

  // gets the selected player ranking data from the server
  protected Vector getRankingList(int level)
  {
    try
    {
      BRequest r = new BRequest(server, "RANK");
      BIdentData iData = new BIdentData(level);
      r.setData(iData);

      BTransmission t = new BTransmission(r);

      // Get the response back from the server...
      BMessage response = server.request(t);

//      BDebug.print("BHistoryPanel - getRankingList - user name, password = " + server.getUserName() + ", " + server.getPassword());
//      BDebug.print("BHistoryPanel - getRankingList - " + response.getData());

      BDataSet dataSet = (BDataSet)response.getData();

      Vector rankList = new Vector();

      if (response.isAcknowledgement())
      {
        //BDebug.print("BHistoryPanel - getRankingList - ACK");
//        BDebug.print("BHistoryPanel - getRankingList - list length = " + dataSet.data.length);
        if(dataSet != null)
        {
          if(dataSet.data != null)
          {
            if(dataSet.data.length > 0)
            {
              for (int i = 0; i < dataSet.data.length; i++)
              {
                BRankingData rData = (BRankingData)dataSet.data[i];
                rankList.add(rData);
              }
            }
          }
        }
        return rankList;
      }
      else
      {
       // BDebug.print("BHistoryPanel - getRankingList - NAK");
        return null;
      }
    }
    catch (Exception e)
    {
      BDebug.print("BHistoryPanel - getRankingList " + e.getMessage());
    }
    return null;
  }
    // Initialization helper methods...
    // -----------------------------------------------------------

    protected void layoutRankingListChildren()
    {
      addSmallMargin();
      if(level.equals("pro rank"))
        addHGlued(new BLabel("Professional rankings"));
      else
        addHGlued(new BLabel("Amateur rankings"));
      addSmallMargin();
      addGlued(listScroll);
    }

    protected void configRankingListChildren()
    {
    }
    // Accessor methods...
    // -----------------------------------------------------------
  }
}
