package roboforge.ui.net;

import roboforge.ui.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;

import roboforge.net.*;
import roboforge.net.client.*;
import roboforge.net.data.*;

import roboforge.util.graphics.*;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.*;
import javax.swing.*;




public class BJoinPanel extends BSystemPanel
{


  // Internal variables...
  // -------------------------------------------------------------

  protected BTournamentScreen   tournaments;
  protected BClientStack        server;

  protected BAccountData        accountData;
  
  
  
  // UI Resources ...
  // -------------------------------------------------------------

  protected static final Integer LABEL_WIDTH = 
    new Integer(100);
  
  protected BUIResources res = new BUIResources( new Object[]
  {
    "center.width",         new Integer(300),

    "title.icon",           BIconCache.getHSB("create_account_title.gif"),
    "title.foreground",     colors.getMidShade(),
    "title.size",           new Dimension(300, 35),
    
    "info.text",            "Please fill in your information below.  All fields are required.\nNOTE: To play online you need to have purchased RoboForge.\nIf you have not purchased RoboForge online, you will need to \nuse the email address supplied with your reg key.",
    "info.height",          new Integer(60),

    "userLabel.text",       "Login ID:  ",
    "userLabel.width",      LABEL_WIDTH,
    "userLabel.h_align",    BLabel.ALIGN_RIGHT,

    "pass01Label.text",     "Password:  ",
    "pass01Label.width",    LABEL_WIDTH,
    "pass01Label.h_align",  BLabel.ALIGN_RIGHT,

    "pass02Label.text",     "Confirm password:  ",
    "pass02Label.width",    LABEL_WIDTH,
    "pass02Label.h_align",  BLabel.ALIGN_RIGHT,

    "nameLabel.text",       "Full name:  ",
    "nameLabel.width",      LABEL_WIDTH,
    "nameLabel.h_align",    BLabel.ALIGN_RIGHT,

    "nickLabel.text",       "Designer name:  ",
    "nickLabel.width",      LABEL_WIDTH,
    "nickLabel.h_align",    BLabel.ALIGN_RIGHT,

    "regoLabel.text",       "Registration code:  ",
    "regoLabel.width",      LABEL_WIDTH,
    "regoLabel.h_align",    BLabel.ALIGN_RIGHT,

    "emailLabel.text",       "Email address:  ",
    "emailLabel.width",      LABEL_WIDTH,
    "emailLabel.h_align",    BLabel.ALIGN_RIGHT,

    "joinAction.name",      "Submit",
    "joinAction.tooltip",   "Submit account registration.",

    "cancelAction.name",    "Cancel",
    "cancelAction.tooltip", "Cancel registration.",

    "finishAction.name",    "Finish",
    "finishAction.tooltip", "Finish registration.",
  });


  
  // Visual elements...
  // -------------------------------------------------------------

  protected BBoxPanel center    = new BBoxPanel(res, "center", BBoxPanel.Y_AXIS);
  protected BBoxPanel buttons   = new BBoxPanel(BBoxPanel.X_AXIS);

  protected BLabel title        = new BLabel(res, "title");
  protected BMultiLabel info    = new BMultiLabel(res, "info");
  protected BMultiLabel notice  = new BMultiLabel(res, "notice");
  
  protected BLabel userLabel    = new BLabel(res, "userLabel");
  protected BLabel pass01Label  = new BLabel(res, "pass01Label");
  protected BLabel pass02Label  = new BLabel(res, "pass02Label");
  protected BLabel nameLabel    = new BLabel(res, "nameLabel");
  protected BLabel nickLabel    = new BLabel(res, "nickLabel");
  protected BLabel regoLabel    = new BLabel(res, "regoLabel");
  protected BLabel emailLabel    = new BLabel(res, "emailLabel");

  protected BTextField user        = new BTextField();
  protected BPasswordField pass01  = new BPasswordField();
  protected BPasswordField pass02  = new BPasswordField();
  protected BTextField name        = new BTextField();
  protected BTextField nick        = new BTextField();
  protected BTextField rego        = new BTextField();
  protected BTextField email        = new BTextField();


  protected BButton             join          = new BButton(res, "join");
  protected BButton             cancel        = new BButton(res, "leave");


  // Actions...
  // -------------------------------------------------------------

  protected JoinAction          joinAction    = new JoinAction(res, "joinAction");
  protected CancelAction        cancelAction  = new CancelAction(res, "cancelAction");
  protected FinishAction        finishAction  = new FinishAction(res, "finishAction");

  
  
  // Constructor.
  // -------------------------------------------------------------

  public BJoinPanel(BTournamentScreen parent)
  {
    super(parent, "Join");

    this.server      = parent.getServer();
    this.tournaments = parent;
    this.accountData = new BAccountData(0);
    
    layoutJoinChildren();
    configJoinChildren();
  }


  // Initialization helper methods...
  // -------------------------------------------------------------

  protected void layoutJoinChildren()
  {
    
    addGlue();
    add(center);
    addGlue();

    center.addGlue();
    center.add(title);
    center.add(info);
    center.addSmallMargin();
    
    center.addLinear(userLabel, user);
    center.addSmallMargin();

    center.addLinear(emailLabel, email);
    center.addSmallMargin();

    center.addLinear(pass01Label, pass01);
    center.addSmallMargin();
    center.addLinear(pass02Label, pass02);
    
    center.addBigMargin();
    center.addLinear(nameLabel, name);
    center.addSmallMargin();
    center.addLinear(nickLabel, nick);
    center.addSmallMargin();
    center.addLinear(regoLabel, rego);

    buttons.addGlue();
    buttons.add(join);
    buttons.addSmallMargin();
    buttons.add(cancel);

    center.addStrut(20);
    center.add(buttons);
    center.addGlue();
  }

  protected void configJoinChildren()
  {
    join.setAction(joinAction);
    cancel.setAction(cancelAction);
  }

  

  
  protected void populateAccountData()
  {
    BAccountData d = accountData;
    
    String p1 = new String(pass01.getPassword()).trim();
    String p2 = new String(pass02.getPassword()).trim();
    
    d.loginID           = user.getText().trim();
    d.password          = p1.equals(p2) ? p1 : "!";
    d.name              = name.getText().trim();
    d.designerName      = nick.getText().trim();
    d.registrationCode  = rego.getText().trim();
    d.email             = email.getText().trim();
  }
  
  protected boolean isValidData()
  {
    BAccountData d = accountData;

    boolean isUserInvalid = d.loginID.equals("");
    boolean isPassInvalid = (d.password.equals("") || d.password.equals("!"));
    boolean isNameInvalid = d.name.equals("");
    boolean isNickInvalid = d.designerName.equals("");
    boolean isRegoInvalid = d.registrationCode.equals("");
    boolean isEmailInvalid =  d.email.equals("");

    boolean isAllValid    = 
      !(isUserInvalid || 
        isPassInvalid || 
        isNameInvalid || 
        isNickInvalid || 
        isRegoInvalid ||
        isEmailInvalid);

    /*

    String problems  = "";
    String sf        = "<FONT COLOR=\"red\">";
    String ef        = "</FONT>";

    if (isUserInvalid)
      problems += "<LI>You did not enter a "+ sf+ "login ID"+ ef+ ".";

    if (d.password.equals(""))
      problems += "<LI>You did not enter a "+ sf+ "password"+ ef+ ".";
    else if (d.password.equals("!"))
      problems += "<LI>Your two "+ sf+ "passwords"+ ef+ " did not match.";

    if (isNameInvalid)
      problems += "<LI>You did not enter your "+ sf+ "full name"+ ef+ ".";

    if (isNickInvalid)
      problems += "<LI>You did not enter a "+ sf+ "nickname"+ ef+ ".";

    if (isRegoInvalid)
      problems += "<LI>You did not enter a "+ sf+ "registration code"+ ef+ ".";


    if (!isAllValid)
      BApplication.notifyUser(
        "Please re-enter data",
        "<HTML>"+
        "<FONT FACE=\"Dialog\" SIZE=-2>"+
        "<B>Please correct the following problems:</B>"+
        "<UL>"+ problems+ "</UL></FONT>");
    */

    Color OK  = colors.getDarkShade();
    Color bad = Color.red;

    userLabel.setForeground(   isUserInvalid ? bad : OK);
    pass01Label.setForeground( isPassInvalid ? bad : OK);
    pass02Label.setForeground( isPassInvalid ? bad : OK);
    nameLabel.setForeground(   isNameInvalid ? bad : OK);
    nickLabel.setForeground(   isNickInvalid ? bad : OK);
    regoLabel.setForeground(   isRegoInvalid ? bad : OK);

    return isAllValid;

  }



  // UI Listeners...
  // -------------------------------------------------------------

  protected class JoinAction extends BThreadedAction
  {
    protected BMessage response;
    
    public JoinAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
    {
      parent.setStatus("Validating data...");
      parent.setBusyWorking(true);

      setEnabled(false);
      populateAccountData();
    }

    public void threadedAction(ActionEvent e)
    {
      response = null;
      
      if (!isValidData())
      { 
        parent.setStatus("Problems with entered data.");
        return;
      }
      
      parent.setStatus("Sending account request to server...");

      try
      { 
        BTransmission t = new BTransmission(
          new BRequest(server, "CAC", accountData));

        response = server.request(t);
          // TODO: Encrypt.
      }
      catch (BNetException ex)
        { parent.setStatus("ERROR: "+ ex.getMessage()); }
    }

    public void swingCleanupAction(ActionEvent e)
    {
      if (response != null)
      {
        if (response.isAcknowledgement())
        {
          parent.setStatus("Account created successfully.");
          join.setAction(finishAction);
          cancelAction.setEnabled(false);
        }
        else
          parent.setStatus("ERROR: "+ response.getData());
      }

      parent.setBusyWorking(false);
      setEnabled(true);
    }
    
  }
  
  
  protected class CancelAction extends BAction
  {
    public CancelAction(BUIResources res, String key)
      { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {
      try
        { server.logout(); }
      catch (BNetException ex)
        { /* IGNORE */ }
      
      tournaments.loginShow(); 
    }
  }

  
  protected class FinishAction extends BAction
  {
    public FinishAction(BUIResources res, String key)
      { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {
       try
       {
        server.login(user.getText(), pass01.getText() );
        tournaments.loginHide();

       }
      catch (BNetException ex)
      {
        parent.setStatus("Login failed: "+ ex.getMessage());
      }
    }
  }
  
}
