package roboforge.ui.net;

import roboforge.BDebug;

import roboforge.net.*;
import roboforge.net.client.*;
import roboforge.BGlobal;

import roboforge.ui.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import roboforge.net.data.*;





public class BLoginPanel extends BSystemPanel
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BTournamentScreen   tournaments;
  protected BClientStack        server;


  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "center.width",       new Integer(300),

    "title.icon",         BIconCache.getHSB("login_title.gif"),
    "title.foreground",   colors.getMidShade(),
    "title.size",         new Dimension(300, 40),

    "info.text",          "If you're new to Roboforge, click 'Sign up' to create an account.",
    "info.height",        new Integer(35),

    "userLabel.text",     "Login ID:",
    "userLabel.width",    new Integer(70),

    "passLabel.text",     "Password:",
    "passLabel.width",    new Integer(70),

    "motdScroller.height", new Integer(350),

    "patchText.border",    new EmptyBorder(10, 10, 10, 10),

  });



  // Visual elements...
  // -------------------------------------------------------------

  protected BTextArea patchText   = new BMultiLabel(res, "patchText");
  protected BScrollPane motdScroller = new BScrollPane(res,"motdScroller" );

  protected BBoxPanel center    = new BBoxPanel(res, "center", BBoxPanel.Y_AXIS);
  protected BBoxPanel buttons   = new BBoxPanel(BBoxPanel.X_AXIS);

  protected BLabel title        = new BLabel(res, "title");
  protected BMultiLabel info    = new BMultiLabel(res, "info");

  protected BLabel userLabel    = new BLabel(res, "userLabel");
  protected BLabel passLabel    = new BLabel(res, "passLabel");

  protected BTextField user     = new BTextField(res, "user");
  protected BPasswordField pass = new BPasswordField(res, "pass");

  protected BButton     login   = new BButton();
  protected BButton     join    = new BButton();


  // Actions...
  // -------------------------------------------------------------

  protected LoginAction loginAction = new LoginAction();
  protected JoinAction  joinAction  = new JoinAction();

  

  // Constructor.
  // -------------------------------------------------------------


  public BLoginPanel(BTournamentScreen parent)
  {
    super(parent, "login");

    this.server      = parent.getServer();
    this.tournaments = parent;
    
    layoutLoginChildren();
    configLoginChildren();
  }


  // Initialization helper methods...
  // -------------------------------------------------------------

  protected void layoutLoginChildren()
  {
    addHStrut(50);
    addHGlued(motdScroller);
    addHStrut(50);
    add(center);
    addHStrut(50);
    //addGlue();

    center.addGlue();
    center.add(title);

    center.add(info);
    center.addLinear(userLabel, user);
    center.addSmallMargin();
    center.addLinear(passLabel, pass);

    buttons.addGlue();
    buttons.add(join);
    buttons.addSmallMargin();
    buttons.add(login);

    center.addStrut(20);
    center.add(buttons);
    center.addGlue();
  }

  protected void configLoginChildren()
  {
    login.setAction(loginAction);
    join.setAction(joinAction);
    
    pass.addActionListener(loginAction);
    pass.setNextFocusableComponent(login);

    motdScroller.getViewport().setView(patchText) ;

    fillPatchText();

  }


  protected void fillPatchText()
  {
    try
    {
      String totalText = "";
      RandomAccessFile inFile = new RandomAccessFile(BGlobal.SYSTEMPATH + "patch.txt", "r" );
      String line = inFile.readLine();

      while (line != null)
      {
        totalText = totalText + "\n" + line;
        line = inFile.readLine();
      }

      patchText.setText(totalText);

    }
    catch(Exception e) {}
  }


  boolean isFirstPaint = true;
  public void paint(Graphics g)
  {
      if (isFirstPaint)
      {
        isFirstPaint = false;
        motdScroller.getViewport().setViewPosition(  new Point(0,0)     ) ;
      }

      super.paint(g);

  }




  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();
    user.requestFocus();
  }


  // UI Listeners...
  // -------------------------------------------------------------

  protected class LoginAction extends BThreadedAction
  {
    public LoginAction()
      { super("Log in"); }

    public void swingPrepAction(ActionEvent e)
    {
      parent.setStatus("Logging in...");
      parent.setBusyWorking(true);

      buttons.setEnabled(false);
    }

    public void threadedAction(ActionEvent e)
    {
      String password = new String(pass.getPassword());

      try
        { server.login(user.getText(), password); }
      catch (BNetException ex)
        { parent.setStatus("Login failed: "+ ex.getMessage()); }
      finally
        { password = null; }
    }

    public void swingCleanupAction(ActionEvent e)
    {
      parent.setBusyWorking(false);
      buttons.setEnabled(true);

      if (server.isLoggedIn())
      {
        tournaments.loginHide();

        BRequest r = new BRequest(server, "UPD");

        // Prepare the request Data...
        BPatchRequestData patchRequestData = new BPatchRequestData(0l);
        patchRequestData.versionID = BGlobal.APP_VERSION;
        r.setData(patchRequestData);

        try
        {
          BTransmission t = new BTransmission(r);
          BMessage response = server.request(t);

          if (response.isAcknowledgement() )
          {}
          else
            setStatus(response + "" );

        }
        catch (BNetException ex)
          { setStatus( ex + "" ); }



      }
    }

  }


 protected class JoinAction extends BAction
  {
    public JoinAction()
      { super("Sign up"); }

    public void actionPerformed(ActionEvent e)
    {
      BJoinPanel join = 
        new BJoinPanel(tournaments);
      
      tournaments.setSystemPanel(join);
    }
  }
   
  
  /*
  protected class LogoutAction extends AbstractAction
  {
    public LogoutAction()
      { super("Log out"); }

    public void actionPerformed(ActionEvent e)
    {
      server.logout();
      parent.setStatus("Logged out.");
    }
  }
  */

}