package roboforge.ui.net;

import roboforge.BDebug;
import roboforge.BGlobal;

import roboforge.net.*;
import roboforge.net.client.*;
import roboforge.net.data.*;
import roboforge.net.services.*;

import roboforge.ui.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.layers.*;

import roboforge.util.graphics.*;
import roboforge.util.io.*;
import roboforge.util.data.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import roboforge.BArenaCanvas;
import roboforge.util.text.*;

import roboforge.util.processors.*;


////////////////////////////////////////////////////////////////////////////////
//
// class BPlayerRankingDialog is a dialog which displays ranking information
// about a selected player
//
//
public class BPlayerRankingDialog extends BSystemDialog implements ActionListener
{
  protected RankingClient client;
  protected BUIResources res = new BUIResources( new Object[]
  {
    "closeButton.text",         "Close",

    "creatorLabel.text",        "creator:",
    "creatorLabel.fontsize",    new Integer(14),
    "creatorLabel.fontstyle",   new Integer(Font.BOLD),

    "statusLabel.text",         "status:",
    "statusLabel.fontsize",     new Integer(14),
    "statusLabel.fontstyle",    new Integer(Font.BOLD),

    "proLabel.text",            "Professional stats:",
    "amLabel.text",             "Amateur stats:",

    "infoScroll.width",         new Integer(220),
    "infoScroll.height",        new Integer(70),
  });

  protected long accountID;
  protected String creatorName;
  protected BClientStack server;

  protected BLine line1 = new BLine();
  protected BLine line2 = new BLine();
  protected BLine line3 = new BLine();

  protected BLabel creatorLabel = new BLabel(res, "creatorLabel");
  protected BLabel statusLabel =  new BLabel(res, "statusLabel");
  protected BLabel proLabel =     new BLabel(res, "proLabel");
  protected BLabel amLabel =      new BLabel(res, "amLabel");

  protected BMultiLabel proStatsLabel = new BMultiLabel(res, "proStatsLabel");
  protected BMultiLabel amStatsLabel  = new BMultiLabel(res, "amStatsLabel");
  protected BMultiLabel infoLabel     = new BMultiLabel(res, "infoLabel");

  protected BScrollPane infoScroll    = new BScrollPane(res, "infoScroll", infoLabel);

  protected BButton closeButton = new BButton(res, "closeButton");

  public BPlayerRankingDialog(long accountID, String creatorName, BClientStack server)
  {
    super("player ranking info", false);
    setSize(new Dimension(460, 340));
    setLocation(170, 140);

    this.accountID = accountID;
    this.creatorName = creatorName;
    this.server = server;

    closeButton.addActionListener(this);

    addButton(closeButton);

    // get the ranking data from the server
    BAccountRankingData rData = getAccountRankingData();

    if(rData != null)
    {
      // start up the UI using the data
      client = new RankingClient(creatorName, rData);
      setClient(client);

      modal = true;
      blocking = false;
    }
    else
    {
      BApplication.hideDialog(BPlayerRankingDialog.this);
      dispose();
    }
  }

  // gets the selected player ranking data from the server
  protected BAccountRankingData getAccountRankingData()
  {
    try
    {
      BRequest r = new BRequest(server, "GPR");

      BDebug.print("BPlayerRankingDialog - getAccountRankingData - accountID = " + accountID);

      BIdentData iData = new BIdentData(accountID);
      r.setData(iData);

      // Get the response back from the server...
      BTransmission t = new BTransmission(r);
      BMessage response = server.request(t);

      BAccountRankingData rData = (BAccountRankingData)response.getData();

      if (response.isAcknowledgement())
      {
        BDebug.print("BPlayerRankingDialog - getAccountRankingData - ACK");
        return rData;
      }
      else
      {
        BDebug.print("BPlayerRankingDialog - getAccountRankingData - NAK");
        return null;
      }
    }
    catch (Exception e)
    {
      BDebug.print("BPlayerRankingDialog - getAccountRankingData " + e.getMessage());
    }
    return null;
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource() == closeButton)
    {
      BApplication.hideDialog(BPlayerRankingDialog.this);
      dispose();
    }
  }

  // the dialog UI
  protected class RankingClient extends SystemDialogClient implements ActionListener
  {
    protected String creatorName;
    protected BAccountRankingData rData;

    public RankingClient(String creatorName, BAccountRankingData rData)
    {
      super(BoxLayout.Y_AXIS);

      this.rData = rData;
      this.creatorName = creatorName;

      layoutDialogChildren();
      configDialogChildren();
    }

    public void layoutDialogChildren()
    {
      BBoxPanel top = new BBoxPanel(BoxLayout.X_AXIS);
      BBoxPanel topLeft = new BBoxPanel(BoxLayout.Y_AXIS);
      BBoxPanel panel = new BBoxPanel(BoxLayout.X_AXIS);
      BBoxPanel left = new BBoxPanel(BoxLayout.Y_AXIS);
      BBoxPanel right = new BBoxPanel(BoxLayout.Y_AXIS);

      left.addHGlued(proLabel);
      left.addTinyMargin();
      left.addHGlued(line2);
      left.addSmallMargin();
      left.addGlued(proStatsLabel);

      right.addHGlued(amLabel);
      right.addTinyMargin();
      right.addHGlued(line3);
      right.addSmallMargin();
      right.addGlued(amStatsLabel);

      panel.add(left);
      panel.addMargin();
      panel.add(right);

      topLeft.addHGlued(creatorLabel);
      topLeft.addTinyMargin();
      topLeft.addHGlued(statusLabel);

      top.add(topLeft);
      top.addMargin();
      top.addGlued(infoScroll);

      add(top);
      addMargin();
      add(panel);
      addMargin();
    }

    public void configDialogChildren()
    {
      String statusString = "novice";

      if(rData.professionalStatus != null)
      {
        if(rData.professionalStatus.equals("M"))
        {
          statusString = "master";
          statusLabel.setForeground(BGlobal.MASTER_COLOR);
        }
        else if(rData.professionalStatus.equals("GM"))
        {
          statusString = "grand master";
          statusLabel.setForeground(BGlobal.GRAND_MASTER_COLOR);
        }
      }

      infoLabel.setText(rData.accountBlurb);

      creatorLabel.setText("creator:  " + BDictionaryTree.getSharedInstance().filter(creatorName));
      statusLabel.setText("status:    " + statusString);

      proStatsLabel.append("Ranking:\t\t"       + rData.profRanking);
      proStatsLabel.append("\nFights:\t\t"        + rData.totalProfFights);
      proStatsLabel.append("\nWins:\t\t"          + rData.totalProfFightsWon);
      proStatsLabel.append("\nTournament wins:\t" + rData.totalProfTournsWon);
      proStatsLabel.append("\nRanking points:\t"  + rData.profRankingPoints);

      amStatsLabel.append("Ranking:\t\t"       + rData.amateurRanking);
      amStatsLabel.append("\nFights:\t\t"        + rData.totalAmateurFights);
      amStatsLabel.append("\nWins:\t\t"          + rData.totalAmateurFightsWon);
      amStatsLabel.append("\nTournament wins:\t" + rData.totalAmateurTournsWon);
      amStatsLabel.append("\nRanking points:\t"  + rData.amateurRankingPoints);
    }

    public void actionPerformed(ActionEvent e)
    {
      if(e.getSource() == closeButton)
      {
        BApplication.hideDialog(BPlayerRankingDialog.this);
        dispose();
      }
    }
  }
}
