package roboforge.util.io;

import java.io.*;


public class BBlockOutputStream extends FilterOutputStream
  // Filters data in fixed-size blocks.
{
  
  // Member variables...
  // -------------------------------------------------------------

  protected BBlockTransform transform;
  

  protected byte[] pending;
    // Pending buffer.
  
  protected int count;
    // Number of bytes pending.
  
  protected int size;
    // Maximum pending bytes.
  
  protected byte[] intStore = new byte[4];
  
  
  
  // Constructor...
  // -------------------------------------------------------------

  public BBlockOutputStream(OutputStream out, BBlockTransform t)
  {
    super(out);
    
    transform = t;
    
    int blocksize = 
      transform.getBlockSize();
    
    if (blocksize <= 0)
      throw new IllegalArgumentException("Blocksize is <= 0.");
      
    pending   = new byte[blocksize];  
    
    count = 0;
    size  = blocksize;
  }
  
  
  
  // Internal methods...
  // -------------------------------------------------------------

  protected void empty() throws IOException
  {
    if (count > 0) 
    {
      byte[] transformed =
        transform.transform(pending, 0, count);
      
      writeInt(transformed.length);
	    out.write(transformed);
      
	    count = 0;
    }
  }
  
  protected void writeInt(int i) throws IOException
  {
    intStore[0] = (byte) (i >>> 24);
    intStore[1] = (byte) (i >>> 16);
    intStore[2] = (byte) (i >>>  8);
    intStore[3] = (byte) (i >>>  0);
    
    out.write(intStore);
  }
  
  
  protected int handle(byte[] data, int off, int len) throws IOException
  {
    int space  = size - count;
    int amount = Math.min(len, space);
    
    if (amount > 0)
    {
      System.arraycopy(data, off, pending, count, amount);
      count += amount;
    }

    if (count == size)
      empty();
    
    return amount;
  }
  

  // Public methods...
  // -------------------------------------------------------------

  public synchronized void write(int b) throws IOException 
  {
	  if (count >= size) 
      { empty(); }
	  
    pending[count++] = (byte) b;
  }

  public synchronized void write(byte b[], int off, int len) throws IOException 
  {
    int handled  = 0;
    
    while (len > 0)
    {
      handled = handle(b, off, len);
      
      off += handled;
      len -= handled;
    }
  }

  public synchronized void flush() throws IOException 
  {
    empty();
	  out.flush();
  }
}
