package roboforge.util.processors;


public class BQueuedObjectProcessor
  implements BObjectProcessor, Runnable
{
  protected static final int QUEUE_SIZE = 50;
  protected static final int QUEUE_LAST = 49;

  protected Object[] queue;
  protected Thread   thread;
  protected int      index;
  protected boolean  isProcessing;

  public BQueuedObjectProcessor()
  {
    queue = new Object[QUEUE_SIZE];
    index = -1;
    isProcessing = false;

    start();
  }

  public void start()
  {
    thread = new Thread(this, "BQueuedObjectProcessor.thread");
    thread.start();
    isProcessing = true;
  }

  public void stop()
  {
    thread = null;
    isProcessing = false;
  }

  public synchronized void request(Object o)
    { enqueue(o); }

  protected synchronized void enqueue(Object o)
  {
    if (index < QUEUE_LAST)
    {
      index++;
      queue[index] = o;
    }
    else
    {
      // Queue overflow.
    }
  }

  protected synchronized Object dequeue()
  {
    Object o = null;

    if (index >= 0)
    {
      o = queue[0];

      // Shuffle queue.
      for (int j=0; j<index; j++)
        queue[j] = queue[j+1];

      index--;
    }
    else
    {
      // Queue underflow.
    }

    return o;
  }


  public void processObject(Object o)
    // Process an object from the queue.
  {
  }

  public synchronized boolean isProcessing()
    { return isProcessing; }

  public void run()
    // Object processing loop.
  {
    Thread t = Thread.currentThread();
    Object o = null;

    while (thread == t)
    {
      if (index >= 0)
      {
        o = dequeue();
        processObject(o);
      }
      else
        stop();
    }
  }

}