package roboforge.util.processors;

import roboforge.BDebug;


public class BSingleObjectProcessor
  implements BObjectProcessor, Runnable
{
  protected Thread   thread;
  protected int priority;
  protected Object   o;
  protected boolean  isProcessing;


  public BSingleObjectProcessor()
    { this(Thread.NORM_PRIORITY);  }

  public BSingleObjectProcessor(int priority)
  {
    this.thread       = null;
    this.priority     = priority;
    this.o            = null;
    this.isProcessing = false;
  }

  public void start()
  {
    thread = new Thread(this, "BSingleObjectProcessor.thread");
    thread.setPriority(priority);
    thread.start();
    isProcessing = true;
  }

  public void stop()
  {
    thread = null;
    isProcessing = false;
  }

  public synchronized void setPriority(int p)
  {
    priority = p;

    if (thread != null)
      thread.setPriority(p);
  }

  public synchronized void request(Object o)
  {
    if (thread == null)
    {
      this.o = o;
      start();
    }
    else
    {
      // Ignore the request if currently processing...
      BDebug.print("BSingleObjectProcessor.request(): WARNING - Request already being processed!");
    }
  }


  public void processObject(Object o)
    // Process an object from the queue.
  {
  }


  public synchronized boolean isProcessing()
    { return isProcessing; }


  public void run()
    // Object processing loop.
  {
    Thread t = Thread.currentThread();

    while (thread == t)
    {
      if (o != null)
      {
        processObject(o);
        o = null;
        stop();
      }
    }
  }

}