package roboforge;

import roboforge.animation.alpha.*;

import java.util.*;
import javax.media.j3d.*;
//import javax.swing.tree.*;

import roboforge.advertising.*;

public interface BArenaObject extends BAlphaAcceptor
{

/*

* Description:
  Defines the interface that all objects contained within BArena must implement

* Example:
* Modification History ( Name, Date, Notes )
  Created HSC, 21 May.

*/

  public void init(); // For initialising 3D object after loading from file
  
  public String getName(); // nuff said

  public void initArenaDetails(BArena arena);

  public BranchGroup getStructure3D(boolean addZones, boolean addSensors);
  // Get the 3d structure of the object

  public void setLiveNodes(BranchGroup wrapperBranch, TransformGroup displacementGroup, TransformGroup arenaRotateGroup);
  // Set the movement and rotation live nodes for the object, this will allow the object to move itself in whatever universe we place it in
  // This must be called after the nodes have been added to the universe

  public void cleanseLiveNodes();
  // Cleanse the live nodes of the object to allow garbage collection to dispose of it.

  public BranchGroup getLiveNode(boolean centerOfObject);
  // get the BrancGroup that contaons all other nodes for the object
  // if  centreOfObject is true the it will return the node that is the cosest to the centre of the object
  // otherwise it return the node that encapsulates all subTrees.
  // use  getLiveNode(false) is you are detaching the object from the graph
  
  public  BArenaObjectPosition getArenaPosition();
  // get the object position

  public void setArenaPosition(BArenaObjectPosition newPos);
  // get the object position

  public void setAlphaValue(BAlphaValue value);
    // SJS [23/03/2001] Equivalent methods for setting arena
    // position. This is done so that BArenaObjects can be
    // animated through the 'BAlphaChannel' framework.

  public void setWireFrame();
  // Wire Frame live Objects

  public void setNormalAppearance();
  // Return to the normal appearance

  public boolean isPartOf(BShape3D theShape);
  // Returns true if the shape passed in belongs to the arenaObject

  public BArenaObjectPosition getSpecialMovePosition(float timeSlice, Vector channels, String specialMove);

  public Enumeration getComponents();
  // returns a Enumerator with all Components within the Arena object.
  // Each Component element will ba a defaultmutabletrree node containing the component
  // If the Arenaobject contains a tree of objects then this will be a Depthfirst traversal.


 // public void testCollisions (BComponent armingComponent, Vector collisionList ,  javax.media.j3d.Locale myLocale, Vector ignoreList, boolean checkZones);
  // checks to see if this component collides with tyhis object,
  // places all collisions into collisionList, does not compare  all components in CompoareList

  public Bounds getBounds();
  // get the Bounding object that encompases all the components within this

  public Vector getArenaComments();
  // Get the Vector containing simulation comments

  public void setArenaComments(String comment);
  // Appends a comment to the bottom of the comments vector

  public void setAdvertID(String advertID);
  // Sets the advert ID of this arena object

  public String getAdvertID();
  // Gets the advert ID of this arena object

  public void setAdvert(BAdvert advert);
  // sets the actual advert for this arena object
}




