package roboforge;

import roboforge.animation.alpha.*;

import java.io.*;
import javax.vecmath.Vector3d;


public class BArenaObjectPosition
  implements Serializable, Cloneable, BAlphaValue
{

/*

* Description:
  Describes the position within BArena of any BArenaObject

* Example:
* Modification History ( Name, Date, Notes )
  Created HSC, 21 May.
          YLY, 3 June : added members
          DG, 25 Sep : added Y member
*/
  // Members

  static final long serialVersionUID = -3505401401571988755l;      // The serial number is this stupid number cause we forgot to set it at the beginning.
  public float x,z; // Absolute 2-dimensional placing on the xz plane. (Canvas units)
  float facing; // Absolute rotational component on the y axis (radians).
                // positive is anti-clockwise looking down from the top.
  float y = 0.0f;  // Added to provide elevation aspect in Arena

  public BArenaObjectPosition()
  {
    this(0,0,0,0);
  }

  public BArenaObjectPosition( BArenaObjectPosition clone )
  {
    this(clone.x, clone.y, clone.z, clone.facing);

  }

  public BArenaObjectPosition(float newx,float newy,float newz,float newfacing)
  {
    x = newx;
    y = newy;
    z = newz;
    facing = newfacing;

  }

  public float getX()
    { return x; }

  public float getY()
    { return y; }

  public float getZ()
    { return z; }

  public float getFacing()
    { return facing; }

  public Vector3d getTranslation()
    { return new Vector3d(x, y, z); }


  public void set(BArenaObjectPosition objToClone)
  {
    if (objToClone != null)
    {
      this.x = objToClone.x;
      y = objToClone.y;
      z = objToClone.z;
      facing = objToClone.facing;
    }
  }


  public void interpolate
  (
  	BArenaObjectPosition a,
    BArenaObjectPosition b,
    float t
  )
  {
  	// Sets this position to an interpolation of a and b.
    // At t = 0, this = a;  At t = 1, this = b.

  	float ti 		= 1f - t;
  	this.x 		 	= a.x * ti + b.x * t;
    this.y 		 	= a.y * ti + b.y * t;
    this.z 		 	= a.z * ti + b.z * t;
    this.facing = a.facing * ti + b.facing * t;
  }


  public boolean equals(BArenaObjectPosition scumFace)
  {
        if (scumFace == null)
                return false;

        return (  (x== scumFace.x)  && ( z == scumFace.z) && ( facing == scumFace.facing )   );

  }



    public void AVG (BArenaObjectPosition otherPos, int weightOnCurrent)
  {
        float weightedx =  x * (float)weightOnCurrent;
        x = (otherPos.x + weightedx) / (weightOnCurrent + 1);

        float weightedy =  y * (float)weightOnCurrent;
        y = (otherPos.y + weightedy) / (weightOnCurrent + 1);

        float weightedz =  z * (float)weightOnCurrent;
        z = (otherPos.z + weightedz) / (weightOnCurrent + 1);

        float weightedfacing =  facing  * (float)weightOnCurrent;
        facing  = (otherPos.facing  + weightedfacing) / (weightOnCurrent + 1);
  }

  public String toString()
  {
        return "x=" + x + " y=" + y + " z=" + z + " facing=" +facing;

  }


  // BAlphaValue implementation...
  // -------------------------------------------------------------

  public void set(BAlphaValue v)
    { set((BArenaObjectPosition) v); }

  public void interpolate(BAlphaValue v, float alpha)
    { interpolate(this, (BArenaObjectPosition) v, alpha); }

  public BAlphaValue copy()
    { return new BArenaObjectPosition(this); }



}