package roboforge;

import java.util.*;
import java.io.*;
import javax.media.j3d.*;
//import java.applet.AudioClip;  // For Audio Clip playing
//import java.net.*;
import javax.swing.table.*;
import javax.swing.*;
//import javax.swing.tree.*;

//import roboforge.ai.worldstate.*;
//import roboforge.sound.*;
import roboforge.arenaSystems.*;
import roboforge.math.*;
import roboforge.animation.*;
import roboforge.animation.animation.b3d.*;
import roboforge.advertising.*;

public class BBattleRecorder implements Serializable, BProgressMonitorable
{
/*

* Description:
  Stores all the movements of the BArenaObjects
  within an Arena

* Example:
* Modification History ( Name, Date, Notes )
  Created HSC, 21 May.
* NOTE Need to ensure that the BBattleRecorder object is created after the
  arena has been populated and just prior to the start of the battle
  so that arenaTimeSlice 0 is set to the initial
  starting positions of the bots (At time 0 - Before battle has begun).


*/
  static final long serialVersionUID = -1L;

  public static int APPLYMODE_PLAY = 1;
  public static int APPLYMODE_RANDOM = 2;


  public static final String WIN_CONDITION_DRAW     = "Draw.";
  public static final String WIN_CONDITION_CHASSIS  = "Chassis destroyed.";
  public static final String WIN_CONDITION_POINTS   = "Time over. Points victory.";
  public static final String WIN_CONDITION_COST     = "Time over. Cost victory.";



  // MEMBERS
  private String arenaName;
  private BBot botA;
  private BBot botB;
  private BBot victoriousBot; // a,b or null.  NULL indicates a draw.
  private String winString;

  private Vector adverts = new Vector(); // holds the adverts associated with this battle

  private BComponent[] destroyedComponentsSaved;  // the array that gets saved
  private BArenaTimeSlice[] battleTimeSlicesSaved;
  private BCollisionRecord[] collisionsSaved;

  private BArenaEvent[] arenaEventsSaved;
  private int[] arenaEventsFrameIndex;

  private float percent = 0.0f;  // deadlock imminence

  private float timeSlice; // the time elaspsed between each frame
  private String tournamentName = "Training bout";
  private String roundName = "";

  private boolean isTournament = false;
    // Whether this battle is a tournament battle.

  private boolean isFinal = false;
    // Whether this battle is a tournament final.

  // transient working variables
  private transient BBot botToShowStatsFor; //botstoShowStatsFor
  private transient Vector destroyedComponents;  // Holds (BComponent )
  private transient Vector battleTimeSlices;  // Vector of BTimeSlices
  private transient Vector collisions;   // Holds BCollisionRecords
  private transient Vector arenaEvents;


  // the static channels
  // constains BChannel for all the channels that do not change throughout a fight
  private boolean recordBothBotsStats;
  private transient BStats aStats;
  private transient BStats bStats;
  private transient BStats currentStats;
  //private transient static String DELIMETER = "`";
  //private transient static String SECONDAY_DELIMETER = "^";

  // battle simulation constants
  private static short CRITICAL_SNAP_SPACING = 100;
  private static int HITS_FOR_LOCK = (int) (15 * BGlobal.FRAMES_PER_SEC);  // Number of Slices that deadlocks are tracked over (15 seconds)
  private static int DELTA_HITS_FOR_SAMEDEADLOCK =  HITS_FOR_LOCK ; // Window for which deadlocks are valid
  private transient int lastDeadLock = 0;
  private static final float DEADLOCK_THRESHOLD = 0.4f; // Deadlocks for 40% of the Time (was 50% before Move Pausing)
  private static final float DEADLOCK_THRESHOLD_UPPERLIMIT = 0.7f; // Above 70% components must be stuck somehow
  private static final float DEADLOCK_DAMAGEDELTA = 10.0f; // 10% over 15 seconds (about 120% damage for full 3 min fight) (was 4%)
  private static int DAMAGE_HITS_FOR_LOCK = HITS_FOR_LOCK / 2; // Half the numer of frames for normal deadlock


  // status bar stuff
  //private static int DAM_FOR_CROWD = 1;
  private static int CROWD_ANTICIPATION = 4;

  private static int LOAD_BOTTIME = 35;
  private static int LOAD_OBJECT = 20;
  private static int LOAD_BOTA = 55;
  private static int LOAD_BOTB = 90;
  private static int LOAD_SOUND = 100;

  private String lastProgMessage = "Please wait...";
  private transient int loadPercent = LOAD_OBJECT;

  // members used for the raplay of the battle,
  // initialised ijn pre-process
  private transient BArenaSystem backgroundSound;
  private transient BVirtualRefModel ref;

  private transient BBattlePlayerAnimation activeAnimation;
  private transient BDeadlockAnimation deadlockAni;
  private transient BArenaEvent deadlockEvent;
  private transient BSimulationCanvas universe;

  private transient int nextAEventFrame = 0;
  private transient int aEventIndex = 0;
  private transient BArenaEvent clearEvent;

  private transient int endFrameIndex;
  private transient int startFrameIndex;

  protected static final int SMOKE_DAMAGE_THRESHOLD = 20;
    // Damage threshold for displaying chassis smoke.

  protected transient BBattleFX fx;
    // SJS [12/03/2001] - AudioVisual effects handler.

  private transient double currentSimTime = 0d;


  public BBattleRecorder(BBot a, BBot b, String arenaNameIn, float timeSliceIn, boolean recordBothBotsStats)
  //public BBattleRecorder(BArenaObjects arenaObjectsIn, String arenaNameIn, float timeSliceIn)
  {

   // BDebug.print ("BBattleRecorder recording both states " +   recordBothBotsStats);

    botA = a;
    botB = b;
    this.recordBothBotsStats = recordBothBotsStats;

    arenaName = arenaNameIn;
    timeSlice = timeSliceIn;
    destroyedComponents = new Vector();
    battleTimeSlices = new Vector();
    collisions = new Vector();
    arenaEvents = new Vector(3);

    lastDeadLock = 0;
    BGlobal.DEADLOCKCOUNT = 0;
    deadlockEvent = new BArenaEvent()
    {
       static final long serialVersionUID = -1L;

       public int getStateModifier()
        { return 0; }

       public String getArenaEventType()
        { return DEADLOCK; }

    };

    aStats = new BStats(botA.getWorldState(), botA);

    if (recordBothBotsStats)
      bStats = new BStats(botB.getWorldState(), botB);


    this.add(null, null);
  }


  public void setFX(BBattleFX fx)
    { this.fx = fx; }


  public void cleanse()
  {
     victoriousBot= null; // a,b or null.  NULL indicates a draw.
     botToShowStatsFor = null; //botstoShowStatsFor
     botA = null;
     botB = null;
     destroyedComponentsSaved = null;
     battleTimeSlicesSaved = null;
     backgroundSound = null;
     ref = null;
     arenaEventsSaved = null;

     activeAnimation = null;
     deadlockAni =  null;
     deadlockEvent = null;
     universe = null;

     clearEvent = null;
     lastZeroedDeadlock = 0;

     jointsToGetTransformsFor.clear();
     completedSlice.clear();
     multipleFramesSnaps.clear();


  }

  // advert add and retrieve
  public void addAdvert(BAdvert advert)
  {
    if(advert != null)
      adverts.add(advert);
  }

  public BAdvert getAdvert(String advertID)
  {
    for (int i = 0; i < adverts.size(); i++)
    {
      if( ((BAdvert)adverts.elementAt(i)).getAdvertID().equals(advertID))
        return (BAdvert)adverts.elementAt(i);
    }
    return null;
  }

  private static Vector jointsToGetTransformsFor = new Vector();
  private static Vector completedSlice = new Vector();

  private Vector getSnapForAllJoints (int index)
  {

    jointsToGetTransformsFor.clear();
    completedSlice.clear();

    BComponent[] botJoints;

    // bot a's joints
    botJoints = botA.getBreathComponents(true);
    for (int i = 0; i< botJoints.length ; i ++)
    {
      if (botJoints[i] instanceof BJoint )
          jointsToGetTransformsFor.add(botJoints[i] );

    }

    // bot B
    botJoints = botB.getBreathComponents(true);
    for (int i = 0; i< botJoints.length ; i ++)
    {
      if (botJoints[i] instanceof BJoint )
          jointsToGetTransformsFor.add(botJoints[i] );

    }



    int i = index;
    BArenaTimeSlice thisSlice;


    // fill dynamnic joints
    while (jointsToGetTransformsFor.size() > 0 )
    {
     // BDebug.print ("BBattleRecorder.checking timeSlice " + i);
      thisSlice =  battleTimeSlicesSaved[i];
      thisSlice.fillCompleteVector(jointsToGetTransformsFor,completedSlice );
      i--;

    }


    return completedSlice;





  }

  private static Vector multipleFramesSnaps = new Vector();

  private Vector getSnapForMultipleFrames(int thisIndex, int lastIndex)
  {
    multipleFramesSnaps.clear();
    for (int i = thisIndex; i >= lastIndex; i-- )
      battleTimeSlicesSaved[i].constructMultipleFrameVector(multipleFramesSnaps );

    return multipleFramesSnaps;


  }



  public boolean isCollidingAt(int index)
  {
     return battleTimeSlicesSaved[index].isColliding();

  }

  private void processArenaEvent(int index, int frameIndex, boolean playing)
  {
     // BDebug.print ( "BBattleRecorder.processArenaEvent arena event found " + arenaEventsSaved[ index] );

      backgroundSound.processEvent( arenaEventsSaved[ index] );

      if (playing)
      {
        ref.processEvent( arenaEventsSaved[ index] );

        if ( arenaEventsSaved[ index].getArenaEventType().equals(BArenaEvent.DEADLOCK) )
        {
          deadlockAni.setDeadlockFrame(frameIndex + 1);
          fx.spawnDeadlock(botA, botB);
          activeAnimation.interrupt(deadlockAni);

        }
        else
          activeAnimation.processEvent( arenaEventsSaved[ index] );

      }

      updateArenaEventIndex(index);
  }

  private void updateArenaEventIndex(int index)
  {
      //BDebug.print ("BBAttleRecorder updateDeadlockIndex called with " + index);

      aEventIndex = index + 1;
      if (aEventIndex < arenaEventsSaved.length)
        nextAEventFrame =   arenaEventsFrameIndex[aEventIndex];
      else
        nextAEventFrame = battleTimeSlicesSaved.length;

  }



  public void apply(int mode, int index, int skipStartFrameIndex)
  {

      // pre apply range check
      if ( index <0 || index >= battleTimeSlicesSaved.length )
      {
        BDebug.print ("BBattleRecorder.apply index out of range:: " + index);
        //return;
      }
       if (skipStartFrameIndex < 0 || skipStartFrameIndex >= battleTimeSlicesSaved.length )
      {
        BDebug.print ("BBattleRecorder.apply skipStartFrameIndex out of range:: " + skipStartFrameIndex);
       // return;
      }

      int collisionsStart;
      int destroyedComponentStart;
      BArenaTimeSlice thisSlice = battleTimeSlicesSaved[index];

      boolean isBattleActive
        = index <= endFrameIndex;

      // Clear out fx when skipping randomly...
      if (mode == APPLYMODE_RANDOM)
         fx.clear();

      if (skipStartFrameIndex ==  index)
      {
        collisionsStart = thisSlice.collisionStartIndex;
        destroyedComponentStart =  thisSlice.destroyedComponentStartIndex;
      }
      else
      {
         collisionsStart = battleTimeSlicesSaved[skipStartFrameIndex].collisionStartIndex;
         destroyedComponentStart =  battleTimeSlicesSaved[skipStartFrameIndex].destroyedComponentStartIndex;
      }

      if (mode == APPLYMODE_PLAY )
      {
        if (index >= nextAEventFrame )
        {
          processArenaEvent(aEventIndex, index, true);
          // don't skip any frames if this is an arenaEvent
          index = skipStartFrameIndex;
        }

        if (skipStartFrameIndex ==  index)
          thisSlice.apply(botA, botB, isBattleActive);
        else
        {
          //BDebug.print ("Calling mutiple frame play for index" + index + " and " +  skipStartFrameIndex );
          Vector completed = getSnapForMultipleFrames(index, skipStartFrameIndex) ;

          //BDebug.print ("Calling mutiple frame play for index completed " + completed );
          thisSlice.apply(botA, botB, completed, isBattleActive);
        }
      }
      else
      {
          Vector completed =  getSnapForAllJoints ( index);
          thisSlice.apply(botA, botB, completed, isBattleActive);

          updateArenaEventIndex(-1);

          boolean containsEvent = false;

          for (int i = 0; i < arenaEventsFrameIndex.length; i++ )
            if (arenaEventsFrameIndex[i] <= index)
            {

              if (arenaEventsFrameIndex[i] == index)
              {
                containsEvent = true;
                processArenaEvent(i, index, false );
              }
              else
                updateArenaEventIndex(i);


            }

          if (!containsEvent)
            ref.processEvent(clearEvent);


       }

       // Play collision sounds
       // don't play collision sounds if there is a explosion sound
        BCollisionRecord cr;

        if ( destroyedComponentStart == thisSlice.destroyedComponentEndIndex  )
          for (int i = collisionsStart ; i < thisSlice.collisionEndIndex ; i++ )
            if ( (i < collisionsSaved.length) && (i >= 0) )
            {
                cr = collisionsSaved[i];
                fx.spawnCollision(cr, !cr.getNullified());

                // Update crowd model...
                if (!cr.getNullified())
                  backgroundSound.processEvent(cr);

            }


        // Update bot damage / energy info from recording...
        // This is done so that other objects can inspect
        // the bot's health/energy directly.

        botA.setPercentageDamageLevel((float) thisSlice.botADamageLevel);
        botB.setPercentageDamageLevel((float) thisSlice.botBDamageLevel);

        botA.setPercentageEnergyLevel((float) thisSlice.botAEnergyLevel);
        botB.setPercentageEnergyLevel((float) thisSlice.botBEnergyLevel);

        if (index >= endFrameIndex)
        {
          // Special case: set bot damage/energy at end of battle...
          if (victoriousBot != null)
          {
            getLoserBot().setPercentageDamageLevel(100f);
            getLoserBot().setPercentageEnergyLevel(0f);
          }
          else if (isChassisDestroyed(endFrameIndex))
          {
            botA.setPercentageDamageLevel(100f);
            botB.setPercentageDamageLevel(100f);
            botA.setPercentageEnergyLevel(0f);
            botB.setPercentageEnergyLevel(0f);
          }
        }
        else
        {
          // Check for bot damage. If above a certain threshold,
          // spawn chassis smoke.

          if (thisSlice.botADamageLevel >= SMOKE_DAMAGE_THRESHOLD)
            fx.spawnSmoke(botA);

          if (thisSlice.botBDamageLevel >= SMOKE_DAMAGE_THRESHOLD)
            fx.spawnSmoke(botB);
        }


        // add or remove destroyed components
        // and play crowdraise

       if (mode == APPLYMODE_PLAY )
       {
        	BComponent c;

          for (int i = destroyedComponentStart; i < thisSlice.destroyedComponentEndIndex ; i++ )
          {
            if ( i >= destroyedComponentsSaved.length)
            {
              BDebug.print ("BBattleRecorder.apply destroyedComponentsSaved out of range:: " + i);

              return;
            }

          	c = destroyedComponentsSaved[i];

          	// Only ever play 1 instance of each sound clip
            boolean soundPlayed = false;

            for (int j = destroyedComponentStart; j < i; j++ )
            	if (c.getMaterialType() == destroyedComponentsSaved[j].getMaterialType())
              	{ soundPlayed = true; break; }

            // Animation FX...
            if (c.getLiveNode().isLive())
			      {
			        if (!c.getChassisLock())
		      	    fx.spawnExplosion(c, !soundPlayed);
			        else
			          fx.spawnBotExplosion(c.getBot());
      			}

            universe.removeComponent(c);
          }


          int forwardAnticipationIndex = index + CROWD_ANTICIPATION;
          forwardAnticipationIndex = java.lang.Math.min(forwardAnticipationIndex, (battleTimeSlicesSaved.length - 1 ) );

          //
          if (thisSlice.destroyedComponentEndIndex != battleTimeSlicesSaved[forwardAnticipationIndex].destroyedComponentEndIndex )
          	for (int i = thisSlice.destroyedComponentEndIndex ; i <  battleTimeSlicesSaved[forwardAnticipationIndex].destroyedComponentEndIndex  ; i++ )
             	backgroundSound.processEvent(destroyedComponentsSaved[i]);
        }

        else
        {
            for (int i = 0; i < thisSlice.destroyedComponentEndIndex ; i++ )
              universe.removeComponent(destroyedComponentsSaved[i]);

            for (int i = thisSlice.destroyedComponentEndIndex; i < destroyedComponentsSaved.length; i++ )
              universe.addComponent(destroyedComponentsSaved[i]);
        }


        // Handle bot destruction...

        if (index >= endFrameIndex)
        {
          if (mode == APPLYMODE_RANDOM)
          {
            // Just hide the bot(s)...
            if (victoriousBot != null)
              getLoserBot().setTransparency(1f);
            else if (isChassisDestroyed(endFrameIndex))
            {
              botA.setTransparency(1f);
              botB.setTransparency(1f);
            }
          }
          else if (index == endFrameIndex)
          {
            // Spawn bot defeat (if any)...
            if (victoriousBot != null)
            {
              if (isPointsVictory())
                fx.spawnBotDefeat(getLoserBot());
              else
                fx.spawnBotExplosion(getLoserBot());
            }
          }
        }
        else if (mode == APPLYMODE_RANDOM)
        {
          // Restore any invisible (destroyed) bots..
          if (botA.getTransparency() > 0f)
            botA.restoreTransparency();

          if (botB.getTransparency() > 0f)
            botB.restoreTransparency();
        }

  }


  public void interpolate(int index, float t)
  {
    // SJS [14/03/2001] - Interpolate to the frame with index 'index'.
    // Only changes the bot's position, orientation and configuration.
    // No events are processed; no components or collisions handled.
    // t is a weighting coefficient [0..1].

    if (index < 0)
    	return;

    if (index >= battleTimeSlicesSaved.length)
    	return;

    boolean isBattleActive
      = index <= endFrameIndex;

    BArenaTimeSlice slice
      = battleTimeSlicesSaved[index];

    // Interpolate to the specified slice...
    slice.interpolate(botA, botB, t, isBattleActive);
  }



  public boolean isChassisDestroyedVictory()
  {
    // HACK: To find out if chassis was destroyed,
    // inspect the winString. This is BAD!
    return winString.startsWith("Chassis");
    // return winString.equals(WIN_CONDITION_CHASSIS);
  }

  public boolean isPointsVictory()
  {
    // HACK: To find out if a points victory was scored,
    // inspect the winString. This is BAD!
    return winString.equals("Time over. Points victory.");
    // return winString.equals(WIN_CONDITION_POINTS);
  }

  public boolean isDraw()
  {
    // HACK: To find out if a draw occurred,
    // inspect the winString. This is BAD!
    return winString == null;
    // return winString.equals(WIN_CONDITION_DRAW);
  }


  protected boolean isChassisDestroyed(int index)
  {
    BArenaTimeSlice slice = battleTimeSlicesSaved[index];

    int start = slice.destroyedComponentStartIndex;
    int stop  = slice.destroyedComponentEndIndex;

    BComponent c;

    for (int i=stop-1; i>=start; i--)
    {
      c = destroyedComponentsSaved[i];
      if (c.getChassisLock()) return true;
    }

    return false;
  }


  public boolean isBotADead(int i)
    { return battleTimeSlicesSaved[i].botADamageLevel >= 100; }

  public boolean isBotBDead(int i)
    { return battleTimeSlicesSaved[i].botBDamageLevel >= 100; }


  public int getBotAEnergy(int index)
    { return   battleTimeSlicesSaved[index].botAEnergyLevel; }

  public int getBotAHealth(int index)
    { return 100 - battleTimeSlicesSaved[index].botADamageLevel; }

  public int getBotADamageDelta(int i)
  {

    return getBotADamageDelta (i, i-1);

  }

  public int getBotADamageDelta(int upper, int lower)
  {
    if (lower < 0)
      lower = 0;

    if ( battleTimeSlicesSaved != null)
      return battleTimeSlicesSaved[upper].botADamageLevel - battleTimeSlicesSaved[lower].botADamageLevel;
    else
      return ((BArenaTimeSlice)battleTimeSlices.get(upper)).botADamageLevel - ((BArenaTimeSlice)battleTimeSlices.get(lower)).botADamageLevel;


  }




  public int getBotBHealth(int index)
    { return 100 - battleTimeSlicesSaved[index].botBDamageLevel; }

  public int getBotBEnergy(int index)
    { return   battleTimeSlicesSaved[index].botBEnergyLevel; }


  public int getBotBDamageDelta(int i)
  {
    return getBotBDamageDelta(i, i-1) ;
  }

  public int getBotBDamageDelta(int upper, int lower)
  {
    if (lower < 0)
      lower = 0;

    if (battleTimeSlicesSaved != null)
      return battleTimeSlicesSaved[upper].botBDamageLevel - battleTimeSlicesSaved[lower].botBDamageLevel;
    else
      return ((BArenaTimeSlice)battleTimeSlices.get(upper)).botBDamageLevel - ((BArenaTimeSlice)battleTimeSlices.get(lower)).botBDamageLevel;

  }


    public void setBattleStarted ()
  {
    // PRE: This must be called before calling the add method that adds the first
    // timeSlice for the start of the battle
    //timeSliceBattleStarted = arenaTimeSlices.size();
  }

  public void setVictoriousBots (BBot botIn)
  {
    victoriousBot = botIn;
  }

  public BBot getVictoriousBot ()
  {
    return victoriousBot;
  }

  public BBot getLoserBot()
    { return (victoriousBot == botA) ? botB : botA; }

  public BBot getBotA()
  {
        return botA;
  }

  public BBot getBotB()
  {
        return botB;
  }



  public int size()
  {

    if (battleTimeSlices != null )
      return battleTimeSlices.size();
    else
      return battleTimeSlicesSaved.length;

  }


  public int getEndBattleIndex()
    { return endFrameIndex; }

  public int getStartBattleIndex()
    { return startFrameIndex; }


  public Vector getBattleSlices()
    { return battleTimeSlices; }


  private void setBotToShowStatsFor (BBot botIn)
  {
   // BDebug.print (" BBattleRecorder Stats Bot is " +  botIn );
    botToShowStatsFor = botIn;
  }

  public BBot getBotToShowStatsFor ()
  {
    return botToShowStatsFor;
  }

  public String getArenaName ()
  {
    return arenaName;
  }


  // -------------------------------------------------------
  // Channel Table Model...
  // -------------------------------------------------------

  protected transient ChannelTableModel channelsModel;

  public TableModel getChannelsModel()
  {
    if (channelsModel == null)
      channelsModel = new ChannelTableModel();
    return channelsModel;
  }

  protected class ChannelTableModel extends AbstractTableModel
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
	protected int frameIndex;
    protected Vector data;

    protected void setFrameIndex(int i)
    {
    /*
      BDebug.assert((i > -1) && (i < channels.size()),
        "Attempt to access a non-existant channel frame. \n"+
        "This battle was probably created earlier than version 0.1.");  */

      frameIndex = i;
      data = currentStats.getWorldAndFillCache(i);
      fireTableDataChanged();
    }

    public int getRowCount()
    {
      if (frameIndex != 0)
        return (data != null) ? ( data.size() ) : 0;
      else
        return 0;
    }

    public int getColumnCount()
      { return 2; }

    public String getColumnName(int i)
      { return (i == 0) ? "Memory Item" : "Value"; }

    public Object getValueAt(int rowIndex, int columnIndex)
    {
      if (data != null)
      {

          // Dynamic channels...
          if (columnIndex == 0)
            return currentStats.getHeadings().get(rowIndex );
          else
            return data.get(rowIndex );

      }
      else
        return null;
    }
  }


  // -------------------------------------------------------
  // States Table Model...
  // -------------------------------------------------------

  protected transient StateTableModel statesModel;

  public TableModel getStatesModel()
  {
    if (statesModel == null)
      statesModel = new StateTableModel();
    return statesModel;
  }

  protected class StateTableModel extends AbstractTableModel
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
	protected int frameIndex;
    protected Vector data;

    protected void setFrameIndex(int i)
    {
      frameIndex = i;
      data = currentStats.getCacheMemValues();

      fireTableDataChanged();
    }

    public int getRowCount()
      { return (data != null) ? data.size(): 0; }

    public int getColumnCount()
      { return 2; }

    public String getColumnName(int i)
      { return (i == 0) ? "Memory Item" : "Value"; }

    public Object getValueAt(int rowIndex, int columnIndex)
    {

      if (data != null)
      {
          // Dynamic channels...
          if (columnIndex == 0)
            return currentStats.getMemoryHeadings().get(rowIndex );
          else
            return data.get(rowIndex );
      }
      else
        return null;

         /*
      if (data != null)
      {
        Vector row = (Vector) data.get(rowIndex);
        return row.get(columnIndex);
      }
      else
        return null;  */
    }
  }


  // Data model updater...
  // -------------------------------------------------------

  public void updateStatsModels(int i)
  {
    // dont do it if there are no stats
    if (currentStats != null)
    {

      if (channelsModel != null)
        channelsModel.setFrameIndex(i);

      if (statesModel != null)
        statesModel.setFrameIndex(i);
    }
  }


  // Accessor methods...
  // -------------------------------------------------------

  public String getChassisMove(int i)
  {
    if (currentStats == null)
      return "";
    else
      return currentStats.getCacheChassisMove();
  }

  private static Vector emptyVector = new Vector(0);

  public Vector getMoves(int i)
  {
    if (currentStats == null)
      return emptyVector;
    else
      return currentStats.getCacheActionNames() ;

  }

  public Vector getComments(int i)
  {
     if (currentStats == null)
      return emptyVector;
    else
      return currentStats.getCacheComments() ;

  }



  public void updateStats(DefaultTableModel channelsModel, DefaultTableModel statesModel, int FrameToPlay, JLabel lChassisMove, JList lMoves, JList lComments)
  {


  }


  private boolean bothBotsCollided(int start, int end, Vector cols)
  {
    boolean botAc = false;
    boolean botBc = false;

    if (start < 0 || end > cols.size() )
    {
      BDebug.print ("ERROR BBattleRecorder. bothBotsCollided start " +  start);
      BDebug.print ("ERROR BBattleRecorder. bothBotsCollided end " +  end);
      BDebug.print ("ERROR BBattleRecorder. cols.size " +  cols.size());

     }

    BCollisionRecord colrec;
    for (int i = start; i < end; i++ )
    {

      if ( (i >= 0) && (i < cols.size() ) )
      {
        colrec = (BCollisionRecord) cols.get(i);
        if (colrec.getArmComponent().getBot() == botA)
          botAc = true;
        else if (colrec.getArmComponent().getBot() == botB)
          botBc = true;

        if (colrec.getTriggerComponent().getBot() == botA)
          botAc = true;
        else if (colrec.getTriggerComponent().getBot() == botB)
          botBc = true;
      }

    }

    return botAc && botBc;


  }


  private transient float deadlockWindow = 0; // Number of slices that contain deadlocks (within HITS_FOR_LOCK window)
  private transient int lastZeroedDeadlock = 0;
  private transient int noOfNoCollisions = 0;
  private transient int noOfNoCollisionsToClearDeadlock = Math.round( 1f / BGlobal.CALC_TIMESLICE) ; // 1 seconds


  public void updateDeadlockWindow(Vector newCols )
  {
    int startOfWindow = battleTimeSlices.size() - HITS_FOR_LOCK;

    if (startOfWindow > lastZeroedDeadlock)
    {
      // Remove Deadlock Count for Slice at beginning of Deadlock Test Window
      BArenaTimeSlice startSlice =  (BArenaTimeSlice) battleTimeSlices.get(startOfWindow);

      if (   startSlice.isColliding()  )
      {
       // BDebug.print ("BBattleRecorder.updateDeadlockWindow taking away deadlock window " + startOfWindow);
        if (bothBotsCollided(startSlice.collisionStartIndex, startSlice.collisionEndIndex, collisions) )
          deadlockWindow--;

      }
    }

    // Now increment deadlock window if collisions detected in this timeslice
    if (newCols != null)
    {
      if (newCols.size() != 0)
      {
        if( bothBotsCollided (0, newCols.size(), newCols))
          deadlockWindow++;

        noOfNoCollisions = 0;

      }
      else
        noOfNoCollisions++;

      // clear deadlock window if the bots have not collided in 2 seconds
      if (noOfNoCollisions > noOfNoCollisionsToClearDeadlock )
        zeroDeadlock();
    }

  }

  /* DJG - June 2001 Not Necessary - Use get deadlock % instead
  private boolean detectBotLock()
  {

    percent = deadlockWindow / (float) HITS_FOR_LOCK;

    int botADelta = getBotADamageDelta(battleTimeSlices.size() - 1,  (battleTimeSlices.size() - HITS_FOR_LOCK) - 1 );
    int botBDelta = getBotBDamageDelta(battleTimeSlices.size() - 1,  (battleTimeSlices.size() - HITS_FOR_LOCK) - 1 );

    //botA.setArenaComments("Deadlock deadlockWindow "  +  deadlockWindow ) ;
    //botA.setArenaComments("Deadlock HITS_FOR_LOCK "  +  HITS_FOR_LOCK ) ;
   // BDebug.print("Deadlock threshold "  +  percent ) ;
   // BDebug.print("Deadlock Bot A Damage Delta"  +  botADelta ) ;
   // BDebug.print("Deadlock Bot B Damage Delta "  +  botBDelta ) ;

    boolean deadlock = (percent > DEADLOCK_THRESHOLD) &&
                       ( ( botADelta < DEADLOCK_DAMAGEDELTA && botBDelta < DEADLOCK_DAMAGEDELTA)
                         || percent >= DEADLOCK_THRESHOLD_UPPERLIMIT )  ;

    return deadlock;

  }
  */

  private transient double deadlockTimer = 0d;

  public float getDeadlockPercent()
  {
    percent = deadlockWindow / (float) HITS_FOR_LOCK;

    float rawPercent = (percent / DEADLOCK_THRESHOLD) * 100.0f;

    //cap
    if (rawPercent > 100 )
      rawPercent = 100;

    float botADelta = (float) getBotADamageDelta(battleTimeSlices.size() - 1,  (battleTimeSlices.size() - DAMAGE_HITS_FOR_LOCK) - 1 );
    float botBDelta = (float) getBotBDamageDelta(battleTimeSlices.size() - 1,  (battleTimeSlices.size() - DAMAGE_HITS_FOR_LOCK) - 1 );


    // take away percent delta factor
    // DJG June 2001 - Got rid of the (- 10%) hack that kept things at 90%...

    //float deltaMinRatio = 0.5f; // Apply no less than 50% of RawPercent
    //if ( percent > deltaMinRatio )
    //  deltaMinRatio = percent / DEADLOCK_THRESHOLD_UPPERLIMIT; // Override influence of delta damage as deadlock freq increases

    if  ( botADelta >= DEADLOCK_DAMAGEDELTA && botADelta >= botBDelta )
        rawPercent = rawPercent * (DEADLOCK_DAMAGEDELTA / botADelta);
    else if  ( botBDelta >= DEADLOCK_DAMAGEDELTA )
        rawPercent = rawPercent * (DEADLOCK_DAMAGEDELTA / botBDelta);

    // If percent "sticks" over 50% force them to deadlock in another 8 seconds
    if ( rawPercent >= 50 || percent >= DEADLOCK_THRESHOLD_UPPERLIMIT)
    {
      if ( deadlockTimer == 0d )
      {
        deadlockTimer = currentSimTime;
      } else {
        double diffTime = currentSimTime - deadlockTimer;

        //BDebug.print("Deadlock over 50% for "+diffTime);

        rawPercent += (100 - rawPercent) * (diffTime / 8d);
        //cap
        if (rawPercent > 100 )
             rawPercent = 100;

      }
    } else { deadlockTimer = 0d; }

    if (rawPercent < 0)
      rawPercent = 0;

    //BDebug.print("Deadlock: pct="+percent+" raw="+rawPercent+" botAdelta="+botADelta+" botBdelta="+botBDelta);

    return rawPercent;
    //return deadlockWindow;

  }

  private void zeroDeadlock()
  {
    deadlockWindow = 0;
    lastZeroedDeadlock = battleTimeSlices.size();

  }

  public void add(Vector componentsToDetach, Vector collisionsIn)
  //
  {

        int destroyedStart = destroyedComponents.size();
        int destroyedEnd = destroyedComponents.size();

        currentSimTime += (double) BGlobal.CALC_TIMESLICE;

        //BDebug.print ("1");

        // destroyed components
        if (componentsToDetach != null)
                if (componentsToDetach.size() != 0)
                {
                    //destroyedStart = destroyedComponents.size();
                    destroyedComponents.addAll(componentsToDetach);
                    destroyedEnd =  destroyedComponents.size();
                }

        //BDebug.print ("2");

        // collided components
        int collisionsStart =  collisions.size();
        int collisionsEnd =  collisions.size();


        updateDeadlockWindow(collisionsIn);

        if (collisionsIn != null)
                if (collisionsIn.size() != 0)
                {
                        //collisionsStart =  collisions.size()
                        collisions.addAll(collisionsIn);
                        collisionsEnd = collisions.size();

                        if ( getDeadlockPercent() >= 100.0f )
                        {

                          if ( battleTimeSlices.size() - lastDeadLock <  DELTA_HITS_FOR_SAMEDEADLOCK  )
                          {
                            BGlobal.DEADLOCKCOUNT = BGlobal.DEADLOCKCOUNT  + 1;
                            //botA.setArenaComments("DEADLOCK DETECTED BGlobal.DEADLOCKCOUNT " + BGlobal.DEADLOCKCOUNT );

                            if (BGlobal.DEADLOCKCOUNT > BGlobal.DEADLOCKS_TO_RESETFIGHT)
                            {
                              addArenaEvent( deadlockEvent );
                              zeroDeadlock();

                            }

                          }
                          else
                            BGlobal.DEADLOCKCOUNT = 0;

                          lastDeadLock =  battleTimeSlices.size() ;

                       //   botA.setArenaComments("DEADLOCK DETECTED BGlobal.DEADLOCKCOUNT " + BGlobal.DEADLOCKCOUNT );
                        //  botB.setArenaComments("DEADLOCK DETECTED BGlobal.DEADLOCKCOUNT " + BGlobal.DEADLOCKCOUNT );

                          BGlobal.DEADLOCKDETECTED = true;

                        }
                        else
                          BGlobal.DEADLOCKDETECTED = false;
                }
                else
                  BGlobal.DEADLOCKDETECTED = false;
        else
          BGlobal.DEADLOCKDETECTED = false;

        // time slices
        if (battleTimeSlices.size() == 0)
          battleTimeSlices.add( new BArenaTimeSlice(botA, botB, destroyedStart, destroyedEnd, collisionsStart, collisionsEnd , new BArenaTimeSlice()) );
        else
          battleTimeSlices.add( new BArenaTimeSlice(botA, botB, destroyedStart, destroyedEnd, collisionsStart, collisionsEnd , (BArenaTimeSlice) battleTimeSlices.get(battleTimeSlices.size() - 1)  ));

        aStats.addSlice(botA);
        if (recordBothBotsStats)
          bStats.addSlice(botB);
        else
          botB.getArenaComments().clear();

  }


  public void addArenaEvent(BArenaEvent event )
  {
    arenaEvents.add(  new ArenaEventFrameIndexPair(event, battleTimeSlices.size() )  );

  }



  public float getTimeSlice()
    { return timeSlice; }

  public float getTimeElapsed(int frame)
    // Returns time elapsed (in seconds) at the specified frame index.
    // Time is measured relative to the actual battle, and excludes
    // start & victory move sequences.
  {
    int min = startFrameIndex;
    int max = endFrameIndex;
    float f = Math.min(max, Math.max(0, frame - min));
    float n = max - min;

    // Check for zero-length battle...
    if (n <= 0f) return 0f;

    float ratio    = f / n;
    float duration = n * timeSlice;

    return ratio * duration;
  }

  /*
  public Vector getTimeSlices()
  {
    return arenaTimeSlices;
  }

  public BArenaObjects getArenaObjects()
  {
    return arenaObjects;
  }   */

  public String getBotVsBot ()
  {
     return botA.getName() + " vs. " +botB.getName();
  }

  public BStats getStats (BBot theBot)
  {



    // if there is a current stats return current Stats
    // this means that the recording is being view in a BAttle player
    if (currentStats != null)
      return currentStats;

    // otherwise return the stats for a bot
    // (the recording is in a simulation )
    if (theBot == botA)
      return aStats;
    else
      return  bStats;

  }

  /* DJG June 2001
  public void setStats(BStats st, BBot bt)
  {
    BDebug.print ("BBattleRecorder.setStats(BStats st, BBot bt) HAS BEEN DEPRECATED use setStats(BStats st)");
    setStats(st);
  }
  */

  public void setStats(BStats st)
  {
        currentStats = st;

        if (st.getOwnerID().equals(botA.getBotSerialID())  )
          setBotToShowStatsFor(botA );
        else if (st.getOwnerID().equals(botB.getBotSerialID())  )
          setBotToShowStatsFor (botB );
        else
          BDebug.print ("WARNING BBattleRecording.setStats invalid bot ");

  }


  public void convertToSaveFormat()
  {
     // Convert collisions to array

        if (collisions != null)
        {

          // collisions
          collisionsSaved = new BCollisionRecord[collisions.size()];
          for (int i = 0; i < collisions.size(); i++)
             collisionsSaved[i] =   (BCollisionRecord)collisions.get(i);

          collisions = null;

          // destroyed components
          destroyedComponentsSaved = new BComponent[destroyedComponents.size()];

          for (int i = 0; i < destroyedComponents.size(); i++ )
            destroyedComponentsSaved[i] = (BComponent)destroyedComponents.get(i);

          destroyedComponents = null;

          // battle slices
          battleTimeSlicesSaved = new BArenaTimeSlice[battleTimeSlices.size()];
          for (int i = 0; i < battleTimeSlices.size(); i++ )
            battleTimeSlicesSaved[i] = (BArenaTimeSlice)battleTimeSlices.get(i);

          battleTimeSlices = null;


          // arena Enevents
          arenaEventsSaved = new BArenaEvent[arenaEvents.size()];
          arenaEventsFrameIndex = new int[arenaEvents.size()];

          for (int i = 0; i < arenaEvents.size(); i++ )
          {
            ArenaEventFrameIndexPair pair = (ArenaEventFrameIndexPair)arenaEvents.get(i);
            arenaEventsSaved[i] = pair.event;
            arenaEventsFrameIndex[i] = pair.frameIndex;

          }
          arenaEvents = null;



          //BDebug.print ("BBAttleRecording .convcertToSaveFormat calling Garbage Collection");
          System.gc();

        }





  }

  public void saveToFile (String filename, BBot a, BBot b)
  {
     try
     {

        if (   BGlobal.RECORDINGSUFFIX.equalsIgnoreCase(     filename.substring(filename.length() - 4 )  )  )
          filename = filename.substring(0, filename.length() - 4);

        if ( (a == botA ) || (a == botB) )
        {
          String aStatsString =  filename + BGlobal.STATSSUFFIXA;
          BStats aStats = getStats(a);
          aStats.saveToFile(aStatsString );

        }
        if ( (b == botA ) || (b == botB) )
        {
          String bStatsString =  filename + BGlobal.STATSSUFFIXB;
          BStats bStats =  getStats(b);
          bStats.saveToFile(bStatsString );

        }

        convertToSaveFormat();

        filename = filename +  BGlobal.RECORDINGSUFFIX;
       // BDebug.print ("BBattleRecorder.saveToFile " + filename + "Save starting");

        //fileName = filename;
        FileOutputStream fout = new FileOutputStream(filename);
        ObjectOutputStream out = new ObjectOutputStream(fout);
        out.writeObject(this);
        out.close();
       // BDebug.print ("BBattleRecorder.saveToFile " + filename +"Save completed");
     }
     catch (Exception e1)
     {
       BDebug.print("BBAttleRecording SaveToFile Error " + e1.getMessage());
     }

  }



  public void preprocessForPlayer(  BArenaSystem backgroundSound,
                                    BVirtualRefModel ref,
                                    BBattlePlayerAnimation ia,
                                    BSimulationCanvas universe  )
  {
        this.backgroundSound = backgroundSound;
        this.ref = ref;
        this.activeAnimation = ia;
        this.universe = universe;
        deadlockAni = new BDeadlockAnimation(botA, botB, this);
        clearEvent = new BArenaEvent()
        {
          /**
			 * 
			 */
			private static final long serialVersionUID = -1L;

		public int getStateModifier()
            { return 0; }

          public String getArenaEventType()
            { return CLEAR_EVENTS; }

        };



        // add in for old battles
        if (arenaEventsSaved  == null)
        {
          arenaEventsSaved = new BArenaEvent[0];
          arenaEventsFrameIndex = new int[0];

        }

        if (arenaEventsSaved.length == 0)
        {
          nextAEventFrame = battleTimeSlicesSaved.length;
          aEventIndex = 0;
        }
        else
        {
          nextAEventFrame = arenaEventsFrameIndex[0];
          aEventIndex = 0;
        }

        //BDebug.print("BBattleRecord.preProcess: Starting...");
        //BDebug.printFreeMem();

        for (int i = 0; i < battleTimeSlicesSaved.length; i++ )
        {

                battleTimeSlicesSaved[i].preprocessForPlayer( i  );

                if ( (i % CRITICAL_SNAP_SPACING == 0) && (i != 0)  )
                {
                  Vector completed =  getSnapForAllJoints ( i);
                  battleTimeSlicesSaved[i].setCriticalSnap(completed);
                }



        }




        for (int i = 0; i < destroyedComponentsSaved.length; i++)
        {
                /*
                if ( destroyedComponentsSaved[i] instanceof BChassis )
                {

                        DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)(destroyedComponentsSaved[i]).getTreeNode();
                        Enumeration enumerator = rootNode.breadthFirstEnumeration();

                        while (enumerator.hasMoreElements())
                        {
                                DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
                                ((BComponent)node.getUserObject()).initDestructSequence();


                        }
                }
                else */

        }

      for (int i = 0; i < collisionsSaved.length; i++)
      {
        if (!collisionsSaved[i].getNullified() )
          collisionsSaved[i].initSound();
      }


      // SJS [14/03/2001] - Look for the end frame, and store its index.
      String type;
      endFrameIndex = battleTimeSlicesSaved.length-1;

      for (int i=arenaEventsSaved.length-1; i>=0; i--)
      {
        type = arenaEventsSaved[i].getArenaEventType();

        if (type.equals(BArenaEvent.END) ||
            type.equals(BArenaEvent.VICTORY_MOVES_START))
        {
          endFrameIndex = arenaEventsFrameIndex[i];
          break;
        }
      }

      // SJS [08/04/2001] - Look for start of fighting frame...
      startFrameIndex = 0;

      for (int i=0; i<arenaEventsSaved.length; i++)
      {
        type = arenaEventsSaved[i].getArenaEventType();

        if (type.equals(BArenaEvent.STARTMOVES_FIN))
        {
          startFrameIndex = arenaEventsFrameIndex[i];
          break;
        }
      }



      /*
      if (victoriousBot == null)
      {
        // TODO: Fix this for the case where both bots die.
        endFrameIndex = battleTimeSlicesSaved.length-1;
      }
      else
      {
        int last = arenaEventsSaved.length-1;

        for (int i=last; i>=0; i--)
          if (arenaEventsSaved[i].getArenaEventType().equals(BArenaEvent.VICTORY_MOVES_START))
            { endFrameIndex = arenaEventsFrameIndex[i]; break; }
      }
      */

      jointsToGetTransformsFor.clear();
      completedSlice.clear();

      BDebug.print("BBattleRecord.preProcess complete.");
      //BDebug.printFreeMem();


  }

  public void optimise(BBot a)
  {
     BComponent[] bComps = a.getBreathComponents();

     // SJS 13/03/2001...
     // Allow particle systems to be attached to bots.
     a.getChassis().getLiveNode().setCapability(Group.ALLOW_CHILDREN_EXTEND);
     a.getChassis().getLiveNode().setCapability(Group.ALLOW_CHILDREN_WRITE);

     
     if (a == this.getBotToShowStatsFor())
     {
      // for zones
      a.getChassis().getOrientNode().setCapability(Group.ALLOW_CHILDREN_EXTEND);
      a.getChassis().getOrientNode().setCapability(Group.ALLOW_CHILDREN_WRITE);

      for (int i = 0; i < bComps.length; i++ )
        if (bComps[i] instanceof BSensor)
        {
          bComps[i].getLiveNode(true).setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
          bComps[i].getLiveNode(true).setCapability(BranchGroup.ALLOW_CHILDREN_WRITE );
        }
        
     }


     a.getLiveNode(true).setCapability(Node.ALLOW_BOUNDS_READ);



     for (int i = 0;  i < destroyedComponentsSaved.length; i++)
     {
      if (destroyedComponentsSaved[i].getLiveNode() != null)
        if (!destroyedComponentsSaved[i].getLiveNode().isLive() )
          destroyedComponentsSaved[i].getLiveNode().setCapability(BranchGroup.ALLOW_DETACH);

      if (destroyedComponentsSaved[i].getDirectLiveParentNode() != null)
        if (!destroyedComponentsSaved[i].getDirectLiveParentNode().isLive() )
        {
          destroyedComponentsSaved[i].getDirectLiveParentNode().setCapability(Group.ALLOW_CHILDREN_EXTEND);
          destroyedComponentsSaved[i].getDirectLiveParentNode().setCapability(Group.ALLOW_CHILDREN_READ);
          destroyedComponentsSaved[i].getDirectLiveParentNode().setCapability(Group.ALLOW_CHILDREN_WRITE);

        }


     }
  }

 

  public void init()
  {
     lastProgMessage = "Initializing " + botA.getName();
     botA.init(false, null, null,true) ;
     loadPercent = LOAD_BOTA;
     lastProgMessage = "Initializing " + botB.getName();
     botB.init(false, null, null,true) ;
     loadPercent = LOAD_BOTB;

     loadPercent = LOAD_SOUND;


  }


  public String  getProgressMessage()
  {
    if ( loadPercent < LOAD_BOTA )
      return botA.getProgressMessage();
    else if
      ( loadPercent < LOAD_BOTB )
      return botB.getProgressMessage();

    return lastProgMessage;
  }

  public int getProgressPercent()
  {
    if (loadPercent < LOAD_BOTA )
      return LOAD_OBJECT + (int)((float)LOAD_BOTTIME  * ( (float)botA.getProgressPercent() / 100f) );
    else if (loadPercent < LOAD_BOTB )
      return LOAD_BOTA + (int)((float)LOAD_BOTTIME  * ( (float)botB.getProgressPercent() / 100f) );
    else
      return loadPercent;
  }

  public static BBattleRecorder loadFromFile (String filename)
  { return  loadFromFile ( filename,  true); }

  public static BBattleRecorder loadFromFile (String filename, boolean init)
  {
   // BDebug.print("BBattleRecord.loadFromFile: loading..." + filename + " ...");

    BBattleRecorder battleRecorder = null;
    try
    {
      FileInputStream fin = new FileInputStream(filename);
      ObjectInputStream in = new ObjectInputStream(fin);
      battleRecorder = (BBattleRecorder)in.readObject();
      in.close();
      fin.close();

   //   BDebug.print("BBattleRecord.loadFromFile finished, initialising 3DStructure ...");
      //loadPercent = LOAD_OBJECT;


      if (init)
        battleRecorder.init();

    }
    catch (Exception e)
    {
      BDebug.print ("BBattleRecorder.loadFromFile Exception " + e.getMessage() );
    }



    String statsStringA = filename.substring(0, filename.length() - 4 ) + BGlobal.STATSSUFFIXA   ;
    BDebug.print("BBattleRecord.loadFromFile: Opening stats file A." + statsStringA);

    BStats statsa = BStats.loadFromFile( statsStringA);

    if (statsa != null)
        battleRecorder.setStats(statsa);



    return battleRecorder;
  }

  

  private void writeObject (ObjectOutputStream stream) throws IOException
  {
    /*
      HSC 15-July-98
      As this method is called as part of the Serialization process, this
      is where the method makeLight is called to remove the AI and move information
      from the bots prior to Serialization
    */

   BBot temp = botA;
   botA = botA.clone_shallow();
   ///if(recordBothBotsStats)
   // botA.removeThumbnails();
   if (victoriousBot == temp)
     victoriousBot = botA;

   temp = botB;
   botB = botB.clone_shallow();
   //botB.removeThumbnails();

   if (victoriousBot == temp)
     victoriousBot = botB;

   stream.defaultWriteObject();

  }

  public void setWinString(String w)
  { winString = w; }

  public String getWinString()
  {
    if (winString == null)
      return "";
    else
      return winString;
  }


  public String getVictorName()
  {
    if (getVictoriousBot() == null)
      return "Draw" ;
    else
     return  getVictoriousBot().getName().toUpperCase() + " WINS.";

  }

  public String getFightDurationString()
    { return BMath.format(Math.round(getFightDuration()), 1 ); }

  public float getFightDuration()
  {
    int startMoveEndIndex = 0;
    int endBattleIndex = battleTimeSlicesSaved.length;

    for (int i = 0 ; i < arenaEventsSaved.length; i++)
      if (arenaEventsSaved[i].getArenaEventType().equals(BArenaEvent.STARTMOVES_FIN) )
        startMoveEndIndex = arenaEventsFrameIndex[i];
      else if (arenaEventsSaved[i].getArenaEventType().equals(BArenaEvent.VICTORY_MOVES_START) )
        endBattleIndex = arenaEventsFrameIndex[i];

    int victorySize       = battleTimeSlicesSaved.length -  endBattleIndex;
    int notOfBattleSlices = battleTimeSlicesSaved.length -1 - startMoveEndIndex - victorySize;

    return timeSlice * notOfBattleSlices;
  }


  public String getTournamentName()
  {
    if (tournamentName == null)
      tournamentName = "Training bout";
      
    return tournamentName;
  }

  public void setTournamentName( String n )
    { tournamentName = n; }

  public void setRoundName( String name)
  { roundName = name; }

  public String getRoundName()
  {
    if (roundName == null)
      roundName = "";
      
    return roundName;
  }


  // SJS [20/03/2001] - Added accessor methods for determining
  // what kind of battle this is.  Used to do special ending
  // animations in the battle player.

  public void setTournament(boolean isTournament)
    { this.isTournament = isTournament; }

  public boolean isTournament()
    { return isTournament; }

  public void setFinal(boolean isFinal)
    { this.isFinal = isFinal; }

  public boolean isFinal()
    { return isFinal; }


  public int getDeadlockCount()
  {
    // for old battles
    if (arenaEventsSaved == null)
      return 0;

    int deadlocks = 0;
    for (int i = 0 ; i < arenaEventsSaved.length; i++)
      if (arenaEventsSaved[i].getArenaEventType().equals(BArenaEvent.DEADLOCK) )
        deadlocks++;

    return deadlocks;
  }
  // inner class used for processing


  class ArenaEventFrameIndexPair
  {
    public BArenaEvent event;
    public int frameIndex;

    public  ArenaEventFrameIndexPair(BArenaEvent event, int frameIndex)
    {
      this.event = event;
      this.frameIndex = frameIndex;
    }

  }




}