package roboforge;

import javax.media.j3d.*;
import javax.vecmath.*;
import javax.swing.tree.*;
import java.util.*;
//import java.io.*;

import roboforge.attributes.*;
import roboforge.btree.*;
import roboforge.math.*;
//import javax.sound.sampled.*;


public class BChassis extends BComponent
/*


* Description:
  Also holds two members of type BComponentTree
  and BMoveList.  The handle for the BComponentTree member is passed to
  BBot to be used as a handle for the BBot BComponent tree.

  The BMoveList member is used to control movements of the components
  attached to the chassis.

* Example:
* Modification History ( Name, Date, Notes )
  Created HSC, 20 April.
  HSC 20 April Added Chassis lock member and get and set methods

*/

{
  static final long serialVersionUID = -1L;


  public static int ADD_MODE_PASSIVE = 1;               // dont override existing move
  public static int ADD_MODE_OVERRIDE = 2;              // overide existing move, update variables if the move is the same.
  public static int ADD_MODE_UPDATE_BUT_NOT_OVERIDE = 3;// update variables if the current move is the same, but do not override move if it is different

  private transient float chassisEnergyUsage;            // Watts used to move 1 KG at 1 m/s/s (W/KG/M/S)
  public float chassisPowerOutput;            // KW
  public float loadCapacity; // Kg;


 // SET CONSTANTS
  public float acceleration;            // m/s^2  (No Torque)
  public float chassisMaxVelocity;            // The maximum speed of the vhassis in m/s per TORQUE
  public float sidewaysMaxVelocity;
  public float backwardsMaxVelocity;

  float rotateSpeed;                          // Radians / Sec
  float rotateAcceleration;                   //Radians / Sec^2
  float maxRotateVelocity;

  private transient BArenaObjectPosition arenaPosition = new BArenaObjectPosition();
  private transient BArenaObjectPosition resultingPosition = new BArenaObjectPosition();  // the position that is a result a collision
                                                   // only uses if the movetype = BMoveList.CHASSIS_RESULTANT_MOVE
  private transient Vector3f resultingDirectionVector;


  private BComponentTree componentTree;
  private BMoveList chassisMoveList;  // The list of all possile moves performed by the Chassis
                                      // (Complex Chassis)

  // TRANSIENT BATTLE ENGINE WORKING VARIABLES

  transient double currentX;
  transient double currentY;

  // PreviousPosition of Chassis
  transient double prevX;
  transient double prevY;

  transient private BArenaObjectPosition[] prevArenaPosition;

  //public transient float moveDelay = 0.0f; // Used to syncronize movement to a snaposhot.
                                            // if moveDelay > timeSlice, a timeSlice is removed instead of perfroming move
                                            // THIS IS ONLY USED FOR CHASSIS-GROUND ORIENTATION, Not Movement over plane

  public transient float timeToNextSnap = 0.0f;
                                            // Metavariable to hold time it will take to move to next snapshot
  private transient TransformGroup arenaRotateGroup;  // the roation in the arena
  private transient TransformGroup displacementGroup; // the displacement in the arena
  private transient TransformGroup orientationNode;
  transient TransformGroup negateNode = null;  // Negate Node

  private transient BMove currentChassisMove = null;
  private transient int currentChassisMoveType;   // Must be one of the ChassisMoveList Constants
  //transient boolean resultantMoveActive = false;
  private transient float distanceToMove;
  private transient float displacementOffset; // angleoffset to any forward movemnent

  // unlike joints these velocities are always positive because the are not restricted to 2-d movement
  transient float currentDisplaceVelocity = 0.0f;
  transient float currentRotateVelocity = 0.0f;


  transient boolean returnToStandActivated = true;
  transient boolean keepVelocity = false;
  transient float height; // Used in BBotShot to avoid having to call calculateHeight again

  transient public float distThisSlice = 0f;
  transient public float distanceMovedThisSnapshot = 0f; // Used for chassis joint movement calculations
  transient public float distanceThisSnapshot = 0f; // Used for chassis joint movement calc

  private static float MAX_CHASSIS_WEIGHT_MULTIPLIER = 1.3f;
  private static float TIMETORETURNTOSTAND = 0.5f;

 // private transient int completeChassisDamage = 0;
  private transient BComponent[] chassisAndFeet;  // set up in chassis.init
  private transient BMove chassisWaitingMove;
  private transient boolean  feetNotValid = false;

//  private transient Vector sideWaysVector; // convenience vector for processing chassis that cannot move sideways

  private transient BranchGroup wrapperNode; // set up in getStructure3D

  private transient boolean containsWheels = false;

  // DJG June 2001
  private transient float simTime = 0;  // millisecs - Updated by BSimulation.run()
  public transient float lastExecTime = 0;  // millisecs - Updated by BChassis.execute
  private transient boolean executed = false;
  public static String COMPONENT_TYPE = "Chassis";
  private transient float pauseTime = 0; // Pause the chassis move after an impact



  public BChassis (String nameIn, String filename)
  {
      super(nameIn, filename);

      // Chassis Locked
      this.setChassisLock(true);

      // Add this chassis as the root node of the component tree.
      componentTree = new BComponentTree(this);
  }

  public void initForArena(BChassis theChassis, BBot owner)
  {
    super.initForArena( theChassis, owner);

    //arenaPosition = new BArenaObjectPosition();
    //resultingPosition = new BArenaObjectPosition();

    /*
    if ( prevArenaPosition == null )
      initPrevArenaPositions();

    prevArenaPosition[0].set(arenaPosition);
    */

    initPrevArenaPositions();

    chassisWaitingMove = null;

    if (  !BGlobal.CONSTRUCTING_CHASSISBOT )
      terminateChassisMove(true);

    //clearCompleteChassisDamage();
    createChassisAndFeetVector();


    // check if the bot is wheeled
    containsWheels = false;

    for (int i = chassisAndFeet.length - 1; i >= 0 && !containsWheels; i-- )
      if ( chassisAndFeet[i] instanceof BWheel )
        containsWheels = true;
  }

  public void initPrevArenaPositions()
  {
    //BDebug.print("initPrevArenaPositions: called - "+BGlobal.FULL_ROLLBACK_DEPTH);

    prevArenaPosition = new BArenaObjectPosition[BGlobal.FULL_ROLLBACK_DEPTH];

    for (int i=0; i < BGlobal.FULL_ROLLBACK_DEPTH; i++ )
  	{
  		prevArenaPosition[i] = new BArenaObjectPosition();
  	}
  }


  public BComponentTree getChassisComponentTree ()
  {
    return componentTree;
  }

  /*
  public void addToCompleteChassisDamage(int x)
  {
    completeChassisDamage = completeChassisDamage + x;
  }

  public void clearCompleteChassisDamage()
  {
    completeChassisDamage = 0;
  }    */

  public void addToDamageCurrent(int x)
 {

    //BDebug.print ("BChassis.addToDamage adding to bot " + java.lang.Math.min( x, damageMax - damageCurrent)   );

    getBot().addToBotDamage( java.lang.Math.min( x, damageMax - damageCurrent) );
    damageCurrent = java.lang.Math.min( damageCurrent + x, damageMax );

   // BDebug.print ("BChassis.addToDamage x " + x  );
   // BDebug.print ("BChassis.addToDamage bot HP" + getBot().getTotalHealth()  );
   /// BDebug.print ("BChassis.addToDamage current Dam " + getBot().getCurrentDamage() );
  //  BDebug.print ("BChassis.addToDamage max HP " + damageMax  );
   // BDebug.print ("BChassis.addToDamage damageCurrent " + damageCurrent );

 }


  public void setChassisComponentTree (BComponentTree newTree)
  {
    componentTree = newTree;
  }

  public BMoveList getChassisMoveList ()
  {
    return chassisMoveList;
  }

  public BMove getChassisMove(int i)
  {
    if (chassisMoveList == null)
      return null;
    else
      return chassisMoveList.getMove(i);
  }


  public float getLoadCapacity()
    { return loadCapacity; }

  public void setChassisMoveList (BMoveList chassisMoveListIn)
  {
    chassisMoveList = chassisMoveListIn;
  }

 // local temp variables,  to reduce garbage collection of player
 // DO NOT USE ANYWHERE BUT the Method Below
 // or else threads will fuck you up BIG TIME
 transient private static Vector3f setArenaPositiontempV = new  Vector3f();
 transient private static Transform3D setArenaPositiontempT = new Transform3D();

 public void setArenaPosition(float x, float z)
 {

     // BDebug.print("New x,z  " + x + " , " + z);

      /*
      if ( updatePrev )
      {
        if ( prevArenaPosition == null )
          initPrevArenaPositions();

        for (int i=0; i < BGlobal.FULL_ROLLBACK_DEPTH-1; i++ )
  			{
  				prevArenaPosition[i+1].set(prevArenaPosition[i]);
  			}
        prevArenaPosition[0].x =  arenaPosition.x;
        prevArenaPosition[0].z =  arenaPosition.z;
      }
      */

      arenaPosition.x = x;
      arenaPosition.z = z;

      applyArenaDisplacement();

 }

 private void setArenaFacing(float newFacing)
 {

   /*
   if ( prevArenaPosition == null )
      initPrevArenaPositions();

   for (int i=0; i < BGlobal.FULL_ROLLBACK_DEPTH-1; i++ )
   {
  	 prevArenaPosition[i+1].set(prevArenaPosition[i]);
   }

   if ( arenaPosition != null )
    prevArenaPosition[0].set(arenaPosition);
   */

   arenaPosition.facing = newFacing;
   applyArenaFacing();

 }


 public void upgradeComponents()
 {
    DefaultMutableTreeNode tree = componentTree.getRoot();
    Enumeration e = tree.breadthFirstEnumeration();
    chassisAndChassisLockWeight = 0f;

    while (e.hasMoreElements())
    {
      BComponentNode node = (BComponentNode) e.nextElement();
      node.getComponent().upgrade();

      chassisAndChassisLockWeight += node.getComponent().getWeight();
    }

 }

 public void upgrade (BComponent c, boolean upgradeMoves)
 {
    BChassis upgradeTwin = (BChassis) c;
    super.upgrade(upgradeTwin);

    chassisEnergyUsage = upgradeTwin.chassisEnergyUsage;
    rotateSpeed = upgradeTwin.rotateSpeed;
    acceleration = upgradeTwin.acceleration;
    chassisMaxVelocity = upgradeTwin.chassisMaxVelocity;
    sidewaysMaxVelocity = upgradeTwin.sidewaysMaxVelocity;
    backwardsMaxVelocity = upgradeTwin.backwardsMaxVelocity;
    rotateAcceleration = upgradeTwin.rotateAcceleration;
    maxRotateVelocity = upgradeTwin.maxRotateVelocity;
    loadCapacity = upgradeTwin.loadCapacity;

    if (upgradeMoves)
      if (this.getBot() != null)
      {
        chassisMoveList.upgrade( upgradeTwin.chassisMoveList, getBot() );
      }

 }
 public void upgrade(BComponent c)
 {
    upgrade(c, true);


 }



 public void upgrade()
 {
    BChassis upgradeTwin =   (BChassis)BComponent.loadFromFile( BGlobal.CHASSISPATH+getComponentFilePath() , false);

   // BDebug.print (getComponentFilePath() );
    upgrade(upgradeTwin);
 }

 public  void setFilePath(String fn)
  // strips off header
  {

    componentFilePath = fn.substring(BGlobal.CHASSISPATH.length(), fn.length());


  }


 private void setArenaPosition(float y)
 {
      //prevArenaPosition.y =  arenaPosition.y;

      arenaPosition.y = y;

      applyArenaDisplacement();

 }


   public void applyArenaDisplacement()
  {
      if (displacementGroup != null)
      {
        //BDebug.print ("ApplyArenaDisplacement called ");
        setArenaPositiontempV.x = arenaPosition.x;
        setArenaPositiontempV.y = arenaPosition.y;
        setArenaPositiontempV.z = arenaPosition.z;
        setArenaPositiontempT.set(setArenaPositiontempV);

        displacementGroup.setTransform(setArenaPositiontempT);
      }

  }



 public void mouseMove(int x,int z,int x_last,int z_last)
 {

      float STEPSCALE = 2.0f;

      float xstep = (x_last - x )* STEPSCALE;
      float zstep = (z_last - z) * STEPSCALE;


      arenaPosition.x = arenaPosition.x - xstep;
      arenaPosition.z = arenaPosition.z - zstep;


      setArenaPosition(arenaPosition.x, arenaPosition.z);

     // BDebug.print("z " +   arenaPosition.z);
     // BDebug.print("x " +   arenaPosition.x);


 }

  private static Transform3D mouseRotate_tempT = new Transform3D();

  public void mouseRotate(int x,int z,int x_last,int z_last)
 {

      float STEPSCALE = 0.02f;
     // float xstep = (x_last - x )* STEPSCALE;
      float zstep = ( z_last - z) * STEPSCALE;

      arenaPosition.facing = arenaPosition.facing +  zstep;

   
      //tempT.setRotation(jointRotate);
      mouseRotate_tempT.rotY(arenaPosition.facing);

      arenaRotateGroup.setTransform(mouseRotate_tempT);

      //BDebug.print("arenaPosition.facing" + arenaPosition.facing);


      //Vector3f test = directionVector();


 }

 public void mouseElevate(int y,int y_last)
 {

      float STEPSCALE = 2.0f;

      float ystep = ( y_last - y) * STEPSCALE;

      arenaPosition.y = arenaPosition.y - ystep;

      setArenaPosition(arenaPosition.y);

 }





 // Local Variables.
 // placed out here to stop memory burn
 // DO NOT USE ANYWAHERE EXCEP IN Calculate Height
// private static Transform3D calculateHeight_negateNodeT = new Transform3D();
 //private static Transform3D calculateHeight_oldTransform = new Transform3D();
 //private static Transform3D calculateHeight_identityy = new Transform3D();
 private static Transform3D calculateHeight_myT  = new Transform3D();
 private static Vector3f calculateHeight_myV = new Vector3f();


 public float calculateHeight( )
 {
   // BDebug.print("BChassis calculateHeight  " );


    if (feetNotValid)
      createChassisAndFeetVector();


    // working variables
    float lowest = Float.MAX_VALUE;
    float minY  =  Float.MAX_VALUE;

    if (this.getBot().isTipped() )
    {
      BComponent[] components = this.getBot().getBreathComponents();

      for (int i = components.length - 1; i >=0; i--)
      {
        if (components[i].getChassisLock() )
        {
          BComponent current =  components[i];
          minY = current.getArbitraryShape_live().getLowestY();
          lowest = java.lang.Math.min( lowest, minY);
        }
      }

    }
    else
      for (int x = 0; x < chassisAndFeet.length ; x++ )
      {
        BComponent current =  chassisAndFeet[x];
        minY = current.getArbitraryShape_live().getLowestY();
        lowest = java.lang.Math.min( lowest, minY);
      }

    getLiveNode().getLocalToVworld(calculateHeight_myT);

    calculateHeight_myT.get(calculateHeight_myV);

    float myHeight = calculateHeight_myV.y;
    float theHeight;

    if (myHeight > lowest)
        theHeight =  myHeight - lowest;
    else
        theHeight = myHeight;


    //BDebug.print(" Calculate height " + theHeight + " scale " + scale);

    return  theHeight ;





 }

 public void clearWheelVelocity()
  {
    if (chassisAndFeet != null)
      for (int x = 0; x < chassisAndFeet.length ; x++ )
        if (chassisAndFeet[x] instanceof BWheel)
          ((BWheel)chassisAndFeet[x]).setJumped();

  }


 public void setLiveNodes(    TransformGroup newDisplacementGroup , TransformGroup newArenaRotateGroup)
 {
      displacementGroup = newDisplacementGroup;
      arenaRotateGroup = newArenaRotateGroup;
      //Transform3D tempTran = new Transform3D();

      //Vector3f test = BPhysics.directionVector(arenaPosition.facing);

      //tempTran.set(new Vector3f(0,100.0f, 0 ));

      /*
      float y =  calculateHeight();
      BDebug.print("height " + y / BGlobal.ARENA_SCALE );
      tempTran.set(new Vector3f(0,y / BGlobal.ARENA_SCALE, 0 ));
      orientationNode.setTransform(tempTran);
      */

      // Set the default position

     // setHeight();




 }




 public void setConstructionChassisMove(BMove newChassisMove,float distance,  int chassisMoveType)
 {

 // BDebug.print ("BChassis.setConstructionChassisMove distance " +  distance );

  currentChassisMoveType =  chassisMoveType;
  currentChassisMove = newChassisMove;
  currentDisplaceVelocity = 0.0f;
  returnToStandActivated = false;
  newChassisMove.resetMove(true,0,0);
  displacementOffset = 0f;


   if ( (currentChassisMoveType == BMoveList.CHASSIS_ROTATE_LEFT) || (currentChassisMoveType == BMoveList.CHASSIS_ROTATE_RIGHT) )
       distanceToMove = distance * BPhysics.RAD_CONVERT;
  else
       distanceToMove = distance;


 }


 public boolean setNewChassisMove( BMove newChassisMove, float distance, int addMode, BMove chassisWaitingMove )
 {
  return setNewChassisMove(  newChassisMove,  distance,  addMode, 0f, chassisWaitingMove );
 }


 public boolean setNewChassisMove( BMove newChassisMove, float distance, int addMode, float displacementOffsetnew, BMove chassisWaitingMove )
 // distance = metres if the move is a plane move, or degrees if the move is a rotation move
 // + displacementOffset comes in as degrees, but gets stored as radians
 {

  if (distance <= 0 || Float.isNaN(distance))
  {
    //getBot().setArenaComments("Invalid Chassis Move Request: " + newChassisMove + " for " + distance + " units.");
    //BDebug.print(this.toString()+" - Invalid Chassis Move Request: "+newChassisMove+" Distance="+distance);
    return false;
  }


  if (getChassisMoveList().indexOf(newChassisMove) == BMoveList.CHASSIS_STAND )
    return false;
  // never overide a resultantmove
  else if (  currentChassisMoveType == BMoveList.CHASSIS_RESULTANT_MOVE)
  {
    // BDebug.print("BChassis.setNewChassisMove:resultantMove in progress - dropping new potential ");
    return false;


  }

  // if passive and anycurent moves exist, then do not override.
  // return to stand can always be overridden
  else if(  (addMode ==  ADD_MODE_PASSIVE )  &&  (currentChassisMove != null && currentChassisMoveType != BMoveList.CHASSIS_STAND) )
  {
   //BDebug.print("BChassis.setNewChassisMove: active in progress - dropping new potential ");
    return false;
   //BDebug.print("BChassis.setNewChassisMove: active " + currentChassisMove);
   //BDebug.print("BChassis.setNewChassisMove: new " + newChassisMove);


  }
  // if update, and the currentMove != newMove     and the currentMOve is not null
  // then do nothing
  else if(  (currentChassisMove != null) &&

            (addMode ==  ADD_MODE_UPDATE_BUT_NOT_OVERIDE )
            &&  ( currentChassisMoveType !=   getChassisMoveList().indexOf(newChassisMove))

          )
  {
    // do nothing
     return false;
    
  }
  else if ( standDown(lastExecTime) )
  {
    //BDebug.print("BChassis.setNewMove: Standown "+newChassisMove+" ("+(simTime - lastExecTime) * 1000+")");
    return false;  // Stand Down Chassis Move Request
  }
  else
  {

    // if the moves are the same, just update the variables

    if ( ( currentChassisMoveType ==   getChassisMoveList().indexOf(newChassisMove))    )
    {
       //BDebug.print("BChassis.setNewChassisMove: overideOn or update on  same move detected ");

    }
    else
    {
      //BDebug.print("BChassis.setNewChassisMove: different move detected, reseting ");
      currentDisplaceVelocity = 0.0f;
      returnToStandActivated = false;

      currentChassisMove = newChassisMove;
      currentChassisMove.resetMove(true,0,0);

    }



    displacementOffset =  displacementOffsetnew * BPhysics.RAD_CONVERT;
    //BDebug.print("BChassis.setNewChassisMove: new displacementOffSet is " + displacementOffset);
    this.chassisWaitingMove = chassisWaitingMove;


    if ( getChassisMoveList() == null )
    {
      //BDebug.print("BChassis.setNewChassisMove: is Null .. Using BGlobal.CHASSIS_BOT_MOVE_TYPE" );
      currentChassisMoveType =  BGlobal.CHASSIS_BOT_MOVE_TYPE;
    }
    else
    {
      currentChassisMoveType = getChassisMoveList().indexOf(newChassisMove);
    }

    if ( (currentChassisMoveType == BMoveList.CHASSIS_ROTATE_LEFT) || (currentChassisMoveType == BMoveList.CHASSIS_ROTATE_RIGHT) )
       distanceToMove = distance * BPhysics.RAD_CONVERT;
    else
       distanceToMove = distance;

    return true;
  }
 }


 private void clearChassisWait()
 {
  if (chassisWaitingMove != null)
     {
      chassisWaitingMove.clearChassisWait();
      chassisWaitingMove = null;

     }
 }

 public void terminateChassisMove(boolean terminateStand)
 {

  // BDebug.print ("BChassis.terminateChassismove called ");
   currentDisplaceVelocity = 0.0f;
   currentRotateVelocity  = 0.0f;
   returnToStandActivated = false;
   pauseTime = 0;
   lastExecTime = 0;

   keepVelocity = false;
   resultingPosition = null;

   if ( terminateStand )
   {
     currentChassisMove = null;
     currentChassisMoveType = 0;

     clearChassisWait();

   }
   else
   {
        if ( currentChassisMoveType != BMoveList.CHASSIS_STAND ) // Should not terminate Stand Move
        {
                currentChassisMove = null;
                clearChassisWait();
                currentChassisMoveType = 0;
        }

        if ( !returnToStandActivated )
                returnToStand();

   }
 }

 public boolean move(double chassisX, double chassisY, float timeSlice, float power )
 // Moves the chassiTilt
 {

    //getBot().setArenaComments("BChassis.move getBot().isBotTipped() " + getBot().isBotTipped() );
    //BDebug.print ("BChassis.move getBot() " + getBot() + getBot().hashCode() );

    if (getBot().isBotTipped() )
      return false;

    boolean moved = false;

   // BDebug.print("Target chassisX " +  chassisX);
    //BDebug.print("Target chassisY " +  chassisY);

    // work out moved boolean
    if ( (chassisX == currentX ) && (chassisY == currentY) )
      moved = false;
    else
      moved = true;

    // direction
    double xdirection;
    double ydirection;

    if (chassisX > currentX )
      xdirection = 1;
    else if (chassisX < currentX )
      xdirection =  -1;
    else
      xdirection = 0;

    if (chassisY > currentY )
      ydirection = 1;
    else if (chassisY < currentY )
      ydirection =  -1;
    else
      ydirection = 0;

    //deltamovement
    double deltaX = rotateSpeed * timeSlice * xdirection * power;
    double deltaY = rotateSpeed * timeSlice * ydirection * power;


  //  BDebug.print("deltaX " +  deltaX);
   // BDebug.print("deltaY " +  deltaY);


    // New Position
    double newX;

    if ( xdirection < 0 )
      newX = java.lang.Math.max ( currentX + deltaX,  chassisX);
    else
      newX = java.lang.Math.min ( currentX + deltaX,  chassisX);

    double newY;

    if ( xdirection < 0 )
      newY = java.lang.Math.max ( currentY + deltaY,  chassisY);
    else
      newY = java.lang.Math.min ( currentY + deltaY,  chassisY);

    prevX = currentX;
    prevY = currentY;

    // set new positions;
    currentX = newX;
    currentY = newY;

   // BDebug.print("newX" +  newX);
   // BDebug.print("newY" +  newY);

    //  transformX.rotX(abs_x);
		//  transformY.rotZ(abs_y);

    Transform3D rotX = new Transform3D();
    rotX.rotX(newX);

    Transform3D rotY = new Transform3D();
    rotY.rotZ(newY);


    Transform3D tempTran = new Transform3D(rotX);

   // tempTran.mul(rotX);
    tempTran.mul(rotY);
    orientationNode.setTransform(tempTran);


    // CalculateHeight After setting rotation so that the rotation os factored into the height






    return moved;



 }

 public void setOrientation(Matrix3f mat )
 {

    if (tiltAxisAngle == null)
    {
      tiltAxisAngle = new AxisAngle4f();
      setOrientationTran = new Transform3D();
      transformedAxis = new Vector3f();

    }

    orientationNode.getTransform(setOrientationTran);
    setOrientationTran.setRotation(mat);
    orientationNode.setTransform(setOrientationTran);

 }

 private static AxisAngle4f tiltAxisAngle;
 private static Transform3D setOrientationTran;
 private static Vector3f transformedAxis;


 public void setOrientation(Vector3f axis, float zenith )
 // absolute axis
 {
  if (tiltAxisAngle == null)
    {
      tiltAxisAngle = new AxisAngle4f();
      setOrientationTran = new Transform3D();
      transformedAxis = new Vector3f();


    }
    transformedAxis.set(axis);

   // BDebug.print (" BChassis.setOrientation arenaPosition.facing " + arenaPosition.facing );

   // invert the arenaFacing because we calculate the axis Vector with the absolute LoacleToVworld
   // vectors
    applyArenaFacing_TempT.rotY(-arenaPosition.facing);
    applyArenaFacing_TempT.transform (transformedAxis);

    tiltAxisAngle.set(transformedAxis.x, transformedAxis.y, transformedAxis.z, zenith);

    setOrientationTran.set(tiltAxisAngle);
    orientationNode.setTransform(setOrientationTran);


 }

 public Matrix4f getOrientationSnap()
 {
  Matrix4f tempM = new  Matrix4f();
  Transform3D tempT = new Transform3D();

  if (orientationNode == null)
    BDebug.print ("Chassis Not live!");
  else
  {
   orientationNode.getTransform(tempT);
   tempT.get(tempM);
  }

  return tempM;


 }

 // local temp variables,  to reduce garbage collection of player
 // DO NOT USE ANYWHERE EXCEPT the Method Below
 // or else threads will fuck you up BIG TIME
 transient private static Transform3D setHeightAndOrientationtempT = new Transform3D();
 //transient private static Vector3f setHeightAndOrientationtempV = new Vector3f();

 protected transient float 	 	lastAppliedHeight;
 protected transient Matrix4f interpolatedHO;


 public void setHeightAndOrientation( Matrix4f m )
 {
 		// Set up interpolation temp vars...
    lastAppliedHeight = m.m13;
    interpolatedHO.set(m);

    setHeightAndOrientationtempT.set(m);
    orientationNode.setTransform(setHeightAndOrientationtempT);
 }



 public void interpolateHeightAndOrientation(Matrix4f m, float t)
 {
 		// Compute interpolated height...
    interpolatedHO.m13 =
    	lastAppliedHeight * (1-t) + m.m13 * t;

    setHeightAndOrientationtempT.set(interpolatedHO);
    orientationNode.setTransform(setHeightAndOrientationtempT);
 }



 public void setOrientationSnap(Matrix4f m)
 {
    Transform3D tempT = new Transform3D();
    tempT.set(m);
    //BDebug.print(m);
    if (orientationNode != null)
        orientationNode.setTransform(tempT);
 }



 public void setHeight()
 // PRE Called after rotatetion and joint moves have been calculated
 {
    float y =  calculateHeight();
    height = y;
    setHeight(y );
 }

 private static Transform3D setHeight_tempTran = new Transform3D();
 private static Vector3f setHeight_HeightV = new Vector3f();

 public void setHeight(float y )
 {
    orientationNode.getTransform(setHeight_tempTran);
    setHeight_HeightV.set(0,y,0);
    setHeight_tempTran.setTranslation(setHeight_HeightV);
    orientationNode.setTransform(setHeight_tempTran);
 }


 private boolean displaceAndDecelerate(float timeSlice)
 {


       float oldVelocity = currentDisplaceVelocity;

       //BDebug.print("BChassis.displace  oldVelocity " +  oldVelocity );

       //float chassisCapability = ((float)damageMax-(float)getDamageCurrent())/(float)damageMax;

       float deltaVelocity =  java.lang.Math.max(  (currentDisplaceVelocity / 2f), 0.1f );
       float distanceInThisSlice = oldVelocity * timeSlice;
       distanceInThisSlice = distanceInThisSlice * BGlobal.WORLD_TO_CANVAS_SCALE;

       //BDebug.print("BChassis.displace  distanceInThisSlice " +  distanceInThisSlice );

       currentDisplaceVelocity = java.lang.Math.max( currentDisplaceVelocity -  deltaVelocity , 0f);

       //BDebug.print("BChassis.displaceAndDecelerate beforePosition  " + arenaPosition );
       //BDebug.print("BChassis.displaceAndDecelerate distanceInThisSlice  " + distanceInThisSlice );

       // resulting Direction vector is normalized so we can just multiply the xyz components
       float deltaX =  resultingDirectionVector.x * distanceInThisSlice;
       float deltaZ =  resultingDirectionVector.z * distanceInThisSlice;
       setArenaPosition (arenaPosition.x + deltaX, arenaPosition.z + deltaZ);

       //BDebug.print("BChassis.displaceAndDecelerate resultingDirectionVector  " + resultingDirectionVector );
       //BDebug.print("BChassis.displaceAndDecelerate arenaPosition  " + arenaPosition );



       return (currentDisplaceVelocity != 0f);




 }




 private boolean displace ( float angleOffSet, float thisMaxVelocity, float power , float timeSlice)
 // Displace the Chassis on the arena,
 // angle offset is radians away from the front to move the chassis
 // so to move left you would use:
 // displace ( PI/2, 0.1f, 1.0f)
 {

    //BDebug.print ("BChassis.displace  ownerBot " + ownerBot);
    //BDebug.print ("BChassis.displace  arenaPosition " + arenaPosition);

    float weight =  getBot().getTotalWeight();
    float weightPercent = (  weight /  loadCapacity ) ;
    float weightCoefficient = 1f + (1f -  weightPercent );
    weightCoefficient = java.lang.Math.max( weightCoefficient, 0);
    weightCoefficient = java.lang.Math.min( weightCoefficient, MAX_CHASSIS_WEIGHT_MULTIPLIER);


    //BDebug.print ("BChassis.displace Limb load:" + ownerBot.getTotalWeight() );
    //BDebug.print ("BChassis.displace loadCapacity:" + loadCapacity );
    //BDebug.print ("BChassis.displace weightPercent" + weightPercent);


   // BDebug.print ("BChassis.displace coefficient" + weightCoefficient);

    // double the penalty for being overweight
    if (weightCoefficient < 1f )
      weightCoefficient = weightCoefficient * weightCoefficient * weightCoefficient;




    float oldVelocity = currentDisplaceVelocity;

    //BDebug.print("BChassis.displace  Old Velocity " +  oldVelocity );

    float chassisCapability = ((float)damageMax-(float)getDamageCurrent())/(float)damageMax;
    if (BGlobal.CONSTRUCTING_CHASSISBOT)
      chassisCapability = 1f;
    // Limb load is not factored into chassis acceleration

    float deltaVelocity =  ( acceleration * chassisCapability * timeSlice * power * weightCoefficient );
    //deltaVelocity = deltaVelocity *  weightCoefficient;

    //BDebug.print("BChassis.displace  chassisCapability " +  chassisCapability );
   // BDebug.print("BChassis.displace  damageCurrent " +  damageCurrent );
   // BDebug.print("BChassis.displace  damageMax " +  damageMax );


    float currentMaxVelocity = thisMaxVelocity * java.lang.Math.min(weightCoefficient, 1f);


    currentDisplaceVelocity = java.lang.Math.min( currentDisplaceVelocity +  deltaVelocity, currentMaxVelocity);

    // delta x as a derivative of ut + 1/2at^2
    // x = ut + 1/2vt
    //    BDebug.print ("BChassis.displace oldVelocity" + oldVelocity);
    //BDebug.print ("BChassis.displace timeSlice" + timeSlice);
    //BDebug.print ("BChassis.displace deltaVelocity" + deltaVelocity);



    float distanceInThisSlice = oldVelocity * timeSlice + ( ( deltaVelocity / 2.0f ) * timeSlice  );
    // BDebug.print ("BChassis.displace before min " + distanceToMove + "distanceIn " + distanceInThisSlice);

    distanceInThisSlice = java.lang.Math.min(distanceToMove, distanceInThisSlice);

   // BDebug.print ("BChassis.displace after min " + distanceToMove );

    //BDebug.print("DamageMax="+damageMax+" DistanceToMove="+distanceToMove+" OldVel="+oldVelocity);
    //BDebug.print("BChassis.displace: delta="+deltaVelocity+"  curr="+currentVelocity+" Dist="+distanceInThisSlice);

    distanceToMove -= distanceInThisSlice;
    distanceMovedThisSnapshot += distanceInThisSlice;
    distThisSlice = distanceInThisSlice; // Used in Chassis Joint Move Calc
    distanceInThisSlice = distanceInThisSlice * BGlobal.WORLD_TO_CANVAS_SCALE;

    //BDebug.print("BChassis.displace  distanceInThisSlice " +  distanceInThisSlice );


    Vector3f direction = BPhysics.directionVector(arenaPosition.facing);

    // Change the direction by the angleOffSet
    Matrix4f offSetMatrix = new Matrix4f();
    offSetMatrix.rotY(angleOffSet);
    offSetMatrix.transform(direction);

    // direction is normalised.

    float deltaX =  direction.x * distanceInThisSlice;
    float deltaZ = direction.z * distanceInThisSlice;
    setArenaPosition (arenaPosition.x + deltaX, arenaPosition.z + deltaZ);

    //float energyUsed = ((chassisEnergyUsage * currentDisplaceVelocity * weight * weightPercent) * timeSlice) / 1000f;
    float energyUsed = ((chassisEnergyUsage * currentDisplaceVelocity * weightPercent) * timeSlice) / 1000f;

    if ( energyUsed / getBot().maxEnergy > 0.1f && getBot().maxEnergy > 10f )
    {
      BDebug.print("!!!!!BChassis.displace ENERGY ERROR: energyUsed="+energyUsed+" vel="+currentDisplaceVelocity+" pct="+weightPercent);
    }

    getBot().useEnergy(energyUsed);
   // BDebug.print("BChassis displace energyUsed " + energyUsed);
    //BDebug.print("BChassis.displace: Total="+getBot().totalEnergy+" thisUsed="+energyUsed);

    return true;

 }


 private boolean rotate (float direction, float power, float timeSlice, boolean decelerate)
 // + direction means right, -direction is left
 {

    //overtakes  and wibbles when doing resultant moves
    float weight =  getBot().getTotalWeight();
    float weightPercent = (  weight /  loadCapacity ) ;
    float weightCoefficient = 1f + (1f -  weightPercent );

    weightCoefficient = java.lang.Math.max( weightCoefficient, 0);
    weightCoefficient = java.lang.Math.min( weightCoefficient, MAX_CHASSIS_WEIGHT_MULTIPLIER);

    // double the penalty for being overweight
    if (weightCoefficient < 1f )
      weightCoefficient = weightCoefficient / 2f;


    // Joints velocity
    float oldVelocity = currentRotateVelocity;
    float deltaVelocity;

    float chassisCapability = ((float)damageMax-(float)getDamageCurrent())/(float)damageMax;
    if (BGlobal.CONSTRUCTING_CHASSISBOT)
      chassisCapability = 1f;


    deltaVelocity = rotateAcceleration * chassisCapability * timeSlice * power ;

    if (decelerate)
         currentRotateVelocity =  java.lang.Math.max(currentRotateVelocity -  (deltaVelocity * 2), 0f);
    else
    {
         //DJG
         if ( maxVelocityExceededCtr > 0 )
         {
         	 deltaVelocity = 0.0f; // No Acceleration
         	 if ( maxVelocityExceededCtr > 1 ) // Decelerate
           {
             float decelerationRate = (timeSlice/BGlobal.CALC_TIMESLICE) * ((maxVelocityForLimb-BPhysics.MAX_VELOCITY_CAP) / maxVelocityForLimb);
         		 deltaVelocity = ( java.lang.Math.abs(currentRotateVelocity) * decelerationRate ) * -1;
           }
         }

         deltaVelocity = deltaVelocity * weightCoefficient;
         float currentMaxVel = weightCoefficient * java.lang.Math.min(maxRotateVelocity, 1f);

         currentRotateVelocity = java.lang.Math.abs(currentRotateVelocity + deltaVelocity);

         if ( currentRotateVelocity > currentMaxVel )
         {
           currentRotateVelocity = currentMaxVel;
           deltaVelocity = 0;
         }
         //DJG

    }

    float oldPosition =  arenaPosition.facing;
    float newPosition = 0;

    // delta x as a derivative of ut + 1/2at^2
    // x = ut + 1/2vt
    float distanceInThisSlice = oldVelocity * timeSlice + ( ( deltaVelocity / 2.0f ) * timeSlice  )  ;

    distanceInThisSlice = direction * java.lang.Math.min( java.lang.Math.abs(distanceInThisSlice), distanceToMove );
    distanceMovedThisSnapshot += java.lang.Math.abs(distanceInThisSlice) / BPhysics.RAD_CONVERT; // Convert to Degrees
    distThisSlice = java.lang.Math.abs(distanceInThisSlice) / BPhysics.RAD_CONVERT;

    //BDebug.print("Rotate2: DistToMove="+distanceToMove+" DistThisSlice="+distanceInThisSlice);
    newPosition = oldPosition + distanceInThisSlice;

    // only move distancetomove if it is not a resultantmove
    if (!decelerate)
        distanceToMove -=  java.lang.Math.abs(distanceInThisSlice);

    /* DJG June 2001
    prevArenaPosition[0].facing = arenaPosition.facing;
    arenaPosition.facing = newPosition;

    applyArenaFacing();
    */

    setArenaFacing(newPosition);


    // if decelerate and Veclocity hd reached 0 then end the result
    if (decelerate)
    {
        // cap the results
        if (direction == -1)
                arenaPosition.facing = java.lang.Math.max(arenaPosition.facing, resultingPosition.facing);
        else
                 arenaPosition.facing = java.lang.Math.min(arenaPosition.facing, resultingPosition.facing);

        if ( currentRotateVelocity == 0.0f )
                resultingPosition.facing =  arenaPosition.facing;
    } else {
      //getBot().totalEnergy -= ((chassisEnergyUsage * java.lang.Math.abs(currentRotateVelocity) * weight) * timeSlice) / 1000f;
      float energyUsed = ((chassisEnergyUsage * java.lang.Math.abs(currentRotateVelocity) * weightPercent) * timeSlice) / 1000f;

      if ( energyUsed / getBot().maxEnergy > 0.1f && getBot().maxEnergy > 10f )
      {
        BDebug.print("!!!!!BChassis.rotate ENERGY ERROR: energyUsed="+energyUsed+" vel="+currentDisplaceVelocity+" pct="+weightPercent);
      }

      //getBot().totalEnergy -= energyUsed;

      getBot().useEnergy(energyUsed);
    //  BDebug.print("BChassis rotate energyUsed " + energyUsed);


    }

    return true;


 }

 private static Transform3D applyArenaFacing_TempT = new Transform3D();

 private void applyArenaFacing()
 {
    //BDebug.print( "BChassis.applyArenaFacing angle " + arenaPosition.facing);
    applyArenaFacing_TempT.rotY(arenaPosition.facing);
    applyArenaFacing(applyArenaFacing_TempT);

 }

 private void applyArenaFacing(Transform3D tempT)
 {
   if (arenaRotateGroup != null)
   {
        //BDebug.print( "BChassis.applyArenaFacing called and set ");
        arenaRotateGroup.setTransform(tempT);

   }
 }




 public boolean execute(float timeSlice, float power)
 // Execute runs the chassis Move

 {

     // BDebug.print ("BChasseis.execute bot " + getBot() );
     // BDebug.print ("BChasseis.execute pre arenaPosition" + arenaPosition);
     // BDebug.print ("BChasseis.execute pre currentChassisMove" + currentChassisMove);

      //getBot().setArenaComments("BChassis.execute  pre arenaPosition " + arenaPosition);

     // DJG June 2001 - Paused after a collision
     if ( standDown(pauseTime) )
       return true;

     //BDebug.print ("BChassis.execute  distanceToMove " + distanceToMove);
     if  (getBot().getEnergy() <= 0f)
      if  (  ! ( (getCurrentMoveType() == BMoveList.CHASSIS_RESULTANT_MOVE ) || (getCurrentMoveType() == BMoveList.CHASSIS_STAND)   )  )
        return false;


     executed = false;

     //BDebug.print ("BChassis.distanceToMove " +   distanceToMove );

     if ( (currentChassisMove == null) && !(currentChassisMoveType == BMoveList.CHASSIS_RESULTANT_MOVE))
       executed = false;
     else
     {
      //  BDebug.print ("BChassis.execute : currentChassisMoveType " + currentChassisMoveType );

       // Displace/Rotate the Chassis First
       if (currentChassisMoveType == BMoveList.CHASSIS_MOVE_FORWARD)
           displace ( 0.0f + displacementOffset, chassisMaxVelocity, power, timeSlice );
       else if (currentChassisMoveType == BMoveList.CHASSIS_MOVE_BACKWARDS)
           displace ( (float)java.lang.Math.PI + displacementOffset, backwardsMaxVelocity, power, timeSlice );
       else if (currentChassisMoveType == BMoveList.CHASSIS_MOVE_LEFT)
           displace ( (float)java.lang.Math.PI / 2.0f + displacementOffset, sidewaysMaxVelocity, power ,timeSlice) ;
       else if (currentChassisMoveType == BMoveList.CHASSIS_MOVE_RIGHT)
           displace ( (float)-java.lang.Math.PI / 2.0f + displacementOffset,sidewaysMaxVelocity, power , timeSlice);
       else  if (currentChassisMoveType == BMoveList.CHASSIS_ROTATE_RIGHT)
            rotate ( -1.0f, power, timeSlice, false);
       else  if (currentChassisMoveType == BMoveList.CHASSIS_ROTATE_LEFT)
            rotate ( 1.0f, power, timeSlice, false);
       else if (currentChassisMoveType == BMoveList.CHASSIS_STAND)
       {
            //BDebug.print ( "BChassis.execute: STANDING..." );
            distThisSlice = distanceThisSnapshot * ( timeSlice / TIMETORETURNTOSTAND );  // 1.5 secs to Return To Stand
            distanceMovedThisSnapshot += distThisSlice;
            distanceToMove = java.lang.Math.max(distanceThisSnapshot - distanceMovedThisSnapshot,0f);
       }
       else if  (currentChassisMoveType == BMoveList.CHASSIS_RESULTANT_MOVE)
       {


               // resultingPosition can be null if it is a chassis clearing move
                if (resultingPosition != null)
                {
                  if (arenaPosition.facing > resultingPosition.facing)
                        rotate ( -1.0f, power, timeSlice, true);
                  else
                        rotate ( 1.0f, power, timeSlice, true);
                }

                boolean displaced = false;

                if ( ! BPhysics.worldEquals(currentDisplaceVelocity,0.0f )  )
                {

                       //BDebug.print (ownerBot.getName() + "BChassis.resultantMoveCalled, currentDisplaceVelocity =  " + currentDisplaceVelocity);
                       displaced = displaceAndDecelerate(timeSlice);
                       //BDebug.print ("BChassis.resultantMoveCalled, displaced =  " + displaced);

                }

                //BDebug.print ("BChasseis.execute pre resultingPosition" + resultingPosition);

                if (resultingPosition == null)
                  distanceToMove = 0.0f;
                else if (  (arenaPosition.facing ==    resultingPosition.facing ) && !displaced )
                {

                        //BDebug.print (ownerBot.getName()  + "BChassis.execute :: equal arenaPositons, stopping rotate");
                        //        BDebug.print ("            -----------__________--------------");
                        distanceToMove = 0.0f;
                }
       }

       else
            BDebug.print ( "BChassis.execute: NOT IMPLEMENTED CHASSIS MOVE TYPE : "  +  currentChassisMoveType);

       //BDebug.print("BChassis.execuyte called ");
       //BDebug.print ("BChassis.distanceToMove (Directly after displace) " +   distanceToMove );


       // Move the Chassis Joints
       if (currentChassisMove != null)
       {
        executed = currentChassisMove.execute(timeSlice, true);

        //BDebug.print("BChassis.execuyte currentChassisMove not Null and execute = " + executed);

       }

       // Transform to stand at the end of moving
       if ( (BPhysics.worldEquals(distanceToMove,0.0f )  ) && !returnToStandActivated
                                     && !BGlobal.CONSTRUCTING_CHASSISBOT && !keepVelocity)
       {
         // BDebug.print ("BChassis .execute return to stand called " +  executed );
          returnToStand();
          executed = true;
       }

        // when the animation finishes, either reset the animation or finish the move
      // BDebug.print ("BChassis .execute executed " +  executed );


       if (!executed)
       {
          //BDebug.print ("BChssis.execute, reseting move distanceToMove" + distanceToMove + " CHASSISSTAND "  + (currentChassisMoveType == BMoveList.CHASSIS_STAND) );

          if ( BPhysics.worldEquals(distanceToMove,0.0f )    || (currentChassisMoveType == BMoveList.CHASSIS_STAND)    )
          {

              currentChassisMove = null;
              clearChassisWait();

              currentChassisMoveType = BMoveList.CHASSIS_STAND;
              if ( keepVelocity )
              {
               // BDebug.print ("BChassis.execute: move finished - keeping velocity");
              }
              else
              {
                currentDisplaceVelocity = 0.0f;
              //  BDebug.print ("BChassis.execute: stopping chassis");
              }
          }
          else
          {
             if (currentChassisMove != null)
                 currentChassisMove.resetMove(true,0,0);
             // BDebug.print ("BChassis.execute: resetting continuous move");
             executed = true;
          }
       }
     }

     setHeight();

     //getBot().setArenaComments("BChassis.execute  post arenaPosition " + arenaPosition);

     // DJG June 2001
     if (executed)
       lastExecTime = simTime;  // For Move Override standoff

     return executed;

 }


 public void returnToStand()
 {
    //BDebug.print ("BChassis.returntoStand this.getChassisMoveList() " + this.getChassisMoveList() );
   if (getBot().isTipped() )
    return;

   if ( ( currentChassisMoveType == BMoveList.CHASSIS_RESULTANT_MOVE && executed )
                   || ( currentChassisMoveType == BMoveList.CHASSIS_STAND && standDown(lastExecTime) ) )
   	return;

   if ( this.getChassisMoveList() != null )
   {

    currentChassisMove = this.getChassisMoveList().getMove(BMoveList.CHASSIS_STAND);

    //BDebug.print ("BChassis.returntoStand currentChassisMove " + currentChassisMove );

    if (currentChassisMove != null)
      currentChassisMove.resetMove(true,0,0);

    returnToStandActivated = true;
    currentDisplaceVelocity = 0.0f;
    currentRotateVelocity = 0.0f;
    currentChassisMoveType = BMoveList.CHASSIS_STAND;
    clearChassisWait();
    distanceToMove = 0.0f;
    // BDebug.print ("BChassis.execute: return to Stand activated");

    if ( this.getDestroyed() )
    {
          //BDebug.print ("BChassis.execute: Chassis Destroyed ");
          getBot().setBotAlive(false);
          getBot().setArenaComments("Chassis Destroyed");
          getBot().getOpponentBot().setArenaComments(getBot() + " chassis destroyed " );
    }
   }

 }

 public void cleanse()
 {
  cleanseLiveNodes();

  if (chassisMoveList != null)
    chassisMoveList.cleanse();


  super.cleanse();
  //BDebug.print("BChassis: "+this.toString()+" cleansed.");

 }

 public void cleanseLiveNodes()
  {
    super.cleanseLiveNodes();
    orientationNode = null;
    displacementGroup = null;
    arenaRotateGroup = null;
    liveBotNode = null;

    prevArenaPosition = null;

    currentDisplaceVelocity = 0.0f;
    currentRotateVelocity = 0.0f;
    currentChassisMove = null;

    simTime = 0;
    lastExecTime = 0;
  	executed = false;
  	pauseTime = 0;

    //BDebug.print("BChassis: "+this.toString()+" cleanseLiveNodes.");

  }

 public void setOrientationGroup(TransformGroup newOr)
 // the OrientationGroup is normally set then the chassis is added to a arenaCanvas using getStructure3d(),
 // this method is placed in so that we can set the Group when we want to play the scene through a botCanvas
 {
  orientationNode = newOr;
  //orientationNode.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);

 }

 public String getCurrentChassisMoveName()
 {
     if (currentChassisMoveType == BMoveList.CHASSIS_RESULTANT_MOVE)
        return ("Resultant Chassis Move ");
     else if (currentChassisMove != null )
        return currentChassisMove.getName();
     else
        return new String ("");

 }




 public BranchGroup getStructure3D()
{

  //BranchGroup tempS = super.cloneStructure3D( false, true, true);
  BranchGroup tempS = super.getStructure3D();

  tempS.setCapability(Group.ALLOW_CHILDREN_WRITE);
  tempS.setCapability(BranchGroup.ALLOW_DETACH);
  tempS.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
  tempS.setCapability(Group.ALLOW_CHILDREN_EXTEND);
  tempS.setCapability(Node.ALLOW_BOUNDS_READ);

  orientationNode = new TransformGroup();
  //orientationNode.setUserData("BChassis OrientNode");
  orientationNode.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
  orientationNode.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
  orientationNode.addChild(tempS);

  orientationNode.setCapability(Group.ALLOW_CHILDREN_WRITE);
  orientationNode.setCapability(BranchGroup.ALLOW_DETACH);
  orientationNode.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
  orientationNode.setCapability(Group.ALLOW_CHILDREN_EXTEND);
  orientationNode.setCapability(Node.ALLOW_BOUNDS_READ);

  wrapperNode= new BranchGroup();
  wrapperNode.setUserData("BChassis wrapper");
  wrapperNode.addChild( orientationNode );

  setLiveBotNode(tempS);
  return wrapperNode;


}

 public Group getOrientNode()
 {
  return orientationNode;
 }

 public void setLiveBotNode(BranchGroup newLive)
 // Only set the live bot node if it is currently null.
 // getStructure3D() calls this, then BBot.loadComponent() calls this.
 // The second call should have no affect on the liveBotNode.
 // CloneStructure3D does not call it.
 {

    myShapesNode.setCapability(Node.ALLOW_PICKABLE_WRITE);

    if (liveBotNode == null)
    {
        liveBotNode = newLive;

        liveBotNode.setUserData("BChassis.liveBot Node ");

    }
 }


 public float workOutProcessingTime(double chassisX, double chassisY,  float power)
 {
    timeToNextSnap = (float)java.lang.Math.max(
                                        java.lang.Math.abs(currentX - chassisX) / ( rotateSpeed * power ),
                                        java.lang.Math.abs(currentY - chassisY) / ( rotateSpeed * power )


                                        );

   return timeToNextSnap;


 }

 public boolean isActive()
 {
    return ( currentChassisMove != null);

 }

 public boolean conflictsWith(BMove chassisMove)
 {
  boolean c = false;


  if ( currentChassisMove == chassisMove )
     c = true;
  else if  (currentChassisMoveType == BMoveList.CHASSIS_RESULTANT_MOVE )
     c = true;
  else if ( currentChassisMove != null)
  {
    int currentType = getChassisMoveList().indexOf(currentChassisMove);

    if (currentType != BMoveList.CHASSIS_STAND )
      c =  true;
  }

  return c;

 }







 public BArenaObjectPosition getArenaPosition()
 {
  if (arenaPosition == null)
    arenaPosition = new BArenaObjectPosition();

  return arenaPosition;

 }

 public void setArenaPosition(BArenaObjectPosition newPos)
 {

  /*
  if ( prevArenaPosition == null )
      initPrevArenaPositions();

  for (int i=0; i < BGlobal.FULL_ROLLBACK_DEPTH-1; i++ )
  {
  	prevArenaPosition[i+1].set(prevArenaPosition[i]);
  }

  if ( arenaPosition != null )
   prevArenaPosition[0].set(arenaPosition);
  else
   prevArenaPosition[0].set(newPos);
  */

  arenaPosition = newPos;

  if ( (arenaRotateGroup != null) && (displacementGroup != null)  )
  {
    setArenaPosition (arenaPosition.x, arenaPosition.z);
    applyArenaFacing();

  }
  
 }



 	// float TEMP_interp_calls_per_apply = 0;

  protected transient BArenaObjectPosition lastAppliedPosition;

  public void applyArenaPositionForBattlPlayer(BArenaObjectPosition newPos )
  {
  	// BDebug.print("TEMP_interp_calls_per_apply: "+ TEMP_interp_calls_per_apply);
  	// TEMP_interp_calls_per_apply = 0;

    if (arenaPosition == null)
      arenaPosition = new  BArenaObjectPosition (newPos);
    else
      arenaPosition.set(newPos);
      
    lastAppliedPosition.set(arenaPosition);

    setArenaPosition(arenaPosition.x, arenaPosition.z);
    applyArenaFacing();
  }

  public void interpolateArenaPosition(BArenaObjectPosition p, float t)
  {
  	// TEMP_interp_calls_per_apply++;

  	arenaPosition.interpolate(lastAppliedPosition, p, t);
		applyArenaDisplacement();
    applyArenaFacing();
  }


  /*

 public void setArenaPosition(BArenaObjectPosition newPos, Transform3D thisTransform)
 {
  // Dont need because this is only used in the battle player.
  //prevArenaPosition = (BArenaObjectPosition) newPos.clone();
  arenaPosition = newPos;

  if ( (arenaRotateGroup != null) && (displacementGroup != null)  )
  {
    setArenaPosition (arenaPosition.x , arenaPosition.z);
    applyArenaFacing(thisTransform);

  }
 }   */


 public boolean getCanMoveSideWays()
 {
  return chassisMoveList.getCanMoveSideWays();
 }

 public void setChassisAndFeetNotValid()
 {
  feetNotValid = true;

 }

 private float chassisAndChassisLockWeight = 0;

 public float getDisplayWeight()   // added here to allow BChassis to override, giving the stats total weight
 {
  if ( chassisAndChassisLockWeight == 0f)
    calculateTotalWeight();

  //BDebug.print ("BChassis.getDisplayWeight returning" + chassisAndChassisLockWeight );

  return chassisAndChassisLockWeight;

 }

 private void calculateTotalWeight()
 {
    BComponentNode thisNode = this.getTreeNode();
    Enumeration enumerator = thisNode.depthFirstEnumeration();
    chassisAndChassisLockWeight = 0f;

    while (enumerator.hasMoreElements())
    {
      BComponentNode node = (BComponentNode)enumerator.nextElement();
      BComponent c = node.getComponent();

      if (c.getChassisLock() )
        chassisAndChassisLockWeight += c.getWeight();

    }

 }



 public void createChassisAndFeetVector()
 // THIS NOLONGER ADDS THE CHASSIS!!!!!!!!!!!!!!!!

 {
    //BDebug.print("BChassis.createChassisAndFeetVector called ");
    feetNotValid = false;

    Vector chassisAndFeetV = new Vector();
    chassisAndFeetV.add(this);

    //BDebug.print("BChassis.createChassisAndFeetVector vector created");

    DefaultMutableTreeNode thisNode = this.getTreeNode();
   // BDebug.print("BChassis.createChassisAndFeetVector then node " + thisNode);

    Enumeration enumerator = thisNode.depthFirstEnumeration();


   if ( BGlobal.CONSTRUCTING_CHASSISBOT )
   {

          while (enumerator.hasMoreElements())
          {

            DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();

            if ( node.isLeaf() )
            {

               chassisAndFeetV.add(node.getUserObject() );

            }
          }


   }
   else
   {
        //  BDebug.print("BChassis.createChassisAndFeetVector NOT  CHASSISBOT ");
          BComponent theComponent;

          while (enumerator.hasMoreElements())
          {
          // Get the component


            DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();

            // TEMP CHECK
            theComponent =  (BComponent)node.getUserObject();


            if (  theComponent.isChassisLockLeaf()   )
              {
                //BDebug.print ("BCassis.init adding " +  node.getUserObject() + " to chassisAndFeet ");
                chassisAndFeetV.add(node.getUserObject() );

              }

          }


   }


       // BDebug.print ("BCassis.init chassisAndFeet INIT Finished" );

        chassisAndFeet = new BComponent[chassisAndFeetV.size()];

        for (int i = 0; i < chassisAndFeetV.size() ; i++)
        {
          chassisAndFeet[i] =  (BComponent)chassisAndFeetV.get(i);
        }



 }

 public BComponent[] getChassisAndFeetList()
 {
  return  chassisAndFeet;

 }



 public void init(boolean optimise)
 {
        //BDebug.print("BChassis.init called ");

        super.init(optimise);

        // SJS [29/05/2000] - Init chassis move-list.
        // YLY [jun 7th 2000] check for not null;
        if (chassisMoveList != null)
          chassisMoveList.init();

        initPrevArenaPositions();
        arenaPosition = new BArenaObjectPosition();

         //BDebug.print("BChassis.init positions created ");

        if ( chassisMoveList == null )
          BGlobal.CONSTRUCTING_CHASSISBOT = true;
        else
          BGlobal.CONSTRUCTING_CHASSISBOT = false;

        if (!optimise)
          createChassisAndFeetVector();

        if ( acceleration == 0.0f )
          acceleration = 1.0f;
        if ( chassisMaxVelocity == 0.0f )
          chassisMaxVelocity = 1.0f;
        if ( backwardsMaxVelocity == 0.0f )
          backwardsMaxVelocity = 1.0f;
        if ( sidewaysMaxVelocity == 0.0f && BGlobal.CONSTRUCTING_CHASSISBOT )
          sidewaysMaxVelocity = 1.0f;
        if ( rotateAcceleration == 0.0f )
          rotateAcceleration = 1.0f;
        if ( maxRotateVelocity == 0.0f )
          maxRotateVelocity = 1.0f;

        chassisEnergyUsage = chassisPowerOutput * BPhysics.CHASSIS_POWER_CONSUMPTION_RATIO;

				lastAppliedPosition	= new BArenaObjectPosition();
        interpolatedHO			= new Matrix4f();
	 }


 public Vector getValidChassisMoves()
 {

    //BMove current;

    Vector chassiMoves = new Vector();

    if (getChassisMoveList() != null)
    {

      // ignoring 0 because that is stand
      for (int i=0; i<getChassisMoveList().size(); i++)
        chassiMoves.add(getChassisMoveList().getAt(i));

      if (!getChassisMoveList().getCanMoveSideWays() )
      {
        chassiMoves.remove(BMoveList.CHASSIS_MOVE_RIGHT);
        chassiMoves.remove(BMoveList.CHASSIS_MOVE_LEFT);

      }

      chassiMoves.remove(BMoveList.CHASSIS_STAND);

     }



    return chassiMoves;


 }



  public void applyArenaPosition()
  {
        applyArenaFacing();
        applyArenaDisplacement();



  }


   public void AVGandApply (BArenaObjectPosition newPos)
  {

     arenaPosition.AVG(newPos, 1);
     applyArenaPosition();

  }

  public BArenaObjectPosition getPrevPosition()
    { return prevArenaPosition[0]; }

  public void rollbackToPrev()
  {

    //BDebug.print("Depth="+BGlobal.FULL_ROLLBACK_DEPTH);

    rollBack(prevArenaPosition[0]);

    for (int i=0; i < BGlobal.FULL_ROLLBACK_DEPTH-1; i++ )
  	{
  		prevArenaPosition[i].set(prevArenaPosition[i+1]);
  	}
  }

  public void rollBack(BArenaObjectPosition newPos)
  {
    arenaPosition.set(newPos);
    applyArenaPosition();
  }

  /*
  public static int ROLL_BACK_DEPTH = 4;
  transient BArenaObjectPosition bestRoll = null;


  public void rollBack(BComponent c,BComponent o, javax.media.j3d.Locale myLocale)
  {

        if (! arenaPosition.equals(prevArenaPosition) )
        {
            if (BCollision.intersect(c, o, myLocale, false))
            {

                bestRoll = null;
                recurseRollBack(c, o, myLocale, 1,  prevArenaPosition);

                if ( bestRoll == null)
                {
                   arenaPosition.set(prevArenaPosition);
                 //  BDebug.print ( getBot() + "BChassis Complete Rollback happened ");
                }
                else
                {
                   arenaPosition.set(bestRoll);
                  // BDebug.print ( getBot() +"BChassis partial Rollback happened ");
                }

                applyArenaPosition();

            }
            //else
             // BDebug.print (getBot() +"BChassis No rollback because collision cleared ");

        }
       // else
        // BDebug.print (getBot() +"BChassis No rollback there was no movement");


  }

  private void recurseRollBack(BComponent c, BComponent o,javax.media.j3d.Locale myLocale,  int depth,  BArenaObjectPosition toTest)
  {
        if (depth <= ROLL_BACK_DEPTH )
        {
               // BDebug.print("BChassis.recurseRollBack Called for depth " + depth );
                //BDebug.print("arenaPosition " + arenaPosition);


                BArenaObjectPosition lastCurrent =  new BArenaObjectPosition();
                lastCurrent.set( arenaPosition ) ;

                arenaPosition.AVG(toTest, 1);

               // BDebug.print("arenaPosition avg" + arenaPosition);
                applyArenaPosition();

                if (BCollision.intersect(c, o, myLocale, false))
                        recurseRollBack( c,  o, myLocale,   depth + 1, toTest);
                else
                {
                        bestRoll = new  BArenaObjectPosition();
                        bestRoll.set( arenaPosition );
                        recurseRollBack( c,  o, myLocale,   depth + 1,  lastCurrent);
                }


        }
  }
            */



 protected float getDisplaceFriction()
 {
    return BPhysics.DISPLACE_FRICTION;
 }

 public int getCurrentMoveType()
 {
        return currentChassisMoveType;
 }

 private static Transform3D createResultantMove_oT = new Transform3D();
 private static Transform3D createResultantMove_myT = new Transform3D();

 private static Vector3f createResultantMove_myPos = new Vector3f();
 private static Vector3f createResultantMove_oppPos = new Vector3f();
 private static Vector3f createResultantMove_oppPosRelative = new Vector3f();
 private static Vector3f createResultantMove_negMomentum = new Vector3f();
//private static Vector3f resultingDirectionVector = new Vector3f();
 private static Vector3f createResultantMove_p2 = new Vector3f();
 private static Vector3f createResultantMove_momDirection = new Vector3f();
 private static Vector3f createResultantMove_directionCross = new Vector3f();

 public void createResultantMove(BComponent childInCollision, BComponent otherComponent)
 {


        // get the momentum and convert to momentum in the zx plane
        Vector3f momentum = childInCollision.collisionInertia;
        momentum.y = 0f;

        //BDebug.print ("BChassis.createResultantMove " + momentum );

        if (momentum.length() != 0)
        {
          this.terminateChassisMove(true);
          currentChassisMoveType = BMoveList.CHASSIS_RESULTANT_MOVE;


          //BDebug.print ("momentum " + momentum);
          //ownerBot.setArenaComments("momentum " + momentum);

          // get the opponents pos in xz
          BComponent opponentComponent;

          // if it is not the Chassis in the collision, use the child and the centre for testing rotational direction
          // else use the opponent
          if (childInCollision == this)
          {
            if (otherComponent == BArena.arenaComponent)
              return;
            else
              opponentComponent = otherComponent;
          }
          else
            opponentComponent = childInCollision;

          //BDebug.print ("BChassis.childInCollision " + childInCollision.getBot() + " " +childInCollision );
         // BDebug.print ("BChassis.opponentComponent " + opponentComponent.getBot() + " " + opponentComponent );


          opponentComponent.getLiveNode().getLocalToVworld(createResultantMove_oT);
          //Vector3f opponentPos = new Vector3f();
          createResultantMove_oT.get(createResultantMove_oppPos);
          createResultantMove_oppPos.y = 0f;

          //BDebug.print ("createResultantMove_oppPos " + createResultantMove_oppPos);


          // get this chassis pos in xz
          getLiveNode().getLocalToVworld(createResultantMove_myT);
          //Vector3f myPos = new Vector3f();
          createResultantMove_myT.get(createResultantMove_myPos);
          createResultantMove_myPos.y = 0f;

         // BDebug.print ("createResultantMove_myPos " + createResultantMove_myPos);
                //ownerBot.setArenaComments("myPos " + myPos);

          // get the pos of the opponent component relative to the chassis
          //Vector3f opponentPosRelative = new Vector3f();
          createResultantMove_oppPosRelative.sub(createResultantMove_oppPos, createResultantMove_myPos);

         // BDebug.print ("createResultantMove_oppPosRelative " + createResultantMove_oppPosRelative);
                //ownerBot.setArenaComments("opponentPosRelative " + opponentPosRelative);



          // DISPLACEMENT
          createResultantMove_negMomentum.set(momentum);
          createResultantMove_negMomentum.negate();

         // BDebug.print ("createResultantMove_negMomentum " + createResultantMove_negMomentum);


          float dispAngle;
          if (createResultantMove_oppPosRelative.length() == 0)
            dispAngle = 0;
          else
            dispAngle = createResultantMove_oppPosRelative.angle(createResultantMove_negMomentum);

          // get Displacement Percentage power ,
          // 45 degrees of more = 0%,
          // 0 degrees = 100%;
          float dispPercentPower = 1f - (dispAngle / ((float)java.lang.Math.PI /4f )  );

          dispPercentPower = java.lang.Math.max(0f,dispPercentPower);


         // BDebug.print ("---------DISPANGLE " +  dispAngle);
          //BDebug.print ("---------dispPercentPower " +  dispPercentPower);


          float v0 =  momentum.length()  / this.weight;

        //  BDebug.print ("v0 " + v0);

          this.currentDisplaceVelocity = v0 * getDisplaceFriction();

         // BDebug.print ("v0 scaled" + v0);

          if (BGlobal.DEADLOCKDETECTED )
          {
            this.currentDisplaceVelocity = java.lang.Math.min(this.currentDisplaceVelocity, 2);
          }


         // BDebug.print ("---------displaceVelocity " +  currentDisplaceVelocity);
          this.currentDisplaceVelocity = this.currentDisplaceVelocity * dispPercentPower;

          if (resultingDirectionVector == null)
            resultingDirectionVector = new Vector3f();
            
          resultingDirectionVector.set( childInCollision.collisionInertia ) ;
          resultingDirectionVector.y = 0f;
          resultingDirectionVector.normalize();
        //  BDebug.print ("resultingDirectionVector " + resultingDirectionVector);




          // ROTATION
          //if (!BGlobal.DEADLOCKDETECTED )
          //{

            // get p2 ( the position of  opponentPos + momentum
          //Vector3f momDirection = new Vector3f(momentum);
          createResultantMove_momDirection.set(momentum);
          createResultantMove_momDirection.normalize();
         // BDebug.print ("createResultantMove_momDirection " + createResultantMove_momDirection);

         // BDebug.print ("momDirection " + momDirection);
          //ownerBot.setArenaComments("momDirection " + momDirection);

          createResultantMove_p2.add(createResultantMove_oppPosRelative, createResultantMove_momDirection);

          //BDebug.print ("p2 " + p2);
          //ownerBot.setArenaComments("p2 " + p2);

          // get a cross product to define direction
          //Vector3f directionCross = new Vector3f();
          createResultantMove_directionCross.cross(createResultantMove_p2, createResultantMove_oppPosRelative);

          //BDebug.print ("directionCross " + directionCross);
          //ownerBot.setArenaComments("directionCross " + directionCross);


          float direction;

          if (createResultantMove_directionCross.y > 0 )
            direction = 1f;
          else
            direction = -1f;

         // BDebug.print ("direction " + direction);
              //ownerBot.setArenaComments("direction " + direction);




            // rotation hack
            float r0 = (float) java.lang.Math.sqrt(v0);

           // BDebug.print ("r0 " + r0);



            // Unlike Joints the velocity is always positive.
            // sqrt and mutiply at the co-efficent
            // and take off the force that is displament result
            this.currentRotateVelocity =  r0   * BPhysics.CHASSIS_ROTATIONAL_INERTIA * (1f - dispPercentPower);                //this.currentVelocity = java.lang.Math.min(this.currentVelocity, getMaxVelocity()  );
            //MDF ??
            //this.currentRotateVelocity = this.currentRotateVelocity ;

           // BDebug.print ("currentRotateVelocity" + currentRotateVelocity);

            if (BGlobal.DEADLOCKDETECTED )
            {
              this.currentRotateVelocity = java.lang.Math.min(this.currentRotateVelocity, 0.5f);
            }

           // BDebug.print ("Resultant Chassis Velocity " + this.currentRotateVelocity);
           // getBot().setArenaComments("Resultant Chassis Velocity  " + this.currentRotateVelocity);
            //opponentComponent.ownerBot.setArenaComments("Resultant Chassis Velocity  " + this.currentRotateVelocity);



            r0 = r0  * BPhysics.CHASSIS_ROTATIONAL_INERTIA;

           // BDebug.print ("r0 after rotation cap" + currentRotateVelocity);

            // was stableparentweight
            // need to be reviewed
            float opponentlimbWeight = ((BComponent)opponentComponent).getStableParentWeight();

           // BDebug.print ("opponentlimbWeight " + opponentlimbWeight);


           // BDebug.print ("this.weight " + this.weight);

            float weightRatio = java.lang.Math.min( opponentlimbWeight / this.weight, 1 );

          //  BDebug.print ("weightRatio " + weightRatio);

            float bounce = r0 * weightRatio ;
           // BDebug.print ("bounce" + bounce);

           // BDebug.print ("bounce " + bounce);

            bounce = bounce * (1f - dispPercentPower);
          //  BDebug.print ("Bounce scaled by displace Power " + bounce);


            if (resultingPosition == null)
              resultingPosition = new BArenaObjectPosition();

            // DJG July 2001
            // Capping
            this.currentRotateVelocity = java.lang.Math.min(this.currentRotateVelocity, 1.0f);
            this.currentDisplaceVelocity = java.lang.Math.min(this.currentDisplaceVelocity, 3.0f);
            bounce = java.lang.Math.min(bounce, 0.5f);


            resultingPosition.set( arenaPosition );
            resultingPosition.facing = resultingPosition.facing - (bounce * direction)   ;

            distanceToMove = bounce ;

           // BDebug.print ("distanceToMove " + distanceToMove);
           // BDebug.print ("resultingPosition " + resultingPosition);

          }

          //currentChassisMove =  getChassisMoveList().getMove(BMoveList.CHASSIS_MOVE_FORWARD );
       // }





 }

   public void setSelected(
    boolean selected,
    boolean showSelectBox,
    boolean showSelectColor,
    boolean showConnectPts,
    boolean showInactiveConnectPoints,
    Color3f selectColor)
  {

    //if there if no box then do not show connect points too

    if (!showSelectBox )
      if (!BGlobal.CONSTRUCTING_CHASSISBOT)
          showConnectPts = false;

    super.setSelected(
      selected,
      showSelectBox,
      showSelectColor,
      showConnectPts,
      showInactiveConnectPoints,
      selectColor);
      
  }

  public String getType()
  {
    return COMPONENT_TYPE;
  }

 public BMove getCurrentChassisMove()
 {
  return currentChassisMove;
 }

 private void  getDevModeStrings(Vector theList)
 {
    Vector tempV;
    tempV = new Vector(2);
    tempV.add("Bot polygon count");
    tempV.add(getBot().getPolyCount() + " stripified triangles");
    tempV.trimToSize();
    theList.add(tempV);

    tempV = new Vector(2);
    tempV.add("Max Torque");
    tempV.add(unbalancedTorque+ " kg m");
    tempV.trimToSize();
    theList.add(tempV);



 }

 public Vector getAttributeStrings()
 // returns a Vector of tuples  (heading, value )
 // 2-d Vector
 {

  Vector theList = super.getAttributeStrings();

  if (BGlobal.DEVELOPER_MODE)
    getDevModeStrings(theList);

  Vector tempV;
  tempV = new Vector(2);
  tempV.add("load capacity");
  tempV.add(loadCapacity+ " kg");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("max. acceleration");
  tempV.add( BMath.format( acceleration )+ " m/s squared");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("max. forward velocity");
  tempV.add(BMath.format( chassisMaxVelocity )+ " m/s");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("max. strafe velocity");
  tempV.add(BMath.format( sidewaysMaxVelocity )+ " m/s");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("max. reverse velocity");
  tempV.add(BMath.format(  backwardsMaxVelocity )+ " m/s");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("max. rotational acceleration");
  tempV.add( BMath.format(  BPhysics.radToDeg(rotateAcceleration), 1 )+ " deg/s squared");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("max. rotational velocity");
  tempV.add( BMath.format( BPhysics.radToDeg(maxRotateVelocity), 1) + " deg/s");
  tempV.trimToSize();
  theList.add(tempV);

  return theList;
  }


  // SJS [28/09/2000]: Bot Attributes...
  // -------------------------------------------------------------

  protected static final float SPEED_MIN = 1.0f;
  protected static final float SPEED_MAX = 5.0f;


  public BAttributeSet getAttributes()
  {
    BAttributeSet as =
      super.getAttributes();

    as.addAll(getChassisAttributes());
    return as;
  }


  public BAttributeSet getChassisAttributes()
  {
    BAttributeSet as =
      new BAttributeSet();

    as.add( new BAttribute(
      "Speed",
      new BRelativeValue(SPEED_MIN, SPEED_MAX, chassisMaxVelocity),
      BAttribute.HIGH));

    as.add( new BAttribute(
      "Load capacity",
      new BAbsoluteValue(BMath.format(loadCapacity)+ " "+ BGlobal.WEIGHT_UNIT),
      BAttribute.HIGH));

    return as;
  }


  public void setPickable(boolean p)
    { myShapesNode.setPickable(p); }

  private float unbalancedTorque = 15000;

  public float getMaxBalancedTorque()  // kgm
  {
    if (unbalancedTorque == 0)
       unbalancedTorque = 15000;

    return unbalancedTorque;
  }

  public void setMaxBalancedTorque(float t)
    { unbalancedTorque = t;}

  // Hover Chassis check (using this instead of instanceof for speed)
  public boolean isHover()
    { return false; }

    // containsWheels is a transient variable that is set in initArenaDetails
  public boolean isWheeled()
    { return !isHover() && containsWheels; }


  public boolean isInternal()
  {
      return true;
  }

  public void setTimer(float inTime)
  {
     simTime = inTime;
  }

  public float getTimer()
  {
     return simTime;
  }

  private boolean standDown(float testTime)
  {
    return ( ((simTime - testTime) * 1000) < BGlobal.STAND_DOWN_MILLISECS  && (simTime*1000) > BGlobal.STAND_DOWN_MILLISECS );
  }

  public String getComponentType()
  { return COMPONENT_TYPE; }

  public void pause()
  {
    pauseTime = simTime;
    currentDisplaceVelocity = 0.0f;
    currentRotateVelocity = 0.0f;
  }

  public void setPreviousArenaPosition()
  {
    if ( prevArenaPosition == null )
      initPrevArenaPositions();

  	for (int i=0; i < BGlobal.FULL_ROLLBACK_DEPTH-1; i++ )
  	{
  		prevArenaPosition[i+1].set(prevArenaPosition[i]);
  	}

  	if ( arenaPosition != null )
   		prevArenaPosition[0].set(arenaPosition);
  	else
   		prevArenaPosition[0].set(new BArenaObjectPosition());
  }

  public void resetForSim()
  {
    //BDebug.print("BChassis.ResetForSim called");
    super.resetForSim();
    terminateChassisMove(true);
    currentX = 0;
  	currentY = 0;
  	prevX = 0;
  	prevY = 0;
  	//prevArenaPosition = null;
  }


}
