package roboforge;

import roboforge.ui.components.*;

import java.awt.*;
//import java.io.*;
//import javax.swing.*;
import java.awt.event.*;
//import javax.swing.border.*;
//import java.awt.image.MemoryImageSource;


public class BColorPicker extends BPanel

/*

* Description:
  Allows the user to select a colour.

* Example:
* Modification History ( Name, Date, Notes )
  Created HSC, 22 April.

*/

{
  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
private double              hue  = 0.0;
  private double              s    = 0.0;
  private double              b    = 0.0;

  private BHuePicker          bHuePicker;
  private BSBPicker           sbPicker;
  private BColorDisplay       colorDisplay;
  private GridBagLayout       layout;
  private GridBagConstraints  constraints;

  //private float               shine;

  private Color selectedColor;

  private ActionListener actionL;

  //private Border border;


  public BColorPicker(Color newC)
  {
    this(null, null, newC);
  }

  public BColorPicker(BUIResources res, String key)
  {
    this(res, key, Color.black);
  }

  public BColorPicker(BUIResources res, String key, Color newC)
  {
    super(res, key);

    sbPicker        = new BSBPicker(this);
    bHuePicker      = new BHuePicker(sbPicker);
    layout          = new GridBagLayout();
    constraints     = new GridBagConstraints();

    if (BGlobal.DEPRECATED)
      colorDisplay    = new BColorDisplay();

    // setBackground(Color.black);
    setLayout(layout);
    // this.setBorder(border);

    constraints.insets = new Insets(0, 0, 0, 0);

    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.weightx = 0.2;
    constraints.weighty = 1.0;

    layout.setConstraints(bHuePicker, constraints);
    add(bHuePicker);

    if (BGlobal.DEPRECATED)
    {
      constraints.gridx = 0;
      constraints.gridy = 1;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.weightx = 0.2;
      constraints.weighty = 0.5;

      layout.setConstraints(colorDisplay, constraints);
      add(colorDisplay);

      constraints.gridheight = 2;
    }

    constraints.gridx = 1;
    constraints.gridy = 0;
    constraints.weightx = 1.0;
    constraints.weighty = 1.0;
    constraints.fill = GridBagConstraints.BOTH;
    
    layout.setConstraints(sbPicker, constraints);
    add(sbPicker);


    setColor(newC);
  }

  public void initializeUI(BUIResources res, String key)
  {
    super.initializeUI(res, key);

    Color c = BGlobal.colors.getDarkerShade();
    setBorder(BUIResources.getLineBorder(c));
  }


  public void setColor(Color newC)
  {
    selectedColor = newC;

    float[] hSB = Color.RGBtoHSB(
      selectedColor.getRed(),
      selectedColor.getGreen(),
      selectedColor.getBlue(),
      null);

    hue = (double) hSB[0];
    s   = (double) hSB[1];
    b   = (double) hSB[2];

    bHuePicker.setHue(hue);
    sbPicker.setHSB(hue, s, b);

    if (BGlobal.DEPRECATED)
      colorDisplay.setColor(selectedColor);
  }

  public Color getColor()
    { return selectedColor; }

  // SJS [24/02/2000] - Deprecated.
  public Color getSelectedColor()
  {
    return selectedColor;
  }

  public void addActionListener(ActionListener newActioner)
    { actionL = newActioner; }


  public void setHSB(double h, double s, double b)
  {
    hue = h;
    setSB(s, b);
  }

  public void setHue(double newHue)
  {
    hue = newHue;

    if (!BGlobal.DEPRECATED)
      sbPicker.setHue(hue);

    if (actionL != null)
      actionL.actionPerformed(new ActionEvent(this, 1, "Action"));
  }

  public void setSB(double saturation, double brightness)
  {
    s = saturation;
    b = brightness;

    Color newColor = Color.getHSBColor((float)hue, (float)saturation,
                                       (float)brightness);
    //component.setColor(newColor);
    selectedColor = newColor;

    if (actionL != null)
        actionL.actionPerformed(new ActionEvent(this, 1, "Action") ) ;

    if (BGlobal.DEPRECATED)
      colorDisplay.setColor(newColor);
  }

  public static String colorToHex(Color color)
  {
    String redStr, greenStr, blueStr;

    redStr = Integer.toHexString(color.getRed());
    greenStr = Integer.toHexString(color.getGreen());
    blueStr = Integer.toHexString(color.getBlue());
    if (redStr.length() < 2)
      redStr = '0' + redStr;
    if (greenStr.length() < 2)
      greenStr = '0' + greenStr;
    if (blueStr.length() < 2)
      blueStr = '0' + blueStr;
    return (redStr + greenStr + blueStr).toUpperCase();
  }

  public double getHue()
  {
    return hue;
  }

}


