package roboforge;

//: BFileSelectionTree.java

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.io.*;
//import java.util.*;

public class BFileSelectionTree extends JTree {

/**
	 * 
	 */
	private static final long serialVersionUID = -1L;
/*
* Description:  Basic File Tree Manager. Requires Start Path and file Type (Optional)
* Modification History ( Name, Date, Notes ):
                          DJG, 30-Mar-99
*/

  String filter;
  boolean hideSuffix;
  String fileSeparator = "/";

  // CONSTRUCTOR
  /*
  * path      = Path to starting directory. Cannot Ascend this directory.
  * fType     = file suffix. Show only files ending with this suffix. can be blank.
  * rootName  = Text to display on root node of tree.
  * hideType  = boolean. if true then the file suffix (ftype) will not be displayed.
  */
	public BFileSelectionTree(String path,String fType,String rootName,boolean hideType) throws FileNotFoundException, SecurityException {
		super((TreeModel)null);			// Create the JTree itself

    filter = fType.toLowerCase();
    hideSuffix = hideType;
    if (path == "") {path = "C:";}
    if (rootName == "") {rootName = path;}
    if (path.endsWith(fileSeparator)) {path = path.substring(0,(int) path.length() - 1);}

		// Use horizontal and vertical lines
		putClientProperty("JTree.lineStyle", "Angled");

		// Create the first node
		FileTreeNode rootNode = new FileTreeNode(null, path, rootName);

		// Populate the root node with its subdirectories
		//boolean addedNodes = 
		rootNode.populateDirectories(true);
		setModel(new DefaultTreeModel(rootNode));

		// Listen for Tree Selection Events
		addTreeExpansionListener(new TreeExpansionHandler());
	}

	// Returns the full pathname for a path, or null if not a known path
	public String getPathName(TreePath path) {
		Object o = path.getLastPathComponent();
		if (o instanceof FileTreeNode) {
			return ((FileTreeNode)o).fullName;
		}
		return null;
	}

	// Inner class that represents a node in this file system tree
	protected class FileTreeNode extends DefaultMutableTreeNode {
		/**
		 * 
		 */
		private static final long serialVersionUID = -1L;
		public FileTreeNode(String parent, String name, String dspName) throws SecurityException,
															FileNotFoundException {
			this.name = name;
      if (dspName.toLowerCase().endsWith(filter) && hideSuffix)
      {
        dspName = dspName.substring(0,(int) dspName.length() - filter.length());
      }
      this.dispName = dspName;

			// See if this node exists and whether it is a directory
			fullName = parent == null ? name : parent + fileSeparator + name;

			File f = new File(fullName);
			if (f.exists() == false) {
				throw new FileNotFoundException("File " + fullName + " does not exist");
			}

			isDir = f.isDirectory();

			// Hack for Windows which doesn't consider a drive to be a directory!
			if (isDir == false && f.isFile() == false) {
				isDir = true;
			}
		}

		// Override isLeaf to check whether this is a directory
		public boolean isLeaf() {
			return !isDir;
		}

		// Override getAllowsChildren to check whether this is a directory
		public boolean getAllowsChildren() {
			return isDir;
		}

		// For display purposes, we return our own name
		public String toString() {
			return dispName;
		}

		// If we are a directory, scan our contents and populate
		// with children. In addition, populate those children
		// if the "descend" flag is true. We only descend once,
		// to avoid recursing the whole subtree.
		// Returns true if some nodes were added
		boolean populateDirectories(boolean descend)
    {
			boolean addedNodes = false;

			// Do this only once
			if (populated == false)
      {
				File f;
				try
        {
					f = new File(fullName);
				}
        catch (SecurityException e)
        {
					populated = true;
					return false;
				}

				if (interim == true)
        {
					// We have had a quick look here before:
					// remove the dummy node that we added last time
					removeAllChildren();
					interim = false;
				}

				String[] names = f.list();    // Get list of contents

				// Process the directories
				for (int i = 0; i < names.length; i++)
        {
					String name = names[i];
					File d = new File(fullName, name);
					try
          {
						if (d.isDirectory())
            {
							FileTreeNode node = new FileTreeNode(fullName, name, name);
							this.add(node);
							if (descend)
              {
                //node.populateDirectories(false);
                // HSC 8 March 2000 Changed from above as need to get all
                // sub directories at start - not just as required
								node.populateDirectories(true);
							}
							addedNodes = true;
							if (descend == false) {
								// Only add one node if not descending
								break;
							}
						}
            else
            {
              if (filter == "" || d.getName().toLowerCase().endsWith(filter))
              {
                this.add(new FileTreeNode(fullName, name, name));
                addedNodes = true;
              }
            }
					} catch (Throwable t)
          {
						// Ignore phantoms or access problems
					}
				}

				// If we were scanning to get all subdirectories,
				// or if we found no subdirectories, there is no
				// reason to look at this directory again, so
				// set populated to true. Otherwise, we set interim
				// so that we look again in the future if we need to
				if (descend == true || addedNodes == false) {
					populated = true;
				} else {
					// Just set interim state
					interim = true;
				}
			}
			return addedNodes;
		}

		protected String name;		// Name of this component
		protected String fullName;	// Full pathname
    protected String dispName; // Display Name
		protected boolean populated;// true if we have been populated
		protected boolean interim;	// true if we are in interim state
		protected boolean isDir;	// true if this is a directory
	}

	// Inner class that handles Tree Expansion Events
	protected class TreeExpansionHandler implements TreeExpansionListener {
		public void treeExpanded(TreeExpansionEvent evt) {
			TreePath path = evt.getPath();			// The expanded path
			JTree tree = (JTree)evt.getSource();	// The tree

			// Get the last component of the path and
			// arrange to have it fully populated.
			FileTreeNode node = (FileTreeNode)path.getLastPathComponent();
			if (node.populateDirectories(true)) {
				((DefaultTreeModel)tree.getModel()).nodeStructureChanged(node);
			}
		}

		public void treeCollapsed(TreeExpansionEvent evt) {
			// Nothing to do
		}
	}
}///:~