package roboforge;


public interface BGhost
{
  // A BGhost is an object that represents another object
  // (usually in a much more memory-compact form).  It is
  // created by a BGhostable, typically when the BGhostable
  // is about to become GC-able.  The object represented by 
  // a BGhost can be 'resurrected' - this generally means that 
  // a new (equivalent) instance of the original is constructed
  // and returned. Alternatively, it can be 'banished', which
  // makes the ghost non-resurrectable.
  
  // Example: UI Screens.
  // UI Screens have a large memory footprint.
  // When opening a new screen, it is desirable to keep memory
  // down (at the cost of some CPU overhead).  Therefore, we
  // make screens Ghostable.  When a new screen is placed over
  // the current screen, the latter is ghosted.  When we wish
  // to use the current screen again, we call resurrect().
  

  public Object resurrect();
    // Return the object that this ghost represents.
  
  public void banish();
    // Permanently deallocate the ghost.
    // Once banish() is called, ressurection is illegal.
    
}
