package roboforge;

import javax.vecmath.*;
import java.util.*;
import javax.media.j3d.*;
//import javax.swing.tree.*;

import roboforge.math.BMath;
import roboforge.btree.*;

//import com.sun.j3d.loaders.ParsingErrorException;
//import com.sun.j3d.loaders.IncorrectFormatException;
import com.sun.j3d.loaders.Scene;



public abstract class BJoint extends BComponent //implements Comparator
/*

* Description:
  A child of BComponent that can be transformed.

* Example:
* Modification History ( Name, Date, Notes ):
  HSC, 25-March-1999
  YLY, 10 May 1999 ------------------- MADE ABSTRACT --------------------

*/

{

  static final long serialVersionUID = -1L;

  public float jointPowerOutput;            // How much power it generates (KW) - translates to acceleration
  transient protected float jointEnergyUsage;  // Watts used to move 1 Unit of Torque at 1 m/s


  protected BJointPosition defaultJointPosition; // the position of the joint when the bot is at rest
  protected transient BJointPosition currentPosition; // The current position of the joint in a simulation

  private transient BJointPosition[] prevPosition; // the previous position in a simulation (1 timeslice ). Used for rollback
  private transient BJointPosition lastSnapPosition = null; // the last poision of the joint when a arenaTimeSlice was taken. used to only take delta snaps
  //private transient BJointPosition lastTargetPosition = null; // The position of the joint the last time a target was set for this joint.

  // DJG June 2001 - Used for capping movement of joints... esp for rollbacks and resultant moves
  //protected transient BJointPosition currentMoveMinPosition = null; // The min position this move allows
  //protected transient BJointPosition currentMoveMaxPosition = null; // The max position this move allows

  private transient BJointPosition lastNonInterpolatedPosition;
    // SJS [14/03/2001] - Last joint position that was not set
    // during interpolation.  Used for playback interpolation.

  private transient BJointPosition interpolatedPosition;
    // SJS [14/03/2001] - Interpolated position of joint.


  transient protected TransformGroup jointTransformNode = null;
  protected float maxFreedomPositive= 10000000000000000.0f;
  protected float maxFreedomNegative= -10000000000000000.0f;
  protected double AXISLINELENGTH = 10.0;

  // TRANSIENT BATTLE VARIABLES
  public transient float currentVelocity = 0f;
  protected transient float timeToNextSnap = 0f;

 // public transient int lastFrameMoved = 0;  // proprecesses the last frame moves for fdast random acess of BBattleRecording slices

  // ABSTRACT METHODS
  public abstract void parseDirectionalLight(DirectionalLight tempL);

  public abstract Transform3D getDeltaTransform (int x, int y, int x_last, int y_last, BJointPosition lastPosition);

  // SJS [10/05/2000]
  public abstract Transform3D getDeltaTransform(float delta, BJointPosition last);

  //******************************** READ BELOW
  public abstract Transform3D getTempTransform(boolean useDefault);
  // The Transform returned  by this method is a static variable filled with the data.
  // It MUST Be Cloned if you are going to store it.

  //public abstract void setAcceleration(float newAcceleration);
  //public abstract float getAcceleration();
  public abstract void setMaxAcceleration(float newAcceleration);
  public abstract float getMaxAcceleration();


  public abstract boolean move(BJointPosition endPosition, float timeSlice, float power , float maxVelocityMutiplier, boolean deccelerate);
  // Move the Joint the the Arena
  // Returns true if the joint was actually moved;
  // Power is percent power 0 to 1;

  public  abstract void setDelay (float totalMoveTime);
  public abstract float workOutProcessingTime(BJointPosition endPosition, float power);
  protected float delay;
  private transient float percentToTarget;
 // public abstract float workOutProcessingTime(BJointPosition endPosition,  BComponentTransform bComponentTransform, float power);
  public abstract void setMaxVelocity(float newVelocity);
  public abstract float getMaxVelocity();

  public abstract float getResultantPosition ( Vector3f momentum, BComponent collidedComponent,  BJointPosition newMove, BComponent otherComponent );
  // given a momentum vector,
  // and collidedComponent  (This Joints Child that collided ! )
  //this method will return
  // newMove,  the Ending joint position of the new Move
  // the return float is, the percentage of the momentum Vector absorbed by this movement (0  to 1 )

  // DJG June 2001
  // Used for capping rollbacks
  //public abstract void setCappedJointPosition(BJointPosition jointPos, float posFloat);


  public static String COMPONENT_TYPE = "Joint";

  protected static float RESIST_POWER_COEFFICIENT = 50f;  // used in getResultantPosition() in foprmula   distanceMoved = distanceMove / (jointPower/RESIST_POWER_COEFFICIENT )

  public transient float pauseTime = 0f; // used for Collision Pausing


  public String getComponentType()
  { return COMPONENT_TYPE; }


  
  public BJoint (String nameIn, String filename)
  {
     super(nameIn);
     defaultJointPosition = new BJointPosition(0.0f);

     structure3DFileName = filename;
     Scene s = loadShape();
     parseLights(s);

  }

  public void initForArena(BChassis theChassis, BBot owner)
  {
    super.initForArena(theChassis, owner);

    lastSnapPosition = null;
    

  }


   public void setFreedomBounds(float newNegative, float newPositive)
 {
        maxFreedomNegative = newNegative;
        maxFreedomPositive = newPositive;



 }

 public void init(boolean optimise)
 {
  super.init( optimise);
  currentPosition = (BJointPosition)defaultJointPosition.clone();

  jointEnergyUsage = jointPowerOutput * BPhysics.POWER_CONSUMPTION_RATIO;

  // SJS [14/03/2001] - Init interpolation vars if in playback mode..
  if (optimise)
    interpolatedPosition = new BJointPosition(0f);

 }

 public float getFreedomMin()
 {
 return maxFreedomNegative;

 }
  public float getFreedomMax()
 {
 return maxFreedomPositive;

 }

 public void setJointTransformNode(TransformGroup newT)
  {
   // BDebug.print ("set JointTransformcalled for " + getName() + " newT" + newT);
    jointTransformNode = newT;
  }

  public BJointPosition getJointPosition()
  {
       if (currentPosition == null)
        currentPosition = (BJointPosition)defaultJointPosition.clone();

       return currentPosition;

  };

  public void setJointPosition(BJointPosition newP)
  {
        currentPosition = newP;
  };


  public BJointPosition getDefaultJointPosition()
  {
        return defaultJointPosition;
  };

  public void setDefaultJointPosition(BJointPosition newP)
  {
        defaultJointPosition = newP;

  };


  public void replaceStructureANDConnectPoints(String newVRML)
  {

     structure3DFileName = newVRML;
     connectPoints = new Vector();

     Scene s = loadShape();
     parseLights(s);


  }


  protected void upgrade (BJoint upgradeTwin)
  {
    super.upgrade(upgradeTwin);


    jointPowerOutput = upgradeTwin.jointPowerOutput;
    //jointEnergyUsage = upgradeTwin.jointEnergyUsage;
    //setAcceleration(upgradeTwin.getAcceleration());
    setMaxVelocity( upgradeTwin.getMaxVelocity() ) ;
    setFreedomBounds( upgradeTwin.getFreedomMin(), upgradeTwin.getFreedomMax() );
    setMaxAcceleration( upgradeTwin.getMaxAcceleration() );

  }


  protected void parseLights(Scene s)
  {

        List lightsv = Arrays.asList(s.getLightNodes());
        BDebug.print("BJoint: Parsing Lights to Connect Points and Hinge Rotate");


        for (int i = lightsv.size()-1 ; i>0; i-- )
      {
       Light tempL = (Light)lightsv.get(i);

       if ( (tempL instanceof SpotLight) )
           {

           SpotLight tempP = (SpotLight)tempL;
           tempP.setCapability(PointLight.ALLOW_POSITION_READ);
           tempP.setCapability(SpotLight.ALLOW_DIRECTION_READ);

           Point3f position= new Point3f();
           tempP.getPosition(position);
           Vector3f direction = new Vector3f();
           tempP.getDirection(direction);

           BDebug.print("Converted" +  tempP.toString() );
           connectPoints.add(new BConnectPoint( new Vector3d( position),
                                          new Vector3d(direction)   )
                            ) ;

           } // if  is spotlight
         if ( (tempL instanceof  DirectionalLight) )
           {

           this.parseDirectionalLight((DirectionalLight)tempL);


           } // if  is dirLight

        } // loop


  }


  public void setMovedHighlight()
    { setHighlight(BGlobal.JOINT_HIGHLIGHT); }

  protected void moveJoint(float delta, boolean useDefaultJointPosition)
  {
    // SJS [10/05/2000] -
    // Move the joint through its single DOF [degree of freedom.]

    if (!this.getChassisLock())
    {
      BJointPosition last = useDefaultJointPosition ?
        defaultJointPosition : getJointPosition();

      Transform3D t =
        this.getDeltaTransform(delta, last);

      jointTransformNode.setTransform(t);

      // If using default position, also update the current position.

      if (useDefaultJointPosition)
        setJointPosition((BJointPosition) last.clone());
    }
  }


  private static BJointPosition mouseMove_changeDetector = new BJointPosition(0);

  public boolean mouseMove(int x, int y, int x_last, int y_last, boolean useDefaultJointPosition)
  {
    if (!this.getChassisLock())
    {

        BJointPosition lastPosition;

        if (useDefaultJointPosition)
           lastPosition =  defaultJointPosition;
        else
           lastPosition =  getJointPosition();

        mouseMove_changeDetector.set(lastPosition);

        Transform3D tempT = this.getDeltaTransform(x, y, x_last, y_last, lastPosition);

        jointTransformNode.setTransform(tempT);

        // If using default posiotion also update the currentposition;

        if (useDefaultJointPosition)
          setJointPosition((BJointPosition) lastPosition.clone());

        return !mouseMove_changeDetector.equals(lastPosition);



    }

    return false;
  }




public void cleanseLiveNodes()
{

  super.cleanseLiveNodes();
  jointTransformNode = null;

  prevPosition = null;

  pauseTime = 0f;

  currentVelocity = 0f;

  //BDebug.print ("cleanseLiveNodes called for " + getName());


}


public void cleanse()
{
        super.cleanse();
        cleanseLiveNodes();

}


public Object clone()
{

  BJoint o = null;

  try
  {
    o = (BJoint)super.clone();
  }
  catch (Exception e)
  {
    BDebug.print( e.getMessage() );
  }

   o.defaultJointPosition = (BJointPosition)defaultJointPosition.clone();
   if (currentPosition != null)
        o.currentPosition =  (BJointPosition)currentPosition.clone();

   return o;


}



public String toString()
{

  return name;

}

public void applyTransform()
{
    applyTransform(false);
}



  public void applyTransform(boolean useDefault )
  {

      //BDebug.print ("BJoint.apply  jointTransformNode " + jointTransformNode );

      if (jointTransformNode != null)
        jointTransformNode.setTransform( getTempTransform(useDefault));


      if (useDefault)
      {
          //BDebug.print ("setting default Position ");
          setJointPosition((BJointPosition) getDefaultJointPosition().clone() );
      }
  }

  public void applyTransform(Transform3D tempT)
  {
  //hasMoved = true;

    if (jointTransformNode != null)
      jointTransformNode.setTransform(tempT);
  }

  public void applyTransform (BJointPosition newTransient )
  {
    this.setJointPosition(newTransient);

    // Remember this position for interpolation...
    lastNonInterpolatedPosition = newTransient;

    applyTransform(false);
  }


  public void interpolate(BJointPosition target, float t)
  {
    // SJS [14/03/2001] - Interpolates joint position
    // between last applied position and the target.
    // t is the weighting coefficient [0..1].

    interpolatedPosition.interpolate(
      lastNonInterpolatedPosition, target, t);

    this.setJointPosition(interpolatedPosition);
    applyTransform(false);
  }




  /*
 public Vector addConnectPointsToScene(Group attachNode)
{

// variables
  BConnectPointShape sph;
  TransformGroup connectTrans;
  BConnectPoint current;
  Transform3D t;

  Transform3D trans = new Transform3D();
  Vector3d   facing = new Vector3d();
  Vector3d  rotAxis = new Vector3d();
  Vector3d    zAxis = new Vector3d(0.0, 0.0, -1.0);
  double      theta = 0.0;
  double        dot = 0.0;
  AxisAngle4d   rot = new AxisAngle4d();
  double        tol = 0.01;
  Vector3d   offset = new Vector3d(0.0, 0.0, 1.5);



  // SJS [17/02/2000] - added a branch group layer.
  myConnectPtsNode = new BranchGroup();
  myConnectPtsNode.setCapability(BranchGroup.ALLOW_DETACH);

  TransformGroup jointTransformG = new TransformGroup();
  Transform3D tempT = getTempTransform(true);
  tempT.invert();

  Vector tempVector = new Vector();


    jointTransformG.setTransform(tempT);
    myConnectPtsNode.addChild(jointTransformG);

    for (int i = 0 ; i < connectPoints.size() ; i++ )
    {

      current = (BConnectPoint)connectPoints.get(i);

      if (current.isActive() || BGlobal.DEPRECATED)
      {

        t = new Transform3D();

        // Get connect point's normalized facing vector.
        facing.normalize(current.getFacing());

        if (facing.z <= -(1.0 - tol))
        {
          // If facing is +z, rotate 180 degrees.
          trans.rotX(BPhysics.PI);
          t.mul(trans, t);
        }
        else if (facing.z <= (1.0 - tol))
        {
          // Reverse z direction.
          facing.z = -facing.z;

          // Get a vector perpendicular to z & facing.
          rotAxis.cross(facing, zAxis);
          rotAxis.normalize();

          // Get angle between the two vectors.
          theta = facing.angle(zAxis);

          // Rotate about the derived vector.
          rot.set(rotAxis.x, rotAxis.y, rotAxis.z, theta);

          trans.setIdentity();
          trans.setRotation(rot);
          t.mul(trans, t);
        }



        // displace connectpoints
        trans.set(current.position);
        t.mul(trans, t);

        // displace connectpoints
        //t = new Transform3D();
        //t.set( current.position  );
        connectTrans = new TransformGroup(t);

        // Create Point
        sph = new BConnectPointShape(current,this );
        connectTrans.addChild(sph);

        // SJS [17/02/2000]: Link connectPt back to its shape...
        current.setShape(sph);

        // if this is the base node, displace it. to represent the movement
        if (connectPointBase != null)
        {
          if (current.position.equals(connectPointBase.position) )
            jointTransformG.addChild(connectTrans);
          else
             myConnectPtsNode.addChild(connectTrans);

        }
        else
        {
          myConnectPtsNode.addChild(connectTrans);


        }

        tempVector.add(sph);
      }

     }

    attachNode.addChild(myConnectPtsNode);


    return tempVector;
  }
  */

  // LINE CLASS


  public BJointPosition getPreviousJointPosition()
  {
    if ( prevPosition[0] == null )
      return currentPosition;
    else
			return prevPosition[0];

  }

  //public void setPreviousJointPosition(BJointPosition newPosition)
  public void setPreviousJointPosition()
  {
     if ( prevPosition == null )
     {
       prevPosition = new BJointPosition[BGlobal.FULL_ROLLBACK_DEPTH];
       for (int i=0; i < BGlobal.FULL_ROLLBACK_DEPTH; i++ )
       {
       	prevPosition[i] = new BJointPosition(0);
       }
     }
     else
     {

       for (int i=0; i < BGlobal.FULL_ROLLBACK_DEPTH-1; i++ )
       {
           prevPosition[i+1].set(prevPosition[i]);
       }
     }

     if ( currentPosition != null )
     	prevPosition[0].set(currentPosition);

  }


  public boolean hasMoved()
  {
    if (prevPosition == null)
      return true;
    else if (prevPosition[0] == null)
      return false;
    
    return !currentPosition.equals(prevPosition[0]);
  }


  public void setLastSnapPosition (BJointPosition lastSnapPosition )
  {
    this.lastSnapPosition = lastSnapPosition;
    
    // DJG June 2001 - Fucking up rollback??
    //setPreviousJointPosition(lastSnapPosition);

  }

  public boolean hasMovedSinceLastSnap()
  {

        if (lastSnapPosition == null)
                return true;
        return !lastSnapPosition.equals(currentPosition);

  }


  public void AVGandApply (BJointPosition newPos)
  {
    currentPosition.AVG(newPos, 1);
    applyTransform();
  }

  /*
  public BJointPosition getPrevJointPosition()
    { return prevPosition; }
  */

  public void rollbackToPrev()
  {
    if ( prevPosition != null )
    {
      //if ( this.toString().startsWith("C200") )
      //BDebug.print("- Rolling Back "+toString()+" Curr="+currentPosition.getAngle()+" Prev="+prevPosition.getAngle());

      //currentPosition = (BJointPosition) prevPosition.clone();
      //float newPrevAngle = prevPosition[0].getAngle() - (currentPosition.getAngle() -  prevPosition[0].getAngle());

      currentPosition.set(prevPosition[0]);
      applyTransform();

      for (int i=0; i < BGlobal.FULL_ROLLBACK_DEPTH-1; i++ )
      {
        prevPosition[i].set(prevPosition[i+1]);
      }

      //setCappedJointPosition(prevPosition, newPrevAngle);

    }

    currentVelocity = 0;
  }
  /*
  public static int ROLL_BACK_DEPTH = 3;
  transient BJointPosition bestRoll = null;
  private transient boolean collisionCleared = false;

  public boolean getlastRollbackwasClear()
    { return collisionCleared; }

  public void clearRollbackFlag()
    { collisionCleared = false; }


  public void rollBack(BComponent c,BComponent o, javax.media.j3d.Locale myLocale)
  {

        if (! currentPosition.equals(prevPosition)  &&  (prevPosition != null) )
        {
           if  ( BCollision.intersect(c, o, myLocale, false))
           {
                collisionCleared = false;
                bestRoll = null;
                recurseRollBack(c, o, myLocale, 1,  prevPosition);

                if ( bestRoll == null)
                {
                   currentPosition = (BJointPosition) prevPosition.clone();
                 //  BDebug.print ( getName() +"Complete Rollback ");

                }
                else
                {
                   currentPosition =  bestRoll;
                 // BDebug.print (getName() + "BJoint partial rollback");
                }

                applyTransform();

            }
            else
            {
               // BDebug.print ("BJoint No rollback because collision cleared ");
                collisionCleared = true;

            }

        }
        else
        {
            collisionCleared = false;
           // BDebug.print ( getName() + "No rollback because of non-movement ");
        }

  }

  private void recurseRollBack(BComponent c, BComponent o,javax.media.j3d.Locale myLocale,  int depth,  BJointPosition toTest)
  {
        if (depth <= ROLL_BACK_DEPTH )
        {
                ///BDebug.print("Joint.recurseRollBack Called for depth " + depth + " for components " + c + " " + o);
               // BDebug.print("currentPosition " + currentPosition);

                BJointPosition lastCurrent =  (BJointPosition)currentPosition.clone();
                currentPosition.AVG(toTest, 1);
               // BDebug.print("currentPosition after rollback " + currentPosition);

                applyTransform();

                if (BCollision.intersect(c, o, myLocale, false))
                        recurseRollBack( c,  o, myLocale,   depth + 1, toTest);
                else
                {
                      //  BDebug.print("Collision Cleared");
                        bestRoll = (BJointPosition) currentPosition.clone();
                        recurseRollBack( c,  o, myLocale,   depth + 1,  lastCurrent);
                }


        }
  }

      */


 public Vector getAttributeStrings()
 // returns a Vector of tuples  (heading, value )
 // 2-d Vector
 {

  Vector theList = super.getAttributeStrings();


  Vector tempV;

  tempV = new Vector(2);
  tempV.add("power");
  tempV.add(jointPowerOutput + " kW");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("range");

//  int range =  java.lang.Math.round(getFreedomRange());
  float range = getFreedomRange();
  String rangeString = BMath.format((double)range, 1);
  String userString;
  if (range > 1000.0f)
    userString = "Infinity";
  else
    userString = rangeString + " "+ getFreedomUnits();

  tempV.add( userString );
  tempV.trimToSize();
  theList.add(tempV);

  /* No More Static Usage
  tempV = new Vector(2);
  tempV.add("energy usage");
  tempV.add(energyUsage + " watts");
  tempV.trimToSize();
  theList.add(tempV);
  */


  return theList;
  }

  public float getFreedomRange()
    { return 0.0f; }

  public String getFreedomUnits()
    { return null; }


    /*
  // THE COMPARE IS USED TO SORT THE FRAME LIST
  // ONLY USED IN BBattleRecorder
  public int compare(Object o1, Object o2)
  {
    BJoint j1 = (BJoint)o1;
    BJoint j2 = (BJoint)o2;

    if (j1.lastFrameMoved < j2.lastFrameMoved)
      return -1;
    else if (j1.lastFrameMoved > j2.lastFrameMoved)
      return 1;
    else
      return 0;



  }    */


 /*
 // Another Young Ly hack taken out in order to correct the damage calculation and make balancing possible
 private static float JOINT_RESISTANCE = 0.4f;

 public void addToDamageCurrent(int x)
 {
  if (getChassisLock() )
    getBot().getChassis().addToDamageCurrent( x);
  else
    damageCurrent = damageCurrent + (int)( x * JOINT_RESISTANCE ) ;
 }
 */


 public boolean getIsInDefaultPosition()
 {
    return ( defaultJointPosition.equals(currentPosition) || (currentPosition == null) );


 }

 /*
 public boolean getIsTopOfStableLimb()
 {
    return false;
 }  */

 //transient private static Vector3f angAccel_tempV = new Vector3f();
 private static final float NON_TORQUE_POWER = 0.11f;
 //private static final float STACK_PENALTY_CONSTANT = 1.5f;

 transient private static Vector3f angVelocity_tempV = new Vector3f();

 protected float getNetLimbLoad(float baseLimbLoad, float power)
 {
   //BDebug.print ( "BJoint.getNetLimbLoad " + this + " baseLimbLoad " + baseLimbLoad  + " power " + power );


   if ( childCount == 0 )
     return 0f;  // Nothing to do here. Move along!

   if ( this.prevAngVelocity.length() == 0f )
     this.prevAngVelocity.set(this.angVelocity);


   // keep current velocity if limb is destroyed
   if ( childCount != prevChildCount )
     angVelocity_tempV.set(this.prevAngVelocity);
   else
     angVelocity_tempV.set(this.angVelocity);

   //BDebug.print("BJoint.getNetLimbLoad pre fudging angVelocity_tempV " + angVelocity_tempV);


   if ( Float.isNaN(angVelocity_tempV.length()) || Float.isInfinite(angVelocity_tempV.length()) )
   {
     angVelocity_tempV.set(0f,0f,0f);
     angAcceleration = 0f;
     directAcceleration = 0f;
     this.angVelocity.set(0f,0f,0f);
     this.prevAngVelocity.set(0f,0f,0f);
   }


   /*
   // Calculates net limb load on joint due to gravity and angular acceleration
   angAccel_tempV.set(angVelocity_tempV);
   angAccel_tempV.sub(this.prevAngVelocity); // Net delta velocity vector being angular acceleration

   float angAccelScalar = java.lang.Math.abs(angAccel_tempV.length()) / (BPhysics.GRAVITY * BGlobal.CALC_TIMESLICE);

   float netLimbLoad = baseLimbLoad * angAccelScalar * 10f; // scalar due to angular acceleration
   // Note - Scaled up by 10 for more influence on joint
   */

   // Calculate small load due to angular acceleration
   // Plus larger load due to static inertia (direct acceleration) of objects
   float angAccelScalar = java.lang.Math.abs(angAcceleration);
   float netLimbLoad = angAccelScalar + ( directAcceleration * BPhysics.GRAVITY);

   //BDebug.print ( "BJoint.getNetLimbLoad angAccelScalar " + angAccelScalar );
   //BDebug.print ( "BJoint.getNetLimbLoad netLimbLoad " + netLimbLoad );

   //accelScalar = angAccel_tempV.y - ( BPhysics.GRAVITY * BGlobal.CALC_TIMESLICE * -1 );
   //accelScalar = java.lang.Math.abs(accelScalar) / (BPhysics.GRAVITY * BGlobal.CALC_TIMESLICE);

   // Change - DG - Use instantaneous vertical velocity instead. Not exact but near enough.
   float gravAccelScalar = 1.0f;

   // Stupid energy hack, because it fits in with the stupid gravity hack
   float gravEneryHack = 1.0f;

   float TERMINAL_VELOCITY = ( BPhysics.TERMINAL_VELOCITY * this.getBot().getLastExecuteTime() * -1 ); // m/s
   if ( angVelocity_tempV.y < 0.0f && angVelocity_tempV.y >= TERMINAL_VELOCITY )
   {
     gravAccelScalar = 0.01f; // Weightless
     gravEneryHack = 0.3f;
   }
   else
   {
     gravAccelScalar = java.lang.Math.abs(angVelocity_tempV.y - TERMINAL_VELOCITY) / java.lang.Math.abs(TERMINAL_VELOCITY);
   }

   netLimbLoad += baseLimbLoad * gravAccelScalar; // scalar due to gravity

  // BDebug.print ( "BJoint.getNetLimbLoad stupid gravAccelScalar.." + gravEneryHack );



   if ( !getChassisLock() )
   {

    // Energy Calculation (penalised by power - 1 = 100% power)
    float nonTorqueUse = jointPowerOutput * getBot().getLastExecuteTime() * NON_TORQUE_POWER;
    float energyUsed = (((jointEnergyUsage * angVelocity_tempV.length() * netLimbLoad) * getBot().getLastExecuteTime()) / 1000f) * power;
    energyUsed = (energyUsed * gravEneryHack) + nonTorqueUse;

    //BDebug.print (this + ".getNetLimbLoad stackedPenalty :" + stackedPenalty);
    // you may have 2 free stack
    //int stackedFactor = stackedPenalty - 2;

    // DJG Sep 2001
    // Stacked Energy Consumption penalty removed
    /*
    if (stackedFactor > 1)
    {
     // BDebug.print ("STACKED JOINT PENALTY FACTOR");
      energyUsed = energyUsed * stackedFactor * STACK_PENALTY_CONSTANT;
    }
    */


    if ( energyUsed / getBot().maxEnergy > 0.1f && getBot().maxEnergy > 10f )
    {
      BDebug.print("!!!!BJoint ENERGY ERROR: "+this.toString()+" Used="+energyUsed+" vel="+angVelocity_tempV.length()+" load="+netLimbLoad);
      BDebug.print("           children="+childCount+" prev="+prevChildCount);
    }

    getBot().useEnergy(energyUsed);

   }
   //BDebug.print("BJoint.getNetLimbLoad: "+this.toString()+" Vel="+angVelocity_tempV.length()+" x="+angVelocity_tempV.x+" y="+angVelocity_tempV.y+" z="+angVelocity_tempV.z);
   //BDebug.print("BJoint.getNetLimbLoad: "+this.toString()+" LimbLoad="+baseLimbLoad+" ang="+angAccelScalar+" dir="+(directAcceleration * BPhysics.GRAVITY)+" vel_Y="+angVelocity_tempV.y+" net="+netLimbLoad+" tv="+TERMINAL_VELOCITY);
   //BDebug.print("BJoint.getNetLimbLoad: "+this.toString()+" NetLimbLoad="+netLimbLoad+" angScalar="+angAccelScalar+" gravScalar="+gravAccelScalar);
   //BDebug.print("BJoint.getNetLimbLoad: "+this.toString()+" netLimbLoad="+netLimbLoad+" speed="+angVelocity_tempV.length()+" energyUsed="+energyUsed);

   // This must be done here - DG
   prevChildCount = childCount;

   return netLimbLoad;
 }

 /* No LOnger referenced
 public void storeCurrentPositionAsTarget()
  {lastTargetPosition = (BJointPosition)currentPosition.clone(); }

 public float workoutPercentToTarget(BJointPosition target )
 {
  float totalDistance = java.lang.Math.abs( target.getAngle() - lastTargetPosition.getAngle() ) ;
  float delta = java.lang.Math.abs( currentPosition.getAngle() - lastTargetPosition.getAngle() );

  if (totalDistance == 0f )
  {
    percentToTarget = 1;
    return -1;
  }
  else
    percentToTarget = delta / totalDistance;

 // BDebug.print ( "BJoint.workoutProcessingTime " + this + percentToTarget);
  return  percentToTarget;
 }
 */

 // DJG June 2001 - Used for capping rollbacks and resultant moves
 // Currently only used for capping rollbacks
 /*
 public void setMoveMinMaxPositions(BJointPosition moveJointPos)
 {
   if ( currentMoveMinPosition == null )
   {
     currentMoveMinPosition = new BJointPosition(0.0f);
     currentMoveMaxPosition = new BJointPosition(0.0f);
   }

   if ( moveJointPos.getAngle() < currentMoveMinPosition.getAngle() )
     currentMoveMinPosition.set(moveJointPos);
   if ( moveJointPos.getAngle() > currentMoveMaxPosition.getAngle() )
     currentMoveMaxPosition.set(moveJointPos);
 }
 */

 private static float FASTFACTOR = 0.7f;
 private static float SLOWFACTOR = 1.4f;

 public float getPercentToTargetScaleFactor( float fastestJ, float slowestJ )
 {
  if (fastestJ == slowestJ )
    return 1;

  float range = fastestJ - slowestJ;
  float relative = (percentToTarget - slowestJ ) / range;

 // BDebug.print ( "BJoint.getPercentToTargetScaleFactor fastestJ " + this + fastestJ);
 // BDebug.print ( "BJoint..getPercentToTargetScaleFactor slowestJ " + slowestJ);

  float scale =  SLOWFACTOR - ((SLOWFACTOR - FASTFACTOR) * relative );
 // BDebug.print ( "BJoint.getPercentToTargetScaleFactor scale " + scale);


  return scale;

 }


 protected int getStackedJointPenalty()
  { return  stackedPenalty; }

 protected transient int stackedPenalty = 1;

 // DJG June 2001 - This would only work for directly stacked joints...
 // Should be moved to calcLimbLoad...

 public void calculateLimbLoad(float timeSlice, boolean worldStateCycle)
 // pre: This must be called depthFirst
 {
    super.calculateLimbLoad(timeSlice, worldStateCycle) ;

    if (worldStateCycle)
    {
    	stackedPenalty = 1;
    	BComponentNode myNode = this.getTreeNode();

    	for (Enumeration componentC = myNode.children(); componentC.hasMoreElements() ;)
    	{

       	BComponentNode child = (BComponentNode)componentC.nextElement();

       	if (child.getComponent().isJoint() )
       	{
          BJoint j = (BJoint) child.getComponent();
          if (j.hasMoved())
            stackedPenalty = stackedPenalty + j.getStackedJointPenalty();
       	}

    	}

    }

 }

 // Pause joint if in collision
 public void pause()
 {
   pauseTime = ownerBot.getTimer();
   currentVelocity = 0f;
 }

 protected boolean standDown(float testTime)
 {
    if ( ownerBot == null)
      return false;
    else
  		return ( ((ownerBot.getTimer() - testTime) * 1000) < BGlobal.STAND_DOWN_MILLISECS && (ownerBot.getTimer()*1000) > BGlobal.STAND_DOWN_MILLISECS);
 }

 public void resetForSim()
 {
    super.resetForSim();
    prevPosition = null;
  	pauseTime = 0f;
  	currentVelocity = 0f;
 }



}


