package roboforge;

import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

import roboforge.ui.components.BMutableListModel;



public class BMonitoredList
  implements BMutableListModel, MutableComboBoxModel, Serializable
{

  // Serialization ID.
  // -----------------------------------------------------------

  static final long serialVersionUID = -1L;


  // Internal variables...
  // -----------------------------------------------------------

  protected Vector list;
    // List data.

  protected Vector permanentListeners;
    // Set of 'permanent' list data listeners.
    // These listener references are serialized.


  // Transient variables...
  // -----------------------------------------------------------

  protected transient Vector transientListeners;
    // Set of 'transient' list data listeners.
    // These are only kept during execution.

  protected transient Object selected;
    // The selected object...



  // Constructor...
  // -----------------------------------------------------------

  public BMonitoredList()
    { list = new Vector(); }

  public BMonitoredList(int initialCapacity)
    { list = new Vector(initialCapacity); }


  // Accessor methods...
  // -------------------------------------------------------------

  public int getSize()
    { return list.size(); }

  public int size()
    { return list.size(); }

  public boolean isEmpty()
    { return (list.size() == 0); }


  public Object getAt(int i)
    { return list.get(i); }

  public Object getElementAt(int i)
    { return list.get(i); }


  public int indexOf(Object o)
    { return list.indexOf(o); }


  public void add(Object o)
  {
    BDebug.print (" BMonitoredList add called ");

    int i = list.size();

    list.add(o);
    fireIntervalAdded(i, i);
  }

  public void addElement(Object o)
    { add(o); }


  public void addAt(Object o, int i)
  {
    list.add(i, o);
    fireIntervalAdded(i, i);
  }

  public void insertElementAt(Object o, int i)
    { addAt(o, i); }



  public void removeElement(Object o)
  {
    int i = list.indexOf(o);
    removeAt(i);
  }

  public void remove(Object o)
  {
    int i = list.indexOf(o);
    removeAt(i);
  }

  public void removeElementAt(int i)
    { removeAt(i); }

  public void removeAt(int i)
  {
    if ((i > -1) && (i < list.size()))
    {
      if (list.get(i) == selected)
      {
        if (i == 0)
          setSelectedItem(list.size() == 1 ? null : list.get(1));
        else
          setSelectedItem(list.get(i-1));
      }

      list.remove(i);
      fireIntervalRemoved(i, i);
    }
  }


  protected void removeAll()
  {
    int n = list.size();

    list.clear();
    fireIntervalRemoved(0, n-1);
  }

  public boolean isRemoveable(Object o)
    { return true; }

  public boolean isRemoveableAt(int i)
    { return true; }

  public void cleanse()
  {
      list.clear();
      list = null;

      permanentListeners = null;
      transientListeners = null;
      selected = null;

  }

  public void setSelectedItem(Object o)
  {
    if (o != null)
    {
      if (!o.equals(selected))
      {
        this.selected = o;
        notifyChange();
      }
    }
    else if (selected != null)
    {
      this.selected = null;
      notifyChange();
    }
  }

  public Object getSelectedItem()
    { return selected; }



  // Listener methods...
  // -----------------------------------------------------------

  public void addListDataListener(ListDataListener l)
  {
    if (transientListeners == null)
      transientListeners = new Vector();

    transientListeners.add(l);
  }

  public void addPermanentListDataListener(ListDataListener l)
  {
    if (permanentListeners == null)
      permanentListeners = new Vector();

    permanentListeners.add(l);
  }

  public void removeListDataListener(ListDataListener l)
  {
    if (transientListeners != null)
      transientListeners.remove(l);

    if (permanentListeners != null)
      permanentListeners.remove(l);
  }

  public void removeAllListDataListeners()
  {
    transientListeners = null;
  }


  // Listener event propagation methods...
  // -----------------------------------------------------------

  public void notifyChange(Object o)
  {
    int i = list.indexOf(o);
    if (i != -1)
      fireContentsChanged(i, i);
  }

  public void notifyChange()
    { fireContentsChanged(-1, -1); }

  protected void fireContentsChanged(int l, int h)
    { fireListDataEvent(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, l, h)); }

  protected void fireIntervalAdded(int l, int h)
    { fireListDataEvent(new ListDataEvent(this, ListDataEvent.INTERVAL_ADDED, l, h)); }

  protected void fireIntervalRemoved(int l, int h)
    { fireListDataEvent(new ListDataEvent(this, ListDataEvent.INTERVAL_REMOVED, l, h)); }


  protected void fireListDataEvent(ListDataEvent e)
  {
    if (permanentListeners != null)
      fireListDataEvent(permanentListeners, e);

    if (transientListeners != null)
      fireListDataEvent(transientListeners, e);
  }

  protected void fireListDataEvent(Vector listeners, ListDataEvent e)
  {
    int n    = listeners.size();
    int type = e.getType();

    ListDataListener l;

    for (int i=0; i<n; i++)
    {
      l = (ListDataListener) listeners.get(i);

      switch (type)
      {
        case ListDataEvent.CONTENTS_CHANGED:
          l.contentsChanged(e);
          break;

        case ListDataEvent.INTERVAL_ADDED:
          l.intervalAdded(e);
          break;

        case ListDataEvent.INTERVAL_REMOVED:
          l.intervalRemoved(e);
          break;
      }
    }
  }

}