package roboforge;

import javax.media.j3d.*;
import javax.vecmath.*;


public class BPhysics
{

  public BPhysics()
  {
  }


  // General
  public static  final float GRAVITY = 9.8f;              // Force of gravity in a (down)
  public static  final float TERMINAL_VELOCITY = 76.0f;   // M/s due to gravity

  public static  final float RAD_CONVERT = (float)java.lang.Math.PI / 180; // Radians per Degree
  public static  final float DEGREE_CONVERT = 180 / (float)java.lang.Math.PI; //  Degrees per Radian


  public static  final float PI = (float)java.lang.Math.PI; // PI.
  public static  final float HALF_PI = (float) (java.lang.Math.PI / 2.0); // PI / 2.
  public static  final float QUARTER_PI = (float) (java.lang.Math.PI / 4.0); // PI / 2.
  public static  final float RADIANS = (float) (java.lang.Math.PI * 2.0); // Radians in a Circle
  public static  final float TWO_PI  = RADIANS;

  public static  float CMS_IN_M_3 = 1000000.0f;     // cubic centimetres in a cubic metre
  public static  float FILL_PERCENT = 0.1f;  // the percentage of the volume filled with the substance
                                                  // used to guess weight

  // Joint Power Output Ratios
  public static  final float POWER_CONSUMPTION_RATIO = 0.6f; // Ratio of power output to energy usage
  public static  final float POWER_ACCELERATION_RATIO = 10.0f; // Ratio of power to torque acceleration


  // Chassis Power Output Ratios
  public static  final float CHASSIS_POWER_CONSUMPTION_RATIO = 15; //50.0f; // Ratio of power output to energy usage

  //used in BJoint.getResultantMove
  // last twiddle 0.9 - 1.4f
  // DJG June 2001 - From 1.1/0.5 to 1.2/0.4 (more bounce)
  public static final float RESULTANT_BOUNCE_SCALE_PERCENT = 1.2f; // Overall percent of bounce
  public static final float JOINTABSORBTION_COEFFICIENT = 0.4f; // percentage of the power absorbed by the joint.  The higher the number the less a joints parent will bounce

  public static final float MIN_MOMENTUM_TO_MOVE = 5.0f; // the momentum required process a bounce
  public static final float DAMAGE_REQD_TO_STOP_MOVE = 20.0f;
  public static final float CHASSIS_ROTATIONAL_INERTIA = 0.4f; // the intial resistance to chassis rotational movements
                                                        // a higher number means more rotational movement (used in bchassis.rotate)
  public static final float DISPLACE_FRICTION = 0.4f; // displace friction.  HIgher the number, further a chassis will slide accross the ground

  public static final float DECELERATE_COEFFICIENT = 6.0f;  // How many times faster joint decelerates
  public static final float MAX_RAWACCELERATION = (float)java.lang.Math.PI; // Rad/s^2

  public static final float MAXRESULTJOINT_VELOCITY = 0.8f;  // the co-efficent to multiply the joints max velocity by to get its max velocity for resultant moves

  public static final float PERCENT_INERTIA_AFFECTS_DAMAGE = 0.05f; // the amount of momentum that is used in the damage calculations.
  public static final float WEIGHT_RATIO_CAP = 3f;
  public static final float COMPONENT_DAMAGE_CAP = 3000.0f; // Maximum Damage (Inertia)

  public static final float SUPERIOR_MOMENTUM_COEFFICENT = 0.03f;  // the amount the momentum a collision component must be large than it collidie to not stop a move
  public static final float SUPERIOR_MOMENTUM_DELETERATE_COEFF = 0.5f;

  public static final float DEADLOCK_CLEARING_MOMENTUM = 10000000f;
  public static final float CHASSISLOCKRESISTANCE = 0.3f;
  public static Point3f CENTEROFUNIVERSE = new Point3f(0,0,0);  // static for centre of univers.  to reduce memory burn
  public static Point3d CENTEROFUNIVERSE_D = new Point3d(0,0,0);  // static for centre of univers.  to reduce memory burn


  public static final float MAX_BOT_RADIUS = 15; // meters

  public static BArenaObjectPosition CENTREOBJECTPOSITION = new BArenaObjectPosition();  // static for centre of univers.  to reduce memory burn

  public static final float GROUND_RATIO = 0.3f;
  public static final float CONSTRUCTING_CHASSIS_MOVE_EXTENSION = 4f;

  public static final boolean CAP_VELOCITY_FLAG = true; // Truns Velocity Capping On and Off
  public static final int MAX_SLICES = 16; // Nbr of Slices the Engine Scales to. 1 Slice = 15 fps
  public static final float MAX_DELTA_DISTANCE = 0.1f; // Metres - the maximum distance any object should be able to travel in a timeslice
  public static final float MAX_VELOCITY_PER_SLICE = MAX_DELTA_DISTANCE * BGlobal.FRAMES_PER_SEC; // Used to scale engine in BSimulation.calcSubCycles()
  public static final float MAX_VELOCITY_CAP = MAX_VELOCITY_PER_SLICE * MAX_SLICES; // M/Sec - Maximum Velocity of any component in the simulator.
  //public static final float MAX_VELOCITY_DECELERATION_RATE = 0.05f; // 5% deceleration per cycle
  public static final int VELOCITY_CHECK_CYCLE = 4; // The rate that Velocity is checked (related to MAX_SLICES) 2 = Every 2nd Slice.


  /*
  public static final boolean CAP_VELOCITY_FLAG = true; // Truns Velocity Capping On and Off
  public static final int MAX_SLICES = 8; // 1 Slice = 15 fps
  public static final float MAX_DELTA_DISTANCE = 0.2f; // Metres - the maxium distance any object should be able to travel in a timeslice
  public static final float MAX_VELOCITY_PER_SLICE = MAX_DELTA_DISTANCE * BGlobal.FRAMES_PER_SEC; // Used to scale engine in BSimulation.calcSubCycles()
  public static final float MAX_VELOCITY_CAP = MAX_VELOCITY_PER_SLICE * MAX_SLICES; // M/Sec - Maximum Velocity of any component in the simulator.
  public static final float MAX_VELOCITY_DECELERATION_RATE = 0.05f; // 5% deceleration per cycle
  */
  //---DJG---




  public static final float modulus (float value, float divisor)
  {
    // Returns the remainder as a float. java % (modulus) only uses integers
    // Always returns a positive value
    if ( value < 0.0f )
      value = -value;
    if ( divisor < 0.0f )
      divisor = -divisor;
    if ( divisor == 0.0f )
      return 0.0f;
    else
    {
      float result = value / divisor;
      return value - (((int) result) * divisor);
    }
  }

  public static final float degToRad(float deg)
    { return deg * RAD_CONVERT; }

  public static final float radToDeg(float rad)
    { return rad * DEGREE_CONVERT; }


  public static final boolean worldEquals(float f1, float f2)
  {
   return ( java.lang.Math.round( f1 * 1000.0f ) == java.lang.Math.round( f2 * 1000.0f )  );

  }


  public static double[] base10 = { 1,
                                    10,
                                    100,
                                    1000,
                                    10000,
                                    100000,
                                    1000000,
                                    10000000,
                                    100000000,
                                    1000000000 };




  public static double round (double theNumber, int decimalPlaces )
  {

    //BDebug.print ("BPhysics.round  " + theNumber + " to " +  decimalPlaces + " decimal places ");
    double decimalMultiplier;

    if (decimalPlaces < base10.length)
      decimalMultiplier = base10[decimalPlaces];
    else
      decimalMultiplier =  java.lang.Math.pow(10.0, (double)decimalPlaces );


    theNumber = theNumber * decimalMultiplier;

    int roundedFloat  = java.lang.Math.round( (float) theNumber );



    double result =    roundedFloat /  (double)decimalMultiplier;
   // BDebug.print ("BPhysics.round  result" + result);

    return result;
  }

  public static float round (float theNumber, int decimalPlaces )
  {
    return  (float) round ((double) theNumber, decimalPlaces );
  }

  public static String roundToString(double theNumber, int decimalPlaces)
  {
    return roundToString(theNumber + "",  decimalPlaces);
  }

  public static String roundToString(float theNumber, int decimalPlaces)
  {
    float x = round(theNumber, decimalPlaces);
    return roundToString(x + "" ,  decimalPlaces);
  }

  private static String roundToString(String result, int decimalPlaces)
    // Returns string repn. of a rounded number, padded with 0's.
  {
    //float x = round(theNumber, decimalPlaces);
    //String result = ""+ x;

    //int n = result.length();
    int p = result.indexOf('.');


    if (decimalPlaces > 0)
    {
       int endIndex =  java.lang.Math.min( p + decimalPlaces + 1, result.length() );

       // if the last 2 digits if they are .0
       if (endIndex >= 2 )
        if ( (result.charAt(endIndex - 1) == '0' ) && (result.charAt(endIndex - 2) == '.' ) )
          endIndex = endIndex -2;

       if (p != -1)
        result = result.substring(0, endIndex);

       /*
      int z = decimalPlaces - (n - p) + 1;

      for (int i=0; i<z; i++)
        result += '0';   */
    }
    else
      if (p != -1)
        result = result.substring(0, p);

    return result;
  }

 private  static Transform3D lookAt_taTrans = new Transform3D();
 private  static Transform3D lookAt_tbTrans = new Transform3D();
 private  static Vector3d lookAt_aPos = new Vector3d();
 private  static Vector3d lookAt_bPos = new Vector3d();


  public static void getMidPoint(Point3d point, BArenaObject a, BArenaObject b)
  {
    // get the position of a
      BranchGroup lookAtr_aBG = a.getLiveNode(true);
      lookAtr_aBG.getLocalToVworld(lookAt_taTrans);
      lookAt_taTrans.get(lookAt_aPos);

      if (a == b)
      {
        point.set(lookAt_aPos);
        return;
      }
      // get the pos of B
      BranchGroup lookAtr_bBG = b.getLiveNode(true);
      lookAtr_bBG.getLocalToVworld(lookAt_tbTrans);
      lookAt_tbTrans.get(lookAt_bPos);

      // get the middle points
      double x;
      double y;
      double z;

      if (lookAt_aPos.x + lookAt_bPos.x == 0 )
        x = 0.0;
      else
        x=  ( lookAt_aPos.x + lookAt_bPos.x ) / 2.0;


      if (lookAt_aPos.y + lookAt_bPos.y == 0 )
        y = 0f;
      else
        y=  ( lookAt_aPos.y + lookAt_bPos.y ) / 2.0;

      if (lookAt_aPos.z + lookAt_bPos.z == 0 )
        z = 0f;
      else
        z=  ( lookAt_aPos.z + lookAt_bPos.z ) / 2.0;




      point.set(x,y,z);

  }


  public static float distanceBetween (BArenaObject a, BArenaObject b)
  // Distance between 2 points in metres ( Canvas-Coordinates go in );
  {

    BArenaObjectPosition aPos = a.getArenaPosition();
    BArenaObjectPosition bPos = b.getArenaPosition();

    //Point3f aPoint = new Point3f(aPos.x, 0, aPos.z);
    //Point3f bPoint = new Point3f(bPos.x, 0, bPos.z);

    return distanceBetween (aPos, bPos);

  }


   public static float distanceBetween (BArenaObjectPosition aPos, BArenaObjectPosition bPos)
  // Distance between 2 points in metres ( Canvas-Coordinates go in );
  {      

    Point3f aPoint = new Point3f(aPos.x, 0, aPos.z);
    Point3f bPoint = new Point3f(bPos.x, 0, bPos.z);

    return distanceBetween (aPoint, bPoint);



  }


  public static float distanceBetween (Point3f aPoint, Point3f bPoint)
  // Distance between 2 point in metres ( Canvas-Coordinates go in );

  {

    float canvasDistance =  aPoint.distance(bPoint);
    return canvasDistance  * BGlobal.CANVAS_TO_WORLD_SCALE;

  }


  private static Vector3f angleFrom_thisVec = new Vector3f() ;
  private static Vector3f angleFrom_otherVec  = new Vector3f();
  private static Vector3f angleFrom_resultant  = new Vector3f();
  private static Vector3f angleFrom_cross  = new Vector3f();



  public static float angleFrom(BArenaObjectPosition thisPos, BArenaObjectPosition otherPos )
  // 0 to 360 in degrees
  // clockwise looking from top
  // SHOULD MOVE TO BBOT WHEN BBOT IS FREE
  {
    // variables
    angleFrom_thisVec.set(thisPos.x,0, thisPos.z);
    angleFrom_otherVec.set(otherPos.x,0, otherPos.z);

    // calculate Angle
    //Vector3f resultant = new Vector3f();
    angleFrom_resultant.sub(angleFrom_otherVec, angleFrom_thisVec);
    Vector3f myDirection =  directionVector(thisPos.facing);
    float angle = myDirection.angle(angleFrom_resultant);


    // make the angle completely clockwise (not eitherway )
    //Vector3f crossdirectionV = new Vector3f();
    angleFrom_cross.cross(myDirection , angleFrom_resultant );
   // BDebug.print(" y " +  crossdirectionV.y);

    if (angleFrom_cross.y > 0)
      angle = (  (float)java.lang.Math.PI - angle  ) + (float)java.lang.Math.PI;

   // BDebug.print(thisObject + "angle " + angle / RAD_CONVERT);

    if ( Float.isNaN(angle) || Float.isInfinite(angle) )
      angle = 0.0f;


    return angle / RAD_CONVERT;


  }


   public static Vector3f directionVector(float myFacing)
 // Calculates a direction Vector, based on the facing in the ArenaPosition
 // Used for movement
 // p2 = p1+ directionVector*time
 {


    float x = 0.0f;
    float z = 0.0f;


    // if the angle is negative, normalise and make positive
    if (myFacing < 0.0f)
      {

        myFacing = -myFacing;

        // normalise --- 0 to 2*PI
        int devisor = (int)(myFacing / ((float)java.lang.Math.PI * 2.0f) );
        myFacing = myFacing - ((float)java.lang.Math.PI * 2.0f * (float)devisor );

        // invert
        myFacing =  ((float)java.lang.Math.PI * 2.0f) - myFacing;

       // BDebug.print (" Norm facing " +myFacing);

      }



    // Calculating the direction Vector using the 4 Quadrant Sin Cos Tan Rules
    // Mapping the mathematical x onto our z and y onto our x

    if (myFacing == 0.0f)
      {
        z = 1.0f;
      }
    else if ( (myFacing <= java.lang.Math.PI / 2.0f ))
      {
        z = (float)java.lang.Math.cos(myFacing);
        x = (float)java.lang.Math.sin(myFacing);


      }
    else if ( (myFacing <= java.lang.Math.PI) )
      {

        z = -(float)java.lang.Math.cos( java.lang.Math.PI - myFacing);
        x = (float)java.lang.Math.sin(java.lang.Math.PI - myFacing);
      }
   else if ( (myFacing <= java.lang.Math.PI  * 1.5f ) )
      {

        z = -(float)java.lang.Math.cos( myFacing - java.lang.Math.PI);
        x = -(float)java.lang.Math.sin( myFacing - java.lang.Math.PI);
      }
   else
      {

        z = (float)java.lang.Math.cos( java.lang.Math.PI * 2.0f - myFacing );
        x = -(float)java.lang.Math.sin( java.lang.Math.PI * 2.0f  - myFacing);
      }



  // BDebug.print("x " + x);
  // BDebug.print("z " + z);


    return new Vector3f(x,0,z);


 }

}