package roboforge;

import javax.vecmath.*;
import java.util.*;
import javax.media.j3d.*;
//import javax.swing.tree.*;
//import com.sun.j3d.utils.geometry.Box;
//import java.lang.Math.*;

import roboforge.geometry.*;



public class BRotateJoint extends BJoint
{

/*

* Description:          Joints that rotate around a centre  (Swivel, Hinge etc)
* Modification History ( Name, Date, Notes ):
                        YLY, 11 May 99

*/


  // This will prevent users placing 10000 kg at the end of a small pole

  //transient public float limbLoad;          // Torque  (newton metres)  The direction of the torque is always assumed to be tangental to the acceleration.
  //                                          // this is just for a gross load calculation.

  // Current Velocity will be stored in the sub joints

 protected float maxVelocity; // Max velocity ( radians Sec)
 //transient protected float acceleration = jointPowerOutput * BPhysics.POWER_ACCELERATION_RATIO;    // (radians / sec squared torque)
                                // the angular acceleration(Alpha) per Torque limbload )
 protected float maxAcceleration;  // radians per second squared

 static final long serialVersionUID = -1L;
 protected Vector3f axisOfRotation;
 private static double STEPSCALE = 0.02;
 private static final float  DEFAULT_MAX_VELOCITY = (float)java.lang.Math.PI;

// pie stuff
 private transient BranchGroup pieGroup;
 private static PolygonAttributes arcPolyAtt = new PolygonAttributes(PolygonAttributes.POLYGON_FILL,PolygonAttributes.CULL_NONE, 0f);
 private static TransparencyAttributes arcTransparency  = new TransparencyAttributes(TransparencyAttributes.NICEST, 0.8f);
 private static ColoringAttributes arcColor = new ColoringAttributes(BGlobal.colors.getLightestShade3f(), ColoringAttributes.NICEST);

 private transient TransformGroup componentCanvasPieTransform;




 public BRotateJoint (String nameIn, String filename)
 { super(nameIn, filename);  }

 public  void setMaxAcceleration(float newMaxAcceleration)
 {  maxAcceleration = newMaxAcceleration; }

 public  float getMaxAcceleration()
 { return maxAcceleration; }

 public void setAxisOfRotation (Vector3f newA)
 {
   if (axisOfRotation == null)
    axisOfRotation = newA;
   else
    axisOfRotation.set(newA);
 }

 public  Vector3f getAxisOfRotation()
 {
  return  axisOfRotation;

 }



 public void parseDirectionalLight(DirectionalLight tempL)
  {
   axisOfRotation = new Vector3f();
   tempL.getDirection(axisOfRotation);
   BDebug.print("BRotateJoint.parseDirectionalLight.Axis of rotation calculated : " +  axisOfRotation.toString() );

  }

  public Transform3D getDeltaTransform (int x, int y, int x_last, int y_last, BJointPosition lastPosition)
  {


  // ONLY USE Y
      double ystep = ( y_last - y) * STEPSCALE;

      lastPosition.setAngle(        java.lang.Math.max(maxFreedomNegative,
                                    java.lang.Math.min(maxFreedomPositive,lastPosition.getTranslate() + (float)ystep)
                                                )
                                        );

      return getTempTransform(false);
  }


  public Transform3D getDeltaTransform(float delta, BJointPosition last)
  {
    float angle = last.getAngle() + delta;

    if (angle < maxFreedomNegative)
      angle = maxFreedomNegative;

    if (angle > maxFreedomPositive)
      angle = maxFreedomPositive;

    last.setAngle(angle);

    return getTempTransform(false);
  }


 // local temp variables,  to reduce garbage collection of player
 // DO NOT USE ANYWHERE BUT the Method Below
 // or else multiple threads will fuck you up BIG TIME
 //transient private static AxisAngle4d jointRotate = new  AxisAngle4d();
 transient private static Transform3D getTransformTempT = new Transform3D();


 private Transform3D getTempTransform(float angle)
 {
   // BDebug.print (" BRotateJoint " + this + " getTempTransform " + angle );

    if (axisOfRotation.x == 1f)
      getTransformTempT.rotX(angle);
    else if (axisOfRotation.x == -1f)
      getTransformTempT.rotX(-angle);
    else if (axisOfRotation.y == 1f)
      getTransformTempT.rotY(angle);
    else if (axisOfRotation.y == -1f)
      getTransformTempT.rotY(-angle);
    else if (axisOfRotation.z == 1f)
      getTransformTempT.rotZ(angle);
    else if (axisOfRotation.z == -1f)
      getTransformTempT.rotZ(-angle);
    else
      BDebug.print ("ERROR BRotateJoint.getTempTransform Incorrect axisOfRotation " +  axisOfRotation );

    
    return getTransformTempT;

 }


  public Transform3D getTempTransform(boolean useDefault)
   // The Transform returned  by this method is a static variable filled with the data.
  // It MUST Be Cloned if you are going to store it.
 {

    if (currentPosition == null)
    {
       // BDebug.print ("BRotateJoint getTempTransform currentPosition created ");
        currentPosition = (BJointPosition)defaultJointPosition.clone();
    }

   

    float angle;

    if (useDefault  )
        angle = defaultJointPosition.getAngle();
    else
        angle = currentPosition.getAngle();

    return getTempTransform(angle);

 }


 public void upgrade()
 {
   BRotateJoint upgradeTwin =   (BRotateJoint)BComponent.loadFromFile( BGlobal.COMPONENTPATH+getComponentFilePath(), false );
   super.upgrade(upgradeTwin);
   setAxisOfRotation ( upgradeTwin.getAxisOfRotation());
   setMaxVelocity ( upgradeTwin.getMaxVelocity());

 }



  public void setSelected(
    boolean selected,
    boolean showSelectBox,
    boolean showSelectColor,
    boolean showConnectPts,
    boolean showInactiveConnectPoints,
    Color3f selectColor)
  {
    super.setSelected(selected, showSelectBox, showSelectColor, showConnectPts, showInactiveConnectPoints, selectColor);

    if (showSelectBox)
    {
      if (!selected)
        if (pieGroup != null)
        {
          pieGroup.detach();
          pieGroup = null;

        }

      if (selected && !this.getChassisLock())
      {
        if (pieGroup == null)
          attachPieToBot();
      }
    }


  }

 


 public BranchGroup cloneStructure3D(boolean cloneAdditions, boolean forLive)
{

  BranchGroup  tempBranch = super.cloneStructure3D(cloneAdditions, forLive);


  // add line of axis
  if (cloneAdditions)
  {
    componentCanvasPieTransform = new TransformGroup();
    componentCanvasPieTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE );


    attachPie(componentCanvasPieTransform);
    tempBranch.addChild(componentCanvasPieTransform);

  }
  return tempBranch;

}

 public void moveArcToSelectedConnectPoint()
 {
// BDebug.print ("moveArcToSelectedConnectPoint() called");
  if (componentCanvasPieTransform != null)
    componentCanvasPieTransform.setTransform(getPieTransform(this.getConnectPointSelected()));
 }

 


  private void attachPieToBot()
  {
        Group parent = getDirectLiveParentNode();

        pieGroup =  new BranchGroup();
        pieGroup.setCapability(BranchGroup.ALLOW_DETACH);

        //connectPointAlignGroup.addChild(pieGroup);
        TransformGroup connectPointAlignGroup = new TransformGroup (  getPieTransform(this.getConnectPointBase()));
        attachPie(connectPointAlignGroup);
        pieGroup.addChild(connectPointAlignGroup);
        parent.addChild(pieGroup);

  }

  private Transform3D getPieTransform( BConnectPoint basePoint)
  {
        Transform3D connectPointRot = new Transform3D();
        Vector3d baseV = new Vector3d(basePoint.getFacing() );
        Vector3d axis = new Vector3d();
        Vector3d cross = new Vector3d();

        if  (axisOfRotation.x == 1f)
        {
          baseV.x = 0;
          if (baseV.length() != 0)
          {
            axis.set(0,-1,0);
            double angle = axis.angle(baseV);
            cross.cross(baseV, axis);
            if (cross.x > 0)
              angle = -angle;
            connectPointRot.rotX(angle);
          }
        }
        else if (axisOfRotation.x == -1f)
        {
          baseV.x = 0;
          if (baseV.length() != 0)
          {
            axis.set(0,1,0);
            double angle = axis.angle(baseV);
            cross.cross(baseV, axis);
            if (cross.x > 0)
              angle = -angle;
            connectPointRot.rotX(angle);
          }

        }
        else if (axisOfRotation.y == 1f)
        {
          baseV.y = 0;
          if (baseV.length() != 0)
          {
            axis.set(0,0,1);
            double angle = axis.angle(baseV);
            cross.cross(baseV, axis);
            if (cross.y < 0)
              angle = -angle;
            connectPointRot.rotY(angle);
          }

        }
        else if (axisOfRotation.y == -1f)
        {
          baseV.y = 0;
          if (baseV.length() != 0)
          {
            axis.set(0,0,-1);
            double angle = axis.angle(baseV);
            cross.cross(baseV, axis);
            if (cross.y > 0)
              angle = -angle;
            connectPointRot.rotY(angle);
          }

        }
        else if (axisOfRotation.z == 1f)
        {
          baseV.z = 0;
          if (baseV.length() != 0)
          {
            axis.set(1,0,0);
            double angle = axis.angle(baseV);
            cross.cross(baseV, axis);
            if (cross.z < 0)
              angle = -angle;
            connectPointRot.rotZ(angle);
          }

        }
        else if (axisOfRotation.z == -1f)
        {
          baseV.z = 0;
          if (baseV.length() != 0)
          {
            axis.set(1,0,0);

            double angle = axis.angle(baseV);
            cross.cross(baseV, axis);
            if (cross.z > 0)
              angle = -angle;

            connectPointRot.rotZ(angle);
          }

        }



        return connectPointRot;



  }

  private void attachPie (Group attachGroup)
  {
       // the appearance
    Appearance a = new Appearance();
    a.setPolygonAttributes(arcPolyAtt);
    a.setTransparencyAttributes(arcTransparency);
    a.setColoringAttributes(arcColor);

    //BDebug.print (axisOfRotation);
    //BDebug.print ("volume " + getVolume() );
    // align the slice
    Transform3D pieAlign = new Transform3D();


    TransformGroup pieAlignGroup = new TransformGroup(pieAlign);
    pieAlignGroup.setPickable(false);

    double arcSize = java.lang.Math.max ( 10,  100.0 * getVolume());



    BPieSlice3D pie = new BPieSlice3D( maxFreedomPositive, maxFreedomNegative, arcSize, 20 , new Vector3d(axisOfRotation));

    pieAlignGroup.addChild(pie);
    pie.setAppearance(a);

    attachGroup.addChild( pieAlignGroup );

  }
  /*
  public void setAcceleration(float newAcceleration)
  {
    acceleration = newAcceleration;
  }

  public float getAcceleration()
  {
    return jointPowerOutput * BPhysics.POWER_ACCELERATION_RATIO;
  }
  */

  public final float STACK_PENALTY_CONSTANT = 1.5f;

  public  boolean move(BJointPosition endPosition, float timeSlice, float power,float maxVelocityMutiplier, boolean deccelerate)
  // Power = 0 to 1 (as a percentage)
  // if you Change the parameters of this method, Make sure you change the parameters of BAutoRotateoint.move so it corrently overides this
  {
    if (delay > 0 && !( getChassisLock() || BGlobal.CONSTRUCTING_CHASSISBOT )  )
    {
      delay = delay - timeSlice;
      return true;

    }

    // Pause Due to collision
    if ( standDown(pauseTime) )
    {
      return true;
    }

    float stackedFactor = (float)stackedPenalty - 2f;

    if(stackedFactor > 1)
    {
      //BDebug.print ( getName() +  " maxVelocityMutiplier pre:" + maxVelocityMutiplier);
      //BDebug.print ( getName() +  " STACK_PENALTY_CONSTANT " + STACK_PENALTY_CONSTANT );
      //BDebug.print ( getName() +  " (stackedFactor * STACK_PENALTY_CONSTANT)" + (stackedFactor * STACK_PENALTY_CONSTANT) );

      maxVelocityMutiplier = maxVelocityMutiplier / (stackedFactor * STACK_PENALTY_CONSTANT) ;
      //BDebug.print ( getName() + " stackedFactor " +  stackedFactor +  " maxVelocityMutiplier " + maxVelocityMutiplier);

    }

    // Cap the End Position in case james decreases the range
    endPosition.setAngle( java.lang.Math.max(maxFreedomNegative ,
                                      java.lang.Math.min(maxFreedomPositive,endPosition.getAngle())
                                                     )
                                  );


    int direction = 0;

    // set Max Velocity if it is not yet set
    if (maxVelocity <= 0.0f)
        maxVelocity = DEFAULT_MAX_VELOCITY;

    if (maxAcceleration == 0.0f)
        maxAcceleration = 2;

    // Get the direction of movement, or return false
    if (endPosition.getAngle() > currentPosition.getAngle() )
      direction = 1;
    else if (endPosition.getAngle() < currentPosition.getAngle() )
      direction =  -1;
    else
    {
      // Joint Move Completed
      return false;
    }


        // Joints velocity
        float oldVelocity;

        //oldVelocity = currentVelocity;

        /*
        if ( currentVelocity < 0f )
                oldVelocity = java.lang.Math.max(  currentVelocity , getMaxVelocity() * -1f * BPhysics.MAXRESULTJOINT_VELOCITY );
        else
                oldVelocity = java.lang.Math.min(  currentVelocity , getMaxVelocity()  * BPhysics.MAXRESULTJOINT_VELOCITY ) ;
        */
        if ( currentVelocity < 0f )
                oldVelocity = java.lang.Math.max(  currentVelocity , getMaxVelocity() * -1f );
        else
                oldVelocity = java.lang.Math.min(  currentVelocity , getMaxVelocity()   ) ;



        float deltaVelocity;

        float distanceInThisSlice = 0f;
        float oldPosition = currentPosition.getAngle();
        float newPosition = 0f;

        // if end position is infinity the it is a wheel type joint that continues in one direction
        if (  ( getChassisLock() || BGlobal.CONSTRUCTING_CHASSISBOT )  && !deccelerate )
        {

         // BDebug.print ("BRotateJoint CHASSISLOCK Move Called " + this);

          if ( getBot().getChassis().distanceThisSnapshot <= getBot().getChassis().distanceMovedThisSnapshot )
          {
              //BDebug.print("BRotateJoint.move: Forcing Chassis Joint to End Snapshot Position");

              currentVelocity = 0;

              /* DJG June 2001 - Causing Passthru
              setPreviousJointPosition(currentPosition);
              currentPosition.setAngle( endPosition.getAngle() );
              applyTransform();
              */

              return false;
          }

          // Part of Chassis. Distance moved is dependant on distance travelled by Chassis
          float chassisMoveDist = java.lang.Math.max( getBot().getChassis().distanceThisSnapshot
                                   - getBot().getChassis().distanceMovedThisSnapshot , 0.0f );

          float distThisSlice = getBot().getChassis().distThisSlice;

          chassisMoveDist = distThisSlice / chassisMoveDist; // ratio of distThisSlice with DistToGo

          distanceInThisSlice = endPosition.getAngle() - currentPosition.getAngle();

          distanceInThisSlice = distanceInThisSlice * chassisMoveDist;

          if ( distanceInThisSlice == 0.0f || Float.isNaN(distanceInThisSlice) )
          {
              currentVelocity = 0;
              //setPreviousJointPosition(currentPosition);
              currentPosition.setAngle( endPosition.getAngle() );
              applyTransform();

              return false;
          }


        }
        else
        {

          //if (endPosition.positionIsInfinity() )
            //BDebug.print (getName() +" BJoint.move chassis Joint at Infinity ");

          float jointCapability = ((float)damageMax-(float)getDamageCurrent())/(float)damageMax;

          float netLimbLoad = this.getNetLimbLoad(this.limbLoad, power);
          float acceleration = jointPowerOutput * BPhysics.POWER_ACCELERATION_RATIO;

          if ( (this.limbLoad == 0.0f) || Float.isNaN(this.limbLoad) )
            deltaVelocity = ( direction *  (acceleration * jointCapability * timeSlice * power) );
          else
            deltaVelocity = ( direction *  ( (acceleration * jointCapability * timeSlice * power) / netLimbLoad ) );

          //BDebug.print ("BRotateJoint.deltaVelocity " + deltaVelocity );




          if (!deccelerate)
          {
            if (deltaVelocity > 0)
              deltaVelocity = java.lang.Math.min( deltaVelocity , maxAcceleration * maxVelocityMutiplier * timeSlice );
            else
              deltaVelocity = java.lang.Math.max( deltaVelocity ,-maxAcceleration * maxVelocityMutiplier * timeSlice );

            //if (stackedFactor > 1)
           // {
           //   deltaVelocity = deltaVelocity / ( stackedFactor * STACK_PENALTY_CONSTANT) ;
           // }

          }
          //BDebug.print ("BRotateJoint.deltaVelocity capoped" + deltaVelocity );

           /*
            BDebug.print("BJoint.move " + this);
            BDebug.print("direction" + direction);
            BDebug.print("acceleration" + acceleration);
            BDebug.print("jointCapability" + jointCapability);
            BDebug.print("timeSlice" + timeSlice);
            BDebug.print("power" + power);
            BDebug.print("netLimbLoad" + netLimbLoad);  */
         
          
          // decelerate
          //BDebug.print("Joint="+toString() + " AccelScalar="+accelScalar+" AccelVectorY=" + torqueAcceleration.y);

          if (   ( (   java.lang.Math.abs( deltaVelocity) +  java.lang.Math.abs( currentVelocity) )
                        >= ( getMaxVelocity() * maxVelocityMutiplier) )
                  && !deccelerate )
            deltaVelocity  = direction * ( (getMaxVelocity() * maxVelocityMutiplier) -  java.lang.Math.abs( currentVelocity ) )     ;

          if (!deccelerate)
          {
            //DJG
             if ( maxVelocityExceededCtr > 0 )
            {
              deltaVelocity = 0.0f; // No Acceleration
              if ( maxVelocityExceededCtr > 1 ) // Decelerate
              {
                float decelerationRate = (timeSlice/BGlobal.CALC_TIMESLICE) * ((maxVelocityForLimb-BPhysics.MAX_VELOCITY_CAP) / maxVelocityForLimb);
                deltaVelocity = ( direction * java.lang.Math.abs(currentVelocity) * decelerationRate) * -1;
              }

              currentVelocity =  java.lang.Math.min( java.lang.Math.abs(currentVelocity + deltaVelocity),  getMaxVelocity()* maxVelocityMutiplier);
              currentVelocity = direction * java.lang.Math.abs(currentVelocity);

              //DJG
            }
            else
            {
              //BDebug.print("BRotateJoint.move: oldVelocity=" +currentVelocity+" dV/t="+deltaVelocity/timeSlice);
              currentVelocity =  java.lang.Math.abs(currentVelocity);
              currentVelocity =  java.lang.Math.min( currentVelocity + java.lang.Math.abs(deltaVelocity),  getMaxVelocity()* maxVelocityMutiplier);
              currentVelocity = direction * currentVelocity;
            }

          }
          else
          {
            //BDebug.print("Old currentVelocity" + currentVelocity);

            currentVelocity =  java.lang.Math.abs(currentVelocity);

            currentVelocity = java.lang.Math.max( currentVelocity -  java.lang.Math.abs(deltaVelocity )  * BPhysics.DECELERATE_COEFFICIENT ,0f);
            currentVelocity = java.lang.Math.min( currentVelocity, getMaxVelocity() );
            if (currentVelocity <= 0f)
            {
              currentVelocity = 0;
              //BDebug.print("BRotateJoint.move reached 0 velocity ");
              endPosition.setAngle( currentPosition.getAngle());
              return false;
            }
            else
            {
              currentVelocity = direction * java.lang.Math.min(java.lang.Math.abs(currentVelocity -  deltaVelocity), getMaxVelocity() );
            }


            //BDebug.print("New " + currentVelocity);
          }

          // delta x as a derivative of ut + 1/2at^2
          // x = ut + 1/2vt


          // cap delta velocity
          if (currentVelocity == oldVelocity)
            deltaVelocity = 0f;



          if (!deccelerate)
            distanceInThisSlice = oldVelocity * timeSlice + ( ( deltaVelocity / 2.0f ) * timeSlice  )  ;
          else
            distanceInThisSlice = oldVelocity * timeSlice ;

        } // End of chassisLock test

        //setPreviousJointPosition(currentPosition);

        if ( direction < 0 )
          newPosition = java.lang.Math.max ( oldPosition + distanceInThisSlice,  endPosition.getAngle());
        else
          newPosition = java.lang.Math.min ( oldPosition + distanceInThisSlice,  endPosition.getAngle());

      //  BDebug.print ("BRotateJoint.move " + this + "oldPos "+ currentPosition + " currentPosition  " + newPosition );

        currentPosition.setAngle(newPosition);


        applyTransform();


        // Calculation for snapshot timing (used for delay estimation)
        /*if ( oldVelocity == 0.0f && currentVelocity != 0.0f )
          bCompTransform.thisExecTime = 0.0f;
        if ( currentVelocity != 0.0f )
          bCompTransform.thisExecTime += timeSlice;  */

        //if ( !getChassisLock() )
        //  getBot().totalEnergy -= ((jointEnergyUsage * java.lang.Math.abs(currentVelocity) * limbLoad) * timeSlice * (maxVelocityMutiplier * maxVelocityMutiplier)) / 1000f;


    //BDebug.print("BRotateJoint.execute currentPosition.getAngle()" +  currentPosition.getAngle() );
    //BDebug.print("BRotateJoint.execute endPosition.getAngle() " + endPosition.getAngle()  );
   //BDebug.print("BRotateJoint.execute distanceInThisSlice " + distanceInThisSlice );


    return true;



  }



  /*
  public float workOutProcessingTime(BJointPosition endPosition, BComponentTransform bComponentTransform, float power)
  {
    // Work out time the time that it would take for the joint to get to this position
    // This is only used while transforming to the first snapshot position
    // All subsequent snapshots should have pre-calculated timing

    if ( endPosition.getAngle() == currentPosition.getAngle() )
    {
      timeToNextSnap = 0.0f;
    }
    else
    {



     // Average the torque between the current position to the target snapshot position
     float avgLimbLoad = this.limbLoad;
     if ( bComponentTransform.getTorque().length() != 0.0f )
      avgLimbLoad = ( this.limbLoad + bComponentTransform.getTorque().length() ) / 2;

     //BDebug.print(this.toString()+"  Avg Limb Load="+avgLimbLoad);

    // if (acceleration == 0.0f )
     // BDebug.print ("WARNING WARNING " + toString() + " has no Acceleration, this will stop the entire move ");

     timeToNextSnap = (float)java.lang.Math.sqrt(
                                java.lang.Math.abs(
                                                    (2.0 * ( endPosition.getAngle() - currentPosition.getAngle() )  ) / (    (acceleration * power) / avgLimbLoad  )
                                                  )
                                                );

    }
    return timeToNextSnap;


  }   */

  public void setMaxVelocity(float newVelocity)
  {
    maxVelocity = newVelocity;
  }

  public float getMaxVelocity()
  {
    if (  maxVelocity <= 0.0f )
      maxVelocity = 9999999.0f;
    return maxVelocity;
  }

  // DJG June 2001 - Used in getResultantPosition
  private transient static Transform3D t = new Transform3D();
  private transient static Vector3f jointPos = new Vector3f();
  private transient static Matrix3f rotOfJoint = new Matrix3f();
  private transient static Transform3D cT = new Transform3D();
  private transient static Vector3f collidedPos = new Vector3f();
  private transient static Vector3f collidedRelative = new Vector3f();
  private transient static Vector3f directionTester = new Vector3f();
  private transient static Vector3f directionCross = new Vector3f();

  public  float getResultantPosition ( Vector3f momentum, BComponent collidedComponent, BJointPosition newMove, BComponent otherComponent)
  // given a momentum vector,
  // collidedComponent is the Child of this join that collided
  // this method will return
  // newMove,  the Ending joint position of the new Move
  // the float is, the percentage of the momentum Vector absorbed by this movement (0  to 1 )
  {
        try
        {

        if (this.getDestroyed())
                return 0.0f;

        //BDebug.print( getName() + " axisOfRotation " + axisOfRotation);
       // ownerBot.setArenaComments( getName() + " axisOfRotation " + axisOfRotation);

        /* DJG These traps didnt work
        if ( collidedComponent == null || newMove == null || otherComponent == null )
        {
					BDebug.print("BRotateJoint.getResultantPosition: nulls detected! "+collidedComponent+" - "+newMove+" - "+otherComponent);
          return 0f;

        }


        if ( this.getLiveNode() == null || collidedComponent.getLiveNode() == null )
        {
					BDebug.print("BRotateJoint.getResultantPosition: live node is null! "+this.getLiveNode()+" - "+collidedComponent.getLiveNode());
          return 0f;
        }
        */

        // get the transform of the Joint
        //Transform3D t = new Transform3D();
        if ( !getLiveNode().isLive())
           BDebug.print("BRotateJoint.getResultantPosition :: This Joint is Not part of a live Graph!" + getName());

        getLiveNode().getLocalToVworld(t);

        //Vector3f jointPos = new Vector3f();
        t.get(jointPos);

        // INVERT , WE WILL USE THIS INVERTED MATRIX TO CONVERT OTHER Nodes into
        // a virtual space where the joint is the centre of the universe
        t.invert();
        // get only the rotational component  of the Joint's Transform
        //Matrix3f rotOfJoint = new Matrix3f();
        t.get(rotOfJoint);

        float powerBefore =  momentum.length();
        //BDebug.print( "power before transform " + momentum.length() );
        //BDebug.print( "Speed before " + momentum.length()  / this.weight);
        //BDebug.print( "   Momentum before transform " + momentum);

        //BDebug.print( "jointPos" + jointPos);
        //ownerBot.setArenaComments( "jointPos " + jointPos);

        // get the position of the collidedComponent
        //Transform3D cT = new Transform3D();
        if ( !collidedComponent.getLiveNode().isLive())
           BDebug.print("BRotateJoint.getResultantPosition :: collidedComponent is Not part of a live Graph!" + collidedComponent.getName());

        collidedComponent.getLiveNode().getLocalToVworld(cT);
        //Vector3f collidedPos = new Vector3f();
        cT.get(collidedPos);

        //BDebug.print( "collidedPos" + collidedPos);
        //ownerBot.setArenaComments( "collidedPos " + collidedPos);

        // get the vector from the joint to the collided component
        //Vector3f collidedRelative = new Vector3f();
        collidedRelative.sub(collidedPos, jointPos);
        // convert that vector into Joint Space (Where the Joint is the centre of the universe
        rotOfJoint.transform(collidedRelative);

        //BDebug.print( "collidedRelative" + collidedRelative);
        //ownerBot.setArenaComments( "collidedRelative " + collidedRelative);


        //Transform the Momentum into Joint Space (Where the joint is the centre of the universe )
        rotOfJoint.transform(momentum);

        //BDebug.print( "Transformed Momentum " + momentum);
        //ownerBot.setArenaComments( "Transformed Momentum " + momentum);

        //project momentum  and collitionRelative onto the joints 2d plane of movement;

        if (axisOfRotation.x != 0)
        {
              //  BDebug.print( "plane is YZ");
                //ownerBot.setArenaComments( "plane is YZ");
                momentum.x = 0;
                collidedRelative.x = 0;

        }
        else if (axisOfRotation.y != 0)
        {
              //  BDebug.print( "plane is XZ");
               // ownerBot.setArenaComments( "plane is XZ");
                momentum.y = 0;
                collidedRelative.y = 0;
        }
        else
        {
              //  BDebug.print( "plane is XY");
               // ownerBot.setArenaComments( "plane is XY");
                momentum.z = 0;
                collidedRelative.z = 0;

        }



        float projectedMomentum = momentum.length();
        //BDebug.print( "projectedMomentum " + projectedMomentum);
        //ownerBot.setArenaComments( "projectedMomentum" + projectedMomentum);

        // Useing the the cross produict (lefthand, righthand ) rule to determine direction
        momentum.normalize();

        //Vector3f directionTester = new Vector3f();
        directionTester.add(collidedRelative, momentum);
        //BDebug.print( "directionTester " + directionTester);

        //Vector3f directionCross = new Vector3f();
        directionCross.cross(directionTester, collidedRelative);

        int direction;

        if (axisOfRotation.x != 0)
        {
                if (directionCross.x > 0.0f )
                        direction = -1;
                else if (directionCross.x < 0.0f)
                        direction = 1;
                else
                        direction = 0;

        }
        else if (axisOfRotation.y != 0)
        {
                if (directionCross.y > 0.0f )
                        direction = 1;
                else if (directionCross.y < 0.0f)
                        direction = -1;
                else
                        direction = 0;
        }
        else
        {
                if (directionCross.z > 0.0f )
                        direction = 1;
                else if (directionCross.z < 0.0f)
                        direction = -1 ;
                else
                        direction = 0;

        }

        //BDebug.print ("JointPosition" + currentPosition.getAngle());
       // BDebug.print ("Default" +  defaultJointPosition .getAngle() );
       // BDebug.print (" Direction "  + direction );

        //ownerBot.setArenaComments("JointPosition " + current4Position.getAngle());
        //ownerBot.setArenaComments ("Default " + defaultJointPosition .getAngle() );
        //ownerBot.setArenaComments (" Direction "  + direction );


        /// XXXXXX THIS WILL CHANGE WHEN INERTIA IS PLACE IN
        // WILL CHANGE TO LIMB LOAD
        float v0 =  projectedMomentum  / ( collidedComponent.getStableNeigbourWeight()  + otherComponent.getStableNeigbourWeight() );   // initial Velocity in m/s

        if (this.getChassisLock())
          v0 = v0 * BPhysics.CHASSISLOCKRESISTANCE ;
        // half the momentum for chassis Joints


        //BDebug.print ("v0  " + v0 );

        // Speeed in R/s is roughly equivelent to the root of m/s (Hack)
        float r0 = (float) java.lang.Math.sqrt(v0);


       float limbWeight = Math.max(this.getLimbWeight(),getLimbLoadScalar() );

       float weightRatio;

        if (otherComponent == BArena.arenaComponent)
          weightRatio = BPhysics.GROUND_RATIO;
        else
          weightRatio = java.lang.Math.min( otherComponent.getStableNeigbourWeight() / limbWeight, BPhysics.WEIGHT_RATIO_CAP);

          /*
        BDebug.print (getName() + " getResultantMove ");
        weightRatio = weightRatio /2;
        BDebug.print ("weightRatio   " + weightRatio );
        BDebug.print ("this.getLimbWeight()   " + this.getLimbWeight()  );
        BDebug.print ("getLimbLoadScalar()   " + getLimbLoadScalar() );
        BDebug.print ("otherComponent.getStableNeigbourWeight()  " + otherComponent.getStableNeigbourWeight() );
         */

        //BDebug.print ( "BJoint.getResultantMove weightRatio" + weightRatio + " otherComponent.getDestroyed() " + otherComponent.getDestroyed());
        if ( otherComponent.getDestroyed())
        {
          this.currentVelocity = this.currentVelocity * BPhysics.SUPERIOR_MOMENTUM_DELETERATE_COEFF;
          return 1f;
        }


        float powerResist = (RESIST_POWER_COEFFICIENT / jointPowerOutput );


       /*
        BDebug.print ("projectedMomentum " +   projectedMomentum);
        BDebug.print ("collidedComponent.weight " +   collidedComponent.weight);
        BDebug.print ("v0 " +   v0);
        BDebug.print ("Direction " +   direction);
        BDebug.print ("r0 " +   r0);
        */
        if ( BGlobal.DEADLOCKDETECTED )
          this.currentVelocity = 0;
        else
        {
          // SET THE CURRENT VELOCITY OF THE JOINT
          this.currentVelocity =  r0  * BPhysics.RESULTANT_BOUNCE_SCALE_PERCENT ;
          this.currentVelocity = java.lang.Math.min(this.currentVelocity, getMaxVelocity()  );
          this.currentVelocity = this.currentVelocity * direction  * java.lang.Math.min( powerResist,1);
        }







        r0 = r0 * BPhysics.RESULTANT_BOUNCE_SCALE_PERCENT;
        //float MINBOUNCE_PERCENT = 0.01f;


        r0 = java.lang.Math.min(r0, getMaxVelocity());


        float bounce = r0 * weightRatio ;
        //BDebug.print ("Bounce   " + bounce );
        //ownerBot.setArenaComments( this.getName() + " resultant delta Movement after joint resisted " + bounce );

        if (this.getChassisLock())
          bounce = BPhysics.CHASSISLOCKRESISTANCE * bounce;


        float momentumAbsorbed = projectedMomentum / powerBefore;


        bounce = bounce * powerResist;

        // THIS IS THE RESUTANT POS USED TO CALCULATE MOMENTUM ABSORBED,
        // THE RESULTANT MOVE IS RECALCULATED
        float resultantPos = currentPosition.getAngle() + (direction * bounce);

        if (resultantPos > maxFreedomPositive)
        {
                resultantPos = maxFreedomPositive;
           //     BDebug.print ( "+momentumAbsorbed was " + momentumAbsorbed);
                // reduce momentumAbsorbed by the actualAmount moved
                momentumAbsorbed =  momentumAbsorbed * ( ( maxFreedomPositive -  currentPosition.getAngle()  )  /  bounce  );

             //    BDebug.print ( "+momentumAbsorbed is now " + momentumAbsorbed);

        }

        else if (resultantPos < maxFreedomNegative)
        {
                resultantPos = maxFreedomNegative;

       //         BDebug.print ( "-momentumAbsorbed is was " + momentumAbsorbed);
                // reduce momentumAbsorbed by the actualAmount moved
                momentumAbsorbed =  momentumAbsorbed * (
                                                                java.lang.Math.abs(
                                                                        (  maxFreedomNegative -   currentPosition.getAngle()  )
                                                                                )
                                                                /  bounce
                                                        );
            //    BDebug.print ( "-momentumAbsorbed is now " + momentumAbsorbed);

        }





        //BDebug.print ("BRotateJoint.getResultantMove()resultantPos" + resultantPos );
        //BDebug.print ("BRotateJoint.getResultantMove()currentPosition" + currentPosition.getAngle() );

        newMove.setAngle(resultantPos);
        if (this.getChassisLock())
          momentumAbsorbed = momentumAbsorbed / BPhysics.CHASSISLOCKRESISTANCE;

        return momentumAbsorbed;

      }catch(Exception e)
        {
                // Happens when hitting floor or wall
                //BDebug.print ("BRotateJoint.getResultantMove() ERROR " + e.getMessage() );
                return 0f;

        }



  }


  public float getFreedomRange()
    { return (float)java.lang.Math.round(BPhysics.radToDeg(maxFreedomPositive - maxFreedomNegative)); }

  public String getFreedomUnits()
    { return "degrees"; }



  public void cleanseLiveNodes()
  {

    super.cleanseLiveNodes();
    pieGroup = null;
    componentCanvasPieTransform = null;


  }

   public Vector getAttributeStrings()
 // returns a Vector of tuples  (heading, value )
 // 2-d Vector
 {

  Vector theList = super.getAttributeStrings();


  Vector tempV;

  tempV = new Vector(2);
  tempV.add("max velocity");
  tempV.add(  java.lang.Math.round( getMaxVelocity() * BPhysics.DEGREE_CONVERT)  +" degrees/sec");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("max acceleration");
  tempV.add(  java.lang.Math.round( getMaxAcceleration() * BPhysics.DEGREE_CONVERT)  +" degrees/sec^2");
  tempV.trimToSize();
  theList.add(tempV);


  return theList;
  }

  public void setDelay (float totalMoveTime)
  {
    delay = ( totalMoveTime - timeToNextSnap ) * 0.4f;

  }

  public float workOutProcessingTime(BJointPosition endPosition, float power)
  {
    // Work out time the time that it would take for the joint to get to this position
    // This is only used while transforming to the first snapshot position
    // All subsequent snapshots should have pre-calculated timing

    if ( endPosition.getAngle() == currentPosition.getAngle() )
    {
      timeToNextSnap = 0.0f;
    }
    else
    {

      /*
      Work out the time taken with formula s=ut + 1/2at^2

      1- re-arange to be a quadratic
        = (1/2)at^2 + ut + s = 0

      2- Combinine with general quadratic to get:
          t =  (   u (-+) sqrt( u^2 - 4(1/2)as )    )
                       /  2(1/2)a
         We are not carrying velocity through,  so the fomula can be simplified to

         t = Sqrt( 2s/a)
      */

     // Average the torque between the current position to the target snapshot position
     float avgLimbLoad = this.limbLoad;
     //if ( bComponentTransform.getTorque().length() != 0.0f )
      //avgLimbLoad = ( this.limbLoad + bComponentTransform.getTorque().length() ) / 2;

     //BDebug.print(this.toString()+"  Avg Limb Load="+avgLimbLoad);
     float acceleration = jointPowerOutput * BPhysics.POWER_ACCELERATION_RATIO;
    // if (acceleration == 0.0f )
   //   BDebug.print ("WARNING WARNING " + toString() + " has no Acceleration, this will stop the entire move ");

     timeToNextSnap = (float)java.lang.Math.sqrt(
                                java.lang.Math.abs(
                                                    (2.0 * ( endPosition.getAngle() - currentPosition.getAngle() )  ) / (    (acceleration * power) / avgLimbLoad  )
                                                  )
                                                );

    }
    return timeToNextSnap;


  }

  /*
  // DJG June 2001
  public void setCappedJointPosition(BJointPosition jointPos, float posFloat)
  {
    jointPos.setAngle( java.lang.Math.max(currentMoveMinPosition.getAngle() ,
                                      java.lang.Math.min(currentMoveMaxPosition.getAngle(),posFloat)
                                                     )
                                  );
  }
  */


}
