package roboforge;

import roboforge.ui.components.*;

//import java.util.*;
import java.awt.*;
//import javax.swing.*;
import java.awt.event.*;
import javax.vecmath.*;
import javax.swing.event.*;

public class BSlider extends roboforge.ui.components.BButton
{

  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
Image innerImage;
  Image mask;

  private Color selectedColor;
  private Color borderColor;

  int min;
  int max;
  int value;
  int lastY;
  int lastX;

  int MIDX;
  int MIDY;

  ChangeListener changeL = null;
  int mode;

  public static int MODE_ROTATE = 69;
  public static int MODE_HORIZONTAL_DEPTH = 70;
  public static int MODE_HORIZONTAL_BRIGHTNESS = 72;
  public static int MODE_HORIZONTAL = 71;

  private static final int barWidth = 8;

  public BSlider(Image innerRotateImage, Image newMask, int newmin, int newmax, int newvalue, int newMode)
  {
    this (null, null, innerRotateImage, newMask, newmin, newmax, newvalue, newMode);
  }

  public BSlider(BUIResources res, String key, Image innerRotateImage, Image newMask, int newmin, int newmax, int newvalue, int newMode)
  {
    super(res, key);

    // SJS [24/02/2000] - Make slider opaque
    // (since in fusion L&F, default is see-thru).
    setOpaque(false);

    min = newmin;
    max = newmax;
    value = newvalue;
    selectedColor = Color.black;

    if ( (newMode != MODE_ROTATE ) && (newMode != MODE_HORIZONTAL ) && (newMode != MODE_HORIZONTAL_DEPTH ) && (newMode != MODE_HORIZONTAL_BRIGHTNESS))
    {
        BDebug.print ( " BSlider.constructor ERROR invalid SLIDER MODE, setting to HORIZONTAL"  );
        mode = MODE_HORIZONTAL;


    }
    else
        mode = newMode;


    innerImage = innerRotateImage;
    mask = newMask;

    if (mode ==  MODE_ROTATE)
    {
      rotateMouseListener rotListener = new rotateMouseListener();
      this.addMouseListener(rotListener);
      this.addMouseMotionListener(rotListener);
    }
    else
    {
      horizMouseListener horizListener = new horizMouseListener();
      this.addMouseListener(horizListener);
      this.addMouseMotionListener(horizListener);
    }


  }

  public void initializeUI(BUIResources res, String key)
  {
    super.initializeUI(res, key);
    borderColor = BGlobal.colors.getDarkerShade();
  }


  public void setColor (Color newColor)
  {
    selectedColor =   newColor;
    repaint();
  }

  public void paint(Graphics g)
  {
    int value = this.getValue();
    Graphics2D g2 = (Graphics2D)g;


    if (mode == MODE_ROTATE)
    {
        g2.rotate(value * BPhysics.RAD_CONVERT, this.getWidth() /2.0, this.getHeight()/2.0 );
        g2.drawImage(innerImage,0,0,this);
        g2.rotate(-value * BPhysics.RAD_CONVERT, this.getWidth() /2.0, this.getHeight()/2.0 );

        g2.drawImage(mask,0,0,this);
    }
    else if (mode == MODE_HORIZONTAL_DEPTH)
    {
           if (BGlobal.DEPRECATED)
             this.getParent().paint(g);
           else
           {
             g2.setColor(this.getBackground());
             g2.fillRect(0,0, getWidth(), getHeight() );
           }

           int mid = this.getHeight() /2;
           int startX = mask.getWidth(this) / 2;
           int xLen =  getWidth() - mask.getWidth(this) ;

           // border

           g.setColor(borderColor);
           g.drawRect(startX - 1,(mid - barWidth)  - 1, xLen + 1 , (barWidth * 2 )+ 1);

           // top bar
           g.setColor(selectedColor);
           g.fillRect(startX,mid - barWidth ,xLen , barWidth );

           // gradient
           float[] hSB = Color.RGBtoHSB(selectedColor.getRed(),selectedColor.getGreen(),selectedColor.getBlue(),null);
           Color highlight = Color.getHSBColor(hSB[0], hSB[1], 1);

           GradientPaint gp = new GradientPaint(startX, mid, selectedColor ,xLen , barWidth  ,highlight);
           g2.setPaint(gp);

           //BDebug.print ("fil;ling " + mid + " end " + (mid + barWidth) );
           g2.fillRect(startX,mid ,xLen, barWidth );

           //int range = max - min;
          // BDebug.print ("range " +  range);
           //float along = (float)value / (float)range;
           //int centre = startX + (int)((float)xLen * along);

           drawIcon(g2);
           //g2.drawImage(mask,centre - (mask.getWidth(this) / 2),mid - (mask.getWidth(this) / 2),this);
    }
    else if (mode == MODE_HORIZONTAL_BRIGHTNESS)
    {
           if (BGlobal.DEPRECATED)
             this.getParent().paint(g);
           else
           {
             g2.setColor(this.getBackground());
             g2.fillRect(0,0, getWidth(), getHeight() );
           }


           int mid = this.getHeight() /2;
           int startX = mask.getWidth(this) / 2;
           int xLen =  getWidth() - mask.getWidth(this) ;

           // border

           g.setColor(borderColor);
           g.drawRect(startX - 1,(mid - barWidth)  - 1, xLen + 1 , (barWidth * 2 )+ 1);


           // BW Gradient
           GradientPaint gp = new GradientPaint(startX, mid - barWidth, Color.black ,xLen , barWidth * 2 ,Color.white);
           g2.setPaint(gp);
           g2.fillRect(startX,mid - barWidth,xLen, barWidth * 2 );

           // icon

           drawIcon(g2);
                      //g2.drawImage(mask,centre - (mask.getWidth(this) / 2),mid - (mask.getWidth(this) / 2),this);

    }
    else
    {


           // background

           g2.setColor(this.getBackground());
           g2.fillRect(0,0, getWidth(), getHeight() );


         //  this.getParent().paint(g);

           int mid = this.getHeight() /2;
           int startX = mask.getWidth(this) / 2;
           int xLen =  getWidth() - mask.getWidth(this) ;

           // border
           //g.setColor(Color.black);
           //g.fillRect(startX - 1,(mid - barWidth)  - 1, xLen + 2 , (barWidth * 2 )+ 2 );


           g2.setColor(this.getForeground());
           g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );

           g2.drawLine (startX, mid , startX + xLen, mid - barWidth);
           g2.drawLine (startX + xLen, mid - barWidth,  startX + xLen, mid + barWidth       );
           g2.drawLine (startX + xLen, mid + barWidth , startX, mid   );

          //g2.drawRect(startX,mid - barWidth ,xLen, barWidth * 2 );


           // icon
           int range = max - min;
           float along = (float)value / (float)range;
           along = java.lang.Math.min(along, 1f);


           //BDebug.print ("Along " + along );
           //int centre =  (int)((float)xLen * along);

           drawIcon(g2);

           //g2.drawImage(mask,centre - (mask.getWidth(this) / 2),mid - (mask.getWidth(this) / 2),this);



    }






  }

  private void drawIcon(Graphics2D g2)
  {
    int mid = this.getHeight() /2;
    int startX = mask.getWidth(this) / 2;
    int xLen =  getWidth() - mask.getWidth(this) ;

    int range = max - min;
    float along = (float)value / (float)range;
    int centre = startX + (int)((float)xLen * along);
    centre = java.lang.Math.min(centre, startX + xLen);

    int GAP = 2;

    g2.setColor(BGlobal.colors.getLightestShade());
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
    g2.drawOval(centre - (mask.getWidth(this) / 2),mid - (mask.getWidth(this) / 2),mask.getWidth(this),mask.getWidth(this) );
    g2.drawOval(centre - ((mask.getWidth(this) / 2 )- GAP),mid - ((mask.getWidth(this) / 2 )- GAP),mask.getWidth(this) - GAP *2 ,mask.getWidth(this) - GAP * 2);

  }


  public void setValue(int newValue)
  {
    if ( newValue >  max)
      value = max;
    else if (newValue < min)
      value = min;
    else
    value = newValue;


    //BDebug.print (value);

    if (changeL != null)
        changeL.stateChanged( new ChangeEvent (this));

     repaint();

  }

  public int getValue()
  {
    return value;

  }

  public void addChangeListener( ChangeListener newC)
  {
    changeL = newC;

  }


  // inner classes

  class horizMouseListener
    implements MouseListener, MouseMotionListener
  {
    public void mouseClicked(MouseEvent e)  { }
    public void mouseEntered(MouseEvent e)  { }
    public void mouseExited(MouseEvent e)   { }
    public void mouseReleased(MouseEvent e) { }
    public void mousePressed(MouseEvent e)
      { handleMouse(e); }
    public void mouseDragged(MouseEvent e)
      { handleMouse(e); }

    public void handleMouse(MouseEvent e)
    {
                int x = e.getX();
                /*
                int STEP = 1;
                setValue (getValue()-( lastX - x));
                BDebug.print (getValue());

                lastX = x;
                */
                int xLen =  getWidth() - mask.getWidth(null) ;
                float proportionAcross = (( (float)x - ( (float)mask.getWidth(null) / 2f)  ) / (float)xLen );


                //BDebug.print (proportionAcross);

                setValue( min + (int)(proportionAcross * ((float)max - (float)min))  );



                repaint();


        }

        public void mouseMoved(MouseEvent e)
        {
          lastY = e.getY();
          lastX = e.getX();
        }



  }


  class rotateMouseListener
    implements MouseListener, MouseMotionListener
  {
    public void mouseClicked(MouseEvent e)  { }
    public void mouseEntered(MouseEvent e)  { }
    public void mouseExited(MouseEvent e)   { }
    public void mouseReleased(MouseEvent e) { }
    public void mousePressed(MouseEvent e)
      { handleMouse(e); }
    public void mouseDragged(MouseEvent e)
      { handleMouse(e); }

    public void handleMouse(MouseEvent e)
    {
          int STEP = 5;

          int y = e.getY();
          int x = e.getX();

          MIDX = getWidth() / 2;
          MIDY = getHeight()/ 2;

          // WORK OUT ROTATION DIRECTION
          Vector3f oldV = new Vector3f(-(MIDX - lastX) ,-( MIDY - lastY  ), 0);
          Vector3f newV = new Vector3f( -(MIDX - x  ) , -( MIDY - y ), 0);
          Vector3f resultant = new Vector3f();

          resultant.cross(newV, oldV);

          if (resultant.z == 0)
            {
            // do noting
            }
          else if ( resultant.z > 0 )
            setValue(getValue() - STEP);
          else
             setValue(getValue() + STEP);

          //BDebug.print(resultant.z);


          lastX = x;
          lastY = y;


          repaint();



        }


         public void mouseMoved(MouseEvent e)
        {
          lastY = e.getY();
          lastX = e.getX();
        }



  }





}