package roboforge;

import java.util.*;
import java.io.*;
import roboforge.util.data.*;
import roboforge.net.data.*;

public class SChallengeRoom extends Observable
{
  /*
   *
   *
   *
   */

  // Member variables...
  // -------------------------------------------------------------
  private boolean changed = false;
  private Vector obs;

  long roomNumber;
  String roomName;
  String roomDescription;
  int maxNumberOfParticipants;
  String arena;
  float maxEntryCost;
  float maxFightTime;
  int simulationSpeed;
  // See status codes on roboforge.net.data.BChallengeRoomData
  String password;

  boolean justCreated = true;
  // Used to give the room a probation period from being deleted if there aren't
  // any people in it due to a delay between the room being created and the creator entering it

  public Map accountCollection = Collections.synchronizedMap(new HashMap());
  // Holds a map of BAccount Objects

  public Map tournamentCollection = Collections.synchronizedMap(new HashMap());
  // In a challenge room there is a one to one relationship between a fight
  // and a tournament ie a Tournament only every has two entries and therefore
  // only one fight

  private PrintWriter roomLog;

  public int getNoOfPeopleInRoom ()
  {
    return accountCollection.size();
  }

  public void addAccount (SAccount account)
  {
    // If the account doesn't already exist add it
    if (accountCollection.containsKey(account.accountID))
      return;
    else
    {
      accountCollection.put(account.accountID,account);
    }

  }
  /**
  * Adds an observer to the set of observers for this object, provided
  * that it is not the same as some observer already in the set.
  * The order in which notifications will be delivered to multiple
  * observers is not specified. See the class comment.
  *
  * @param   o   an observer to be added.
  */
  public synchronized void addObserver(Observer o)
  {
    // Update the time last connected
    SAccount account = (SAccount)accountCollection.get(((SChallengeRoomServlet.MessageSink)o).accountID);
    account.timeLastConnected = System.currentTimeMillis();
    account.afk = false;
    
    if (!obs.contains(o))
      {
        obs.addElement(o);
	    }
  }

  public synchronized boolean observerExists(Observer o)
  {
    if (obs.contains(o))
      return true;
    else
      return false;

  }

  /**
  * Deletes an observer from the set of observers of this object.
  *
  * @param   o   the observer to be deleted.
  */
  public synchronized void deleteObserver(Observer o)
  {
    obs.removeElement(o);
  }

  public synchronized void logoutAccount(Long account)
  {
    obs.removeElement(new SChallengeRoomServlet.MessageSink(account.longValue()));
    accountCollection.remove(account);
  }


  /**
  * If this object has changed, as indicated by the
  * <code>hasChanged</code> method, then notify all of its observers
  * and then call the <code>clearChanged</code> method to
  * indicate that this object has no longer changed.
  * <p>
  * Each observer has its <code>update</code> method called with two
  * arguments: this observable object and <code>null</code>. In other
  * words, this method is equivalent to:
  * <blockquote><tt>
  * notifyObservers(null)</tt></blockquote>
  *
  * @see     java.util.Observable#clearChanged()
  * @see     java.util.Observable#hasChanged()
  * @see     java.util.Observer#update(java.util.Observable, java.lang.Object)
  */
  public void notifyObservers()
  {
    notifyObservers(null);
  }

  /**
  * If this object has changed, as indicated by the
  * <code>hasChanged</code> method, then notify all of its observers
  * and then call the <code>clearChanged</code> method to indicate
  * that this object has no longer changed.
  * <p>
  * Each observer has its <code>update</code> method called with two
  * arguments: this observable object and the <code>arg</code> argument.
  *
  * @param   arg   any object.
  * @see     java.util.Observable#clearChanged()
  * @see     java.util.Observable#hasChanged()
  * @see     java.util.Observer#update(java.util.Observable, java.lang.Object)
  */
  public void notifyObservers(Long accountID, Object arg)
  {
    /*
    * a temporary array buffer, used as a snapshot of the state of
    * current Observers.
    */
    Object[] arrLocal;

    synchronized (this)
    {
      /* We don't want the Observer doing callbacks into
      * arbitrary code while holding its own Monitor.
      * The code where we extract each Observable from
      * the Vector and store the state of the Observer
      * needs synchronization, but notifying observers
      * does not (should not).  The worst result of any
      * potential race-condition here is that:
      * 1) a newly-added Observer will miss a
      *   notification in progress
      * 2) a recently unregistered Observer will be
      *   wrongly notified when it doesn't care
      */

      // Update all the registered members with
      // the message

      if (accountID != null)
      {
        // Send a message to an individual


        // Log the Message
        if (SGlobal.isRoomLogingOn)
        {
          // Log the message to file if it's a message
          if (arg instanceof BMessageData)
          {
            BMessageData messageTemp = (BMessageData)arg;
            roomLog.println(messageTemp.senderAccountID + ": " + messageTemp.designerName + ": Whispers to: " +
            accountID + ": " + messageTemp.message);
          }
        }



        SAccount account = (SAccount)accountCollection.get(accountID);

        if (!account.afk)
        {
          account.setMessage(arg);

          try
          {
            // Find the sleeping message sink and send it the message
            SChallengeRoomServlet.MessageSink messageSinkToFind = new SChallengeRoomServlet.MessageSink(accountID.longValue());

            int indexOfSink = obs.indexOf(messageSinkToFind);
            SChallengeRoomServlet.MessageSink sleepingMessageSink = (SChallengeRoomServlet.MessageSink)obs.get(indexOfSink);

            // Check to see if the ID match, just in case someone logged out between getting the index and
            // actually getting the message sink.
            // TODO: Is this needed as we are synchronized
            if (sleepingMessageSink.accountID.equals(accountID))
            {
              // Send the messages
              BDataSet dataSet = new BDataSet();
              dataSet.add(account.getAndClearMessages());
              sleepingMessageSink.update(this,dataSet);
              deleteObserver(sleepingMessageSink);
            }
          }
          catch (Exception e)
          {
            // Is ok to throw away any errors at this stage as
            // any message has already made it to the message store and will be given to the
            // client once they reconnect
            BDebug.print("Error: SChallengeRoom/notifyObserver: " + e.getMessage());
          }
        }
      }
      else
      {
        // Message to everyone

        // Log the Message
        if (SGlobal.isRoomLogingOn)
        {
          // Log the message to file if it's a message
          if (arg instanceof BMessageData)
          {
            BMessageData messageTemp = (BMessageData)arg;
            roomLog.println(messageTemp.senderAccountID + ": " + messageTemp.designerName +
            ": " + messageTemp.message);
          }
        }

        setChanged();

        Iterator accounts = accountCollection.values().iterator();

        while(accounts.hasNext())
        {
          SAccount account = (SAccount)accounts.next();
          account.setMessage(arg);
        }

        if (!changed)
          return;
        arrLocal = obs.toArray();
        clearChanged();


        for (int i = arrLocal.length-1; i>=0; i--)
        {
          SChallengeRoomServlet.MessageSink messageSink = (SChallengeRoomServlet.MessageSink)arrLocal[i];
          SAccount account = (SAccount)accountCollection.get(messageSink.accountID);


          BDataSet dataSet = new BDataSet();
          dataSet.add(account.getAndClearMessages());

          if (account.accountID.longValue() == 24)
          {
            // Print out all the messages being sent
            /*BDebug.print("$$$$$ Length of Data set : " + dataSet.data.length);
            for (int j = 0;j<dataSet.data.length;j++)
            {
              BDebug.print("Data sent of type: " + ((BAbstractAtom)dataSet.data[j]).getType());

            } */
          }


          messageSink.update(this,dataSet);
          deleteObserver(messageSink);

          // TODO: Slight possibility between the sending of the messages
          // and the clearing of the message, another message might be popped added
          // in which case it would be deleted and never sent
          // Solution to this would be to count the number of messages sent and only
          // clear this number of messages in the Vector
        }
      }
    }
  }

  /*
  // Sends a message to a single observer
  public synchronized void notifyObserver (Long accountID, Object arg)
  {
    SAccount account = (SAccount)accountCollection.get(accountID);
    account.messages.addElement(arg);

    // Find the sleeping message sink and send it the message
    SChallengeRoomServlet.MessageSink messageSinkToFind = new SChallengeRoomServlet.MessageSink(accountID.longValue());



    int indexOfSink = obs.indexOf(messageSinkToFind);
    SChallengeRoomServlet.MessageSink sleepingMessageSink = (SChallengeRoomServlet.MessageSink)obs.get(indexOfSink);

    // Check to see if the ID match, just in case someone logged out between getting the index and
    // actually getting the message sink.
    if (sleepingMessageSink.accountID.equals(accountID))
    {
      // Send the messages
      BDataSet dataSet = new BDataSet();
      dataSet.add(account.messages);
      sleepingMessageSink.update(this,dataSet);
      account.messages.clear();
    }
  }  */

  /**
  * Clears the observer list so that this object no longer has any observers.
  */
  public synchronized void deleteObservers()
  {
    obs.removeAllElements();
  }

  /**
  * Marks this <tt>Observable</tt> object as having been changed; the
  * <tt>hasChanged</tt> method will now return <tt>true</tt>.
  */
  protected synchronized void setChanged()
  {
    changed = true;
  }

  /**
  * Indicates that this object has no longer changed, or that it has
  * already notified all of its observers of its most recent change,
  * so that the <tt>hasChanged</tt> method will now return <tt>false</tt>.
  * This method is called automatically by the
  * <code>notifyObservers</code> methods.
  *
  * @see     java.util.Observable#notifyObservers()
  * @see     java.util.Observable#notifyObservers(java.lang.Object)
  */
  protected synchronized void clearChanged()
  {
    changed = false;
  }

  /**
  * Tests if this object has changed.
  *
  * @return  <code>true</code> if and only if the <code>setChanged</code>
  *          method has been called more recently than the
  *          <code>clearChanged</code> method on this object;
  *          <code>false</code> otherwise.
  * @see     java.util.Observable#clearChanged()
  * @see     java.util.Observable#setChanged()
  */
  public synchronized boolean hasChanged()
  {
    return changed;
  }

  /**
  * Returns the number of observers of this <tt>Observable</tt> object.
  *
  * @return  the number of observers of this object.
  */
  public synchronized int countObservers()
  {
    return obs.size();
  }

  public SChallengeRoom(int roomNumber,
                        String roomName,
                        String roomDescription,
                        int maxNumberOfParticipants,
                        String arena,
                        float maxEntryCost,
                        float maxFightTime,
                        int simulationSpeed,
                        String password
                        )
  {
    this.roomNumber               = roomNumber;
    this.roomName                 = roomName;
    this.roomDescription          = roomDescription;
    this.maxNumberOfParticipants  = maxNumberOfParticipants;
    this.arena                    = arena;
    this.maxEntryCost             = maxEntryCost;
    this.maxFightTime             = maxFightTime;
    this.simulationSpeed          = simulationSpeed;
    this.password                 = password;

    obs = new Vector();

    try
    {
      if (SGlobal.isRoomLogingOn)
      {
        // Create the Room Log
        roomLog = new PrintWriter(new BufferedWriter(new FileWriter(SGlobal.LOG_PATH + roomName + roomNumber + ".txt")));
        roomLog.println("Room Description: " + roomDescription);
        roomLog.println("Room Password: " + password);
      }
    }
    catch (IOException e)
    {
      System.out.println("Error: SChallengeRoom Constructor: Can't Create Roon Log: "+ e);
    }
  }
  public void destroy()
  {
    if (roomLog != null)
    {
      roomLog.flush();
      roomLog.close();
    }
  }
}