package roboforge;

import java.util.*;
import java.sql.*;
import javax.swing.tree.*;
/*

* Description:
  Handles requests for Forum related information
* Example:
* Modification History ( Name, Date, Notes ):
  Created HSC, 29-May-2000

*/
public class SForumController
{
  private Vector forums = new Vector();

  // OK this is needed as initally we need to send all the Forum information to the client
  // except the tree information this solves several issues including the timing of updating the forums
  private Vector forumsWithoutTrees = new Vector();


  public SForumController(Connection con)
  {

    Statement statement = null;
    ResultSet rs = null;


    try
    {

      statement = con.createStatement();
      String sql = "SELECT id, name, purpose FROM FORUM ORDER BY name";
      rs = statement.executeQuery(sql);

      // Populate the forum Vector
      while (rs.next())
      {
        SForum forum = new SForum(rs.getInt("id"),rs.getString("name"),rs.getString("purpose"),null);
        forums.addElement(forum);

        SForum forumWithoutATree = new SForum(rs.getInt("id"),rs.getString("name"),rs.getString("purpose"),null);
        forumsWithoutTrees.addElement(forumWithoutATree);

        // A little bit Kludgy, but need to seperate out the update method from the creation of the forum
        updateForum(con, forum);
      }

      rs.close();
      statement.close();

    }
    catch (Exception e)
    {
      System.out.println("Error: " + e.getMessage());
      try
      {
        rs.close();
        statement.close();
      }
      catch (Exception e1)
      {
        System.out.println("Error: " + e1.getMessage());
      }
    }
  }

  public Vector getForumsWithoutTrees ()
  {
    // Return the list of available forums
    return forumsWithoutTrees;
  }

  public DefaultTreeModel getForumTree(int forumID, Connection con)
  {
    // Return the tree model from the forum but first updating the forum
    // if it has changed and/or the update time is due
    SForum requestedForum = null;
    DefaultTreeModel treeModel = null;

    // Find the requested forum

    try
    {
      Enumeration e = forums.elements();

      boolean found = false;
      while(e.hasMoreElements() && !found)
      {
        SForum currentForum = (SForum)e.nextElement();
        if (currentForum.forumID == forumID)
        {
          // Update the forum to changed and exit the loop
          requestedForum = currentForum;
          found = true;
        }
      }

      // Check to see if it's being rebuilt
      int loop = 0;
      while (requestedForum.beingRebuilt && loop < 5)
      {
        Thread.currentThread().sleep(1000);
        loop++;
      }

      if (!requestedForum.beingRebuilt)
      {
        // So it's not being rebuilt
        
        // Determine if the requested forum needs to be updated
        // Only update the requested forum if the current update time has passed and it has changed
        Calendar currentCalendar = Calendar.getInstance();
        currentCalendar.setTime(new java.util.Date(System.currentTimeMillis()));

        Calendar forumCalendar = Calendar.getInstance();
        forumCalendar.setTime(requestedForum.lastUpdated);
        forumCalendar.add(Calendar.HOUR_OF_DAY,SGlobal.FORUM_REFRESH_HOURS);

        if (forumCalendar.after(currentCalendar) && requestedForum.updated);
        {
          // And it needs rebuilding
          updateForum(con, requestedForum);
        }

       treeModel = requestedForum.forumTreeModel;
      }
    }
    catch (Exception e)
    {
      System.out.println("Error: " + e.getMessage());
    }

    return treeModel;

  }

  public String setDetails(int forumID, int accountID, int parentThread, String subject, String details, Connection con)
  {
    Statement statement = null;
    try
    {

      statement = con.createStatement();

      String sql = "INSERT INTO FORUM_ENTRY (forum_id, account_id, subject, details, parent_thread) VALUES (" + forumID + "," + accountID + ",'" + SServerAccess.parseString(subject) + "','" + SServerAccess.parseString(details) + "'," + parentThread + ")";
      statement.executeUpdate(sql);

      statement.close();

      // Find the related forum in the list and set its updated status to true
      Enumeration e = forums.elements();

      boolean found = false;
      while(e.hasMoreElements() && !found)
      {
        SForum currentForum = (SForum)e.nextElement();
        if (currentForum.forumID == forumID)
        {
          // Update the forum to changed and exit the loop
          currentForum.updated = true;
          found = true;
        }
      }

    }
    catch (Exception e)
    {
      System.out.println("Error: " + e.getMessage());
      try
      {
        statement.close();
      }
      catch (Exception e1)
      {
        System.out.println("Error: " + e1.getMessage());
      }
    }

    return details;
  }

  public String getDetails(int forumThreadID, Connection con)
  {
    Statement statement = null;
    ResultSet rs = null;
    String details = "";

    try
    {

      statement = con.createStatement();
      String sql = "SELECT details FROM FORUM_ENTRY WHERE id = "+ forumThreadID;
      rs = statement.executeQuery(sql);

      // Populate the forum Vector
      while (rs.next())
      {
        details = rs.getString("details");
      }
      rs.close();
      statement.close();

    }
    catch (Exception e)
    {
      System.out.println("Error: " + e.getMessage());
      try
      {
        rs.close();
        statement.close();
      }
      catch (Exception e1)
      {
        System.out.println("Error: " + e1.getMessage());
      }
    }

    return details;
  }

  public void updateForum(Connection con, SForum forum)
  {
    forum.beingRebuilt = true;

    // Get all the records for this forum sorted by parent
    Statement statement = null;
    ResultSet rs = null;


    try
    {
      // %% Might need to set date range for forum_entries (threads)

      statement = con.createStatement();
      String sql = "SELECT fe.id, fe.subject, fe.date_of_entry, fe.parent_thread, a.name";
      sql = sql + " FROM FORUM_ENTRY fe, ACCOUNT a";
      sql = sql + " WHERE fe.account_id = a.id AND fe.forum_id = " + forum.forumID;
      sql = sql + " ORDER BY fe.parent_thread, fe.date_of_entry";

      rs = statement.executeQuery(sql);


      // Create a dummy ForumThread object to sit in the root node with an id value of 0.
      // This is needed so that all threads with no parents (parent value of zero) attach themselves
      // to the root node of the tree
      SForumThread dummyRootForumThread = new SForumThread(0,"New Thread",null,0,null);

      DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(dummyRootForumThread);
      DefaultTreeModel forumTree = new DefaultTreeModel(rootNode);

      // Set the default values to the root node
      int currentParent = 0;
      DefaultMutableTreeNode currentParentNode = rootNode;

      // Populate the forum Vector Tree
      while (rs.next())
      {
        SForumThread forumThread = new SForumThread(rs.getInt("id"),rs.getString("subject"),rs.getDate("date_of_entry"),rs.getInt("parent_thread"),rs.getString("name"));
        DefaultMutableTreeNode threadNode = new DefaultMutableTreeNode(forumThread);
        if(currentParent != forumThread.parent_thread)
        {
          // The currentParentNode isn't the parent of the forum thread
          // so we need to do a tree walk until we find the node which is the parent of the current thread
          // and make the currentParent Node equal to it
          Enumeration e = rootNode.breadthFirstEnumeration();
          boolean found = false;
          while (e.hasMoreElements() && !found)
          {
            // Doing the tree walk
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)e.nextElement();
            SForumThread currentThread = (SForumThread)node.getUserObject();
            if(currentThread.id == forumThread.parent_thread)
            {
              // Have found the parent for the thread
              currentParentNode = node;
              currentParent = currentThread.id;
              found = true;
            }
          }
        }

        forumTree.insertNodeInto(threadNode,currentParentNode,0);
        //threadNode.setParent(currentParentNode);
      }

      rs.close();
      statement.close();

      // Now that the Tree has been populated set the forum tree to point to it
      // and set the flag to indicate that the rebuild has finished

      forum.forumTreeModel = forumTree;

      forum.lastUpdated = new java.util.Date(System.currentTimeMillis());
      forum.updated = false;

      forum.beingRebuilt = false;

    }
    catch (Exception e)
    {
      System.out.println("Error: " + e.getMessage());
      try
      {
        rs.close();
        statement.close();
      }
      catch (Exception e1)
      {
        System.out.println("Error: " + e1.getMessage());
      }
    }
    forum.beingRebuilt = false;
  }
} 