package roboforge;

import java.io.*;
import java.util.*;
import java.sql.*;
import javax.servlet.*;
//import javax.swing.ImageIcon;
import java.net.*;
import java.math.*;

import oracle.sql.*;
import oracle.jdbc.driver.*;

import roboforge.net.*;
import roboforge.net.data.*;
import roboforge.net.client.*;
//import roboforge.net.server.*;
import roboforge.net.services.*;
//import roboforge.util.io.*;
import roboforge.util.data.*;
import roboforge.server.util.*;
import roboforge.util.*;


public class SGateway extends SGatewayAbstract
{

  // Member variables...
  // -------------------------------------------------------------

  // Location of Patch Server
  private String URLPATCH;

  // Location of Tournament Server
  private String URLTOURNAMENT;

  // Location of Challenge Rooms Server
  private String URLCHALLENGE;


  long roboforgeAccountID;

  // Servlet info method...
  // -------------------------------------------------------------

  public String getServletInfo()
    { return "roboforge.SGateway"; }

  // Request Handler initialization...
  // -------------------------------------------------------------

  protected void initializeHandlers()
  {
    handlers = new HashMap();

    register(new EchoHandler());
    //register(new AccountCreationHandler());
    register(new LoginHandler());
    register(new OpenTournamentRequestHandler());
    register(new BattleRecordingRequestHandler());
    register(new TournamentHistoryRequestHandler());
    register(new SubmitBotRequestHandler());
    register(new AccountInformationRequestHandler());
    register(new RemoveBotFromTournamentRequestHandler());
    register(new CreateAccountRequestHandler());
    register(new AutoPatchRequestHandler());
    register(new MessageRequestHandler());
    register(new ChangeAccountRequestHandler());
    register(new SetTournamentImageRequestHandler());
    register(new GetSoftURLHandler());
    register(new UpdateClientVersionRequestHandler());
    register(new BotRequestHandler());
    register(new GetServerTimeHandler());
    register(new NominateFeaturedFightRequestHandler());
    register(new FeaturedFightInfoRequestHandler());
    register(new UpdateFeaturedFightRequestHandler());
    register(new FeaturedFightTypesRequestHandler());
    register(new MoveFeaturedFightRequestHandler());
    register(new CopyFeaturedFightRequestHandler());
    register(new DeleteFeaturedFightRequestHandler());
    register(new UpdateFeaturedFightCommentRequestHandler());
    register(new CleanUpFightsRequestHandler());
    register(new GetWaitingTournamentsHandler());
    register(new AddAdvertHandler());
    register(new GetAdvertsHandler());
    register(new DeleteAdvertsHandler());
    register(new SubmitHouseBotRequestHandler());
    register(new GetPlayerRankingHandler());
    register(new GetAllRankingsHandler());
    register(new CreateActiveAccountRequestHandler());
    register(new GetAccountTotalBalanceHandler());
    register(new UpdateCurrentBalWithFPHandler());
    register(new WithdrawWinningsHandler());
    register(new GetAccountTransactionsHandler());
    register(new PatchDemoRequestHandler());



  }

//Initialize global variables
  public void init(ServletConfig config) throws ServletException
  {
    super.init(config);

    BDebug.print("Java Home: " + System.getProperty("java.home"));
    BDebug.print("Java Class Path: " + System.getProperty("java.class.path"));
    BDebug.print("User Dir: " + System.getProperty("user.dir"));

    BGlobal.init();

    Statement statement = null;
    ResultSet rs = null;

    Connection con1 = null;
    try
    {
      con1 = getDatabaseConnection();
      //  Get the Roboforge Account id
      String sql = "SELECT id FROM ACCOUNT WHERE loginID = 'RoboForge'";
      statement = con1.createStatement();
      rs = statement.executeQuery(sql);

      if (rs.next())
        roboforgeAccountID = rs.getLong("id");
      else
        System.err.println("Error: Could Not Find the RoboForge Account");

      rs.close();
      statement.close();
    }

    catch (Exception e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      try
      {
        if (con1!= null)
        {
          //Close the connection no matter what
          myBroker.freeConnection(con1);
        }

      }
      catch(Exception sqle1)
      {
        System.err.println("SGateway/Init: " + sqle1.getMessage());
      }
    }
  }


  // Echo handler (this class should be deleted)...
  // -------------------------------------------------------------

  protected static class EchoHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "ECHO"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      BMessage response =
        BMessage.ACK("ECHO");

      response.setData(request.getData());

      // Zip and encrypt the response...
      services.add(BServices.zip());
      //services.add(BServices.encrypt());

      return response;
    }
  }


  protected static void closeResultSet(ResultSet rs, String location)
  {
    try
    {
      if (rs != null)
        rs.close();
    }
    catch (Exception e)
    {
      System.err.println("SGateway/closeResultSet: "+ location + ": " + e);
    }

  }

  protected static void closeStatement(Statement statement, String location)
  {
    try
    {
      if (statement != null)
        statement.close();
    }
    catch (Exception e)
    {
      System.err.println("SGateway/closeStatement: "+ location + ": " + e);
    }

  }


  // Login request handler...
  // -------------------------------------------------------------

  protected static class LoginHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "LOG"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

       BLoginData loginData = new BLoginData(-1);
       loginData.accountID = reqState.accountID;

       Calendar cal = Calendar.getInstance(TimeZone.getDefault());
       cal.setTime(new java.util.Date(System.currentTimeMillis()));
    

       loginData.serverTime = cal;

       return BMessage.ACK("LOG", loginData);

    }
  }

  // Get Soft URL Request Handler...
  // -------------------------------------------------------------

  protected static class GetSoftURLHandler extends ValidatedRequestHandler
  {
    // Class used if we want to change the location of the client primary server
    // i.e. change the value of SGatewayURL on the client
    
    public String getType()
      { return "SURL"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      // Always validate the Logon as it's a soft URL request
      return null;
    }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

       BURLData urlData = new BURLData(0l);

       // Set to null for now as no intention to redirect
       // primary server

       urlData.URL = null;

       // Can also redirect the Challenge Rooms and
       // The farm feeder

       return BMessage.ACK("SURL", urlData);

      /*
      BRequestHeader header =
        request.getRequestHeader();

      // TODO: Lookup user in database...
      boolean isValidUser =
        header.getUser().equals("hc");

      if (isValidUser)
        return BMessage.ACK("LOG");
      else
        return BMessage.error("LOG", "Not a valid user."); */
    }
  }

  // Get the Server Time Request Handler...
  // -------------------------------------------------------------

  protected static class GetServerTimeHandler extends ValidatedRequestHandler
  {
    // Class used if we want to change the location of the client primary server
    // i.e. change the value of SGatewayURL on the client

    public String getType()
      { return "TIME"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      // Always validate the Logon as it's just a time request
      return null;
    }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

       BTimeData timeData = new BTimeData(System.currentTimeMillis());

       return BMessage.ACK("TIME", timeData);

    }
  }


  // Get the AutoPatch request handler...
  // -------------------------------------------------------------
  protected static class AutoPatchRequestHandler extends ValidatedRequestHandler
  {

    static Map patchContainer = Collections.synchronizedMap(new BSoftCache());
    //static HashMap patchContainer = new HashMap();

   //Map m = Collections.synchronizedMap(new HashMap(...));
    public String getType()
      { return "AUT"; }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      // Always validate the Logon as it's a patch request
      return null;
    }

    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {

      try
      {

        Float clientVersion = new Float(((BPatchRequestData)request.getData()).versionID);

        // Necessary condition as can't patch versions prior to 0.80
        if (clientVersion.floatValue() < 1.0f)
        {
          return BMessage.error("AUT", "RoboForge has been released. Please visit www.roboforge.net to purchase the release version.");
        }

        // Find the upgrade patch that matches the client
        // Create an inner class that implements the BCompare Interface for the sorting
        class FloatCompare implements BCompare
        {
          public boolean lessThan(Object l, Object r)
          {
            return ((Float)l).compareTo(((Float)r)) < 0;
          }
          public boolean lessThanOrEqual(Object l, Object r)
          {
            return ((Float)l).compareTo(((Float)r)) <= 0;
          }
        }

        BSortVector sortVector = new BSortVector(new FloatCompare());

        // Get a list of all the files in the release directory
        File file = new File(SGlobal.RELEASE_DIRECTORY);
        String[] cacheTitles = file.list();

        // Loop through and add all the file names to the vector if they are of type zip, removing the suffix
        for (int k = 0;k< cacheTitles.length;k++)
        {
          //BDebug.print("Vector Name: " + cacheTitles[k].substring(cacheTitles[k].length()-4));
          if(cacheTitles[k].substring(cacheTitles[k].length()-4).equals(".zip"))
            sortVector.addElement(new Float(cacheTitles[k].substring(0,cacheTitles[k].length()-4)));
        }

        // If the Vector is empty then there are no patches available
        if (sortVector.isEmpty())
        {
          // No need to patch as have latest version
          return BMessage.ACK("AUT",null);
        }

        sortVector.sort();

        float hightestPatchNumber = ((Float)sortVector.lastElement()).floatValue();

        if (clientVersion.floatValue() > hightestPatchNumber)
        {
          // No need to patch as have latest version
          return BMessage.ACK("AUT",null);
        }

        // Not the latest version so get the patch needed to take the
        // client to the latest version

        String fileToGet = "" + clientVersion.floatValue();

        // Check to see if this file exists in the HashMap

        BDebug.print(fileToGet);

        if (patchContainer.containsKey(fileToGet))
        {
          // Get and return the patch
          //BDebug.print("Found the patch in the Hash table");
          BPatchData patchData = (BPatchData)patchContainer.get(fileToGet);

          if (patchData == null)
          {
            // The Data has been cleaned out by the System, we need to replenish it
            System.err.println("Replenishing patch: " +  fileToGet);
          }
          else
          {
            //BDebug.print("Got the Patch");
            return BMessage.ACK("AUT",patchData);
          }
        }

        //BDebug.print("Didn't find the patch - Putting it in the Hash Table");

        // Check to see if the patch zip file exists
        File patchFile = new File(SGlobal.RELEASE_DIRECTORY + fileToGet + ".zip");

        if (!patchFile.exists())
        {
          // Can't find patch zip file
          return BMessage.error("AUT", "Can't Patch this Version Please contact admin@roboforge.com");
        }

        // Check to see if the patch pat file exists
        patchFile = new File(SGlobal.RELEASE_DIRECTORY + fileToGet + ".pat");

        if (!patchFile.exists())
        {
          // Can't find patach zip file
          return BMessage.error("AUT", "Can't Patch this Version Please contact admin@roboforge.com");
        }

        // Get the update file
        FileInputStream fin = new FileInputStream(SGlobal.RELEASE_DIRECTORY + fileToGet + ".zip");
        byte[] zippedFile = SInputStreamToByteArray.getBytes(fin);


        // Create and add the BPatchData object
        BPatchData patchData = BPatchData.loadFromFile(SGlobal.RELEASE_DIRECTORY + fileToGet + ".pat");

        // Set the members
        patchData.patch = zippedFile;

        fin.close();

        // Put the requested patch in the Hash Table
        patchContainer.put(fileToGet,patchData);

        return BMessage.ACK("AUT",patchData);

      }
      catch (Exception e)
      {
        System.err.println("SGateway/AutoPatchRequestHandler: " + e.getMessage());
        return BMessage.error("AUT", "SGateway/AutoPatchRequestHandler: " + e.getMessage());
      }
    }

  }


  // Patches the Demo to version 1.0 RoboForge
  protected static class PatchDemoRequestHandler extends ValidatedRequestHandler
  {

    static Map demoPatchContainer = Collections.synchronizedMap(new BSoftCache());
    //static HashMap patchContainer = new HashMap();

   //Map m = Collections.synchronizedMap(new HashMap(...));
    public String getType()
      { return "PDEM"; }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      // Always validate the Logon as it's a patch request
      return null;
    }

    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {

      String sql;
      Statement statement = null;
      ResultSet rs = null;
      Connection con = reqState.con;

      try
      {

        BDemoRegData demoRegData = (BDemoRegData)request.getData();
        long accountID = reqState.accountID;

        // Check to see if it's a valid RegKey and Email
        statement = con.createStatement();
        sql = "Select id, download_count FROM ACCOUNT WHERE LOWER(registration_key) = LOWER('" + demoRegData.regKey + "')  AND LOWER(email) = LOWER('" + demoRegData.emailAddress + "')";
        rs = statement.executeQuery(sql);
        if (!rs.next())
        {
          // No rows returned invalid registration key

          rs.close();
          statement.close();

          System.err.println("SGateway/PatchDemoRequestHandler: Invalid Registration Key: " + demoRegData.regKey + " Email: " + demoRegData.emailAddress );
          return BMessage.error("PDEM", "Invalid Registration Key");
        }

        int downloadCount = rs.getInt("download_count");

        rs.close();
        statement.close();


        // Check to see that they haven't exceeded their download quota
        if (downloadCount >= 10)
        {
          // Exceeded download count
          System.err.println("SGateway/PatchDemoRequestHandler: EXCEEDED REG QUOTA.  Email: " + demoRegData.emailAddress + " RegKey: " + demoRegData.regKey);
          return BMessage.error("PDEM", "Exceeded Registration Quota");
        }

        // Update the number of downloads
        sql = "UPDATE ACCOUNT SET download_count = " + (downloadCount + 1);
        sql += " WHERE id = " + accountID;

        statement = con.createStatement();
        statement.executeUpdate(sql);

        statement.close();

        // Check to see if this file exists in the HashMap
        if (demoPatchContainer.containsKey("demo"))
        {
          // Get and return the patch
          //BDebug.print("Found the patch in the Hash table");
          BDemoPatchData demoPatchData = (BDemoPatchData)demoPatchContainer.get("demo");

          if (demoPatchData == null)
          {
            // The Data has been cleaned out by the System, we need to replenish it
            System.err.println("SGateway/PatchDemoRequestHandler: Replenishing patch");
          }
          else
          {
            //BDebug.print("Got the Patch");
            return BMessage.ACK("PDEM",demoPatchData);
          }
        }

        //BDebug.print("Didn't find the patch - Putting it in the Hash Table");

        // Put the requested patch in the Hash Table
        BDemoPatchData demoPatchData = new BDemoPatchData(-1);
        demoPatchContainer.put("demo",demoPatchData);

        services.add(BServices.zip());

        return BMessage.ACK("PDEM",demoPatchData);

      }
      catch (Exception e)
      {
        System.err.println("SGateway/PatchDemoRequestHandler: " + e.getMessage());
        return BMessage.error("PDEM", "SGateway/PatchDemoRequestHandler: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/PatchDemoRequestHandler");
        closeStatement(statement,"SGateway/PatchDemoRequestHandler");
      }
    }

  }

  // Get the Tournament Image request handler...
  // -------------------------------------------------------------
  protected static class SetTournamentImageRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "STI"; }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {
      Connection con = reqState.con;

      Statement statement = null;
      PreparedStatement pstmt = null;
      ResultSet rs = null;
      String sql;

      try
      {
        con.setAutoCommit(false);

        BTournamentImageData imageData = (BTournamentImageData)request.getData();

        long tournamentID = imageData.getID();
        byte[] graphicBytes = imageData.getImage();

        pstmt = con.prepareStatement("UPDATE TOURNAMENT SET sponsor_pic = empty_blob() WHERE id = ?");
        pstmt.setLong(1,tournamentID);
        pstmt.execute();
        pstmt.close();

        sql = "SELECT sponsor_pic FROM TOURNAMENT WHERE id=" + tournamentID + " FOR UPDATE";

        statement = con.createStatement ();
        rs = statement.executeQuery(sql);
        rs.next();
        BLOB blob = ((OracleResultSet)rs).getBLOB(1);
        rs.close();
        statement.close();
        System.out.println("bannerBytes: " + graphicBytes.length);

        // Insert the graphic into the table
        OutputStream outstream = blob.getBinaryOutputStream();
        outstream.write(graphicBytes);
        outstream.close();

        con.commit();

        return BMessage.ACK("STI");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/SetTournamentImageRequestHandler: " + sqle1);
          return BMessage.error("STI", sqle1.getMessage());
        }

        System.err.println("SGateway/SetTournamentImageRequestHandler: " + e.getMessage());
        return BMessage.error("STI", "SGateway/SetTournamentImageRequestHandler: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/SetTournamentImageRequestHandler");
        closeStatement(pstmt,"SGateway/SetTournamentImageRequestHandler");
        closeStatement(statement,"SGateway/SetTournamentImageRequestHandler");
      }
    }

  }

  // Get the AccountInformation request handler...
  // -------------------------------------------------------------
  protected static class AccountInformationRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "GAI"; }

    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {
      Statement statement = null;
      ResultSet rs = null;

      try
      {
        Connection con = reqState.con;
        long id = reqState.accountID;

        statement = con.createStatement();
        String sql = "Select * FROM ACCOUNT WHERE id = " + id;
        rs = statement.executeQuery(sql);

        rs.next();

        BAccountData accountData = new BAccountData((long)id);
        accountData.name = rs.getString("name");
        accountData.address1 = rs.getString("address1");
        accountData.address2 = rs.getString("address2");
        accountData.city = rs.getString("city");
        accountData.country = rs.getString("country");

        accountData.postCode = rs.getString("post_code");
        accountData.email = rs.getString("email");
        accountData.blurb = rs.getString("blurb");
        accountData.loginID = rs.getString("loginid");
        accountData.password =rs.getString("password");
        accountData.designerName = rs.getString("designer_name");
        accountData.state = rs.getString("state");
        accountData.currentBalance = rs.getFloat("current_balance");
        accountData.winningsBalance = rs.getFloat("winnings_balance");

        BDataSet dataSet = new BDataSet();
        dataSet.add(accountData);

        rs.close();
        statement.close();

        services.add(BServices.zip());
        //services.add(BServices.encrypt());

        return BMessage.ACK("GAI", dataSet);
      }
      catch (Exception e)
      {
        System.err.println(e.getMessage());
        return BMessage.error("GAI",e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/AccountInformationRequestHandler");
        closeStatement(statement,"SGateway/AccountInformationRequestHandler");
      }
    }
  }

  // Update the client version...
  // -------------------------------------------------------------
  protected static class UpdateClientVersionRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "UPD"; }

    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {
      Statement statement = null;
      ResultSet rs = null;

      try
      {

        Connection con = reqState.con;
        long id = reqState.accountID;

        Float clientVersion = new Float(((BPatchRequestData)request.getData()).versionID);

        statement = con.createStatement();
        String sql = "UPDATE ACCOUNT SET current_version = " + clientVersion + "  WHERE id = " + id;
        statement.executeUpdate(sql);
        statement.close();

        //services.add(BServices.zip());
        //services.add(BServices.encrypt());

        return BMessage.ACK("UPD");
      }
      catch (Exception e)
      {
        System.err.println(e.getMessage());
        return BMessage.error("UPD",e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/UpdateClientVersionRequestHandler");
        closeStatement(statement,"SGateway/UpdateClientVersionRequestHandler");
      }
    }
  }

  // Get the SubmitBot request handler...
  // -------------------------------------------------------------
  protected static class SubmitBotRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "SUB"; }

    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {
      // TODO: Probably need to see if the amount they have agreed to transfer ie the amount in STransferDetail actually matches
      // the amount that is going to be transfered (ie the tournament entry fee) just incase they have hacked it and changed the
      // value for the fee in the STransferDetail.  Should send back an error value "FNM", Fee Not Match
      //


      Connection con = reqState.con;

      String sql;

      Statement statement = null;
      Statement statement2 = null;
      PreparedStatement pstmt = null;
      ResultSet rs = null;

      
      try
      {
        con.setAutoCommit(false);

        BTournamentSubmitData tournamentSubmitData = (BTournamentSubmitData)request.getData();
        //System.out.println("Bot Data Length: " + tournamentSubmitData.botData.length);

        long id = reqState.accountID;
        long tournamentID = tournamentSubmitData.getID();


        // Check that the tournament is still open
        sql = "SELECT Count(*) AS count FROM TOURNAMENT WHERE status = 'OPEN' AND id = " + tournamentID;
        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();

        if (rs.getInt("count") == 0)
        {
          // Tournament not open

          rs.close();
          statement.close();
          return BMessage.error("SUB", "Tournament closed");
        }

        rs.close();
        statement.close();

        // Check that the tournament isn't full
        sql =  "SELECT Count(*) AS count FROM TOURNAMENT_CLASSIFICATION";
        sql += " WHERE max_tournament_entries >";
        sql += " (SELECT Count(*) FROM TOURNAMENT_ENTRY WHERE tournament_id = " + tournamentID + ")";
        sql += " AND id = " + tournamentID;

        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();

        if (rs.getInt("count") == 0)
        {
          // Tournament not open
          rs.close();
          statement.close();

          return BMessage.error("SUB", "Tournament full");
        }

        rs.close();
        statement.close();


        // Check that the player hasn't aleady entered their Quota
        sql =  "SELECT Count(*) AS count FROM TOURNAMENT_CLASSIFICATION";
        sql += " WHERE max_entries_per_account >";
        sql += " (SELECT Count(*) FROM TOURNAMENT_ENTRY WHERE tournament_id = " + tournamentID;
        sql += " AND account_id = " + id + ")";
        sql += " AND id = " + tournamentID;

        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();

        if (rs.getInt("count") == 0)
        {
          // Entry limit exceeded

          rs.close();
          statement.close();
          return BMessage.error("SUB", "Entry limit exceeded");
        }

        rs.close();
        statement.close();


        /*
          If the tournament requires an entry fee, check to see if the client
          has enough in their account to cover it.
        */

        sql = "SELECT entry_fee FROM TOURNAMENT_CLASSIFICATION";
        sql += " WHERE id = " + tournamentID;

        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();

        BigDecimal entryFee = rs.getBigDecimal("entry_fee");

        rs.close();
        statement.close();

        if (entryFee.doubleValue() > 0)
        {
          // There is an entry fee for this tournament
          sql = "SELECT nvl(current_balance,0) current_balance, nvl(winnings_balance,0) winnings_balance";
          sql += " FROM ACCOUNT WHERE id = " + id;
          sql += " FOR UPDATE";

          statement = con.createStatement();
          rs = statement.executeQuery(sql);
          rs.next();

          BigDecimal currentBal = rs.getBigDecimal("current_balance");
          BigDecimal winningsBal = rs.getBigDecimal("winnings_balance");

          // Don't close the cursor just yet keep it open so that a lock remains
          // on this account for when we update the the current balances

          if (currentBal.add(winningsBal).compareTo(entryFee) == -1)
          {
            rs.close();
            statement.close();
            System.err.println("Not enough funds in accounts: " + id + "  to enter tournament: " + tournamentID);
            System.err.println("Total Funds: " + currentBal + winningsBal);
            con.rollback();
            return BMessage.error("SUB", "Insufficient Funds In Account to Enter Tournament");
          }


          if (currentBal.equals(entryFee) || currentBal.compareTo(entryFee) == 1)
          {
            // Transfer the entry fee out of the current balance

            // Create the transaction record
            sql = "INSERT INTO TRANSACTION (account_id, type, details, amount, status, tournament_id, account_from, bot_name)";
            sql += "VALUES (" + id + ",'TOURN','Entry fee',-" + entryFee + ",'C',"+tournamentID+",'C','" + tournamentSubmitData.botName + "')";
            statement2 = con.createStatement();
            statement2.executeUpdate(sql);
            statement2.close();

            // Deduct the amount from the account

            sql = "UPDATE ACCOUNT SET current_balance = nvl(current_balance,0) -" + entryFee + " WHERE id = " + id;
            statement2 = con.createStatement();
            statement2.executeUpdate(sql);
            statement2.close();
          }
          else
          {
            // Need to transfer from Winnings account to Current

            BigDecimal transAmt = entryFee.subtract(currentBal);

            // Create the transaction transfer record from the Winnings account
            sql = "INSERT INTO TRANSACTION (account_id, type, details, amount, status, tournament_id, account_from)";
            sql += "VALUES (" + id + ",'TFROUT','Trans For Entry Fee',-" + transAmt + ",'C',"+tournamentID+",'W')";
            statement2 = con.createStatement();
            statement2.executeUpdate(sql);
            statement2.close();

            // Create the transfer into the Current_Account
            sql = "INSERT INTO TRANSACTION (account_id, type, details, amount, status, tournament_id, account_from)";
            sql += "VALUES (" + id + ",'TFRIN','Trans For Entry Fee'," + transAmt + ",'C',"+tournamentID+",'C')";
            statement2 = con.createStatement();
            statement2.executeUpdate(sql);
            statement2.close();

            // Update the Current Account and Winnings Account balances
            sql = "UPDATE ACCOUNT SET winnings_balance = nvl(winnings_balance,0) -" + transAmt;
            sql += ", current_balance = nvl(current_balance,0) + " + transAmt + " WHERE id = " + id;
            statement2 = con.createStatement();
            statement2.executeUpdate(sql);
            statement2.close();

            // Create the entry transaction Record
            sql = "INSERT INTO TRANSACTION (account_id, type, details, amount, status, tournament_id, account_from, bot_name)";
            sql += "VALUES (" + id + ",'TOURN','Entry fee',-" + entryFee + ",'C',"+tournamentID+",'C','" + tournamentSubmitData.botName + "')";
            statement2 = con.createStatement();
            statement2.executeUpdate(sql);
            statement2.close();

            // Update the Current Account
            sql = "UPDATE ACCOUNT SET current_balance = nvl(current_balance,0) -" + entryFee + " WHERE id = " + id;
            statement2 = con.createStatement();
            statement2.executeUpdate(sql);
            statement2.close();
          }

          rs.close();
          statement.close();
          // The lock is now removed on the Account table
        }

        // Get the next tournament entry ID
        sql = "SELECT TOURNAMENT_ENTRY_IDS.nextval As ID FROM DUAL";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();
        long tournament_entry_id = rs.getLong("id");
        rs.close();
        statement.close();

        // Create the record in the Tournament_Entry table for the bot to be streamed into
        pstmt = con.prepareStatement("INSERT INTO TOURNAMENT_ENTRY (id, tournament_id, bot_name, bot_file, account_id, current_round) values (?, ?, ?, empty_blob(), ?,0)");
        pstmt.setLong(1,tournament_entry_id);
        pstmt.setLong(2,tournamentID);
        pstmt.setString(3,SServerAccess.parseString(tournamentSubmitData.botName));
        pstmt.setLong(4,id);

        pstmt.execute();
        pstmt.close();

        sql = "SELECT bot_file FROM TOURNAMENT_ENTRY WHERE id=" + tournament_entry_id + " FOR UPDATE";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();

        BLOB blob = ((OracleResultSet)rs).getBLOB(1);

        byte[] botBytes = tournamentSubmitData.botData;

        // Insert the bot into the table
        OutputStream outstream = blob.getBinaryOutputStream();

        outstream.write(botBytes);
        outstream.close();

        rs.close();
        statement.close();


        BTournamentEntryData tEntryData = new BTournamentEntryData(tournament_entry_id);
        tEntryData.accountID = new Long(id);
        tEntryData.botName = tournamentSubmitData.botName;
        tEntryData.tournamentID = new Long(tournamentID);
        tEntryData.statusCode = tEntryData.WAITING;

        con.commit();
        con.setAutoCommit(true);

        return BMessage.ACK("SUB",tEntryData);

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/SubmitBotRequestHandler: " + sqle1);
          return BMessage.error("SUB", sqle1.getMessage());
        }

        System.err.println("SGateway/SubmitBotRequestHandler: " + e);
        return BMessage.error("SUB", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/SubmitBotRequestHandler");
        closeStatement(statement,"SGateway/SubmitBotRequestHandler");
        closeStatement(statement2,"SGateway/SubmitBotRequestHandler");
        closeStatement(pstmt,"SGateway/SubmitBotRequestHandler");
      }
    }
    
  }

// Get the SubmitBot request handler...
  // -------------------------------------------------------------
  protected static class SubmitHouseBotRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "SHSE"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {
      Connection con = reqState.con;

      String sql;

      Statement statement = null;
      ResultSet rs = null;

      float entryFee = 0f;
      try
      {
        BTournamentSubmitData tournamentSubmitData = (BTournamentSubmitData)request.getData();

        con.setAutoCommit(false);

        // Create the record in the Constants table for the bot to be streamed into
        sql = "UPDATE CONSTANTS SET house_bot = empty_blob()";
        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();


        sql = "SELECT house_bot FROM CONSTANTS FOR UPDATE";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();

        BLOB blob = ((OracleResultSet)rs).getBLOB(1);

        byte[] botBytes = tournamentSubmitData.botData;

        // Insert the bot into the table
        OutputStream outstream = blob.getBinaryOutputStream();

        outstream.write(botBytes);
        outstream.flush();
        outstream.close();

        rs.close();
        statement.close();

        con.setAutoCommit(true);
        con.commit();

        return BMessage.ACK("SHSE");
      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/SubmitHouseBotRequestHandler: " + sqle1);
          return BMessage.error("SHSE", sqle1.getMessage());
        }

        System.err.println("SGateway/SubmitHouseBotRequestHandler: " + e);
        return BMessage.error("SHSE", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/SubmitHouseBotRequestHandler");
        closeStatement(statement,"SGateway/SubmitHouseBotRequestHandler");
      }
    }
  }

  // Get the CreateAccount request handler...
  // -------------------------------------------------------------
  protected static class CreateAccountRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "CAC"; }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      // Always validate the Logon as it's a create account request
      return null;
    }

    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {
      Connection con = reqState.con;

      String sql = "";

      Statement statement = null;
      ResultSet rs = null;

      try
      {
        // Turn off autocommit so we can rollback in the event of any errors
        con.setAutoCommit(false);

        BAccountData accountData = ((BAccountData)request.getData());

        // Process the accound data to ensure that it's suitable for inserting
        String processErrors = ProcessAccountInformation.processForServer(accountData);

        if (processErrors != null)
          return BMessage.error("CAC", processErrors);


        // Determine if the RegKey hasn't already been used
        sql = "SELECT Count(*) As count FROM ACCOUNT WHERE LOWER(registration_key) = LOWER('" + accountData.registrationCode + "') AND status <> 'P'";
        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();

        int count = rs.getInt("count");
        rs.close();
        statement.close();

        if (count > 0)
        {
          // This RegKey is already in use by an active account
          System.err.println("SGateway/CreateAccountRequestHandler: Key Already Used: " + accountData.registrationCode + " Email: " + accountData.email );
          return BMessage.error("CAC", "Key Already Used To Activate Account - Use The Login Screen");
        }

        // Determine if the Reg Key is valid
        statement = con.createStatement();
        sql = "Select id, promotion_id FROM ACCOUNT WHERE LOWER(registration_key) = LOWER('" + accountData.registrationCode + "')  AND LOWER(email) = LOWER('" + accountData.email + "') AND status = 'P'";
        rs = statement.executeQuery(sql);
        if (!rs.next())
        {
          // No rows returned invalid registration key

          rs.close();
          statement.close();

          System.err.println("SGateway/CreateAccountRequestHandler: Invalid Registration Key: " + accountData.registrationCode + " Email: " + accountData.email );
          return BMessage.error("CAC", "Invalid Registration Key");
        }
        else
        {

          long accountID = rs.getLong("id");
          String promotionID = rs.getString("promotion_id");
          rs.close();
          statement.close();

          // Check to see if the login name is Unique
          statement = con.createStatement();
          sql = "Select id FROM ACCOUNT WHERE LOWER(loginid) = LOWER('" +  accountData.loginID.trim() + "')";
          rs = statement.executeQuery(sql);
          if (rs.next())
          {
            // User name already exists

            rs.close();
            statement.close();

            System.err.println("SGateway/CreateAccountRequestHandler: Login already exists" );
            return BMessage.error("CAC", "Login already exists");
          }


          rs.close();
          statement.close();

          // Make sure that the login id doesn't start with Client
          if (accountData.loginID.trim().length() >= 6)
          {
            if (accountData.loginID.trim().substring(0,6).equalsIgnoreCase("Client"))
            {
              System.err.println("SGateway/CreateAccountRequestHandler: Login already exists" );
              return BMessage.error("CAC", "Login already exists");
            }
          }

          // Check to see if the designer name is Unique
          statement = con.createStatement();
          sql = "Select id FROM ACCOUNT WHERE LOWER(designer_name) = LOWER('" +  accountData.designerName.trim() + "')";
          rs = statement.executeQuery(sql);
          if (rs.next())
          {
            // Designer Name name already exists

            rs.close();
            statement.close();

            System.err.println("SGateway/CreateAccountRequestHandler: Designer Name already exists" );
            return BMessage.error("CAC", "Designer Name already exists");
          }
          else
          {

            rs.close();
            statement.close();

            // Finally Create the Account and the Account Transaction Information

            /*
            // First Debit the account creation fee to the account
            // TODO: The account setup fee should be held in the Database
            sql = "INSERT INTO TRANSACTION (account_id, type, details, amount, status) VALUES (" + accountID + ",'New','New Account Fee',-20,'Ok')";
            statement = con.createStatement();
            statement.executeUpdate(sql);
            statement.close();

            // Next Credit the account with the promotion
            // TODO: Need to insert code that works out the amount credited as per the promotion type
            // As currently not using the promotion ID link
            // A check should be done so that the amount credited can never exceed the account creation fee
            if(promotionID != null)
            {
              // A promotion exists for this account
              sql = "INSERT INTO TRANSACTION (account_id, type, details, amount, status) VALUES (" + accountID + ",'Promo','New Account Discount',10,'Ok')";
              statement = con.createStatement();
              statement.executeUpdate(sql);
              statement.close();
            }
            */

            // Finally Activate the account
            //sql = "UPDATE ACCOUNT SET status = 'OPEN', loginID = '" +  accountData.loginID + "', designer_name = '" + accountData.designerName + "', password = '" +  accountData.password + "', name = '" +  accountData.name + "', address1 = '" +  accountData.address1 + "', address2 = '";
            //sql = sql +  accountData.address2 + "', city = '" +  accountData.city + "', country = '" +  accountData.country + "', post_code = '" +  accountData.postCode + "', email = '" + accountData.email + "', blurb = '" +  accountData.blurb + "' WHERE id = " + accountID ;

             // Finally Activate the account
            sql = "UPDATE ACCOUNT SET status = 'OPEN', loginID = '" +  accountData.loginID + "', designer_name = '" + accountData.designerName + "', password = '" +  accountData.password + "', name = '" +  accountData.name + "'";
            sql = sql +  ", email = LOWER('" + accountData.email + "') WHERE id = " + accountID ;

            statement = con.createStatement();
            statement.executeUpdate(sql);
            statement.close();

            // Send back the accountID
            BIdentData identData = new BIdentData(accountID);

            con.commit();

            return BMessage.ACK("CAC",identData);
          }
        }
      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle)
        {
          System.err.println("SGateway/CreateAccountRequestHandler: " + sqle );
          return BMessage.error("CAC", sqle.getMessage());
        }

          System.err.println("SGateway/CreateAccountRequestHandler: SQL: " + sql + " Error: " + e );
          return BMessage.error("CAC", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/BattleRecordingRequestHandler");
        closeStatement(statement,"SGateway/BattleRecordingRequestHandler");
      }
    }
  }


  // For marketing use, creates an OPEN (Active account) complete with password
  // and login...
  // -------------------------------------------------------------
  protected static class CreateActiveAccountRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "CACC"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {
      Connection con = reqState.con;

      String sql = "";

      Statement statement = null;
      ResultSet rs = null;

      try
      {
        // Turn off autocommit so we can rollback in the event of any errors
        con.setAutoCommit(false);

        BCreateActiveAccountData accD = ((BCreateActiveAccountData)request.getData());

        // Make sure that the login id doesn't start with Client

        if (accD.loginID.trim().length() >= 6)
        {
          if (accD.loginID.trim().substring(0,6).equalsIgnoreCase("Client"))
          {
            System.err.println("SGateway/CreateActiveAccountRequestHandler: Login already exists" );
            return BMessage.error("CACC", "Login already exists");
          }
        }

        // Check to see if the email doesn't already exist
        sql = "Select id FROM ACCOUNT WHERE LOWER(email) = LOWER('" +  accD.email.trim() + "')";
        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        if (rs.next())
        {
          // User name already exists
          long userID = rs.getLong("id");

          rs.close();
          statement.close();

          System.err.println("SGateway/CreateActiveAccountRequestHandler: Email already exists" );
          return BMessage.error("CACC", "Email already exists.  Belongs to Account: "+ userID);
        }

        rs.close();
        statement.close();


        // Check to see if the login name is Unique

        sql = "Select id FROM ACCOUNT WHERE LOWER(loginid) = LOWER('" +  accD.loginID.trim() + "')";
        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        if (rs.next())
        {
          // User name already exists

          rs.close();
          statement.close();

          System.err.println("SGateway/CreateActiveAccountRequestHandler: Login already exists" );
          return BMessage.error("CACC", "Login already exists");
        }


        rs.close();
        statement.close();

        // Finally Create the Account the account

        String name = accD.fName + " " + accD.lName;

        sql = "INSERT INTO ACCOUNT (email, first_name, last_name, name)";
        sql += " VALUES (LOWER('" + SServerAccess.parseString(accD.email) + "'),'" + SServerAccess.parseString(accD.fName) + "','" + SServerAccess.parseString(accD.lName) + "','" + SServerAccess.parseString(name) + "')";
        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        // Now update the account with loginId and password as these are automatically generated by a trigger

        sql = "UPDATE ACCOUNT SET status = 'OPEN', registration_key = getRegKey, loginID = '" +  SServerAccess.parseString(accD.loginID) + "', designer_name = '" + SServerAccess.parseString(name) + "', password = '" +  SServerAccess.parseString(accD.password) + "'";
        sql = sql +  " WHERE LOWER(email) = LOWER('" + SServerAccess.parseString(accD.email) + "')";

        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();


        con.commit();

        return BMessage.ACK("CACC");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle)
        {
          System.err.println("SGateway/CreateActiveAccountRequestHandler: " + sqle );
          return BMessage.error("CACC", sqle.getMessage());
        }

          System.err.println("SGateway/CreateActiveAccountRequestHandler: SQL: " + sql + " Error: " + e );
          return BMessage.error("CACC", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/CreateActiveAccountRequestHandler");
        closeStatement(statement,"SGateway/CreateActiveAccountRequestHandler");
      }
    }
  }



  // Get the Change Account Details request handler...
  // -------------------------------------------------------------
  protected static class ProcessAccountInformation
  {

    public static String processForServer (BAccountData accountData)
    {
      try
      {
        // TODO: Correctly implement the this processing of information
        // checking that all required members are valid, eg not null etc

        if (accountData.address1 == null)
          accountData.address1 = "";
        if (accountData.address2 == null)
          accountData.address2 = "";
        if (accountData.city == null)
          accountData.city = "";
        if (accountData.country == null)
          accountData.country = "";
        if (accountData.blurb == null)
          accountData.blurb = "";
        if (accountData.postCode == null)
          accountData.postCode = "";
        if (accountData.email == null)
          accountData.email = "";
        if (accountData.name == null)
          accountData.name = "";
        if (accountData.designerName == null)
          accountData.designerName = "";
        if (accountData.state == null)
          accountData.state = "";
        if (accountData.password == null)
          accountData.password = "";

        // Trim all the names
        accountData.address1      = accountData.address1.trim();
        accountData.address2      = accountData.address2.trim();
        accountData.blurb         = accountData.blurb.trim();
        accountData.city          = accountData.city.trim();
        accountData.country       = accountData.country.trim();
        accountData.designerName  = accountData.designerName.trim();
        accountData.email         = accountData.email.trim();
        //accountData.loginID       = accountData.loginID.trim();
        accountData.name          = accountData.name.trim();
        //accountData.password      = accountData.password.trim();
        accountData.state          = accountData.state.trim();

        // Ensure that all the names are the right length

        if (accountData.address1.length()>255)
          return "Address 1 Exceeds 255 characters";

        if (accountData.address2.length()>50)
          return "Address 2 Exceeds 50 characters";

        if (accountData.blurb.length()>2000)
          return "Description Exceeds 2000 characters";

        if (accountData.city.length()>50)
          return "City Exceeds 50 characters";

        if (accountData.country.length()>50)
          return "Country Exceeds 50 characters";

        if (accountData.designerName.length()>50)
          return "Designer Name Exceeds 50 characters";

        if(accountData.designerName.trim().length()== 0)
          return "Must have a Designer Name";

        if (accountData.email.length()>100)
          return "Email Exceeds 100 characters";

        if (accountData.loginID != null)
        {
          if (accountData.loginID.length()>15)
            return "Login Exceeds 15 characters";
        }

        if (accountData.name.length()>50)
          return "Name Exceeds 50 characters";

        if (accountData.password.length()>15)
          return "Password Exceeds 15 characters";

        if (accountData.state.length()>30)
          return "State Exceeds 30 Characters";

        accountData.state          = accountData.state.trim();


        // Make sure that the Designer Name doesn't start with RF

        if (accountData.designerName.length()>= 2)
        {
          if (accountData.designerName.trim().substring(0,2).equalsIgnoreCase("RF"))
          {
            return "Designer Name already exists";
          }
        }

        // Make sure that the Designer Name doesn't start with Gandalf
        if (accountData.designerName.length()>=7)
        {
          if (accountData.designerName.trim().substring(0,7).equalsIgnoreCase("Gandalf"))
          {
            return "Designer Name already taken by JRR Tolkien";
          }
        }

        // Make sure the designer name isn't offensive
        String filteredName = roboforge.util.text.BDictionaryTree.getSharedInstance().filter(accountData.designerName.trim());
        if (!filteredName.equals(accountData.designerName.trim()))
        {
          return "Designer Name possibly offensive. Please choose another";
        }

        // Make sure that the designer doesn't start with Client
        if (accountData.designerName.trim().length() >= 6)
        {
          if (accountData.designerName.trim().substring(0,6).equalsIgnoreCase("Client"))
          {
            return "Designer Name already exists";
          }
        }

        // Fixup any quotations from the strings to make them Oracle safe

        accountData.address1      = SServerAccess.parseString(accountData.address1);
        accountData.address2      = SServerAccess.parseString(accountData.address2);
        accountData.blurb         = SServerAccess.parseString(accountData.blurb);
        accountData.city          = SServerAccess.parseString(accountData.city);
        accountData.country       = SServerAccess.parseString(accountData.country);
        accountData.designerName  = SServerAccess.parseString(accountData.designerName);
        accountData.email         = SServerAccess.parseString(accountData.email);
        // Make the email lower case
        accountData.email         = accountData.email.toLowerCase();
        if (accountData.loginID != null)
          accountData.loginID       = SServerAccess.parseString(accountData.loginID);
        accountData.name          = SServerAccess.parseString(accountData.name);
        accountData.password      = SServerAccess.parseString(accountData.password);
        accountData.state         = SServerAccess.parseString(accountData.state);

        return null;
      }
      catch (Exception e)
      {
        return "ProcessAccountInformation: " + e.getMessage();
      }

    }
  }

  // Get the Change Account Details request handler...
  // -------------------------------------------------------------
  protected static class ChangeAccountRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "CAI"; }

    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      ResultSet rs = null;
      Connection con = reqState.con;

      try
      {
        BAccountData accountData = (BAccountData)request.getData();

        // Process the accound data to ensure that it's suitable for inserting
        String processErrors = ProcessAccountInformation.processForServer(accountData);

        if (processErrors != null)
        {
          System.err.println("SGateway/ChangeAccountRequestHandler:Process Errors " + processErrors );
          return BMessage.error("CAI", processErrors);
        }

        long id = reqState.accountID;
        long tempID;

        // Check to see if the email address is Unique
        statement = con.createStatement();
        sql = "Select id FROM ACCOUNT WHERE LOWER(email) = LOWER('" +  accountData.email.trim() + "')";
        rs = statement.executeQuery(sql);
        if (rs.next())
        {
          // Email Address name already exists

          tempID = rs.getLong("id");

          rs.close();
          statement.close();

          if (id != tempID)
          {
            System.err.println("SGateway/ChangeAccountRequestHandler: Email Address already exists" );
            return BMessage.error("CAI", "Email Address already exists");
          }
        }

        rs.close();
        statement.close();
        // Check to see if the designer name is Unique
        statement = con.createStatement();
        sql = "Select id FROM ACCOUNT WHERE LOWER(designer_name) = LOWER('" +  accountData.designerName.trim() + "')";
        rs = statement.executeQuery(sql);
        if (rs.next())
        {
          // Designer Name name already exists

          tempID = rs.getLong("id");

          rs.close();
          statement.close();

          if (id != tempID)
          {
            System.err.println("SGateway/ChangeAccountRequestHandler: Designer Name already exists" );
            return BMessage.error("CAI", "Designer Name already exists");
          }
        }

        rs.close();
        statement.close();

        System.out.println("designer_name = " + accountData.designerName + " state = " + accountData.state);

        if(accountData.password == null || accountData.password.equals(""))
        {
          // Don't update the password
          sql = "UPDATE ACCOUNT SET name = '" + accountData.name + "', address1 = '" + accountData.address1 + "', address2 = '" + accountData.address2 + "', city = '" + accountData.city + "', country = '" + accountData.country + "', blurb = '" + accountData.blurb + "', post_code = '" + accountData.postCode + "', email = '" +accountData.email + "', designer_name = '" +accountData.designerName + "', state = '" +accountData.state + "' WHERE id = " + id;
        }
        else
        {
          sql = "UPDATE ACCOUNT SET name = '" + accountData.name + "', password = '" + accountData.password + "', address1 = '" + accountData.address1 + "', address2 = '" + accountData.address2 + "', city = '" + accountData.city + "', country = '" + accountData.country + "', blurb = '" + accountData.blurb + "', post_code = '" +accountData.postCode + "', email = '" + accountData.email + "', designer_name = '" + accountData.designerName + "', state = '" +accountData.state +"' WHERE id = " + id;
        }

        // Turn off autocommit so we can rollback in the event of any errors
        con.setAutoCommit(false);

        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        return BMessage.ACK("CAI");
        
      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle)
        {
          System.err.println("SGateway/ChangeAccountRequestHandler: " + sqle );
          return BMessage.error("CAI", sqle.getMessage());
        }

        System.err.println("SGateway/ChangeAccountRequestHandler: " + e );
        return BMessage.error("CAI", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/ChangeAccountRequestHandler");
        closeStatement(statement,"SGateway/ChangeAccountRequestHandler");
      }

    }
  }


  // Get the Message request handler...
  // -------------------------------------------------------------
  protected static class MessageRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "MOTD"; }

    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {

      try
      {

        BMessageData messageData = new BMessageData(0l);
        messageData.message = "What hath God wrought?";

        return BMessage.ACK("MOTD",messageData);
      }
      catch (Exception e)
      {
        System.err.println("SGateway/MessageRequestHandler: " + e );
        return BMessage.error("MOTD", e.getMessage());
      }

    }
  }

  // Get the TournamentHistory request handler...
  // -------------------------------------------------------------
  protected static class TournamentHistoryRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "HIS"; }

    public BMessage respond (BRequest request, RequestState reqState, BServiceSet services)
    {
      Vector tournamentDataVector = new Vector();
      Vector tournamentEntryDataVector = new Vector();
      Vector fightDataVector = new Vector();

      String sql;
      Statement getTournamentInfo = null;
      ResultSet rs = null;

      Statement statement = null;
      ResultSet rs1 = null;


      try
      {
        //System.err.println("Start Get History: " + System.currentTimeMillis());
        Connection con = reqState.con;
        long accountID = reqState.accountID;

        // Get all the Finished, Fighting or Pending Tournaments and Add them to a Vector
        sql = "SELECT * FROM ";
        sql += " (SELECT * FROM TOURNAMENT_CLASSIFICATION";
        sql += " WHERE (status = 'FINISHED' OR status = 'FIGHTING' OR status = 'PENDING')";
        //sql += " AND type = 'INTFC' ORDER BY id DESC)";
        sql += " AND type = 'INTFC' ORDER BY CLOSE_DATE DESC)";
        sql += " WHERE ROWNUM <= 7";

        getTournamentInfo = con.createStatement();
        rs = getTournamentInfo.executeQuery(sql);

        while (rs.next())
        {
          BTournamentData tData = new BTournamentData(rs.getLong("id"));

          // Generate summary info for the tournament
          tData.name = rs.getString("name");


          //java.util.Date closeDate =

          Calendar cal = Calendar.getInstance(TimeZone.getDefault());
          cal.setTime(rs.getTimestamp("close_date"));

          tData.closingDate = cal;
          //tData.closingDate = closeDate.getTime();
          //tData.closingDate = rs.getDate("close_date");
          tData.maximumEntries = rs.getInt("max_tournament_entries");
          tData.fightDuration = rs.getInt("fight_duration");
          tData.entryFee = rs.getFloat("entry_fee");
          tData.description = rs.getString("description");
          tData.arena = rs.getString("arena_file_name");
          tData.botPriceCap = rs.getFloat("bot_price_cap");
          tData.commentary = rs.getString("commentary");
          tData.summary = rs.getString("summary");
          tData.currentRound = rs.getInt("current_round");
          tData.tournamentType = rs.getString("tournament_type_id");

          if (rs.getString("status").equals("FINISHED"))
            tData.statusCode = BTournamentData.COMPLETE;
          else if (rs.getString("status").equals("FIGHTING"))
            tData.statusCode = BTournamentData.FIGHTING;
          else
            tData.statusCode = BTournamentData.PENDING;

            
          tData.sponsorImage = SServerUtility.blobToByteArray (((OracleResultSet)rs).getBLOB("sponsor_pic"));
          tournamentDataVector.addElement(tData);
        }

        getTournamentInfo.close();
        rs.close();

        // Get all the Related Tournament Entries and add them to a Vector
        sql = "SELECT te.id AS TEID, account_id, bot_name, te.status,tournament_id, validation_notes FROM TOURNAMENT t, TOURNAMENT_ENTRY te";
        sql += " WHERE t.id = te.tournament_id AND te.account_id = " + accountID + " AND t.id IN (";
        sql += " SELECT id FROM";

        // Put in 'INTFC' clause
        //sql += " (SELECT * FROM TOURNAMENT WHERE (status = 'FINISHED' OR status = 'FIGHTING') AND type = 'INTFC' ORDER BY id DESC)";
        sql += " (SELECT * FROM TOURNAMENT WHERE (status = 'FINISHED' OR status = 'FIGHTING') AND type = 'INTFC' ORDER BY CLOSE_DATE DESC)";
        sql += " WHERE ROWNUM <= 7)";

        // Following line doesn't restrict Tournament Entries to the account
        //sql += " WHERE t.id = te.tournament_id AND t.status = 'FINISHED'";

        getTournamentInfo = con.createStatement();
        rs =  getTournamentInfo.executeQuery(sql);

        while (rs.next())
        {
          BTournamentEntryData tEData = new BTournamentEntryData(rs.getLong("TEID"));
          tEData.accountID = new Long(rs.getLong("account_id"));
          tEData.botName = rs.getString("bot_name");
          tEData.tournamentID = new Long(rs.getLong("tournament_id"));
          tEData.validationNotes = rs.getString("validation_notes");

          String status  = rs.getString("status");

          // Assign the correct status
          if (status.equals("Y"))
            tEData.statusCode = BTournamentEntryData.VALID;
          else if (status.equals("N"))
            tEData.statusCode = BTournamentEntryData.INVALID;
          else if (status.equals("P") || status.equals("BAD"))
            tEData.statusCode = BTournamentEntryData.VALIDATING;
          else
            tEData.statusCode = BTournamentEntryData.WAITING;

          tournamentEntryDataVector.addElement(tEData);
        }

        getTournamentInfo.close();
        rs.close();

        // Get all the Related Fights and add them to a Vector
        //sql = "SELECT f.id AS FID, tournament_id, tournament_entry1_id, tournament_entry2_id, round, result, f.status, duration, t.current_round";
        //sql += " FROM FIGHT f, TOURNAMENT t";
        //sql += "  WHERE t.id = f.tournament_id AND t.status = 'FINISHED' AND f.STATUS = 'FOUGHT'";

        // Get only the final or semi-final fights and featured_fights and
        // and fights where the requesting account had bots entered
        sql = "SELECT";
        sql += 	  " f.id AS FID, ";
        sql += 		" tournament_id,";
        sql += 		" tournament_entry1_id,";
        sql += 		" tournament_entry2_id,";
        sql += 		" round,";
        sql += 		" result,";
        sql += 		" f.status,";
        sql += 		" duration,";
        sql += 		" t.current_round,";
        sql += 		" f.won_by,";
        sql += 		" f.narrative,";
        sql += 		" t.status AS tStatus";
        sql += 	" FROM";
        sql += 		" FIGHT f,";
        sql += 		" TOURNAMENT t";
        sql += 	" WHERE";
        sql += 		" t.id = f.tournament_id";
        sql +=    " AND t.type = 'INTFC'";
        // DELETED status added to allow for All Time Greats Featured fights that are in
        // Deleted tournaments
        sql += 		" AND (t.status = 'FINISHED' OR t.status = 'FIGHTING' OR t.status = 'DELETED')";

        // Only shows the fights for rounds which have completed
        sql += 		" AND round < current_round";
        sql += 		" AND f.STATUS = 'FOUGHT'";
        // Only shows the fights that are in the last 7 tournaments or are in the All time Greats
        // Or in the Finals or SemiFinals
        sql +=    " AND (t.id IN (";
        sql +=                    " SELECT id FROM";
        sql +=                                  " (SELECT *";
        sql +=                                  " FROM TOURNAMENT";
        //sql +=                                  " WHERE (status = 'FINISHED' OR status = 'FIGHTING') AND type = 'INTFC' ORDER BY id DESC)";
        sql +=                                  " WHERE (status = 'FINISHED' OR status = 'FIGHTING') AND type = 'INTFC' ORDER BY CLOSE_DATE DESC)";
        sql +=                     " WHERE ROWNUM <= 7) OR";
        sql += 		                 " (SELECT COUNT(*) FROM FEATURED_FIGHT WHERE fight_id = f.id AND featured_fight_type_id = " + BFeaturedFightTypeData.ALL_TIME_GREAT + ") > 0";
        sql += 		                 ")";
        sql += 		" AND ((round >= (current_round-2) AND t.status = 'FINISHED') OR";
        // Following line selects any assigned (type <> 1) featured fights
        sql +=    " (SELECT COUNT(*) FROM FEATURED_FIGHT WHERE fight_id = f.id AND featured_fight_type_id <> " + BFeaturedFightTypeData.UNASSIGNED + ") > 0 OR";
        sql += 		" EXISTS(";
        sql += 	   					" SELECT";
        sql += 								" f1.id";
        sql += 	 					  " FROM";
        sql += 								" FIGHT f1,";
        sql += 								" TOURNAMENT_ENTRY te,";
        sql += 								" TOURNAMENT_ENTRY te2";
        sql += 	  					" WHERE";
        sql += 						    " f1.tournament_entry1_id = te.id";
        sql += 	 					    " AND f1.tournament_entry2_id   = te2.id";
        sql += 	   						" AND (te.account_id = " + accountID + " OR te2.account_id = " + accountID +")";
        sql += 	  						" AND f1.id = f.id";
        sql += 							"))";
        sql += 	" ORDER BY f.id";

        getTournamentInfo = con.createStatement();
        rs =  getTournamentInfo.executeQuery(sql);

        while (rs.next())
        {

          int highestRound = rs.getInt("current_round");

          long te1ID = rs.getLong("tournament_entry1_id");
          long te2ID = rs.getLong("tournament_entry2_id");

          BFightData fData = new BFightData(rs.getLong("FID"));
          fData.tournamentID = new Long(rs.getLong("tournament_id"));
          fData.entry01ID = new Long(te1ID);
          fData.entry02ID = new Long(te2ID);
          fData.round = rs.getInt("round");
          fData.winnerID = new Long(rs.getLong("result"));
          fData.status = BFightData.COMPLETE;
          fData.duration = rs.getFloat("duration");
          fData.winDescription = rs.getString("won_by");
          fData.comments = rs.getString("narrative");

          //String featured_fight = rs.getString("featured_fight");


          // If the tournament isn't finished then don't assign
          // Final and semi final tags

          if (rs.getString("tStatus").equals("FINISHED"))
          {
            // If the fight is a final or semi final then show it
            if (fData.round == (highestRound-1))
              fData.roundDescription = "Final";
            else if (fData.round == (highestRound-2))
              fData.roundDescription = "Semi Final";
          }
          /*else if (featured_fight != null)
          {
            if (featured_fight.equals("Y"))
              fData.roundDescription = "Featured Fight";
          }*/
          // Get the name of the two bots involved in the tournament along with the designer name
          String fightName;

          // Get the first name
          sql = "SELECT te.bot_name, a.id, a.designer_name FROM TOURNAMENT_ENTRY te, ACCOUNT a WHERE te.id = " + te1ID;
          sql += " AND te.account_id = a.id";
          statement = con.createStatement();
          rs1 = statement.executeQuery(sql);

          rs1.next();
          String botName = rs1.getString("bot_name");

          fData.designerAccountID1 = rs1.getLong("id");
          fData.designerName1 = rs1.getString("designer_name");
          fData.botName1 = botName;


          if (te1ID == fData.winnerID.longValue())
          {
            // The the winning bot name
            fData.winnerName = botName;
          }

          fightName = botName + " vs. ";

          rs1.close();
          statement.close();

          // Get the second name
          sql = "SELECT te.bot_name, a.id, a.designer_name FROM TOURNAMENT_ENTRY te, ACCOUNT a WHERE te.id = " + te2ID;
          sql += " AND te.account_id = a.id";
          statement = con.createStatement();
          rs1 = statement.executeQuery(sql);

          rs1.next();

          botName = rs1.getString("bot_name");

          fData.designerAccountID2 = rs1.getLong("id");
          fData.designerName2 = rs1.getString("designer_name");
          fData.botName2 = botName;

          if (te2ID == fData.winnerID.longValue())
          {
            // The the winning bot name
            fData.winnerName = botName;
          }
          fightName += botName;

          rs1.close();
          statement.close();

          // Assign the name
          fData.name = fightName;


          // For each fight get the list of Featured Fight Types
          sql = "SELECT featured_fight_type_id";
          sql += " FROM FEATURED_FIGHT";
          sql += " WHERE featured_fight_type_id <> " + BFeaturedFightTypeData.UNKNOWN_TYPE + " AND fight_id = " + fData.getID();

          statement = con.createStatement();
          rs1 = statement.executeQuery(sql);

          Vector featuredFightVector =  new Vector();

          while (rs1.next())
          {

            long temp = rs1.getLong("featured_fight_type_id");

            if (temp == 0)
              temp = BFeaturedFightTypeData.UNASSIGNED;
            else if (temp == 1)
              temp = BFeaturedFightTypeData.FEATURED;
            else if (temp == 2)
              temp = BFeaturedFightTypeData.ALL_TIME_GREAT;
            else
              temp = BFeaturedFightTypeData.UNKNOWN_TYPE;

            featuredFightVector.addElement(new Long(temp));

          }

          rs1.close();
          statement.close();

          // The fight does have releated assigned featured fight records
          // i.e It is a featured fight
          if (featuredFightVector.size() > 0)
          {
            // Loop through the Vector adding it to an array of longs

            long[] array = new long[featuredFightVector.size()];
            for (int i = 0;i < featuredFightVector.size();i++)
            {
              array[i] = ((Long)featuredFightVector.elementAt(i)).longValue();
            }

            fData.featuredFightTypes = array;
          }
          fightDataVector.addElement(fData);
        }

        getTournamentInfo.close();
        rs.close();

        // Create the BDataSet and populate it
        BDataSet dataSet = new BDataSet();

        dataSet.add(tournamentDataVector);
        dataSet.add(tournamentEntryDataVector);
        dataSet.add(fightDataVector);

        services.add(BServices.zip());

        //System.err.println("Start Get History: " + System.currentTimeMillis());

        return BMessage.ACK("HIS", dataSet);
      }
      catch (Exception e)
      {
        System.err.println("SGateway/TournamentHistoryRequestHandler: " + e);
        return BMessage.error("HIS", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/TournamentHistoryRequestHandler");
        closeResultSet(rs1,"SGateway/TournamentHistoryRequestHandler");
        closeStatement(getTournamentInfo,"SGateway/TournamentHistoryRequestHandler");
        closeStatement(statement,"SGateway/TournamentHistoryRequestHandler");
      }
    }

  }


  // Get the RemoveEnteredBot request handler...
  // -------------------------------------------------------------
  protected static class RemoveBotFromTournamentRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "RBT"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      BigDecimal entryFee = new BigDecimal(0d);
      Connection con = reqState.con;
      String sql;

      Statement statement = null;
      PreparedStatement pstmt = null;

      ResultSet rs = null;

      try
      {
        long id = reqState.accountID;

        // Turn Commit off
        con.setAutoCommit(false);

        long tournament_entry_ID = ((BIdentData)request.getData()).getID();

        // Check to see if it is possible to delete this Tournament_entry
        sql = "SELECT t.entry_fee,t.id, te.bot_name FROM TOURNAMENT_ENTRY te, TOURNAMENT_CLASSIFICATION t WHERE te.tournament_id = t.id";
        sql = sql + " AND te.id = "+ tournament_entry_ID + " AND te.account_id = " + id + " AND t.status = 'OPEN'";

        //  AND te.account_id = " + id Ensures that the person removing the entry is the same as the one who placed it!

        statement = con.createStatement ();
        rs = statement.executeQuery(sql);

        if (!rs.next())
        {
          // No Record exists so it isn't possible to delete this entry

          rs.close();
          statement.close();
          con.rollback();

          System.err.println("Tournament Closed or couldn't find entry to remove");
          return BMessage.error("RBT", "Tournament Closed or couldn't find entry to remove");
        }

        entryFee = rs.getBigDecimal("entry_fee");
        long tournamentID = rs.getLong("id");
        String botName = rs.getString("bot_name");
        rs.close();
        statement.close();

        // Attempt to delete the Tournament Entry
        sql = "DELETE ";
        sql = sql + " FROM TOURNAMENT_ENTRY te";
        sql = sql + " WHERE te.id = ?";
        sql = sql + " AND EXISTS";
	      sql = sql + " (SELECT *";
	      sql = sql + " FROM TOURNAMENT";
	      sql = sql + " WHERE te.tournament_id = id";
	      sql = sql + " AND status = 'OPEN')";

        pstmt = con.prepareStatement(sql);
        pstmt.setLong(1,tournament_entry_ID);
        int count = pstmt.executeUpdate();
        pstmt.close();

        if (count == 0)
        {
          // The highly unlikely has occured  - the Tournament Status changed between
          // request and action - stop the process and inform the user

          pstmt.close();
          con.rollback();

          System.err.println("Tournament Closed - Can't remove entry");
          return BMessage.error("RBT", "Tournament Closed - Can't remove entry");
        }

        // Subtract the cost from the account and add the transaction
        if (entryFee.doubleValue() > 0d)
        {

          sql = "INSERT INTO TRANSACTION (account_id, type, details, amount, status, tournament_id, account_from,bot_name)";
          sql += "VALUES (" + id + ",'REFND','Bot Withdrawn'," + entryFee + ",'C',"+tournamentID+",'C','" + botName +"')";
          statement = con.createStatement();
          statement.executeUpdate(sql);
          statement.close();

          // Deduct the amount from the account
          sql = "UPDATE ACCOUNT SET current_balance = nvl(current_balance,0) +" + entryFee + " WHERE id = " + id;
          statement = con.createStatement();
          statement.executeUpdate(sql);
          statement.close();

        }
        pstmt.close();
        con.commit();

        return BMessage.ACK("RBT");
      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/RemoveBotFromTournamentRequestHandler: "+ sqle1);
          return BMessage.error("RBT", "SGateway/RemoveBotFromTournamentRequestHandler: "+ sqle1);
        }

        System.err.println("SGateway/RemoveBotFromTournamentRequestHandler: "+ e);
        return BMessage.error("RBT", "SGateway/RemoveBotFromTournamentRequestHandler: "+ e);
      }
      finally
      {
        closeResultSet(rs,"SGateway/RemoveBotFromTournamentRequestHandler");
        closeStatement(statement,"SGateway/RemoveBotFromTournamentRequestHandler");
        closeStatement(pstmt,"SGateway/RemoveBotFromTournamentRequestHandler");
      }

    }
  }

  // Returns a requested bot, but only if user has 'Admin' permissions...
  // -------------------------------------------------------------
  protected static class BotRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "GBOT"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      String sql;
      Statement statement = null;
      ResultSet rs = null;

      try
      {
        BTournamentSubmitData tournamentSubmitData = (BTournamentSubmitData)request.getData();
        long TEID = tournamentSubmitData.getID();
        Connection con = reqState.con;

        // Get the bot out of the tournament entry
        sql = "SELECT bot_file";
        sql = sql + " FROM TOURNAMENT_ENTRY";
        sql = sql + " WHERE id = " + TEID;
        statement = con.createStatement();
        rs = statement.executeQuery(sql);

        // Zip and encrypt the response...
        services.add(BServices.zip());


        if (rs.next())
        {
          // Found the entry
          // Get the bot turn it into a byte array and send it back
          byte[] botBytes = SServerUtility.blobToByteArray (((OracleResultSet)rs).getBLOB("bot_file"));
          rs.close();
          statement.close();
          tournamentSubmitData.botData = botBytes;

          return BMessage.ACK("GBOT", tournamentSubmitData);
        }
        else
        {
          // Couldn't find the Tournament Entry
          rs.close();
          statement.close();

          return BMessage.error("GBOT", "Couldn't Find Tournament Entry");

        }
      }
      catch (Exception e)
      {
        System.err.println("SGateway/BotRequestHandler: " + e.getMessage());
        return BMessage.error("GBOT", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/BotRequestHandler");
        closeStatement(statement,"SGateway/BotRequestHandler");
      }

    }

  }


  // Process the nomination for a featured fight request handler...
  // -------------------------------------------------------------
  protected static class NominateFeaturedFightRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "NFF"; }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      ResultSet rs = null;
      Connection con = reqState.con;

      try
      {
        con.setAutoCommit(false);

        long fightID = ((BIdentData)request.getData()).getID();
        long id = reqState.accountID;


        // Determine if the nominated fight is in a valid tournament
        // (Is one of the last seven tournaments)
        sql = "SELECT Count(*) AS fightcount FROM FIGHT";
        sql += " WHERE id = " + fightID;

        /* Removed as Illegal Fight Errors Were Caused - Still Uncertain as to why
        sql += " AND tournament_id IN";
        sql += " (SELECT id FROM ";
        sql += " (SELECT id FROM TOURNAMENT WHERE (status = 'FINISHED' OR status = 'FIGHTING' OR status = 'PENDING') ORDER BY id DESC)";
        sql += " WHERE ROWNUM <= 7)";*/


        // Get the battleRecording and/or stats
        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();

        if (rs.getLong("fightcount") == 0)
        {
          // Client isn't allowed to vote for this fight or fight doesn't exist

          rs.close();
          statement.close();

          System.err.println("SGateway/NominateFeaturedFightRequestHandler: Illegal Fight "+ fightID );
          return BMessage.error("NFF", "Illegal Fight");
        }

        rs.close();
        statement.close();

        // It's a valid fight so if it hasn't already been entered into the Featured_Fight table then do so
        sql = "SELECT COUNT(*) AS count FROM FEATURED_FIGHT WHERE fight_id = " + fightID;

        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();

        if (rs.getLong("count") == 0)
        {
          rs.close();
          statement.close();

          // Fight hasn't already been nominated so Insert it
          sql = "INSERT INTO FEATURED_FIGHT (fight_id, featured_fight_type_id, account_id) VALUES (" + fightID + "," + BFeaturedFightTypeData.UNASSIGNED + ", " + id + ")";
          statement = con.createStatement();
          statement.executeUpdate(sql);
          statement.close();
        }

        rs.close();
        statement.close();
        
        return BMessage.ACK("NFF");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/NominateFeaturedFightRequestHandler: " + sqle1);
          return BMessage.error("MFF", sqle1.getMessage());
        }

        System.err.println("SGateway/NominateFeaturedFightRequestHandler: " + e.getMessage());
        return BMessage.error("NFF", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/NominateFeaturedFightRequestHandler");
        closeStatement(statement,"SGateway/NominateFeaturedFightRequestHandler");
      }

    }
  }

   // Process the update for a featured fight request handler...
   // *** NOT USED SHOULD PROBABLY DELETE
  // -------------------------------------------------------------
  protected static class UpdateFeaturedFightRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "UFF"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      ResultSet rs = null;
      Connection con = reqState.con;

      try
      {
        con.setAutoCommit(false);

        BFightData fightData = (BFightData)request.getData();

        // Update the fight comments
        sql = "UPDATE FIGHT SET narrative = " + fightData.comments;
        sql += " WHERE id = " + fightData.getID();


        // Get the battleRecording and/or stats
        statement = con.createStatement();
        statement.executeUpdate(sql);

        statement.close();

        // Update the Featured_Fight

        // First delete all the releated records
        sql = "DELETE FROM FEATURED_FIGHT WHERE fight_id = " + fightData.getID();

        statement = con.createStatement();
        statement.executeUpdate(sql);

        statement.close();

        // Insert the new Featured Fight types

        for (int i = 0;i <= fightData.featuredFightTypes.length;i++)
        {
          sql = "INSERT INTO FEATURED_FIGHT (fight_id,featured_fight_type_id)";
          sql += " VALUES (" + fightData.getID() + "," + fightData.featuredFightTypes[i] + ")";

          statement = con.createStatement();
          statement.executeUpdate(sql);

          statement.close();
        }

        return BMessage.ACK("UFF");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/UpdateFeaturedFightRequestHandler: " + sqle1);
          return BMessage.error("MFF", sqle1.getMessage());
        }
        
        System.err.println("SGateway/UpdateFeaturedFightRequestHandler: " + e.getMessage());
        return BMessage.error("UFF", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/UpdateFeaturedFightRequestHandler");
        closeStatement(statement,"SGateway/UpdateFeaturedFightRequestHandler");
      }

    }
  }

   // Process the request for a featured fight types request handler...
  // -------------------------------------------------------------
  protected static class FeaturedFightTypesRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "FFT"; }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      ResultSet rs = null;

      try
      {
        Connection con = reqState.con;
        Vector featuredFightTypeVector = new Vector();

        // Return List of the Different Types of Featured Fights
        sql = "SELECT id, description";
        sql += " FROM FEATURED_FIGHT_TYPE";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);


        while (rs.next())
        {
          // Use the BFeaturedFightTypeData Object to provide mapping between the database
          // and the object values

          long temp = rs.getLong("id");

          if (temp == 0)
            temp = BFeaturedFightTypeData.UNASSIGNED;
          else if (temp == 1)
            temp = BFeaturedFightTypeData.FEATURED;
          else if (temp == 2)
            temp = BFeaturedFightTypeData.ALL_TIME_GREAT;
          else
            temp = BFeaturedFightTypeData.UNKNOWN_TYPE;


          BFeaturedFightTypeData featuredFightType = new BFeaturedFightTypeData(temp);
          featuredFightType.typeName = rs.getString("description");
          featuredFightTypeVector.addElement(featuredFightType);
        }

        rs.close();
        statement.close();

        // Populate the container
        BFeaturedFightInfoData featuredFightInfoData = new BFeaturedFightInfoData(-1);
        featuredFightInfoData.featuredFightTypeData = featuredFightTypeVector;

        return BMessage.ACK("FFT",featuredFightInfoData);

      }
      catch (Exception e)
      {
        System.err.println("SGateway/FeaturedFightTypesRequestHandler: " + e.getMessage());
        return BMessage.error("FFT", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/FeaturedFightTypesRequestHandler");
        closeStatement(statement,"SGateway/FeaturedFightTypesRequestHandler");
      }

    }
  }

  // Moves a featured Fight to a new Type
  // -------------------------------------------------------------
  protected static class MoveFeaturedFightRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "MFF"; }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      ResultSet rs = null;
      Connection con = reqState.con;
      try
      {
        con.setAutoCommit(false);


        BFeaturedFightUpdateData ffUpdateData = (BFeaturedFightUpdateData)request.getData();

        // Delete the Type
        sql = "DELETE FROM FEATURED_FIGHT";
        sql += " WHERE fight_id = " + ffUpdateData.getID() + " AND featured_fight_type_id = " + ffUpdateData.currentType;

        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();


        // Add the new type
        sql = "INSERT INTO FEATURED_FIGHT";
        sql += " (fight_id, featured_fight_type_id) VALUES (" + ffUpdateData.getID() + "," + ffUpdateData.newType + ")";

        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        con.commit();
        return BMessage.ACK("MFF");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/MoveFeaturedFightRequestHandler: " + sqle1);
          return BMessage.error("MFF", sqle1.getMessage());
        }

        System.err.println("SGateway/MoveFeaturedFightRequestHandler: " + e.getMessage());
        return BMessage.error("MFF", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/MoveFeaturedFightRequestHandler");
        closeStatement(statement,"SGateway/MoveFeaturedFightRequestHandler");
      }

    }
  }

  // Creates a new type of Featured Fight
  // -------------------------------------------------------------
  protected static class CopyFeaturedFightRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "CFF"; }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      ResultSet rs = null;
      Connection con = reqState.con;
      try
      {
        con.setAutoCommit(false);


        BFeaturedFightUpdateData ffUpdateData = (BFeaturedFightUpdateData)request.getData();

        // Add the new type so effectively copying the old and assigning a new value
        sql = "INSERT INTO FEATURED_FIGHT";
        sql += " (fight_id, featured_fight_type_id) VALUES (" + ffUpdateData.getID() + "," + ffUpdateData.newType + ")";

        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        con.commit();
        
        return BMessage.ACK("CFF");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/CopyFeaturedFightRequestHandler: " + sqle1);
          return BMessage.error("CFF", sqle1.getMessage());
        }

        System.err.println("SGateway/CopyFeaturedFightRequestHandler: " + e.getMessage());
        return BMessage.error("CFF", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/CopyFeaturedFightRequestHandler");
        closeStatement(statement,"SGateway/CopyFeaturedFightRequestHandler");
      }

    }
  }

  // Deletes a new type of Featured Fight
  // -------------------------------------------------------------
  protected static class DeleteFeaturedFightRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "DFF"; }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      ResultSet rs = null;
      Connection con = reqState.con;
      try
      {
        con.setAutoCommit(false);


        BFeaturedFightUpdateData ffUpdateData = (BFeaturedFightUpdateData)request.getData();

        // Delete the Type
        sql = "DELETE FROM FEATURED_FIGHT";
        sql += " WHERE fight_id = " + ffUpdateData.getID() + " AND featured_fight_type_id = " + ffUpdateData.currentType;

        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        con.commit();

        return BMessage.ACK("DFF");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/DeleteFeaturedFightRequestHandler: " + sqle1);
          return BMessage.error("DFF", sqle1.getMessage());
        }

        System.err.println("SGateway/DeleteFeaturedFightRequestHandler: " + e.getMessage());
        return BMessage.error("DFF", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/DeleteFeaturedFightRequestHandler");
        closeStatement(statement,"SGateway/CopyFeaturedFightRequestHandler");
      }

    }
  }

  // Deletes fight and stat information from fights not shown
  // -------------------------------------------------------------
  protected static class CleanUpFightsRequestHandler extends ValidatedRequestHandler
  {
    // CUP - Clean Up Fights
    public String getType()
      { return "CUP"; }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      Statement statement1 = null;
      ResultSet rs = null;
      Connection con = reqState.con;
      try
      {
        con.setAutoCommit(false);

        // Find all the tournaments to be set to deleted status

        sql = "SELECT id FROM TOURNAMENT t";
        sql += " WHERE type = 'INTFC'";
        sql += " AND status = 'FINISHED'";
        sql += " AND t.id NOT IN (";
        sql +=  		" SELECT id FROM";
        sql +=  		" (SELECT *";
        sql +=  		" FROM TOURNAMENT";
        sql +=  		" WHERE (status = 'FINISHED') AND type = 'INTFC' ORDER BY id DESC)";
        sql +=  		" WHERE ROWNUM <= 7)";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);

        while (rs.next())
        {
          long tournamentID = rs.getLong("id");

          // Set the tournament Status to Deleted
          sql =   "UPDATE TOURNAMENT t SET status = 'DELETED'";
          sql += " WHERE id = " + tournamentID;

          statement1 = con.createStatement();
          statement1.executeUpdate(sql);
          statement1.close();

          // Clear all the related tournament entry bot files
          sql = "UPDATE TOURNAMENT_ENTRY SET BOT_FILE = NULL";
          sql += " WHERE tournament_id = " + tournamentID;

          statement1 = con.createStatement();
          statement1.executeUpdate(sql);
          statement1.close();

          // Delete all the log files related to the tournament_entry
          sql = "DELETE FROM FIGHT_CLIENT_LOG fcl ";
          sql += " WHERE EXISTS (SELECT id FROM TOURNAMENT_ENTRY te";
          sql += " WHERE te.id = fcl.tournament_entry_id AND tournament_id = " + tournamentID + ")";

          statement1 = con.createStatement();
          statement1.executeUpdate(sql);
          statement1.close();

          // Clean up the fights
          sql = "UPDATE FIGHT fgt ";
	        sql += " SET battle_recording = NULL,";
	        sql += " battle_stats1 = NULL,";
	        sql += " battle_stats2 = NULL,";
	        sql += " status = 'DELETED'";
          sql += " WHERE EXISTS";
	        sql +=      " (SELECT f.id";
	        sql +=      " FROM FIGHT f, TOURNAMENT t";
	        sql +=      " WHERE  fgt.id = f.id";
	        sql +=      " AND f.STATUS = 'FOUGHT'";
	        sql +=      " AND t.STATUS = 'DELETED'";
	        sql +=      " AND f.tournament_id = t.id";
          // Is not a final of semi-final
	        sql +=      " AND f.round < t.current_round -2";
          // The related tournament isn't in the last 7 tournaments
	        sql +=      " AND t.id = " + tournamentID;
          // The fight isn't an all time great
	        sql +=   " AND f.id NOT IN (SELECT fight_id";
			    sql +=   " FROM FEATURED_FIGHT";
			    sql += " WHERE featured_fight_type_id = " + BFeaturedFightTypeData.ALL_TIME_GREAT + "))";


          statement1 = con.createStatement();
          statement1.executeUpdate(sql);
          statement1.close();

          // Delete all the log files releated to the fight
          sql = "DELETE FROM FIGHT_CLIENT_LOG fcl ";
          sql += " WHERE EXISTS (SELECT id FROM FIGHT f";
          sql += " WHERE f.id = fcl.fight_id AND tournament_id = " + tournamentID + ")";

          statement1 = con.createStatement();
          statement1.executeUpdate(sql);
          statement1.close();
        }

        rs.close();
        statement.close();

        // Also delete all the Featured Fights that aren't all time greats
        // that have the status "DELETED"

        sql = "DELETE FROM FEATURED_FIGHT fght";
        sql += " WHERE fght.fight_id IN";
	      sql += " (SELECT ff.fight_id FROM FEATURED_FIGHT ff, FIGHT f";
	      sql += " WHERE";
        sql += " ff.featured_fight_type_id <> " + BFeaturedFightTypeData.ALL_TIME_GREAT;
        sql += " AND ff.fight_id = f.id";
        sql += " AND f.status = 'DELETED')";
        sql += " AND fght.featured_fight_type_id <> " + BFeaturedFightTypeData.ALL_TIME_GREAT;

        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        con.setAutoCommit(true);
        con.commit();

        return BMessage.ACK("CUP");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/CleanUpFightsRequestHandler: " + sqle1);
          return BMessage.error("CUP", sqle1.getMessage());
        }

        System.err.println("SGateway/CleanUpFightsRequestHandler: " + e.getMessage());
        return BMessage.error("CUP", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/CleanUpFightsRequestHandler");
        closeStatement(statement,"SGateway/CleanUpFightsRequestHandler");
        closeStatement(statement1,"SGateway/CleanUpFightsRequestHandler");
      }

    }
  }


  // Updates the comments on a featured fight
  // -------------------------------------------------------------
  protected static class UpdateFeaturedFightCommentRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "UFFC"; }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      ResultSet rs = null;
      Connection con = reqState.con;
      try
      {
        con.setAutoCommit(false);


        BFeaturedFightUpdateData ffUpdateData = (BFeaturedFightUpdateData)request.getData();

        // Update the Type
        sql = "UPDATE FIGHT SET narrative = '" + SServerAccess.parseString(ffUpdateData.fightComment) + "'";
        sql += " WHERE id = " + ffUpdateData.getID();

        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        con.commit();

        return BMessage.ACK("UFFC");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/UpdateFeaturedFightCommentRequestHandler: " + sqle1);
          return BMessage.error("UFFC", sqle1.getMessage());
        }

        System.err.println("SGateway/UpdateFeaturedFightCommentRequestHandler: " + e.getMessage());
        return BMessage.error("UFFC", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/UpdateFeaturedFightCommentRequestHandler");
        closeStatement(statement,"SGateway/UpdateFeaturedFightCommentRequestHandler");
      }

    }
  }

  // Get list of all the voted for featured fights, their related types, and
  // the list of available types
  // -------------------------------------------------------------
  protected static class FeaturedFightInfoRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "FFI"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;

      Statement statement = null;
      Statement statement1 = null;

      ResultSet rs = null;
      ResultSet rs1 = null;

      try
      {
        Vector fightDataVector = new Vector();
        Vector featuredFightVector = new Vector();
        Vector featuredFightTypeVector = new Vector();


        Connection con = reqState.con;

        // Get a list of all the Fight's that are in the Featured Fights's
        // table

        sql = "SELECT ";
        sql += 	  " fight.id AS FID, ";
        sql += 		" tournament_id,";
        sql += 		" tournament_entry1_id,";
        sql += 		" tournament_entry2_id,";
        sql += 		" round,";
        sql += 		" result,";
        sql += 		" fight.status,";
        sql += 		" duration,";
        sql += 		" t.current_round,";
        sql += 		" fight.won_by,";
        sql += 		" fight.narrative";
        sql += " FROM FIGHT, TOURNAMENT t";
        sql += " WHERE EXISTS";
        sql += " (	SELECT *";
        sql += " 	FROM FEATURED_FIGHT";
        sql += " 	WHERE FIGHT.id = fight_id";
        sql += " 	GROUP BY fight_id";
        sql += " )";
        sql += 		" AND t.id = tournament_id";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);

        while (rs.next())
        {
          int highestRound = rs.getInt("current_round");

          long te1ID = rs.getLong("tournament_entry1_id");
          long te2ID = rs.getLong("tournament_entry2_id");

          BFightData fData = new BFightData(rs.getLong("FID"));
          fData.tournamentID = new Long(rs.getLong("tournament_id"));
          fData.entry01ID = new Long(te1ID);
          fData.entry02ID = new Long(te2ID);
          fData.round = rs.getInt("round");
          fData.winnerID = new Long(rs.getLong("result"));
          fData.status = BFightData.COMPLETE;
          fData.duration = rs.getFloat("duration");
          fData.winDescription = rs.getString("won_by");
          fData.comments = rs.getString("narrative");

          //String featured_fight = rs.getString("featured_fight");

          // If the fight is a final or semi final then show it
          if (fData.round == (highestRound-1))
            fData.roundDescription = "Final";
          else if (fData.round == (highestRound-2))
            fData.roundDescription = "Semi Final";

          // Get the name of the two bots involved in the tournament
          String fightName;

          // Get the first name
          sql = "SELECT bot_name FROM TOURNAMENT_ENTRY WHERE id = " + te1ID;
          statement1 = con.createStatement();
          rs1 = statement1.executeQuery(sql);
          

          rs1.next();
          fightName = rs1.getString("bot_name") + " vs. ";

          rs1.close();
          statement1.close();

          // Get the second name
          sql = "SELECT bot_name FROM TOURNAMENT_ENTRY WHERE id = " + te2ID;
          statement1 = con.createStatement();
          rs1 = statement1.executeQuery(sql);

          rs1.next();
          fightName += rs1.getString("bot_name");

          rs1.close();
          statement1.close();

          // Assign the name
          fData.name = fightName;


          // For each fight get the list of Featured Fight Types
          sql = "SELECT featured_fight_type_id";
          sql += " FROM FEATURED_FIGHT";
          sql += " WHERE fight_id = " + fData.getID();

          statement1 = con.createStatement();
          rs1 = statement1.executeQuery(sql);

          while (rs1.next())
          {
            featuredFightVector.addElement(new Long(rs1.getLong("featured_fight_type_id")));

          }

          rs1.close();
          statement1.close();

          // Loop through the Vector adding it to an array of longs

          long[] array = new long[featuredFightVector.size()];
          for (int i = 0;i < featuredFightVector.size();i++)
          {

            // Use the BFeaturedFightTypeData Object to provide mapping between the database
            // and the object values

            long temp = ((Long)featuredFightVector.elementAt(i)).longValue();

            if (temp == 0)
              temp = BFeaturedFightTypeData.UNASSIGNED;
            else if (temp == 1)
              temp = BFeaturedFightTypeData.FEATURED;
            else if (temp == 2)
              temp = BFeaturedFightTypeData.ALL_TIME_GREAT;
            else
              temp = BFeaturedFightTypeData.UNKNOWN_TYPE;

            array[i] = temp;
          }

          // Empty the Featured Fight Vector
          featuredFightVector.clear();

          fData.featuredFightTypes = array;
          fightDataVector.addElement(fData);
        }

        // Return List of the Different Types of Featured Fights
        sql = "SELECT id, description";
        sql += " FROM FEATURED_FIGHT_TYPE";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);


        while (rs.next())
        {
          // Use the BFeaturedFightTypeData Object to provide mapping between the database
          // and the object values

          long temp = rs.getLong("id");

          if (temp == 0)
            temp = BFeaturedFightTypeData.UNASSIGNED;
          else if (temp == 1)
            temp = BFeaturedFightTypeData.FEATURED;
          else if (temp == 2)
            temp = BFeaturedFightTypeData.ALL_TIME_GREAT;
          else
            temp = BFeaturedFightTypeData.UNKNOWN_TYPE;

          BFeaturedFightTypeData featuredFightType = new BFeaturedFightTypeData(temp);
          featuredFightType.typeName = rs.getString("description");
          featuredFightTypeVector.addElement(featuredFightType);
        }

        rs.close();
        statement.close();


        // Populate the container
        BFeaturedFightInfoData featuredFightInfoData = new BFeaturedFightInfoData(-1);

        featuredFightInfoData.fightData = fightDataVector;
        featuredFightInfoData.featuredFightData = featuredFightVector;
        featuredFightInfoData.featuredFightTypeData = featuredFightTypeVector;

        
        return BMessage.ACK("FFI",featuredFightInfoData);

      }
      catch (Exception e)
      {
        System.err.println("SGateway/FeaturedFightInfoRequestHandler: " + e.getMessage());
        return BMessage.error("FFI", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/FeaturedFightInfoRequestHandler");
        closeStatement(statement,"SGateway/FeaturedFightInfoRequestHandler");
        closeResultSet(rs1,"SGateway/FeaturedFightInfoRequestHandler");
        closeStatement(statement1,"SGateway/FeaturedFightInfoRequestHandler");
      }

    }
  }

  // Get list of all waiting tournaments
  // Used to find tournaments to set adverts for
  // -------------------------------------------------------------
  protected static class GetWaitingTournamentsHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "GWAT"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      ResultSet rs = null;

      try
      {

        Connection con = reqState.con;

        // Get a list of all the Fight's that are in the Featured Fights's
        // table

        sql = "SELECT ";
        sql += 	  " id, ";
        sql += 		" name,";
        sql += 		" arena_file_name";
        sql += " FROM TOURNAMENT";
        sql += " WHERE status = 'WAITING' OR status = 'OPEN'";
        sql += " AND type = 'INTFC'";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);

        BDataSet dataSet = new BDataSet();

        while (rs.next())
        {

          BTournamentData t = new BTournamentData(rs.getLong("id"));
          t.name = rs.getString("name");
          t.arena = rs.getString("arena_file_name");
          dataSet.add(t);
        }


        rs.close();
        statement.close();

        return BMessage.ACK("GWAT",dataSet);

      }
      catch (Exception e)
      {
        System.err.println("SGateway/GetWaitingTournamentsHandler: " + e.getMessage());
        return BMessage.error("GWAT", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/GetWaitingTournamentsHandler");
        closeStatement(statement,"SGateway/GetWaitingTournamentsHandler");
      }

    }
  }

  // Add an advert against a waiting tournament
  // -------------------------------------------------------------
  protected static class AddAdvertHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "AADV"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      Connection con = reqState.con;

      Statement statement = null;
      PreparedStatement pstmt = null;
      ResultSet rs = null;
      String sql;

      try
      {
        con.setAutoCommit(false);

        BAdvertData advert = (BAdvertData)request.getData();

        // Get the next value for the Advert record
        sql = "SELECT TOURNAMENT_ADVERT_IDS.nextval AS next_val FROM DUAL";
        statement = con.createStatement ();
        rs = statement.executeQuery(sql);
        rs.next();

        long nextVal = rs.getLong("next_val");

        statement.close();
        rs.close();

        pstmt = con.prepareStatement("INSERT INTO TOURNAMENT_ADVERT (id, tournament_id, advert) VALUES (?,?,empty_blob())");
        pstmt.setLong(1,nextVal);
        pstmt.setLong(2,advert.tournamentID);
        pstmt.execute();
        pstmt.close();

        sql = "SELECT advert FROM TOURNAMENT_ADVERT WHERE id=" + nextVal + " FOR UPDATE";

        statement = con.createStatement ();
        rs = statement.executeQuery(sql);
        rs.next();
        BLOB blob = ((OracleResultSet)rs).getBLOB(1);
        rs.close();
        statement.close();

        // Insert the graphic into the table
        OutputStream outstream = blob.getBinaryOutputStream();
        outstream.write(advert.getAdvertArray());
        outstream.close();

        con.commit();

        return BMessage.ACK("AADV");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/AddAdvertHandler: " + sqle1);
          return BMessage.error("STI", sqle1.getMessage());
        }

        System.err.println("SGateway/AddAdvertHandler: " + e.getMessage());
        return BMessage.error("STI", "SGateway/AddAdvertHandler: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/AddAdvertHandler");
        closeStatement(pstmt,"SGateway/AddAdvertHandler");
        closeStatement(statement,"SGateway/AddAdvertHandler");
      }
    }
  }

  // Gets the advert(s) related to a tournament
  // -------------------------------------------------------------
  protected static class GetAdvertsHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "GADV"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      Connection con = reqState.con;

      Statement statement = null;
      ResultSet rs = null;
      String sql;

      try
      {

        BIdentData identData = (BIdentData)request.getData();

        // Get the next value for the Advert record
        sql = "SELECT advert, id FROM TOURNAMENT_ADVERT WHERE tournament_id = " + identData.getID();
        statement = con.createStatement ();
        rs = statement.executeQuery(sql);

        BDataSet dataSet = new BDataSet();

        while (rs.next())
        {
          BAdvertData advertData = new BAdvertData(rs.getLong("id"));
          advertData.setAdvertArray(SServerUtility.blobToByteArray (((OracleResultSet)rs).getBLOB("advert")));
          dataSet.add(advertData);
        }

        rs.close();
        statement.close();

        return BMessage.ACK("GADV", dataSet);

      }
      catch (Exception e)
      {
        System.err.println("SGateway/GetAdvertsHandler: " + e.getMessage());
        return BMessage.error("GADV", "SGateway/GetAdvertsHandler: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/GetAdvertsHandler");
        closeStatement(statement,"SGateway/GetAdvertsHandler");
      }
    }
  }

  // Gets the player rankings
  // -------------------------------------------------------------
  protected static class GetPlayerRankingHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "GPR"; }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      Connection con = reqState.con;

      Statement statement = null;
      ResultSet rs = null;
      String sql;

      try
      {

        BIdentData identData = (BIdentData)request.getData();

        // Get the next value for the Advert record
        sql = "SELECT total_prof_fights, total_prof_fights_won, total_prof_tourns_won,";
        sql += " prof_ranking_points, prof_ranking, total_amateur_fights, total_amateur_fights_won,";
        sql += " total_amateur_tourns_won, amateur_ranking_points, amateur_ranking, professional_status, blurb";
        sql += " FROM ACCOUNT WHERE id = " + identData.getID();
        statement = con.createStatement ();
        rs = statement.executeQuery(sql);

        BAccountRankingData accountRanking = new BAccountRankingData(identData.getID());

        if (rs.next())
        {
          accountRanking.totalProfFights = rs.getInt("total_prof_fights");
          accountRanking.totalProfFightsWon = rs.getInt("total_prof_fights_won");
          accountRanking.totalProfTournsWon = rs.getInt("total_prof_tourns_won");
          accountRanking.profRankingPoints = rs.getInt("prof_ranking_points");
          accountRanking.profRanking = rs.getInt("prof_ranking");
          accountRanking.totalAmateurFights = rs.getInt("total_amateur_fights");
          accountRanking.totalAmateurFightsWon = rs.getInt("total_amateur_fights_won");
          accountRanking.totalAmateurTournsWon = rs.getInt("total_amateur_tourns_won");
          accountRanking.amateurRankingPoints = rs.getInt("amateur_ranking_points");
          accountRanking.amateurRanking = rs.getInt("amateur_ranking");
          accountRanking.professionalStatus = rs.getString("professional_status");
          accountRanking.accountBlurb = rs.getString("blurb");
        }

        rs.close();
        statement.close();

        return BMessage.ACK("GPR", accountRanking);

      }
      catch (Exception e)
      {
        System.err.println("SGateway/GetPlayerRankingHandler: " + e.getMessage());
        return BMessage.error("GPR", "SGateway/GetPlayerRankingHandler: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/GetPlayerRankingHandler");
        closeStatement(statement,"SGateway/GetPlayerRankingHandler");
      }
    }
  }

  // Gets total value of all the Accounts for this Player
  // -------------------------------------------------------------
  protected static class GetAccountTotalBalanceHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "GATB"; }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      Connection con = reqState.con;

      Statement statement = null;
      ResultSet rs = null;
      String sql;

      try
      {

        // Try and find the account
        sql = "SELECT NVL(current_balance,0) current_balance, NVL(winnings_balance,0) winning_balance,";
        sql += " wp_id";
        sql += " FROM ACCOUNT WHERE id = " + reqState.accountID;

        statement = con.createStatement ();
        rs = statement.executeQuery(sql);

        if (!rs.next())
        {
          // Couldn't find the account
          rs.close();
          statement.close();
          System.err.println("SGateway/GetAccountTotalBalanceHandler:Couldn't locate account details");
          return BMessage.error("GATB", "Couldn't Locate Account Details");
        }

        String worldPayID   = rs.getString("wp_id");

        BBalanceData balance = new BBalanceData(-1);

        balance.currentBalance = rs.getFloat("current_balance");
        balance.winningBalance = rs.getFloat("winning_balance");


        if (worldPayID != null)
          balance.isFuturePayActive = true;

        rs.close();
        statement.close();

        return BMessage.ACK("GATB", balance);

      }
      catch (Exception e)
      {
        System.err.println("SGateway/GetAccountTotalBalanceHandler: " + e.getMessage());
        return BMessage.error("GATB", "SGateway/GetAccountTotalBalanceHandler: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/GetAccountTotalBalanceHandler");
        closeStatement(statement,"SGateway/GetAccountTotalBalanceHandler");
      }
    }
  }


  // Gets all the transactions for this player
  // Get Account TRansactions
  // -------------------------------------------------------------
  protected static class GetAccountTransactionsHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "GATR"; }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

     Connection con = reqState.con;

      Statement statement = null;
      Statement statement2 = null;
      ResultSet rs = null;
      ResultSet rs2 = null;
      String sql;

      try
      {
        sql = "SELECT id, transaction_date, details, tournament_id, amount, status, bot_name, account_from";
        sql += " FROM TRANSACTION";
        sql += " WHERE account_id = " + reqState.accountID;
        sql += " ORDER BY transaction_date";

        statement = con.createStatement ();
        rs = statement.executeQuery(sql);

        BDataSet dataSet = new BDataSet();

        while (rs.next())
        {
          BTransactionData transData = new BTransactionData(rs.getLong("id"));

          Calendar cal = Calendar.getInstance(TimeZone.getDefault());
          cal.setTime(rs.getTimestamp("transaction_date"));
          transData.date = cal;

          transData.amount = rs.getFloat("amount");

          if(rs.getString("status").equals("U"))
            transData.status = "Unprocessed";
          else if (rs.getString("status").equals("P"))
            transData.status = "Processing";
          else if (rs.getString("status").equals("C"))
            transData.status = "Complete";
          else
          {
            System.err.println("SGateway/GetAccountTransactionsHandler: Unknown Transaction Type: account " + reqState.accountID);
            transData.status = "Unknown";
          }

          transData.account = rs.getString("account_from");

          // Build the detail
          transData.detail = rs.getString("details");

          String botName = rs.getString("bot_name");

          if (botName != null)
            transData.detail += " - " + botName;

          long tournID = rs.getLong("tournament_id");

          if (tournID != 0)
          {
            sql = "SELECT name FROM TOURNAMENT WHERE id = " + tournID;
            statement2 = con.createStatement ();
            rs2 = statement2.executeQuery(sql);

            if (rs2.next())
            {
              String tName = rs2.getString("name");
              transData.detail += " - " + tName;
            }

            rs2.close();
            statement2.close();
          }

          dataSet.add(transData);
        }

        rs.close();
        statement.close();

        services.add(BServices.zip());
        return BMessage.ACK("GATR", dataSet);

      }
      catch (Exception e)
      {
        System.err.println("SGateway/GetAccountTransactionsHandler: " + e.getMessage());
        return BMessage.error("GATR", "SGateway/GetAccountTransactionsHandler: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/GetAccountTransactionsHandler");
        closeResultSet(rs2,"SGateway/GetAccountTransactionsHandler");
        closeStatement(statement,"SGateway/GetAccountTransactionsHandler");
        closeStatement(statement2,"SGateway/GetAccountTransactionsHandler");
      }
    }
  }

  
  // Update account With Future Pay
  // Update the current account balance by using Future Pay
  // Stored against the account
  // -------------------------------------------------------------
  protected static class UpdateCurrentBalWithFPHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "UWFP"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      Connection con = reqState.con;

      Statement statement = null;
      ResultSet rs = null;
      String sql;

      try
      {

        con.setAutoCommit(false);

        BFPTransferAmtData transferData = (BFPTransferAmtData)request.getData();

        // Check to see if the account has a FP number stored against it
        sql = "SELECT wp_id";
        sql += " FROM ACCOUNT WHERE wp_id IS NOT NULL AND id = " + reqState.accountID;

        statement = con.createStatement ();
        rs = statement.executeQuery(sql);

        if (!rs.next())
        {
          // Future Pay ID Doesn't exist for this account
          rs.close();
          statement.close();
          System.err.println("SGateway/UpdateCurrentBalWithFPHandler:No Future Pay ID for this account");
          return BMessage.error("UWFP", "No Future Pay details stored against this account");
        }

        // Get the Future Pay ID

        long fpID = rs.getLong("wp_id");

        rs.close();
        statement.close();

        // Send the FP Update Request to WorldPay

        // Build up the string to post

        String post = "instId=" + SGlobal.instID;
        post += "&authPW=" + SGlobal.authPW;
        //post += ";authPW=" + "3e3cd";
        post += "&futurePayId=" + fpID;
        //post += "&futurePayId=1";
        post += "&desc=Future_Pay_Payment_To_Account";
        // TODO: Remove this line when moving when we go live
        // Or change to 0
        post += "&testMode=0";
        post += "&cartId=FPAC";
        // FTOP = Future Pay To Account - A future pay top up of the account
        post += "&amount=" + transferData.fpTransAmt;
        post += "&op-paymentLFP";
        

        /*String post = "instId=" + URLEncoder.encode("28972");
        post += ";authPW=" + URLEncoder.encode("proto99");
        post += ";futurePayId=" + URLEncoder.encode("48401");
        post += ";desc=" + URLEncoder.encode("FP");
        post += ";testMode=" + URLEncoder.encode("100");
        post += ";cartId=" + URLEncoder.encode("FPAC");
        post += ";amount=" + URLEncoder.encode("5.0");
        post += ";op-paymentLFP"; */

        /*String post = "instId=" + URLEncoder.encode("28972");
        post += "&authPW=" + URLEncoder.encode("proto99");
        post += "&futurePayId=" + URLEncoder.encode("48401");
        post += "&desc=" + URLEncoder.encode("FP");
        post += "&testMode=" + URLEncoder.encode("100");
        post += "&cartId=" + URLEncoder.encode("FPAC");
        post += "&amount=" + URLEncoder.encode("5.0");
        post += "&op-paymentLFP";*/

        byte[] bytes = post.getBytes();

        System.out.println("-" + post + "-");

        URL url = new URL("http://select.worldpay.com/wcc/iadmin");
        HttpURLConnection connection = (HttpURLConnection)url.openConnection();
        connection.setDoOutput(true);
        connection.setDoInput(true);
        connection.setRequestMethod("POST");
        connection.setRequestProperty("Content-length", String.valueOf(bytes.length));

        OutputStream out = connection.getOutputStream(); out.write(bytes); out.flush();

        // Get the Reply

        System.out.println("WorldPay Response");
        BufferedReader in = new BufferedReader(
        new InputStreamReader( connection.getInputStream()));
        String inputLine;
        String fpReturnLine = "";

        while ((inputLine = in.readLine()) != null)
        {
          System.out.println(inputLine);
          fpReturnLine = inputLine;
        }

        in.close();

        StringTokenizer tokens = new StringTokenizer(fpReturnLine,",");

        String valid = tokens.nextToken();

        if(!valid.equals("Y"))
        {
          // Invalid Response From WP

          String error = tokens.nextToken();
          System.err.println("SGateway/UpdateCurrentBalWithFPHandler: Account: " +reqState.accountID + " FP Error: " + error);
          return BMessage.error("UWFP", error);
        }

        String transID = tokens.nextToken();

        sql = "INSERT INTO TRANSACTION (account_id, type, details, amount, status,account_from,ref,wp_id)";
        sql += "VALUES (" + reqState.accountID + ",'PYMNT','WorldPay Payment'," + transferData.fpTransAmt + ",'C','C','FPAC'," + transID + ")";
        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        // Deduct the amount from the account
        sql = "UPDATE ACCOUNT SET current_balance = nvl(current_balance,0) +" + transferData.fpTransAmt + " WHERE id = " + reqState.accountID;
        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();


        con.setAutoCommit(true);
        con.commit();

        return BMessage.ACK("UWFP");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/UpdateCurrentBalWithFPHandler: " + sqle1);
          return BMessage.error("UWFP", sqle1.getMessage());
        }

        System.err.println("SGateway/UpdateCurrentBalWithFPHandler: " + e.getMessage());
        return BMessage.error("UWFP", "SGateway/UpdateCurrentBalWithFPHandler: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/UpdateCurrentBalWithFPHandler");
        closeStatement(statement,"SGateway/UpdateCurrentBalWithFPHandler");
      }
    }
  }


  // Allows the client to put in a request for Winnings withdraw
  // -------------------------------------------------------------
  protected static class WithdrawWinningsHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "WWIN"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      Connection con = reqState.con;

      Statement statement = null;
      Statement statement2 = null;
      ResultSet rs = null;
      String sql;

      try
      {

        con.setAutoCommit(false);

        BFPTransferAmtData transferData = (BFPTransferAmtData)request.getData();

        // Check to see there is enough money in the account
        sql = "SELECT nvl(winnings_balance,0) winnings_balance";
        sql += " FROM ACCOUNT WHERE id = " + reqState.accountID + " FOR UPDATE";


        statement = con.createStatement();
        rs = statement.executeQuery(sql);

        rs.next();

        float winBal = rs.getFloat("winnings_balance");

        if (winBal < transferData.fpTransAmt)
        {
          rs.close();
          statement.close();
          System.err.println("Insufficient Winnings Bal for Withdraw: Account :" + reqState.accountID + " Amount: " + transferData.fpTransAmt);
          return BMessage.error("WWIN", "Insufficient Winnings Balance For Withdraw");
        }

        sql = "INSERT INTO TRANSACTION (account_id, type, details, amount, status,account_from)";
        sql += "VALUES (" + reqState.accountID + ",'WDRAW','Withdraw from winnings',-" + transferData.fpTransAmt + ",'U','W')";
        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        // Deduct the amount from the account
        sql = "UPDATE ACCOUNT SET winnings_balance = nvl(winnings_balance,0) -" + transferData.fpTransAmt + " WHERE id = " + reqState.accountID;
        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();


        // Now close the account cursor so that the lock is kept on the row
        // while we update the balance

        rs.close();
        statement.close();

        con.setAutoCommit(true);
        con.commit();

        return BMessage.ACK("WWIN");

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SGateway/WithdrawWinningsHandler: " + sqle1);
          return BMessage.error("WWIN", sqle1.getMessage());
        }

        System.err.println("SGateway/WithdrawWinningsHandler: " + e.getMessage());
        return BMessage.error("WWIN", "SGateway/WithdrawWinningsHandler: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/WithdrawWinningsHandler");
        closeStatement(statement,"SGateway/WithdrawWinningsHandler");
        closeStatement(statement2,"SGateway/WithdrawWinningsHandler");
      }
    }
  }


  // Gets all the player rankings
  // -------------------------------------------------------------
  protected static class GetAllRankingsHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "RANK"; }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      Connection con = reqState.con;

      Statement statement = null;
      ResultSet rs = null;
      String sql;

      try
      {

        BIdentData identData = (BIdentData)request.getData();

        String tournamentType;

        if (identData.getID() == BRankingData.PROFESSIONAL)
          tournamentType = BRankingData.PRO;
        else
          tournamentType = BRankingData.AM;

        // Get the next value for the Advert record
        sql = "SELECT designer_name, account_id, tournament_type_id, rank, points";
        sql += " FROM RANKING r, ACCOUNT a WHERE a.id = r.account_id AND tournament_type_id = '" + tournamentType;
        sql += "' ORDER BY rank";

        statement = con.createStatement ();
        rs = statement.executeQuery(sql);

        BDataSet dataSet = new BDataSet();

        while (rs.next())
        {
          BRankingData rankingData  = new BRankingData(rs.getLong("account_id"));
          rankingData.designerName  = rs.getString("designer_name");
          rankingData.ranking       = rs.getInt("rank");
          rankingData.points        = rs.getFloat("points");
          dataSet.add(rankingData);
        }

        rs.close();
        statement.close();

        return BMessage.ACK("RANK", dataSet);

      }
      catch (Exception e)
      {
        System.err.println("SGateway/GetAllRankingsHandler: " + e.getMessage());
        return BMessage.error("RANK", "SGateway/GetAllRankingsHandler: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/GetAllRankingsHandler");
        closeStatement(statement,"SGateway/GetAllRankingsHandler");
      }
    }
  }


  // Deletes the advert related to a tournament
  // -------------------------------------------------------------
  protected static class DeleteAdvertsHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "DADV"; }


    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        BMessage validation = super.validate(request,reqState);

        if (validation !=null)
          return validation;

        if (hasPermission("ADMIN",reqState.con, reqState.accountID))
          return null;
        else
          return BMessage.error(request.getType(), "No permissions for request");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
    }


    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      Connection con = reqState.con;

      Statement statement = null;
      String sql;

      try
      {

        BIdentData addID = (BIdentData)request.getData();

        // Get the next value for the Advert record
        sql = "DELETE FROM TOURNAMENT_ADVERT WHERE id = " + addID.getID();
        statement = con.createStatement ();
        statement.executeUpdate(sql);
        statement.close();

        return BMessage.ACK("DADV");

      }
      catch (Exception e)
      {
        System.err.println("SGateway/DeleteAdvertsHandler: " + e.getMessage());
        return BMessage.error("DADV", "SGateway/DeleteAdvertsHandler: " + e.getMessage());
      }
      finally
      {
        closeStatement(statement,"SGateway/DeleteAdvertsHandler");
      }
    }
  }

  // Get the BattleRecording request handler...
  // -------------------------------------------------------------
  protected static class BattleRecordingRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "GBA"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      byte[] battleRecording = null;
      byte[] stats = null;
      String sql;
      Statement statement = null;
      ResultSet rs = null;

      try
      {
        long fightID = ((BIdentData)request.getData()).getID();
        Connection con = reqState.con;
        long id = reqState.accountID;
        //long fightID = ((Long)requestObject).longValue();

        /*
        // Get details on the requested fight
        // HSC 18-Sept-00 Commented out two lines as on review the quesy doesn't need
        // to join to the table tournament
        sql = " SELECT te.account_id As account_1, te1.account_id As account_2";
        //sql = sql + " FROM TOURNAMENT_ENTRY te, TOURNAMENT_ENTRY te1, FIGHT f, TOURNAMENT t";
        sql = sql + " FROM TOURNAMENT_ENTRY te, TOURNAMENT_ENTRY te1, FIGHT f";
        sql = sql + " WHERE f.tournament_entry1_id = te.id";
        sql = sql + " AND f.tournament_entry2_id = te1.id";
        //sql = sql + " AND t.id = f.tournament_id";
        sql = sql + " AND f.id = " + fightID;
        */

        // Replaced above code with below code to prevent clients requesting battles that they
        // aren't allowed through hacking, ie fight's that aren't their own and aren't in the semi finals
        // or finals

        if (hasPermission("ADMIN",con,id))
        {
          // If the User has Admin permissions they can see any fights
          sql = "SELECT";
          sql += 	  " te.account_id As account_1,";
          sql += 		" te1.account_id As account_2";
          sql += 	" FROM";
          sql += 		" FIGHT f,";
          sql += 		" TOURNAMENT t,";
          sql += 		" TOURNAMENT_ENTRY te,";
          sql += 		" TOURNAMENT_ENTRY te1";
          sql += 	" WHERE";
          sql += 		" f.tournament_entry1_id = te.id";
          sql += 		" AND f.tournament_entry2_id = te1.id";
          sql += 		" AND t.id = f.tournament_id";
          sql += 		" AND f.id = " + fightID;
        }
        else
        {
          // Get only the final or semi-final fights and fights where the requesting account had bots entered
          sql = "SELECT";
          sql += 	  " te.account_id As account_1,";
          sql += 		" te1.account_id As account_2";
          sql += 	" FROM";
          sql += 		" FIGHT f,";
          sql += 		" TOURNAMENT t,";
          sql += 		" TOURNAMENT_ENTRY te,";
          sql += 		" TOURNAMENT_ENTRY te1";
          sql += 	" WHERE";
          sql += 		" f.tournament_entry1_id = te.id";
          sql += 		" AND f.tournament_entry2_id = te1.id";
          sql += 		" AND t.id = f.tournament_id";
          sql += 		" AND (t.status = 'FINISHED' OR t.status = 'FIGHTING' OR t.status = 'DELETED')";
          sql += 		" AND f.STATUS = 'FOUGHT'";
          sql += 		" AND f.id = " + fightID;
          sql += 		" AND (f.round >= (t.current_round-2) OR";
          // Following line selects any assigned featured fights
          sql +=    " (SELECT COUNT(*) FROM FEATURED_FIGHT WHERE fight_id = f.id AND featured_fight_type_id <> " + BFeaturedFightTypeData.UNASSIGNED + ") > 0 OR";
          sql += 		" EXISTS(";
          sql += 	   					" SELECT";
          sql += 								" f1.id";
          sql += 	 					" FROM";
          sql += 								" FIGHT f1,";
          sql += 								" TOURNAMENT_ENTRY te,";
          sql += 								" TOURNAMENT_ENTRY te2";
          sql += 	  					" WHERE";
          sql += 						    " f1.tournament_entry1_id = te.id";
          sql += 	 					    " AND f1.tournament_entry2_id   = te2.id";
          sql += 	   						" AND (te.account_id = " + id + " OR te2.account_id = " + id +")";
          sql += 	  						" AND f1.id = f.id";
          sql += 							"))";
        }

        statement = con.createStatement();
        rs = statement.executeQuery(sql);

        int botStatsIsFor = 0;

        if (rs.next())
        {
          int account1 = rs.getInt("account_1");
          int account2 = rs.getInt("account_2");

          rs.close();
          statement.close();

          if (id == account1)
          {
            botStatsIsFor = 1;
            sql = "SELECT battle_recording, battle_stats1 As stats";
            sql = sql + " FROM FIGHT";
            sql = sql + " WHERE id = " + fightID;

          }
          else if(id == account2)
          {
            botStatsIsFor = 2;
            sql = "SELECT battle_recording, battle_stats2 As stats";
            sql = sql + " FROM FIGHT";
            sql = sql + " WHERE id = " + fightID;
          }
          else
          {
            // The account requesting the fight has no bots in it so don't give any stats
            sql = "SELECT battle_recording";
            sql = sql + " FROM FIGHT";
            sql = sql + " WHERE id = " + fightID;
          }

          // Get the battleRecording and/or stats
          statement = con.createStatement();
          rs = statement.executeQuery(sql);
          rs.next();

          if (botStatsIsFor == 0)
          {
            // Get the recording And turn it into a byte array
            battleRecording = SServerUtility.blobToByteArray (((OracleResultSet)rs).getBLOB("battle_recording"));

            rs.close();
            statement.close();
          }
          else
          {
            // Get the recording And turn it into a byte array
            battleRecording = SServerUtility.blobToByteArray (((OracleResultSet)rs).getBLOB("battle_recording"));
            stats = SServerUtility.blobToByteArray (((OracleResultSet)rs).getBLOB("stats"));
            //System.out.println("***** Stats: " + stats.length);
            rs.close();
            statement.close();

          }

          BFightRecordData fightRecordData = new BFightRecordData(fightID);
          fightRecordData.fightRecording = battleRecording;
          fightRecordData.fightStats = stats;

          // Zip and encrypt the response...
          services.add(BServices.zip());
          //services.add(BServices.encrypt());

          return BMessage.ACK("GBA", fightRecordData);

          //System.err.println("** The Size of the Battle Recording from db is: " + Array.getLength(battleRecording));
        }
        else
        {
          System.err.println("SGateway/BattleRecordingRequestHandler/Couldn't find fight: " + fightID + " AccountID " + id);
          return BMessage.error("GBA", "Couldn't Find Fight");
        }
      }
      catch (Exception e)
      {
        System.err.println("SGateway/BattleRecordingRequestHandler: " + e.getMessage());
        return BMessage.error("GBA", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/BattleRecordingRequestHandler");
        closeStatement(statement,"SGateway/BattleRecordingRequestHandler");
      }

    }

  }


  // Get Open Tournaments request handler...
  // -------------------------------------------------------------

  protected static class OpenTournamentRequestHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "GOT"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      PreparedStatement getTournamentInfo = null;
      PreparedStatement getEnteredBots = null;
      ResultSet rs = null;
      ResultSet rs1 = null;
      ResultSet rs2 = null;

      try
      {
        Connection con = reqState.con;
        long id = reqState.accountID;

        // Get the current date and set the time to 0
        //tz = TimeZone.getDefault();

        //Calendar cal = Calendar.getInstance(tz);
        //cal.set(Calendar.HOUR,0);
        //cal.set(Calendar.MINUTE,0);
        //cal.set(Calendar.SECOND,0);
        //cal.set(Calendar.MILLISECOND,0);

        //long millis = cal.getTime().getTime();



        java.sql.Timestamp currentTimestamp = new java.sql.Timestamp(System.currentTimeMillis());

        //BDebug.print("System Date: " + currentTimestamp.toString());


        // Get the details of all the open tournaments
        sql = "SELECT * FROM TOURNAMENT_CLASSIFICATION WHERE close_date > ? AND status = 'OPEN' ORDER BY close_date";
        getTournamentInfo = con.prepareStatement(sql);
        getTournamentInfo.setTimestamp(1,currentTimestamp);
        rs =  getTournamentInfo.executeQuery();

        Vector tournamentData = new Vector();
        Vector tournamentEntryData = new Vector();

        while (rs.next())
        {

          long tournament_id = rs.getLong("id");

          // Find the number of tournament entries for each tournament
          sql = "SELECT COUNT(*) AS count FROM TOURNAMENT_ENTRY WHERE tournament_id = " + tournament_id;
          statement = con.createStatement();
          rs2 = statement.executeQuery(sql);
          rs2.next();

          int submissions = rs2.getInt("count");
          rs2.close();
          statement.close();


          BTournamentData tData = new BTournamentData(tournament_id);
          tData.name = rs.getString("name");

          Calendar cal = Calendar.getInstance(TimeZone.getDefault());
          cal.setTime(rs.getTimestamp("close_date"));

          //Timestamp timeStamp = rs.getTimestamp("close_date");

          //TimeZone tz = TimeZone.getDefault();

          //Calendar cal = Calendar.getInstance(tz);
          //cal.setTime(timeStamp);

          //tData.closingDate = rs.getTimestamp("close_date");
          tData.closingDate = cal;

          tData.tournamentType = rs.getString("tournament_type_id");

          tData.maximumEntries = rs.getInt("max_tournament_entries");
          tData.currentEntries = submissions;
          tData.fightDuration = rs.getInt("fight_duration");
          tData.entryFee = rs.getFloat("entry_fee");
          tData.description = rs.getString("description");
          tData.arena = rs.getString("arena_file_name");
          tData.botPriceCap = rs.getFloat("bot_price_cap");
          tData.allowedEntriesPerAccount = rs.getInt("max_entries_per_account");
          tData.statusCode = BTournamentData.OPEN;

          tData.sponsorImage = SServerUtility.blobToByteArray (((OracleResultSet)rs).getBLOB("sponsor_pic"));

          tournamentData.addElement(tData);

          // Determine if any bots for this account have been entered in this tournament
          sql = "SELECT id, bot_name, status, validation_notes FROM TOURNAMENT_ENTRY WHERE tournament_id = ? AND account_id = ?";
          getEnteredBots = con.prepareStatement(sql);
          getEnteredBots.setLong(1,tData.getID());
          getEnteredBots.setLong(2,id);
          rs1 = getEnteredBots.executeQuery();
          while (rs1.next())
          {
            BTournamentEntryData tEntryData = new BTournamentEntryData(rs1.getLong("id"));
            tEntryData.accountID = new Long((long)id);
            tEntryData.botName = rs1.getString("bot_name");
            tEntryData.tournamentID = tData.getIDLong();

            String status  = rs1.getString("status");

            // Assign the correct status
            if (status != null)
            {
              if (status.equals("Y"))
                tEntryData.statusCode = tEntryData.VALID;
              else if (status.equals("N"))
                tEntryData.statusCode = tEntryData.INVALID;
              else if (status.equals("P") || status.equals("B"))
                tEntryData.statusCode = tEntryData.VALIDATING;
              else
                tEntryData.statusCode = tEntryData.WAITING;
            }
            else
              tEntryData.statusCode = tEntryData.WAITING;

            tEntryData.validationNotes = rs1.getString("validation_notes");

            tournamentEntryData.addElement(tEntryData);
          }
          rs1.close();
          getEnteredBots.close();
        }

        rs1.close();
        getTournamentInfo.close();


        // Create the BDataSet and populate it
        BDataSet dataSet = new BDataSet();

        dataSet.add(tournamentData);
        dataSet.add(tournamentEntryData);

        services.add(BServices.zip());
        //services.add(BServices.encrypt());

        return BMessage.ACK("GOT", dataSet);
      }
      catch (Exception e)
      {
        System.err.println("SGateway/OpenTournamentRequestHandler: " + e);
        return BMessage.error("GOT", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/OpenTournamentRequestHandler");
        closeResultSet(rs1,"SGateway/OpenTournamentRequestHandler");
        closeResultSet(rs2,"SGateway/OpenTournamentRequestHandler");
        closeStatement(getEnteredBots,"SGateway/OpenTournamentRequestHandler");
        closeStatement(getTournamentInfo,"SGateway/OpenTournamentRequestHandler");
        closeStatement(statement,"SGateway/OpenTournamentRequestHandler");
      }
    }
  }


  private static boolean hasPermission(String permission, Connection con, long accountID)
  {

    Statement statement = null;
    ResultSet rs = null;
    try
    {
      String sql = "SELECT count(*) As count";
      sql += " FROM PERMISSION P, ACCOUNT_PERMISSION AP";
      sql += " WHERE AP.permission_id = P.id";
      sql += " AND P.permission = '" + permission + "'";
      sql += " AND AP.account_ID = " + accountID;

      statement = con.createStatement();

      rs = statement.executeQuery(sql);
      rs.next();
      int count = rs.getInt("count");
      rs.close();
      statement.close();

      if (count !=0)
      {
        // Has the required permission
        return true;
      }
      else
      {
        // Doesn't have the required permission
        return false;
      }
    }
    catch (Exception e)
    {
      System.err.println(e.getMessage());
      return false;
    }
    finally
    {
      closeResultSet(rs,"SGateway/hasPermission");
      closeStatement(statement,"SGateway/hasPermission");
    }

  }

}



