package roboforge;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import oracle.sql.*;
import oracle.jdbc.driver.*;
import java.awt.event.*;
import javax.swing.*;

import roboforge.net.data.*;

public class STournamentController extends SHttpServletJXGB implements ActionListener  {

  /*

  * Description:
    Schedules the rounds, and creates fights
    NOTE VERY IMPORTANT Do not allow multiple instances of this thread to be run (ie Allow multiple instances
    setting in the Application Server/Servlet Server should be disallowed for this Servlet)
  * Example:
  * Modification History ( Name, Date, Notes ):
    Created HSC, 8-Feb-2000

    Took Out: implementation of SingleThreadModel
  */

  Connection con = null;
  javax.swing.Timer t;
  int maxFightTime;
  int maxValidationTime;
  long roboforgeAccountID;
  boolean purge = false;

  //Initialize global variables
  public void init(ServletConfig config) throws ServletException
  {
    super.init(config);
    
    t = new javax.swing.Timer(10000,this);
    Connection con1 = null;
    try
    {
      con1 = getDatabaseConnection();
      // Set the class variables
      String sql;

      sql = "SELECT fight_time, validation_time FROM CONSTANTS";

      Statement statement = con1.createStatement();
      ResultSet rs = statement.executeQuery(sql);

      rs.next();

      maxFightTime = rs.getInt("fight_time");
      maxValidationTime = rs.getInt("validation_time");

      rs.close();
      statement.close();


      //  Get the Roboforge Account id
      sql = "SELECT id FROM ACCOUNT WHERE loginID = 'RoboForge'";
      statement = con1.createStatement();
      rs = statement.executeQuery(sql);

      if (rs.next())
        roboforgeAccountID = rs.getLong("id");

      rs.close();
      statement.close();
    }

    catch (Exception e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      try
      {
        if (con1!= null)
        {
          //Close the connection no matter what
          myBroker.freeConnection(con1);
        }

      }
      catch(Exception sqle1)
      {
        System.err.println("STournamentController/Init: " + sqle1.getMessage());
      }
    }
  }

  //Service the request
  public void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
  {
   SResponseWrapper responseWrapper = null;


    try
    {
      // First get the Request object from the stream
      ObjectInputStream ois = new ObjectInputStream(request.getInputStream());
      SRequestWrapper requestWrapper = (SRequestWrapper)ois.readObject();

      // Determine if the password is ok
      if (requestWrapper.getPassword().equals("bkd835gbb$*(KJnmcz"))
      {
        if (requestWrapper.getRequestType().equals("STA"))
        {
          if (t.isRunning())
          {
            // Tournament Server is already running
            responseWrapper = new SResponseWrapper("ERR","Tournament Server Already Running");
          }
          else
          {
            // Tournament Server started
            responseWrapper = new SResponseWrapper("STA",null);
            t.start();
          }
        }
        else if (requestWrapper.getRequestType().equals("STO"))
        {
          // Request the Service to stop
          responseWrapper = new SResponseWrapper("STO",null);
          t.stop();
        }
        else if (requestWrapper.getRequestType().equals("PURG"))
        {
          // Set the Service to start the Purge Job
          purge = true;
          responseWrapper = new SResponseWrapper("PURG",null);
        }
        else
        {
          // Unknown Request Type
          responseWrapper = new SResponseWrapper("ERR","Unknown Request");
        }
      }
      else
      {
        // Password invalid
        responseWrapper = new SResponseWrapper("ERR","Password Invalid");
      }

      response.setContentType("application/octet-stream");
      ObjectOutputStream oos = new ObjectOutputStream(response.getOutputStream());
      oos.writeObject(responseWrapper);
      oos.flush();
      oos.close();

    }
    catch (ClassNotFoundException cnfe)
    {
      responseWrapper = new SResponseWrapper("ERR",(Object)cnfe.getMessage());
      System.err.println("STournamentController/Service: " + cnfe.getMessage());
    }
    catch (Exception e)
    {
      responseWrapper = new SResponseWrapper("ERR",(Object)e.getMessage());
      System.err.println("STournamentController/Service: " + e.getMessage());
    }
  }

  public void actionPerformed (ActionEvent ae)
  {

    try
    {
      con = getDatabaseConnection();

        // *** THE ORDER IN WHICH THESE METHOD CALLS ARE PERFORMED IS IMPORTANT ***
        if (purge)
        {
          purge = false;
          purgeTournaments(con);
        }
        setOpenToPending(con);
        setPendingToFighting(con);
        assignFights(con);
        invalidateFights(con);
        invalidateBots(con);
        reassignFights(con);
        reassignValidation(con);
        setWaitingToOpen(con);
    }

    catch (Exception e)
    {
      System.err.println("STournamentController/actionPerformed: " + e.getMessage());
    }
    finally
    {
      try
      {
        if (con!= null)
        {
          //Close the connection no matter what
          //con.close();
          myBroker.freeConnection(con);
        }

      }
      catch(Exception sqle1)
      {
        System.err.println("STournamentController/actionPerformed: " + sqle1.getMessage());
      }
    }
  }

  private void invalidateFights(Connection con)
  {
    /*
      Locates all the fights that have taken too long more than once and
      set them to invalid.
    */

    Statement statement = null;

    try
    {
      //System.out.print("*** invalidateFights ***");

      con.setAutoCommit(false);

      String sql = "UPDATE FIGHT F SET status = 'BAD'";
      sql += " WHERE status = 'FIGHTING'";
      sql += " AND 2 <= (SELECT Count(*)";
      sql += " FROM FIGHT_CLIENT_LOG FCL";
      sql += " WHERE FCL.fight_id = F.id";
      sql += " AND date_time_finish IS NULL";
      sql += " AND ((sysDate - date_time_start)* 86400) > " + maxFightTime + ")";


      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      con.commit();
      con.setAutoCommit(true);
    }
    catch (Exception e)
    {
      try
      {
        con.rollback();
        con.setAutoCommit(true);
      }
      catch (SQLException sqle1)
      {
        System.err.println("STournamentController/invalidateFights: " + sqle1.getMessage());
      }
      
      System.err.println("STournamentController/invalidateFights: " + e.getMessage());
    }
    finally
    {
      closeStatement(statement,"STournamentController/invalidateFights");
    }


  }
  private void invalidateBots(Connection con)
  {
    /*
      Locates all the bot validations that have taken too long more than once and
      set them to invalid.
    */

    Statement statement = null;
    try
    {
      //System.out.print("*** invalidateBots ***");

      con.setAutoCommit(false);

      String sql = "UPDATE TOURNAMENT_ENTRY TE SET status = 'BAD'";
      sql += " WHERE status = 'P'";
      sql += " AND 2 <= (SELECT Count(*)";
      sql += " FROM FIGHT_CLIENT_LOG FCL";
      sql += " WHERE FCL.tournament_entry_id = TE.id";
      sql += " AND date_time_finish IS NULL";
      sql += " AND ((sysDate - date_time_start)* 86400) > " + maxValidationTime + ")";

      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      con.commit();
      con.setAutoCommit(true);
    }
    catch (Exception e)
    {
      try
      {
        con.rollback();
        con.setAutoCommit(true);
      }
      catch (SQLException sqle1)
      {
        System.err.println("STournamentController/invalidateBots: " + sqle1.getMessage());
      }
      
      System.err.println("STournamentController/invalidateBots: " + e.getMessage());
    }
    finally
    {
      closeStatement(statement,"STournamentController/invalidateBots");
    }


  }

  private void reassignFights(Connection con)
  {
    /*
      Locates all the fights that are taking too long and puts them back in the
      pool so another fight client can attempt to run the fight
    */

    Statement statement = null;

    try
    {
      //System.out.print("*** reassignFights ***");

      con.setAutoCommit(false);

      // If a fight fighting on a farm client that has taken longer then the
      // allowed time, and there is no other farm client running this fight then
      // reset the fight to allow another farm client an attempt at running
      // the fight in the allowed time

      String sql = "UPDATE FIGHT F SET status = 'WAITING', fight_client_id = NULL";
      sql += " WHERE status = 'FIGHTING'";
      sql += " AND 1 = (SELECT Count(*)";
      sql += " FROM FIGHT_CLIENT_LOG FCL";
      sql += " WHERE FCL.fight_id = F.id";
      sql += " AND date_time_finish IS NULL";
      sql += " AND ((sysDate - date_time_start)* 86400) > " + maxFightTime + ")";

      // This section ensures that even if one client has taken longer that allowed
      // in running the fight, the the fight isn't reset
      // if another client is attempting to run it
      sql += " AND 1 = (SELECT Count(*)";
      sql += " FROM FIGHT_CLIENT_LOG FCL";
      sql += " WHERE FCL.fight_id = F.id";
      sql += " AND date_time_finish IS NULL)";


      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      con.commit();
      con.setAutoCommit(true);
    }
    catch (Exception e)
    {
      try
      {
        con.rollback();
        con.setAutoCommit(true);
      }
      catch (SQLException sqle1)
      {
        System.err.println("STournamentController/reassignFights: " + sqle1.getMessage());
      }
      
      System.err.println("STournamentController/reassignFights: " + e.getMessage());
    }
    finally
    {
      closeStatement(statement,"STournamentController/reassignFights");
    }


  }

  private void updateAccountRankings(Connection con, long tournamentID) throws Exception
  {
    /*
      If the tournament that has just finished is of a ranking type then update all
      the ranking data
    */

    Statement statement = null;
    ResultSet rs = null;

    Statement statement1 = null;

    String sql;

    try
    {
      sql = "SELECT COUNT(c.tournament_type_id) AS count FROM tournament t, classification c";
      sql += " WHERE t.classification_id = c.id AND (c.tournament_type_id = 'PR' OR c.tournament_type_id = 'AR')";
      sql += " AND t.id = " + tournamentID;

      //BDebug.print(sql);

      statement = con.createStatement();
      rs = statement.executeQuery(sql);

      rs.next();
      int count = rs.getInt("count");

      rs.close();
      statement.close();

      if (count == 0)
      {
        // Tournament not of ranking type so nothing to update
        return;
      }

      // Set all the professional and amateur ranking points to NULL
      sql = "UPDATE account SET prof_ranking_points = NULL, prof_ranking = NULL, amateur_ranking_points = NULL, amateur_ranking = NULL";
      sql += " WHERE account.status = 'OPEN'";

      //BDebug.print(sql);
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      // Set all the master states to null
      sql = "UPDATE account SET professional_status = NULL WHERE professional_status = 'M'";
      //BDebug.print(sql);
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      // Delete everything from the ranking table
      sql = "DELETE FROM ranking";
      //BDebug.print(sql);
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      sql = "UPDATE ACCOUNT a SET prof_ranking_points =";
      sql +="                      (SELECT";
      sql +="                                SUM(points_given)";
      sql +="                       FROM";
      sql +="                                tournament_entry te,";
      sql +="                                tournament t,";
      sql +="                                classification c,";
      sql +="                                ranking_points rp";
      sql +="                       WHERE";
      sql +="                                c.tournament_type_id = 'PR' AND";
      sql +="                                t.type = 'INTFC' AND";
      sql +="                                (t.status = 'FINISHED' OR t.status = 'DELETED') AND";
      sql +="					                       t.close_date BETWEEN (SYSDATE - 57) AND SYSDATE AND";
      sql +="                                te.account_id = a.id AND";
      sql +="                                te.tournament_id = t.id AND";
      sql +="                                t.classification_id = c.id AND";
      sql +="                                c.tournament_type_id = rp.tournament_type_id AND";
      sql +="                                rp.level_from_top = (t.current_round - te.highest_round) + 1)";
      sql +=" WHERE a.status = 'OPEN' AND a.id <> " + roboforgeAccountID;
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      sql ="UPDATE ACCOUNT a SET amateur_ranking_points =";
      sql +="                        (SELECT";
      sql +="                                SUM(points_given)";
      sql +="                         FROM";
      sql +="                                tournament_entry te,";
      sql +="                                tournament t,";
      sql +="                                classification c,";
      sql +="                                ranking_points rp";
      sql +="                         WHERE";
      sql +="                                c.tournament_type_id = 'AR' AND";
      sql +="                                t.type = 'INTFC' AND";
      sql +="                                (t.status = 'FINISHED' OR t.status = 'DELETED') AND";
      sql +="					                       t.close_date BETWEEN (SYSDATE - 57) AND SYSDATE AND";
      sql +="                                te.account_id = a.id AND";
      sql +="                                te.tournament_id = t.id AND";
      sql +="                                t.classification_id = c.id AND";
      sql +="                                c.tournament_type_id = rp.tournament_type_id AND";
      sql +="                                rp.level_from_top = (t.current_round - te.highest_round) + 1)";
      sql +=" WHERE a.status = 'OPEN' AND a.id <> " + roboforgeAccountID;
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      sql ="UPDATE ACCOUNT a SET total_prof_fights   =";
      sql +="                          (SELECT";
      sql +="                                COUNT(f.id)";
      sql +="                           FROM";
      sql +="                                tournament_entry te,";
      sql +="                                tournament t,";
      sql +="					                       fight f,";
      sql +="                                classification c";
      sql +="                           WHERE";
      sql +="                                c.tournament_type_id = 'PR' AND";
      sql +="                                t.type = 'INTFC' AND";
      sql +="                                (t.status = 'FINISHED' OR t.status = 'DELETED') AND";
      sql +="                                te.account_id = a.id AND";
      sql +="                                te.tournament_id = t.id AND";
      sql +="                                t.classification_id = c.id AND";
      sql +="					                       f.tournament_id = t.id AND";
      sql +="					                       (f.tournament_entry1_id = te.id OR f.tournament_entry2_id = te.id))";
      sql +=" WHERE a.status = 'OPEN' AND a.id <> " + roboforgeAccountID;
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      sql ="UPDATE ACCOUNT a SET total_amateur_fights   =";
      sql +="                           (SELECT";
      sql +="                                COUNT(f.id)";
      sql +="                            FROM";
      sql +="                                 tournament_entry te,";
      sql +="                                 tournament t,";
      sql +="					                        fight f,";
      sql +="                                 classification c";
      sql +="                            WHERE";
      sql +="                                 c.tournament_type_id = 'AR' AND";
      sql +="                                 t.type = 'INTFC' AND";
      sql +="                                 (t.status = 'FINISHED' OR t.status = 'DELETED') AND";
      sql +="                                 te.account_id = a.id AND";
      sql +="                                 te.tournament_id = t.id AND";
      sql +="                                 t.classification_id = c.id AND";
      sql +="					                        f.tournament_id = t.id AND";
      sql +="					                        (f.tournament_entry1_id = te.id OR f.tournament_entry2_id = te.id))";
      sql +=" WHERE a.status = 'OPEN' AND a.id <> " + roboforgeAccountID;
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      sql ="UPDATE ACCOUNT a SET total_prof_fights_won  =";
      sql +="                            (SELECT";
      sql +="                                 COUNT(f.id)";
      sql +="                             FROM";
      sql +="                                 tournament_entry te,";
      sql +="                                 tournament t,";
      sql +="					                        fight f,";
      sql +="                                 classification c";
      sql +="                             WHERE";
      sql +="                                 c.tournament_type_id = 'PR' AND";
      sql +="                                 t.type = 'INTFC' AND";
      sql +="                                 (t.status = 'FINISHED' OR t.status = 'DELETED') AND";
      sql +="                                 te.account_id = a.id AND";
      sql +="                                 te.tournament_id = t.id AND";
      sql +="                                 t.classification_id = c.id AND";
      sql +="					                        f.tournament_id = t.id AND";
      sql +="					                        f.result = te.id)";
      sql +=" WHERE a.status = 'OPEN' AND a.id <> " + roboforgeAccountID;
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      sql ="UPDATE ACCOUNT a SET total_amateur_fights_won  =";
      sql +="                             (SELECT";
      sql +="                                 COUNT(f.id)";
      sql +="                             FROM";
      sql +="                                 tournament_entry te,";
      sql +="                                 tournament t,";
      sql +="					                        fight f,";
      sql +="                                 classification c";
      sql +="                             WHERE";
      sql +="                                 c.tournament_type_id = 'AR' AND";
      sql +="                                 t.type = 'INTFC' AND";
      sql +="                                 (t.status = 'FINISHED' OR t.status = 'DELETED') AND";
      sql +="                                 te.account_id = a.id AND";
      sql +="                                 te.tournament_id = t.id AND";
      sql +="                                 t.classification_id = c.id AND";
      sql +="					                        f.tournament_id = t.id AND";
      sql +="					                        f.result = te.id)";
      sql +=" WHERE a.status = 'OPEN' AND a.id <> " + roboforgeAccountID;
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      sql ="UPDATE ACCOUNT a SET total_prof_tourns_won  =";
      sql +="                             (SELECT";
      sql +="                                 COUNT(t.id)";
      sql +="                              FROM";
      sql +="                                 tournament_entry te,";
      sql +="                                 tournament t,";
      sql +="                                 classification c";
      sql +="                              WHERE";
      sql +="                                 c.tournament_type_id = 'PR' AND";
      sql +="                                 t.type = 'INTFC' AND";
      sql +="                                 (t.status = 'FINISHED' OR t.status = 'DELETED') AND";
      sql +="                                 te.account_id = a.id AND";
      sql +="                                 te.tournament_id = t.id AND";
      sql +="                                 t.classification_id = c.id AND";
      sql +="					                        te.highest_round = t.current_round)";
      sql +=" WHERE a.status = 'OPEN' AND a.id <> " + roboforgeAccountID;
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      sql ="UPDATE ACCOUNT a SET	total_amateur_tourns_won  =";
      sql +="                              (SELECT";
      sql +="                                 COUNT(t.id)";
      sql +="                              FROM";
      sql +="                                 tournament_entry te,";
      sql +="                                 tournament t,";
      sql +="                                 classification c";
      sql +="                              WHERE";
      sql +="                                 c.tournament_type_id = 'AR' AND";
      sql +="                                 t.type = 'INTFC' AND";
      sql +="                                 (t.status = 'FINISHED' OR t.status = 'DELETED') AND";
      sql +="                                 te.account_id = a.id AND";
      sql +="                                 te.tournament_id = t.id AND";
      sql +="                                 t.classification_id = c.id AND";
      sql +="					                        te.highest_round = t.current_round)";
      sql +=" WHERE a.status = 'OPEN' AND a.id <> " + roboforgeAccountID;

      //BDebug.print(sql);
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      // Update the Professional Ranking
      sql = "SELECT prof_ranking_points points, COUNT(prof_ranking_points) count";
      sql += " FROM account a";
      sql += " WHERE prof_ranking_points > 0";
      sql += " GROUP BY prof_ranking_points";
      sql += " ORDER BY prof_ranking_points desc";

      //BDebug.print(sql);
      statement = con.createStatement();
      rs = statement.executeQuery(sql);

      int rankingCount = 1;

      while (rs.next())
      {
        int points = rs.getInt("points");

        sql = "UPDATE account SET prof_ranking = " + rankingCount;
        sql +=" WHERE prof_ranking_points = " + points;

        //BDebug.print(sql);
        statement1 = con.createStatement();
        statement1.executeUpdate(sql);
        statement1.close();

        rankingCount = rankingCount + rs.getInt("count");
      }

      rs.close();
      statement.close();

      sql = "UPDATE account SET professional_status = 'M'";
      sql +=" WHERE prof_ranking_points > 0 ";
      sql +=" AND ((professional_status <> 'GM' AND professional_status <> 'AM') OR professional_status IS NULL)";

      //BDebug.print(sql);
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();


      // Update the Ranking Table
      sql = "INSERT INTO ranking (account_id, tournament_type_id, rank, points)";
      sql+= " SELECT id, 'PR', prof_ranking, prof_ranking_points FROM account WHERE prof_ranking IS NOT NULL";

      //BDebug.print(sql);
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();


      // Update the Amateur Ranking
      sql = "SELECT amateur_ranking_points points, COUNT(amateur_ranking_points) count";
      sql += " FROM account a";
      sql += " WHERE amateur_ranking_points > 0";
      sql += " GROUP BY amateur_ranking_points";
      sql += " ORDER BY amateur_ranking_points desc";

      //BDebug.print(sql);
      statement = con.createStatement();
      rs = statement.executeQuery(sql);

      rankingCount = 1;

      while (rs.next())
      {
        sql = "UPDATE account SET amateur_ranking = " + rankingCount;
        sql +=" WHERE amateur_ranking_points = " + rs.getInt("points");

        //BDebug.print(sql);
        statement1 = con.createStatement();
        statement1.executeUpdate(sql);
        statement1.close();

        rankingCount = rankingCount + rs.getInt("count");
      }

      rs.close();
      statement.close();


      // Update the Ranking Table
      sql = "INSERT INTO ranking (account_id, tournament_type_id, rank, points)";
      sql+= " SELECT id, 'AR',amateur_ranking, amateur_ranking_points FROM account WHERE amateur_ranking IS NOT NULL";

      //BDebug.print(sql);
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

    }
    catch (Exception e)
    {
      System.err.println("STournamentController/updateAccountRankings: " + e.getMessage());
      throw new Exception(e.getMessage());
    }
    finally
    {
      closeResultSet(rs,"STournamentController/updateAccountRankings");
      closeStatement(statement,"STournamentController/updateAccountRankings");
      closeStatement(statement1,"STournamentController/updateAccountRankings");
    }
  }

  private void allocatePrizes(Connection con, long tournamentID) throws Exception
  {
    /*
      If the tournament that has just finished is of a Professional type
      then allocate the prize money
    */

    Statement statement = null;
    ResultSet rs = null;

    Statement statement2 = null;

    String sql;

    try
    {
      // Only select the Professional tournaments

      sql = "SELECT COUNT(c.tournament_type_id) AS count FROM tournament t, classification c";
      sql += " WHERE t.classification_id = c.id AND (c.tournament_type_id = 'PR' OR c.tournament_type_id = 'PN')";
      sql += " AND t.id = " + tournamentID;

      //BDebug.print(sql);

      statement = con.createStatement();
      rs = statement.executeQuery(sql);

      rs.next();
      int count = rs.getInt("count");

      rs.close();
      statement.close();

      if (count == 0)
      {
        // Tournament not of professional type so nothing to update
        return;
      }

      sql = " SELECT te.account_id,te.bot_name, pm.amount";
      sql += " FROM";
      sql +=  " tournament_entry te,";
      sql +=  " tournament t,";
      sql +=  " classification c,";
      sql +=  " prize_money pm";
      sql += " WHERE";
		  sql +=  " t.id = " + tournamentID;
		  sql +=  " AND (c.tournament_type_id = 'PR' OR c.tournament_type_id = 'PN')";
      sql +=  " AND t.type = 'INTFC'";
      sql +=  " AND t.status = 'FINISHED'";
      sql +=  " AND te.tournament_id = t.id";
      sql +=  " AND t.classification_id = c.id";
      sql +=  " AND c.id = pm.classification_id";
      sql +=  " AND pm.level_from_top = (t.current_round - te.highest_round) + 1";
      sql +=  " AND te.account_id <> " + roboforgeAccountID;

      statement = con.createStatement();
      rs = statement.executeQuery(sql);

      // Loop through all the winners creating transactions and
      // Updating balances

      while(rs.next())
      {

        long id         = rs.getLong("account_id");
        String bot_name = rs.getString("bot_name");
        float amount    = rs.getFloat("amount");

        // Create the transaction record
        sql = "INSERT INTO TRANSACTION (account_id, type, details, amount, status, tournament_id, account_from, bot_name)";
        sql += "VALUES (" + id + ",'WIN','Winnings Payment'," + amount + ",'C',"+tournamentID+",'W','" + bot_name + "')";
        statement2 = con.createStatement();
        statement2.executeUpdate(sql);
        statement2.close();

        // Add the amount from the account
        sql = "UPDATE ACCOUNT SET winnings_balance = nvl(winnings_balance,0) + " + amount + " WHERE id = " + id;
        statement2 = con.createStatement();
        statement2.executeUpdate(sql);
        statement2.close();

      }
      rs.close();
      statement.close();
    }
    catch (Exception e)
    {
      System.err.println("STournamentController/allocatePrizes: " + e.getMessage());
      throw new Exception(e.getMessage());
    }
    finally
    {
      closeResultSet(rs,"STournamentController/allocatePrizes");
      closeStatement(statement,"STournamentController/allocatePrizes");
      closeStatement(statement2,"STournamentController/allocatePrizes");
    }
  }

 private void reassignValidation(Connection con)
  {
    /*
      Locates all the bot validations that are taking too long and puts them back
      in the pool so another farm client can attempt to validate them
    */

    Statement statement = null;

    try
    {
      //System.out.print("*** reassignValidation ***");

      con.setAutoCommit(false);

      // If a tournament entry has a validation by a farm client that has taken longer then the
      // allowed time, and there is no other farm client validating this tournament entry then
      // set the status to null to allow another farm client an attempt at validating
      // the tournament entry

      String sql = "UPDATE TOURNAMENT_ENTRY TE SET status = NULL";
      sql += " WHERE status = 'P'";
      sql += " AND 1 = (SELECT Count(*)";
      sql += " FROM FIGHT_CLIENT_LOG FCL";
      sql += " WHERE FCL.tournament_entry_id = TE.id";
      sql += " AND date_time_finish IS NULL";
      sql += " AND ((sysDate - date_time_start)* 86400) > " + maxValidationTime + ")";

      // This section ensures that even if one client has taken longer that allowed
      // in validating the tournament entry, the status isn't set to null
      // if another client is attempting to validate it
      sql += " AND 1 = (SELECT Count(*)";
      sql += " FROM FIGHT_CLIENT_LOG FCL";
      sql += " WHERE FCL.tournament_entry_id = TE.id";
      sql += " AND date_time_finish IS NULL)";

      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      con.commit();
      con.setAutoCommit(true);
    }
    catch (Exception e)
    {
      try
      {
        con.rollback();
        con.setAutoCommit(true);
      }
      catch (SQLException sqle1)
      {
        System.err.println("STournamentController/reassignValidation: " + sqle1.getMessage());
      }
      
      System.err.println("STournamentController/reassignValidation: " + e.getMessage());
    }
    finally
    {
      closeStatement(statement,"STournamentController/reassignValidation");
    }
  }

  private void setOpenToPending(Connection con)
  {
    /*
      Finds all the OPEN tournaments where the close date is less than the
      current date and sets them to PENDING
    */

    Statement statement = null;

    try
    {
      //System.out.print("*** setOpenToPending ***");

      con.setAutoCommit(false);

      String sql = "UPDATE TOURNAMENT SET status = 'PENDING' WHERE status = 'OPEN' AND close_date < SYSDATE";
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      con.commit();
      con.setAutoCommit(true);
    }
    catch (Exception e)
    {
      try
      {
        con.rollback();
        con.setAutoCommit(true);
      }
      catch (SQLException sqle1)
      {
        System.err.println("STournamentController/setOpenToPending: " + sqle1.getMessage());
      }

      System.err.println("STournamentController/setOpenToPending: " + e.getMessage());
    }
    finally
    {
      closeStatement(statement,"STournamentController/setOpenToPending");
    }

  }

  private void purgeTournaments(Connection con)
  {
    /*
      Finds all the OPEN tournaments where the close date is less than the
      current date and sets them to PENDING
    */
      String sql;
      Statement statement = null;
      Statement statement1 = null;
      ResultSet rs = null;
      try
      {
        con.setAutoCommit(false);

        System.err.println("Starting Purge: " + System.currentTimeMillis());

        // Find all the tournaments to be set to deleted status

        sql = "SELECT id FROM TOURNAMENT t";
        sql += " WHERE type = 'INTFC'";
        sql += " AND status = 'FINISHED'";
        sql += " AND t.id NOT IN (";
        sql +=  		" SELECT id FROM";
        sql +=  		" (SELECT *";
        sql +=  		" FROM TOURNAMENT";
        sql +=  		" WHERE (status = 'FINISHED') AND type = 'INTFC' ORDER BY id DESC)";
        sql +=  		" WHERE ROWNUM <= 7)";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);

        while (rs.next())
        {
          long tournamentID = rs.getLong("id");

          // Set the tournament Status to Deleted
          sql =   "UPDATE TOURNAMENT t SET status = 'DELETED'";
          sql += " WHERE id = " + tournamentID;

          statement1 = con.createStatement();
          statement1.executeUpdate(sql);
          statement1.close();

          // Clear all the related tournament entry bot files
          sql = "UPDATE TOURNAMENT_ENTRY SET BOT_FILE = NULL";
          sql += " WHERE tournament_id = " + tournamentID;

          statement1 = con.createStatement();
          statement1.executeUpdate(sql);
          statement1.close();

          // Delete all the log files related to the tournament_entry
          sql = "DELETE FROM FIGHT_CLIENT_LOG fcl ";
          sql += " WHERE EXISTS (SELECT id FROM TOURNAMENT_ENTRY te";
          sql += " WHERE te.id = fcl.tournament_entry_id AND tournament_id = " + tournamentID + ")";

          statement1 = con.createStatement();
          statement1.executeUpdate(sql);
          statement1.close();

          // Clean up the fights
          sql = "UPDATE FIGHT fgt ";
	        sql += " SET battle_recording = NULL,";
	        sql += " battle_stats1 = NULL,";
	        sql += " battle_stats2 = NULL,";
	        sql += " status = 'DELETED'";
          sql += " WHERE EXISTS";
	        sql +=      " (SELECT f.id";
	        sql +=      " FROM FIGHT f, TOURNAMENT t";
	        sql +=      " WHERE  fgt.id = f.id";
	        sql +=      " AND f.STATUS = 'FOUGHT'";
	        sql +=      " AND t.STATUS = 'DELETED'";
	        sql +=      " AND f.tournament_id = t.id";
          // Is not a final of semi-final
	        sql +=      " AND f.round < t.current_round -2";
          // The related tournament isn't in the last 7 tournaments
	        sql +=      " AND t.id = " + tournamentID;
          // The fight isn't an all time great
	        sql +=   " AND f.id NOT IN (SELECT fight_id";
			    sql +=   " FROM FEATURED_FIGHT";
			    sql += " WHERE featured_fight_type_id = " + BFeaturedFightTypeData.ALL_TIME_GREAT + "))";


          statement1 = con.createStatement();
          statement1.executeUpdate(sql);
          statement1.close();

          // Delete all the log files releated to the fight
          sql = "DELETE FROM FIGHT_CLIENT_LOG fcl ";
          sql += " WHERE EXISTS (SELECT id FROM FIGHT f";
          sql += " WHERE f.id = fcl.fight_id AND tournament_id = " + tournamentID + ")";

          statement1 = con.createStatement();
          statement1.executeUpdate(sql);
          statement1.close();
        }

        rs.close();
        statement.close();

        // Also delete all the Featured Fights that aren't all time greats
        // that have the status "DELETED"

        sql = "DELETE FROM FEATURED_FIGHT fght";
        sql += " WHERE fght.fight_id IN";
	      sql += " (SELECT ff.fight_id FROM FEATURED_FIGHT ff, FIGHT f";
	      sql += " WHERE";
        sql += " ff.featured_fight_type_id <> " + BFeaturedFightTypeData.ALL_TIME_GREAT;
        sql += " AND ff.fight_id = f.id";
        sql += " AND f.status = 'DELETED')";
        sql += " AND fght.featured_fight_type_id <> " + BFeaturedFightTypeData.ALL_TIME_GREAT;

        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        con.setAutoCommit(true);
        con.commit();

        System.err.println("Ending Purge: " + System.currentTimeMillis());

      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
          con.setAutoCommit(true);
        }
        catch (SQLException sqle1)
        {
          System.err.println("STournamentController/assignFights: " + sqle1);
        }

        System.err.println("STournamentController/assignFights: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"STournamentController/assignFights:");
        closeStatement(statement,"STournamentController/assignFights:");
        closeStatement(statement1,"STournamentController/assignFights:");
      }


  }

  private void setWaitingToOpen(Connection con)
  {
    /*
      Finds all the Waiting tournaments where the open date is less than the
      current date and sets them to OPEN
    */

    Statement statement = null;

    try
    {
      //System.out.print("*** setOpenToPending ***");

      con.setAutoCommit(false);

      String sql = "UPDATE TOURNAMENT SET status = 'OPEN' WHERE status = 'WAITING' AND open_date < SYSDATE";
      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      con.commit();
      con.setAutoCommit(true);
    }
    catch (Exception e)
    {
      try
      {
        con.rollback();
        con.setAutoCommit(true);
      }
      catch (SQLException sqle1)
      {
        System.err.println("STournamentController/setWaitingToOpen: " + sqle1.getMessage());
      }

      System.err.println("STournamentController/setWaitingToOpen: " + e.getMessage());
    }
    finally
    {
      closeStatement(statement,"STournamentController/setWaitingToOpen");
    }
  }

  private void setPendingToFighting(Connection con)
  {
    /*
     If any PENDING tournaments are ready - there are at least two valid entries and all entries have
     been validated (ie are either valid of invalid) - then set the status to fighting.

     08/08/00 HSC Added condition  that the tournament flag auto_fight must = 'Y' before
     any of the below updates are performed.

     30/08/00 HSC Added 'BAD' to one of the torunament entry states that will prevent the tournament from proceeding
    */

    Statement statement = null;

    try
    {
       //System.out.print("*** setPendingToFighting ***");

       con.setAutoCommit(false);
      //String sql;
      //Statement statement;

      // OK the following SQL is a little involved so I'll explain it as I build it

      // SUMMARY:  It updates the sort sequence for all valid tournament entries that meet
      // criteria as defined below

      // Update the tournament entry sort_sequence to a random number generated by the Java
      // stored procedure random
      String sql = "UPDATE TOURNAMENT_ENTRY te1 SET te1.sort_sequence = random";
      sql = sql + " WHERE EXISTS";

      // Interelated Sub Query to select the tournament entry id based on conditions
      // including the results of two other interelated Sub Queries
      sql = sql + " (";
      sql = sql + " SELECT te.id";
      sql = sql + " FROM TOURNAMENT t, TOURNAMENT_ENTRY te";
      sql = sql + " WHERE t.id = te.tournament_id";
      sql = sql + " AND te1.id = te.id";

      // Ensures that for the current tournament_entry being looked at that the related
      // tournament doesn't have any entries that are either Null or P i.e haven't been
      // validated or are in the process of being validated
      sql = sql + " AND (SELECT COUNT(*)";
      sql = sql + " FROM TOURNAMENT, TOURNAMENT_ENTRY";
      sql = sql + " WHERE TOURNAMENT.id = TOURNAMENT_ENTRY.tournament_id";
      sql = sql + " AND TOURNAMENT.id = t.id";

      // Only update if the tournament Auto_Fight field = 'Y'
      sql = sql + " AND TOURNAMENT.auto_fight = 'Y'";

      sql = sql + " AND (TOURNAMENT_ENTRY.status IS NULL OR TOURNAMENT_ENTRY.status = 'P' OR TOURNAMENT_ENTRY.status = 'BAD')) = 0";

      // Ensures that for the current tournament_entry being looked at that the related
      // tournament has more than one valid entry
      sql = sql + " AND (SELECT COUNT(*)";
      sql = sql + " FROM TOURNAMENT, TOURNAMENT_ENTRY";
      sql = sql + " WHERE TOURNAMENT.id = TOURNAMENT_ENTRY.tournament_id";
      sql = sql + " AND TOURNAMENT.id = t.id";
      sql = sql + " AND TOURNAMENT_ENTRY.status = 'Y') > 1";

      // Ensures that we only update the sort sequence on valid entries.  I guess this doesn't really matter
      // but might make some marginal speed inprovement
      sql = sql + " AND te.status = 'Y'";

      // Finally we only want to perform the update on tournaments that have been closed but aren't yet
      // fighting i.e are in the PENDING state
      sql = sql + " AND t.status = 'PENDING'";
      sql = sql + " )";

      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();


      // The next lot of SQL is very similar to the above in terms of the restrictions
      // the only difference is that it updates the tournament status from PENDING to FIGHTING

      sql =  "UPDATE TOURNAMENT t1 SET t1.status = 'FIGHTING'";
      
      // Only set the status to Fighting if the Auto_Fight field = 'Y'
      sql = sql + " WHERE t1.auto_fight = 'Y'";

      sql = sql + " AND EXISTS";
      sql = sql + " (";
      sql = sql + " SELECT t.id";
      sql = sql + " FROM TOURNAMENT t, TOURNAMENT_ENTRY te";
      sql = sql + " WHERE t.id = te.tournament_id";
      sql = sql + " AND t1.id = t.id";
      sql = sql + " AND (SELECT COUNT(*)";
      sql = sql + " FROM TOURNAMENT, TOURNAMENT_ENTRY";
      sql = sql + " WHERE TOURNAMENT.id = TOURNAMENT_ENTRY.tournament_id";
      sql = sql + " AND TOURNAMENT.id = t.id";
      sql = sql + " AND (TOURNAMENT_ENTRY.status IS NULL OR TOURNAMENT_ENTRY.status = 'P'OR TOURNAMENT_ENTRY.status = 'BAD')) = 0";
      sql = sql + " AND (SELECT COUNT(*)";
      sql = sql + " FROM TOURNAMENT, TOURNAMENT_ENTRY";
      sql = sql + " WHERE TOURNAMENT.id = TOURNAMENT_ENTRY.tournament_id";
      sql = sql + " AND TOURNAMENT.id = t.id";
      sql = sql + " AND TOURNAMENT_ENTRY.status = 'Y') > 1";
      sql = sql + " AND te.status = 'Y'";
      sql = sql + " AND t.status = 'PENDING'";
      sql = sql + " GROUP BY t.id";
      sql = sql + " )";

      statement = con.createStatement();
      statement.executeUpdate(sql);
      statement.close();

      con.commit();
      con.setAutoCommit(true);

    }
    catch (Exception e)
    {
      try
      {
        con.rollback();
        con.setAutoCommit(true);
      }
      catch (SQLException sqle1)
      {
        System.err.println("STournamentController/setPendingToFighting: " + sqle1.getMessage());
      }

      System.err.println("STournamentController/setPendingToFighting: " + e.getMessage());
    }
    finally
    {
      closeStatement(statement,"STournamentController/setPendingToFighting");
    }


  }

   private void assignFights(Connection con)
  {
    /*
     Look after all the Fighting Touranaments
    */

    Statement statement = null;
    ResultSet rs        = null;

    Statement statement1 = null;
    ResultSet rs1        = null;

    Statement statement2 = null;
    ResultSet rs2        = null;

    try
    {

      //System.out.print("*** assignFights ***");

      con.setAutoCommit(false);

      // Find the first tournament where the status is set to FIGHTING

      String sql = "SELECT id";
      sql = sql + " FROM TOURNAMENT ";
      sql = sql + " WHERE status = 'FIGHTING'";
      sql = sql + " ORDER BY close_date";


      statement = con.createStatement();
      rs = statement.executeQuery(sql);

      while (rs.next())
      {
        // Get the tournament id
        int tournamentID = rs.getInt("id");
        //rs.close();
        //statement.close();

        //System.out.println("Tournament ID " + tournamentID);

        // Get the count of all the valid tournament entries, without a hightest round
        // (this means they haven't been beaten) that have fights with the status
        // WAITING or FIGHTING or BAD.  If the result is 0 This means that either all the fights have been run for the
        // current level (have a status of FOUGHT) and it's time to calculate the next OR that no fights have been
        // entered in the FIGHT table in which case it's time to enter in the first row of
        // fights into the FIGHT table.

        sql = "SELECT COUNT(te.id) AS COUNT";
        sql = sql + " FROM TOURNAMENT_ENTRY te, FIGHT f";
        sql = sql + " WHERE (f.status = 'FIGHTING' OR f.status = 'WAITING' OR f.status = 'BAD')";
        sql = sql + " AND te.status = 'Y'";
        sql = sql + " AND (te.id = f.tournament_entry1_id OR te.id = f.tournament_entry2_id)";
        sql = sql + " AND te.highest_round IS NULL";
        sql = sql + " AND te.tournament_id = " + tournamentID;

        statement1 = con.createStatement();
        rs1 = statement1.executeQuery(sql);
        rs1.next();

        int count = rs1.getInt("COUNT");
        rs1.close();
        statement1.close();

        //System.out.println("Valid Tournaments with no highest round that are either Waiting or Fighting or Bad " + count);

        if (count == 0)
        {
          // It's time to move the fight to the next round

          // Count how many valid tournament entries without a hightest round for this tournament
          sql = "SELECT COUNT(te.id) AS COUNT";
          sql = sql + " FROM TOURNAMENT_ENTRY te";
          sql = sql + " WHERE te.status = 'Y'";
          sql = sql + " AND te.highest_round IS NULL";
          sql = sql + " AND te.tournament_id = " + tournamentID;

          statement1 = con.createStatement();
          rs1 = statement1.executeQuery(sql);
          rs1.next();

          int count1 = rs1.getInt("COUNT");
          rs1.close();
          statement1.close();

          System.out.println("Number of valid TE with no Highest Round " + count1);

          if (count1 == 1)
          {
            // Then this entry is the winner

            // Increase this tournament round by one and assign that number to the entry
            sql = "UPDATE TOURNAMENT SET current_round = current_round + 1";
            sql = sql + " WHERE id = " + tournamentID;
            statement1 = con.createStatement();
            statement1.executeUpdate(sql);
            statement1.close();


            // Get the current_round number
            sql = "SELECT current_round FROM TOURNAMENT";
            sql = sql + " WHERE id = " + tournamentID;
            statement1 = con.createStatement();
            rs1 = statement1.executeQuery(sql);
            rs1.next();
            int current_round = rs1.getInt("current_round");

            rs1.close();
            statement1.close();

            //System.out.println("Current Round " + current_round);

            // Update the winning tournament entry
            sql = "UPDATE TOURNAMENT_ENTRY";
            sql = sql + " SET highest_round = " + current_round + ", current_round = " + current_round;
            sql = sql + " WHERE status = 'Y'";
            sql = sql + " AND highest_round IS NULL";
            sql = sql + " AND tournament_id = " + tournamentID;
            statement1 = con.createStatement();
            statement1.executeUpdate(sql);
            statement1.close();

            // Find Summary Information

            // Find the number of tournament entries for each tournament
            sql = "SELECT COUNT(*) AS count FROM TOURNAMENT_ENTRY WHERE tournament_id = " + tournamentID;
            sql += " AND account_id <> " + roboforgeAccountID;
            statement1 = con.createStatement();
            rs1 = statement1.executeQuery(sql);
            rs1.next();

            int submissions = rs1.getInt("count");

            rs1.close();
            statement1.close();

            // Find the winning entry
            sql = "SELECT acc.designer_name, te.bot_name";
            sql = sql + " FROM TOURNAMENT_ENTRY te, ACCOUNT acc";
            sql = sql + " WHERE te.highest_round = " + current_round;
            sql = sql + " AND te.account_id = acc.id";
            sql = sql + " AND te.tournament_id = " + tournamentID;

            statement1 = con.createStatement();
            rs1 = statement1.executeQuery(sql);
            rs1.next();

            String summary = "Tournament Statistics\n";
            summary += "Number of Entries: " + submissions + "\n";
            summary += "Winning " + BGlobal.BOT + ": " + rs1.getString("bot_name") + "\n";
            summary += BGlobal.DESIGNER.substring(0,1).toUpperCase()
              + BGlobal.DESIGNER.substring(1,BGlobal.DESIGNER.length()) + ": " + rs1.getString("designer_name");

            rs1.close();
            statement1.close();

            // Change the status of the tournament to 'FINISHED'
            // And populate the number of submissions and the summary fields
            sql = "UPDATE TOURNAMENT";
            sql = sql + " SET status = 'FINISHED'";
            sql = sql + " ,summary = '" + summary + "'";
            sql = sql + " WHERE id = " + tournamentID;
            statement1 = con.createStatement();
            statement1.executeUpdate(sql);
            statement1.close();

            updateAccountRankings(con,tournamentID);
            allocatePrizes(con,tournamentID);
          }
          else if (count1 == 0)
          {
            // This is very unlikely that the tournament finished in a draw
            // however just in case

            // Change the status of the tournament to 'FINISHED'
            sql = "UPDATE TOURNAMENT";
            sql = sql + " SET status = 'FINISHED', current_round = current_round + 1";
            sql = sql + " WHERE id = " + tournamentID;
            statement1 = con.createStatement();
            statement1.executeUpdate(sql);
            statement1.close();

            updateAccountRankings(con,tournamentID);
            allocatePrizes(con,tournamentID);

            System.out.println("*** Final Round Draw Fight Set To Finished ***");;

          }
          else
          {

            // If there isn't a power of two number of entries in this round then
            // add in enough house bot to take the round to the next power of 2

            double nextPowerOf2 = roboforge.math.BMath.getNearestEnclosingPowerOf2((double)count1);
            if (((nextPowerOf2 - (double)count1) != 0.0))
            {
              sql = "SELECT current_round FROM TOURNAMENT";
              sql = sql + " WHERE id = " + tournamentID;
              statement1 = con.createStatement();
              rs1 = statement1.executeQuery(sql);
              rs1.next();

              // +1 cause the current round hasn't been updated yet
              int currentRound = rs1.getInt("current_round");

              rs1.close();
              statement1.close();

              // Loop through adding house bot to the current round until we have
              // a power of two number of entries
              for(double i = 0;i< nextPowerOf2 - (double)count1;i++)
              {

                // Get the next tournament entry ID
                sql = "SELECT TOURNAMENT_ENTRY_IDS.nextval As ID FROM DUAL";

                statement1 = con.createStatement();
                rs1 = statement1.executeQuery(sql);
                rs1.next();
                long tournament_entry_id = rs1.getLong("id");
                rs1.close();
                statement1.close();

                sql = "INSERT INTO TOURNAMENT_ENTRY";
                sql = sql + " (id, tournament_id, account_id, bot_name, bot_file";
                sql = sql + " , status, current_round, sort_sequence)";
                sql = sql + " VALUES(" + tournament_entry_id + "," + tournamentID + "," + roboforgeAccountID;
                sql = sql + ",'RoboForge HouseBot',(SELECT house_bot FROM CONSTANTS),'Y'," + currentRound;
                sql = sql + ", random)";

                statement1 = con.createStatement();
                statement1.executeUpdate(sql);
                statement1.close();
              }
            }

            // Now create fights for all the available tournaments

            // Get the current_round number
            sql = "SELECT current_round FROM TOURNAMENT";
            sql = sql + " WHERE id = " + tournamentID;
            statement1 = con.createStatement();
            rs1 = statement1.executeQuery(sql);
            rs1.next();
            int currentRound1 = rs1.getInt("current_round");

            rs1.close();
            statement1.close();

            //System.out.println("Current Round 1 " + currentRound1);

            // Update the all the entries to the next round
            sql = "UPDATE TOURNAMENT_ENTRY";
            sql = sql + " SET current_round = current_round + 1";
            sql = sql + " WHERE status = 'Y'";
            sql = sql + " AND highest_round IS NULL";
            sql = sql + " AND current_round = " + currentRound1;
            sql = sql + " AND tournament_id = " + tournamentID;
            statement1 = con.createStatement();
            statement1.executeUpdate(sql);
            statement1.close();

            //System.out.println("Updated Entries");

            // Update the current round on the Tournament
            sql = "UPDATE TOURNAMENT SET current_round = current_round + 1";
            sql = sql + " WHERE id = " + tournamentID;
            statement1 = con.createStatement();
            statement1.executeUpdate(sql);
            statement1.close();

            //System.out.println("Updated Current Round");

            // Insert the tournaments entries, Two by Two, into the Fight table

            // This result holds all the non-housebots
            // As we want to match up as many Bots as possible with house bots, to ensure that
            // Housebots don't fight each other, causing more Housebots
            // to pregress to higher levels
            sql = "SELECT te.id AS id";
            sql = sql + " FROM TOURNAMENT t, TOURNAMENT_ENTRY te";
            sql = sql + " WHERE te.status = 'Y'";
            sql = sql + " AND te.highest_round IS NULL";
            sql = sql + " AND te.tournament_id = t.id";
            sql = sql + " AND te.current_round = t.current_round";
            sql = sql + " AND t.id = " + tournamentID;
            sql = sql + " AND te.account_id <> " + roboforgeAccountID;
            sql = sql + " ORDER BY te.sort_sequence";

            statement1 = con.createStatement();
            rs1 = statement1.executeQuery(sql);


            // This recordset holds all the HouseBots
            sql = "SELECT te.id AS id";
            sql = sql + " FROM TOURNAMENT t, TOURNAMENT_ENTRY te";
            sql = sql + " WHERE te.status = 'Y'";
            sql = sql + " AND te.highest_round IS NULL";
            sql = sql + " AND te.tournament_id = t.id";
            sql = sql + " AND te.current_round = t.current_round";
            sql = sql + " AND t.id = " + tournamentID;
            sql = sql + " AND te.account_id = " + roboforgeAccountID;
            sql = sql + " ORDER BY te.sort_sequence";

            statement2 = con.createStatement();
            rs2 = statement2.executeQuery(sql);

            int counter = 0;
            int entry1  = 0;
            int entry2  = 0;

            //System.out.println("Starting Insert");

            while (rs1.next())
            {
              counter++;
              //System.out.println("Counter Value " + counter);
              if (counter == 1)
              {
                entry1 = rs1.getInt("ID");

                // If there are any available house bots then use them
               if (rs2.next())
               {
                  entry2 = rs2.getInt("ID");
                  counter++;
               }
              }
              else
              {
                entry2 = rs1.getInt("ID");
              }

              if(counter == 2)
              {
                String isFinal;
                if (count1 == 2)
                  isFinal = "Y";
                else
                  isFinal = "N";
                  
                /*
                // This code shouldn't be needed
                if (count1 == 2)
                {
                  // Check to see if there are any entries at a higher level
                  sql = "SELECT count(id) As count FROM TOURNAMENT_ENTRY WHERE tournament_id = " + tournamentID;
                  sql += " AND round = " + (currentRound1 + 2);
                  Statement statement3 = con.createStatement();
                  ResultSet rs3 = statement3.executeQuery(sql);

                  int higherCount = rs3.getInt("count");

                  rs3.close();
                  statement3.close();

                  if(higherCount == 0)
                    isFinal = "Y";
                  else
                    isFinal = "N";

                }
                else
                  isFinal = "N";
                */

                // counter must = 2 so therefore insert the fight
                sql = "INSERT INTO FIGHT (tournament_id, tournament_entry1_id, tournament_entry2_id, round, status,battle_recording,battle_stats1,battle_stats2,final)";
                sql = sql + " VALUES (" + tournamentID + ", " + entry1 + ", " + entry2 + ", " + (currentRound1 + 1) + ", 'WAITING',empty_blob(),empty_blob(),empty_blob(),'" + isFinal + "')";
                //System.out.println(sql);
                Statement statementUpdate = con.createStatement();
                statementUpdate.executeUpdate(sql);
                statementUpdate.close();

                counter = 0;
              }
            }

            rs1.close();
            statement1.close();

            rs2.close();
            statement2.close();
          }
        }
      }

      rs.close();
      statement.close();


      con.commit();
      con.setAutoCommit(true);
    }
    catch (Exception e)
    {
      try
      {
        con.rollback();
        con.setAutoCommit(true);
      }
      catch (SQLException sqle1)
      {
        System.err.println("STournamentController/assignFights: " + sqle1.getMessage());
      }

      System.err.println("STournamentController/assignFights: " + e.getMessage());
    }
    finally
    {
      closeResultSet(rs,"STournamentController/assignFights");
      closeStatement(statement,"STournamentController/assignFights");

      closeResultSet(rs1,"STournamentController/assignFights");
      closeStatement(statement1,"STournamentController/assignFights");

      closeResultSet(rs2,"STournamentController/assignFights");
      closeStatement(statement2,"STournamentController/assignFights");
    }
  }

protected static void closeResultSet(ResultSet rs, String location)
  {
    try
    {
      if (rs != null)
        rs.close();
    }
    catch (Exception e)
    {
      System.err.println("SChallengeRoomServlet/closeResultSet: "+ location + ": " + e);
    }

  }

  protected static void closeStatement(Statement statement, String location)
  {
    try
    {
      if (statement != null)
        statement.close();
    }
    catch (Exception e)
    {
      System.err.println("SChallengeRoomServlet/closeStatement: "+ location + ": " + e);
    }

  }
  
  //Get Servlet information
  public String getServletInfo()
  {
    return "roboforge.STournamentController";
  }

   private Connection getDatabaseConnection()
  {
    Connection con = null;
    try
    {
      con = myBroker.getConnection();
      //Class.forName ("oracle.jdbc.driver.OracleDriver");
      //con = DriverManager.getConnection("jdbc:oracle:thin:@hugh:1521:edge", "edge", "edge");
      //Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
      //con = DriverManager.getConnection("jdbc:odbc:Testy");
    }
    catch (Exception sqle)
    {
      System.err.println(sqle.getMessage());
    }
  return con;
  }
} 