package roboforge.ai;

import roboforge.BBot;
import roboforge.BGlobal;
import roboforge.BDebug;
import roboforge.BZone;
import roboforge.BZones;
import roboforge.btree.*;
import roboforge.BMoveList;

import roboforge.ai.decisions.*;
import roboforge.ai.memory.*;
import roboforge.btree.*;

import roboforge.util.graphics.*;

import java.io.*;
import java.util.*;
import javax.swing.tree.*;
import javax.swing.Icon;


public class BAIBlock implements  Serializable
  {
    // Serialization version Unique ID
    // -----------------------------------------------------------
    static final long serialVersionUID = -1L;

    protected BSerializableTree tree;
    protected String name;
    protected String iconName;
    protected String description;

    // block dependencies
    private Vector dependentZones;
    private Vector dependentMemoryItems;
    private Vector dependentMoves;

    private transient boolean newBotSet = false;

    public BAIBlock(BAIBlock other)
    {
      this.tree = new BSerializableTree(other.tree.getRoot().cloneTree() );
      this.name = other.name;
      this.iconName = other.iconName;
      this.description = other.description;

      this.dependentZones = other.dependentZones;
      this.dependentMemoryItems = other.dependentMemoryItems;
      this.dependentMoves = other.dependentMoves;

    }

    public BAIBlock(BDecisionTreeNode node, BBot bot)
    {
      BDecisionTreeNode newNode =  (BDecisionTreeNode)node.cloneTree();

      if (newNode instanceof BBlockNode)
        ((BBlockNode)newNode).setClosed(true);
      else
      {
        BBlockNode newGroup = new BBlockNode();
        newGroup.setClosed(true);
        newGroup.add(newNode);
        newNode = newGroup;
      }

      tree = new BSerializableTree( newNode );
      extractDependencies(bot);
      setBot(null);

    }

    private void addMemoryItems(BBot bot)
    {
      BDecisionTreeAI ai = (BDecisionTreeAI) bot.getAI();
      BMemoryBank bank = ai.getMemoryBank();

      for (int i = dependentMemoryItems.size() -1; i >= 0 ; i-- )
      {
        if (bank.get( ((BMemoryItem)dependentMemoryItems.get(i)).getName()  ) == null)
        {
          BMemoryItem m = (BMemoryItem)dependentMemoryItems.get(i);
          bank.add(m);
          m.validateAndFixInitialValue();

        }
      }


    }

    private void addDependentMoves(BBot bot)
    {
      BMoveList mList = bot.getMoveList();

      for (int i = dependentMoves.size() -1; i >= 0 ; i-- )
      {
         if (mList.getMove((String)dependentMoves.get(i) ) == null      )
        {
          mList.add( bot.createNewMove( (String)dependentMoves.get(i)) );

        }
      }


    }




    private void addZones(BBot bot)
    {
      BZones zones= bot.getZones();

      for (int i = dependentZones.size() - 1; i>=0; i-- )
      {
        //BDebug.print ("BAIBlock checking zone " + dependentZones.get(i));

        BZone z = zones.getZoneForName(dependentZones.get(i).toString() );

        //BDebug.print ("BAIBlock returned " + z);

        if( z == null)
          zones.addZone( new BZone( (BZone)dependentZones.get(i) ) );
      }

    }

    public boolean isAssignedToBot()
      { return newBotSet; }


    private String addLine(String original,String newLine)
    {
      return original.concat(newLine + "\n");
     // BDebug.print ("BAIBlock.addLine called " + original);
    }

    public String getNewBotMessage(BBot bot)
    {
        String msg = getName() + " imported into AI...\n";

        if (dependentZones != null)
          if (dependentZones.size() != 0)
          {
            msg = addLine(msg, "");
            msg = addLine(msg, "The following zones were processed:");

            BZones zones= bot.getZones();

            for (int i = dependentZones.size() - 1; i>=0; i-- )
            {
              BZone z = zones.getZoneForName(dependentZones.get(i).toString() );

              if( z == null)
                msg = addLine(msg , dependentZones.get(i) + " was added to the bot.");
              else
                msg = addLine(msg , dependentZones.get(i) + " was found and matched.");


            }


          }

      if (dependentMemoryItems != null)
        if (dependentMemoryItems.size() != 0)
        {
          msg = addLine(msg, "");
          msg = addLine(msg, "The following memory items were processed:");

          BDecisionTreeAI ai = (BDecisionTreeAI) bot.getAI();
          BMemoryBank bank = ai.getMemoryBank();

          for (int i = dependentMemoryItems.size() -1; i >= 0 ; i-- )
          {
            if (bank.get( ((BMemoryItem)dependentMemoryItems.get(i)).getName()  ) == null)
              msg = addLine(msg, dependentMemoryItems.get(i) + " was added to the bot.");
            else
              msg = addLine(msg, dependentMemoryItems.get(i) + " was found and matched.");


          }



        }


        if (dependentMoves != null)
          if (dependentMoves.size() != 0)
          {
                msg = addLine(msg, "");
                msg = addLine(msg, "The following moves were processed:");
                BMoveList mList = bot.getMoveList();

                for (int i = dependentMoves.size() -1; i >= 0 ; i-- )
                {
                  if (mList.getMove((String)dependentMoves.get(i) ) == null      )
                    msg = addLine(msg, "A blank move called " + dependentMoves.get(i) + " was added to the bot.");
                  else
                    msg = addLine(msg, dependentMoves.get(i) + " was found and matched.");


              }

          }

      return msg;



    }

    public void setBot(BBot newBot)
    {
      // update dependancies
      if (newBot != null)
      {
        newBotSet = true;

        if(dependentZones != null)
          addZones(newBot);
        if (dependentMemoryItems != null)
          addMemoryItems(newBot);

        if (dependentMoves != null)
          addDependentMoves(newBot);

      }

      Enumeration allAi = ((DefaultMutableTreeNode)tree.getRoot()).breadthFirstEnumeration();
      while(allAi.hasMoreElements())
      {
        Object next = allAi.nextElement();
        if (next instanceof BDecisionTreeNode)
          ((BDecisionTreeNode)next).setActor(newBot);
      }


    }

    public void extractDependencies(BBot bot)
    {
      Vector totalZones = new Vector();
      dependentMemoryItems = new Vector();
      dependentMoves = new Vector();


      Enumeration allAi = ((DefaultMutableTreeNode)tree.getRoot()).breadthFirstEnumeration();
      while(allAi.hasMoreElements())
      {
        Object next = allAi.nextElement();
        if (next instanceof BDecisionNode)
        {
          BDecisionNode node = (BDecisionNode)next;
          Vector nodeZones = node.getQuestionedZones();
          if (nodeZones != null)
            totalZones.addAll(nodeZones);
        }
        else if (next instanceof BAssignmentNode)
        {
          BAssignmentNode node = (BAssignmentNode)next;
          dependentMemoryItems.add(node.getMemoryItem() );

        }
        else if (next instanceof BMoveActionNode)
        {
          BMoveActionNode node = (BMoveActionNode) next;
          if (node.getMoveAction() != null)
            dependentMoves.add(node.getMoveAction().getName() );
          

        }

      }



      // create dependency zones
      if (totalZones.size() != 0)
      {
        BDebug.print ("BAIBlock adding zones " +  totalZones );
        createDependentZones(totalZones, bot);
      }
      // cleanup
      if (dependentMoves.size() == 0)
        dependentMoves = null;

    
      if (dependentMemoryItems.size() == 0)
        dependentMemoryItems = null;


    }


    private void createDependentZones(Vector totalZones, BBot bot)
    {
        dependentZones = new Vector(totalZones.size() );

        for(int i = 0; i < totalZones.size(); i++ )
        {
          BZone currentZ = bot.getZones().getZoneForName((String)totalZones.get(i));
          if (currentZ != null)
            dependentZones.add( new BZone(currentZ) );
        }


    }




    public BDecisionTreeNode getRoot()
    {
      BGroupNode node = (BGroupNode)tree.getRoot();
      node.setName(this.getName() );
      return node;
    }


    public boolean isAttachedToBot()
    {
      return tree.getRoot().getParent() != null;
    }

    public String getName()
      { return name; }

    public void setName(String name)
      { this.name = name; }

    public void setIconName(String s)
      { this.iconName = s; }
      

    public void saveToFile(String filename)
    throws IOException
    {
      if (!BGlobal.BLOCKSUFFIX.equalsIgnoreCase(filename.substring(filename.length()-4)))
        filename = filename +  BGlobal.BLOCKSUFFIX;
      save(filename);
    }

    protected void save ( String filename)
    throws IOException
    {
      
      FileOutputStream fout = new FileOutputStream(filename);
      ObjectOutputStream out = new ObjectOutputStream(fout);

      out.writeObject(this);
      out.close();

      BSerializableTree.clearCache();
      

    }

    public static BAIBlock loadFromFile(String filename)
    {
      BAIBlock block = null;

      try
      {
        filename = filename.toLowerCase();

        FileInputStream fin = new FileInputStream(filename);
        ObjectInputStream in = new ObjectInputStream(fin);

        block = (BAIBlock) in.readObject();

        in.close();
        fin.close();

      }

      catch (Exception e)
        { e.printStackTrace(); }

      BSerializableTree.clearCache();

      return block;
    }



    public Icon getIcon()
    {
      if (iconName == null)
        return BIconCache.getHSB("ai/block_30.gif");
      else
        return BIconCache.getHSB(iconName);
    }

    public String getDescription()
      {return description;  }

    public void setDescription(String s)
      { this.description = s;}

    public Object clone()
    {
      return new BAIBlock(this);

    }

    public String toString()
    {
      if(name != null)
        return name;
      else
        return super.toString(); 
    }

  }