package roboforge.ai.decisions;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BZones;
import roboforge.BBot;
import roboforge.ai.actions.BActor;
import roboforge.ai.BDecisionTreeAI;

import roboforge.ai.expressions.*;
import roboforge.ai.values.*;
import roboforge.ui.components.*;

import java.util.Enumeration;
import java.util.Vector;
import roboforge.util.parser.*;

import javax.swing.Icon;
import javax.swing.tree.*;
import java.awt.Color;



public class BDecisionNode extends BBranchNode
  // Node with two child branches - 'yes' and 'no'.
  // Only one of these children can ever be visited.
{

  // Class variables...
  // -----------------------------------------------------------

  protected static BBorderedIcon icon =
    createTreeIcon("ai/complex_decision_18.gif", BGlobal.colors.getAIDecisionColor_Me());

  //protected static Icon selectedIcon =
  //  createTreeIcon("ai/complex_decision_18.gif", BGlobal.SELECTED_COLOR, BGlobal.colors.getAIDecisionColor_Me());


  // Internal variables...
  // -----------------------------------------------------------

  protected BExpression question;
  protected String nullifiedActorString;

  protected boolean invalid = false;
  // using the opposite of valid so that old bots have the right value


  // Temporary variables...
  // -----------------------------------------------------------

  protected transient int branch;


  // Constructor...
  // -----------------------------------------------------------

  public BDecisionNode()
    { super(); }

  public BDecisionNode(BDecisionNode node)
  {
    super(node);
    this.question = node.question;
    this.nullifiedActorString = node.nullifiedActorString;
  }

  public Object clone()
  {
    //BDebug.print("BDecisionNode.clone()" );

    BDecisionNode node =
      new BDecisionNode(this);

    // Add in dummy nodes as needed...

    // HACK: I put this code here to avoid problems
    //       with BSerializedTree's instantiation
    //       method (it calls the no-args constructor).

    int n = getMaxChildren() - getChildCount();

    for (int i=n; --i >=0; )
      node.add(new BNOPNode());

    return node;
  }


  // Initialization method.
  // -------------------------------------------------------------

  public void init()
  {
    if (question != null)
      question.init();

    super.init();
  }


  // Accessor methods...
  // -----------------------------------------------------------

  public void setQuestion(BExpression q)
  {
    // if (d.getType() != BExpression.BOOLEAN)
    //  throw new IllegalArgumentException("Expression is non-boolean.");

    this.question = q;
    setDefaultName();

   // BDebug.print ("BDecisionNode setQuestion question" + question);


    // set the valid flag
    if (question != null)
    {
     // BDebug.print ("BDecisionNode setQuestion question.evaluate " + question.evaluate().getType() );
      invalid = ! question.evaluate().isBoolean();

    }
  }

  public BExpression getQuestion()
    { return question; }


  public void prune()
  {

    if (question != null)
      invalid = !question.evaluate().isBoolean();
    //setQuestion(question);

   // BDebug.print("BDecisionNode.prune called with result " + !invalid);

  }

  public boolean isValid()
  {
    //BDebug.print("BDecisionNode.isValid called  " + !invalid);
    return !invalid;

  }



  // TODO: Handle cases where nodes are added outside of setYes/No!

  public boolean canAddChildren()
    { return getChildCount() < getMaxChildren(); }

  protected String generateDefaultName()
  {
    if (question == null)
      return "Decision";
    else
      return question.toString();
  }


 // public boolean getContainsZoneRef()

 public int getMaxChildren()
    { return 2; }

  // Child maintenance methods...
  // -------------------------------------------------------------

  public void insert(MutableTreeNode child, int index)
  {
    if (index > getMaxChildren() )
      throw new IllegalArgumentException("index ["+index+"] out of range: "+
        "BDecisionNodes can only have " + getMaxChildren() +" kids.");

    super.insert(child, index);

    // If size limit exceeded, remove displaced sibling.
    if (getChildCount() > getMaxChildren() )
      super.remove(index+1);
  }

  public void remove(int index)
  {
    super.remove(index);
    insert(new BNOPNode(), index);
  }

  public void remove(MutableTreeNode child)
  {
    int index = getIndex(child);

    super.remove(child);
    insert(new BNOPNode(), index);
  }

  public void removeAllChildren()
  {
    super.removeAllChildren();

    if(canAddChildren())
      add(new BNOPNode());
    if (canAddChildren())
      add(new BNOPNode());
  }



  
  // Visitable methods...
  // -----------------------------------------------------------

  public void visit()
  {
    branch = -1;
      // That is, do not take either branch.

    if (question != null)
    {
      BValue answer =
        question.evaluate();

      if (answer.isBoolean())
      {

        BBoolean b = (BBoolean) answer;
        branch = b.getValue() ? 0 : 1;

        //if (this instanceof BSimpleDecisionNode)
        //BDebug.print("BDecision Node "+ getName()+ " evaluates to "+ answer);

      }
    }
    //else
    //  BDebug.print("BDecision Node: null question.");

  }


  public Vector getQuestionedZones()
  {
    if (question == null)
      return null;
    else
      return getQuestionedToken(BZones.FULL_WORLDSTATE_CODE );

  }

  protected Vector getQuestionedToken(String token)
  {
    Vector returnList = null;
    String expressionString = question.toString();
    if (expressionString.indexOf(token) != -1)
    {
      returnList = new Vector(1,2);
      while ( expressionString.indexOf(token) != -1)
      {
        //BDebug.print ("BDecisionNode getQuestionedToken expressionString " + expressionString);
        expressionString = expressionString.substring(expressionString.indexOf(token) + token.length() );
        //BDebug.print ("BDecisionNode getQuestionedToken expressionString " + expressionString);

        if ( expressionString.substring(0,9).equalsIgnoreCase("anyActive") )
          expressionString = expressionString.substring(9);
        else
        { 
          String tokenName = expressionString.substring(0, expressionString.indexOf('.'));
          //BDebug.print("BDecisionNode getQuestionedToken adding " + tokenName);
          returnList.add(tokenName);
          expressionString = expressionString.substring(tokenName.length() );
        }
        


      }

    }


    return returnList;

  }



  public Enumeration visitableChildren()
    { return new VisitableKidsEnumeration(); }


  protected class VisitableKidsEnumeration implements Enumeration
  {
    protected boolean more = true;

    public boolean hasMoreElements()
    {
      if (more)
      {
        if (branch < 0)
          more = false;
        else if (branch > BDecisionNode.this.getChildCount()-1 )
          more = false;
        else
          more = true;
      }
      //if (BDecisionNode.this instanceof BSimpleDecisionNode)
      //BDebug.print("BDecision VisitableKidsEnumeration.has more "+ BDecisionNode.this.getName()+ " returning "+ more);

      return  more;
    }

    public Object nextElement()
    {
      more = false;
      //if (BDecisionNode.this instanceof BSimpleDecisionNode)
        //BDebug.print("BDecision VisitableKids nextElement "+ BDecisionNode.this.getName()+ " returning "+ getChildAt(branch));

      return getChildAt(branch);
    }
  }

  public Icon getIcon()
  {

    if (this.getVisited())
      icon.setBorderColor(BGlobal.colors.getLightestShade());
    else
      icon.setBorderColor(BGlobal.colors.getAIDecisionColor_Me());

    return icon;

  }

  public Icon getSelectedIcon()
    { return getIcon(); }


  public void setActor(BActor a)
  {
    super.setActor(a);

    if ( (a == null) )
    {
      if (question == null)
        nullifiedActorString = null;
      else
        nullifiedActorString = question.toString();

      question = null;
      //BDebug.print ("BDecisionNode.setActor nullifiedActorString " + nullifiedActorString);

    }
    else if ( nullifiedActorString != null)
    {
      BBot bot = (BBot) a;
      BDecisionTreeAI ai = (BDecisionTreeAI) bot.getAI();
      //BDebug.print ("BDecisionNode.setActor new parsing ... " + nullifiedActorString);

      try
        { question = ai.getParser().parse(nullifiedActorString);
        }
      catch ( BParseException ex ){}

    }

  }

  // State storage / retrieval...
  // -----------------------------------------------------------

  public void setState(Object o)
  {
    super.setState(o);

    DecisionNodeState state =
      (DecisionNodeState) o;

    this.question = state.question;
    this.nullifiedActorString = state.nullifiedActorString;
  }

  public Object getState()
    { return new DecisionNodeState(this); }

  protected static class DecisionNodeState extends DecisionTreeNodeState
  {
    static final long serialVersionUID = -1L;
    protected BExpression question;
    protected String  nullifiedActorString;

    public DecisionNodeState(BDecisionNode node)
    {
      super(node);
      this.question = node.question;
      this.nullifiedActorString = node.nullifiedActorString;
    }
  }

}
