package roboforge.ai.decisions;

import java.io.Serializable;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Stack;

import javax.swing.tree.*;

import roboforge.btree.*;


public abstract class BVisitableTreeNode extends BTreeNode
{
  public BVisitableTreeNode()
    { super(); }

  // public abstract void acceptVisitor(BVisitor v);

  public abstract void visit();
  public abstract Enumeration visitableChildren();



  public VisitableSubtreeEnumeration visitableSubtree()
    { return new VisitableSubtreeEnumeration(this); }

  protected final class VisitableSubtreeEnumeration implements Enumeration
  {
	  protected Stack	      stack;
    protected Enumeration	kids;
    protected boolean     deferredActionsPending;

  	public VisitableSubtreeEnumeration(BVisitableTreeNode n)
    {
	    super();
	    stack = new Stack();
	    stack.push(new RootEnumeration(n));

      deferredActionsPending = false;
  	}

	  public boolean hasMoreElements()
    {
      if (deferredActionsPending)
        doDeferredActions();

	    return (!stack.empty() &&
		    ((Enumeration) stack.peek()).hasMoreElements());
  	}

  	public Object nextElement()
    {
      if (deferredActionsPending)
        doDeferredActions();

	    Enumeration	e = (Enumeration) stack.peek();

	    BVisitableTreeNode node =
        (BVisitableTreeNode) e.nextElement();

	    kids = node.visitableChildren();

	    if (!e.hasMoreElements())
    		stack.pop();

      deferredActionsPending = true;

	    return node;
  	}

    protected void doDeferredActions()
    {
	    if (kids.hasMoreElements())
        stack.push(kids);

      deferredActionsPending = false;
    }
  }


  protected final class RootEnumeration implements Enumeration
  {
    protected TreeNode root;
    protected boolean  more;

    public RootEnumeration(TreeNode root)
      { this.root = root; more = true; }

	  public boolean hasMoreElements()
      { return more; }

	  public Object nextElement()
      { more = false; return root; }
  }

}
