package roboforge.animation.animation.b3d;

import roboforge.animation.*;
import roboforge.*;
import roboforge.sound.*;
import roboforge.util.timing.*;


public class BDeadlockAnimation implements BAnimation
{

  private static float FADE_OUT_TIME = 2000f; // ml
  double fadeStart = 0;

  BComponent[] aComponents;
  BComponent[] bComponents;
  BBattleRecorder battleRecord;
  int deadlockFrame = 0;
  boolean frameNotSwitched = false;

  BSound fadeSound;


  public BDeadlockAnimation(BBot botA,BBot botB, BBattleRecorder battleRecord)
  {
    aComponents =  botA.getBreathComponents();
    bComponents = botB.getBreathComponents();
    this.battleRecord = battleRecord;
    fadeSound = BSound.getSound(BGlobal.SOUNDPATH + "arena/deadlock.wav" , BSound.LOOP_CONTINUOUSLY, BSound.SOUND_TYPE_FOREGROUND);

  }

  public void start()
  {
    fadeStart = BClock.time() * 1000;
    frameNotSwitched = true;
    fadeSound.play();
  }

  public void setDeadlockFrame(int index)
  {
    deadlockFrame = index;
  }

  public boolean animate( )
  {
      double deltaT = (BClock.time() * 1000) - fadeStart;
     // BDebug.print ("BBot.fadeout deltaT " + deltaT);
      float fadeT;
      if (deltaT == 0 )
        fadeT = 0;
      else
        fadeT = (float)deltaT / FADE_OUT_TIME ;

     if (fadeT < 1f) // fadeout
     {

      for (int i = 0 ; i < aComponents.length; i++)
      {
        if (fadeT > aComponents[i].getTransparency() )
          aComponents[i].setTransparency(fadeT, false);
      }


      for (int i = 0 ; i < bComponents.length; i++)
      {
        if (fadeT > bComponents[i].getTransparency() )
          bComponents[i].setTransparency(fadeT, false);
      }

     }
     else if (frameNotSwitched)
     {
      frameNotSwitched = false;
      battleRecord.apply(BBattleRecorder.APPLYMODE_PLAY,deadlockFrame, deadlockFrame);
     }
     else
     {
      float fadeInT = 1f - ( fadeT - 1f) ;


      for (int i = 0 ; i < aComponents.length; i++)
        if (fadeInT > aComponents[i].getTransparency() )
          aComponents[i].setTransparency(fadeInT, false);


      for (int i = 0 ; i < bComponents.length; i++)
        if (fadeInT > bComponents[i].getTransparency() )
          bComponents[i].setTransparency(fadeInT, false);
     }

      return (fadeT < 2f);

  }

   public void stop()
   {

     // BDebug.print( "DeadlockAnimation .stop called ");
      for (int i = 0 ; i < aComponents.length; i++)
        aComponents[i].setTransparency(aComponents[i].getTransparency()  , false);

      for (int i = 0 ; i < bComponents.length; i++)
        bComponents[i].setTransparency(bComponents[i].getTransparency(), false);

      fadeSound.stop();

   }

} 