package roboforge.animation.arena;

import roboforge.BArenaCanvas;
import roboforge.BArenaOrnament;
import roboforge.BArenaObjectPosition;
import roboforge.BGlobal;

import roboforge.animation.*;
import roboforge.animation.alpha.*;




public class BArenaOrnamentAnimation
  extends BAlphaChannelAnimation
{

  protected BArenaOrnament ornament;
    // The arena object to animate.

  protected BArenaCanvas canvas;
    // Arena canvas to add the ornament to.
    // May be null.

  protected boolean isCompiled;
    // Whether the ornament has been compiled for 3d.

  protected boolean isLoaded;
    // Whether the ornament has been loaded into the 3d canvas.

  protected boolean isAutoUnloading;
    // Whether the ornament is unloaded automatically
    // by this animation.


  public BArenaOrnamentAnimation(String filename)
    { this(filename, null); }

  public BArenaOrnamentAnimation(String filename, BArenaCanvas canvas)
    { this((BArenaOrnament) BArenaOrnament.loadFromFile(BGlobal.ORNAMENTPATH+ filename), canvas); }

  public BArenaOrnamentAnimation(BArenaOrnament ornament)
    { this(ornament, null); }

  public BArenaOrnamentAnimation(BArenaOrnament ornament, BArenaCanvas canvas)
  {
    super();
    setOrnament(ornament);
    this.canvas           = canvas;
    this.isCompiled       = false;
    this.isLoaded         = false;
    this.isAutoUnloading  = true;
  }


  public void setOrnament(BArenaOrnament ornament)
  {
    this.ornament = ornament;
    setAcceptor(ornament);
  }

  public BArenaOrnament getOrnament()
    { return ornament; }


  public void alphaStart(float a)
  {
    super.alphaStart(a);
    setLoaded(true);
  }


  public void alphaStop(float a)
  {
    super.alphaStop(a);

    if (isAutoUnloading)
      setLoaded(false);
  }


  public void setAutoUnloading(boolean isAutoUnloading)
    { this.isAutoUnloading = isAutoUnloading; }

  public boolean isAutoUnloading()
    { return isAutoUnloading; }

    
  public void setLoaded(boolean load)
  {
    // Abort if ornament is null.
    if (ornament == null) return;

    // If loading, ornament is added to canvas.
    if (load && !isLoaded)
      if (canvas != null)
      {
        if (!isCompiled)
        {
          canvas.loadArenaObject(ornament);
          isCompiled = true;
        }
        else
          canvas.addComponent(ornament);
      }

    // If unloading, ornament is removed from canvas.
    if (!load && isLoaded)
      if (canvas != null)
        canvas.removeComponent(ornament);

    this.isLoaded = load;
  }

  public boolean isLoaded()
    { return isLoaded; }

}