package roboforge.geometry;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;

import roboforge.BDebug;


public class BCircle3D extends Shape3D
{

  // Constructors
  public BCircle3D()
  {
    this(5);
  }

  public BCircle3D(double radius)
  {
    this(radius, 20);
  }

  public BCircle3D(double radius, int segments)
  {
    super();

    double arc = (java.lang.Math.PI * 2.0) / (double)segments;

    LineArray geometry = createGeometry( radius, segments, arc );

    setGeometry(geometry);
  }

  private static void setTransform (Transform3D tran, Vector3d axisOfRotation, double amount)
  {
    if (axisOfRotation.x == 1f)
      tran.rotX(amount);
    else if (axisOfRotation.x == -1f)
      tran.rotX(-amount);
    else if (axisOfRotation.y == 1f)
      tran.rotY(amount);
    else if (axisOfRotation.y == -1f)
      tran.rotY(-amount);
    else if (axisOfRotation.z == 1f)
      tran.rotZ(amount);
    else if (axisOfRotation.z == -1f)
      tran.rotZ(-amount);
    else
      BDebug.print ("ERROR BPieSliceD.setTransform Incorrect axisOfRotation " +  axisOfRotation );


  }

  private static Vector3f getUpVector(Vector3d axisOfRotation )
  {
    Vector3f up = new Vector3f();

    if (axisOfRotation.x == 1f)
      up.set(1,0,0);
    else if (axisOfRotation.x == -1f)
      up.set(-1,0,0);
    else if (axisOfRotation.y == 1f)
      up.set(0,1,0);
    else if (axisOfRotation.y == -1f)
      up.set(0,-1,0);
    else if (axisOfRotation.z == 1f)
      up.set(0,0,1);
    else if (axisOfRotation.z == -1f)
      up.set(0,0,-1);
    else
      BDebug.print ("ERROR BPieSliceD.getUpVector Incorrect axisOfRotation " +  axisOfRotation );

    return up;

  }

    private static Point3d getEdge(Vector3d axisOfRotation, double edgeSize )
  {
    Point3d edge = new Point3d();

    if (axisOfRotation.x == 1f)
      edge.set(0,edgeSize,0);
    else if (axisOfRotation.x == -1f)
      edge.set(0,-edgeSize,0);
    else if (axisOfRotation.y == 1f)
      edge.set(0,0,edgeSize);
    else if (axisOfRotation.y == -1f)
      edge.set(0,0,edgeSize);
    else if (axisOfRotation.z == 1f)
      edge.set(edgeSize,0,0);
    else if (axisOfRotation.z == -1f)
      edge.set(-edgeSize,0,0);
    else
      BDebug.print ("ERROR BPieSliceD.getEdge Incorrect axisOfRotation " +  axisOfRotation );

    return edge;

  }


  private static LineArray createGeometry( double radius, int segments, double arc)
  {

    LineArray geometry = new LineArray(segments * 2, LineArray.COORDINATES);

    double currentAngle = 0.0;
    double lastAngle = 0.0;

    double pointX1 = 0.0;
    double pointZ1 = 0.0;
    double pointX2 = 0.0;
    double pointZ2 = 0.0;

    // create vert array
    Point3d verts[] = new Point3d[segments * 2];

    for (int i = 0 ; i < segments; i++)
    {
      // edges
      lastAngle = currentAngle;

      currentAngle += arc;

      pointX1 = radius *  Math.sin(lastAngle);
      pointZ1 = radius *  Math.cos(lastAngle);

      pointX2 = radius *  Math.sin(currentAngle);
      pointZ2 = radius *  Math.cos(currentAngle);

      verts[(i * 2)] = new Point3d(pointX1, 0, pointZ1);
      verts[(i * 2)+ 1] = new Point3d(pointX2, 0, pointZ2);

    }

    // set normal and edges
    geometry.setCoordinates(0, verts);
    return geometry;
  }
}
