package roboforge.geometry;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.Color;
import java.awt.event.*;
import com.sun.j3d.utils.geometry.*;
import java.awt.Font;


import roboforge.*;


public class BPlanarGrid3D extends BranchGroup
{

  // Internal variables.
  // -------------------------------------------------------------

  protected LineArray majorLines = null;
  protected LineArray minorLines = null;

  protected static final int MAJOR = 0;
  protected static final int MINOR = 1;

  protected int tick         = 0;
  protected int majorIndex   = 0;
  protected int minorIndex   = 0;

  
  private static TransparencyAttributes majorAlpha =
      new TransparencyAttributes(TransparencyAttributes.FASTEST , 0.7f);

  private static TransparencyAttributes minorAlpha =
      new TransparencyAttributes(TransparencyAttributes.FASTEST , 0.9f);


  private static RenderingAttributes renderAttr = new RenderingAttributes(true, false, 0, RenderingAttributes.ALWAYS);


  // Constructor.
  // -------------------------------------------------------------

  public BPlanarGrid3D
  (
    float size,
    int majorDivisions,
    int minorDivisions,
    Color3f majorColor,
    Color3f minorColor
  )
  {
    this(size, majorDivisions, minorDivisions, majorColor, minorColor, false);
  }

  public BPlanarGrid3D
  (
    float size,
    int majorDivisions,
    int minorDivisions,
    Color3f majorColor,
    Color3f minorColor,
    boolean drawDimension
  )
  {
    int numMajorVertices = (majorDivisions + 1) * 4;
    int numMinorVertices = majorDivisions * (minorDivisions - 1) * 4;

    majorLines = new LineArray(
      numMajorVertices + 6,
      LineArray.COORDINATES);

    minorLines = new LineArray(
      numMinorVertices,
      LineArray.COORDINATES);

    // Increment sizes...
    float dMajor = size   / majorDivisions;
    float dMinor = dMajor / minorDivisions;

    // Initial vertex coordinates...
    float x1 = -(size / 2.0f);
    float cx = x1;
    float z1 = -(size / 2.0f);
    float cz = z1;
    float x2 = (size / 2.0f);
    float z2 = x2;

    int n    = (majorDivisions * minorDivisions);
    int type = MAJOR;

    for (int i=0; i <= n; i++)
    {
      // Decide what type of line to make...
      if ((tick == minorDivisions) || (tick == 0))
      {
        type = MAJOR;
        tick = 0;
      }
      else
        type = MINOR;

      tick++;

      // Make one vertical line...
      makeLine(cx,z1, cx,z2, type);

      // Make one horizontal line...
      makeLine(x1,cz, x2,cz, type);

      // Increment x & y...
      cx += dMinor;
      cz += dMinor;
    }

    if(drawDimension)
    {
      makeLine(size - dMajor + dMinor, 0, size - dMajor + dMinor, dMajor, MAJOR);
      makeLine(size - dMajor + dMinor / 2.0f, 0, size - dMajor + dMinor * 1.5f, 0, MAJOR);
      makeLine(size - dMajor + dMinor / 2.0f, dMajor, size - dMajor + dMinor * 1.5f, dMajor, MAJOR);
    }

    // hack to fix bug in j3d 1.2 where the grid lines cannot be the same colour
    float colorDiffHack = 0.01f;

    if (majorColor.equals(minorColor))
    {
      minorColor = new Color3f(   minorColor.x -  colorDiffHack,
                                  minorColor.y -  colorDiffHack,
                                  minorColor.z -  colorDiffHack   );

    }

    // Create appearance subnodes...
    
    ColoringAttributes majorColoring =
      new ColoringAttributes(majorColor, ColoringAttributes.SHADE_FLAT );

    ColoringAttributes minorColoring =
      new ColoringAttributes(minorColor, ColoringAttributes.SHADE_FLAT);


    // Create appearance nodes...

    Appearance majorAppearance = new Appearance();
    majorAppearance.setColoringAttributes(majorColoring);
    majorAppearance.setTransparencyAttributes(majorAlpha);
    majorAppearance.setRenderingAttributes(renderAttr);

    Appearance minorAppearance = new Appearance();
    minorAppearance.setColoringAttributes(minorColoring);
    minorAppearance.setTransparencyAttributes(minorAlpha);
    majorAppearance.setRenderingAttributes(renderAttr);
    // Create Shape3Ds for major/minor gridlines.

    this.addChild(new Shape3D(majorLines, majorAppearance));
    this.addChild(new Shape3D(minorLines, minorAppearance));
    if(drawDimension)
    {
      Text2D txt =
        new Text2D("5m", BGlobal.colors.getDarkShade3f(), BGlobal.FONT_NAME, 180, Font.PLAIN);

      Transform3D master = new Transform3D();
      Transform3D rotX = new Transform3D();
      Transform3D rotZ = new Transform3D();
      rotX.rotX( - Math.PI / 2);
      rotZ.rotZ(Math.PI / 2);

      master.mul(rotX);
      master.mul(rotZ);
      master.setTranslation(new Vector3f(dMajor + dMinor, 0, (dMajor / 2.0f) + (dMinor / 2.0f)));
      TransformGroup tg = new TransformGroup(master);

      tg.addChild(new Shape3D(txt.getGeometry(), txt.getAppearance()));

      this.addChild(tg);
    }

  }

  // Temp vars. for makeLine function.
  protected Point3f p1 = null;
  protected Point3f p2 = null;


  protected void makeLine(float x1, float z1, float x2, float z2, int type)
  {
    // Method to add a line to the grid.

    p1 = new Point3f(x1, 0.0f, z1);
    p2 = new Point3f(x2, 0.0f, z2);

    switch (type)
    {
      case MAJOR:
        majorLines.setCoordinate(majorIndex++, p1);
        majorLines.setCoordinate(majorIndex++, p2);
        break;

      case MINOR:
        minorLines.setCoordinate(minorIndex++, p1);
        minorLines.setCoordinate(minorIndex++, p2);
        break;
    }
  }
}

