package roboforge.net;

import java.io.Serializable;


public class BMessageHeader implements Serializable
{

  // A header is the initial part of a packet (BPacket).
  // It carries descriptive information about the packet
  // and its (optional) accompanying data.


  // Serialization ID.
  // -----------------------------------------------------------

  static final long serialVersionUID = -1L;


  // Protocol Codes...
  // -------------------------------------------------------------

  public static final int REQ = 0;
    // Request code.

  public static final int ACK = 1;
    // Acknowledgement code.

  public static final int NAK = 2;
    // Negative acknowledgement code.



  // Member variables...
  // -------------------------------------------------------------

  protected int code;
    // Protocol code (REQ/ACK/NAK).

  protected String type;
    // Indicates what type of information is contained
    // in the packet that this header belongs to.



  // Constructor...
  // -----------------------------------------------------------

  public BMessageHeader(int code, String type)
  {
    this.code        = code;
    this.type        = type;
  }


  // Accessor methods...
  // -----------------------------------------------------------

  public void setCode(int code)
    { this.code = code; }

  public int getCode()
    { return code; }

  public void setType(String type)
    { this.type = type; }

  public String getType()
    { return type; }


  public String toString()
  {
    String s;

    switch (code)
    {
      case REQ:
        s = "REQ"; break;
      case ACK:
        s = "ACK"; break;
      case NAK:
        s = "NAK"; break;
      default:
        s = "???"; break;
    }

    return s + ","+ type;
  }
}