package roboforge.net.services;

import roboforge.BDebug;
import roboforge.BProgressMonitorable;

import roboforge.net.*;
import roboforge.net.sockets.*;

import java.io.*;



public class BMonitorService extends BAbstractDataService
  implements BProgressMonitorable
{
 
  // NB: This service is NOT intended to travel over the
  // network to remote JVMs.  Therefore, configure() is
  // overridden to do nothing.
  
  protected BCountSocket  socket;
  protected boolean       isStopped;
  protected String        progressMessage;
  protected long          estimatedInput;
  protected long          estimatedOutput;


  // protected TimeoutThread timer;
    // Thread for handling transmission timeout.


  public BMonitorService()
  {
    super();
    isStopped = false;
    progressMessage = "";
  }


  // Implementation methods...
  // -----------------------------------------------------------

  public synchronized BSocket getSocket(BSocket s) throws IOException
  {
    if (isStopped)
      throw new IOException("Service stopped.");

    socket = new BCountSocket(s);
    return socket;
  }
  
  public String toString()
    { return progressMessage; }

  
  // Monitor methods...
  // -----------------------------------------------------------

  public String getProgressMessage()
    { return progressMessage; }
  
  public synchronized int getProgressPercent()
  {
    double progress  = 0;
    double totalSize = getEstimatedTotal();
    int    percent   = 0;

    if (socket != null)
      progress = socket.getTotalCount();

    if (totalSize > 0)
    {
      percent = (int) Math.round(progress / totalSize * 100.0);
      percent = Math.max(0, Math.min(percent, 100));
    }
    
    // System.out.println("p "+ progress+ " T "+ totalSize + " % "+ percent);

    return percent;
  }
  
  public synchronized long getInputCount()
    { return (socket != null) ? socket.getInputCount() : 0; }
    
  public synchronized long getOutputCount()
    { return (socket != null) ? socket.getOutputCount() : 0; }

  public synchronized long getTotalCount()
    { return (socket != null) ? socket.getTotalCount() : 0; }



  // Overridden encoding method...
  // -----------------------------------------------------------
  
  public void configure(BPresentationHeader header)
    { /* DO NOTHING. */ }


  // Inform method...
  // -----------------------------------------------------------
  
  public void inform(BServiceInfo info)
  { 
    if (info instanceof BMonitorInfo)
    {
      BMonitorInfo m = (BMonitorInfo) info;
      
      String message = m.getMessage();
      if (message != null)
        progressMessage = message;
      
      long in = m.getEstimatedInput();
      
      if (in != BMonitorInfo.NONE)
        estimatedInput = in;
      
      long out = m.getEstimatedOutput();
      
      if (out != BMonitorInfo.NONE)
        estimatedOutput = out;
    }
  }


  // Stop methods...
  // -----------------------------------------------------------

  public synchronized void setStopped(boolean stop)
  {
    // BDebug.print("BMonitorService.setStopped("+ stop+ ")");

    this.isStopped = stop;

    if (isStopped)
    {
      // Attempt to interrupt existing traffic.
      if (socket != null)
        try
        {
          // BDebug.print("BMonitorService.setStopped(): Closing socket...");
          socket.close();
        }
        catch (IOException e)
          { } // BDebug.print("BMonitorService.setStopped(): "+ e); }
        finally
          { socket = null; }

      /*
      // Interrupt timeout-thread...
      if (timer != null)
        timer.interrupt();

      BDebug.print("BMonitorService.setStopped(): done.");
      */
    }
  }

  public synchronized boolean isStopped()
    { return isStopped; }



  // Accessor methods...
  // -----------------------------------------------------------

  public void setEstimatedInput(long bytes)
    { this.estimatedInput = bytes; }

  public void setEstimatedOutput(long bytes)
    { this.estimatedOutput = bytes; }

  public long getEstimatedInput()
    { return estimatedInput; }

  public long getEstimatedOutput()
    { return estimatedOutput; }
  
  protected long getEstimatedTotal()
    { return estimatedInput+ estimatedOutput; }



  // Timeout capability...
  // -----------------------------------------------------------

  public void setTimeout(long ms)
  {
    /*
    BDebug.print("BMonitorService.setTimeout("+ ms+ ")");

    // Do nothing if no timeout period is set.
    if (ms <= 0) return;

    // Do nothing if a timer is already active.
    if (timer != null) return;

    // Spawn a timeout-thread...
    timer = new TimeoutThread(ms);
    timer.start();
    */
  }


  /*
  protected class TimeoutThread extends Thread
  {
    protected boolean isDone;
  	protected long    timeout;

  	public TimeoutThread(long ms)
    {
    	super();
      this.isDone  = false;
      this.timeout = ms;
    }

    public void run()
    {
    	long total = getTotalCount();

      BDebug.print("TimeoutThread.run(): Starting at "+total+" bytes.");

    	while (!isDone)
      {
      	// Check to see if we're finished.
      	// if (total >= getEstimatedTotal())
        //  	return;

	    	try
        {
          BDebug.print("TimeoutThread.run(): Sleeping for "+timeout+" ms...");

        	// Wait for timeout.
        	Thread.sleep(timeout);

          BDebug.print("TimeoutThread.run(): Sleep completed.");
        }
    	  catch (InterruptedException e)
        {
          BDebug.print("TimeoutThread.run(): INTERRUPTED!");
          isDone = isStopped();
          continue;
        }

        BDebug.print("TimeoutThread.run(): total="+total+", count="+getTotalCount()+ ".");

    	  if (total == getTotalCount())
        {
          BDebug.print("TimeoutThread.run(): TIMEOUT EVENT!");

        	// Timeout has occurred.
        	setStopped(true);
          isDone = isStopped();
        }

        else
        	// Remember last total.
        	total = getTotalCount();
      }

      // Cleanup...
      BDebug.print("TimeoutThread.run(): Exiting.");
      timer = null;
    }
  }
  */

}

