package roboforge.particles.emitters;

import roboforge.particles.*;

import javax.vecmath.*;
import javax.media.j3d.*;



public class BSystemEmitter3D
  extends BSystemEmitterShared
{

  protected Group parent;
    // Parent group.

  protected BranchGroup root;
    // Branch group containing the 3D form.

  protected Transform3D emitterTransform
    = new Transform3D();

  protected Point3f origin
  	= new Point3f();



  public BSystemEmitter3D(BParticleSystem s)
    { this(s, null, null, null); }

  public BSystemEmitter3D(BParticleSystem s, BranchGroup parent)
    { this(s, null, null, parent); }

  public BSystemEmitter3D
  (
    BParticleSystem s,
    BEmissionPattern ep,
    BParticleEmitter emitter,
    BranchGroup parent
  )
  {
    super(s, ep, emitter);
    this.parent = parent;

    root = new BranchGroup();
    root.setCapability(BranchGroup.ALLOW_DETACH);
    root.setCapability(BranchGroup.ALLOW_LOCAL_TO_VWORLD_READ);
    root.compile();
  }


  public void setParent(Group parent)
    { this.parent = parent; }

  public Group getParent()
    { return parent; }


  public Transform3D getEmitterTransform()
    { return emitterTransform; }

  public Point3f getOrigin()
  	{ return origin; }


  public void start()
  {
    super.start();

    if (!root.isLive())
      parent.addChild(root);

    updateEmitterInfo();
  }

  public void update(float t, float dt)
  {
    super.update(t, dt);
    updateEmitterInfo();
  }

  public void stop()
  {
    super.stop();

    // if (root.isLive())
	  root.detach();
  }


  protected void updateEmitterInfo()
  {
    if (root.isLive())
      root.getLocalToVworld(emitterTransform);

    origin.set(0,0,0);
    emitterTransform.transform(origin);
  }

}