package roboforge.particles.forms;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BRenderer;
import roboforge.BRenderable;
import roboforge.particles.*;

import javax.vecmath.*;
import javax.media.j3d.*;



public abstract class BSystemFormRenderableGeomArray extends BSystemFormRenderable
{

  // Members...
  // -------------------------------------------------------------

  protected int n;
    // Total indices available.

  protected int used;
    // The number of indices used by client form objects.


  // Java3D Members...
  // -------------------------------------------------------------

  protected Shape3D shape;
    // Shape3D - holds points geometry.

  protected GeometryArray geometry;
    // Array of geometry.

  // Appearance objects...
  protected Appearance appearance;
  protected Material material;
  protected TransparencyAttributes ta;
  protected RenderingAttributes ra;

  protected Texture currentTexture = null;
    // Current texture assigned to the quad array.



  // Constructor...
  // -------------------------------------------------------------

  public BSystemFormRenderableGeomArray(BRenderer renderer, boolean isAdditive, int n)
  {
    super(renderer, isAdditive);

    this.n        = n;
    this.used     = 0;
    this.geometry = createGeometry();
    this.shape    = createShape();
  }


  // Abstract Methods...
  // -------------------------------------------------------------

  protected abstract GeometryArray createGeometry();
    // Instantiate & initialize the geometry array.



  // Methods...
  // -------------------------------------------------------------

  protected Shape3D createShape()
  {
    Shape3D s = new Shape3D(geometry);

    createAppearance();
    s.setAppearance(appearance);

    return s;
  }


  protected void createAppearance()
  {
    appearance = new Appearance();
    appearance.setCapability(Appearance.ALLOW_TEXTURE_WRITE);

    ra = new RenderingAttributes();
    // ra.setCapability(RenderingAttributes.ALLOW_VISIBLE_WRITE);

    TextureAttributes xa = new TextureAttributes();
    // xa.setTextureMode(TextureAttributes.MODULATE);
    xa.setTextureMode(TextureAttributes.REPLACE);

    material = new Material();
    material.setLightingEnable(false);
    material.setAmbientColor(BGlobal.WHITE_3F);
    material.setDiffuseColor(BGlobal.WHITE_3F);
    material.setEmissiveColor(BGlobal.WHITE_3F);
    material.setSpecularColor(BGlobal.WHITE_3F);

    appearance.setRenderingAttributes(ra);
    appearance.setTextureAttributes(xa);
    appearance.setMaterial(material);

    // Force update of transparency attribs.
    setAdditive(isAdditive);
  }


  public void setAdditive(boolean isAdditive)
  {
    super.setAdditive(isAdditive);

    int destination = isAdditive ?
      TransparencyAttributes.BLEND_ONE :
      TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA;

    ta = new TransparencyAttributes
      (TransparencyAttributes.BLENDED,
       0.1f,
       TransparencyAttributes.BLEND_SRC_ALPHA,
       destination);

    // ta.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);

    if (appearance != null)
      appearance.setTransparencyAttributes(ta);
  }


  public void setTexture(Texture t)
  {
    if (t != currentTexture)
    {
      appearance.setTexture(t);
      currentTexture = t;
    }
  }



  // BRenderable implementation...
  // -------------------------------------------------------------

  public void render(GraphicsContext3D g)
  {
    // BDebug.print("Rendering: ta=null? "+ (ta == null)+ ", texure=null? "+ (currentTexture == null));
    // g.setAppearance(appearance);
    // g.draw(geometry);
    g.draw(shape);
  }

}