package roboforge.particles.looks;

import roboforge.BDebug;
import roboforge.BTextures;
import roboforge.particles.*;
import roboforge.particles.forms.*;

import java.util.*;
import javax.media.j3d.*;


public class BParticleLookAnimated
  extends    BParticleAspectAdapter
  implements BParticleLook
{

  public static final int FOREVER = 100000;
		// Constant for infinite looping animation.

  protected static final int STOPPED = -1;
		// Constant for stopped animation.


  protected List textures;
    // Set of textures to cycle through.

  protected int current;
  	// Index of the current texture.

  protected int loops;
  	// Number of loops to complete.

  protected int loop;
  	// Current loop index.

  protected float last;
  	// Last frame timestamp, in seconds relative to start().



  public BParticleLookAnimated(String root, int digits, String ext)
    { this(BTextures.getTextureSet(root, digits, ext), 1); }

  public BParticleLookAnimated(String root, int digits, String ext, int loops)
    { this(BTextures.getTextureSet(root, digits, ext), loops); }

  public BParticleLookAnimated(List ts)
  	{ this(ts, 1); }
    
  public BParticleLookAnimated(List ts, int loops)
  {
  	this.textures 		= (ts.size() > 0) ? ts : null;
    this.current  		= 0;
    this.loops				= loops;
  }



  public void setLoops(int loops)
  {

  	if (loops <= 0)
    	throw new IllegalArgumentException(
      	"Illegal loop count: "+ loops);

  	this.loops = loops;
  }

  public int getLoops()
  	{ return loops; }


  public void start(BParticle p)
  {
    last    		= 0f;
    current 		= 0;
		loop				= 0;

    if (textures == null)
    	{ loop = STOPPED; return; }

    Texture tex = (Texture) textures.get(0);
    p.form.setTexture(tex);
  }

  public void update(BParticle p, float t, float dt)
  {
  	if (loop == STOPPED)
    	return;

    // Measure time elapsed.
    float elapsed = t - last;
    if (elapsed < 0.05f) return;

    // Swap textures if appropriate...
    Texture tex = (Texture) textures.get(current++);
    p.form.setTexture(tex);

    // Loop if at end of texture list.
    if (current == textures.size())
    {
    	if ((loops != FOREVER) && (++loop >= loops))
      	// Stop animating.
      	loop = STOPPED;
      else
      	// Reset animation loop.
	      current = 0;
    }

    last = t;
  }

}
