package roboforge.ui;

import roboforge.BBot;
import roboforge.BDebug;
import roboforge.BGlobal;

import roboforge.ai.*;
import roboforge.ai.actions.*;
import roboforge.ai.decisions.*;
import roboforge.ai.expressions.*;
import roboforge.ai.values.*;

import roboforge.ui.ai.*;
import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.modes.*;
import roboforge.util.graphics.*;
import roboforge.util.processors.*;
import roboforge.util.io.*;
import roboforge.BWizardTemplateBot;

import java.awt.Point;

import java.io.*;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.util.Hashtable;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import java.awt.Dimension;
import javax.swing.Action;
import java.awt.dnd.*;
import java.awt.datatransfer.*;

import roboforge.BDebug;
import roboforge.ui.dnd.*;




public class BAIPanel extends BTooledPanel
  implements PropertyChangeListener
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BBot bot;
  protected BDecisionTreeAI ai;



  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "remove.icon",                BIconCache.getHSB("trash.gif"),
    "remove.tooltip",             "Remove selected nodes",
    "remove.keystroke",           KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0),
    "remove.width",               new Integer(31),
    "remove.height",              new Integer(31),

    "show_labels.icon",           BIconCache.getHSB("ai/reveal_18.gif"),
    "show_labels.tooltip",        "Show labels",
    "show_labels.width",          new Integer(31),
    "show_labels.height",         new Integer(31),

    "expand.icon",                BIconCache.getHSB("ai/expand_icon_20.gif"),
    "expand.tooltip",             "Expand or contract current node",
    "expand.width",               new Integer(31),
    "expand.height",              new Integer(31),


    "import_ai.icon",             BIconCache.getHSB("snap_to_floor.gif"),
    "import_ai.tooltip",          "Import another bot's AI.",

    "surrender.icon",             BIconCache.getHSB("ai_action_icon.gif"),
    "surrender.tooltip",          "Surrender.",
    
  });



  // Visual elements...
  // -------------------------------------------------------------

  protected BToggleButton show_labels = new BToggleButton(res, "show_labels");
  protected BTrashCanButton remove    = new BTrashCanButton(res, "remove");

  protected BButton importAI    			= new BActionButton(res, "import_ai");
  protected BButton surrender   			= new BActionButton(res, "surrender");

  protected BTitlebarLabel botName  	= new BTitlebarLabel();
  protected BDecisionTree tree;
  protected BScrollPane   scroller;

  protected BButton expand 						= new BButton(res, "expand");
  protected HSBImageIcon expandIcon 	= BIconCache.getHSB("ai/expand_icon_20.gif");
  protected HSBImageIcon contractIcon = BIconCache.getHSB("ai/contract_icon_20.gif");


  // Internal variables...
  // -------------------------------------------------------------

  //protected BDecisionTreeModel  treeModel;
  protected TreeSelectionModel  selectionModel;
  protected SelectionListener   selectionListener;
  protected DragListener        dragListener;

  protected DropTarget          dropTarget;
  protected DropTarget          dropTarget2;

  protected BAINodePanelLibrary library;
  protected BDecisionTreeNode   displayedNode;
  protected BAINodePanel        displayedNodePanel;

  protected BAINodeLibrary      nodelibrary;
  protected BBlockPanel         blockPanel;


  // Constructor.
  // -------------------------------------------------------------

  public BAIPanel
  (
    BSystemScreen parent,
    BBot bot
  )
  {
    super(parent, "AI");

    BAI botAI = bot.getAI();

    if (!(botAI instanceof BDecisionTreeAI))
      throw new IllegalArgumentException("Unrecognized bot AI type.");

    this.bot = bot;
    this.ai = (BDecisionTreeAI) botAI;

    bot.addPropertyChangeListener(this);

    tree           = new BDecisionTree(ai.getRoot());
    //treeModel       = tree.getTreeModel();

    selectionModel = tree.getSelectionModel();
    scroller       = new BScrollPane(tree);
    library        = new BAINodePanelLibrary();

    nodelibrary = new BAINodeLibrary(bot);
    blockPanel = new BBlockPanel(bot, tree);

    layoutAIPanelChildren();
    configAIPanelChildren();

    res.cleanse();
    res = null;
  }


  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutAIPanelChildren()
  {

    openToolPanel(nodelibrary);
    openToolPanel(blockPanel);

    selectToolPanel(nodelibrary);
    
    layoutToolbar();

    contentArea.addHGlued(botName);
    contentArea.addSmallMargin();
    contentArea.addHVGlued(scroller);

    scroller.setViewportBorder(new EmptyBorder(12,12,12,12));

    show_labels.setSelected(false);



  }


  protected void layoutToolbar()
  {

    toolbar.addBigMargin();
    toolbar.add(show_labels);
    toolbar.addMargin();
    toolbar.add(expand);
    toolbar.addMargin();
    toolbar.add(remove);


    toolbar.addGlue();
    
    if (BGlobal.DEVELOPER_MODE)
      toolbar.add(importAI);

    if (BGlobal.DEVELOPER_MODE)
    {
      toolbar.addSmallMargin();
      toolbar.add(surrender);
    }
  }


  protected void configAIPanelChildren()
  {
    tree.addMouseListener(new TreeClickListener());
    scroller.getVerticalScrollBar().addMouseListener( new MouseAdapter()
        {
            public void mouseReleased(MouseEvent e)
            {
              tree.repaint();
            }
        }

        );

  scroller.getHorizontalScrollBar().addMouseListener( new MouseAdapter()
        {
            public void mouseReleased(MouseEvent e)
            {
              tree.repaint();
            }
        }

        );



    botName.setText(bot.getName());

    selectionListener = new SelectionListener();
    selectionModel.addTreeSelectionListener(selectionListener);

    dragListener = new DragListener();

    dropTarget = new DropTarget(scroller, new DragListener());
   // dropTarget2 = new DropTarget(tree, new DragListener());

    remove.addActionListener(      new RemoveNodesListener());

    // check BGlobal for the state of the labels
    if(BGlobal.SHOW_AI_LABELS)
    {
      show_labels.setSelected(true);
      tree.setTextVisible(true);
    }
    else
    {
      show_labels.setSelected(false);
      tree.setTextVisible(false);
    }

    show_labels.addActionListener( new ShowLabelsListener());
    importAI.addActionListener(    new ImportAIListener());
    expand.addActionListener( new ExpandListener() );

    if (BGlobal.DEVELOPER_MODE)
    {
      BAction a = new BSurrenderAction(bot);
      //surrender.addActionListener( new AddNodeListener(new BActionNode(bot, a)));
    }

    selectNode(ai.getRoot());
    expandNode(ai.getRoot());
  }


  // UI Listeners...
  // -------------------------------------------------------------

  protected class TreeClickListener extends MouseAdapter
  {

    private void selectPanel()
    {
        if (selected[0] instanceof BNOPNode)
          selectToolPanel(nodelibrary);
        else
          selectToolPanel(displayedNodePanel);

    }

    public void mouseClicked(MouseEvent e)
    {
      if (e.getClickCount() == 2)
      {
      	expand.doClick();
      	/*
         if (displayedNodePanel != null)
            if (selected != null)
              selectPanel();
        */
      }
      else if (displayedNodePanel != null)
          if (selected != null)
            if ( !blockPanel.isShowing() )
              selectPanel();


    }

  }

  protected BDecisionTreeNode[] selected;

  // checks to see if the mouse is dragging a droppable object over the  scrollable area
  protected class DragListener implements DropTargetListener
  {
    public void dragOver (DropTargetDragEvent e)
    {
      Point mousePos = e.getLocation();

      JScrollBar scrollBarV = scroller.getVerticalScrollBar();
      JScrollBar scrollBarH = scroller.getHorizontalScrollBar();

      int currentV = scrollBarV.getValue();
      int currentH = scrollBarH.getValue();
      int incV = scrollBarV.getBlockIncrement() * 2;
      int incH = scrollBarH.getBlockIncrement() * 2;

      int incVFast = scrollBarV.getBlockIncrement() * 4;
      int incHFast = scrollBarH.getBlockIncrement() * 4;

      int scrollW = 0;
      int scrollH = 0;

      if(scrollBarV.isVisible())
        scrollW = scrollBarV.getWidth();
      if(scrollBarH.isVisible())
        scrollH = scrollBarH.getHeight();

      if(mousePos.y < 6)
        scrollBarV.setValue(currentV - incVFast);
      else if(mousePos.y < 13)
        scrollBarV.setValue(currentV - incV);

      if(mousePos.x < 6)
        scrollBarH.setValue(currentH - incHFast);
      else if(mousePos.x < 13)
        scrollBarH.setValue(currentH - incH);

      if(mousePos.y > scroller.getHeight() - 6 - scrollH)
        scrollBarV.setValue(currentV + incVFast);
      else if(mousePos.y > scroller.getHeight() - 13 - scrollH)
        scrollBarV.setValue(currentV + incV);

      if(mousePos.x > scroller.getWidth() - 6 - scrollW)
        scrollBarH.setValue(currentH + incHFast);
      else if(mousePos.x > scroller.getWidth() - 13 - scrollW)
        scrollBarH.setValue(currentH + incH);
    }

    public void dragEnter (DropTargetDragEvent e) {}
    public void dragExit (DropTargetEvent e) {}
    public void drop (DropTargetDropEvent e) {}
    public void dropActionChanged ( DropTargetDragEvent e ){}
  }

  protected class SelectionListener implements TreeSelectionListener
  {
    protected boolean enabled = true;

    public void setEnabled(boolean e)
      { enabled = e; }

    public boolean getEnabled()
      { return enabled; }

    public void valueChanged(TreeSelectionEvent e)
    {
      if (enabled)
      {
        TreePath[] paths =
          selectionModel.getSelectionPaths();

        if (paths != null)
        {
          int n = paths.length;
            selected = new BDecisionTreeNode[n];

          for (int i=0; i<n; i++)
            selected[i] = (BDecisionTreeNode) paths[i].getLastPathComponent();

          if (n == 1)
          {
            setNodePanel(selected[0]);
            //expand.setSelected(selected[0].isClosed());
          }
          else
            setNodePanel(null);

          if(selected[0].isClosed())
            expand.setIcon(expandIcon);
          else
            expand.setIcon(contractIcon);

        }
        else
          selected = null;

        setToolBarEnabledStatus(selected);


      }
    }
  }





  protected class ImportAIListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      BProcessor p = new BProcessor()
          {
            public void process()
              { openAndImport(); }
          };

      p.start();

    }

    protected void openAndImport()
    {
      try
      {
        BDebug.print ("OpenandImport started ");

        String path;

        if (bot.getType() == BWizardTemplateBot.BOT_TYPE )
          path = BGlobal.WIZARDPATH;
        else
          path = BGlobal.BOTPATH;


        BFileDialog open =
          BFileDialog.createOpenDialog(
          "open existing bot", "Open bot and plunder");

        open.setFilter(new BRoboforgeFilter(BGlobal.BOTSUFFIX));
        open.setPath(path);

        BApplication.showDialog(open);

        String filename = open.getSelectedFile();

        BBot openBot = BBot.loadFromFile(filename, false);
        openBot.getAI().init();
        BDebug.print ("Opened " + openBot);

        bot.setAI(openBot.getAI() );
        ((BDecisionTreeAI)bot.getAI()).setBot(bot);

        bot.prune();
        setStatus("New AI Imported.  You will need to save and re-load the Bot to rdit");



      }
      catch (Exception e)
        {BDebug.print ("BAIPAnel.importAI.openAndImport ERROR " + e.getMessage() ); }

    }

  }


  protected class ExpandListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      if (selected == null)
        return;

      BDecisionTreeNode node;

      node = selected[0];
      node.setClosed( !node.isClosed()  );
      ((BDecisionTreeModel)tree.getModel()).notifyTree();

      if(node.isClosed())
        expand.setIcon(expandIcon);
      else
        expand.setIcon(contractIcon);

      scroller.repaint();
      //BDebug.print("BAIPanel.ExpandListener called " );

     // tree.updateExpansion();
    }

  }


  protected class ShowLabelsListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      boolean showLabels = show_labels.isSelected();
      tree.setTextVisible(showLabels);

      if(showLabels)
        BGlobal.SHOW_AI_LABELS = true;
      else
        BGlobal.SHOW_AI_LABELS = false;

      BGlobal.save();

    }

  }

  protected class RemoveNodesListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {

      if (selected == null)
        return;

      int n = selected.length;

      if (n == 0)
        return;

      selectionListener.setEnabled(false);

      BDecisionTreeNode node, next;

      // Which node gets select after removal?
      next = (BDecisionTreeNode) selected[n-1].getNextSibling();

      if (next == null)
        next = (BDecisionTreeNode) selected[0].getPreviousSibling();

      if (next == null)
        next = (BDecisionTreeNode) selected[0].getParent();


      // (Try to) remove all selected nodes...
      for (int i=0; i<n; i++)
      {
        node = selected[i];
        if (node.isRemoveable())
          tree.getDecisionModel().removeNodeFromParent(node);
      }

      selectionListener.setEnabled(true);
      selectNode(next);
    }
  }



  public void propertyChange(PropertyChangeEvent e)
  {
    if (e.getPropertyName().equals("name"))
      botName.setText((String) e.getNewValue());
  }

  
  // Internal methods...
  // -------------------------------------------------------------

  protected void selectNode(BDecisionTreeNode node)
  {
    if (node != null)
      selectionModel.setSelectionPath(new TreePath(node.getPath()));
    else
      selectionModel.clearSelection();

  }

  protected void expandNode(BDecisionTreeNode node)
  {
    if (node != null)
      tree.expandPath(new TreePath(node.getPath()));
  }

  protected void setNodePanel(BDecisionTreeNode node)
  {

   // BDebug.print ("BAIPanel setNodePanel(BDecisionTreeNode node) " + node);

    boolean isPanelChanging = !library.getPanelClassName(node).equals( library.getPanelClassName(displayedNode) );
    boolean oldPanelWasSelected = false;

    if (displayedNodePanel != null)
    {
      oldPanelWasSelected = displayedNodePanel.isShowing();

      if (isPanelChanging)
        hideToolPanel(displayedNodePanel);
      else
        displayedNodePanel.requestDeactivate();
      
    }

    if (displayedNode != null)
      tree.getDecisionModel().nodeChanged(displayedNode);


    BAINodePanel panel =
      library.getPanelForNode(node);

    // Configure new panel...
    
    if (panel.getAI() == null)
      panel.setAI(ai);

    if (panel.getModel() == null)
      panel.setModel(tree.getDecisionModel());

    panel.setNode(node);

    
    // Show new panel...
    
    if (isPanelChanging)
    {
      openToolPanel(panel);
      //moveToolPanelToFront(panel);
      if (oldPanelWasSelected)
        selectToolPanel(panel);

    }
    else
      panel.requestActivate();
    
    // Remember new panel...
    displayedNode      = node;
    displayedNodePanel = panel;
  }

  public void onActivate()
  {
    ((BWorkshopScreen)parent).autosave();

    super.onActivate();
    tree.updateExpansion();
  }

  public void onDeactivate()
  {
    super.onDeactivate();
    //displayedNodePanel.requestDeactivate();
  }


  public void onSaveBegin()
  {
    displayedNodePanel.updateModel();
  }

  public void onSaveEnd()
    {}

  protected void setToolBarEnabledStatus(BDecisionTreeNode[] nodes)
  {
    boolean canAdd, canRemove, canExpand;

    if (nodes != null)
    {
      int n = nodes.length;

      // Can I add nodes?
      canAdd = (n == 1) ? nodes[0].canAddChildren() : false;

      // Can I remove nodes?
      canRemove = false;
      for (int i=0; i<n; i++)
        if (nodes[i].isRemoveable())
        {
          canRemove = true;
          break;
        }

      // Can I expand/contract nodes?
      canExpand = (n == 1) ? nodes[0].isCloseable() : false;
    }
    else
    {
      canAdd    = false;
      canRemove = false;
      canExpand = false;
    }

    /*
    addGroup.setEnabled(canAdd);
    addDecision.setEnabled(canAdd);
    addMove.setEnabled(canAdd);
    addChassis.setEnabled(canAdd);
    addOrient.setEnabled(canAdd);
    addMemory.setEnabled(canAdd);
    remove.setEnabled(canRemove);
    */

    expand.setEnabled(canExpand);
  }


  public void onClose()
  {
    bot = null;
    ai = null;
    res = null;
  /*  addGroup   = null;
    addDecision = null;
    addChassis  = null;
    addMove     = null;
    addOrient   = null;
    addMemory   = null;  */
    show_labels = null;
    remove      = null;

    tree = null;
    scroller =null;

 
    selectionModel = null;
    selectionListener = null;

    if (library != null)
      library.onClose();
    library = null;

    displayedNode = null;
    displayedNodePanel = null;

    nodelibrary = null;

    super.onClose();

  }

  //HACK TEMP ACCESSOR METHODS FOR TESTING
  // REMEMBER TO REMOVE

  public BDecisionTreeNode[] getSelectedTreeNodes()
  {
    return selected;
  }

  public BDecisionTree getTree()
    { return tree; }



  
}
