package roboforge.ui;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BComponentAppearance;
import roboforge.BColorPicker;
import roboforge.BPaintable;
import roboforge.BSlider;

import roboforge.util.graphics.*;
import roboforge.ui.components.*;
import roboforge.ui.modes.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;



public class BAppearanceChooser extends BToolPanel 
  implements BPaintable, PropertyChangeListener
{

  // Class variables...
  // -------------------------------------------------------------
  
  public static final int DEPTH_01 = 
    BComponentAppearance.MIN_DEPTH;
  
  public static final int DEPTH_02 = 
    (BComponentAppearance.MAX_DEPTH - BComponentAppearance.MIN_DEPTH) / 3;
  
  public static final int DEPTH_03 = 
    (BComponentAppearance.MAX_DEPTH - BComponentAppearance.MIN_DEPTH) * 2 / 3;
  
  public static final int DEPTH_04 = 
    BComponentAppearance.MAX_DEPTH;

  
  public static final BComponentAppearance MATERIAL_01 =
    new BComponentAppearance(Color.black, DEPTH_01, 
      BComponentAppearance.MIN_SHINE, "", 0f);

  public static final BComponentAppearance MATERIAL_02 =
    new BComponentAppearance(Color.black, DEPTH_02,
      (BComponentAppearance.MAX_SHINE - BComponentAppearance.MIN_SHINE) / 3, "", 0f);

  public static final BComponentAppearance MATERIAL_03 =
    new BComponentAppearance(Color.black, DEPTH_03, 
      (BComponentAppearance.MAX_SHINE - BComponentAppearance.MIN_SHINE) * 2 / 3, "", 0f);
  
  public static final BComponentAppearance MATERIAL_04 =
    new BComponentAppearance(Color.black, DEPTH_04,
      BComponentAppearance.MAX_SHINE, "", 0f);
  

  
  protected static final RenderingHints smooth = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );

  
  
  // Internal variables...
  // -------------------------------------------------------------

  protected B3DCanvas canvas;
  protected BComponentAppearance current;
  
  

  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "textureTable.columns",      new Integer(4),
    "textureLabel.v_align",      new Integer(SwingConstants.BOTTOM),
    "textureScroller.height",    new Integer(187),
    "textureScroller.border",    BUIResources.getLineBorder(colors.getDarkerShade()),
    
    "appearancePick.icon",       BIconCache.getHSB("eyedropper_icon.gif"),
    "appearancePick.tooltip",    "Appearance Pick tool",

    "appearancePaint.icon",      BIconCache.getHSB("paintbucket_icon.gif"),
    "appearancePaint.tooltip",   "Appearance Paint tool",
    
    "matt.size",                 new Dimension(30, 30),
    "matt.icon",                 BIconCache.getHSB("icon_material_01.gif"),

    "midmatt.size",              new Dimension(30, 30),
    "midmatt.icon",              BIconCache.getHSB("icon_material_02.gif"),

    "midshiny.size",             new Dimension(30, 30),
    "midshiny.icon",             BIconCache.getHSB("icon_material_03.gif"),

    "shiny.size",                new Dimension(30, 30),
    "shiny.icon",                BIconCache.getHSB("icon_material_04.gif"),
  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BColorPicker picker         = new BColorPicker(res, "picker");
  protected BFileComboBox genre         = new BFileComboBox(res, "genre");
  protected BImageTable textures        = new BImageTable(res, "textureTable");
  protected BScrollPane scroller        = new BScrollPane(res, "textureScroller", textures);


  protected BModeButton pick            = new BModeButton(res, "appearancePick");
  protected BModeButton paint           = new BModeButton(res, "appearancePaint");

  protected BModeAppearancePaint paintMode;
  protected BModeViewer mPaint;
  
  protected ButtonGroup materials       = new ButtonGroup();
  protected BMaterialButton matt        = new BMaterialButton(res, "matt",     MATERIAL_01);
  protected BMaterialButton midmatt     = new BMaterialButton(res, "midmatt",  MATERIAL_02);
  protected BMaterialButton midshiny    = new BMaterialButton(res, "midshiny", MATERIAL_03);
  protected BMaterialButton shiny       = new BMaterialButton(res, "shiny",    MATERIAL_04);
  

    

  // Constructor.
  // -------------------------------------------------------------

  public BAppearanceChooser(B3DEditingPanel parent)
  {
    super(parent, "appearance");

    canvas  = parent.getCanvas();
    current = new BComponentAppearance();
    
    layoutAppearanceChooserChildren();
    configAppearanceChooserChildren();
    
    res.cleanse();
    res = null;
    
    setAppearance(current);
  }


  
  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutAppearanceChooserChildren()
  {
    BBoxPanel box  = new BBoxPanel(BBoxPanel.X_AXIS);
    BBoxPanel thin = new BBoxPanel(BBoxPanel.Y_AXIS);
    BBoxPanel wide = new BBoxPanel(BBoxPanel.Y_AXIS);
    
    box.add(thin);
    box.addSmallMargin();
    box.add(wide);

    wide.addHGlued(genre);
    wide.addSmallMargin();
    wide.addHGlued(scroller);
    wide.addMargin();
    wide.addHVGlued(picker);

    thin.addBigMargin();
    thin.add(pick);
    thin.addSmallMargin();
    thin.add(paint);
    thin.addGlue();
    
    thin.add(matt);
    thin.addSmallMargin();
    thin.add(midmatt);
    thin.addSmallMargin();
    thin.add(midshiny);
    thin.addSmallMargin();
    thin.add(shiny);
    thin.addSmallMargin();
    
    add(box);
  
    scroller.setVerticalScrollBarPolicy
      (BScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
  }

  protected void configAppearanceChooserChildren()
  {
    configModes();
    
    picker.addActionListener(new PickerListener());
    
    materials.add(matt);
    materials.add(midmatt);
    materials.add(midshiny);
    materials.add(shiny);
    
    // Add listeners.
    genre.addActionListener(new GenreListener());

    try
    {
      // Set the genre combo-box to display available texture genres.
      genre.setSourcePath(BGlobal.TEXTUREPATH);

      // if any genres exist, display the first set in the texture window.
      if (!genre.isEmpty())
      {
        String firstGenre = genre.getFileAt(0);
        textures.setSourcePath(BGlobal.TEXTUREPATH+ firstGenre);
      }
    }
    catch (FileNotFoundException e)
      { BDebug.print("BAppearanceChooser.<init>: "+e); }  // BApplication.onAbort("BAppearanceChooser.<init>: "+e); }

    textures.addListSelectionListener(new TextureListener());
  }

  
  
  protected void configModes()
  {
    BModeAppearancePick pickMode =
      new BModeAppearancePick(canvas);

    paintMode =
      new BModeAppearancePaint(canvas, this);
      
    BModeViewer mPick  =
      new BModeViewer(canvas, pickMode);

    mPaint =
      new BModeViewer(canvas, paintMode);

    setupModeButton(pick, mPick);
    setupModeButton(paint, mPaint);

    pickMode.addPropertyChangeListener(this);
  }
  
  
  protected void setupModeButton(BModeButton b, BModeMouse mode)
    { b.setMode(mode, canvas); }
  
  
  
  // Accessor methods...
  // -------------------------------------------------------------


  public void setAppearance(BComponentAppearance ca)
  {
    current = new BComponentAppearance(ca);

    setMaterialColor(current.getColor());
    setMaterial();

    String texture = current.getTexture();
    
    //BDebug.print("TEXTURE: "+ texture);

    if (texture != null)
    {
      File f = new File(BGlobal.TEXTUREPATH+ texture);

      // If we can't find the texture file, exit.
      if (!f.exists())
        return;

      int i = texture.indexOf('/');

      String folder       = texture.substring(0, i).toLowerCase();
      String file         = texture.substring(i+1).toLowerCase();
      String currentGenre = genre.getSelectedFile();

      //BDebug.print("BREAKDOWN: folder:"+ folder+ " file:"+ file+ " currentGenre:"+ currentGenre);
      
      if (!folder.equalsIgnoreCase(currentGenre))
        genre.setSelectedFile(folder);

      textures.setSelectedFile(file);
    }
  }
  
  public BComponentAppearance getAppearance()
    { return current; }

  
  
  protected void setMaterialColor(Color c)
  {
    current.setColor(c);
      
    if (!c.equals(picker.getColor()))
      picker.setColor(c);
    
    textures.setHighlightColor(c);
    matt.setForeground(c);
    midmatt.setForeground(c);
    midshiny.setForeground(c);
    shiny.setForeground(c);
  }
  
  
  // Contrast Slider stuff...
  // ----------------------------------------------------


  protected void setMaterial()
  { 
    int depth = current.getColorDepth();
    
    if (depth < DEPTH_02)
      matt.select();
    else if (depth < DEPTH_03)
      midmatt.select();
    else if (depth < DEPTH_04)
      midshiny.select();
    else
      shiny.select();
  }


  public void setVisible(boolean v)
  {
    boolean old = isVisible();
    super.setVisible(v);

    if (v)
      if (propertyManager != null)
        propertyManager.firePropertyChange("chooser", old, v);


  }

  
  // UI Listeners...
  // -------------------------------------------------------------

  protected class PickerListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      { setMaterialColor(picker.getColor()); }
  }

  protected class GenreListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      try
      {
        String s = genre.getSelectedFile();
        textures.setSourcePath(BGlobal.TEXTUREPATH+ s);
        scroller.revalidate();
      }
      catch (FileNotFoundException ex)
        { BDebug.print("BAppearanceChooser.BGenreListener: "+e); }
    }
  }

  protected class TextureListener implements ListSelectionListener
  {
    public void valueChanged(ListSelectionEvent e)
    {
      String selected = textures.getSelectedItem();

      if (selected != null)
      {
        String folder   = genre.getSelectedFile();
        current.setTexture(folder+ "/"+ selected);
      }
      else
        current.setTexture(null);
    }
  }


  // Property change Listener...
  // -------------------------------------------------------------

  public void propertyChange(PropertyChangeEvent e)
  {
    String p = e.getPropertyName();

    if (p.equals("appearancePick"))
    {
      BComponentAppearance ca =
        (BComponentAppearance) e.getNewValue();

      setAppearance(ca);
    }
  }

  
  
  // Material Button inner class...
  // -------------------------------------------------------------
  
  protected class BMaterialButton extends BRadioButton
    implements ActionListener
  {
    
    public float shine;
    public int   depth;
    
    
    public BMaterialButton(BUIResources res, String key, BComponentAppearance ca)
    {
      super(res, key);
      
      this.shine = ca.getShine();
      this.depth = ca.getColorDepth();
      
      addActionListener(this);
    }
    
    public void initializeUI(BUIResources res, String key)
    {
      super.initializeUI(res, key);
      setForeground(colors.getMidShade());
    }
    
    public void select()
    {
      setSelected(true);
      actionPerformed(null);
    }
    
    public void actionPerformed(ActionEvent e)
    {
      if (current != null)
      {
        current.setShine(shine);
        current.setColorDepth(depth);
      }
    }
    
    public void paint(Graphics g)
    {
      if (isSelected())
      {
        Graphics2D g2D = (Graphics2D) g;
        g2D.addRenderingHints(smooth);
          
        // Paint highlight border...
        Dimension d = getSize();
        g.setColor(colors.getDarkShade());
        g.fillOval(0, 0, d.width, d.height);
      }
      
      super.paint(g);
    }
  }
  
}
