package roboforge.ui;

import roboforge.BGlobal;
import roboforge.BBot;
import roboforge.BShape3D;
import roboforge.BComponent;
import roboforge.BComponentAppearance;
import roboforge.BConnectPoint;
import roboforge.BDebug;
import roboforge.BLimb;

import roboforge.sound.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.modes.*;
import roboforge.util.graphics.*;
import roboforge.BPhysics;

import java.awt.Dimension;
import java.awt.Event;
import java.awt.event.*;
import java.util.Vector;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import javax.swing.*;


public class BBodyPanel extends B3DEditingPanel
  implements PropertyChangeListener
{

  // Internal variables...
  // -------------------------------------------------------------


  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "add.icon",                  BIconCache.getHSB("addc_icon.gif"),
    "remove.icon",               BIconCache.getHSB("removec_icon.gif"),
  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BActionButton addButton       = new BActionButton();
  protected BActionButton removeButton    = new BActionButton();


  protected BComponentLibrary  library    = null;
  protected BAppearanceChooser chooser    = null;
  protected BLimbPanel limbPanel          = null;


  // Actions...
  // -------------------------------------------------------------

  protected Action add;
  protected Action remove;
  protected Action lookup;
  protected Action compRot45Clock;
  protected Action compRot45Anti;




  // Constructor.
  // -------------------------------------------------------------

  public BBodyPanel
  (
    BWorkshopScreen parent,
    B3DBotCanvas botCanvas,
    BBot bot
  )
  {
    super(parent, "Design", botCanvas, bot);

    add    = new AddAction();
    remove = new RemoveAction();
    lookup = new LookupAction();
    compRot45Clock = new CompRot(45);
    compRot45Anti = new CompRot(-45);

    initComponentLibrary();
    initAppearanceChooser();
    initLimbPanel();
    selectToolPanel(library);
    
    layoutBodyPanelChildren();
    configBodyPanelChildren();

    res.cleanse();
    res = null;
  }


  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutBodyPanelChildren()
  {
    layoutBodyPanelToolbar();
  }

  protected void layoutBodyPanelToolbar()
  {
    toolbar.addBigMargin();
    toolbar.add(addButton);
    toolbar.addSmallMargin();
    toolbar.add(removeButton);
    toolbar.addGlue();
  }

  
  protected void configBodyPanelChildren()
  {
    // Joint rotate mode...
    BModeViewer mRotJ  = new BModeViewer(canvas,
      new BModeJointRot(canvas, BModeJointRot.PERMANENT));

    setupModeButton(rotj, mRotJ);

    // Add/Remove buttons...
    addButton.setAction(add);
    removeButton.setAction(remove);

    add.setEnabled(false);
    remove.setEnabled(false);

   // limbPanel.addPropertyChangeListener(this);
   // library.addPropertyChangeListener(this);
   // chooser.addPropertyChangeListener(this);

  }

  
  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    ((BWorkshopScreen)parent).autosave();

    if (!canvas.isActive())
      canvas.addPropertyChangeListener(this);

    canvas.setFloor(canvas.getSmallGrid());
    addItemsToContextMenu();
    
    super.onActivate();
  }

  public void onClose()
  {
    BDebug.print ("BBodyPanel onClose called");

    addButton.setAction(null);
    addButton = null;
    removeButton.setAction(null);
    removeButton = null;


    library    = null;
    chooser    = null;
    limbPanel = null;


    add = null;
    remove = null;
    lookup = null;
    compRot45Clock = null;
    compRot45Anti = null;

    super.onClose();

  }

  public void onDeactivate()
  {
    if (canvas.isActive())
      canvas.removePropertyChangeListener(this);

    removeItemsFromContextMenu();
    
    super.onDeactivate();

  }

  
  protected void addItemsToContextMenu()
  {
    // Context menu...
    BModeContextMenu context =
      BModeViewer.getSharedContextMenu();

    context.addConnectPtAction(add);
    context.addComponentAction(lookup);
    context.addComponentAction(compRot45Clock);
    context.addComponentAction(compRot45Anti);

    context.addComponentAction(remove);


  }

  protected void removeItemsFromContextMenu()
  {
    // Context menu...
    BModeContextMenu context =
      BModeViewer.getSharedContextMenu();

    context.removeConnectPtAction(add);
    context.removeComponentAction(lookup);
    context.removeComponentAction(remove);
    context.removeComponentAction(compRot45Clock);
    context.removeComponentAction(compRot45Anti);

  }
    
    

  // Initialization helper methods...
  // -------------------------------------------------------------

  protected void initComponentLibrary()
  {
    library = new BComponentLibrary(this);
    library.addPropertyChangeListener(this);
    openToolPanel(library);
  }

  protected void initAppearanceChooser()
  {
    chooser = new BAppearanceChooser(this);
    chooser.addPropertyChangeListener(this);
    openToolPanel(chooser);
  }

  protected void initLimbPanel()
  {
    limbPanel = new BLimbPanel(this);
    limbPanel.addPropertyChangeListener(this);
    openToolPanel(limbPanel);

  }


  // UI Actions...
  // -------------------------------------------------------------

  protected class AddAction extends BAction
  {

        
    public AddAction()
    {
      super("Add");
      setIcon(res.getIcon("add.icon"));
      setShortDescription("Add Component to bot.");
    }

    public void actionPerformed(ActionEvent e)
    {
      if (library.isShowing() )
        addFromComponentLibrary();
      else if (limbPanel.isShowing() )
        addFromLimbPanel();

    }
  }

  protected void addFromLimbPanel()
  {
    BComponent newParent = canvas.getSelectedComponent();

    BLimb limb =  limbPanel.getClonedLimb();
   // BLimb clonedLimb = (BLimb) limb.clone();
    //clonedLimb.getBaseComponent().setConnectPointSelected(limb.getBaseComponent().getConnectPointSelected());


    canvas.addLimbToBot(limb,newParent );

    addSound.play();
    
  }

  protected BSound addSound =
      BSoundSystem.getUISound("8-6.wav");


  protected void addFromComponentLibrary()
  {
         // Get parent component.
      BComponent newParent = canvas.getSelectedComponent();

      // Get library component.
      BComponent libChild = library.getSelectedComponent();
      BConnectPoint libPt = library.getSelectedConnectPt();

      // Get connect points for the new child component.
      BConnectPoint childPt  = libPt;
      BConnectPoint parentPt = newParent.getConnectPointSelected();

      // Get child (clone of the one displayed in component lib.)
      BComponent newChild = (BComponent) libChild.clone();

      // Add this cloned component to the bot.
      // This particular method also adds the component to the bot.
      // created for dnd integration
      if ( canvas.addComponent(newChild, childPt, newParent, parentPt) )
      {
        // newChild.cleanseLiveNodes();
        addSound.play();

        // Deselect components.
        canvas.setSelectedComponent(null);
        library.setSelectedComponent(null);

        // Add this cloned component to the canvas.
        //canvas.addComponent(newChild, newParent);

        // Reset selection status.
        canvas.setSelectedComponent(newChild);
        library.setSelectedComponent(libChild);
        library.setSelectedConnectPt(libPt);

        parent.setStatus(newChild.getName()+ " added.");

        addButton.setEnabled(false);
      }

  }


  protected class RemoveAction extends BThreadedAction
  {
    protected boolean    isRemoving;
    protected BComponent toRemove;
    
    protected BSound removeSound =
      BSoundSystem.getUISound("8-7.wav");

    
    protected BPromptDialog removeDialog =
      BPromptDialog.createYesNoDialog("Remove component?",
        "Do you wish to remove the selected component?");

    public RemoveAction()
    {
      super("Remove");
      setIcon(res.getIcon("remove.icon"));
      setShortDescription("Remove Component from bot.");
    }

    public void swingPrepAction(ActionEvent e)
    {
      toRemove = canvas.getSelectedComponent();
      parent.setStatus("Removing "+ toRemove.getName()+ "...");
    }

    public void threadedAction(ActionEvent e)
    {
      BApplication.showDialog(removeDialog);
      isRemoving = removeDialog.getResult();
      
      if (isRemoving)
      {
        removeSound.play();
        setWorking(true);
        
        canvas.removeComponent(toRemove);
        bot.removeComponent(toRemove);
      }
    }

    public void swingCleanupAction(ActionEvent e)
    {
      setWorking(false);

      parent.setStatus(isRemoving ?
        (toRemove.getName()+ " removed.") : 
        "Remove cancelled.");
    }
  }

  protected class LookupAction extends BAction
  {
    public LookupAction()
    {
      super("Open in library");
      setShortDescription("Open in the component library.");
    }

    public void actionPerformed(ActionEvent e)
    {
      BComponent c = canvas.getSelectedComponent();
      library.lookup(c);
      selectToolPanel(library);

    }
  }


  protected class CompRot extends BAction
  {
    int dx;

    public CompRot(int dx)
    {
      super(dx > 0 ? "Snap component rotate " + dx + "deg clockwise" :"Snap component rotate " + ( dx * -1) + "deg anticlockwise" );
      this.dx = dx;
    }

    public void actionPerformed(ActionEvent e)
    {
      BComponent c = canvas.getSelectedComponent();
      if (c != null)
        if (!c.getChassisLock() )
          c.snapRotateConnect(dx);
    }
  }


  // UI Listeners...
  // -------------------------------------------------------------

  public void propertyChange(PropertyChangeEvent e)
  {

    super.propertyChange(e);

    String p = e.getPropertyName();
   // BDebug.print("BBodyPanel proppertyChanged "  + p);

    if ( (canvas == null) || (library == null) || ( limbPanel == null) )
      return;

    if (p.equals("selectedComponent"))
    {

      BComponent c = canvas.getSelectedComponent();
      if (c != null)
      {
        remove.setEnabled(!c.getChassisLock());
        lookup.setEnabled(!c.isInternal());
        compRot45Clock.setEnabled(!c.getChassisLock());
        compRot45Anti.setEnabled(!c.getChassisLock());
        parent.setStatus(c.getName()+ " selected.");


      }
      else
        remove.setEnabled(false);
    }

    // Check whether to enable addButton button...
    boolean isAddEnabled;

    if (library.isShowing() )
    {
      isAddEnabled = (canvas.getSelectedConnectPt()  != null) &&  (library.getSelectedConnectPt() != null);
    }
    else if (limbPanel.isShowing() )
    {
      isAddEnabled = (canvas.getSelectedConnectPt()  != null) &&  (limbPanel.getSelectedConnectPt() != null);
    }
    else
      isAddEnabled = false;

    add.setEnabled(isAddEnabled);
  }
}
