package roboforge.ui;


import roboforge.BBot;
import roboforge.BGlobal;
import roboforge.BDebug;
import roboforge.BComponent;
import roboforge.BLimb;
import roboforge.BJoint;
import roboforge.BJointPosition;
import roboforge.BConnectPoint;

import roboforge.ai.*;
import roboforge.ai.decisions.*;

import roboforge.ui.BToolPanel;
import roboforge.ui.events.*;
import roboforge.ui.components.*;
import roboforge.ui.dnd.*;
import roboforge.ui.*;
import roboforge.ui.modes.*;
import roboforge.util.text.*;
import roboforge.util.graphics.*;

import roboforge.btree.*;
import roboforge.util.io.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.io.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;


public class BLimbPanel extends BToolPanel
{
  B3DCanvas workShopCanvas;



  protected BUIResources res = new BUIResources( new Object[]
  {
    "addLimb.icon",       BIconCache.getHSB("add_limb_30.gif"),
    "addLimb.width",      new Integer(31),
    "addLimb.height",     new Integer(31),
    "addLimb.tooltip",    "Add selected limb to limb library.",

    "removeLimb.icon",       BIconCache.getHSB("delete_limb_30.gif"),
    "removeLimb.width",      new Integer(31),
    "removeLimb.height",     new Integer(31),
    "removeLimb.tooltip",    "remove selected limb from library.",

   // "limbCanvas.width",       new Integer(310),
   // "limbCanvas.height",      new Integer(310),

    "limbViewer.width",       new Integer(310),
    "limbViewer.height",      new Integer(250),

    "3Dborder.foreground",     colors.getDarkerShade(),



  });


  protected static final DataFlavor OBJECT =
    ObjectSelection.createObjectFlavor();


  protected BLimbList limbs         = new BLimbList(res, "limbs");
  protected BScrollPane limbScroller = new BScrollPane(limbs);
  protected BLimbViewer limbViewer  = new BLimbViewer(res, "limbViewer");
  protected BButton addLimb         = new BButton(res, "addLimb");
  protected BButton removeLimb         = new BButton(res, "removeLimb");

  public BLimbPanel(B3DEditingPanel parent)
  {
    super(parent, "limbs");

    workShopCanvas  = parent.getCanvas();

    layoutLimbChildren();
    configLimbChildren();

    res.cleanse();
    res = null;
  }

  protected void layoutLimbChildren()
  {
    BBoxPanel library = new BBoxPanel(BBoxPanel.X_AXIS);
      library.addLinear(addLimb, getSmallMargin(), removeLimb, getVGlue());
      library.addSmallMargin();
      library.add(limbScroller);


    add(library);
    addSmallMargin();
    addHGlued( new BLine() );
    addSmallMargin();
    //add(canvasBorder);
    add(limbViewer);

  }

  protected void acceptAndSaveLimb(BLimb limb)
  {
    BComponent baseComponent = limb.getBaseComponent() ;

    if (baseComponent instanceof BJoint )
    {
      BJoint j = (BJoint)baseComponent;
      j.setDefaultJointPosition(new BJointPosition(0));
    }

    baseComponent.setConnectPointBase(baseComponent.getConnectPointBase());
    baseComponent.setConnectPointBase(null);

    try
    {

      limb.setName(limb.getBaseComponent().getBot().getName() + limb.getName());
      String fileName = BGlobal.LIMBPATH + BStrings.convertToFilename(limb.getName()) + BGlobal.LIMBSUFFIX ;

      File tester = new File(fileName);
      while (tester.exists() )
      {
        limb.setName( limb.getName() + " NEW");
        fileName = BGlobal.LIMBPATH + BStrings.convertToFilename(limb.getName()) + BGlobal.LIMBSUFFIX ;
        tester = new File(fileName);

      }

      limb.saveToFile(fileName );
      limbs.reload();

      limbs.setSelectedFile( BStrings.convertToFilename(limb.getName()) + BGlobal.LIMBSUFFIX );

    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }

  }

  public void onActivate()
  {
    super.onActivate();

    setVisible(this.isVisible() );
    if ( this.isVisible() )
      limbViewer.limbCanvas.repaint();
    
  }

  public void onDeactivate()
  {
    super.onDeactivate();
   // limbViewer.onDeactivate();
  }


  public void onClose()
  {
    BDebug.print ("BLimbPanel.onClose called ");
  
    limbViewer.onClose();

    limbs         = null;
    limbScroller = null;
    limbViewer  = null;
    addLimb         = null;
    removeLimb         = null;
    workShopCanvas = null;

    super.onClose();
  }
  public void setVisible(boolean b)
  {
    boolean old = isVisible();

    super.setVisible(b);

    if (limbViewer != null)
      limbViewer.setVisible(b);

    if (b)
      if (propertyManager != null)
        propertyManager.firePropertyChange("limbPanel", old, b);

    //if (b)
      //limbViewer.onActivate();
    //else
      //limbViewer.onDeactivate();
    if ( !b )
      limbs.clearSelection();

  }

   protected void configLimbChildren()
  {
    try
    {
      limbs.setSourcePath(BGlobal.LIMBPATH);
    }
    catch(Exception e)
    { e.printStackTrace(); }

    addLimb.addActionListener(new AddLimbActionListener() );
    removeLimb.addActionListener(new RemoveLimbActionListener() );

  }


  public BLimb getClonedLimb()
  {
    B3DLimbCanvas canvas = limbViewer.limbCanvas;
    BLimb activeLimb = limbViewer.selectedLimb;


    BConnectPoint cp = activeLimb.getBaseComponent().getConnectPointSelected();
    // remove conenct points
    canvas.setSelectedComponent(null);

    BLimb clonedLimb = (BLimb) activeLimb.clone();
    clonedLimb.getBaseComponent().setConnectPointSelected(cp);

    // re-add connectpoints
    canvas.setSelectedComponent(activeLimb.getBaseComponent());
    activeLimb.getBaseComponent().setConnectPointSelected(cp);

    return clonedLimb;

  }
  public BLimb getSelectedLimb()
  {
    return limbViewer.selectedLimb;

  }

  private void loadLimb(String fileName)
  {
    BLimb limb = BLimb.loadFromFile(fileName);
    if(limb != null)
      limbViewer.setLimb(limb, fileName);
  }



  protected BConnectPoint getSelectedConnectPt()
  {
    return limbViewer.limbCanvas.getSelectedConnectPt();
  }
  //----------------------------------------
  // innerClasses

  class RemoveLimbActionListener implements ActionListener
  {
    public void actionPerformed (ActionEvent e)
    {
      //blockViewer.setBlock(null, "");

      int selectedIndex = limbs.getSelectedIndex();
      if (selectedIndex != -1 )
      {
        String fileName = limbs.getSelectedFileWithPath();
        File file = new File(fileName);
        file.delete();

        limbViewer.clear();

        limbs.reload();

        if (limbs.getModel().getSize() != 0)
        {
          if (selectedIndex <limbs.getModel().getSize() )
            limbs.setSelectedIndex(selectedIndex);
          else
            limbs.setSelectedIndex(selectedIndex - 1);
        }

      }

    }
  }

  class AddLimbActionListener implements ActionListener
  {
    public void actionPerformed (ActionEvent e)
    {
      BComponent c = workShopCanvas.getSelectedComponent();
      if (c != null)
      {
            if (c.getChassisLock() )
              return;


            BComponentNode compLimb = (BComponentNode)c.getTreeNode().cloneTree();
            BLimb limb = new BLimb( compLimb.getComponent() );

            acceptAndSaveLimb(limb);



      }

    }
  }



   protected class BLimbListListener implements ListSelectionListener
  {
    public void valueChanged(ListSelectionEvent e)
    {
      String path      = BGlobal.LIMBPATH;
      String limb    =  limbs.getSelectedFile();

      if (limb != null)
        loadLimb(path+ limb);
    }
  }


  class BLimbList extends BFileList implements DropTargetListener
  {

    public BLimbList( BUIResources res, String key)
    {
      super(res, key);

      String[] extensions = new String[1];
      extensions[0] = BGlobal.LIMBSUFFIX;
      this.setFilter(new BRoboforgeFilter(extensions));

      this.addListSelectionListener(new BLimbListListener());

      new DropTarget(this,DnDConstants.ACTION_COPY_OR_MOVE, this);

    }


    protected void reload()
    {
    //  if (listener != null)
      //  this.removeListSelectionListener(listener);

      try
      {

        this.setSourcePath(BGlobal.LIMBPATH);

      }
      catch(Exception e)
        { e.printStackTrace(); }

     // if (listener != null)
       // this.addListSelectionListener(listener);



    }

    // drop events
    //-------------------------------------------------------

    public void dragEnter(DropTargetDragEvent dtde)
    {};

    public void dragExit(DropTargetEvent dte){};
    public void dragOver(DropTargetDragEvent dtde)
    {};
    public void dropActionChanged(DropTargetDragEvent dtde){};
    public void drop(DropTargetDropEvent dtde)
    {
      //BDebug.print ( "BBlockList.source " + dtde.getSource());
      //BDebug.print("B.blockList.drop " + myDrop);
        try
        {

          Transferable t = dtde.getTransferable();

          BLimb limb   = (BLimb)t.getTransferData(OBJECT);
          if (limb == limbViewer.limbCanvas.getClonedLimb() )
          {
            BDebug.print("BLimbList.clonedLimb detected");
            return;

          }
          acceptAndSaveLimb(limb);

        }
        catch(Exception e)
        {

        }

      dtde.rejectDrop();
      dtde.dropComplete(true);

    }




  }

  class BLimbViewer extends BBoxPanel
  {

    B3DLimbCanvas limbCanvas;
    BModeComponentStudy  componentRot;
    BLimb selectedLimb;
    BTextField nameField = new BTextField(res, "nameField");


    public BLimbViewer ( BUIResources res, String key)
    {
      super(  res,  key, BBoxPanel.Y_AXIS);
      layoutLimbViewerChildren();
      configureLimbViewerChildren();
    }

    protected void layoutLimbViewerChildren()
    {
      limbCanvas = new B3DLimbCanvas(res, "limbCanvas");
      
      addLinear(new BLabel("name:"), getHStrut(5), nameField);
      addSmallMargin();
      add( limbCanvas );


    }
    protected void configureLimbViewerChildren()
    {
      componentRot = new BModeComponentStudy(limbCanvas);
      limbCanvas.setMode(componentRot);


      nameField.addFocusListener( new FocusListener()
      {
            public void focusGained(FocusEvent e)
              {}

            public void focusLost(FocusEvent e)
            {
              if (selectedLimb != null)
              {
                updateSelectedLimb();
              }
            }
      });


    }

    protected void updateSelectedLimb()
    {
     if (selectedLimb != null)
      if (! nameField.getText().equals(selectedLimb.getName() ) )
      {
        try
        {
          // delete the old File
          String oldFileName =  BGlobal.LIMBPATH + BStrings.convertToFilename(selectedLimb.getName()) + BGlobal.LIMBSUFFIX;
          File oldFile = new File(oldFileName);
          oldFile.delete();

          // set new name
          selectedLimb.setName( nameField.getText() );
          String fileName = BGlobal.LIMBPATH + BStrings.convertToFilename(selectedLimb.getName()) + BGlobal.LIMBSUFFIX ;

          File tester = new File(fileName);
          while (tester.exists() )
          {
            selectedLimb.setName( selectedLimb.getName() + " NEW");
            fileName = BGlobal.LIMBPATH + BStrings.convertToFilename(selectedLimb.getName()) + BGlobal.LIMBSUFFIX ;
            tester = new File(fileName);

          }


          selectedLimb.saveToFile(fileName );
          limbs.reload();
          limbs.setSelectedFile( BStrings.convertToFilename(selectedLimb.getName()) + BGlobal.LIMBSUFFIX );

        }
        catch(Exception ex)
        {
          ex.printStackTrace();
        }

      };

    }

   public void onClose()
   {
      if (limbCanvas != null)
        limbCanvas.onClose();
        
      limbCanvas = null;
      componentRot = null;
      selectedLimb = null;
      nameField = null;

      super.onClose();
   }

    public void setVisible(boolean b)
    {
      super.setVisible(b);

      if (limbCanvas != null)
      {
        if (b)
          limbCanvas.onActivate();
        else
          limbCanvas.onDeactivate();

        limbCanvas.setVisible(b);
      }

    }

    protected void setLimb(BLimb limb,String fileName)
    {
      limbCanvas.addLimb(limb);
      if (selectedLimb != null)
        selectedLimb.cleanse();

      selectedLimb = limb;

      nameField.setText( limb.getName() );

    }

    protected void clear()
    {
      if ( limbCanvas != null )
        limbCanvas.clear();
    }



  }



}