package roboforge.ui;

import roboforge.BGlobal;
import roboforge.BProgressMonitorable;
import roboforge.ui.components.*;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class BLoadingScreen extends BBasicScreen
{

  // Class variables...
  // -------------------------------------------------------------

  protected static BLoadingScreen instance;


  // Member variables...
  // -------------------------------------------------------------

  protected Timer timer;
  protected BProgressMonitorable source;



  // UI Resources ...
  // -------------------------------------------------------------

  // were getMidShade

  protected BUIResources res = new BUIResources( new Object[]
  {
    "title.bounds",               new Rectangle(280, 307, 300, 30),
    "title.icon",                 BIconCache.getHSB("loading_title.gif"),
    "title.foreground",           colors.getMidShade(),

    "background.foreground",      colors.getMidShade(),
    "background.bounds",          new Rectangle(0, 1, 800, 450),
    
    "cogs.bounds",                new Rectangle(180, 307, 100, 100),
    "cogs.icon",                  BGraphics.getHSBIcon("anim_cogs.gif", colors.getMidShade()),

    "progress.bounds",            new Rectangle(280, 347, 300, 20),
    "message.bounds",             new Rectangle(280, 377, 300, 20),
    "message.foreground",         colors.getLighterShade(),

    "transparentBack.bounds",     new Rectangle(180, 287, 420, 120),
    "transparentBack.foreground", colors.getDarkestShade(),


  });


  // Visual elements ...
  // -------------------------------------------------------------

  protected BLabel              title       = new BLabel(res, "title");
  protected BLabel              background  = new BLabel(res, "background");
  protected BAnimatedIconLabel  cogs        = new BAnimatedIconLabel(res, "cogs");
  protected BProgressBar        progress    = new BProgressBar(res, "progress");
  protected BLabel              message     = new BLabel(res, "message");

  protected BPanel              transparentBack = new BPanel(res, "transparentBack");



  // Constructor ...
  // -------------------------------------------------------------

  public BLoadingScreen()
  {
    super();

    layoutLoadingChildren();
    configLoadingChildren();
  }


  protected void layoutLoadingChildren()
  {
    body.setLayout(null);

    body.addOnBottom(background);

    //body.addOnTop(transparentBack);
    body.addOnTop(title);
    body.addOnTop(cogs);
    body.addOnTop(progress);
    body.addOnTop(message);
  }

  protected void configLoadingChildren()
  {
    timer = new Timer(50, new TimerListener());
  }



  // Accessor methods...
  // -------------------------------------------------------------

  public void setRefreshDelay(int delay)
    { timer.setDelay(delay); }

  protected void setBackgroundIcon(Icon icon)
    { background.setIcon(icon); }


  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();

    BApplication.setBusy(true);
    timer.start();
  }

  public void onDeactivate()
  {
    timer.stop();
    BApplication.setBusy(false);

    source = null;
    setBackgroundIcon(null);
    
    super.onDeactivate();
  }



  // UI Listeners...
  // -------------------------------------------------------------

  protected class TimerListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      { updateDisplay(); }
  }


  protected void updateDisplay()
  {
    if (source != null)
    {
      int    p = source.getProgressPercent();
      String s = source.getProgressMessage();

      progress.setValue(p);
      message.setText(s);

      cogs.animation();
    }
  }

  protected void reset()
  {
    progress.setValue(0);
    message.setText("");
  }



  // Class accessor methods...
  // -------------------------------------------------------------

  public static BLoadingScreen getInstance()
  { 
    ensureInstanceExists();
    return instance;
  }

  public static void open()
  {
    ensureInstanceExists();

    Runnable openInstance = new Runnable()
      { public void run() { instance.reset();
          BApplication.openScreen(instance); }; };

    SwingUtilities.invokeLater(openInstance);
  }

  public static void close()
  {
    ensureInstanceExists();

    Runnable closeInstance = new Runnable()
      { public void run() { BApplication.hideScreen(instance); }; };

    SwingUtilities.invokeLater(closeInstance);
  }

  public static void setSource(BProgressMonitorable source)
  {
    ensureInstanceExists();
    instance.source = source;
  }


  public static void forceUpdate()
  {
    ensureInstanceExists();

    Runnable updateInstance = new Runnable()
      { public void run() { instance.updateDisplay(); }; };

    SwingUtilities.invokeLater(updateInstance);
  }


  public static void setBackdrop(Icon icon)
  {
    ensureInstanceExists();
    instance.setBackgroundIcon(icon);
  }


  // Instance loader method...
  // -------------------------------------------------------------

  protected static void ensureInstanceExists()
  {
    if (instance == null)
    {
      if (SwingUtilities.isEventDispatchThread())
        instance = new BLoadingScreen();
      else
        try
        {
          Runnable loadInstance = new Runnable()
            { public void run()
              { instance = new BLoadingScreen(); }; };

          SwingUtilities.invokeAndWait(loadInstance);
        }
        catch (Exception ex)
          { }
    }
  }

  public static void  clearStatic()
  {
    //instance = null;
  }
  
}