package roboforge.ui;

import roboforge.BArena;
import roboforge.BArenaCanvas;
import roboforge.BArenaObject;
import roboforge.BBattleRecorder;
import roboforge.BBot;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BPlaceHolder;
import roboforge.BSimulation;
import roboforge.BSimulationBehaviour;

import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.modes.*;
import roboforge.BPhysics;
import roboforge.BArenaObjectPosition;

import java.io.*;
import java.util.Vector;

import javax.swing.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import javax.vecmath.*;
import javax.media.j3d.*;


public class BTestController
  implements ChangeListener, BActivateable, BProgressive
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BArenaCanvas         canvas        = null;
  protected BBot                 bot           = null;
  protected BBot                 enemy         = null;
  protected BBot                 oldEnemy      = null;
  protected BArenaObjectPosition oldPos        = new BArenaObjectPosition();

  protected BArena               arena         = null;
  protected BSimulation          sim           = null;
  protected BSimulationBehaviour testBehavior  = null;

  protected boolean              isStopping    = false;
  protected float                duration      = 0.0f;


  // Support class for handling activation listeners...
  protected BActivationSupport activationSupport
    = new BActivationSupport(this);

  // Support class for handling property-change listeners...
  protected SwingPropertyChangeSupport propertyManager
    = new SwingPropertyChangeSupport(this);


  // Member variables...
  // -------------------------------------------------------------

  protected String               arenaFileName  = null;
  protected boolean              isSimulating   = false;
  protected BProgressListener    listener       = null;


  // Constructor...
  // -------------------------------------------------------------

  public BTestController(BArenaCanvas canvas)
  {
    this.canvas = canvas;
  }

  // Accessor methods...
  // -------------------------------------------------------------

  public void setBot(BBot bot)
  {
    this.bot = bot;
    canvas.setBotCam(bot);
  }

    // replaces only the ememy bot - used when the opponent is selected from
  // the combobox in the test panel
  public void initEnemyBot()
  {
    BApplication.setBusy(true);

    //BDebug.print ("BTestController.enemy " + enemy);
    // remove the enemy if one exists
    if(oldEnemy != null)
    {
      //BDebug.print ("BTestController.oldEnemyEntered " + oldEnemy);
      //BDebug.print ("BTestController.arena " + arena);
      //BDebug.print ("BTestController.canvas " + canvas);
      if (arena != null)
      {
        arena.removeArenaObject(oldEnemy);
        canvas.remove(oldEnemy);

        // cleanse old enemy
        // DO NOT NEED TO CLEANSE, testPanel does that
        //YLY 18-4-01
        //oldEnemy.cleanse();

      }
    }

    if (enemy != null)
    {
      //BDebug.print ("BTestController.EnemyEntered " + enemy);
      // Clean-up the bot.
      enemy.clearCurrentJointTransforms();
      canvas.loadArenaObject(enemy);

      // set the camera and the bot location
      canvas.setAutoCam(bot, enemy);
      enemy.setArenaPosition(oldPos);
      enemy.setZonesVisible(false);
      oldPos = enemy.getArenaPosition();
    }
      // save as the old enemy for future use
    oldEnemy = enemy;

    BApplication.setBusy(false);
  }



  public void setEnemy(BBot enemy)
  {

    if (this.enemy != enemy)
    {
      this.enemy = enemy;
      initEnemyBot();
    }

   //if (this.enemy != null)
    //  if (this.enemy != enemy )
   //     this.enemy.cleanse();

   // do not need to cleanse, The test Panle is responsible fopr cleansing

  }

  public BArena getArena()
    { return arena; }

  public BBot getBot()
    { return bot; }

  public BBot getEnemy()
    { return enemy; }

  public void setArenaFileName(String name)
    { arenaFileName = name; }

  public float getElapsedTime()
    { return testBehavior.getElapsedSimTime(); }

  // Activation methods...
  // -------------------------------------------------------------

  public boolean isActive()
    { return activationSupport.isActive(); }

  public boolean isClosed()
    { return activationSupport.isClosed(); }


  public void requestActivate()
    { activationSupport.requestActivate(); }

  public void requestDeactivate()
    { activationSupport.requestDeactivate(); }

  public void requestClose()
    { activationSupport.requestClose(); }


  public void onActivate()
  {
    // Set this global to ensure heights are correct.
    BGlobal.HEIGHT_SCALE = BGlobal.ARENA_SCALE;
    BGlobal.MOVE_FLOOR = false;

    // Create the move behavior...
    testBehavior = new BSimulationBehaviour();
    testBehavior.addChangeListener(this);

    // Notify listeners that we're active.
    activationSupport.fireActivate();
  }

  public void onDeactivate()
  {
    if (this.isActive())
      doSafeCleanup();

    // Notify listeners that we're deactivated.
    activationSupport.fireDeactivate();
  }

  public void onClose()
  {
    BDebug.print("BTestController.onCloseCalled");
    
    if (this.isActive())
      doSafeCleanup();

    bot    = null;
    enemy  = null;
    canvas = null;

    // Notify listeners that we're close.
    activationSupport.fireClose();
  }


  protected void doSafeCleanup()
  {
    // Ask controller to stop playing.
    stop();

    // canvas.cleanse();
    canvas.clear();

    // Cleanse existing simulation, if any.
    if (sim != null)
      sim.cleanse();


    // Clean up resources...
    testBehavior = null;
    arena        = null;
    sim          = null;
  }


  // Listener methods...
  // -------------------------------------------------------------

  public void addActivationListener(BActivationListener l)
    { activationSupport.addActivationListener(l); }

  public void removeActivationListener(BActivationListener l)
    { activationSupport.addActivationListener(l); }


  public void addProgressListener(BProgressListener p)
    { listener = p; }

  public void removeProgressListener(BProgressListener p)
    { listener = null; }

  public String getProgressDescriptor()
    { return "Running test simulation..."; }



  // Player commands...
  // -------------------------------------------------------------

  // initialize the simulation
  public void init()
  {
      BApplication.setBusy(true);

      // Clear the canvas of any existing objects...
      canvas.clear();

      // Create an arena in which to do simulation.
      arena = BArena.loadFromFile(arenaFileName, false);

      canvas.setArenaSize( arena.getSize());

      stripOutArenaDetails();

      addBotToArena(bot);
      addBotToCanvas(bot);

      addBotToArena(enemy);
      addBotToCanvas(enemy);
      enemy.setZonesVisible(false);

      oldEnemy = enemy;
      oldPos = enemy.getArenaPosition();

      BApplication.setBusy(false);

      canvas.setMode(BGlobal.CANVAS_MODE_ARENA_AUTOCAM);
      canvas.setAutoCam(bot, enemy);
      canvas.updateCamera();
  }


  // start up the simulation
  public void play(float duration)
  {
    if (!isSimulating)
    {
       isSimulating = true;
       this.duration = duration;

       canvas.addBehavior(testBehavior);

       if (sim != null)
         sim.cleanse();

        // Create the simulation.
       sim = new BSimulation(bot, enemy, arena, duration, canvas, true, false);
       testBehavior.start(sim);
       propertyManager.firePropertyChange("playing", false, true);
    }
  }


  public boolean isPlaying()
    { return (isSimulating); }

  public void pauseSim(boolean paused)
  {
    if (testBehavior != null)
      testBehavior.setPaused(paused);
  }

  public void stop()
  {
    if (isSimulating)
    {
      // Signal that we wish to deallocate resources.
      isStopping = true;

      // Request behaviour stoppage.
      testBehavior.stopNextLoop();

      // Block until stop is complete.
      while (isStopping)
        try { Thread.sleep(50); }
        catch
          (InterruptedException e)
            { BDebug.print("stop(): blocking was interrupted..."); }

      // Behavior will generate a ChangeEvent when it's ready to stop.
      // We listen for ChangeEvents, calling onBehaviorStopped() upon receipt.
    }
  }

  public void onBehaviorStopped()
  {
    // Signal that stopping is complete...
    // [used by stop().]
    if (isStopping && isActive())
      isStopping = false;

    // Update our status on this end.
    isSimulating = false;

    // Remove the test behaviour from canvas.
   // BDebug.print ("BTestController.removing simBehaviour");
    canvas.removeBehavior(testBehavior);


    propertyManager.firePropertyChange("playing", true, false);

    // Update our status on this end.
   // isSimulating = false;

  }


  public BBattleRecorder getBattleRecorder()
  {
    if (sim != null)
      return sim.getBattleRecorder();
    else
      return null;
  }



  // Internal methods...
  // -------------------------------------------------------------

  protected void stripOutArenaDetails()
  {
    // Remove anena ornaments and cache them
    Vector arenaObjects = arena.getArenaObjects(false);
    BArenaObject a = null;

    for (int i=0; i<arenaObjects.size(); i++)
    {
      a = (BArenaObject) arenaObjects.get(i);
      arena.removeArenaObject(a);
    }
  }

  protected void addBotToArena(BBot b)
  {
    // Clean-up the bot.
    //b.cleanseLiveNodes();
    b.clearCurrentJointTransforms();

    // Place the Bot into the first spare slot in the arena.
    Vector v = arena.getPlaceHolders(BPlaceHolder.TYPE_BOT);
    arena.addArenaObject((BPlaceHolder) v.firstElement(), b);
  }

  protected void addBotToCanvas(BBot b)
  {
    /*
    boolean tStub  = BGlobal.SHOW_TEXTURES;
    boolean sStub  = BGlobal.SHOW_SENSORLIGHTS ;
    boolean trStub = BGlobal.USE_TRANSPARENCY ;

    BGlobal.SHOW_TEXTURES     = false;
    BGlobal.SHOW_SENSORLIGHTS = false;
    BGlobal.USE_TRANSPARENCY  = false;
    */

    // Update the canvas
    canvas.loadArenaObject(b);

    /*
    BGlobal.SHOW_TEXTURES = tStub ;
    BGlobal.SHOW_SENSORLIGHTS = sStub ;
    BGlobal.SHOW_SENSORLIGHTS = trStub ;
    */
  }



  // Listeners...
  // -------------------------------------------------------------

  protected BProgressEvent  progressEvent
    = new BProgressEvent(this, 0.0f);


  public void stateChanged(ChangeEvent e)
  {
    float progress = 1.0f;

    // Check if behavior has stopped simulating...
    if (!testBehavior.isRunningSimulation())
      onBehaviorStopped();
    else
    {
      canvas.updateCamera();

      progress = testBehavior.getElapsedSimTime() / duration;
    }

    if (listener != null)
    {
      progressEvent.setProgress(progress);
      listener.progressUpdated(progressEvent);
    }
  }

  // Property change listeners
  // -------------------------------------------------------------

  public void addPropertyChangeListener(PropertyChangeListener listener)
    { propertyManager.addPropertyChangeListener(listener); }

  public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
    { propertyManager.addPropertyChangeListener(propertyName, listener); }

  public void removePropertyChangeListener(PropertyChangeListener listener)
    { propertyManager.removePropertyChangeListener(listener); }

  public void removePropertyChangeListener( String propertyName, PropertyChangeListener listener)
   { propertyManager.removePropertyChangeListener(propertyName, listener); }

}