package roboforge.ui;

import roboforge.BDebug;
import roboforge.BGlobal;

import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.layers.*;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.SwingPropertyChangeSupport;


public class BToolPanel extends BBoxLayer
  implements SwingConstants
{

  // Member variables...
  // -------------------------------------------------------------

  protected String title = null;

  // The UI color scheme - used to get UI colors.
  protected static BUIColorScheme colors = BGlobal.colors;


  // Internal variables...
  // -------------------------------------------------------------

  protected BTooledPanel parent = null;
  protected SwingPropertyChangeSupport propertyManager;
  


  // Constructor...
  // -------------------------------------------------------------

  public BToolPanel(String title)
    { this(null, title); }

  public BToolPanel(BTooledPanel parent, String title)
  {
    super(BBoxPanel.Y_AXIS);

    this.parent = parent;

    setTitle(title);
    setBorder(new EmptyBorder(SMALL_MARGIN,0,0,0));
    setOpaque(true);

    // Initialize property manager...
    propertyManager = new SwingPropertyChangeSupport(this);
  }


  // Property change listener methods...
  // -------------------------------------------------------------

  public void addPropertyChangeListener(PropertyChangeListener listener)
  {
    if (propertyManager != null)
      propertyManager.addPropertyChangeListener(listener);
  }

  public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
  {
    if (propertyManager != null)
      propertyManager.addPropertyChangeListener(propertyName, listener);
  }

  public void removePropertyChangeListener(PropertyChangeListener listener)
  {
    if (propertyManager != null)
      propertyManager.removePropertyChangeListener(listener);
  }

  public void removePropertyChangeListener( String propertyName, PropertyChangeListener listener)
  {
    if (propertyManager != null)
      propertyManager.removePropertyChangeListener(propertyName, listener);
  }

  

  // Accessor methods...
  // -------------------------------------------------------------

  public void setTitle(String title)
    { this.title = title; }

  public String getTitle()
    { return title; }

  public void setParent(BTooledPanel p)
    { this.parent = p; }


  public void onClose()
  {
      title = null;

      parent = null;
      propertyManager = null;

      super.onClose();

  }
  

  // Layer methods...
  // -------------------------------------------------------------

  public void addTo(JComponent holder)
  {
    BTabbedPane pane =
      (BTabbedPane) holder;

    pane.add(title, this);
  }

  public void removeFrom(JComponent holder)
    { holder.remove(this); }


}
