package roboforge.ui;

import roboforge.BGlobal;
import roboforge.BDebug;

import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.layers.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.awt.*;
import java.util.Vector;
import java.awt.event.*;
import javax.swing.*;

public class BWindow extends JWindow
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BLayerStack stack;
    // Layer stack, for organizing screens.

  protected JComponent contentPanel = null;
    // The content panel for this JWindow.

  protected BGlassPane glassPane    = null;
    // Invisible pane that sits above the content.

  protected BDialog currentModalDialog = null;
    // The currently displayed modal dialog.

  protected static Cursor defaultCursor
    = new Cursor(Cursor.DEFAULT_CURSOR);
    // The default cursor.

  // private boolean isFirstPaint = true;
    // Flag for one-time painting of window backdrop.



  // Constructor.
  // -------------------------------------------------------------

  public BWindow()
  {
    // Set window color to black
    // TODO: See if this can be done by L&F automatically.
    setBackground(Color.black);

    // Make window fullscreen
    setSize(BGlobal.SCREEN_RESOLUTION);
this.
    // Set cursor to the default...
    setCursor(defaultCursor);

    // Turn on double buffering...
    contentPanel = (JComponent) getContentPane();
    contentPanel.setDoubleBuffered(true);

    // Construct screen manager...
    stack = new BSingleLayerStack(contentPanel);
    stack.onActivate();

    // Setup glasspane.
    glassPane = new BGlassPane();
    setGlassPane(glassPane);
  }


  // Input methods...
  // -------------------------------------------------------------

  public void setInputEnabled(boolean enabled)
    { glassPane.setVisible(!enabled); }


  // Focus workaround...
  // -------------------------------------------------------------

  protected void dispatchActivateEvent()
  {
    // SJS [11/03/2000]:

    // JTextFields don't seem to get focus in a JWindow correctly.
    // This workaround dispatches WINDOW_ACTIVATED events to the
    // Window at opportune moments.

    // See Sun's bug database for details:
    // * http://developer.java.sun.com/developer/bugParade/bugs/4254005.html
    // * http://developer.java.sun.com/developer/bugParade/bugs/4186928.html

    dispatchEvent(new WindowEvent(this,
      WindowEvent.WINDOW_ACTIVATED));
  }


  // Activation methods...
  // -------------------------------------------------------------

  public void onClose()
    { stack.requestClose(); }


  // Action methods...
  // -------------------------------------------------------------

  public void update(Graphics g)
  {
    if (isShowing())
      { paint(g); }
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void push(BScreen screen)
  {
    stack.push(screen);

    // Force a layout.
    contentPanel.validate();
    repaint();

    // BUG WORKAROUND: Dispatch an window ACTIVATE event.
    // Read the method comments for more info.
    dispatchActivateEvent();
  }

  public BScreen pop(boolean close)
    { return (BScreen) stack.pop(close);  }

  public BScreen peek()
    { return (BScreen) stack.peek(); }

  public boolean remove(BScreen screen, boolean close)
    { return stack.remove(screen, close); }



  // Dialog methods...
  // -------------------------------------------------------------

  public void showDialog(BDialog dialog)
  {
    if (currentModalDialog == null)
    {
      // If dialog is modal, ignore user input..
      if (dialog.isModal())
      {
        currentModalDialog = dialog;
        setInputEnabled(false);
      }

      dialog.show();
      dialog.requestActivate();

      if (dialog.isBlocking())
      {
        // Block invoking thread until dialog closes...
        // TODO: Change to dialog.block(), or dialog.showAndWait();
        //       Use thread lock wait() & notify().
        
        try
        {
          Thread t = Thread.currentThread();

          while (dialog.isActive())
            { t.sleep(50); }
        }
        catch (InterruptedException e)
          { }
      }
    }
  }

  public void hideDialog(BDialog d)
  {
    if (d != null)
    {
      d.requestDeactivate();

      if (d.isModal())
      {
        d.dispose();

        // modalPane.remove(d);
        currentModalDialog = null;

        // Allow input methods.
        setInputEnabled(true);

        // BUG WORKAROUND: Dispatch an window ACTIVATE event.
        // Read the method comments for more info.
        dispatchActivateEvent();

       // Get focus for screen...
       peek().requestFocus();
      }
      else
        { d.dispose(); }

    }
  }



  // Cursor methods...
  // -------------------------------------------------------------

  public void setBusyCursor()
    { setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)); }

  public void setCursor()
    { setCursor(Cursor.getDefaultCursor()); }



  // Glass pane class...
  // -------------------------------------------------------------

  protected class BGlassPane extends JPanel
  {
    protected boolean consuming = false;

    public BGlassPane()
    {
      super();
      setOpaque(false);
      setLayout(null);

      super.addMouseListener(new MouseAdapter() {});
      super.addMouseMotionListener(new MouseMotionAdapter() {});
      super.addKeyListener(new KeyAdapter() {});
    }

    public void addMouseListener(MouseListener l)
      { /* Ignore listener registration.. */ }

    public void addMouseMotionListener(MouseMotionListener l)
      { /* Ignore listener registration.. */ }
  }

}

