package roboforge.ui.action;

import roboforge.util.processors.*;
import roboforge.ui.components.*;

import java.awt.event.*;
import javax.swing.*;


public class BThreadedAction extends BAction
{

  protected BSingleObjectProcessor processor;


  public BThreadedAction()
    { super(); init(); }

  public BThreadedAction(String name)
    { super(name); init(); }

  public BThreadedAction(String name, Icon icon)
    { super(name, icon); init(); }

  public BThreadedAction(BUIResources res, String key)
    { super(res, key); init(); }


  protected void init()
    { processor = new ActionProcessor(); }

  public final void actionPerformed(ActionEvent e)
  {
    if (SwingUtilities.isEventDispatchThread())
      swingPrepAction(e);
    else 
      try
        { SwingUtilities.invokeAndWait(new SwingPrep(e)); }
      catch (Exception ex)
        { }
      
    processor.request(e);
  }



  public void swingPrepAction(ActionEvent e)
    { }

  public void threadedAction(ActionEvent e)
    { }

  public void swingCleanupAction(ActionEvent e)
    { }



  protected class ActionProcessor
    extends BSingleObjectProcessor
  {

    public ActionProcessor()
      { super(Thread.MIN_PRIORITY); }

    public void processObject(Object o)
    {
      ActionEvent e = (ActionEvent) o;
      threadedAction(e);

      SwingUtilities.invokeLater(
        new SwingCleanup(e));
    }
  }


  protected class SwingPrep implements Runnable
  {
    protected ActionEvent event;

    public SwingPrep(ActionEvent e)
      { this.event = e; }

    public void run()
      { swingPrepAction(event); }
  }

  
  protected class SwingCleanup implements Runnable
  {
    protected ActionEvent event;

    public SwingCleanup(ActionEvent e)
      { this.event = e; }

    public void run()
      { swingCleanupAction(event); }
  }

} 