package roboforge.ui.ai;

import roboforge.ui.fusion.*;
import roboforge.BDebug;
import roboforge.sound.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.tree.*;
import roboforge.ai.decisions.*;

import roboforge.BGlobal;



public class BAITreeUI extends FusionTreeUI
{

  private static int ROWS_PER_ARROW = 3;
  private int[] arrowX = new int[3];
  private int[] arrowY = new int[3];
  private static int HALF_WIDTH = 4; // half width of the arrow

  protected static RenderingHints hints = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );

  protected static Color visitedNodeLinkColor = BGlobal.colors.getLightestShade();
  protected static Color nonVisitedNodeLinkColor = BGlobal.colors.getDarkShade();


  public BAITreeUI()
  {
    super();



  }

   protected void installDefaults()
  {

    super.installDefaults();
    setExpandedIcon(null);
    setCollapsedIcon(null);

  }

  public static ComponentUI createUI(JComponent b)
    { return new BAITreeUI(); }


  protected Color getLineColor(TreePath path)
  {
    boolean allSelected = true;

    for (int i = path.getPathCount() -1; i >=0; i--)
    {
      BDecisionTreeNode node = (BDecisionTreeNode)path.getPathComponent(i);
      //BDebug.print ("BAITreeUI.getLineColor checking " + node + " visited " + node.getVisited());

      if (!node.getVisited() )
        allSelected = false;

    }

    if (allSelected)
      return visitedNodeLinkColor;
    else
      return nonVisitedNodeLinkColor;


  }

   protected Color getLegColor(TreePath path)
  {
    return getLineColor(path);

  }

  protected void paintVerticalPartOfLeg
  (
    Graphics g, Rectangle clipBounds,
    Insets insets, TreePath path
  )
  {
    int lineX = ((path.getPathCount() - 1 + depthOffset) * totalChildIndent) + 8 + insets.left;

  	int clipLeft = clipBounds.x;
	  int clipRight = clipBounds.x + (clipBounds.width - 1);

  	if (lineX > clipLeft && lineX < clipRight)
    {
	    int clipTop = clipBounds.y;
	    int clipBottom = clipBounds.y + clipBounds.height;
	    Rectangle parentBounds = getPathBounds(tree, path);
	    Rectangle lastChildBounds = getPathBounds(tree, getLastChildPath(path));

	    if(lastChildBounds == null)
    		// This shouldn't happen, but if the model is modified
    		// in another thread it is possible for this to happen.
		    // Swing isn't multithreaded, but I'll add this check in
    		// anyway.
		    return;

	    int top;

	    if (parentBounds == null)
        {	top = Math.max(insets.top + getVerticalLegBuffer(), clipTop); }
	    else
    		top = Math.max(parentBounds.y + parentBounds.height +
			          getVerticalLegBuffer(), clipTop);

	    if (path.getPathCount() == 1 && !isRootVisible())
      {
    		TreeModel model = getModel();

    		if (model != null)
        {
  		    Object root = model.getRoot();

  		    if(model.getChildCount(root) > 0)
          {
      			parentBounds = getPathBounds(tree, path.
			  	  pathByAddingChild(model.getChild(root, 0)));

      			if(parentBounds != null)
			        top = Math.max(insets.top + getVerticalLegBuffer(), parentBounds.y + parentBounds.height / 2);
  		    }
    		}
	    }


      int bottom = Math.min(lastChildBounds.y + (lastChildBounds.height / 2), clipBottom);

      // color segments for battle viewer
      TreeNode deepNode = (TreeNode)path.getLastPathComponent();
      TreePath extendedpath = path.pathByAddingChild( deepNode.getChildAt(deepNode.getChildCount()-1 )     );

      Color segmentColor = getLineColor(extendedpath);
      g.setColor(segmentColor);
      paintVerticalLine(g, tree, lineX, top, bottom);

      if (segmentColor == visitedNodeLinkColor)
      {
        bottom = Math.min(lastChildBounds.y + (lastChildBounds.height / 2), clipBottom);
        int currentArrow = getNextArrowPos (top);
        while (currentArrow < bottom + HALF_WIDTH)
        {
          drawArrow( g,  lineX, currentArrow);
          currentArrow = currentArrow + (this.getRowHeight() * ROWS_PER_ARROW);
        }

      }
      else
        for (int i =  deepNode.getChildCount()-2; i >= 0 &&(segmentColor == nonVisitedNodeLinkColor) ; i-- )
        {
          BDecisionTreeNode child = (BDecisionTreeNode)deepNode.getChildAt(i);
          extendedpath = path.pathByAddingChild( child )     ;
          Rectangle childBounds = getPathBounds(tree, extendedpath);
          bottom = Math.min(childBounds.y + (childBounds.height / 2), clipBottom);
          segmentColor = getLineColor(extendedpath);
          g.setColor(segmentColor);
          paintVerticalLine(g, tree, lineX, top, bottom    );

          int currentArrow = getNextArrowPos (top);
          while (currentArrow < bottom + HALF_WIDTH)
          {
            drawArrow( g,  lineX, currentArrow);
            currentArrow = currentArrow + (this.getRowHeight() * ROWS_PER_ARROW);
          }
        }



  	}
  }

  protected int getNextArrowPos (int y)
  {
    int arrowGap = this.getRowHeight() * ROWS_PER_ARROW;
    return ( (y / arrowGap ) * arrowGap) + arrowGap;
  }

  protected void drawArrow(Graphics g, int x, int y)
  {
    if (y > 0 )
    {
      ((Graphics2D)g).setRenderingHints(hints);

      alignArrow(x, y);

      //g.setColor(BGlobal.colors.getDarkShade());
      g.fillPolygon(arrowX, arrowY, 3);
    }
  }


  protected void alignArrow(int x, int half)
  {

   // BDebug.print (" half " + half);

    //left Top
    arrowX[0] = x - HALF_WIDTH;
    arrowY[0] = half - HALF_WIDTH;

    // right top

    arrowX[1] = x + HALF_WIDTH;
    arrowY[1] = half - HALF_WIDTH;
    //bottom

    arrowX[2] = x;
    arrowY[2] = half + HALF_WIDTH ;

  }


}