package roboforge.ui.ai;

import roboforge.BGlobal;
import roboforge.BDebug;

import roboforge.ai.decisions.*;
import roboforge.ui.components.BLabel;
import roboforge.ui.components.BCompoundIcon;
import roboforge.ui.components.BUIResources;

import roboforge.util.graphics.*;

import java.awt.Color;
import java.awt.Component;

import javax.swing.Icon;
import javax.swing.JTree;
import javax.swing.tree.*;

import java.awt.*;



public class BDecisionTreeRenderer extends DefaultTreeCellRenderer
{

  // Class variables...
  // -------------------------------------------------------------

  protected static Color color_normal =
    BGlobal.colors.getDarkShade();

  protected static Color color_selected =
    BGlobal.colors.getWhiteShade();

  protected static Icon yes_normal =
    BGraphics.getHSBIcon("ai_icon_yes.gif", color_normal);
  protected static Icon yes_normal2 =
    BGraphics.getHSBIcon("ai_icon_yes2.gif", color_normal);

  protected static Icon no_normal =
    BGraphics.getHSBIcon("ai_icon_no.gif",  color_normal);
  protected static Icon no_normal2 =
    BGraphics.getHSBIcon("ai_icon_no2.gif",  color_normal);

  protected static Icon yes_selected2 =
    BGraphics.getHSBIcon("ai_icon_yes2.gif", color_selected);

  protected static Icon no_selected2 =
    BGraphics.getHSBIcon("ai_icon_no2.gif", color_selected);

    protected static Icon yes_selected =
    BGraphics.getHSBIcon("ai_icon_yes.gif", color_selected);

  protected static Icon no_selected =
    BGraphics.getHSBIcon("ai_icon_no.gif", color_selected);


  protected BCompoundIcon compound =
    new BCompoundIcon();


  public static final int CELLHEIGHT = 34;
  public static final int CELLWIDTH= 200;

//  private static int MAX_TEXT_LENGTH = 40;   
  public static Dimension d = new Dimension(CELLWIDTH, CELLHEIGHT);


  //private static final Font nodeFont =  new Font("Eurostile", Font.PLAIN, 10);


  protected boolean showText = false;
  protected boolean highlightErrors = true;

  // Constructors...
  // -------------------------------------------------------------

  public BDecisionTreeRenderer()
  {
    super();
    highlightErrors = true;
  }

  // Accessor methods...
  // -------------------------------------------------------------

  public boolean getTextVisible()
    { return showText; }

  public void setTextVisible(boolean b)
    {showText = b;}

  public void setHighlightErrors(boolean b)
  {
    highlightErrors = b;
  }

  public Component getTreeCellRendererComponent
  (
    JTree tree, Object value,
    boolean isSelected, boolean expanded,
		boolean leaf, int row, boolean hasFocus
  )
  {
    if ( !(value instanceof BDecisionTreeNode))
      throw new IllegalArgumentException(  "BDecisionTreeRenderer.getTreeCellRendererComponent(): "+
        "value is not an instance of BDecisionTreeNode.");

   // setFont(nodeFont);
        
    BDecisionTreeNode node =
      (BDecisionTreeNode) value;

    BDecisionTreeNode parent =
      (BDecisionTreeNode) node.getParent();

    String name = node.getName();
  //
    if (showText)
      setText(name);
    else
      setText("");

    BDecisionTreeIcon nodeIcon;

    if (isSelected || node.getVisited() )
    {
      nodeIcon = (BDecisionTreeIcon) node.getSelectedIcon();
      setForeground(getTextSelectionColor());
    }
    else
    {
      nodeIcon = (BDecisionTreeIcon) node.getIcon();

      Color textColor = getTextNonSelectionColor();

      if (highlightErrors)
        if (!node.isValid() )
        textColor = BGlobal.WARNING_COLOR;

      setForeground(textColor);
    }


    nodeIcon.setClosed(node.isClosed());

    if (parent instanceof BDecisionNode)
    {
      int i = parent.getIndex(node);
      boolean groupNotExpanded = true;

      groupNotExpanded = node.isClosed();

      switch(i)
      {
        case 0:

        //  if (isSelected)
        //    compound.setFirstIcon(yes_selected);
        //  else

         if(leaf || groupNotExpanded)
         {
            if (node.getVisited() )
              compound.setFirstIcon(yes_selected);
            else
              compound.setFirstIcon(yes_normal);
         }
         else
         {
            BDecisionTreeNode firstChild = (BDecisionTreeNode) node.getChildAt(0);

            if (firstChild.getVisited())
              compound.setFirstIcon(yes_selected2);
            else if (node.getVisited() )
              compound.setFirstIcon(yes_selected);
            else
              compound.setFirstIcon(yes_normal2);


         }
          break;

        case 1:
        //  if (isSelected)
        //    compound.setFirstIcon(no_selected);
        //  else
         if(leaf || groupNotExpanded)
         {
            if (node.getVisited() )
              compound.setFirstIcon(no_selected);
            else
              compound.setFirstIcon(no_normal);
         }
         else
         {
            BDecisionTreeNode firstChild = (BDecisionTreeNode) node.getChildAt(0);

            if (firstChild.getVisited())
              compound.setFirstIcon(no_selected2);
            else if (node.getVisited() )
              compound.setFirstIcon(no_selected);
            else
              compound.setFirstIcon(no_normal2);
         }
          break;
      }

      compound.setSecondIcon(nodeIcon);
      setIcon(compound);
    }
    else
      setIcon(nodeIcon);

  	this.selected = isSelected;

    if (showText)
      this.setPreferredSize(d);
    else
      this.setPreferredSize(null);


    return this;
	}







}