package roboforge.ui.ai;

import roboforge.BBot;
import roboforge.BGlobal;

import roboforge.ai.BDecisionTreeAI;
import roboforge.ai.actions.BOrientAction;
import roboforge.ai.decisions.*;
import roboforge.ai.orient.*;

import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.util.graphics.*;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.event.*;


public class BOrientActionNodePanel extends BAINodePanel
{

  // Class variables...
  // -------------------------------------------------------------

  protected static BAINodePanel instance = null;
    // A single instance of this panel type.
    // Returned by getInstance().


  // Member variables...
  // -------------------------------------------------------------

  protected BOrientActionNode node;

  protected OrientTypeListener listener;


  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIColorScheme colors = BGlobal.colors;

  protected BUIResources res = new BUIResources( new Object[]
  {
    "picker.size",                new Dimension(230, 230),
    "picker.background",          colors.getVeryDarkShade(),
    "picker.meters",              new Double(20),
    "picker.metersPerMajor",      new Double(5),
    "picker.minorsPerMajor",      new Integer(5),
    "picker.icon",                BIconCache.getHSB("bot_icon.gif"),

    "timeout.width",              new Integer(150),

    "timeoutSlider.max",          new Integer(30),
    "timeoutSlider.min",          new Integer(1),


  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BComboBox orientCombo = new BComboBox();
  protected BComboBox searchCombo = new BComboBox();
  protected BOrientPicker picker  = new BOrientPicker(res, "picker");
  protected BLabel timeout   = new BLabel(res, "timeout");
  protected BSlider timeoutSlider = new BSlider(res, "timeoutSlider" );


  // Constructor.
  // -------------------------------------------------------------

  public BOrientActionNodePanel()
  {
    super("orientation");
    layoutPanelChildren();
    configPanelChildren();

    res.cleanse();
    res = null;
  }

  protected void layoutPanelChildren()
  {
    addHGlued(new BLabel("Type of orientation: "));
    addTinyMargin();
    add(orientCombo);

    addSmallMargin();
    addHGlued(new BLabel("If bot loses sight of enemy: "));
    addTinyMargin();
    add(searchCombo);

    addVStrut(20);
    addLinear(getHGlue(), picker, getHGlue());
    addVStrut(20);

    addLinear(timeout, timeoutSlider, getHGlue());
   /* BBoxPanel box = createBoxPanel();
    box.add(new BLabel("Stop trying after:"));
    box.addSmallMargin();
    box.add(timeout);
    box.addSmallMargin();
    box.add(new BLabel("seconds."));
    box.addGlue();
    add(box);*/

    // addGlue();
  }

  protected void configPanelChildren()
  {
    Vector orientOptions =
      BOrientAction.getOrientOptions();

    orientCombo.setModel(
      new BComboBoxModel(orientOptions));

    listener = new OrientTypeListener();

    orientCombo.addActionListener(listener);

    Vector searchOptions =
      BOrientAction.getSearchOptions();

    searchCombo.setModel(
      new BComboBoxModel(searchOptions));

    timeoutSlider.addChangeListener
    (
      new ChangeListener()
      {
        public void stateChanged(ChangeEvent e)
        {
          timeout.setText("Stop trying after " + timeoutSlider.getValue() + " seconds.");

        }
      }
    );
  }


  // Activation methods...
  // -------------------------------------------------------------

  public void onClose()
  {
    instance = null;
    super.onClose();
  }

  public static void clearInstance()
    { instance = null; }


  
  // Accessor methods...
  // -------------------------------------------------------------

  public BDecisionTreeNode getNode()
    { return node; }

  public static BAINodePanel getInstance()
  {
    if (instance == null)
      instance = new BOrientActionNodePanel();

    return instance;
  }

  // UI methods...
  // -------------------------------------------------------------

  public void setPanelFromNode(BDecisionTreeNode n)
  {
    BBot bot  = ai.getBot(); // (BBot) node.getActor();
    this.node = (BOrientActionNode) n;

    BOrientAction action = node.getOrientAction();

    if (action != null)
    {
      timeoutSlider.setValue (java.lang.Math.round( action.getTimeout() ) );
      //this.setText(action.getTimeout());

      BOrient orient =
        action.getOrient();

      setPanelFromOrient(orient);

      listener.setEnabled(false);

      orientCombo.setSelectedItem(orient);
        // TODO: This relies on overriden equals() methods
        //       in BOrient subclasses.
        
      listener.setEnabled(true);

      BOrientSearchPattern search =
        action.getSearch();

      searchCombo.setSelectedItem(search);
    }
  }

  protected void setNodeFromPanel()
  {
    updateListener.setEnabled(false);

    if (node != null)
    {
      BOrient orient =
        (BOrient) orientCombo.getSelectedItem();

      // Take a clone of the prototypical orient.
      orient = (BOrient) orient.clone();

      setOrientFromPanel(orient);

      BOrientSearchPattern search =
        (BOrientSearchPattern) searchCombo.getSelectedItem();

      BOrientAction action =
        new BOrientAction
        (
          (BBot) node.getActor(),
          orient, search,
          timeoutSlider.getValue() // TODO

        );

      node.setOrientAction(action);
    }

    updateListener.setEnabled(true);
  }


  protected void setPanelFromOrient(BOrient orient)
    { picker.setPickerFromOrient(orient); }

  protected void setOrientFromPanel(BOrient orient)
    { picker.setOrientFromPicker(orient); }



  protected class OrientTypeListener extends BActionAdapter
  {
    public void actionApproved(ActionEvent e)
    {
      BOrient orient =
        (BOrient) orientCombo.getSelectedItem();

      // Take a clone of the prototypical orient.
      orient = (BOrient) orient.clone();

      setPanelFromOrient(orient);

      updateListener.actionPerformed(e);
    }
  }
}
