package roboforge.ui.ai;

import roboforge.BBot;

import roboforge.BDebug;

import roboforge.ai.BDecisionTreeAI;
import roboforge.ai.actions.*;
import roboforge.ai.decisions.*;
import roboforge.ai.expressions.*;

import roboforge.ai.values.*;
import roboforge.ui.components.*;

import roboforge.util.graphics.*;
import java.awt.Dimension;

import roboforge.BGlobal;

import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;

import java.util.Hashtable;

import javax.swing.*;

/*
  A panel that displays world item decisions that can be expressed numerically
  */

public class BSimpleNumberPanel extends BSimpleDecisionPanel implements ActionListener
{
  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleIcon.foreground",         colors.getMidShade(),
    "titleIcon.size",               new Dimension(40, 40),

    "titleLabel.text",              "",
    "titleLabel.width",             new Integer(200),
    "titleLabel.fontsize",          new Integer(14),
    "titleLabel.height",            new Integer(20),
    "titleLabel.fontstyle",         new Integer(Font.BOLD),
    "titleLabel.v_align",           new Integer(SwingConstants.BOTTOM),

    "smallLine.width",              new Integer(10),

    "infoLabel.text",               "",
    "infoLabel.width",              new Integer(200),

    "optionCombo.tooltip",          "Options",

    "valueSlider.tooltip",          "Value",

    "greaterButton.icon",           BIconCache.getHSB("op_icon_gtr.gif", BGlobal.colors.getDarkShade()),
    "greaterButton.tooltip",        "Greater than",

    "equalsButton.icon",            BIconCache.getHSB("op_icon_eq.gif", BGlobal.colors.getDarkShade()),
    "equalsButton.tooltip",         "Is equal to",

    "lessButton.icon",              BIconCache.getHSB("op_icon_less.gif", BGlobal.colors.getDarkShade()),
    "lessButton.tooltip",           "Less than",

  });


  // hashtable containing world item refs mathched with actual words
  protected Hashtable optionTable = new Hashtable(6, 0.75f);

  //-------------- variable between different ref types
  protected String title;
 // private HSBImageIcon image;
  protected String [] options;

  protected double min;
  protected double max;

  protected String units;
  //---------------------------------------------------

  private static BSimpleNumberPanel instance = null;


  protected BComboBox optionCombo = new BComboBox(res, "optionCombo");
  protected BLabel titleLabel = new BLabel(res, "titleLabel");
  //protected BLabel titleIcon = new BFilledLabel(res, "titleIcon");

  // hack label
  protected BButton titleIcon = new BButton(res, "titleIcon")
  {
    protected void processMouseEvent(MouseEvent e)
    {}
  };
  
  protected BLabel infoLabel = new BLabel(res, "infoLabel");

  protected BLine titleLine = new BLine();
  protected BLine smallLine = new BLine(res, "smallLine");

  protected BRadioButton greaterButton = new BRadioButton(res, "greaterButton");
  protected BRadioButton equalsButton = new BRadioButton(res, "equalsButton");
  protected BRadioButton lessButton = new BRadioButton(res, "lessButton");

  protected ButtonGroup operatorGroup = new ButtonGroup();

  protected BSlider valueSlider = new BSlider(res, "valueSlider");

  protected boolean initialized = false;

  protected String optionString = "";
  protected String operatorString = "";
  protected String valueString = "";

  // constructor
  public BSimpleNumberPanel()
  {
    super("Simple Decision");

    layoutPanelChildren();
    configPanelChildren();
  }

  protected void layoutPanelChildren()
  {
    BBoxPanel mainBox = createHBoxPanel();
    BBoxPanel leftBox = createVBoxPanel();
    BBoxPanel iconBox = createHBoxPanel();
    BBoxPanel lineBox = createVBoxPanel();
    BBoxPanel rightBox = createVBoxPanel();
    BBoxPanel operatorBox = createHBoxPanel();

    titleIcon.setHorizontalAlignment(BLabel.CENTER);

    leftBox.add(iconBox);
    leftBox.addGlue();

    iconBox.add(titleIcon);
    iconBox.add(lineBox);

    lineBox.addVStrut(1);
    lineBox.addSmallMargin();
    lineBox.add(smallLine);

    rightBox.addHGlued(titleLabel);
    rightBox.addTinyMargin();
    rightBox.addHGlued(titleLine);
    rightBox.addBigMargin();
    rightBox.add(optionCombo);
    rightBox.addSmallMargin();
    rightBox.add(operatorBox);
    rightBox.addSmallMargin();
    rightBox.add(valueSlider);
    rightBox.addSmallMargin();
    rightBox.addHGlued(infoLabel);
    rightBox.addGlue();

    operatorBox.add(lessButton);
    operatorBox.add(equalsButton);
    operatorBox.add(greaterButton);
    operatorBox.addGlue();

    mainBox.add(leftBox);
    mainBox.add(rightBox);
    add(mainBox);
 }


  protected void configPanelChildren()
  {
    operatorGroup.add(greaterButton);
    operatorGroup.add(equalsButton);
    operatorGroup.add(lessButton);

    greaterButton.setSelectedIcon(BGraphics.getHSBIcon("op_icon_gtr.gif", BGlobal.colors.getMidShade()));
    equalsButton.setSelectedIcon(BGraphics.getHSBIcon("op_icon_eq.gif", BGlobal.colors.getMidShade()));
    lessButton.setSelectedIcon(BGraphics.getHSBIcon("op_icon_less.gif", BGlobal.colors.getMidShade()));

    greaterButton.addActionListener(this);
    equalsButton.addActionListener(this);
    lessButton.addActionListener(this);

    optionCombo.addActionListener(this);

    valueSlider.addChangeListener(new ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        if(roboforge.ui.ai.BSimpleNumberPanel.this.isActive()) {
          valueString = "" + valueSlider.getValue();
          if(initialized)
          {
            updateInfo();
            //setNodeFromPanel();  // no Need
          }
        }
      }
    });
  }

  public void actionPerformed(ActionEvent e)
  {
    if(this.isActive()) {
      if (e.getSource() == lessButton)
        operatorString = "less than";
      if (e.getSource() == greaterButton)
        operatorString = "greater than";
      if (e.getSource() == equalsButton)
        operatorString = "equal to";
      if(e.getSource() == optionCombo)
        optionString = (String)optionCombo.getSelectedItem();

      if(initialized)
        updateInfo();
      // don't need to set Node from Panel
    }


  }

  // save the data from the panel to the node
  protected void setNodeFromPanel()
  {
    BDebug.print("BSimpleNumberPanel - value " + valueSlider.getValue());

    if(node != null)
    {

      node.setWorldItemRef((String)optionTable.get(optionString));

      String op;
      if(operatorString.equals("less than"))
        op = "<";
      else if(operatorString.equals("equal to"))
        op = "=";
      else
        op = ">";
      node.setOperatorRef(op);
      //BDebug.print("BSimpleNumberPanel - value " + valueSlider.getValue());

      node.setValueRef(new BNumber((double)valueSlider.getValue()));
      String exp = "[" + optionTable.get(optionString) + "] " + op + " " + valueSlider.getValue() ;
      updateNodeExpression(exp);

      // setr nice name
      //node.setName(title);
      node.setName(optionString + " " + op + " " + valueString + "?");

    }


  }

  // set up the panel data from the node
  protected void setPanelFromNode(BDecisionTreeNode n)
  {
    if(initialized)
    {
      this.node = (BSimpleDecisionNode)n;

      //BDebug.print ("BSimpleNumberPanel node.getWorldItemRef() " + node.getWorldItemRef());
      //BDebug.print ("BSimpleNumberPanel hashItem " + optionTable.get(node.getWorldItemRef()) );

      optionCombo.setSelectedItem(optionTable.get(node.getWorldItemRef()));
      optionString = (String)optionCombo.getSelectedItem();

      if(node.getOperatorRef().equals("<")) {
        lessButton.setSelected(true);
        operatorString = "less than";
      }
      else if (node.getOperatorRef().equals("=")) {
        equalsButton.setSelected(true);
        operatorString = "equal to";
      }
      else
      {
        greaterButton.setSelected(true);
        operatorString = "greater than";
      }

      int valueLength = valueSlider.getMaximum() - valueSlider.getMinimum();
      int majorSpacing = (int)((double)valueLength / 10.0);
      int minorSpacing = (int)((double)valueLength / 50.0);

      valueSlider.setLabelTable(valueSlider.createStandardLabels((int)((double)valueLength / 2.0)));

      valueSlider.setMajorTickSpacing(majorSpacing);
      valueSlider.setMinorTickSpacing(minorSpacing);
      valueSlider.setSnapToTicks(false);
     // BDebug.print("BSimpleNumberPanel.setPanelFromNode " + node.getValueRef());
      valueSlider.setValue((int)((BNumber)node.getValueRef()).getValue());


      valueString = "" + valueSlider.getValue();

      updateInfo();
    }
  }

  // setup the AI
  public void setAI(BDecisionTreeAI ai)
  {
    super.setAI(ai);
  }

  public static BAINodePanel getInstance()
  {
    if (instance == null)
      instance = new BSimpleNumberPanel();

    return instance;
  }

  // updates the useful message at the base of the panel
  private void updateInfo()
  {
    if (optionString == null)
      optionString = "";
      
    infoLabel.setText("Is " + optionString.toLowerCase() + " " + operatorString + " " + valueString + units + " ?");
  }


  // sets the panel data from a simple number data object
  protected void setData(BSimpleDecisionData unformatedData)
  {
    super.setData(unformatedData);

    BSimpleNumberData data = (BSimpleNumberData) unformatedData;

    initialized = false;
   // image = data.getIcon();
    title = data.getTitle();
    options = data.getOptions();

    min = data.getMin();
    max = data.getMax();

    units = data.getUnits();

    titleLabel.setText(title + " (" + units + ")");
    titleIcon.setIcon(data.getIcon());

    valueSlider.setMinimum((int)min);
    valueSlider.setMaximum((int)max);

    // clear the comboBox and hashtable
    optionCombo.removeAllItems();
    optionTable.clear();

    // fill the comboBox and hashtable
    for (int i = 0; i < options.length; i += 2) {
      optionTable.put(options[i], options[i+1]);
      optionTable.put(options[i+1], options[i]);
      optionCombo.addItem(options[i + 1]);
    }

    initialized = true;

  }

  public void onClose()
  {
    instance = null;
  }

  public static void clearInstance()
  { instance = null; }
}