package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.ui.BApplication;
import roboforge.ui.events.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;


public class BDialog extends JWindow
  implements BActivateable, SwingConstants
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BActivationSupport activationSupport =
    new BActivationSupport(this);

  protected boolean modal;
  protected boolean blocking;



  // Constructor...
  // -------------------------------------------------------------

  public BDialog()
  {
    super(BApplication.getWindow());
    modal     = true;
    blocking  = true;
  }

  public void update(Graphics g)
  {
    if (isShowing())
      { paint(g); }
  }

  // Accessor methods...
  // -------------------------------------------------------------

  public void setModal(boolean m)
    { this.modal = m; }

  public boolean isModal()
    { return modal; }

  public void setBlocking(boolean b)
    { this.blocking = b; }

  public boolean isBlocking()
    { return blocking; }

  public boolean isFocusCycleRoot()
    { return true; }


  // Activation methods...
  // -------------------------------------------------------------

  public boolean isActive()
    { return activationSupport.isActive(); }

  public boolean isClosed()
    { return activationSupport.isClosed(); }


  public void requestActivate()
    { activationSupport.requestActivate(); }

  public void requestDeactivate()
    { activationSupport.requestDeactivate(); }

  public void requestClose()
    { activationSupport.requestClose(); }


  public void onActivate()
  {
    // SJS [06/07/2000]: Copied this from BWindow...

    // JTextFields don't seem to get focus in a JWindow correctly.
    // This workaround dispatches WINDOW_ACTIVATED events to the
    // Window at opportune moments.

    // See Sun's bug database for details:
    // * http://developer.java.sun.com/developer/bugParade/bugs/4254005.html
    // * http://developer.java.sun.com/developer/bugParade/bugs/4186928.html

    dispatchEvent(new WindowEvent(this,
      WindowEvent.WINDOW_ACTIVATED));

    requestFocus();
    
    activationSupport.fireActivate();
  }

  public void onDeactivate()
    { activationSupport.fireDeactivate(); }

  public void onClose()
    { activationSupport.fireClose(); }

    
  public void addActivationListener(BActivationListener l)
    { activationSupport.addActivationListener(l); }

  public void removeActivationListener(BActivationListener l)
    { activationSupport.addActivationListener(l); }

}
