package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.ui.dnd.*;
import roboforge.util.graphics.BGraphics;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import java.io.*;
import java.io.IOException;

import javax.swing.JTree;
import javax.swing.tree.*;


public class BDragDropTree extends BTree
  implements
    // BDragDroppable,
    DragSourceListener,
    DragGestureListener
{


  // Internal variables...
  // -------------------------------------------------------------

  protected MouseListener mouseListener = new MouseListener();
  
  protected DragSource dragSource  = null;
  protected DropTarget dropTarget  = null;

  protected DefaultMutableTreeNode draggedNode       = null;
  protected DefaultMutableTreeNode draggedNodeParent = null;
  protected int                    draggedNodeIndex  = 0;

  protected DefaultMutableTreeNode targetNode        = null;


  protected static final DataFlavor OBJECT =
    ObjectSelection.createObjectFlavor();

  protected Cursor linkCursor = BGraphics.getCursor("link_cursor.gif", 0, 0);
  // Constructor...
  // -------------------------------------------------------------

  public BDragDropTree()
    { this(null, null); }

  public BDragDropTree(BUIResources res, String key)
  {
    super(res, key);

    dragSource = new DragSource();
    dropTarget = new DropTarget(this, new DropListener());

    BDragGestureRecognizer dragRecognizer =
      new BDragGestureRecognizer
        (dragSource, this, DnDConstants.ACTION_COPY_OR_MOVE, this);

    /*
    dragSource.createDefaultDragGestureRecognizer
      (this, DnDConstants.ACTION_MOVE, this);
    */
    
    addMouseMotionListener(mouseListener);

    // DragGestureRecognizer
  }


  // Helper methods...
  // -------------------------------------------------------------

  public DefaultMutableTreeNode getSelectedNode()
  {
    TreePath path = getLeadSelectionPath();

    if (path != null)
      return (DefaultMutableTreeNode) path.getLastPathComponent();
    else
      return null;
  }


  // Mouse Listener...
  // -------------------------------------------------------------
  
  protected class MouseListener extends MouseMotionAdapter
  {
    protected Point drag;
    
    public void mouseDragged(MouseEvent e)
      { drag = e.getPoint(); }
    
    public Point getDragPosition()
      { return drag; }
  }
  
  
  // Drag 'n Drop methods...
  // -------------------------------------------------------------


  protected boolean acceptDrop(DefaultMutableTreeNode dropped, DefaultMutableTreeNode target)
    // Determines what nodes can be dropped on this list.
    { return true; }

  protected boolean acceptDrag(DefaultMutableTreeNode dragged)
    // Determines what nodes can be dragged off this list.
    { return true; }


  public void dragGestureRecognized(DragGestureEvent e)
    // a drag gesture has been initiated
  {
   // BDebug.print("dragGestureRecognized(): o="+
   //   e.getDragOrigin()+ ", p="+ mouseListener.getDragPosition());
    
    // If we are already dragging, abort.
    if (draggedNode != null)
      return;

    draggedNode = getSelectedNode();

    // If nothing has been selected, abort.
    if (draggedNode == null)
      return;

    if (acceptDrag(draggedNode))
    {
      //System.out.println("Drag data: "+ draggedNode);
      targetNode  = null;

      extractDragData();

      ObjectSelection o = new ObjectSelection(draggedNode);
      dragSource.startDrag (e, linkCursor, o, this);
//      dragSource.startDrag (e, DragSource.DefaultCopyDrop, o, this);
    }
    else
      { draggedNode = null; }
  }

  public void dragDropEnd (DragSourceDropEvent e)
    // goes to DragSourceListener, informing it that dragging has ended.
  {
    //System.out.println("DragDropEnd: success = "+ e.getDropSuccess());

    if (!e.getDropSuccess())
      returnDraggedData();

    draggedNode = null;
    targetNode  = null;
  }

  protected void returnDraggedData()
  {
    DefaultTreeModel model =
      (DefaultTreeModel) getModel();

    model.insertNodeInto(draggedNode, draggedNodeParent, draggedNodeIndex);

    selectNode(draggedNode);
    expandNode(draggedNode);
  }


  public void dragEnter (DragSourceDragEvent e)
    // goes to DragSourceListener, informing it that dragging has
    // entered the DropSite.
  {
    // System.out.println( "dragEnter");
      DragSourceContext context =
      e.getDragSourceContext();

      context.setCursor(null);
      context.setCursor(linkCursor);
  }

  public void dragExit (DragSourceEvent e)
    // this message goes to DragSourceListener, informing it that the
    // dragging has exited the DropSite.
  {
    // System.out.println( "dragExit");
  }

  public void dragOver (DragSourceDragEvent e)
    // this message goes to DragSourceListener, informing it that dragging
    // is currently occuring over the DropSite
  {
    // System.out.println( "dragExit");
      DragSourceContext context =
      e.getDragSourceContext();

      context.setCursor(null);
      context.setCursor(linkCursor);
  }

  public void dropActionChanged (DragSourceDragEvent e)
    // is invoked when the user changes the dropAction
  {
    // System.out.println( "dropActionChanged");
  }


  // -------------------------------------------------------------
  // Drop Target Listener Class...
  // -------------------------------------------------------------

  protected class DropListener implements DropTargetListener
  {

    public void dragEnter (DropTargetDragEvent e)
      // is invoked when you are dragging over the DropSite
    {
      // debug messages for diagnostics
      // System.out.println( "dragEnter");
      //e.acceptDrag(DnDConstants.ACTION_COPY_OR_MOVE);
    }

    public void dragExit (DropTargetEvent e)
      // is invoked when you are exit the DropSite without dropping
    {
      // System.out.println( "dragExit");
    }

    public void dragOver (DropTargetDragEvent e)
      // is invoked when a drag operation is going on
    {
      // System.out.println( "dragOver");
    }

    public void drop (DropTargetDropEvent e)
      // a drop has occurred
    {
      //BDebug.print("DROP DETECTED");

      try
      {
        Transferable t = e.getTransferable();
        if (t.isDataFlavorSupported(OBJECT))
        {
          Object o   = t.getTransferData(OBJECT);
          targetNode = getNode(e.getLocation());

          BDebug.print ("BDragDropTree Drop recognized as recieving object " + o );
          if (!(o instanceof DefaultMutableTreeNode))
            { e.rejectDrop(); return; }

          BDebug.print ("BDragDropTree Drop accepting drop" );

          DefaultMutableTreeNode droppedNode
            = (DefaultMutableTreeNode) o;

          if (acceptDrop(droppedNode, targetNode))
          {
            e.acceptDrop(DnDConstants.ACTION_MOVE);
            insertDropData(droppedNode, targetNode);
            e.getDropTargetContext().dropComplete(true);
          }
          else
            e.rejectDrop();
        }
        else
          e.rejectDrop();
      }
      catch (IOException ex)
      {
        System.err.println("Exception: " + ex.getMessage());
        e.rejectDrop();
      }
      catch (UnsupportedFlavorException ex)
      {
        System.err.println("Exception: " + ex.getMessage());
        e.rejectDrop();
      }
    }

    public void dropActionChanged ( DropTargetDragEvent e )
      // is invoked if the use modifies the current drop gesture
    { }

  }


  protected DefaultMutableTreeNode getNode(Point p)
  {
    int i = getClosestRowForLocation(p.x, p.y);
    TreePath path = getPathForRow(i);

    //Rectangle rowBounds = getRowBounds(i);
   //if ((rowBounds.y + rowBounds.height) > p.y )  // if the pick is the last item
      return (DefaultMutableTreeNode) path.getLastPathComponent();
   // else
    //  return (DefaultMutableTreeNode)getModel().getRoot();
  }


  // Add/Remove methods...
  // -------------------------------------------------------------

  protected void insertDropData(
    DefaultMutableTreeNode dropped,
    DefaultMutableTreeNode target)
  {
    DefaultTreeModel model =
      (DefaultTreeModel) getModel();

    if (target.getAllowsChildren())
      model.insertNodeInto(dropped, target, target.getChildCount());
    else
    {
      DefaultMutableTreeNode parent =
        (DefaultMutableTreeNode) target.getParent();

      if (parent != null)
        model.insertNodeInto(dropped, parent, parent.getIndex(target)+1);
      else
        model.insertNodeInto(dropped, target, 0);
    }

    selectNode(dropped);
    expandNode(dropped);
  }

  protected void extractDragData()
  {
    DefaultTreeModel model =
      (DefaultTreeModel) getModel();

    draggedNodeParent =
      (DefaultMutableTreeNode) draggedNode.getParent();

    draggedNodeIndex  =
      draggedNodeParent.getIndex(draggedNode);

    model.removeNodeFromParent(draggedNode);
  }

}