package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.ui.components.text.*;

import java.awt.Toolkit;
import javax.swing.text.*;



public class BGuardedTextField extends BTextField
{

  // Class variables...
  // -------------------------------------------------------------
  
  protected static final Toolkit toolkit =
    Toolkit.getDefaultToolkit();

  protected static int MAX = Short.MAX_VALUE;

  
  // Member variables...
  // -------------------------------------------------------------
  
  protected BTextGuard guard;

  protected int size;
  


  
  // Constructor.
  // -------------------------------------------------------------

  public BGuardedTextField(BUIResources res, String key)
    { super(res, key); }

  public BGuardedTextField()
    { super(); }


  
  // Visual Initialization methods...
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {
    super.initializeUI(res, key);

    setFieldSize(MAX);

    if (res != null)
    {
      Integer s = (Integer) res.get(key+ ".fieldsize");
      if (s != null)
        setFieldSize(s.intValue());
    }

  }


  // Accessor methods.
  // -------------------------------------------------------------

  public void setGuard(BTextGuard guard)
    { this.guard = guard; }
  
  public BTextGuard getGuard()
    { return this.guard; }

  public int getFieldSize()
    { return size; }

  public void setFieldSize(int i)
  {
    size = i;

    // TODO:
    // enforceSizeConstraint();
  }

  
  
  
  // Document model stuff.
  // -------------------------------------------------------------

  protected Document createDefaultModel()
    { return new GuardedTextDocument(); }

  protected class GuardedTextDocument extends PlainDocument
  {
    public void insertString(int offset, String str, AttributeSet a)
      throws BadLocationException
    {
      int length = getLength();

      // Trivial rejection...
      if ((str == null) || (length >= size))
        { toolkit.beep(); return; }
      
      // Validate the insertion...
      if (guard != null)
      {
        String cur = getText(0, length);

        if (!guard.isValidInsertion(cur, str, offset))
          { toolkit.beep(); return; }
      }

      // Insert the filtered string...
      super.insertString(offset, str, a);
      enforceSizeConstraint();
    }

    protected void enforceSizeConstraint()
      throws BadLocationException
    {
      if (size != MAX)
      {
        int length = getLength();
        if (length > size)
        {
          int cutPos = size-1;
          int n = length - size;
          remove(cutPos, n);
        }
      }
    }
  }

}
