package roboforge.ui.components;

import roboforge.BGlobal;
import roboforge.ui.BApplication;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class BOptionDialog extends BOKCancelDialog
{

  protected String promptText;

  protected Object[] options;
  protected Object   selectedOption = null;

  protected BOptionClient client;


  // UI Resources...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
    { "this.size",        new Dimension(300, 250) });


  // Constructor...
  // -------------------------------------------------------------

  public BOptionDialog
  (
    String    title,
    String    prompt,
    Object[]  options
  )
  {
    super(title, true, true, "OK", "Cancel");

    initializeUI(res, "this");

    this.promptText = prompt;
    this.options    = options;

    client = new BOptionClient();
    setClient(client);

    res.cleanse();
    res = null;
  }


  public Object getSelectedOption()
    { return selectedOption; }



  protected class BOptionClient extends OKCancelClient
  {
    protected BUIResources res = new BUIResources( new Object[]
    {
      "prompt.height",        new Integer(15),
      "prompt.foreground",    BGlobal.colors.getMidShade(),
      
      "client.background",    BGlobal.colors.getDarkestShade(),
      
      "scroller.opaque",      Boolean.FALSE,
      "scroller.border",      new EmptyBorder(10, 15, 10, 0),
      
    });

    
    // Visual elements...
    // -------------------------------------------------------------

    protected BLabel prompt          = new BLabel(res, "prompt");
    protected ButtonGroup group      = new ButtonGroup();
    protected BRadioButton[] buttons;
    
    protected BBoxPanel optionClient = new BBoxPanel(res, "client", BBoxPanel.Y_AXIS);
    protected BScrollPane scroller   = new BScrollPane(res, "scroller", optionClient);

    
    public BOptionClient()
    {
      super();

      buttons = new BRadioButton[options.length];

      layoutFileChildren();
      configFileChildren();

      // setDefaultFocus(buttons[0]);
    }

    protected void layoutFileChildren()
    {
      addTinyMargin();
      addHGlued(prompt);

      for (int i=0; i<options.length; i++)
      {
        Object o = options[i];

        buttons[i] = new BRadioButton(o.toString());
        group.add(buttons[i]);

        optionClient.addHGlued(buttons[i]);
      }

      add(scroller);
      addGlue();
      
      scroller.setHorizontalScrollBarPolicy(
        BScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
      
    }

    protected void configFileChildren()
    {
      prompt.setText(promptText);
      buttons[0].setSelected(true);
    }

    protected void onOKPressed(ActionEvent e)
    {
      for (int i=0; i<options.length; i++)
        if (buttons[i].isSelected())
          selectedOption = options[i];

      super.onOKPressed(e);
    }

    protected void onCancelPressed(ActionEvent e)
    {
      selectedOption = null;
      super.onCancelPressed(e);
    }

    protected void onClosePressed(ActionEvent e)
    {
      selectedOption = null;
      super.onClosePressed(e);
    }

  }
}