package roboforge.ui.components;

import java.awt.Component;
//import java.awt.Container;
import java.awt.LayoutManager;

import javax.swing.*;
import javax.swing.border.*;

//import roboforge.BDebug;

public class BPanel extends JPanel
{

  // Internal variables.
  // -------------------------------------------------------------

  // If panel is disabled, this array holds kids' prior status.
  // protected boolean[] kidEnabled = null;

  // Constructors.
  // -------------------------------------------------------------

  public BPanel(BUIResources res, String resourceKey)
  {
    super();

    // Use absolute positioning, by default.
    setLayout(null);

    initializeUI(res, resourceKey);
  }

  public BPanel()
    { this(null, null); }

  public BPanel(LayoutManager layout)
  {
    this(null, null);
    setLayout(layout);
  }



  // Visual Initialization methods...
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {
    if (res != null)
      res.initComponent(this, key);

    setAlignmentX(Component.CENTER_ALIGNMENT);
    setAlignmentY(Component.CENTER_ALIGNMENT);
  }

  // Action methods.
  // -------------------------------------------------------------

  public void addOnTop(Component c)
    { add(c, 0); }

  public void addOnBottom(Component c)
    { add(c, -1); }

  public void addBorderInside(Border b)
    { setBorder(BorderFactory.createCompoundBorder(getBorder(), b)); }

  public void addBorderOutside(Border b)
    { setBorder(BorderFactory.createCompoundBorder(b, getBorder())); }

  public void setEnabled(boolean enabled)
  {
    // ASSUMPTIONS:
    //
    // 1. The component subtree will not be modified
    //    between calls to this.setEnabled().
    //
    // 2. No calls will be made to kids' setEnabled().
    //    between calls to this.setEnabled().
    //

    super.setEnabled(enabled);

    synchronized (getTreeLock())
    {
      Component[] kids = getComponents();

      for (int i=0; i<kids.length; i++)
        kids[i].setEnabled(enabled);

      /*
      if (!enabled)
      {
        kidEnabled = new boolean[kids.length];
        for (int i=0; i<kids.length; i++)
        {
          kidEnabled[i] = kids[i].isEnabled();
          kids[i].setEnabled(false);
        }
      }
      else
      {
        if (kidEnabled != null)
          for (int i=0; i<kids.length; i++)
            kids[i].setEnabled(kidEnabled[i]);
        else
          for (int i=0; i<kids.length; i++)
            kids[i].setEnabled(true);

        kidEnabled = null;
      }
      */
    }
  }

  public void onClose()
  {
   // BDebug.print ("BPanel.onClose called for " + this);
    ToolTipManager tooltips = ToolTipManager.sharedInstance();
    tooltips.unregisterComponent(this);
    getActionMap().clear();

    removeAll();
  }


  /*
  public Component findComponentAt(int x, int y)
    // Workaround for DND bug #4193463...
    // Looks like the bug is to do with no visibility checking
    // of child containers in JComponent.findComponentAt(x,y).
  {
    if (!contains(x, y))
      return null;

    int n = getComponentCount();
    for (int i=0; i<n; i++)
    {
      // Changed from getComponentAt(i) - fix was originally
      // designed for use in a JTabbedPane subclass.
      Component c = getComponent(i);

      if (c != null)
      {
        if (c instanceof Container)
        {
          // Here's the major change.
      	  if(c.isVisible())
	          c = ((Container) c).findComponentAt(x - c.getX(), y - c.getY());
        }
        else
          { c = c.locate(x - c.getX(), y - c.getY()); }

        if (c != null && c.isVisible())
          { return c; }
      }
    }
    return this;
  }
  */


}
