package roboforge.ui.components;

import roboforge.BDebug;

import java.awt.Dimension;
import java.awt.event.*;

import javax.swing.JTextField;
import javax.swing.FocusManager;


public class BTextField extends JTextField
{

  // Class variables...
  // -------------------------------------------------------------

  protected static final int PREF_HEIGHT = 19;
  

  // Member variables...
  // -------------------------------------------------------------

  protected TextEntryListener listener;
  
  
  // Constructor.
  // -------------------------------------------------------------

  public BTextField(BUIResources res, String key)
  {
    super();
    initializeUI(res, key);
  }

  public BTextField()
  {
    super();
    initializeUI(null, null);
  }


  // Visual Initialization methods...
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {

    setMinimumSize(new Dimension(0, PREF_HEIGHT));
    setPreferredSize(new Dimension(20, PREF_HEIGHT));
    setMaximumSize(new Dimension(Short.MAX_VALUE, PREF_HEIGHT));

    if (res != null)
      res.initComponent(this, key);

    setActionOnFocusLost(true);
  }


  public void setActionOnFocusLost(boolean b)
  {
    if (!b)
    {
      if (listener != null)
      {
        removeFocusListener(listener);
        removeActionListener(listener);
        listener = null;
      }
    }
    else
    {
      if (listener == null)
      {
        listener = new TextEntryListener();
        addFocusListener(listener);
        addActionListener(listener);
      }
    }
  }

  
  // Text entry listener class...
  // -------------------------------------------------------------

  protected class TextEntryListener implements
    FocusListener, ActionListener
  {
    protected String lastGoodText;

    public void focusGained(FocusEvent e)
      { lastGoodText = getText(); }

    public void focusLost(FocusEvent e)
    {
      if (lastGoodText != null)
        if (!lastGoodText.equals(getText()))
          fireActionPerformed();

      // If focus has changed, paint the control again.
      // Ensures correct focus diplay on borders.
      repaint();
    }

    public void actionPerformed(ActionEvent e)
    {
      lastGoodText = getText();

      FocusManager.getCurrentManager()
        .focusNextComponent(BTextField.this);
    }
  }
}
