package roboforge.ui.components;

import roboforge.util.graphics.BColors;
import java.awt.Color;
import javax.vecmath.*;


public class BUIColorScheme
{

  // Number of distinct 'shades' in the scheme, inc. B&W.
  protected static int NUM_SHADES = 11;

  // Constants for the various shades.
  public static final int BLACK     = 0;
  public static final int EXTRADARK = 1;
  public static final int VERYDARK  = 2;
  public static final int DARKEST   = 3;
  public static final int DARKER    = 4;
  public static final int DARK      = 5;
  public static final int MID       = 6;
  public static final int LIGHT     = 7;
  public static final int LIGHTER   = 8;
  public static final int LIGHTEST  = 9;
  public static final int WHITE     = 10;

  // The current 'primary' color.
  protected float[] hsb = {0.0f, 0.0f, 0.0f};

  // Holder for preset shade variations (black...white).
  protected Color[] shades = new Color[NUM_SHADES];

  // J3D Holder for preset shade variations (black...white).
  protected Color3f[] shades3f = new Color3f[NUM_SHADES];

  // Coefficients for each shade of the primary color.
  protected static float[] shadeSatCoeffs =
    { 1.00f, 0.75f, 0.75f, 1.00f, 1.00f, 1.00f, 1.00f, 0.50f, 0.30f, 0.10f, 0.00f };

  protected static float[] shadeBriCoeffs =
    { 0.00f, 0.05f, 0.07f, 0.10f, 0.30f, 0.50f, 1.00f, 1.00f, 1.00f, 1.00f, 1.00f };


  protected Color   SELECTED_SHADE     = new Color(255,255,255);
  protected Color3f SELECTED_SHADE_3F  = getShade3f(SELECTED_SHADE);
  protected String  SELECTED_SHADE_STR = getHexString(SELECTED_SHADE);


  public BUIColorScheme(Color c)
  {
    setPrimaryColor(c);
  }

  public void setPrimaryColor(Color c)
  {
    Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), hsb);

    for (int i=0; i<NUM_SHADES; i++)
    {
      shades[i]   = calculateShade(shadeSatCoeffs[i], shadeBriCoeffs[i]);
      shades3f[i] = getShade3f(shades[i]);
    }
  }

  protected Color calculateShade(float s, float b)
    { return Color.getHSBColor(hsb[0], hsb[1] * s, hsb[2] * b); }

  protected static Color3f getShade3f(Color c)
  {
    /*
      float[] comp = c.getColorComponents(null);
      return new Color3f(comp);
    */

    return BColors.convert(c);
  }

  protected static String getHexString(Color c)
  {
    int rgb = c.getRGB() & 0x00FFFFFF;
    return "#"+ Integer.toHexString(rgb).toUpperCase();
  }


  public Color getShade(int shadeID)
    { return shades[shadeID]; }

  public Color getBlackShade()
    { return shades[BLACK]; }

  public Color getExtraDarkShade()
    { return shades[EXTRADARK]; }

  public Color getVeryDarkShade()
    { return shades[VERYDARK]; }

  public Color getDarkestShade()
    { return shades[DARKEST]; }

  public Color getDarkerShade()
    { return shades[DARKER]; }

  public Color getDarkShade()
    { return shades[DARK]; }

  public Color getMidShade()
    { return shades[MID]; }

  public Color getLightShade()
    { return shades[LIGHT]; }

  public Color getLighterShade()
    { return shades[LIGHTER]; }

  public Color getLightestShade()
    { return shades[LIGHTEST]; }

  public Color getWhiteShade()
    { return shades[WHITE]; }

  public Color getSelectedShade()
    { return SELECTED_SHADE; }



  public Color3f getShade3f(int shadeID)
    { return shades3f[shadeID]; }

  public Color3f getBlackShade3f()
    { return shades3f[BLACK]; }

  public Color3f getExtraDarkShade3f()
    { return shades3f[EXTRADARK]; }

  public Color3f getVeryDarkShade3f()
    { return shades3f[VERYDARK]; }

  public Color3f getDarkestShade3f()
    { return shades3f[DARKEST]; }

  public Color3f getDarkerShade3f()
    { return shades3f[DARKER]; }

  public Color3f getDarkShade3f()
    { return shades3f[DARK]; }

  public Color3f getMidShade3f()
    { return shades3f[MID]; }

  public Color3f getLightShade3f()
    { return shades3f[LIGHT]; }

  public Color3f getLighterShade3f()
    { return shades3f[LIGHTER]; }

  public Color3f getLightestShade3f()
    { return shades3f[LIGHTEST]; }

  public Color3f getWhiteShade3f()
    { return shades3f[WHITE]; }

  public Color3f getSelectedShade3f()
    { return SELECTED_SHADE_3F; }



  public String getBlackStr()
    { return getHexString(shades[BLACK]); }

  public String getVeryDarkStr()
    { return getHexString(shades[VERYDARK]); }

  public String getDarkestStr()
    { return getHexString(shades[DARKEST]); }

  public String getDarkerStr()
    { return getHexString(shades[DARKER]); }

  public String getDarkStr()
    { return getHexString(shades[DARK]); }

  public String getMidStr()
    { return getHexString(shades[MID]); }

  public String getLightStr()
    { return getHexString(shades[LIGHT]); }

  public String getLighterStr()
    { return getHexString(shades[LIGHTER]); }

  public String getLightestStr()
    { return getHexString(shades[LIGHTEST]); }

  public String getWhiteStr()
    { return getHexString(shades[WHITE]); }

  public String getSelectedStr()
    { return SELECTED_SHADE_STR; }

  // AI Colors

  private static Color AI_MEMORY_COLOR = new Color(0.6f,0.6f, 0);
  private static Color AI_ACTION_COLOR = new Color(0,0.6f, 0);
  private static Color AI_GROUP_COLOR    = new Color (0.7f, 0.4f,0.0f);
  private static Color AI_DECISION_COLOR    = new Color (0.0f, 0.4f,0.7f);

  public Color getAIMemoryColor()
    { return AI_MEMORY_COLOR; }

  public Color getAIActionColor()
    { return AI_ACTION_COLOR; }

  public Color getAIGroupColor()
    { return AI_GROUP_COLOR; }


  public Color getAIDecisionColor_Me()
    { return AI_DECISION_COLOR; }

//  public Color getAIDecisionColor_Me()
//    { return getDarkerShade(); }

  public Color getAIDecisionColor_Enemy()
    { return getDarkerShade(); }

}
