package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.BGlobal;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

public class BUIResources extends UIDefaults
{

  // Internal variables...
  // ------------------------------------------------------

  // System toolkit.
  protected static Toolkit toolkit = Toolkit.getDefaultToolkit();



  // Constructor
  // ------------------------------------------------------

  public BUIResources(Object[] resourceData)
  {
    super();
    putDefaults(resourceData);
  }

  public void cleanse()
  {
    clear();
  }

  /*
  protected void finalize()
    { BDebug.print("BUIResources.finalize()"); }
  */


  // Resource creation helper functions...
  // ------------------------------------------------------

  /*
  public static Rectangle bounds(int x, int y, int w, int h)
    { return new Rectangle(x, y, w, h); }

  public static Dimension size(int w, int h)
    { return new Dimension(w, h); }

  public static Point loc(int x, int y)
    { return new Point(x, y); }
  */


    
  // Component Initialization methods...
  // ------------------------------------------------------

  public void initComponent(Component c, String key)
  {
    // Initialize a component given its key in the resources table.
    // Looks for various properties (name, bounds, icon etc.)
    // If found, these properties will be set.

    Rectangle r = getBounds(key+ ".bounds");
    if (r != null)
      c.setBounds(r);

    Point loc = getLocation(key+ ".location");
    if (loc != null)
      c.setLocation(loc);

    Color bg = getColor(key+ ".background");
    if (bg != null)
      c.setBackground(bg);

    Color fg = getColor(key+ ".foreground");
    if (fg != null)
      c.setForeground(fg);

    Dimension size = getSize(key+ ".size");
    if (size != null)
      c.setSize(size);
      
    Integer width = (Integer) get(key+ ".width");
    if (width != null)
    {
      Dimension d = c.getSize();
      d.width = width.intValue();
      c.setSize(d);
    }

    Integer height = (Integer) get(key+ ".height");
    if (height != null)
    {
      Dimension d = c.getSize();
      d.height = height.intValue();
      c.setSize(d);
    }

    // Force a re-layout based on changes made above.
    // TODO: Is this needed?
    c.validate();
  }

  public void initComponent(JComponent c, String key)
  {
    String toolTip = getString(key+ ".tooltip");
    if (toolTip != null)
      c.setToolTipText(toolTip);

    Boolean b = (Boolean) get(key+ ".opaque");
    if (b != null)
      c.setOpaque(b.booleanValue());

    Dimension size = getDimension(key+ ".size");
    if (size != null)
    {
      c.setMinimumSize(size);
      c.setMaximumSize(size);
      c.setPreferredSize(size);
    }
    
    Dimension pref = getDimension(key+ ".prefsize");
    if (pref != null)
      c.setPreferredSize(pref);
   

    Integer width = (Integer) get(key+ ".width");
    if (width != null)
    {
      int w = width.intValue();
      Dimension d = c.getMinimumSize();
      Dimension n = new Dimension(w, d.height);
      c.setMinimumSize(n);

      d = c.getPreferredSize();
      n = new Dimension(w, d.height);
      c.setPreferredSize(n);

      d = c.getMaximumSize();
      n = new Dimension(w, d.height);
      c.setMaximumSize(n);
    }

    Integer height = (Integer) get(key+ ".height");
    if (height != null)
    {
      int h = height.intValue();
      Dimension d = c.getMinimumSize();
      Dimension n = new Dimension(d.width, h);
      c.setMinimumSize(n);

      d = c.getPreferredSize();
      n = new Dimension(d.width, h);
      c.setPreferredSize(n);

      d = c.getMaximumSize();
      n = new Dimension(d.width, h);
      c.setMaximumSize(n);
    }

    Border border = getBorder(key+ ".border");
    if (border != null)
      c.setBorder(border);
    
    Boolean autoscrolls = (Boolean) get(key+ ".autoscrolls");
    if (autoscrolls != null)
      c.setAutoscrolls(autoscrolls.booleanValue());


    /*
    // TODO: Support more than one sound per component?
    BPropertySound s = (BPropertySound) get(key+ ".sound");
    if (s != null)
      s.install(c);
    */
    
    initComponent((Component) c, key);
  }

  
  
  
  
  public void initComponent(AbstractButton b, String key)
  {
    Icon icon = getIcon(key+ ".icon");
    if (icon != null)
    {
      b.setIcon(icon);

      if (icon instanceof HSBImageIcon)
        b.setDisabledIcon(icon);
    }

    String text = getString(key+ ".text");
    if (text != null)
      b.setText(text);

    String   fontname =     getString(key+ ".fontname");
    Integer fontstyle = (Integer) get(key+ ".fontstyle");
    Integer  fontsize = (Integer) get(key+ ".fontsize");

    if ((fontname != null) || (fontstyle != null) || (fontsize != null))
    {
      Font f = b.getFont();

      String fn = (fontname  != null) ? fontname             : f.getFontName();
      int    fs = (fontstyle != null) ? fontstyle.intValue() : f.getStyle();
      int    ft = (fontsize  != null) ? fontsize.intValue()  : f.getSize();

      b.setFont(new Font(fn, fs, ft));
    }

    Font font = getFont(key+ ".font");
    if (font != null)
      b.setFont(font);

    KeyStroke k = getKeyStroke(key+ ".keystroke");
    if (k != null)
      b.registerKeyboardAction(new ButtonListener(b), k, JComponent.WHEN_IN_FOCUSED_WINDOW);

    Character c = getCharacter(key+ ".mnemonic");
    if (c != null)
      b.setMnemonic(c.charValue());
    
    initComponent((JComponent) b, key);
  }

  protected class ButtonListener implements ActionListener
  {
    protected AbstractButton button;

    public ButtonListener(AbstractButton b)
      { this.button = b; }

    public void actionPerformed(ActionEvent e)
      { button.doClick(); }
  }


  // Type-safe Accessor methods...
  // ------------------------------------------------------

    public Rectangle getBounds(Object key)
    {
      Object value = get(key);
      return (value instanceof Rectangle) ?
        (Rectangle) value : null;
    }

    public Point getLocation(Object key)
    {
      Object value = get(key);
      return (value instanceof Point) ?
        (Point) value : null;
    }

    public Dimension getSize(Object key)
    {
      Object value = get(key);
      return (value instanceof Dimension) ?
        (Dimension) value : null;
    }

    public double getDouble(Object key)
    {
      Object value = get(key);
      return (value instanceof Double) ?
        ((Double) value).doubleValue() : 0.0;
    }

    public float getFloat(Object key)
    {
      Object value = get(key);
      return (value instanceof Float) ?
        ((Float) value).floatValue() : 0.0f;
    }

    public Cursor getCursor(Object key)
    {
      Object value = get(key);
      return (value instanceof Cursor) ?
        (Cursor) value: null;
    }

    public KeyStroke getKeyStroke(Object key)
    {
      Object value = get(key);
      return (value instanceof KeyStroke) ?
        (KeyStroke) value: null;
    }

    public Character getCharacter(Object key)
    {
      Object value = get(key);
      return (value instanceof Character) ?
        (Character) value: null;
    }

    public static Border getLineBorder(Color c)
    {
      return BorderFactory.createLineBorder(c, 1);
    }
}

