package roboforge.ui.console;

import roboforge.BGlobal;
import roboforge.geometry.BRectangle3D;

import javax.media.j3d.*;
import javax.vecmath.*;


public class BConsoleRectangle extends BConsoleGeometry
{
  protected Appearance             appearance;
  protected Material               material;
  protected BRectangle3D           rectangle;
  protected TransparencyAttributes transparencyAttributes;
  protected ColoringAttributes colorA;


  public BConsoleRectangle()
  {
    super();

    appearance = new Appearance();
    // appearance.setTexture(texture);

    TextureAttributes ta = new TextureAttributes();
    ta.setTextureMode(TextureAttributes.REPLACE);
    appearance.setTextureAttributes(ta);

    material = new Material();
    material.setDiffuseColor(BGlobal.BLACK_3F);
    material.setAmbientColor(BGlobal.BLACK_3F);
    material.setSpecularColor(BGlobal.BLACK_3F);
    material.setEmissiveColor(BGlobal.WHITE_3F);
    material.setLightingEnable(false);
    material.setCapability(Material.ALLOW_COMPONENT_WRITE);
    appearance.setMaterial(material);
    

    colorA = new ColoringAttributes(BGlobal.WHITE_3F, ColoringAttributes.SHADE_FLAT);
    colorA.setCapability(ColoringAttributes.ALLOW_COLOR_WRITE);
    appearance.setColoringAttributes(colorA);

  }

  public Appearance getAppearance()
    { return appearance; }

  public void setTransparency(float t)
  {
    if (transparencyAttributes == null)
    {
      transparencyAttributes = new TransparencyAttributes(
        TransparencyAttributes.BLENDED,
        0.0f,
        TransparencyAttributes.BLEND_SRC_ALPHA,
        TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA);

      appearance.setTransparencyAttributes(transparencyAttributes);

      RenderingAttributes renderAttr = new RenderingAttributes();
      renderAttr.setDepthBufferWriteEnable(false);
      renderAttr.setDepthBufferEnable(false);
      // renderAttr.setAlphaTestFunction(RenderingAttributes.ALWAYS);
      appearance.setRenderingAttributes(renderAttr);

    }

    transparencyAttributes.setTransparency(t);

  }

  public void setColor(Color3f c)
  {
    material.setEmissiveColor(c);
    colorA.setColor(c);
    //appearance.setMaterial(material);
  }

  public void setSize(int w, int h)
  {

    super.setSize(w, h);

    rectangle = new BRectangle3D(w, h);
    rectangle.setAppearance(appearance);
    setGeometry(rectangle);

    setOffset(-w/2, -h/2);
  }

  public void cleanse()
  {
    super.cleanse();
    appearance = null;
    material = null;
    transparencyAttributes = null;
    rectangle.cleanse();
  }

}