package roboforge.ui.dnd;

import roboforge.BDebug;

import java.awt.Component;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.dnd.*;



public class BDragGestureRecognizer extends MouseDragGestureRecognizer
{

  // Member variables...
  // -------------------------------------------------------------

  protected boolean isDragging;
  protected boolean isGestureRecognized;
  protected boolean isRegistered;

  protected MouseEvent origin;

  protected int dragAction = DnDConstants.ACTION_MOVE;
    
  
  // Constructors...
  // -------------------------------------------------------------

  public BDragGestureRecognizer(DragSource ds, Component c, int act, DragGestureListener dgl)
  { 
    super(ds, c, act, dgl); 
    
    this.isDragging           = false;
    this.isGestureRecognized  = false;
    this.isRegistered         = true;
  }

  public BDragGestureRecognizer(DragSource ds, Component c, int act)
    { this(ds, c, act, null); }

  public BDragGestureRecognizer(DragSource ds, Component c)
    { this(ds, c, DnDConstants.ACTION_NONE); }

  public BDragGestureRecognizer(DragSource ds)
    { this(ds, null); }


  
  // Accessor methods...
  // -------------------------------------------------------------
  
  public void setDragAction(int action)
    { this.dragAction = action; }
  
  public void setEnabled(boolean enabled)
  {
    if (enabled && !isRegistered)
      registerListeners();
    
    else if (!enabled && isRegistered)
    {
      unregisterListeners();
      resetDragState();
    }
  }
  
  protected void registerListeners()
  {
    super.registerListeners();
    isRegistered = true;
  }

  protected void unregisterListeners()
  {
    super.unregisterListeners();
    isRegistered = false;
  }
  
  protected void resetDragState()
  {
    isDragging = false;
    isGestureRecognized = false;

    origin = null;
    events.clear();
  }
  
  
  
  // Mouse methods...
  // -------------------------------------------------------------

  public void mousePressed(MouseEvent e)
  {
    isDragging          = true;
    isGestureRecognized = false;

    appendEvent(e);
    origin = e;
  }

  public void mouseDragged(MouseEvent e)
  {
    if (!isDragging || isGestureRecognized)
      return;

    if (isADrag(origin, e))
    {
      appendEvent(e);
      fireDrag(dragAction);
    }
  }

  public void mouseReleased(MouseEvent e)
  {
    isDragging = false;
    events.clear();
  }


  // Implementation methods...
  // -------------------------------------------------------------

  protected boolean isADrag(MouseEvent o, MouseEvent e)
  {
    // Meta filter (right mouse button)
    if (e.isMetaDown()) return false;
    
    // Time filter...
    long dt = e.getWhen() - o.getWhen();
    if (dt < 200) return false;

    // Distance filter...
    Point op = o.getPoint();
    Point ep = e.getPoint();

    int dx = Math.abs(ep.x - op.x);
    int dy = Math.abs(ep.y - op.y);

    return (dx >= 20) || (dy >= 20);
  }

  protected void fireDrag(int action)
  {
    isGestureRecognized = true;
    fireDragGestureRecognized(action, origin.getPoint());
  }

}