package roboforge.ui.fusion;

import roboforge.BDebug;
import roboforge.sound.*;
import roboforge.ui.components.HSBImageIcon;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import javax.swing.plaf.*;
import javax.swing.plaf.basic.BasicButtonUI;
import javax.swing.plaf.basic.BasicGraphicsUtils;



public class FusionButtonUI extends BasicButtonUI
{

  // Shared UI object
  private final static FusionButtonUI buttonUI = 
    new FusionButtonUI();

  private static RenderingHints hints = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );

  protected static Color   pressedColor  = Color.white;
  protected static Color   disabledColor = Color.white;
  protected static Color   rolloverColor = Color.white;
  
  protected static float   roundness     = 0.0f;
  protected static Insets  insets        = new Insets(0,0,0,0);

  
  protected static BSound clickSound;
    
  protected static ButtonSoundListener sounds =
    new ButtonSoundListener();
  
  

  public FusionButtonUI()
  { 
    super(); 
  }

  public static ComponentUI createUI(JComponent c)
    { return buttonUI; }

  protected void installDefaults(AbstractButton b)
  {
    super.installDefaults(b);
    String pp = "Button.";

    pressedColor   = UIManager.getColor(pp+ "pressedColor");
    disabledColor  = UIManager.getColor(pp+ "disabledColor");
    rolloverColor  = disabledColor;
    roundness      = ((Float) UIManager.get(pp + "roundness")).floatValue();
    insets         = UIManager.getInsets(pp + "insets");
    clickSound     = (BSound) UIManager.get(pp+ "clickSound");
  }
  
  
  protected void installListeners(AbstractButton b)
  {
    super.installListeners(b);
    b.addMouseListener(sounds);
  }

  protected void uninstallListeners(AbstractButton b)
  {
    super.uninstallListeners(b);
    b.removeMouseListener(sounds);
  }

  
  
  public void paint(Graphics g, JComponent c)
  {
    Graphics2D g2D = (Graphics2D) g;
    g2D.addRenderingHints(hints);

    AbstractButton b  = (AbstractButton) c;
    ButtonModel model = b.getModel();
    
    if (b.getIcon() instanceof HSBImageIcon)
    {
      HSBImageIcon icon = (HSBImageIcon) b.getIcon();
      icon.setDisabledColor(disabledColor);
    }
    
    if (model.isArmed() && !model.isPressed())
      paintButtonRollover(g, b);

    super.paint(g, c);
  }

  
  protected void paintButtonRollover(Graphics g, AbstractButton b)
  {
    //if (b.isRolloverEnabled())
    //{
      Dimension size = b.getSize();
      g.setColor(rolloverColor);

      int x = insets.left;
      int y = insets.top;
      int w = size.width  - x - insets.right;
      int h = size.height - y - insets.bottom;

      int arcsize = Math.round(roundness * Math.min(w, h));

      g.fillRoundRect(x, y, w, h, arcsize, arcsize);
    //}
  }

  
  protected void paintButtonPressed(Graphics g, AbstractButton b)
  {
    Dimension size = b.getSize();
    g.setColor(pressedColor);

    int x = insets.left;
    int y = insets.top;
    int w = size.width  - x - insets.right;
    int h = size.height - y - insets.bottom;

    int arcsize = Math.round(roundness * Math.min(w, h));

    g.fillRoundRect(x, y, w, h, arcsize, arcsize);
  }

  /*
  protected void paintFocus
  (
    Graphics g,
    AbstractButton b,
    Rectangle viewRect,
    Rectangle textRect,
    Rectangle iconRect
  )
  {
    Dimension size = b.getSize();
    g.setColor(Color.blue);

    int x = insets.left;
    int y = insets.top;
    int w = size.width  - x - insets.right;
    int h = size.height - y - insets.bottom;

    int arcsize = Math.round(roundness * Math.min(w, h));

    g.fillRoundRect(x, y, w, h, arcsize, arcsize);
  }
  */


  protected void paintText(Graphics g, JComponent c, Rectangle textRect, String text)
  {
    AbstractButton b = (AbstractButton) c;
    ButtonModel model = b.getModel();
    FontMetrics fm = g.getFontMetrics();

	  if(model.isEnabled())
    {
	    g.setColor(b.getForeground());
	    BasicGraphicsUtils.drawString(g, text, model.getMnemonic(),
					  textRect.x + getTextShiftOffset(),
					  textRect.y + fm.getAscent() + getTextShiftOffset());
  	}
	  else
    {
	    g.setColor(disabledColor);
	    BasicGraphicsUtils.drawString(g, text,model.getMnemonic(),
					  textRect.x, textRect.y + fm.getAscent());
  	}
  }

  
  public Dimension getPreferredSize(JComponent c)
  {
    Dimension d = super.getPreferredSize(c);
    AbstractButton b = (AbstractButton) c;

    if (b.getIcon() == null)
    {
      d.width  += 40;
      d.height += 15;
    }

    return d;
  }
  
  public Dimension getMinimumSize(JComponent c)
    { return super.getPreferredSize(c); }
  
  
  
  
  protected static class ButtonSoundListener extends MouseAdapter
  {
    public void mousePressed(MouseEvent e)
    { 
      BSound click = FusionButtonUI.clickSound;
      if (click != null) click.play();        
    }
  }
  
  
}
