package roboforge.ui.fusion;

import roboforge.sound.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class FusionConnectorButtonUI extends BasicRadioButtonUI
{

  
  private final static FusionConnectorButtonUI radioButtonUI
    = new FusionConnectorButtonUI();

  protected static RenderingHints hints = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );

  private static float STROKE_WIDTH = 3.0f;
  
  private static int X_OFFSET = 
    Math.round(STROKE_WIDTH / 2.0f);
  
  private static BasicStroke circleStroke = 
    new BasicStroke(STROKE_WIDTH);

  protected static ButtonSoundListener sounds =
    new ButtonSoundListener();

  protected static BSound clickSound;

  
  
  protected Color  disabledColor = null;
  protected Color      fillColor = null;
  protected Color      lineColor = null;
  protected Color      textColor = null;
  protected Font            font = null;
  protected Dimension circleSize = null;
  protected Insets    fillInsets = null;
  protected int      textIconGap = 0;


  

  public static ComponentUI createUI(JComponent b)
    { return radioButtonUI; }

  protected void installDefaults(AbstractButton b)
  {
    super.installDefaults(b);

    String pp = "ConnectorButton.";

    this.disabledColor  = UIManager.getColor(pp+ "disabledColor");
    this.fillColor      = UIManager.getColor(pp+ "fillColor");
    this.lineColor      = UIManager.getColor(pp+ "lineColor");
    this.textColor      = UIManager.getColor(pp+ "textColor");
    this.font           = UIManager.getFont(pp+ "font");
    this.circleSize     = (Dimension) UIManager.get(pp+ "circleSize");
    this.fillInsets     = UIManager.getInsets(pp+ "fillInsets");
    this.textIconGap    = UIManager.getInt(pp+ "textIconGap");
    this.clickSound     = (BSound) UIManager.get(pp+ "clickSound");
  }

  
  protected void installListeners(AbstractButton b)
  {
    super.installListeners(b);
    b.addMouseListener(sounds);
  }

  protected void uninstallListeners(AbstractButton b)
  {
    super.uninstallListeners(b);
    b.removeMouseListener(sounds);
  }

  
  public Dimension getPreferredSize(JComponent c)
  {
    Dimension d = super.getPreferredSize(c);

    if (d != null)
    {
      d.height =  Short.MAX_VALUE;
      d.width  += 75;
    }

    return d;
   }


  public void paint(Graphics g, JComponent c)
  {
    Graphics2D g2D = (Graphics2D) g;
    g2D.addRenderingHints(hints);

    Dimension d = c.getSize();

    AbstractButton b = (AbstractButton) c;
    ButtonModel model = b.getModel();

    g.setFont(font);
    FontMetrics fm = g.getFontMetrics();

    // Calculate positions of various graphic elements...
    int lineX = X_OFFSET+ (circleSize.width / 2);
    int circleY1 = (d.height - circleSize.height) / 2;
    int circleY2 = circleY1 + circleSize.height;
    int textX = X_OFFSET+ (circleSize.width + textIconGap);
    int textY = circleY1+ (circleSize.height / 2) + 5; // TODO: fix hack - fm.getAscent()?

    // Draw top connector line.
    g2D.setColor(lineColor);
    g2D.drawLine(lineX, 0, lineX, circleY1);

    if(model.isEnabled())
    {
      // Calculate inner circle bounds...
      int cx = X_OFFSET + fillInsets.left;
      int cy = circleY1 + fillInsets.top;
      int cw = circleSize.width  - fillInsets.left - fillInsets.right;
      int ch = circleSize.height - fillInsets.top  - fillInsets.bottom;

      if (model.isArmed())
      {
        // Draw highlighted text.
        g.setColor(fillColor);
        BasicGraphicsUtils.drawString(g, b.getText(), model.getMnemonic(), textX, textY);

        // Draw circle inner outline.
        g2D.setColor(fillColor);
        g2D.drawOval(cx, cy, cw, ch);
      }
      else if (model.isSelected())
      {
        // Draw selected text.
        g.setColor(fillColor);
        BasicGraphicsUtils.drawString(g, b.getText(), model.getMnemonic(), textX, textY);

        // Draw circle inner fill.
        g2D.setColor(fillColor);
        g2D.fillOval(cx, cy, cw+1, ch+1);

        // Draw bottom connector line.
        g2D.setColor(lineColor);
        g2D.drawLine(lineX, circleY2, lineX, d.height);
      }
      else
      {
        // Draw unselected text.
        g.setColor(textColor);
        BasicGraphicsUtils.drawString(g, b.getText(), model.getMnemonic(), textX, textY);
      }

      // Draw outer circle.
      g2D.setColor(lineColor);
      g2D.setStroke(circleStroke);
      g2D.drawOval(X_OFFSET, circleY1, circleSize.width, circleSize.height);
    }
    else
    {
      // Draw text.
      g.setColor(disabledColor);
      BasicGraphicsUtils.drawString(g, b.getText(), model.getMnemonic(), textX, textY);

      // Draw outer circle.
      g2D.setColor(disabledColor);
      g2D.setStroke(circleStroke);
      g2D.drawOval(X_OFFSET, circleY1, circleSize.width, circleSize.height);
    }

  }
  
  
  protected static class ButtonSoundListener extends MouseAdapter
  {
    public void mousePressed(MouseEvent e)
    { 
      BSound click = FusionButtonUI.clickSound;
      if (click != null) click.play();        
    }
  }


}



