package roboforge.ui.layers;

import roboforge.ui.events.*;

import java.awt.*;
import java.util.*;
import javax.swing.*;



public abstract class BAbstractLayerStack implements BLayerStack
{

  
  // Member variables...
  // -------------------------------------------------------------

  protected JComponent holder;
    // The container that holds our layers.
  
  protected Stack stack;
    // The Layer stack.

  protected BActivationSupport activation
    = new BActivationSupport(this);


  // Constructors...
  // -------------------------------------------------------------

  public BAbstractLayerStack(JComponent holder)
  {
    this.holder = holder;
    this.stack = new Stack();
  }

  

  // Implementation methods...
  // -------------------------------------------------------------
  
  protected void show(BLayer l)
  {
    if (l != null)
    {
      l.addTo(holder);

      if (isActive())
        l.requestActivate();
    }
  }

  protected void hide(BLayer l)
    { hide(l, false); }
  
  protected void hide(BLayer l, boolean close)
  {
    if (l != null)
    {
      /*
      l.requestDeactivate();
      l.removeFrom(holder);
      */
      
      if (close)
        l.requestClose();
      else
        l.requestDeactivate();
      
      l.removeFrom(holder);
    }
  }
  
  
  // Activation methods...
  // -------------------------------------------------------------

  public boolean isActive()
    { return activation.isActive(); }

  public boolean isClosed()
    { return activation.isClosed(); }

  public void requestActivate()
    { activation.requestActivate(); }

  public void requestDeactivate()
    { activation.requestDeactivate(); }

  public void requestClose()
    { activation.requestClose(); }

  public void onActivate()
  {
    activateLayers();
    activation.fireActivate();
  }

  public void onDeactivate()
  {
    deactivateLayers();
    activation.fireDeactivate();
  }

  public void onClose()
  {
    closeLayers();
    activation.fireClose();
  }

  public void addActivationListener(BActivationListener l)
    { activation.addActivationListener(l); }

  public void removeActivationListener(BActivationListener l)
    { activation.addActivationListener(l); }


  
  
  
  // Abstract methods...
  // -------------------------------------------------------------
  
  protected abstract void activateLayers();
  protected abstract void deactivateLayers();
  protected abstract void closeLayers();
  
  
  
}
