package roboforge.ui.modes;

import javax.media.j3d.*;

import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.lang.Math.*;

import roboforge.*;
import roboforge.ui.*;




public class BModeArenaFixedCamera extends BModeFOVZoom
{
  private TransformGroup tg;


  private Vector3d position;
  private Transform3D transform = new Transform3D();
  private Transform3D rotYTran = new Transform3D();
  private Transform3D rotXTran = new Transform3D();
  private Transform3D totalYTran = new Transform3D();
  private Transform3D oldTransform = new Transform3D();

  private Vector3d oldPos = new Vector3d();
  private Vector3d zeroVector = new Vector3d();
  private Vector3d angleTester = new Vector3d();
  private Vector3d yAnglePlane = new Vector3d(0,0,1);

  private static double MAXUPDOWN = BPhysics.HALF_PI - ( BPhysics.QUARTER_PI/  2.0);
  private static double rot_factor = .0015;

  private double totalX, totalY = 0;
  private AxisAngle4d xRot = new AxisAngle4d();

  public BModeArenaFixedCamera(TransformGroup tg, Vector3d pos, BArenaCanvas canvas)
  {
    super(canvas);

    this.tg = tg;
    this.position = pos;

  }

  public void setPosition(double x,double y,double z)
  {
    if (position == null)
      position = new Vector3d();

    position.x = x;
    position.y = y;
    position.z = z;

    setPosition();

  }

  public void setPosition()
  {
    tg.getTransform(transform);
    transform.setTranslation(position);
    tg.setTransform(transform);

  }

  public void onActivate()
  {
    super.onActivate();
    setPosition();
    canvas.lookAt(true);
  }


  public void animationStep(double t, double dt)
  // do a slow automatic arena animation
  {
    /*
    if (!active)
    {
      long time = System.currentTimeMillis();
      long deltaTime =  time -lastTime;
      int moveDelta = (int)java.lang.Math.min( deltaTime /10, 5);
      lastTime = time;
      moveCamera(moveDelta,0);


    } */
  }


    //private working members
  private Vector3d cross = new Vector3d();
  private Vector3d xAxis = new Vector3d(1.0,0,0);
  private Vector3d tester  = new Vector3d(1.0,0,0);

  private void moveCamera(int dx, int dy)
  {
        double x_angle =  -dy * rot_factor;
        double y_angle =  -dx * rot_factor;

        tg.getTransform(transform);
        transform.get(oldPos);

        // nullify translation
        transform.setTranslation(zeroVector);

        // Y Angle
        xAxis.set(1.0,0,0);
        tester.set(1.0,0,0);

        transform.transform(tester);
        tester.y = 0;
        double yAngle = tester.angle(xAxis);

        cross.cross(xAxis, tester);

        if (cross.y < 0)
          yAngle = -yAngle;

        totalYTran.rotY(yAngle);

        xRot.set(xAxis, x_angle);
        rotXTran.set(xRot);

        oldTransform.set(transform);
        transform.mul( rotXTran);

        // Check Max upDown
        angleTester.set(0,0,1);
        transform.transform(angleTester);
        yAnglePlane.set(angleTester);
        angleTester.y = 0;
        double updownAngle =  angleTester.angle(yAnglePlane) ;

        if (updownAngle > MAXUPDOWN)
          transform.set(oldTransform);

        oldTransform.set(transform);

        // rot Y
        rotXTran.rotY(y_angle);
        transform.mul(rotXTran, transform);

        angleTester.set(0,0,1);
        transform.transform(angleTester);

        angleTester.y = 0;

        // cap left right at 45 deg away from centre
        if (oldPos.angle(angleTester) > BPhysics.QUARTER_PI)
          transform.set(oldTransform);
        //BDebug.print ("BArenaFixedCam angleFromCentre " + oldPos.angle(angleTester) );

        transform.setTranslation(oldPos);


        tg.setTransform(transform);
  }





  public void mouseDragged(MouseEvent e)
  {
    super.mouseDragged(e);

    if (isMouseActive && !e.isShiftDown())
    {
      if (BGlobal.INVERT_PLAYER_Y)
        dy = -dy;

      moveCamera(dx, dy);
    }
  }


  protected static final int KEY_DX = 5;
  protected static final int KEY_DY = 5;

  public void keyUpPressed(KeyEvent e)
    { moveCamera(0, BGlobal.INVERT_PLAYER_Y ? -KEY_DY : +KEY_DY); }

  public void keyDownPressed(KeyEvent e)
    { moveCamera(0, BGlobal.INVERT_PLAYER_Y ? +KEY_DY : -KEY_DY); }

  public void keyLeftPressed(KeyEvent e)
    { moveCamera(+KEY_DX, 0); }

  public void keyRightPressed(KeyEvent e)
    { moveCamera(-KEY_DX, 0); }



}