package roboforge.ui.modes;

//import roboforge.BBot;
//import roboforge.BCollisionRecord;
import roboforge.BComponent;
import roboforge.BGlobal;
import roboforge.BJoint;

import roboforge.ui.*;
//import roboforge.ui.components.BUIResources;
import roboforge.util.graphics.*;

import java.awt.Cursor;
import java.awt.event.MouseEvent;

//import java.util.Enumeration;
//import java.util.Vector;

import javax.swing.tree.*;
import roboforge.sound.*;


public class BModeJointRot extends BModeManipulator
{
  public static final boolean TRANSIENT = false;
  public static final boolean PERMANENT = true;

  protected boolean modifyDefaultRotation = true;

  protected BJoint currentJoint;
  protected BDependantSound jointMoveSound;

  protected Cursor jointRotateCursor = BGraphics.getCursor("limb_rot_cursor.gif", 16, 16);

  // Constructor.
  // -------------------------------------------------------------

  public BModeJointRot(B3DBotCanvas canvas, boolean modify)
  {
    super(canvas);
    this.modifyDefaultRotation = modify;

    jointMoveSound = (BDependantSound)BDependantSound.getSound(BGlobal.SOUNDPATH + "Interface/JointMovement.wav", 0, BSound.SOUND_TYPE_INTERFACE  );
    jointMoveSound.reset();
    BSound jointMoveEnd =  BSound.getSound(BGlobal.SOUNDPATH + "Interface/jointMovementEnd.wav" , 0, BSound.SOUND_TYPE_INTERFACE );
    jointMoveSound.chainSound(jointMoveEnd);

  }


  // Activation methods.
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();
    BComponent c = canvas.getSelectedComponent();

    if (c != null)
      onPickComponent(c, null);
  }

  // Mouse-event methods.
  // -------------------------------------------------------------

  public void mousePressed(MouseEvent e)
  {
    super.mousePressed(e);

    canvas.setCursor(jointRotateCursor);

    BComponent c = canvas.getSelectedComponent();

    currentJoint = null;

    // YLY jun 16 00, changed instanceof to componenttype ==
    if (c != null)
      if (c.getComponentType() == BJoint.COMPONENT_TYPE)
        currentJoint = (BJoint) c;
  }

  public void mouseDragged(MouseEvent e)
  {
    super.mouseDragged(e);

    if (currentJoint != null)
    {
      if ( currentJoint.mouseMove(x, y, x+dx, y+dy, modifyDefaultRotation))
        jointMoveSound.play();
    }

  }

  public void mouseReleased(MouseEvent e)
  {
    super.mouseReleased(e);
    canvas.setCursor(Cursor.getDefaultCursor());
  }

  protected void onPickNothing(MouseEvent e)
  {
    canvas.setSelectedComponent(null);
  }

  protected void onPickComponent(BComponent c, MouseEvent e)
  {
    BJoint j = null;

    if (c instanceof BJoint)
      j = (BJoint) c;
    else
      j = getAncestorJoint(c);

    if ((j != null) && !j.getChassisLock())
      canvas.setSelectedComponent(j, true, true, false, false, BGlobal.SELECTED_COLOR_3F);
    else
      canvas.setSelectedComponent(j, true, true, false, false, BGlobal.LOCKED_COLOR_3F);
  }


  protected BJoint getAncestorJoint(BComponent c)
  {
    DefaultMutableTreeNode n = c.getTreeNode();

    while (!n.isRoot())
    {
      n = (DefaultMutableTreeNode) n.getParent();
      c = (BComponent) n.getUserObject();

      if (c instanceof BJoint)
        return (BJoint) c;
    }

    return null;
  }

}

