package roboforge.ui.modes;

import roboforge.BDebug;
import roboforge.ui.*;

import java.awt.event.MouseEvent;

public class BModeMulti extends BModeMouse
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BModeMouse currentSubMode  = null;
  protected BModeMouse defaultMode     = null;

  protected boolean isDragging          = false;

  // Constructor...
  // -------------------------------------------------------------

  public BModeMulti()
    { super(); }

  public BModeMulti(BModeMouse defaultMode)
  {
    super();
    this.defaultMode    = defaultMode;
    this.currentSubMode = defaultMode;
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setDefaultMode(BModeMouse m)
  {
    defaultMode = m;
    m.setSubMode(true);

    if (currentSubMode == null)
      currentSubMode = defaultMode;
  }

  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();
    onActivateSubMode();
  }

  public void onDeactivate()
  {
    onDeactivateSubMode();
    super.onDeactivate();
  }

  public void onClose()
  {
    onCloseSubMode();
    super.onClose();
  }


  public void onActivateSubMode()
  {
    if (currentSubMode != null)
      currentSubMode.requestActivate();
  }

  public void onDeactivateSubMode()
  {
    if (currentSubMode != null)
      currentSubMode.requestDeactivate();
  }

  public void onCloseSubMode()
  {
    if (currentSubMode != null)
      currentSubMode.onClose();
  }

  public void mousePressed(MouseEvent e)
  {
    super.mousePressed(e);

    if (!isDragging)
    {
      BModeMouse cur = getModeForEvent(e);

      if (cur != currentSubMode)
        activateSubMode(cur);

      currentSubMode.mousePressed(e);
    }

    isDragging = true;
  }


  public void mouseDragged(MouseEvent e)
  {
    // SJS [18/04/2000] - HACK:
    // Not calling super.mouseDragged in order to
    // reduce overhead.  This may become a bug if
    // superclasses introduce new behaviour.

    // super.mouseDragged(e);

    currentSubMode.mouseDragged(e);
  }

  public void mouseReleased(MouseEvent e)
  {
    super.mouseReleased(e);

    currentSubMode.mouseReleased(e);
    switchToDefaultMode();

    isDragging = false;
  }

  public String getToolTipText(int x, int y)
    { return currentSubMode.getToolTipText(x, y); }


  protected void switchToDefaultMode()
  {
    BModeMouse old = currentSubMode;
    BModeMouse cur = defaultMode;

    if (cur != old)
      activateSubMode(cur);
  }

  protected BModeMouse getModeForEvent(MouseEvent e)
    { return defaultMode; }

  protected void setCurrentSubMode(BModeMouse m)
    { currentSubMode = m; }

  protected void activateSubMode(BModeMouse m)
  {
    onDeactivateSubMode();
    currentSubMode = m;
    onActivateSubMode();
  }

}
