package roboforge.ui.modes;

import roboforge.BComponent;
import roboforge.BConnectPoint;
import roboforge.BConnectPointShape;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BHighlightShape;
import roboforge.BShape3D;
import roboforge.BZone;
import roboforge.BZoneShape;
import roboforge.ui.*;
import roboforge.ui.components.BUIResources;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import javax.media.j3d.*;

import com.sun.j3d.utils.picking.*;
// import com.sun.j3d.utils.behaviors.picking.*;


public class BModePick extends BModeCanvas
{

  // Internal variables.
  // -------------------------------------------------------------

  protected PickCanvas pickCanvas;
  // protected PickObject pickObject  = null;
    // Object used to do picking within the scene.

  protected Shape3D pickedShape;
    // Shape last picked from the scene.


  // Constructor.
  // -------------------------------------------------------------

  public BModePick(B3DCanvas canvas)
  {
    super(canvas);

    pickCanvas = new PickCanvas(canvas, canvas.getPickGroup());
    pickCanvas.setMode(PickCanvas.GEOMETRY);
    pickCanvas.setTolerance(0f);

    // pickObject = new PickObject(canvas, canvas.getPickGroup());
  }


  // Mouse-event methods.
  // -------------------------------------------------------------

  /*public void onActivate()
  {
    super.onActivate();
    canvas.setDragActive(true);
  }

  public void onDeactivate()
  {

    canvas.setDragActive(false);
    super.onDeactivate();
  }  */


  public void mousePressed(MouseEvent e)
  {
    super.mousePressed(e);

    // Shape3D pickedShape;

    if (pickedShape == null)
    {
      pickCanvas.setShapeLocation(e);
      PickResult picked = pickCanvas.pickClosest();
      pickedShape = (picked == null) ?
        null : (Shape3D) picked.getObject();
    }

      /*
        pickObject.pickNode(
          pickObject.pickClosest(e.getX(), e.getY(), PickObject.USE_GEOMETRY),
          PickObject.SHAPE3D);
      */

    if (pickedShape == null)
      onPickNothing(e);

    if (pickedShape instanceof BShape3D)
    {
      BShape3D s = (BShape3D) pickedShape;
      onPickComponent(s.getComponent(), e);
    }

    else if (pickedShape instanceof BConnectPointShape)
    {
      BConnectPointShape s = (BConnectPointShape) pickedShape;
      onPickConnectPt(s.getConnectPoint(), e);
    }

    else if (pickedShape instanceof BZoneShape)
    {
      BZoneShape z = (BZoneShape) pickedShape;
      onPickZone(z.getZone(), e);
    }

    pickedShape = null;
  }


  // Picking methods.
  // -------------------------------------------------------------

  protected void onPickNothing(MouseEvent e)
    { }

  protected void onPickComponent(BComponent c, MouseEvent e)
    { }

  protected void onPickConnectPt(BConnectPoint c, MouseEvent e)
    { }

  protected void onPickZone(BZone z, MouseEvent e)
    { }



  // Accessor methods...
  // -------------------------------------------------------------

  protected void setPickedShape(Shape3D shape)
    { pickedShape = shape; }



  // ToolTip method.
  // -------------------------------------------------------------

  public String getToolTipText(int x, int y)
  {

    pickCanvas.setShapeLocation(x, y);

    PickResult picked = pickCanvas.pickClosest();

    Shape3D pickedShape =
      (picked == null) ? null : (Shape3D) picked.getObject();

    /*
  	Shape3D pickedShape =
      (Shape3D) pickObject.pickNode(
        pickObject.pickClosest(x, y, PickObject.USE_GEOMETRY),
        PickObject.SHAPE3D
      );
    */

    if (pickedShape == null)
      return super.getToolTipText(x, y);

    if (pickedShape instanceof BShape3D)
    {
      BShape3D s = (BShape3D) pickedShape;
      BComponent c = s.getComponent();

      return c.getToolTipText();
    }

    else if (pickedShape instanceof BConnectPointShape)
    {
      BConnectPointShape s = (BConnectPointShape) pickedShape;
      BConnectPoint c = s.getConnectPoint();

      return "Connect point";
    }

    else if (pickedShape instanceof BZoneShape)
    {
      BZoneShape s = (BZoneShape) pickedShape;
      BZone z = s.getZone();

      return "Zone: "+ z.getName();
    }

    else return "";
  }

}





