package roboforge.ui.modes;

import roboforge.BGlobal;
import roboforge.ui.*;
import roboforge.ui.components.BUIResources;
import roboforge.util.graphics.*;

import java.awt.Cursor;
import java.awt.event.MouseEvent;

import javax.media.j3d.*;
import javax.vecmath.*;

import com.sun.j3d.utils.universe.*;


public class BModeViewZoom extends BModeCanvas
{

  // Class variables...
  // -------------------------------------------------------------

  private static final Cursor CURSOR =
    BGraphics.getCursor("zoom_cursor.gif", 15, 15);


  // Internal variables.
  // -------------------------------------------------------------

  private static final double baseWeightFactor   = 2.5;
  private static final double standardZDistance  = 300.0;
  private static final double logBias            = 1.0;

  private double closestZDistance   = 0;
  private double furthestZDistance  = 0;

  protected double weightFactor   = 0.0;

  private double totalZoom  = 0.0;

  protected Vector3d    zoom       = null;
  protected Transform3D translation = new Transform3D();
  protected TransformGroup translationGroup;



  // Constructor.
  // -------------------------------------------------------------

  public BModeViewZoom(B3DCanvas canvas)
  {
    super(canvas);
    cursor = CURSOR;

    translationGroup = canvas.getViewTranslateTG();
    closestZDistance = canvas.getViewTranslationMin();
    furthestZDistance = canvas.getViewTranslationMax();

  }

  // Activation methods.
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();

    // Remember the bot's position...
    zoom = canvas.getViewTranslation();
  }

  public void onDeactivate()
  {
    // Tell the bot to remember this pan [i.e. translation]...
    canvas.setViewTranslation(zoom);

    super.onDeactivate();
  }

  // Mouse-event methods.
  // -------------------------------------------------------------

  public void mouseDragged(MouseEvent e)
  {
    super.mouseDragged(e);
    weightFactor = getWeightFactor(zoom.z);
		totalZoom = (dy - dx) * weightFactor;

    // Zoom by the prescribed amount.
    zoom.z += totalZoom;

    // Clamp zoom distance to specified range...
    if (zoom.z < closestZDistance)
      zoom.z = closestZDistance;

    else if (zoom.z > furthestZDistance)
      zoom.z = furthestZDistance;

    // Update transform group.
    translation.setTranslation(zoom);
    translationGroup.setTransform(translation);

    // Tell canvas to update view.
		canvas.setViewTranslation(zoom);
  }

  protected double getWeightFactor(double z)
    { return baseWeightFactor * Math.log((z / standardZDistance) + logBias); }


  public void setClosestZDistance(double d)
    { closestZDistance = d; }

  public void setFurthestZDistance(double d)
    { furthestZDistance = d; }

}


