package roboforge.ui.modes;

import roboforge.ui.B3DCanvas;
import roboforge.ui.components.*;
import roboforge.util.graphics.*;


public class BModeViewer extends BModeKeyedMulti
{

  // Class variables...
  // -------------------------------------------------------------

  protected static BModeContextMenu sharedContext;
  protected static BModePickMulti   sharedMeta;
  protected static BModeViewZoom    sharedZoom;
  protected static BModeViewPan     sharedPan;
  protected static BModeViewRot     sharedRot;

  protected static boolean sharedInitialized = false;


  // Member variables...
  // -------------------------------------------------------------

  protected B3DCanvas canvas        = null;


  // UI Resources ...
  // -------------------------------------------------------------

  public BModeViewer(B3DCanvas canvas, BModeMouse defaultMode)
  {
    this(canvas);
    setDefaultMode(defaultMode);
  }

  public BModeViewer(B3DCanvas canvas)
  {
    super();

    this.canvas = canvas;

    if (!sharedInitialized)
      initializeShared(canvas);

    setRightMouseMode(sharedMeta);
    setAltMode(sharedRot);
    setCtrlMode(sharedPan);
    setShiftMode(sharedZoom);
  }

  // Shared mode methods...
  // -------------------------------------------------------------

  protected void initializeShared(B3DCanvas canvas)
  {
    sharedZoom    = new BModeViewZoom(canvas);
    sharedPan     = new BModeViewPan(canvas);
    sharedRot     = new BModeViewRot(canvas);

    sharedContext = new BModeContextMenu(canvas);
    sharedMeta    = new BModePickMulti(canvas);

    sharedMeta.setDefaultMode(sharedContext);
    sharedMeta.setPickAnyMode(sharedContext);
    sharedMeta.setPickNoneMode(sharedRot);

    sharedInitialized = true;
  }

  protected void uninitializeShared()
  {
    sharedContext = null;
    sharedMeta    = null;
    sharedZoom    = null;
    sharedPan     = null;
    sharedRot     = null;

    sharedInitialized = false;
  }

  public static BModeContextMenu getSharedContextMenu()
    { return sharedContext; }
    


  // Activation methods...
  // -------------------------------------------------------------

  /*
  public void onActivate()
  {
    super.onActivate();
    if (!isSubMode)
      canvas.setMode(this);
  }

  public void onDeactivate()
  {
    if (!isSubMode)
      canvas.setMode(null);

    super.onDeactivate();
  }
  */

  public void onClose()
  {
    canvas = null;

    if (sharedInitialized)
      uninitializeShared();

    super.onClose();
  }

}
