package roboforge.ui.net;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.ui.*;
import roboforge.ui.components.*;
import roboforge.util.graphics.*;
import roboforge.util.data.*;
import java.text.DateFormat;

import roboforge.ui.action.BThreadedAction;

import roboforge.util.io.BFileSystem;
import roboforge.util.processors.BProcessor;
import roboforge.math.*;
import roboforge.net.*;
import roboforge.net.client.*;
import roboforge.net.data.*;
//import roboforge.net.services.*;

import java.awt.*;
import java.awt.event.*;
//import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.text.NumberFormat;



import roboforge.util.BSystemBrowser;

public class BTournamentScreen extends BSystemScreen
{


  // Member variables...
  // -------------------------------------------------------------

  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;

protected BClientStack      server;

  protected BLoginPanel       loginPanel;
  protected BEnterPanel       enterPanel;
  protected BHistoryPanel     historyPanel;
  protected BAccountPanel     accountPanel;
  protected BChallengePanel   challengePanel;
  protected BBalancePanel     balancePanel;


  protected BPromptDialog cancelSimDialog =
    BPromptDialog.createYesNoDialog("Cancel fight...",
      "You are running a challenge room fight. Are you sure you wish to quit?");


  private static long clearCacheTime = 604800000; // 1 week in ms

  private static DateFormat df =   DateFormat.getDateInstance(DateFormat.SHORT);
  private static long LIST_HEADING_HACK_ID = -666;


  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleIcon.icon",       BIconCache.getHSB("tournament_icon.gif"),
    "titleLabel.icon",      BIconCache.getHSB("tournament_title.gif"),

    "enter.text",           "enter",
    "enter.tooltip",        "Enter your bots into tournaments.",

    "results.text",         "results",
    "results.tooltip",      "Get your tournament results.",

    "account.text",         "account",
    "account.tooltip",      "Maintain your account information.",

    "challenge.text",       "challenge",
    "challenge.tooltip",    "Challenge other Roboforge players.",
  });


  // Visual elements ...
  // -------------------------------------------------------------

  protected BConnectorButton enterButton   = new BConnectorButton(res, "enter");
  protected BConnectorButton resultsButton = new BConnectorButton(res, "results");
  protected BConnectorButton accountButton = new BConnectorButton(res, "account");
  protected BConnectorButton challengeButton = new BConnectorButton(res, "challenge");



  // Constructor.
  // -------------------------------------------------------------

  public BTournamentScreen(BClientStack stack)
  {
    super();

    this.server = stack;
    
    layoutWorkshopChildren();
    configWorkshopChildren();

    res.cleanse();
    res = null;
    
    loginShow();
  }

  
  protected void finalize()
    { System.out.println("BTournamentScreen.finalize()"); }



  // Activation methods...
  // -------------------------------------------------------------
  protected BPromptDialog emailDialog =
    BPromptDialog.createNotifyDialog("Email address incorrect",
      "Please change your email address in the accounts section.");



  public void onActivate()
  {
    super.onActivate();

    if (!server.isLoggedIn())
      loginShow();

    setStatus ("Clearing cache");
    BFileSystem.clearOldFiles (BGlobal.BATTLERECORDINGCACHEPATH, clearCacheTime );



  }

  public void onClose()
  {
    try
      { server.logout(); }
    catch (BNetException e)
      { /* Ignore */ }
      
    super.onClose();
  }

  protected void setPanelButtonsEnabled(boolean enabled)
  {
    super.setPanelButtonsEnabled(enabled);
  }
  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutWorkshopChildren()
  {
    titleIcon.setIcon(res.getIcon("titleIcon.icon"));
    titleLabel.setIcon(res.getIcon("titleLabel.icon"));
  }

  protected void configWorkshopChildren()
  {
    setStatus("Entering tournament area...");

    BSystemPanel panel =
      new BSystemPanel(this, "Client");

    loginPanel     = new BLoginPanel(this);
    enterPanel     = new BEnterPanel(this);
    historyPanel   = new BHistoryPanel(this);
    accountPanel   = new BAccountPanel(this);
    challengePanel = new BChallengePanel(this);
    balancePanel = new BBalancePanel(this); 

    addSystemPanel(enterPanel,     enterButton);
    addSystemPanel(historyPanel,   resultsButton);
    addSystemPanel(challengePanel, challengeButton);
    addSystemPanel(accountPanel,   accountButton);
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public BClientStack getServer()
    { return server; }


  public void loginShow()
  {
    setPanelButtonsEnabled(false);
    setSystemPanel(loginPanel);
  }


  public void loginHide()
  {
    setPanelButtonsEnabled(true);
    setSystemPanel(enterPanel);
    enterButton.setSelected(true);

    // Check email address
    if (server.isStubEmail() )
    {
      Thread dialogThread = new Thread()
      {
        public void run()
        {
          BApplication.showDialog(emailDialog);

        }
      };

      dialogThread.start();

    }

  }

  protected boolean confirmBackWithUser()
  {
    if (challengePanel.isBusy())
    {
       BApplication.showDialog(cancelSimDialog);

       return   cancelSimDialog.getResult();

    }
    else return true;





  }

  protected void showBalancePanel()
  {
    showBalancePanel(null);
  }

  protected BSystemPanel oldPanel;

  protected void showBalancePanel(BBalanceData balance)
  {
    oldPanel = getSystemPanel();
    setPanelButtonsEnabled(false);
    setSystemPanel(balancePanel);
    balancePanel.setBalance(balance);
    setCloseButtonEnabled(false);

  }

  protected void hideBalancePanel()
  {
    
    setCloseButtonEnabled(true);
    setPanelButtonsEnabled(true);
    setSystemPanel(oldPanel);
    oldPanel = null;
  }

  protected class BBalancePanel extends  BSystemPanel
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;

	protected BPromptDialog submitPaymentDialog =
    BPromptDialog.createYesNoDialog("Make Payment...",
      "Do you wish to transfer money from your credit card to your roboforge entry balance");

    protected BPromptDialog submitWithdrawDialog =
    BPromptDialog.createYesNoDialog("Submit Withdraw...",
      "Do you wish to withdraw money from your Roboforge winnings Account?");

    int paymentGrain = 5;

    protected BUIResources res = new BUIResources( new Object[]
    {
        "left.width",                 new Integer(320),
        "right.width",                new Integer(350),
        "exitButton.text",            "back",

        "balanceTitle.icon",          BIconCache.getHSB("account_balance.gif"),
        "balanceTitle.foreground",    colors.getMidShade(),
        "balanceTitle.height",        new Integer(30),

        "transactionTitle.icon",          BIconCache.getHSB("balance_transactions_title.gif"),
        "transactionTitle.foreground",    colors.getMidShade(),
        "transactionTitle.height",        new Integer(30),


        "futureSlider.min",           new Integer(5),
        "futureSlider.max",           new Integer(85),
        "futureSlider.value",         new Integer(paymentGrain),
        "futureSlider.majorSpacing",  new Integer(10),
        "futureSlider.minorSpacing",  new Integer(paymentGrain),

        "sliderText.text",            "",

        "browserButton.text",         "add money..." ,

        "refresh.name",               "Refresh",
        "refresh.icon",               BIconCache.getHSB("net_refresh_icon.gif"),
        "refresh.tooltip",            "Refresh Transaction view.",

        "widthdrawRight.width",       new Integer(200),
        "widthdrawRight.height",      new Integer(15),

        "paypal.text",                "Pay pal..",
        "paypal.height",              new Integer(15),


    });

    protected BActionButton   refresh         = new BActionButton();
    protected RefreshAction   refreshAction   = new RefreshAction(res, "refresh");

    protected BButton         paymentExitButton      = new BButton (res, "exitButton");
    protected BButton         withdrawExitButton      = new BButton (res, "exitButton");

    protected BLabel          balanceTitle    = new BLabel(res, "balanceTitle");
    protected BLabel          transactionTitle    = new BLabel(res, "transactionTitle");

    protected BTextField      entryField      = new BTextField();
    protected BTextField      winningsField   = new BTextField();

    // for the payment Panel
    protected BMultiLabel paymentInfo         = new BMultiLabel(res, "paymentInfo");
    protected BPaymentPanel paymentPanel      = new BPaymentPanel() ;


    protected BList transactionsList = new BList(res, "transactionsList");
    protected BScrollPane transactionScroller = new BScrollPane(transactionsList);

    // widthdrawPanel
    protected float prizeBalance;
    protected BTextField withdrawAmount = new BTextField (res,  "widthdrawRight");
    protected BWithdrawPanel withdrawPanel    = new BWithdrawPanel();





    public BBalancePanel(BTournamentScreen parent)
    {
      super(parent, "balance");
      //add(fin);

      layoutBalanceChildren();
      configureBalanceChildren();


    }

    protected void layoutBalanceChildren()
    {
      // layout tool bar
      //toolbar.add(refresh);


      // layout main screen
      BBoxPanel left    = new BBoxPanel(res, "left",  BBoxPanel.Y_AXIS);
      BBoxPanel right   = new BBoxPanel(res, "right", BBoxPanel.Y_AXIS);

      setBorder(new EmptyBorder(30, 0, 20, 45));
      addMargin();
      addLinear(getVStrut(45), refresh, getVGlue());
      addMargin();
      addVGlued(left);
      addBigMargin();
      addVGlued(right);

      //left.addVGlue();
      left.addGlued(transactionTitle);
      left.addMargin();
      left.addVGlued(transactionScroller);
      //left.add(exitButton);

      BTabbedPane tab     = new BTabbedPane(this.res, "tab");
        tab.addTab("entry payments", paymentPanel);
        tab.addTab("withdraw winnings", withdrawPanel);


      right.addGlued(balanceTitle);
      right.addMargin();
      right.add(new BalanceInfoPanel() );
      right.addTinyMargin();
      right.add(tab);
    }


    protected void configureBalanceChildren()
    {
      transactionsList.setCellRenderer(new TranCellRenderer() );
      refresh.setAction(refreshAction);
      paymentExitButton.addActionListener( new CloseActionListener() );
      withdrawExitButton.addActionListener( new CloseActionListener() );

    }


    protected BBalanceData reQuery()
    {
      BRequest r = new BRequest(server, "GATB");

      try
        {
          BTransmission t = new BTransmission(r);
          setMonitored(t);

          BMessage balanceResponse = server.request(t);

          setMonitored(null);

          if (balanceResponse.isAcknowledgement())
          {
            BBalanceData data = (BBalanceData) balanceResponse.getData();
            return data;
          }
          else
          {
            setStatus( "ERROR: " + balanceResponse );

          }


        }
        catch (Exception ex)
        {

          setStatus("ERROR: "+ ex.getMessage());
        }

        setMonitored(null);

        return null;

    }

    protected void setBalance (BBalanceData data)
    {
      if (data == null)
        data = reQuery();

      if (data != null)
        setUI(data);
    }


    public void onActivate()
    {
      super.onActivate();
      refreshAction.actionPerformed(new ActionEvent(this, 1, ""));
    }

    protected void setUI(BBalanceData balance)
    {
        entryField.setText("$" + BMath.format(balance.currentBalance, 2) + "    (Can only be used to enter Tournaments)" );
        winningsField.setText("$" + BMath.format(balance.winningBalance, 2)+ "    (You can withdraw this balance)");

        paymentPanel.setFuturePay(balance.isFuturePayActive);
        withdrawPanel.setUI( balance.winningBalance );
        

    }


  //  inner classes--------------------------------
  protected class RefreshAction extends BThreadedAction
  {

    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;

	public RefreshAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
    {
      setStatus("Refreshing transactions...");
      setBusyWorking(true);
    }

    public void threadedAction(ActionEvent e)
    {

      try
      {
        BDebug.print ("BTournamentScreen.refreshAction.ThreadedAction server " + server);

        //GATB

        BTransmission t = new BTransmission(
          new BRequest(server, "GATR"));

        BMessage tranResponse =  server.request(t);
       // BDebug.print ("BTournamentScreen.refreshAction.ThreadedAction response " + tranResponse);


        if (tranResponse.isAcknowledgement())
        {
          BDataSet transactions = (BDataSet) tranResponse.getData();

         /// BDebug.print ("BTournamentScreen.refreshAction.ThreadedAction transactions"  + transactions);
         // BDebug.print ("BTournamentScreen.refreshAction.ThreadedAction transactions.data"  + transactions.data);
         // BDebug.print ("BTournamentScreen.refreshAction.ThreadedAction transactionsList"  + transactionsList);

          if (transactions.data != null)
          {
            // hacky , but we are out of time.
            BTransactionData heading = new BTransactionData(LIST_HEADING_HACK_ID);

            Vector transV = new Vector(transactions.data.length + 1);
            transV.add(heading);
            transV.addAll( Arrays.asList(transactions.data)  );

            transactionsList.setListData(transV) ;

            setBalance(null);
          }
          //setStatus("Transactions refreshed.");

        }
        else
        {
          setStatus("ERROR: "+ tranResponse);
        }



      }
      catch (Exception ex)
      {
        setStatus("ERROR: "+ ex.getMessage());

      }
    }

    public void swingCleanupAction(ActionEvent e)
    {

        parent.setBusyWorking(false);
        setStatus("Transactions updated.");

    }

  }



  class CloseActionListener implements ActionListener
  {
      public void actionPerformed(ActionEvent e)
      {
        hideBalancePanel();
      }

    }

  protected class BWithdrawPanel extends BBoxPanel
  {

    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
	protected BButton withdrawButton = new BButton ("withdraw");
    protected BButton paypalButton = new BButton (res, "paypal");
    protected BMultiLabel withdrawDesc = new BMultiLabel();


    public  BWithdrawPanel()
    {
      super(BBoxPanel.Y_AXIS) ;

      // layout
      addMargin();
      addLinear(new BLabel("Amount to withdraw:  "),  getHStrut(15), withdrawAmount);
      addSmallMargin();
      addMargin();
      addHGlued(withdrawDesc);
      addSmallMargin();
      addLinear(getHGlue(), paypalButton);
      addBigMargin();
      addBigMargin();
      addLinear( getHGlue(),withdrawButton, getHStrut(15), withdrawExitButton);

     // configure

      paypalButton.addActionListener(new PayPalActionListener());

      withdrawButton.addActionListener( new WithDrawAction() );
     // withdrawButton.setText("withdraw");
      withdrawDesc.setText("We will pay from your Tournament Winnings the amount you have entered above using PayPal before the 20th of next month. For more information on how PayPal works click the 'pay pal' button.");


    }

    public void setUI (float prizeAmount)
    {
      prizeBalance = prizeAmount;
    }


    class PayPalActionListener implements ActionListener
    {
      public void actionPerformed(ActionEvent e)
      {


        try
        {

          BSystemBrowser.displayURL(BGlobal.PAYPAL_URL);
        }
        catch(Exception ex)
        {
          setStatus( "ERROR calling default browser. please visit: " + BGlobal.PAYPAL_URL );
        }

      }
    }

  }


  protected void submitWithdraw(float dollarAmount)
    {

      BFPTransferAmtData amount = new BFPTransferAmtData(-1);
      amount.fpTransAmt = dollarAmount;

      BRequest r = new BRequest(BTournamentScreen.this.server, "WWIN", amount);

      try
        {
          BTransmission t = new BTransmission(r);

          setMonitored(t);
          BMessage response = BTournamentScreen.this.server.request(t);

          if ( response.isAcknowledgement() )
          {
            refreshAction.actionPerformed(new ActionEvent(this, 1, "") );
            setStatus("Payment sucessfully updated, refreshing balance..." );
          }
          else
            { setStatus("ERROR: "+ response ); }
        }
        catch (BNetException ex)
          { setStatus("Submit pay ERROR: "+ ex.getMessage()); }
        finally
          { setMonitored(null); }

    }


  protected class WithDrawAction implements ActionListener
  {
    float lastParsedAmount;

    public void actionPerformed(ActionEvent e)
    {
    
      try
      {
        lastParsedAmount = Float.parseFloat(withdrawAmount.getText());
        if (lastParsedAmount < paymentGrain)
        {
           setStatus("The smallest amount you can withdraw is $" + paymentGrain);
        }
        else if (lastParsedAmount > prizeBalance)
          setStatus("Insufficent funds. Payment not sent");
        else
        {
          BProcessor p = new BProcessor()
          {
            public void process()
            {
              BApplication.showDialog( submitWithdrawDialog  );

              if ( submitWithdrawDialog.getResult() )
                submitWithdraw(lastParsedAmount);
            }

          };

          p.start();



        }


      }
      catch (Exception ex)
      {
        setStatus("Invalid amount, payment not sent.");
      }

    }

  }

  protected class BPaymentPanel extends BBoxPanel
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
	protected BSlider futureSlider  = new BSlider(res, "futureSlider");
    protected BButton payButton = new BButton("make payment");

    protected BLabel sliderText = new BLabel(res, "sliderText");
    protected BButton browserButton = new BButton(res, "browserButton");
    protected BLabel sliderLabel = new BLabel("Payment amount:");

    boolean futurePayActive = true;

    public BPaymentPanel()
    {

      super(BBoxPanel.Y_AXIS);

      layoutPaymentPanel();
      configurePaymentPanel();
      updateInfo();

    }

     protected void layoutPaymentPanel()
    {
      addMargin();
      addLinear(sliderLabel,getHStrut(15), futureSlider);
      addTinyMargin();
      addGlued(sliderText);
      addMargin();
      add(paymentInfo);

      BBoxPanel bottomBox = new BBoxPanel(BBoxPanel.X_AXIS);
        bottomBox.addHGlue();
        bottomBox.add(payButton);
        bottomBox.add(browserButton);
        bottomBox.add(getHStrut(15));
        bottomBox.add(paymentExitButton);

      add(bottomBox);

      //addLinear( getHGlue(),  payButton, browserButton, getHStrut(15), paymentExitButton);


    }

    protected void setFuturePay(boolean active)
    {
      futurePayActive = active;
      futureSlider.setVisible(active);
      payButton.setVisible(active);
      sliderText.setVisible(active);
      sliderLabel.setVisible(active);
      browserButton.setVisible(!active);


      if (active)
        paymentInfo.setText("By clicking the 'Make Payment' button your credit card will automatically be billed using your 'Future Pay' account. All Entry Fees are non-refundable.");
      else
        paymentInfo.setText("To add money to your Tournament Account click on the button below. You will be taken to World Pay's secure server where you can add money to your account by either using 'Future Pay' or by making a one-off payment.");

    }

    protected boolean isFuturePayActive()
    {
      return futurePayActive;
    }

    protected void  configurePaymentPanel()
    {
      futureSlider.addChangeListener(new ChangeListener()
      {
        public void stateChanged(ChangeEvent e)
        {
            updateInfo();
        }
      });

      futureSlider.setValue(paymentGrain);

      payButton.addActionListener( new PayAction() );
      browserButton.addActionListener( new BrowserAction() );

    }





    protected void updateInfo()
    {
      if (isFuturePayActive() )
      {
        int value = futureSlider.getValue();
        int remainder = value %  paymentGrain;
        int midPoint = paymentGrain / 2;

        if ( remainder <= midPoint )
          value = value - remainder;
        else
          value = value + (paymentGrain - remainder);

        sliderText.setText( "Add $" + value + " to my Roboforge account." );
      }
      else
        sliderText.setText("");
        
    }



    protected void submitPay()
    {

      BFPTransferAmtData amount = new BFPTransferAmtData(-1);
      amount.fpTransAmt = futureSlider.getValue();

      BRequest r = new BRequest(BTournamentScreen.this.server, "UWFP", amount);

      try
        {
          BTransmission t = new BTransmission(r);

          setMonitored(t);
          BMessage response = BTournamentScreen.this.server.request(t);

          if ( response.isAcknowledgement() )
          {
            //setBalance (null);
            refreshAction.actionPerformed(new ActionEvent(this,-1,"") );

            setStatus("Payment sucessfully updated, refreshing balance..." );
          }
          else
          {
            setStatus("ERROR: "+ response );
          }


        }
        catch (Exception ex)
        {

          setStatus("Submit pay ERROR: "+ ex.getMessage());
        }
        finally
          { setMonitored(null); }


    }


    class PayAction implements ActionListener
    {
      public void actionPerformed(ActionEvent e)
      {
        BProcessor p = new BProcessor()
        {
          public void process()
          {
            BApplication.showDialog( submitPaymentDialog  );

            if ( submitPaymentDialog.getResult() )
              submitPay();
          }

        };

        p.start();

      }
    }


  }



  class BrowserAction implements ActionListener
  {

    public void actionPerformed(ActionEvent e)
    {
      String site = null;

       try
        {

          if (paymentPanel.isFuturePayActive() )
            site = BGlobal.WORLDPAY_ACCOUNT_URL;
          else
            site =  BGlobal.WORLDPAY_URL;



          BSystemBrowser.displayURL( site + "?M_ACCTID=" + server.getAccountID() );

        }
        catch(Exception ex)
        {
          setStatus("ERROR connecting to account, please visit the following web site: " + site );

        }

    }
  }

  protected class BalanceInfoPanel extends BBoxPanel
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;


	protected Border labelBorder =
      new EmptyBorder(0,0,0,10);


    protected BUIResources res = new BUIResources( new Object[]
    {
      "entry.width",     new Integer(80),
      "entry.text",      "Entry balance:",
      "entry.h_align",   BLabel.ALIGN_RIGHT,
      "entry.border",    labelBorder,

      "winning.width",     new Integer(80),
      "winning.text",      "winnings:",
      "winning.h_align",   BLabel.ALIGN_RIGHT,
      "winning.border",    labelBorder,


    });

     public BalanceInfoPanel()
    {
      super(BBoxPanel.Y_AXIS);
      setBorder(new EmptyBorder(5,0,25,0));

      addLinear(new BLabel(this.res, "entry"), entryField);
      addSmallMargin();
      addLinear(new BLabel(this.res, "winning"), winningsField);

      entryField.setEnabled(false);
      winningsField.setEnabled(false);

    }
  }





  }

  protected static final NumberFormat numberFormat = NumberFormat.getInstance(BGlobal.getLocale());

  class TranCellRenderer extends BBoxPanel implements ListCellRenderer
    {
       /**
	 * 
	 */
	private static final long serialVersionUID = -1L;




	protected BUIResources res = new BUIResources( new Object[]
    {
        "ID.width",               new Integer(40),
        "description.width",      new Integer(210),
        "date.width",             new Integer(50),
        "status.width",           new Integer(100),
        "amount.width",           new Integer(60),
        "account.width",          new Integer(100)

    });




      protected BLabel ID = new BLabel(this.res, "ID");
      protected BLabel description = new BLabel(this.res, "description");
      protected BLabel date = new BLabel(this.res, "date");
      protected BLabel status = new BLabel(this.res, "status");
      protected BLabel account = new BLabel(this.res, "account");
      protected BLabel amount = new BLabel(this.res, "amount");

      public TranCellRenderer()
      {
        super(BBoxPanel.X_AXIS);
        //ID.setText("Hello");

        /*description.setPreferredSize(new Dimension(40, 10));;
        date.setPreferredSize(new Dimension(40, 10));
        status.setPreferredSize(new Dimension(40, 10));
        amount.setPreferredSize(new Dimension(40, 10));     */
        numberFormat.setMinimumFractionDigits(2);
        amount.setHorizontalAlignment(JTextField.RIGHT );

        add(date);
        add(description);
        add(amount);
        add(status);
        add(account);
        add(ID);


      }

      protected void setColumnColors(Color colour )
      {
        ID.setForeground(colour);
        description.setForeground(colour);
        date.setForeground(colour);
        account.setForeground(colour);
        status.setForeground(colour);
        amount.setForeground(colour);
        
      }


        public Component getListCellRendererComponent(
         JList list,
         Object value,
         int index,
         boolean isSelected,
         boolean cellHasFocus)
      {
          BTransactionData data = (BTransactionData) value;

          try
          {
            if (data.getID() == LIST_HEADING_HACK_ID )
            {
              ID.setText("ID");
              description.setText("Description");
              date.setText("Date");
              account.setText("Account");
              status.setText(" Status");
              amount.setText("Amount  ");
              setColumnColors(BGlobal.colors.getLightShade());
            }
            else
            {
              ID.setText(data.getID() + "");
              description.setText(data.detail);
              date.setText(df.format(data.date.getTime() ));

              String c = "C";
              if (c.equalsIgnoreCase(data.account))
              {
                setColumnColors(BGlobal.colors.getAIActionColor() );
                account.setText("entry");

              }
              else
              {
                setColumnColors(BGlobal.colors.getAIMemoryColor() );
                account.setText("winnings");

              }

              status.setText(" " + data.status);

              if (data.amount < 0)
                amount.setText("-$" +  numberFormat.format( (double)data.amount * -1)  + "  " );
              else
                amount.setText("$" + numberFormat.format( (double)data.amount) + "  ");

            }

          }
          catch(Exception e)
          {
            BDebug.print ("BTournament screen. TranSellRendered.getListRenderer exception" + e);


          }



          return this;

    }


   }


}