package roboforge.util.processors;


public abstract class BCountdownTimer implements Runnable
{
  
  // Member variables...
  // -------------------------------------------------------------

  protected Thread  counter;
  protected long    count_time;
  protected boolean isResetting;
  
  
  // Constructor...
  // -------------------------------------------------------------

  public BCountdownTimer()
    { this(30000); }

  
  public BCountdownTimer(long delay)
  {
    super();
    
    this.counter     = null;
    this.count_time  = delay;
    this.isResetting = false;
  }


  // Accessor methods...
  // -------------------------------------------------------------
  
  public synchronized void setTimeout(long ms)
    { this.count_time = ms; }

  public synchronized void setTimeoutSecs(int secs)
    { this.count_time = (secs * 1000); }

  public synchronized long getTimeout()
    { return count_time; }

  public synchronized int getTimeoutSecs()
    { return (int) (count_time / 1000); }

  
  // Start/stop/reset methods...
  // -------------------------------------------------------------

  public void start()
  {
    if (counter == null)
    {
      counter = new Thread(this);
      counter.setPriority(Thread.MAX_PRIORITY);
      counter.start();
    }
    else
      synchronized(this)
        { notify(); }
  }

  public void reset()
  {
    if ((counter != null) && !isResetting)
    {
      isResetting = true;
      counter.interrupt(); 
    }
  }

  public void stop()
  {
    if (counter != null)
    {
      Thread c = counter;
    
      counter = null;
      c.interrupt();
    }
  }

  
  // Run method...
  // -------------------------------------------------------------
  
  public void run()
  {
    while (counter != null)
    {
      try
      {
        counter.sleep(count_time);
        counter = null;
        countdownComplete();
      }
      catch (InterruptedException e)
        { }
      
      if (isResetting)
      {
        try
        {
          synchronized (this)
            { wait(); }
        }
        catch (InterruptedException e)
          { }
        finally
          { isResetting = false; }
      }
    }
    
  }
  
  
  // Countdown completion hook...
  // -------------------------------------------------------------
  
  public abstract void countdownComplete();
  
}
